!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the programme: !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!  - Control Variables                                     !
! and initialize constants                                 !
!==========================================================!

MODULE FAMREL_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=12
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !==========================================================================

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: FAMREL_INPUT_FILE                    !File name of file with FAMREL input data
    CHARACTER(len=150)                                      :: FAMREL_INPUT                         !Internal name to store Path+Name of file with FAMREL input data
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    !Variables for locating output (local variables)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL output data (output is fixed over all modules)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL output data
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: FAMREL_PAR_FORMAT                    !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: FAMREL_PAR_BREAK='@'                 !Character used to identify end of string
    CHARACTER(len=10)                                       :: FAMREL_PAR_CURR                      !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: FAMREL_PAR_NAMES                     !Vector of parameter names
    INTEGER                                                 :: FAMREL_PAR_L                         !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: FAMREL_PAR_C                         !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(5)                                    :: FAMREL_PAR_BREAK_POINT               !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the FAMREL module
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: FAMREL_RELMAT                        !Matrix used to convert relationship codes if individual is not head of the family
    !=======================================================================!  
    !Exogenous variables
    !FAMREL_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    !Define exogenous variables read from the file with sample weights
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: GECODEERD_INSZ_WEIGHT                !Personal identification number in the file with sample weights
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION (:)          :: MIMOSIS_WEIGHT                       !sample weight correcting for non random sampling of observation units
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_YEAR_AGE                      !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_MONTH_AGE                     !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_MONTH_BIRTH                   !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_YEAR_BIRTH                    !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_HH_SIZE                       !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_START_HH                      !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_END_HH                        !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_FAMREL                          !number of households analysed
    !To construct variables listed in the module note
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_CHECK                         !switch variable to 1 if the position of the head and his partner have been checked
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_SAMESEXC                      !dummy variable identifying that individual is part of a couple with equal sex
    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    INTEGER                                                 :: COUNTFAM !Count number of families already met

END MODULE FAMREL_DATA
!=========================================================================!

!==========================================!
! This module contains all the subroutines !
! and functions used in the FAMREL MODULE !
!==========================================!

MODULE FAMREL_SUBROUTINES

    USE FAMREL_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!===============================================!
!Allocate all vectors of dimension DIM_ALLOC
!===============================================!

SUBROUTINE FAMREL_ALLOC(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !FAMREL_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    !Define exogenous variables read from the file with sample weights
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(FAMREL_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(FAMREL_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(FAMREL_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(FAMREL_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(FAMREL_HH_SIZE(DIM_ALLOC))
    ALLOCATE(FAMREL_START_HH(DIM_ALLOC))
    ALLOCATE(FAMREL_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    ALLOCATE(FAMREL_CHECK(DIM_ALLOC))
    ALLOCATE(FAMREL_SAMESEXC(DIM_ALLOC))

END SUBROUTINE FAMREL_ALLOC

SUBROUTINE FAMREL_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !Variables read from FAMREL_INPUT.txt
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_SEX)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_RELATION)
    !Define exogenous variables read from the file with sample weights
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)
    !=======================================================================!
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(FAMREL_YEAR_AGE)
    DEALLOCATE(FAMREL_MONTH_AGE)
    DEALLOCATE(FAMREL_MONTH_BIRTH)
    DEALLOCATE(FAMREL_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(FAMREL_HH_SIZE)
    DEALLOCATE(FAMREL_START_HH)
    DEALLOCATE(FAMREL_END_HH)
    !To construct variables listed in the module note
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    DEALLOCATE(FAMREL_CHECK)
    DEALLOCATE(FAMREL_SAMESEXC)
    !=======================================================================!
    !Deallocate parameters
    DEALLOCATE(FAMREL_RELMAT)
    
END SUBROUTINE FAMREL_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE FAMREL_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                               !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j)=0
        !to construct age variables
        FAMREL_YEAR_AGE(pers_j)=0
        FAMREL_MONTH_AGE(pers_j)=0
        FAMREL_MONTH_BIRTH(pers_j)=0
        FAMREL_YEAR_BIRTH(pers_j)=0
        !to construct information at the level of the sociological household
        FAMREL_HH_SIZE(pers_j)=0
        FAMREL_START_HH(pers_j)=0
        FAMREL_END_HH(pers_j)=0
        !To construct variables listed in the module note
        FAMREL_FAMILY(pers_j)=0
        FAMREL_RELATION(pers_j)=0
        FAMREL_FAMTYPE(pers_j)=0
        FAMREL_COUPLE(pers_j)=0
        FAMREL_CHECK(pers_j)=0
        FAMREL_SAMESEXC(pers_j)=0
    END DO
    N_HH_FAMREL=0

END SUBROUTINE FAMREL_INITENDO

!========== SUBROUTINE FAMREL_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!======================================================!

SUBROUTINE FAMREL_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !Sort variables from the INPUT data file    
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    !Rank individual identification variable read from the file with sample weights
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !Sort variables read from the weight file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL FAMREL_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables from the INPUT data file
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    !Sort variables read from the weight file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL FAMREL_CONSISTENCY_CHECK

END SUBROUTINE FAMREL_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from         !
!the parameter file                                    !
!======================================================!
SUBROUTINE FAMREL_PARAMETER_FILEREAD

    !open parameter file        
    OPEN(UNIT=3, FILE=FAMREL_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMREL_PARLOC
        WRITE(15,'("Could not find ", a100)') FAMREL_PARLOC
        PAUSE
        STOP
    END IF        

    !Read in parameters for path and name of input and output files
    DO i=1,n_loclines/2
        !Read default path names
        !Read in input file information
        IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_FILE') THEN
            FAMREL_INPUT_FILE = CONTROL_IO(2*i)
        !Read in intermediate output file information
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        END IF    
    END DO
    !check for length of default path and file name parameters
    FAMREL_PAR_BREAK_POINT(1) = INDEX(FAMREL_INPUT_PATH,FAMREL_PAR_BREAK)
    FAMREL_PAR_BREAK_POINT(2) = INDEX(FAMREL_INPUT_FILE,FAMREL_PAR_BREAK)
    FAMREL_PAR_BREAK_POINT(3) = INDEX(MIMOSIS_FAMREL_FILE,FAMREL_PAR_BREAK)
    FAMREL_PAR_BREAK_POINT(4) = INDEX(CONSTANT_INPUT_PATH,FAMREL_PAR_BREAK)
    FAMREL_PAR_BREAK_POINT(5) = INDEX(MIMOSIS_SAMPWEIG_FILE,FAMREL_PAR_BREAK)
    !merge information (path+name) to create input file names
    FAMREL_INPUT = FAMREL_INPUT_PATH(1:FAMREL_PAR_BREAK_POINT(1)-1)//&
    &FAMREL_INPUT_FILE(1:FAMREL_PAR_BREAK_POINT(2)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:FAMREL_PAR_BREAK_POINT(4)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:FAMREL_PAR_BREAK_POINT(5)-1)    
    !merge information (path+name) to create output file names
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:FAMREL_PAR_BREAK_POINT(1)-1)//&
    &MIMOSIS_FAMREL_FILE(1:FAMREL_PAR_BREAK_POINT(3)-1)
    !Trim the result
    CALL TRIM@(FAMREL_INPUT)
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    CALL TRIM@(MIMOSIS_FAMREL)
    !write names to log file
    WRITE(15,*) 'FAMREL_INPUT= ',FAMREL_INPUT
    WRITE(15,*) 'MIMOSIS_SAMPWEIG= ',MIMOSIS_SAMPWEIG
    WRITE(15,*) 'MIMOSIS_FAMREL= ',MIMOSIS_FAMREL
    
    ALLOCATE(FAMREL_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    i=1
    DO
        READ(3,'(a40,2i10,a10)') FAMREL_PAR_NAMES(i), FAMREL_PAR_L, &
        &FAMREL_PAR_C, FAMREL_PAR_CURR
        IF (FAMREL_PAR_NAMES(i)=='FAMREL_RELMAT') THEN
            WRITE(15,'(a40)') FAMREL_PAR_NAMES(i)
            ALLOCATE(FAMREL_RELMAT(FAMREL_PAR_L,FAMREL_PAR_C))
            READ(3,'(a20)') FAMREL_PAR_FORMAT
            READ(3,FAMREL_PAR_FORMAT)((FAMREL_RELMAT(j,k),k=1,FAMREL_PAR_C),j=1,FAMREL_PAR_L)
            WRITE(15,FAMREL_PAR_FORMAT)((FAMREL_RELMAT(j,k),k=1,FAMREL_PAR_C),j=1,FAMREL_PAR_L)
        ELSE IF (FAMREL_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(FAMREL_PAR_NAMES)

    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine FAMREL_PARAMETER_FILEREAD .................runned correctly'  

END SUBROUTINE FAMREL_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file        !
!======================================================!
SUBROUTINE FAMREL_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=FAMREL_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMREL_INPUT
        WRITE(15,'("Could not find ", a100)') FAMREL_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,I10,A7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
            &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,*) 'An error occurred in line', pers_j
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j)'')')
                WRITE(15,'(A14,I10,I10,A7,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,I10,A7,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMREL_INPUT_FILEREAD .....................runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE FAMREL_INPUT_FILEREAD

!====================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!====================================!

SUBROUTINE FAMREL_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        WRITE(15,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.10)',IOSTAT=ierror) GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine FAMREL_WEIGHT_FILEREAD ....................runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE FAMREL_WEIGHT_FILEREAD

!====================================!
!Check whether individual identification numbers are sorted consistently!
!====================================!

SUBROUTINE FAMREL_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1,n_rec
        IF ((GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j)) ) THEN
            WRITE (*,*) "Error: Ranking of external files does not match!"
            PAUSE
            STOP
        END IF
    END DO   

END SUBROUTINE FAMREL_CONSISTENCY_CHECK 

!===============================================!
! HOUSEHOLD_SIZE computes household size      !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!===============================================!

SUBROUTINE FAMREL_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        FAMREL_HH_SIZE(k)=n
        FAMREL_START_HH(k)=j
        FAMREL_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_FAMREL=K

END SUBROUTINE FAMREL_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: FAMREL_YEAR_AGE    !
!                         FAMREL_MONTH_AGE   !
!                         FAMREL_YEAR_BIRTH  !
!                         FAMREL_MONTH_BIRTH !   
!============================================!

SUBROUTINE FAMREL_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        FAMREL_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:FAMREL_PAR_BREAK_POINT(1)-1),  '(I10)') FAMREL_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(FAMREL_PAR_BREAK_POINT(1)+1:),   '(I10)') FAMREL_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        FAMREL_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-FAMREL_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        FAMREL_MONTH_AGE(pers_j) =FAMREL_YEAR_AGE(pers_j)*12+(12-FAMREL_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE FAMREL_AGE

!Start construction of family type unit 1
SUBROUTINE FAMTYPE1_CONSTRUCT_START(hh_i)

    INTEGER:: pers_j,pers_k,hh_i
    INTEGER:: CountPart         !count number of partners of the head of the household
    INTEGER:: PosPart           !position of the partner if there are more partners in one household
    INTEGER:: AgeDif            !compute the age difference with the head of the household
    INTEGER:: AgeHead           !age of the head of the household
    INTEGER:: HeadChec          !If head of the household is found, variable is set equal to 1
    INTEGER:: OldMemb           !Store the line of the oldest household member
    INTEGER:: AgeOld            !Store the age of the oldest household member
    
    CountPart = 0
    AgeHead   = 0
    HeadChec  = 0
    !count number of partners and store age of the head
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !Check whether the partner is unique
        IF (NAREGNIS_RELATION(pers_j).EQ.2) THEN
            CountPart = CountPart + 1
        END IF
        !Store age of the head of the household
        IF (NAREGNIS_RELATION(pers_j).EQ.1) THEN
            AgeHead = FAMREL_YEAR_AGE(pers_j)
            HeadChec = 1
        END IF
    END DO

    OldMemb  = 0
    AgeOld   = 0
    !if there is no head, make the oldest household member head of the household
    !Remark that we manipulate a basic input variable for this purpose, i.e. value of NAREGNIS_RELATION
    IF (HeadChec.EQ.0) THEN
        !look for oldest household member
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            IF (FAMREL_YEAR_AGE(pers_j).GT.AgeOld) THEN
                AgeOld = FAMREL_YEAR_AGE(pers_j)
                OldMemb  = pers_j
            END IF
        END DO
        IF (OldMemb.NE.0) THEN
            NAREGNIS_RELATION(OldMemb) = 1
            AgeHead = FAMREL_YEAR_AGE(OldMemb)
        END IF
    END IF

    PosPart = 0
    AgeDif  = 100
    IF (CountPart.LT.2) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !search head of the sociological household
            IF (NAREGNIS_RELATION(pers_j).EQ.1) THEN
                FAMREL_RELATION(pers_j) = 1
                FAMREL_FAMTYPE(pers_j)  = 1
                COUNTFAM                = COUNTFAM + 1
                FAMREL_FAMILY(pers_j)   = COUNTFAM
                !search his partner
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    !always add the condition that it should concern different individuals if you loop over household
                    !members twice, is a safety condition
                    IF (NAREGNIS_RELATION(pers_k).EQ.2.AND.pers_j.NE.pers_k) THEN
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 1
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                    END IF
                END DO
            END IF
        END DO    
    ELSE
        !search different partners and store position of the one with smallest age difference with the head of the sociological household
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            IF (NAREGNIS_RELATION(pers_j).EQ.2.AND.ABS(FAMREL_YEAR_AGE(pers_j)-AgeHead).LT.AgeDif) THEN
                PosPart = pers_j
                AgeDif  = ABS(FAMREL_YEAR_AGE(pers_j)-AgeHead)
            END If
        END DO        
        !construct unit of head and his spouse
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !search head of the sociological household
            IF (NAREGNIS_RELATION(pers_j).EQ.1) THEN
                !allocate head
                FAMREL_RELATION(pers_j)     = 1
                FAMREL_FAMTYPE(pers_j)      = 1
                COUNTFAM                    = COUNTFAM + 1
                FAMREL_FAMILY(pers_j)       = COUNTFAM
                !allocate partner
                FAMREL_RELATION(PosPart)    = 2
                FAMREL_FAMTYPE(PosPart)     = 1
                FAMREL_FAMILY(PosPart)      = FAMREL_FAMILY(pers_j)
                FAMREL_COUPLE(pers_j)       = 1
                FAMREL_COUPLE(PosPart)      = 1                        
            END IF
        END DO    
        write(15,'(''More than one partner of head in household '',2i10)') NAREGNIS_HH_ID(FAMREL_END_HH(hh_i)),CountPart
    END IF

END SUBROUTINE FAMTYPE1_CONSTRUCT_START

!Construction family type unit 2 and 6
SUBROUTINE FAMTYPE26_CONSTRUCT(hh_i)

    INTEGER:: pers_j,pers_k,hh_i
    INTEGER:: CountChild                !count the number of children
    INTEGER:: CountGrandChildren        !count the number of grandschildren
    INTEGER:: CountGreatGrandChildren   !count the number of grandschildren
    INTEGER:: CountChildreninlaw        !count the number of sons or daughters in law
    INTEGER:: AgeOldestGCh              !count the age of oldest grandchild or great grandchild
    INTEGER:: AgeOldestGreatGCh         !count the age of oldest great grandchild
    
    CountChild                  = 0
    CountGrandChildren          = 0
    CountGreatGrandChildren     = 0
    CountChildreninlaw          = 0
    AgeOldestGCh                = 0
    AgeOldestGreatGCh           = 0
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !search children of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.3.OR.NAREGNIS_RELATION(pers_j).EQ.13) THEN
            CountChild = CountChild + 1
        END IF
        !search children in law of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.4) THEN
            CountChildreninlaw = CountChildreninlaw + 1
        END IF
        !search grandchildren of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.5) THEN
            CountGrandChildren = CountGrandChildren + 1
            !store the age of the oldest grand or great grantchild
            IF (FAMREL_YEAR_AGE(pers_j).GT.AgeOldestGCh) THEN
                AgeOldestGCh = FAMREL_YEAR_AGE(pers_j)
            END IF
        END IF
        !search greatgrandchildren of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.14) THEN
            CountGreatGrandChildren = CountGreatGrandChildren + 1
            !store the age of the oldest grand or great grantchild
            IF (FAMREL_YEAR_AGE(pers_j).GT.AgeOldestGCh) THEN
                AgeOldestGCh = FAMREL_YEAR_AGE(pers_j)
            END IF
            !store the age of the oldest great grantchild
            IF (FAMREL_YEAR_AGE(pers_j).GT.AgeOldestGreatGCh) THEN
                AgeOldestGreatGCh = FAMREL_YEAR_AGE(pers_j)
            END IF
        END IF        
    END DO    

    !if there is only one child and if there are grandchildren
    !make separate unit if there is sufficient age difference
    IF (CountChild.EQ.1.AND.(CountGrandChildren.GT.0.OR.CountGreatGrandChildren.GT.0)) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            IF (NAREGNIS_RELATION(pers_j).EQ.3.OR.NAREGNIS_RELATION(pers_j).EQ.13) THEN            
                IF (FAMREL_YEAR_AGE(pers_j).GT.(15+AgeOldestGCh)) THEN
                    FAMREL_RELATION(pers_j) = 1
                    FAMREL_FAMTYPE(pers_j)  = 2
                    COUNTFAM                = COUNTFAM + 1
                    FAMREL_FAMILY(pers_j)   = COUNTFAM
                    !Search grandchildren and great grandchildren and allocate partner if unique child in law
                    DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                        IF (NAREGNIS_RELATION(pers_k).EQ.5.AND.pers_j.NE.pers_k) THEN
                            FAMREL_RELATION(pers_k) = 3
                            FAMREL_FAMTYPE(pers_k)  = 2
                            FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        END IF
                        IF (NAREGNIS_RELATION(pers_k).EQ.14.AND.pers_j.NE.pers_k) THEN
                            FAMREL_RELATION(pers_k) = 6
                            FAMREL_FAMTYPE(pers_k)  = 2
                            FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        END IF
                        IF (NAREGNIS_RELATION(pers_k).EQ.4.AND.CountChildreninlaw.EQ.1.AND.pers_j.NE.pers_k) THEN
                            FAMREL_RELATION(pers_k) = 2
                            FAMREL_FAMTYPE(pers_k)  = 2
                            FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                            FAMREL_COUPLE(pers_j)   = 1
                            FAMREL_COUPLE(pers_k)   = 1                        
                        END IF
                    END DO
                END IF
            END IF
        END DO
    END IF

    !if there is no child but if there are grandchildren and great grandchildren
    !make separate unit if there is sufficient age difference
    IF (CountChild.EQ.0.AND.CountGrandChildren.EQ.1.AND.CountGreatGrandChildren.GT.0) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            IF (NAREGNIS_RELATION(pers_j).EQ.5) THEN            
                IF (FAMREL_YEAR_AGE(pers_j).GT.(15+AgeOldestGreatGCh)) THEN
                    FAMREL_RELATION(pers_j) = 1
                    FAMREL_FAMTYPE(pers_j)  = 6
                    COUNTFAM                = COUNTFAM + 1
                    FAMREL_FAMILY(pers_j)   = COUNTFAM
                    !Search great grandchildren 
                    DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                        IF (NAREGNIS_RELATION(pers_k).EQ.14.AND.pers_j.NE.pers_k) THEN
                            FAMREL_RELATION(pers_k) = 3
                            FAMREL_FAMTYPE(pers_k)  = 6
                            FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        END IF
                    END DO
                END IF
            END IF
        END DO
    END IF

END SUBROUTINE FAMTYPE26_CONSTRUCT

!Construction family type unit 3, 4 and 5
SUBROUTINE FAMTYPE345_CONSTRUCT(hh_i)

    INTEGER:: pers_j,pers_k,hh_i
    INTEGER:: CountFathMoth             !count the number of fathers and mothers
    INTEGER:: CountFathMothinLaw        !count the number of fathers and mothers in law
    INTEGER:: CountGrandparents         !count the number of grandparents in the household 
    INTEGER:: CountBrothSist            !count the number of borthers and sisters of the head of the sociological household
    INTEGER:: CountBrothSistinLaw       !count the number of borthers and sisters in law of the head of the sociological household
    
    !construct counters of different household members
    CountFathMoth               = 0
    CountFathMothinLaw          = 0
    CountGrandparents           = 0
    CountBrothSist              = 0
    CountBrothSistinLaw         = 0
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !search father or mother of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.6) THEN
            CountFathMoth = CountFathMoth + 1
        END IF
        !search father or mother in law of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.7) THEN
            CountFathMothinLaw = CountFathMothinLaw + 1
        END IF
        !search grandparents of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.8) THEN
            CountGrandparents = CountGrandparents + 1
        END IF
        !search brothers or sisters of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.9) THEN
            CountBrothSist = CountBrothSist + 1
        END IF
        !search brothers or sisters in law of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.10) THEN
            CountBrothSistinLaw = CountBrothSistinLaw + 1
        END IF
    END DO    

    !if there are two parents construct separate family of type 3
    IF (CountFathMoth.EQ.2) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !search for the man and make him head of a separate family
            IF (NAREGNIS_RELATION(pers_j).EQ.6.AND.NAREGNIS_SEX(pers_j).EQ.1) THEN            
                FAMREL_RELATION(pers_j) = 1
                FAMREL_FAMTYPE(pers_j)  = 3
                COUNTFAM                = COUNTFAM + 1
                FAMREL_FAMILY(pers_j)   = COUNTFAM
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.6.AND.pers_j.NE.pers_k) THEN  
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 3
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                    END IF                            
                END DO
            END IF
        END DO
    END IF        

    !if there are two parents in law construct separate family of type 3 if they are of different sex
    IF (CountFathMothinLaw.EQ.2) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !the first parent in law
            IF (NAREGNIS_RELATION(pers_j).EQ.7.AND.FAMREL_RELATION(pers_j).EQ.0) THEN            
                !if found check the sex of the second parent in law
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.7.AND.NAREGNIS_SEX(pers_k).NE.NAREGNIS_SEX(pers_j)&
                    &.AND.FAMREL_RELATION(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN  
                        !at this stage the first individual is made the head
                        FAMREL_RELATION(pers_j) = 1
                        FAMREL_FAMTYPE(pers_j)  = 3
                        COUNTFAM                = COUNTFAM + 1
                        FAMREL_FAMILY(pers_j)   = COUNTFAM
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 3
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j) 
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                    END IF                            
                END DO
            END IF
        END DO
    END IF        

    !if there are two grandparents construct separate family of type 4 if they are of different sex
    IF (CountGrandparents.EQ.2) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !the first grandparent
            IF (NAREGNIS_RELATION(pers_j).EQ.8.AND.FAMREL_RELATION(pers_j).EQ.0) THEN            
                !if found check the sex of the second grandparent
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.8.AND.NAREGNIS_SEX(pers_k).NE.NAREGNIS_SEX(pers_j)&
                    &.AND.FAMREL_RELATION(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN  
                        !at this stage the first individual is made the head
                        FAMREL_RELATION(pers_j) = 1
                        FAMREL_FAMTYPE(pers_j)  = 4
                        COUNTFAM                = COUNTFAM + 1
                        FAMREL_FAMILY(pers_j)   = COUNTFAM
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 4
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                    END IF                            
                END DO
            END IF
        END DO
    END IF        

    !if there is only one brother or sister and one borther or sister in law 
    !construct a separate family of type 5 if they are of different sex
    IF (CountBrothSist.EQ.1.AND.CountBrothSistinLaw.EQ.1) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !the first brother or sister
            IF (NAREGNIS_RELATION(pers_j).EQ.9.AND.FAMREL_RELATION(pers_j).EQ.0) THEN            
                !if found check the sex of the brother or sister in law
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.10.AND.NAREGNIS_SEX(pers_k).NE.NAREGNIS_SEX(pers_j)&
                    &.AND.FAMREL_RELATION(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN  
                        !at this stage the first individual is made the head
                        FAMREL_RELATION(pers_j) = 1
                        FAMREL_FAMTYPE(pers_j)  = 5
                        COUNTFAM                = COUNTFAM + 1
                        FAMREL_FAMILY(pers_j)   = COUNTFAM
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 5
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j) 
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                    END IF                            
                END DO
            END IF
        END DO
    END IF        


END SUBROUTINE FAMTYPE345_CONSTRUCT

!Construction family type unit 7
SUBROUTINE FAMTYPE7_CONSTRUCT(hh_i)

    INTEGER:: pers_j,pers_k,pers_l,hh_i
    INTEGER:: CountAunts                !count the number of aunts or uncles
    INTEGER:: CountReltoThird           !count the number of relatives up till the third degree
    
    !construct counters of different household members
    CountAunts                  = 0
    CountReltoThird             = 0
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !search aunts or uncles of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.15) THEN
            CountAunts  = CountAunts  + 1
        END IF
        !search relatives up till the third degree of the head of the sociological household
        IF (NAREGNIS_RELATION(pers_j).EQ.16) THEN
            CountReltoThird  = CountReltoThird  + 1
        END IF
    END DO    

    !if there are two aunts or uncles construct separate family of type 7 if they are of different sex
    IF (CountAunts.EQ.2) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !the first aunt
            IF (NAREGNIS_RELATION(pers_j).EQ.15.AND.FAMREL_RELATION(pers_j).EQ.0) THEN            
                !if found check the sex of the second aunt or uncle
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.15.AND.NAREGNIS_SEX(pers_k).NE.NAREGNIS_SEX(pers_j)&
                    &.AND.FAMREL_RELATION(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN  
                        !at this stage the first individual is made the head
                        FAMREL_RELATION(pers_j) = 1
                        FAMREL_FAMTYPE(pers_j)  = 7
                        COUNTFAM                = COUNTFAM + 1
                        FAMREL_FAMILY(pers_j)   = COUNTFAM
                        FAMREL_RELATION(pers_k) = 2
                        FAMREL_FAMTYPE(pers_k)  = 7
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j)
                        FAMREL_COUPLE(pers_j)   = 1
                        FAMREL_COUPLE(pers_k)   = 1                        
                        !search for the children of this couple
                        DO pers_l=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                            IF (NAREGNIS_RELATION(pers_l).EQ.16.AND.FAMREL_RELATION(pers_l).EQ.0.AND.pers_j.NE.pers_l) THEN  
                                !at this stage the first individual is made the head
                                FAMREL_RELATION(pers_l) = 3
                                FAMREL_FAMTYPE(pers_l)  = 7
                                FAMREL_FAMILY(pers_l)   = FAMREL_FAMILY(pers_j) 
                            END IF
                        END DO                            
                    END IF                            
                END DO
            END IF
        END DO
    END IF        

    !if there is only one aunt or uncle construct separate family of type 7 if there are alse relatives up till the third degree
    IF (CountAunts.EQ.1.AND.CountReltoThird.GT.0) THEN
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !the first aunt
            IF (NAREGNIS_RELATION(pers_j).EQ.15.AND.FAMREL_RELATION(pers_j).EQ.0) THEN            
                FAMREL_RELATION(pers_j) = 1
                FAMREL_FAMTYPE(pers_j)  = 7
                COUNTFAM                = COUNTFAM + 1
                FAMREL_FAMILY(pers_j)   = COUNTFAM
                !search for the children of this single
                DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.16.AND.FAMREL_RELATION(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN  
                        !at this stage the first individual is made the head
                        FAMREL_RELATION(pers_k) = 3
                        FAMREL_FAMTYPE(pers_k)  = 7
                        FAMREL_FAMILY(pers_k)   = FAMREL_FAMILY(pers_j) 
                    END IF
                END DO                            
            END IF
        END DO
    END IF        

END SUBROUTINE FAMTYPE7_CONSTRUCT

!Construction family type unit 1 continued
SUBROUTINE FAMTYPE1_CONSTRUCT_CONTINUE(hh_i)

    INTEGER:: pers_j,hh_i
    INTEGER:: PosHead       !determine position of head of the sociological household
    INTEGER:: CountChild    !Count the number of children in the family of the head of the household
    INTEGER:: CountHHMemb   !Count the number of members in the family of the head of the household
    INTEGER:: CountNonRel   !Count the number of non relatives
    
    !identify position of the head of the household
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        IF (pers_j.EQ.FAMREL_START_HH(hh_i)) THEN
            PosHead = pers_j
        END IF    
        IF (NAREGNIS_RELATION(pers_j).EQ.1) THEN
            PosHead = pers_j
        END IF    
    END DO

    !Allocate all individuals below the age of 18 as children if they satisfy certain
    !relation conditions
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        IF (FAMREL_YEAR_AGE(pers_j).LT.18.AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/3,4,5,9,11,12,13,14,16,17/))) THEN
                FAMREL_RELATION(pers_j) = 3
                FAMREL_FAMTYPE(pers_j)  = 1
                FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
            END IF    
        END IF    
    END DO

    !count children and all members in family of the head of the household
    CountChild  = 0
    CountHHMemb = 0
    CountNonRel = 0
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !count children and all members in family of the head of the household
        IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(PosHead)) THEN
            CountHHMemb = CountHHMemb + 1
            IF (FAMREL_RELATION(pers_j).EQ.3) THEN
                CountChild = CountChild + 1
            END IF    
        END IF    
        !count the number non identified non relatives
        IF (NAREGNIS_RELATION(pers_j).EQ.12.AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            CountNonRel = CountNonRel + 1    
        END IF
    END DO
    
    !if the family of the head of the sociological household only consists of children
    !and there is only one non relative not assigned yet
    !then consider the non relative as the partner of the head of the household
    !and identify the couple as a de facto couple
    IF (CountNonRel.EQ.1.AND.(CountHHMemb-CountChild).EQ.1) THEN
        !search the non only non relative and make him partner of the head of the family
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            IF (NAREGNIS_RELATION(pers_j).EQ.12.AND.FAMREL_RELATION(pers_j).EQ.0) THEN
                FAMREL_RELATION(pers_j) = 2
                FAMREL_FAMTYPE(pers_j)  = 1
                FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead)
                FAMREL_COUPLE(pers_j)   = 2
                FAMREL_COUPLE(PosHead)  = 2                        
            END IF
        END DO
    END IF
    
    !attribute the remaining individuals to the family of the head of the sociological household
    !or make them separate units if they live in a collective household
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        !identify children of the family of the head of the sociological household
        IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/3,4,13/)).AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 3
            FAMREL_FAMTYPE(pers_j)  = 1
            FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
        END IF    
        !identify ascendants of the first degree of the family of the head of the sociological household
        IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/6,7/)).AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 4
            FAMREL_FAMTYPE(pers_j)  = 1
            FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
        END IF    
        !identify ascendants of the second degree of the family of the head of the sociological household
        IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/8/)).AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 5
            FAMREL_FAMTYPE(pers_j)  = 1
            FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
        END IF    
        !identify relatives up till the third degree of the family of the head of the sociological household
        IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/5,9,10,14,15,16/)).AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 6
            FAMREL_FAMTYPE(pers_j)  = 1
            FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
        END IF    
        !identify other family members of the family of the head of the sociological household
        IF (ANY(NAREGNIS_RELATION(pers_j).EQ.(/11,12,17/)).AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 7
            FAMREL_FAMTYPE(pers_j)  = 1
            FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
        END IF    
        !identify individuals living in a collective household 
        IF (NAREGNIS_RELATION(pers_j).EQ.20.AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            FAMREL_RELATION(pers_j) = 1
            FAMREL_FAMTYPE(pers_j)  = 8
            COUNTFAM                = COUNTFAM + 1
            FAMREL_FAMILY(pers_j)   = COUNTFAM
        END IF    
        !identify household member that is partner but not allocated as partner
        !see routine FAMTYPE1_CONSTRUCT_START
        IF (NAREGNIS_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_j).EQ.0) THEN
            IF (FAMREL_YEAR_AGE(pers_j).LT.26) THEN
                FAMREL_RELATION(pers_j) = 3
                FAMREL_FAMTYPE(pers_j)  = 1
                FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
            ELSE
                FAMREL_RELATION(pers_j) = 6
                FAMREL_FAMTYPE(pers_j)  = 1
                FAMREL_FAMILY(pers_j)   = FAMREL_FAMILY(PosHead) 
            END IF
        END IF    
    END DO

END SUBROUTINE FAMTYPE1_CONSTRUCT_CONTINUE

!Make the man head of the family
!if couples of equal sex, make the oldes head

SUBROUTINE FAMTYPES_HEADS(hh_i)

    INTEGER:: pers_j,pers_k,hh_i

    !loop over all members of the same sociological household and check all other members  
    DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
        DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !look for member of the same family
            IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                !if they are a couple and their position has not been checked yet
                IF (FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2.AND.&
                &FAMREL_CHECK(pers_j).EQ.0.AND.FAMREL_CHECK(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN
                    !if they are of opposite sex make man head of the family
                    IF (NAREGNIS_SEX(pers_j).NE.NAREGNIS_SEX(pers_k)) THEN
                        IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                            FAMREL_RELATION(pers_j) = 1
                            FAMREL_RELATION(pers_k) = 2
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        ELSE
                            FAMREL_RELATION(pers_j) = 2
                            FAMREL_RELATION(pers_k) = 1
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        END IF
                    !if they ar of the same sex, make the oldest the head of the family
                    ELSE
                        IF (FAMREL_YEAR_AGE(pers_j).GE.FAMREL_YEAR_AGE(pers_k)) THEN
                            FAMREL_RELATION(pers_j) = 1
                            FAMREL_RELATION(pers_k) = 2
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        ELSE
                            FAMREL_RELATION(pers_j) = 2
                            FAMREL_RELATION(pers_k) = 1
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        END IF
                    END IF
                ELSE IF (FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1.AND.&
                &FAMREL_CHECK(pers_j).EQ.0.AND.FAMREL_CHECK(pers_k).EQ.0.AND.pers_j.NE.pers_k) THEN
                    !if they are of opposite sex make man head of the family
                    IF (NAREGNIS_SEX(pers_j).NE.NAREGNIS_SEX(pers_k)) THEN
                        IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                            FAMREL_RELATION(pers_j) = 1
                            FAMREL_RELATION(pers_k) = 2
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        ELSE
                            FAMREL_RELATION(pers_j) = 2
                            FAMREL_RELATION(pers_k) = 1
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        END IF
                    !if they ar of the same sex, make the oldest the head of the family
                    ELSE
                        IF (FAMREL_YEAR_AGE(pers_j).GE.FAMREL_YEAR_AGE(pers_k)) THEN
                            FAMREL_RELATION(pers_j) = 1
                            FAMREL_RELATION(pers_k) = 2
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        ELSE
                            FAMREL_RELATION(pers_j) = 2
                            FAMREL_RELATION(pers_k) = 1
                            FAMREL_CHECK(pers_j)    = 1
                            FAMREL_CHECK(pers_k)    = 1
                        END IF
                    END IF
                END IF    
            END IF
        END DO
    END DO        

END SUBROUTINE FAMTYPES_HEADS

!Produce some aggregate statistics to check the constuction
SUBROUTINE FAMREL_TEST

    INTEGER:: pers_j,pers_k,hh_i
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: FAMREL_RESULT_TEST                   !matrix with test results

    !Write cases that have not been allocated yet
    DO hh_i=1,N_HH_FAMREL
        !loop over all members of the same sociological household and check all other members  
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            !check that all individuals are allocated to a family
            IF (FAMREL_RELATION(pers_j).EQ.0.OR.FAMREL_FAMILY(pers_j).EQ.0) THEN
                WRITE(15,'(''Value for FAMREL_RELATION(pers_j) or FAMREL_FAMILY(pers_j) is still equal to 0'',2i10)')&
                &NAREGNIS_HH_ID(pers_j),GECODEERD_INSZ_INPUT(pers_j)
            END IF
        END DO
    END DO
    
    !Write the total amount of sociogical households and of families
    WRITE(15,'(''There are '', i10,'' sociological households in the sample'')')N_HH_FAMREL
    WRITE(15,'(''There are '', i10,'' families in the sample'')')COUNTFAM
    
    !Compute aggregate statistics on the different family types
    !Set number of summary statistics to be analysed
    NumAnal = 10 !total number of family types distinguishid
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(FAMREL_RESULT_TEST(NumAnal,3))
    FAMREL_RESULT_TEST(1:NumAnal,1:3)=0.D0
         
    DO pers_j=1,N_REC
        IF (FAMREL_FAMTYPE(pers_j).EQ.1) THEN
            FAMREL_RESULT_TEST(1,1) =  FAMREL_RESULT_TEST(1,1) + 1   
            FAMREL_RESULT_TEST(1,2) =  FAMREL_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.2) THEN
            FAMREL_RESULT_TEST(2,1) =  FAMREL_RESULT_TEST(2,1) + 1   
            FAMREL_RESULT_TEST(2,2) =  FAMREL_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.3) THEN
            FAMREL_RESULT_TEST(3,1) =  FAMREL_RESULT_TEST(3,1) + 1   
            FAMREL_RESULT_TEST(3,2) =  FAMREL_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.4) THEN
            FAMREL_RESULT_TEST(4,1) =  FAMREL_RESULT_TEST(4,1) + 1   
            FAMREL_RESULT_TEST(4,2) =  FAMREL_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.5) THEN
            FAMREL_RESULT_TEST(5,1) =  FAMREL_RESULT_TEST(5,1) + 1   
            FAMREL_RESULT_TEST(5,2) =  FAMREL_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.6) THEN
            FAMREL_RESULT_TEST(6,1) =  FAMREL_RESULT_TEST(6,1) + 1   
            FAMREL_RESULT_TEST(6,2) =  FAMREL_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.7) THEN
            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 1   
            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_FAMTYPE(pers_j).EQ.8) THEN
            FAMREL_RESULT_TEST(8,1) =  FAMREL_RESULT_TEST(8,1) + 1   
            FAMREL_RESULT_TEST(8,2) =  FAMREL_RESULT_TEST(8,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE
            FAMREL_RESULT_TEST(9,1) =  FAMREL_RESULT_TEST(9,1) + 1   
            FAMREL_RESULT_TEST(9,2) =  FAMREL_RESULT_TEST(9,2) + MIMOSIS_WEIGHT(pers_j)          
        END IF 
        FAMREL_RESULT_TEST(10,1) =  FAMREL_RESULT_TEST(10,1) + 1   
        FAMREL_RESULT_TEST(10,2) =  FAMREL_RESULT_TEST(10,2) + MIMOSIS_WEIGHT(pers_j)          
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            FAMREL_RESULT_TEST(i,3)=100.D0*FAMREL_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    Write(15,'(/,''                                                          TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Counts of different family types at the level of the individual'')')
    Write(15,'(''Family of head of the soc. househ.                     '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(1,j),j=1,3)
    Write(15,'(''Family of children of the head of the soc. househ.     '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(2,j),j=1,3) 
    Write(15,'(''Family of parents of the head of the soc. househ.      '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(3,j),j=1,3) 
    Write(15,'(''Family of grandparents of the head of the soc. househ. '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(4,j),j=1,3)
    Write(15,'(''Family of broth/sist of head of the soc. househ.       '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(5,j),j=1,3) 
    Write(15,'(''Family of grandchildren of head of the soc. househ.    '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(6,j),j=1,3) 
    Write(15,'(''Family of aunt/uncle of head of the soc. househ.       '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(7,j),j=1,3)
    Write(15,'(''Person living in a collective household                '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(8,j),j=1,3)
    Write(15,'(''Value unknown                                          '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(9,j),j=1,3)
    Write(15,'(''Total of all individuals                               '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(10,j),j=1,3)

    DEALLOCATE(FAMREL_RESULT_TEST)
    
    !Compute aggregate statistics on the different relationships vis a vis the head
    !Set number of summary statistics to be analysed
    NumAnal = 9 !total number of family types distinguishid
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(FAMREL_RESULT_TEST(NumAnal,3))
    FAMREL_RESULT_TEST(1:NumAnal,1:3)=0.D0
         
    DO pers_j=1,N_REC
        IF (FAMREL_RELATION(pers_j).EQ.1) THEN
            FAMREL_RESULT_TEST(1,1) =  FAMREL_RESULT_TEST(1,1) + 1   
            FAMREL_RESULT_TEST(1,2) =  FAMREL_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.2) THEN
            FAMREL_RESULT_TEST(2,1) =  FAMREL_RESULT_TEST(2,1) + 1   
            FAMREL_RESULT_TEST(2,2) =  FAMREL_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.3) THEN
            FAMREL_RESULT_TEST(3,1) =  FAMREL_RESULT_TEST(3,1) + 1   
            FAMREL_RESULT_TEST(3,2) =  FAMREL_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.4) THEN
            FAMREL_RESULT_TEST(4,1) =  FAMREL_RESULT_TEST(4,1) + 1   
            FAMREL_RESULT_TEST(4,2) =  FAMREL_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.5) THEN
            FAMREL_RESULT_TEST(5,1) =  FAMREL_RESULT_TEST(5,1) + 1   
            FAMREL_RESULT_TEST(5,2) =  FAMREL_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.6) THEN
            FAMREL_RESULT_TEST(6,1) =  FAMREL_RESULT_TEST(6,1) + 1   
            FAMREL_RESULT_TEST(6,2) =  FAMREL_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_RELATION(pers_j).EQ.7) THEN
            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 1   
            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE
            FAMREL_RESULT_TEST(8,1) =  FAMREL_RESULT_TEST(8,1) + 1   
            FAMREL_RESULT_TEST(8,2) =  FAMREL_RESULT_TEST(8,2) + MIMOSIS_WEIGHT(pers_j)          
        END IF
        FAMREL_RESULT_TEST(9,1) =  FAMREL_RESULT_TEST(9,1) + 1   
        FAMREL_RESULT_TEST(9,2) =  FAMREL_RESULT_TEST(9,2) + MIMOSIS_WEIGHT(pers_j)          
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            FAMREL_RESULT_TEST(i,3)=100.D0*FAMREL_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    Write(15,'(/,''                                                          TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Counts of different relationships at the level of the individual vis a vis the head of the family'')')
    Write(15,'(''Head of the family               .                     '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(1,j),j=1,3)
    Write(15,'(''Partner of the head of the family                      '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(2,j),j=1,3) 
    Write(15,'(''Child of the head of the family or of his partner      '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(3,j),j=1,3) 
    Write(15,'(''Ascendant in the first degree of the head or partner   '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(4,j),j=1,3)
    Write(15,'(''Ascendant in the second degree of the head or partner  '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(5,j),j=1,3) 
    Write(15,'(''Relatives up till the third degree of head or partner  '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(6,j),j=1,3) 
    Write(15,'(''Other family members of head or partner                '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(7,j),j=1,3)
    Write(15,'(''Value unknown                                          '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(8,j),j=1,3)
    Write(15,'(''Total of all individuals                               '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(9,j),j=1,3)

    DEALLOCATE(FAMREL_RESULT_TEST)
    
    !Compute aggregate statistics on the different types of couples
    !Set number of summary statistics to be analysed
    NumAnal = 5 !total number of couples distinguishid
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(FAMREL_RESULT_TEST(NumAnal,3))
    FAMREL_RESULT_TEST(1:NumAnal,1:3)=0.D0
         
    DO pers_j=1,N_REC
        IF (FAMREL_COUPLE(pers_j).EQ.0) THEN
            FAMREL_RESULT_TEST(1,1) =  FAMREL_RESULT_TEST(1,1) + 1   
            FAMREL_RESULT_TEST(1,2) =  FAMREL_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_COUPLE(pers_j).EQ.1) THEN
            FAMREL_RESULT_TEST(2,1) =  FAMREL_RESULT_TEST(2,1) + 1   
            FAMREL_RESULT_TEST(2,2) =  FAMREL_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE IF (FAMREL_COUPLE(pers_j).EQ.2) THEN
            FAMREL_RESULT_TEST(3,1) =  FAMREL_RESULT_TEST(3,1) + 1   
            FAMREL_RESULT_TEST(3,2) =  FAMREL_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)          
        ELSE 
            FAMREL_RESULT_TEST(4,1) =  FAMREL_RESULT_TEST(4,1) + 1   
            FAMREL_RESULT_TEST(4,2) =  FAMREL_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)          
        END IF 
        FAMREL_RESULT_TEST(5,1) =  FAMREL_RESULT_TEST(5,1) + 1   
        FAMREL_RESULT_TEST(5,2) =  FAMREL_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)          
    END DO

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            FAMREL_RESULT_TEST(i,3)=100.D0*FAMREL_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    Write(15,'(/,''                                                          TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Counts of different types of couples at the individual level'')')
    Write(15,'(''individual is not part of a couple                     '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(1,j),j=1,3)
    Write(15,'(''individual is part of a married couple                 '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(2,j),j=1,3) 
    Write(15,'(''individual is part of a de facto couple                '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(3,j),j=1,3) 
    Write(15,'(''Value unknown                                          '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(4,j),j=1,3)
    Write(15,'(''Total of all individuals                               '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(5,j),j=1,3)

    DEALLOCATE(FAMREL_RESULT_TEST)

    !Compute aggregate statistics on the different types of couples by sex
    !Set number of summary statistics to be analysed
    NumAnal = 7 !total number of couples distinguishid
    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    !Allocate and initialise test result
    ALLOCATE(FAMREL_RESULT_TEST(NumAnal,3))
    FAMREL_RESULT_TEST(1:NumAnal,1:3)=0.D0

    !initialise FAMREL_CHECK
    DO pers_j=1,N_REC
        FAMREL_CHECK(pers_j) = 0
    END DO    

    DO hh_i=1,N_HH_FAMREL
        !loop over all members of the same sociological household and check all other members  
        DO pers_j=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
            DO pers_k=FAMREL_START_HH(hh_i),FAMREL_END_HH(hh_i)
                !look for member of the same family
                IF (FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                    !if they are a married couple
                    IF (FAMREL_COUPLE(pers_j).EQ.1.AND.FAMREL_COUPLE(pers_k).EQ.1.AND.pers_j.NE.pers_k) THEN
                        !if they are of different sex
                        IF (NAREGNIS_SEX(pers_j).NE.NAREGNIS_SEX(pers_k).AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(1,1) =  FAMREL_RESULT_TEST(1,1) + 2  
                            FAMREL_RESULT_TEST(1,2) =  FAMREL_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        !if they are both male
                        ELSE IF (NAREGNIS_SEX(pers_j).EQ.1.AND.NAREGNIS_SEX(pers_k).EQ.1.AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(2,1) =  FAMREL_RESULT_TEST(2,1) + 2  
                            FAMREL_RESULT_TEST(2,2) =  FAMREL_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                            
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        !if they are both female
                        ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.NAREGNIS_SEX(pers_k).EQ.2.AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(3,1) =  FAMREL_RESULT_TEST(3,1) + 2  
                            FAMREL_RESULT_TEST(3,2) =  FAMREL_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                            
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        END IF
                    !if they are a de facto couple
                    ELSE IF (FAMREL_COUPLE(pers_j).EQ.2.AND.FAMREL_COUPLE(pers_k).EQ.2.AND.pers_j.NE.pers_k) THEN
                        !if they are of different sex
                        IF (NAREGNIS_SEX(pers_j).NE.NAREGNIS_SEX(pers_k).AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(4,1) =  FAMREL_RESULT_TEST(4,1) + 2  
                            FAMREL_RESULT_TEST(4,2) =  FAMREL_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                            
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        !if they are both male
                        ELSE IF (NAREGNIS_SEX(pers_j).EQ.1.AND.NAREGNIS_SEX(pers_k).EQ.1.AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(5,1) =  FAMREL_RESULT_TEST(5,1) + 2  
                            FAMREL_RESULT_TEST(5,2) =  FAMREL_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                            
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_SAMESEXC(pers_j) = 1
                            FAMREL_SAMESEXC(pers_k) = 1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        !if they are both female
                        ELSE IF (NAREGNIS_SEX(pers_j).EQ.2.AND.NAREGNIS_SEX(pers_k).EQ.2.AND.FAMREL_CHECK(pers_j).EQ.0.AND.&
                        &FAMREL_CHECK(pers_k).EQ.0) THEN
                            FAMREL_RESULT_TEST(6,1) =  FAMREL_RESULT_TEST(6,1) + 2  
                            FAMREL_RESULT_TEST(6,2) =  FAMREL_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                            
                            FAMREL_CHECK(pers_j)=1
                            FAMREL_CHECK(pers_k)=1
                            FAMREL_SAMESEXC(pers_j) = 1
                            FAMREL_SAMESEXC(pers_k) = 1
                            FAMREL_RESULT_TEST(7,1) =  FAMREL_RESULT_TEST(7,1) + 2  
                            FAMREL_RESULT_TEST(7,2) =  FAMREL_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j) + MIMOSIS_WEIGHT(pers_k)                              
                        END IF
                    END IF
                END IF
            END DO
        END DO
    END DO        

    DO i=1,NumAnal
        IF (INT(FAMREL_RESULT_TEST(7,2)).NE.0) THEN
            FAMREL_RESULT_TEST(i,3)=100.D0*FAMREL_RESULT_TEST(i,2)/FAMREL_RESULT_TEST(7,2)
        END IF        
    END DO

    Write(15,'(/,''                                                          TotSamp  TotSampW      %Pop'')')
    Write(15,'(''Counts of different types of couples by sex for those individuals that are part of couple'')')
    Write(15,'(''married couple of different sex                        '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(1,j),j=1,3)
    Write(15,'(''married couple of both males                           '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(2,j),j=1,3) 
    Write(15,'(''married couple of both females                         '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(3,j),j=1,3) 
    Write(15,'(''de facto couple of different sex                       '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(4,j),j=1,3)
    Write(15,'(''de facto couple of both males                          '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(5,j),j=1,3)
    Write(15,'(''de facto couple of both females                        '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(6,j),j=1,3)
    Write(15,'(''Total of all individuals living in couples             '',2f10.0,f10.2)')(FAMREL_RESULT_TEST(7,j),j=1,3)

    DEALLOCATE(FAMREL_RESULT_TEST)
    
END SUBROUTINE FAMREL_TEST

!Write output to the output file
SUBROUTINE FAMREL_OUTPUT_FILEWRIT

    INTEGER:: pers_j

    OPEN(UNIT=12, FILE=MIMOSIS_FAMREL,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_FAMREL
        WRITE(15,'("Could not find ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF

    DO pers_j=1,n_rec
        WRITE(12,'(A14,4i10)') GECODEERD_INSZ_CHAR(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
        &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)   
    END DO
    
    CLOSE(12)
    
    WRITE(*,*) 'Subroutine FAMREL_OUTPUT_FILEWRIT ....................runned correctly'

END SUBROUTINE FAMREL_OUTPUT_FILEWRIT

!=================================================================================!
!Routine to write FAMREL variables for labsupply
!=================================================================================!

SUBROUTINE WRITELAB_FAMREL

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\FAMREL_LABSUPPLY.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,5i10)') GECODEERD_INSZ_CHAR(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
        &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j),FAMREL_SAMESEXC(pers_j)   
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_FAMREL............................runned correctly'

END SUBROUTINE WRITELAB_FAMREL

SUBROUTINE FAMREL_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS FAMREL IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=FAMREL_LOG, STATUS='REPLACE', ACTION='WRITE')
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMREL_LOG
        WRITE(15,'("Could not find ", a100)') FAMREL_LOG
        PAUSE
        STOP
    END IF

    !Allocate all vectors with dimension n_rec and initialise them to zero
    CALL FAMREL_ALLOC(n_rec)
    !Initialise endogenous variables
    CALL FAMREL_INITENDO(n_rec)
    
    !Reads in PARAMETER file
    CALL FAMREL_PARAMETER_FILEREAD
    !Reads in INPUT files
    CALL FAMREL_INPUT_FILEREAD
    CALL FAMREL_WEIGHT_FILEREAD    
    
    !Rank and Sort all variables read in
    CALL FAMREL_RANKANDSORT

    !Determine household size  
    CALL FAMREL_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL FAMREL_AGE

    !Loop over all households  
    !set counter over households equal to 0
    COUNTFAM = 0
    DO hh_i=1,N_HH_FAMREL
        !construct basic unit of family type 1
        CALL FAMTYPE1_CONSTRUCT_START(hh_i)
        !construct units of family type 2 and 6
        CALL FAMTYPE26_CONSTRUCT(hh_i)
        !construct units of family type 3, 4 and 5       
        CALL FAMTYPE345_CONSTRUCT(hh_i)
        !construct units of family type 7       
        CALL FAMTYPE7_CONSTRUCT(hh_i)
        !continue construction of unit of family type 1 
        CALL FAMTYPE1_CONSTRUCT_CONTINUE(hh_i)
        !Switch members of couples in a uniform manner
        CALL FAMTYPES_HEADS(hh_i)
    END DO  

    !test construction of family relations
    CALL FAMREL_TEST

    !Write output to output file
    CALL FAMREL_OUTPUT_FILEWRIT

    !Write output for labsupply data set
!    CALL WRITELAB_FAMREL

    !Deallocate allocated variables
    CALL FAMREL_DEALLOC
    CLOSE(15)

END SUBROUTINE FAMREL_CALC

END MODULE FAMREL_SUBROUTINES
