!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declares all variables used in the program   !       
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!  - Control Variables                                     !
! and initializes constants                                !
!==========================================================!

MODULE EVAL_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=50                                             !   
    ! number of lines with input-controls                                    !   
    INTEGER, PARAMETER::n_lineval=14                                         !
    ! year of analysis                                                       !
    INTEGER, PARAMETER::year=2008                                            !
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=1                                              !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !========================================================================!

    !==========================================================================
    !Define variables to read locations and input parameters
    !Parameters for locating input 
    CHARACTER(len=150)                                      :: EVALRUN_HEADING                      !Heading of the evaluation run, stored in the first line of EVAL_LOCATION.txt
    CHARACTER(len=150)                                      :: EVAL_PARAMETERS                      !Location of the file with evaluation parameters
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where the file EVAL_INPUT.txt and other simulation independent information is stored
    CHARACTER(len=150)                                      :: EVAL_INPUT_FILE                      !File name of EVAL_INPUT.txt   
    CHARACTER(len=150)                                      :: EVAL_INPUT                           !full location of EVAL_INPUT.txt
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of the MIMOSIS_WEIGHT file                    
    CHARACTER(len=150)                                      :: EVAL_WEIGHT                          !full location of MIMOSIS_WEIGHT.txt
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_FILE                    !Filename of the file with global budget figures for evaluation year for employees
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_REFY                    !full location of the file with global budget figures for evaluation year for employees
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_FILE                !Filename of the file with global budget figures for evaluation year for self employed
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_REFY                !full location of the file with global budget figures for evaluation year for self employed
    CHARACTER(len=150)                                      :: EVAL_CONTRIB_PRE                     !Location of the file with the pre reform contrib variables
    CHARACTER(len=150)                                      :: EVAL_PIT_PRE                         !Location of the file with the pre reform pit variables    
    CHARACTER(len=150)                                      :: EVAL_SOCBEN_PRE                      !Location of the file with the pre reform MINIMEX variables    
    CHARACTER(len=150)                                      :: WORKHOUR_PRE                         !Location of the workhour parameter file with the pre reform location    
    CHARACTER(len=150)                                      :: NDISPINC_PARLOC_PRE                  !Location of file with net disposable income per individual of pre reform
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_PRE                   !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module) (with pre reform values)
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_PRE                   !Internal name to store Path+Name of file with MIMOSIS gross wages (with pre reform values)
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_PRE                     !location of the file with global budget figures for pre reform for employees
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_PRE                 !location of the file with global budget figures for pre reform for self employed
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_POST                    !location of the file with global budget figures for post reform for employees
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_POST                !location of the file with global budget figures for post reform for self employed
    CHARACTER(len=150)                                      :: EVAL_CONTRIB_POST                    !Location of the file with the post reform contrib variables
    CHARACTER(len=150)                                      :: EVAL_PIT_POST                        !Location of the file with the post reform pit variables    
    CHARACTER(len=150)                                      :: EVAL_SOCBEN_POST                     !Location of the file with the post reform MINIMEX variables    
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_POST                  !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module) (with post reform values)
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_POST                  !Internal name to store Path+Name of file with MIMOSIS gross wages (with post reform values)
    CHARACTER(len=150)                                      :: WORKHOUR_POST                        !Location of the workhour parameter file with the post reform location    
    CHARACTER(len=150)                                      :: NDISPINC_PARLOC_POST                 !Location of file with net disposable income per individual of post reform
    !Parameters for locating output (local variables)
    CHARACTER(len=150)                                      :: EVAL_OUTPUT_PATH                     !Path name where the output file of the evaluation is stored
    CHARACTER(len=150)                                      :: EVAL_OUTPUT_FILE                     !File name of the output file   
    CHARACTER(len=150)                                      :: EVAL_OUTPUT                          !full location of EVAL_OUTPUT.txt
    CHARACTER(len=150)                                      :: EVAL_LOG                             !Location of the log file with information on the evalution
    !Tools to read input, output and parameters
    CHARACTER(len=150),DIMENSION(n_lineval)                 :: CONTROL_EVAL                         !vector with input and output path and file names
    CHARACTER(len=10)                                       :: EVAL_PAR_CURR                        !Currency read from the paramter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: EVAL_PAR_NAMES                       !Vector of parameter names, dimension is n_par= number of lines in the paramter file including the start and stop line
    INTEGER                                                 :: EVAL_PAR_L                           !Number of lines (if parameter is a matrix)
    INTEGER                                                 :: EVAL_PAR_C                           !Number of columns (if parameter is a matrix)
    CHARACTER(len=20)                                       :: EVAL_PAR_FORMAT                      !Parameters' format
    CHARACTER(len=1)                                        :: EVAL_PAR_BREAK='@'                   !Character used to identify end of string
    INTEGER,DIMENSION(9)                                    :: EVAL_PAR_BREAK_POINT                  !Vector registering end of string
    !==========================================================================
    !Define type of parameters
    REAL(KIND=KIND_REAL)                                    :: EVAL_PRECIS                          !Precision level to classify changes as "no change"
    INTEGER                                                 :: EVAL_TYPEES_WELF                     !Type of equivalence scale to be selected for the welfare concept
    INTEGER                                                 :: EVAL_TYPEES_ANAL                     !Type of equivalence scale to be selected for the concept of analysis used within the tables
    INTEGER                                                 :: EVAL_TYPEPOV                         !Type of poverty line to be used
    INTEGER                                                 :: EVAL_NUMPPERC                        !number of poverty percentages used in the vector EVAL_POVPERC
    REAL(KIND=KIND_REAL),ALLOCATABLE, DIMENSION (:)         :: EVAL_POVPERC                         !vector with percentages to be used to compute the poverty line
    REAL(KIND=KIND_REAL),ALLOCATABLE, DIMENSION (:)         :: EVAL_POVABS                          !Vector with absolute poverty line values
    INTEGER                                                 :: EVAL_SUBSET                          !Indicator saying whether the analysis is to be carried out for a subset only yes or no
    INTEGER                                                 :: EVAL_NQUANT                          !Number of quantiles to be produced   
    REAL(KIND=KIND_REAL),ALLOCATABLE, DIMENSION (:)         :: EVAL_ES                              !Parameters necessary to compute equivalence scale
    REAL(KIND=KIND_REAL),ALLOCATABLE, DIMENSION (:,:)       :: EVAL_MATRIX                          !Example of parameter in matrix form, no special content, figures expressed in Euros
    REAL(KIND=KIND_REAL)                                    :: EVAL_MINNINC                         !Minimal amount of income to be earned on the labour market by a potential dependent before being split off (used to create economic independents)
    INTEGER                                                 :: EVAL_INCLMIN                         !Include the simulated amount of social benefits in the welfare concept (1=yes, 0 is no)    
    !==========================================================================
    !EVAL_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_CIVIL                       ! Burgerlijke staat van het individu op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_COMMUNIT                    ! NIS gemeentecode van de woonplaats waar het individu op 1 januari 2009 woonde
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_COLL                        ! Aanduiding of het gezin waartoe het individu op 1 januari 2009 behoort een collectief gezin is
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RND_SLCT                    ! Aanduiding of het individu geselecteerd werd bij het trekken van de steekproef (= ja of nee)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HEADOFHH                    ! Aanduiding of het individu op 1 januari 2009 gezinshoofd is van het gezin waartoe het individu behoort
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden

    !Define variables exogenous for the EVAL module but produced by the CONTRIB module in the PRE REFORM run
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: GECODEERD_INSZ_PRECONT               !Personal identification number in the pre reform CONTRIB file
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_QT_PRE               !Gross taxable labour income earned as wage earner on the private labour market (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_QT_PRE                !Gross taxable labour income earned as wage earner on the public labour market (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_QT_PRE               !Gross taxable labour income earned as self employed (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_QT_PRE               !Gross taxable pension benefits (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_QT_PRE              !Gross taxable early retirement benefits of the new type (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_QT_PRE              !Gross taxable early retirement benefits of the old type (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_QT_PRE              !Gross taxable unemployment benefits of older unemployed with seniority supplement (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_QT_PRE              !Gross taxable unemployment benefits, other than early retirement benefits (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_QT_PRE              !Gross taxable disability benefits of wage earners (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_QT_PRE              !Gross taxable benefits in case of industrial accidents or occupational diseases (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_QT_PRE               !Gross taxable benefits in case of sickness other than disability benefits (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_QT_PRE              !Gross taxable income from family allowances (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT_PRE           !Employee social security contributions paid for employee in the private labour market (PRE reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT_PRE            !Employee social security contributions paid for employee in the public labour market (PRE reform values)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT_PRE           !Social security contributions net of deductions paid by self employed (PRE reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMER_QT_PRE           !Employer social security contributions paid for employee in the private labour market (PRE reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMER_QT_PRE            !Employer social security contributions paid for employee in the public labour market (PRE reform values)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSREPLAC_QT_PRE              !Social security contributions paid on all sorts of replacement income (PRE reform values)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFEXTR_QT_PRE              !Labour income, earned as self employed and not subjected to contribution laws (=extra amount imputed on the basis of external sources) (PRE reform values)
    
    !Define variables exogenous for the EVAL module but produced by the PIT module in the PRE REFORM run
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: GECODEERD_INSZ_PREPIT                !Personal identification number in the pre reform PIT file 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TURANK_ANN_PRE                   !Rank of the individual within the tax unit the individual belongs to (PRE reform values) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TUNUMB_ANN_PRE                   !Unique identification number of the tax unit (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_FINTAX_ANN_PRE                   !Sum of all personal incomes taxes paid by the tax unit (PRE reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_TUWEIGHT_PRE                     !Weights to inflate the tax units to the level of the tax population

    !Define variables exogenous for the EVAL module but produced by the MINIMEX module in the PRE REFORM run
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PREMINM               !Personal identification number in the PRE reform MINIMEX file
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MINIMEX_PRE                   !minimex received by individual (PRE reform values)   

    !Variables read from file MIMOSIS_FAMREL.txt                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMPRE                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY_PRE                    !Unique identifier identifying the family to which the individual belongs (PRE reform values) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION_PRE                  !Relation of the individual with respect to the head of the family (PRE reform values) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE_PRE                   !Type of family to which the individual belongs (PRE reform values) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE_PRE                    !Type of couple to which the individual belongs (PRE reform values) 

    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal

    !Define variables exogenous for the EVAL module but produced by the CONTRIB module in the POST REFORM run
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: GECODEERD_INSZ_POSTCONT              !Personal identification number in the post reform CONTRIB file
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_QT_POST              !Gross taxable labour income earned as wage earner on the private labour market (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_QT_POST               !Gross taxable labour income earned as wage earner on the public labour market (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_QT_POST              !Gross taxable labour income earned as self employed (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_QT_POST              !Gross taxable pension benefits (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_QT_POST             !Gross taxable early retirement benefits of the new type (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_QT_POST             !Gross taxable early retirement benefits of the old type (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_QT_POST             !Gross taxable unemployment benefits of older unemployed with seniority supplement (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_QT_POST             !Gross taxable unemployment benefits, other than early retirement benefits (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_QT_POST             !Gross taxable disability benefits of wage earners (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_QT_POST             !Gross taxable benefits in case of industrial accidents or occupational diseases (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_QT_POST              !Gross taxable benefits in case of sickness other than disability benefits (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_QT_POST             !Gross taxable income from family allowances (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT_POST          !Employee social security contributions paid for employee in the private labour market (POST reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT_POST           !Employee social security contributions paid for employee in the public labour market (POST reform values)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT_POST          !Social security contributions net of deductions paid by self employed (POST reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMER_QT_POST          !Employer social security contributions paid for employee in the private labour market (POST reform values)           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMER_QT_POST           !Employer social security contributions paid for employee in the public labour market (POST reform values)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSREPLAC_QT_POST             !Social security contributions paid on all sorts of replacement income (POST reform values)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SELFEXTR_QT_POST             !Labour income, earned as self employed and not subjected to contribution laws (=extra amount imputed on the basis of external sources) (POST reform values)

    !Define variables exogenous for the EVAL module but produced by the PIT module in the POST REFORM run
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_POSTPIT               !Personal identification number in the post reform PIT file
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TURANK_ANN_POST                  !Rank of the individual within the tax unit the individual belongs to (POST reform values) 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TUNUMB_ANN_POST                  !Unique identification number of the tax unit (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_FINTAX_ANN_POST                  !Sum of all personal incomes taxes paid by the tax unit (POST reform values) 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_TUWEIGHT_POST                    !Weights to inflate the tax units to the level of the tax population

    !Define variables exogenous for the EVAL module but produced by the MINIMEX module in the POST REFORM run
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_POSTMINM              !Personal identification number in the post reform MINIMEX file
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MINIMEX_POST                  !minimex received by individual (POST reform values)   

    !endogeneous variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_HH_SIZE                         !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_START_HH                        !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_END_HH                          !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_EVAL                            !number of households analysed

    !Intermediate 'construction' variables
    INTEGER                                                 :: i,j,k,n                              !index variables to index other loops than individual or household loops
    INTEGER                                                 :: pers_j                               !index variable to loop over individuals
    INTEGER                                                 :: hh_i                                 !index variable to loop over households   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    INTEGER                                                 :: n_rec_sub                            !number of individuals in the subgroup

    !Variables introduced because of routines on distributional characteristics
    CHARACTER(len=25),DIMENSION(4)                          :: CHARNAME                             !Vector containing the title of analysis to be carried out    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPRE                         !disposable income before reform
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPOST                        !disposable income after reform    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_D1DISP                          !change in disposable income due to reform
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_D1DISP_NOES                     !non equivalised change in disposable income due to reform
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPRE_BTRAN                   !disposable income before reform before transfers
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPOST_BTRAN                  !disposable income after reform before transfers    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_D1DISP_BTRAN                    !change in disposable income due to reform before transfers
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPRE_NPENS                   !disposable income after pension transfers before all other transfers
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPPOST_NPENS                  !disposable income after pension transfers before all other transfers    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_D1DISP_NPENS                    !change in disposable income due to reform after pension transfers before all other transfers
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_WELFPRE                         !Welfare pre reform (= equivalised disposable income of the sociological household)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_WELFPOST                        !Welfare post reform (= equivalised disposable income of the sociological household)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_DECNR                           !Vector to store the decile number result
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_AGECLASS                        !Vector to store the age class of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_REGION                          !Vector to store the region class of the individual    
    REAL(KIND=KIND_REAL)                                    :: GINI0,GINI1                          !Gini pre and post
    INTEGER                                                 :: IPAR,IFILE                           !indexes to index FGT parameters and output file
    REAL(KIND=KIND_REAL)                                    :: FGTPAR                               !REAL(KIND=KIND_REAL) value for FGT computation containing conversion of integer index in IPAR
    REAL(KIND=KIND_REAL),DIMENSION(0:2)                     :: FGT0                                 !Vector containing FGT computations on pre reform data
    REAL(KIND=KIND_REAL),DIMENSION(0:2)                     :: FGT1                                 !Vector containing FGT computations on post reform data
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: GAMMA0V                              !Vector of size n_rec containing computations for pre reform data necessary to compute FGT index
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: GAMMA1V                              !Vector of size n_rec containing computations for post reform data necessary to compute FGT index
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_YEAR_AGE                        !Years of age
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_MONTH_AGE                       !Month of age (from last full year)
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_NADULT                          !Number of adults in the household
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_NKIDS                           !Number of kids (below the age of 14 in the household)
    REAL(KIND=DPRECI)                                       :: EVAL_POVLINE_PRE                     !variable that contains the pre reform eval poverty line value selected by the user (= result of different user options)
    REAL(KIND=DPRECI)                                       :: EVAL_POVLINE_POST                    !variable that contains the pre reform eval poverty line value selected by the user (= result of different user options)
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_TYPEPROF                        !Type professioneel statuut
    REAL(KIND=KIND_REAL),DIMENSION(7,4)                     :: SUMMARY_TABLE                        !matrix in which all figures for the summary table are stored
    INTEGER                                                 :: COUNT_TAB                            !Count the number of tables that already has been produced in the program
    CHARACTER(len=100)                                      :: LABES_ANAL                           !Label of the equivalence scale applied on the concept of analysis
    INTEGER,ALLOCATABLE,DIMENSION (:)                       :: EVAL_SUBGROUP                        !This variable contains an indicator for each individual whether the individual should be taken into acount for the analysis later on, yes or no (0=yes, 1=no)
    
    !computation net income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_RESTPRE                         !Vector of size n_rec containing the remaining amount of taxes not subtracted from the income of the head of the tax unit (pre reform)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_RESTPOST                        !Vector of size n_rec containing the remaining amount of taxes not subtracted from the income of the head of the tax unit (post reform)

    !Creation of economic independent units
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_ECIRANK_ANN                     !Rank of the individual within the economic independent unit the individual belongs to
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_ECINUMB_ANN                     !Unique identification number of the economic independent unit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_ECICOUP_ANN                     !Individual has a partner that is comprised in the economic independent unit as well
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: EVAL_CHECK_ANN                       !Vector to store identification of a check
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_DISPLAB_PRE                     !Vector of size n_rec containing disposable income from labour market activity (pre reform)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: EVAL_RESTLAB_PRE                     !Vector of size n_rec containing the remaining amount of taxes not subtracted from the income of the head of the tax unit (pre reform)

    !Intstruments
    INTEGER                                                 :: NUNIT,ierror
    
END MODULE EVAL_DATA

!============== MODULE EVAL_SUBROUTINES ====================================!
!This module contains all subroutines and functions used in the EVAL module
!===========================================================================!

MODULE EVAL_SUBROUTINES

    USE EVAL_DATA
    USE MIMOSIS_DATA      
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!===============================================!
!Allocate all vectors of dimension DIM_ALLOC and intialise all variables to a predefined value!
!===============================================!

SUBROUTINE EVAL_ALLOC(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC

    !EVAL_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    ALLOCATE(NAREGNIS_CIVIL(DIM_ALLOC))
    ALLOCATE(NAREGNIS_COMMUNIT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_COLL(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RND_SLCT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HEADOFHH(DIM_ALLOC))
        
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

    !Allocate variables exogenous for the EVAL module but produced by the CONTRIB module in the PRE REFORM run
    ALLOCATE(GECODEERD_INSZ_PRECONT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPRIV_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPUB_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISELF_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPENS_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEARNE_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEAROL_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOLDUN_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOTHUN_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIDISAB_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTINDOCC_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISICK_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIFAMAL_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT_PRE(DIM_ALLOC))   
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT_PRE(DIM_ALLOC))    
    ALLOCATE(CONTRIB_SSSELF_EMER_QT_PRE(DIM_ALLOC))   
    ALLOCATE(CONTRIB_SSPRIV_EMER_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMER_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSREPLAC_QT_PRE(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFEXTR_QT_PRE(DIM_ALLOC))
   !Allocate variables exogenous for the EVAL module but produced by the PIT module in the PRE REFORM run
    ALLOCATE(GECODEERD_INSZ_PREPIT(DIM_ALLOC))
    ALLOCATE(PIT_TURANK_ANN_PRE(DIM_ALLOC))
    ALLOCATE(PIT_TUNUMB_ANN_PRE(DIM_ALLOC))
    ALLOCATE(PIT_FINTAX_ANN_PRE(DIM_ALLOC))
    ALLOCATE(PIT_TUWEIGHT_PRE(DIM_ALLOC))
    !Allocate variables exogenous for the EVAL module but produced by the MINIMEX module in the PRE REFORM run
    ALLOCATE(GECODEERD_INSZ_PREMINM(DIM_ALLOC))
    ALLOCATE(SOCBEN_MINIMEX_PRE(DIM_ALLOC))   
    !Variables read from file MIMOSIS_FAMREL.txt in the PRE REFORM run                                          
    ALLOCATE(GECODEERD_INSZ_FAMPRE(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY_PRE(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION_PRE(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE_PRE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE_PRE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))

    !Allocate variables exogenous for the EVAL module but produced by the CONTRIB module in the POST REFORM run
    ALLOCATE(GECODEERD_INSZ_POSTCONT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPRIV_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPUB_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISELF_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPENS_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEARNE_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEAROL_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOLDUN_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOTHUN_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIDISAB_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTINDOCC_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISICK_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIFAMAL_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT_POST(DIM_ALLOC))   
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT_POST(DIM_ALLOC))    
    ALLOCATE(CONTRIB_SSSELF_EMER_QT_POST(DIM_ALLOC))   
    ALLOCATE(CONTRIB_SSPRIV_EMER_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMER_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSREPLAC_QT_POST(DIM_ALLOC))
    ALLOCATE(CONTRIB_SELFEXTR_QT_POST(DIM_ALLOC))
    !Allocate variables exogenous for the EVAL module but produced by the PIT module in the POST REFORM run
    ALLOCATE(GECODEERD_INSZ_POSTPIT(DIM_ALLOC))
    ALLOCATE(PIT_TURANK_ANN_POST(DIM_ALLOC))
    ALLOCATE(PIT_TUNUMB_ANN_POST(DIM_ALLOC))
    ALLOCATE(PIT_FINTAX_ANN_POST(DIM_ALLOC))
    ALLOCATE(PIT_TUWEIGHT_POST(DIM_ALLOC))
    !Allocate variables exogenous for the EVAL module but produced by the MINIMEX module in the POST REFORM run
    ALLOCATE(GECODEERD_INSZ_POSTMINM(DIM_ALLOC))
    ALLOCATE(SOCBEN_MINIMEX_POST(DIM_ALLOC))   

    !Allocate endogeneous variables
    ALLOCATE(EVAL_HH_SIZE(DIM_ALLOC))
    ALLOCATE(EVAL_START_HH(DIM_ALLOC))
    ALLOCATE(EVAL_END_HH(DIM_ALLOC))
    !Allocate intermediate 'construction' variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !Variables introduced similar to DARTS variables
    ALLOCATE(EVAL_DISPPRE(DIM_ALLOC))    
    ALLOCATE(EVAL_DISPPOST(DIM_ALLOC))        
    ALLOCATE(EVAL_D1DISP(DIM_ALLOC))    
    ALLOCATE(EVAL_D1DISP_NOES(DIM_ALLOC))    
    ALLOCATE(EVAL_DISPPRE_BTRAN(DIM_ALLOC))    
    ALLOCATE(EVAL_DISPPOST_BTRAN(DIM_ALLOC))        
    ALLOCATE(EVAL_D1DISP_BTRAN(DIM_ALLOC))    
    ALLOCATE(EVAL_DISPPRE_NPENS(DIM_ALLOC))    
    ALLOCATE(EVAL_DISPPOST_NPENS(DIM_ALLOC))        
    ALLOCATE(EVAL_D1DISP_NPENS(DIM_ALLOC))    
    ALLOCATE(EVAL_WELFPRE(DIM_ALLOC))
    ALLOCATE(EVAL_WELFPOST(DIM_ALLOC))
    ALLOCATE(EVAL_DECNR(DIM_ALLOC))    
    ALLOCATE(EVAL_AGECLASS(DIM_ALLOC))        
    ALLOCATE(EVAL_REGION(DIM_ALLOC))            
    ALLOCATE(GAMMA0V(DIM_ALLOC))    
    ALLOCATE(GAMMA1V(DIM_ALLOC))    
    ALLOCATE(EVAL_YEAR_AGE(DIM_ALLOC))    
    ALLOCATE(EVAL_MONTH_AGE(DIM_ALLOC))    
    ALLOCATE(EVAL_NADULT(DIM_ALLOC))    
    ALLOCATE(EVAL_NKIDS(DIM_ALLOC))    
    ALLOCATE(EVAL_TYPEPROF(DIM_ALLOC))    
    ALLOCATE(EVAL_SUBGROUP(DIM_ALLOC))    

    !Computation net income
    ALLOCATE(EVAL_RESTPRE(DIM_ALLOC)) 
    ALLOCATE(EVAL_RESTPOST(DIM_ALLOC)) 

    !Creation of economic independent units
    ALLOCATE(EVAL_ECIRANK_ANN(DIM_ALLOC)) 
    ALLOCATE(EVAL_ECINUMB_ANN(DIM_ALLOC)) 
    ALLOCATE(EVAL_ECICOUP_ANN(DIM_ALLOC)) 
    ALLOCATE(EVAL_CHECK_ANN(DIM_ALLOC))
    ALLOCATE(EVAL_DISPLAB_PRE(DIM_ALLOC))
    ALLOCATE(EVAL_RESTLAB_PRE(DIM_ALLOC))    

END SUBROUTINE EVAL_ALLOC

!===============================================!
!DeAllocate all vectors of dimension DIM_ALLOC !
!===============================================!

SUBROUTINE EVAL_DEALLOC

    !EVAL_INPUT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_RELATION)
    DEALLOCATE(NAREGNIS_SEX)
    DEALLOCATE(NAREGNIS_CIVIL)
    DEALLOCATE(NAREGNIS_COMMUNIT)
    DEALLOCATE(NAREGNIS_COLL)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_RND_SLCT)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_HEADOFHH)
        
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !DEALLOCATE variables exogenous for the EVAL module but produced by the CONTRIB module in the PRE REFORM run
    DEALLOCATE(GECODEERD_INSZ_PRECONT)
    DEALLOCATE(CONTRIB_GTIPRIV_QT_PRE)
    DEALLOCATE(CONTRIB_GTIPUB_QT_PRE)
    DEALLOCATE(CONTRIB_GTISELF_QT_PRE)
    DEALLOCATE(CONTRIB_GTIPENS_QT_PRE)
    DEALLOCATE(CONTRIB_GTIEARNE_QT_PRE)
    DEALLOCATE(CONTRIB_GTIEAROL_QT_PRE)
    DEALLOCATE(CONTRIB_GTIOLDUN_QT_PRE)
    DEALLOCATE(CONTRIB_GTIOTHUN_QT_PRE)
    DEALLOCATE(CONTRIB_GTIDISAB_QT_PRE)
    DEALLOCATE(CONTRIB_GTINDOCC_QT_PRE)
    DEALLOCATE(CONTRIB_GTISICK_QT_PRE)
    DEALLOCATE(CONTRIB_GTIFAMAL_QT_PRE)
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT_PRE) 
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT_PRE)    
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT_PRE)  
    DEALLOCATE(CONTRIB_SSPRIV_EMER_QT_PRE)
    DEALLOCATE(CONTRIB_SSPUB_EMER_QT_PRE)
    DEALLOCATE(CONTRIB_SSREPLAC_QT_PRE)
    DEALLOCATE(CONTRIB_SELFEXTR_QT_PRE)
    !DEALLOCATE variables exogenous for the EVAL module but produced by the PIT module in the PRE REFORM run
    DEALLOCATE(GECODEERD_INSZ_PREPIT)
    DEALLOCATE(PIT_TURANK_ANN_PRE)
    DEALLOCATE(PIT_TUNUMB_ANN_PRE)
    DEALLOCATE(PIT_FINTAX_ANN_PRE)
    DEALLOCATE(PIT_TUWEIGHT_PRE)
    !Deallocate variables exogenous for the EVAL module but produced by the MINIMEX module in the PRE REFORM run
    DEALLOCATE(GECODEERD_INSZ_PREMINM)
    DEALLOCATE(SOCBEN_MINIMEX_PRE)
    !Variables read from file MIMOSIS_FAMREL.txt in the PRE REFORM run                                           
    DEALLOCATE(GECODEERD_INSZ_FAMPRE)
    DEALLOCATE(FAMREL_FAMILY_PRE)
    DEALLOCATE(FAMREL_RELATION_PRE)
    DEALLOCATE(FAMREL_FAMTYPE_PRE)
    DEALLOCATE(FAMREL_COUPLE_PRE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)

    !DEALLOCATE variables exogenous for the EVAL module but produced by the CONTRIB module in the POST REFORM run
    DEALLOCATE(GECODEERD_INSZ_POSTCONT)
    DEALLOCATE(CONTRIB_GTIPRIV_QT_POST)
    DEALLOCATE(CONTRIB_GTIPUB_QT_POST)
    DEALLOCATE(CONTRIB_GTISELF_QT_POST)
    DEALLOCATE(CONTRIB_GTIPENS_QT_POST)
    DEALLOCATE(CONTRIB_GTIEARNE_QT_POST)
    DEALLOCATE(CONTRIB_GTIEAROL_QT_POST)
    DEALLOCATE(CONTRIB_GTIOLDUN_QT_POST)
    DEALLOCATE(CONTRIB_GTIOTHUN_QT_POST)
    DEALLOCATE(CONTRIB_GTIDISAB_QT_POST)
    DEALLOCATE(CONTRIB_GTINDOCC_QT_POST)
    DEALLOCATE(CONTRIB_GTISICK_QT_POST)
    DEALLOCATE(CONTRIB_GTIFAMAL_QT_POST) 
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT_POST) 
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT_POST)    
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT_POST)  
    DEALLOCATE(CONTRIB_SSPRIV_EMER_QT_POST)
    DEALLOCATE(CONTRIB_SSPUB_EMER_QT_POST)
    DEALLOCATE(CONTRIB_SSREPLAC_QT_POST)
    DEALLOCATE(CONTRIB_SELFEXTR_QT_POST)
    !DEALLOCATE variables exogenous for the EVAL module but produced by the PIT module in the POST REFORM run
    DEALLOCATE(GECODEERD_INSZ_POSTPIT)
    DEALLOCATE(PIT_TURANK_ANN_POST)
    DEALLOCATE(PIT_TUNUMB_ANN_POST)
    DEALLOCATE(PIT_FINTAX_ANN_POST)
    DEALLOCATE(PIT_TUWEIGHT_POST)
    !Deallocate variables exogenous for the EVAL module but produced by the MINIMEX module in the PRE REFORM run
    DEALLOCATE(GECODEERD_INSZ_POSTMINM)
    DEALLOCATE(SOCBEN_MINIMEX_POST)
    !DEALLOCATE endogeneous variables
    DEALLOCATE(EVAL_HH_SIZE)
    DEALLOCATE(EVAL_START_HH)
    DEALLOCATE(EVAL_END_HH)
    !DEALLOCATE Intermediate variables
    DEALLOCATE(NEW_RANK)
    !Variables introduced because of DARTS routines
    DEALLOCATE(EVAL_DISPPRE)
    DEALLOCATE(EVAL_DISPPOST)
    DEALLOCATE(EVAL_D1DISP)    
    DEALLOCATE(EVAL_D1DISP_NOES)    
    DEALLOCATE(EVAL_DISPPRE_BTRAN)
    DEALLOCATE(EVAL_DISPPOST_BTRAN)
    DEALLOCATE(EVAL_D1DISP_BTRAN)    
    DEALLOCATE(EVAL_DISPPRE_NPENS)
    DEALLOCATE(EVAL_DISPPOST_NPENS)
    DEALLOCATE(EVAL_D1DISP_NPENS)    
    DEALLOCATE(EVAL_WELFPRE)
    DEALLOCATE(EVAL_WELFPOST)
    DEALLOCATE(EVAL_DECNR)    
    DEALLOCATE(EVAL_AGECLASS)        
    DEALLOCATE(EVAL_REGION)
    DEALLOCATE(GAMMA0V)
    DEALLOCATE(GAMMA1V)
    DEALLOCATE(EVAL_YEAR_AGE)
    DEALLOCATE(EVAL_MONTH_AGE)
    DEALLOCATE(EVAL_NADULT)
    DEALLOCATE(EVAL_NKIDS)
    DEALLOCATE(EVAL_TYPEPROF)
    DEALLOCATE(EVAL_SUBGROUP)

    !Computation net income
    DEALLOCATE(EVAL_RESTPRE)
    DEALLOCATE(EVAL_RESTPOST)

    !Creation of economic independent units
    DEALLOCATE(EVAL_ECIRANK_ANN)
    DEALLOCATE(EVAL_ECINUMB_ANN)
    DEALLOCATE(EVAL_ECICOUP_ANN) 
    DEALLOCATE(EVAL_CHECK_ANN)
    DEALLOCATE(EVAL_DISPLAB_PRE)
    DEALLOCATE(EVAL_RESTLAB_PRE)    

    !Deallocate allocated parameters
    DEALLOCATE(EVAL_POVPERC)                         
    DEALLOCATE(EVAL_POVABS)
    DEALLOCATE(EVAL_ES)
    DEALLOCATE(EVAL_MATRIX)

END SUBROUTINE EVAL_DEALLOC

!========== SUBROUTINE EVAL_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!======================================================!

SUBROUTINE EVAL_RANKANDSORT(n_size)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis

!STEP1: ranking and sorting with the INTEGER value of the individuals identification number
    !Rank individual identification variable read from the input file 
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_size),NEW_RANK)
    !EVAL_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COLL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RND_SLCT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HEADOFHH(1:n_rec),NEW_RANK)
    !Rank individual identification variable read from the file with sample weights
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_size),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)
    !Rank individual identification variable read from the CONTRIB module in the PRE REFORM run
    CALL RANKING(GECODEERD_INSZ_PRECONT(1:n_size),NEW_RANK)
    !Sort variables read from the CONTRIB pre reform file according to the individual identification number
    CALL SORTING(GECODEERD_INSZ_PRECONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_PRE(1:n_size),NEW_RANK)
    !Rank individual identification variable read from the PIT module in the PRE REFORM run
    CALL RANKING(GECODEERD_INSZ_PREPIT(1:n_size),NEW_RANK)
    !Sort variables read from the PIT pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_PRE(1:n_size),NEW_RANK)
    !Rank individual identification variable read from the MINIMEX module in the PRE REFORM run
    CALL RANKING(GECODEERD_INSZ_PREMINM(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_PRE(1:n_size),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt in the PRE REFORM run                                           
    CALL RANKING(GECODEERD_INSZ_FAMPRE(1:n_size),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMPRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_RELATION_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE_PRE(1:n_size),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt in the PRE REFORM run                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_size),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification variable read from the CONTRIB module in the POST REFORM run
    CALL RANKING(GECODEERD_INSZ_POSTCONT(1:n_size),NEW_RANK)
    !Sort variables read from the CONTRIB post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTCONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_POST(1:n_size),NEW_RANK)
    !Rank individual identification variable read from the PIT module in the POST REFORM run
    CALL RANKING(GECODEERD_INSZ_POSTPIT(1:n_size),NEW_RANK)
    !Sort variables read from the PIT post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_POST(1:n_size),NEW_RANK)
    !Rank individual identification variable read from the MINIMEX module in the POST REFORM run
    CALL RANKING(GECODEERD_INSZ_POSTMINM(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX POST reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_POST(1:n_size),NEW_RANK)
    
!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL EVAL_CONSISTENCY_CHECK(n_size)

!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_size),NEW_RANK)
    !EVAL_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COLL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RND_SLCT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HEADOFHH(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB pre reform file according to the individual identification number
    CALL SORTING(GECODEERD_INSZ_PRECONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_PRE(1:n_size),NEW_RANK)
    !Sort variables read from the PIT pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_PRE(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_PRE(1:n_size),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt pre reform file according to the individual identification number                                              
    CALL SORTING(GECODEERD_INSZ_FAMPRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_RELATION_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE_PRE(1:n_size),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTCONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_POST(1:n_size),NEW_RANK)
    !Sort variables read from the PIT post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_POST(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX POST reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_POST(1:n_size),NEW_RANK)
    
!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL EVAL_CONSISTENCY_CHECK(n_size)

END SUBROUTINE EVAL_RANKANDSORT

!========== SUBROUTINE EVAL_RANKANDSORT_SUB ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to the variable
!           that identifies the subgroup
!   Step 2: rank and sort all variables for individuals within the subgroup according to the 
!           household identification number
!======================================================!

SUBROUTINE EVAL_RANKANDSORT_SUB(n_size,n_size_sub)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis
    INTEGER         :: n_size_sub       !number of individuals over the households that are considered for analysis in the subgroup
    
!STEP1: ranking and sorting with the subgroup identifier
    !Rank household number
    CALL RANKING(EVAL_SUBGROUP(1:n_size),NEW_RANK)
    !EVAL_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COLL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RND_SLCT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HEADOFHH(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB pre reform file according to the individual identification number
    CALL SORTING(GECODEERD_INSZ_PRECONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_PRE(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_PRE(1:n_size),NEW_RANK)
    !Sort variables read from the PIT pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_PRE(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_PRE(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_PRE(1:n_size),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt pre reform file according to the individual identification number                                           
    CALL SORTING(GECODEERD_INSZ_FAMPRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_RELATION_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE_PRE(1:n_size),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE_PRE(1:n_size),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTCONT(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_POST(1:n_size),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_POST(1:n_size),NEW_RANK)
    !Sort variables read from the PIT post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTPIT(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_POST(1:n_size),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_POST(1:n_size),NEW_RANK)
    !Sort variables read from the MINIMEX POST reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTMINM(1:n_size),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_POST(1:n_size),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL EVAL_CONSISTENCY_CHECK(n_size)

!STEP2: ranking and sorting the subgroup with the household identifier
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_size_sub),NEW_RANK)
    !EVAL_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COLL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RND_SLCT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HEADOFHH(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB pre reform file according to the individual identification number
    CALL SORTING(GECODEERD_INSZ_PRECONT(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_PRE(1:n_size_sub),NEW_RANK)
    !Sort variables read from the PIT pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREPIT(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_PRE(1:n_size_sub),NEW_RANK)
    !Sort variables read from the MINIMEX pre reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_PREMINM(1:n_size_sub),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_PRE(1:n_size_sub),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt pre reform file according to the individual identification number                                               
    CALL SORTING(GECODEERD_INSZ_FAMPRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(FAMREL_RELATION_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE_PRE(1:n_size_sub),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE_PRE(1:n_size_sub),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Sort variables read from the CONTRIB post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTCONT(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMER_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMER_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SSREPLAC_QT_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(CONTRIB_SELFEXTR_QT_POST(1:n_size_sub),NEW_RANK)
    !Sort variables read from the PIT post reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTPIT(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TURANK_ANN_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TUNUMB_ANN_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_FINTAX_ANN_POST(1:n_size_sub),NEW_RANK)
    CALL SORTING(PIT_TUWEIGHT_POST(1:n_size_sub),NEW_RANK)
    !Sort variables read from the MINIMEX POST reform file according to the individual identification number    
    CALL SORTING(GECODEERD_INSZ_POSTMINM(1:n_size_sub),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_POST(1:n_size_sub),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL EVAL_CONSISTENCY_CHECK(n_size_sub)

END SUBROUTINE EVAL_RANKANDSORT_SUB

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE EVAL_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                               !index for do loop

    !Initialise endogenous variables that are non character
    DO pers_j=1,DIM_ALLOC
        !Initialise endogeneous variables
        EVAL_HH_SIZE(pers_j)=0
        EVAL_START_HH(pers_j)=0
        EVAL_END_HH(pers_j)=0
        !Initialise intermediate variables
        NEW_RANK(pers_j)=0
        !Variables introduced because of DARTS routines
        EVAL_DISPPRE(pers_j)=0                
        EVAL_DISPPOST(pers_j)=0                        
        EVAL_D1DISP(pers_j)=0                
        EVAL_D1DISP_NOES(pers_j)=0                
        EVAL_DISPPRE_BTRAN(pers_j)=0                
        EVAL_DISPPOST_BTRAN(pers_j)=0                        
        EVAL_D1DISP_BTRAN(pers_j)=0                
        EVAL_DISPPRE_NPENS(pers_j)=0                
        EVAL_DISPPOST_NPENS(pers_j)=0                        
        EVAL_D1DISP_NPENS(pers_j)=0                
        EVAL_WELFPRE(pers_j)=0
        EVAL_WELFPOST(pers_j)=0
        EVAL_DECNR(pers_j)=0        
        EVAL_AGECLASS(pers_j)=0                
        EVAL_REGION(pers_j)=0                        
        GAMMA0V(pers_j)=0        
        GAMMA1V(pers_j)=0        
        EVAL_D1DISP(pers_j)=0                
        EVAL_YEAR_AGE(pers_j)=0                
        EVAL_MONTH_AGE(pers_j)=0                
        EVAL_NADULT(pers_j)=0                
        EVAL_NKIDS(pers_j)=0                
        EVAL_TYPEPROF(pers_j)=0                
        EVAL_SUBGROUP(pers_j)=0
        !Computation net income
        EVAL_RESTPRE(pers_j)=0
        EVAL_RESTPOST(pers_j)=0
        !Creation of economic independent units
        EVAL_ECIRANK_ANN(pers_j)=0
        EVAL_ECINUMB_ANN(pers_j)=0
        EVAL_ECICOUP_ANN(pers_j)=0 
        EVAL_CHECK_ANN(pers_j)=0
        EVAL_DISPLAB_PRE(pers_j)=0
        EVAL_RESTLAB_PRE(pers_j)=0    
    END DO   
    SUMMARY_TABLE(1:7,1:4)=0.D0
        
END SUBROUTINE EVAL_INITENDO

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from         !
!the parameter file                                    !
!======================================================!

SUBROUTINE EVAL_PARAMETER_FILEREAD

    INTEGER :: i,j,k
    !open parameter file
    OPEN(UNIT=3, FILE=EVAL_PARAMETERS, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_PARAMETERS
        WRITE(15,'("Could not find ", a100)') EVAL_PARAMETERS
        PAUSE
        STOP
    END IF        

    ALLOCATE(EVAL_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    i=1
    DO
        READ(3,'(a40,2i10,a10)') EVAL_PAR_NAMES(i),EVAL_PAR_L,EVAL_PAR_C,EVAL_PAR_CURR
        IF (EVAL_PAR_NAMES(i)=='EVAL_PRECIS') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_PRECIS
            WRITE(15,EVAL_PAR_FORMAT) EVAL_PRECIS
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_TYPEES_WELF') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_TYPEES_WELF
            WRITE(15,EVAL_PAR_FORMAT) EVAL_TYPEES_WELF
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_TYPEES_ANAL') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_TYPEES_ANAL
            WRITE(15,EVAL_PAR_FORMAT) EVAL_TYPEES_ANAL
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_TYPEPOV') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_TYPEPOV
            WRITE(15,EVAL_PAR_FORMAT) EVAL_TYPEPOV
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_NUMPPERC') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_NUMPPERC
            WRITE(15,EVAL_PAR_FORMAT) EVAL_NUMPPERC
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_POVPERC')THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            ALLOCATE (EVAL_POVPERC(EVAL_PAR_L)) 
            READ (3,'(a20)') EVAL_PAR_FORMAT
            READ (3,EVAL_PAR_FORMAT)  (EVAL_POVPERC(j),j=1,EVAL_PAR_L) 
            WRITE(15,EVAL_PAR_FORMAT) (EVAL_POVPERC(j),j=1,EVAL_PAR_L)
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_POVABS')THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            ALLOCATE (EVAL_POVABS(EVAL_PAR_L)) 
            READ (3,'(a20)') EVAL_PAR_FORMAT
            READ (3,EVAL_PAR_FORMAT)  (EVAL_POVABS(j),j=1,EVAL_PAR_L) 
            WRITE(15,EVAL_PAR_FORMAT) (EVAL_POVABS(j),j=1,EVAL_PAR_L)
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_SUBSET') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_SUBSET
            WRITE(15,EVAL_PAR_FORMAT) EVAL_SUBSET
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_NQUANT') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_NQUANT
            WRITE(15,EVAL_PAR_FORMAT) EVAL_NQUANT
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_ES')THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            ALLOCATE (EVAL_ES(EVAL_PAR_L)) 
            READ (3,'(a20)') EVAL_PAR_FORMAT
            READ (3,EVAL_PAR_FORMAT)  (EVAL_ES(j),j=1,EVAL_PAR_L) 
            WRITE(15,EVAL_PAR_FORMAT) (EVAL_ES(j),j=1,EVAL_PAR_L)
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_MATRIX')THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            ALLOCATE (EVAL_MATRIX(EVAL_PAR_L,EVAL_PAR_C)) 
            READ (3,'(a20)') EVAL_PAR_FORMAT
            READ (3,EVAL_PAR_FORMAT)  ((EVAL_MATRIX(j,k),j=1,EVAL_PAR_L),k=1,EVAL_PAR_C) 
            WRITE(15,EVAL_PAR_FORMAT) ((EVAL_MATRIX(j,k),j=1,EVAL_PAR_L),k=1,EVAL_PAR_C)
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_MINNINC') THEN
             WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
             READ(3,'(a20)') EVAL_PAR_FORMAT
             READ(3,EVAL_PAR_FORMAT) EVAL_MINNINC
             WRITE(15,EVAL_PAR_FORMAT) EVAL_MINNINC
        ELSE IF (EVAL_PAR_NAMES(i)=='EVAL_INCLMIN') THEN
            WRITE(15,'(a40)') EVAL_PAR_NAMES(i)
            READ(3,'(a20)') EVAL_PAR_FORMAT
            READ(3,EVAL_PAR_FORMAT) EVAL_INCLMIN
            WRITE(15,EVAL_PAR_FORMAT) EVAL_INCLMIN
        ELSE IF (EVAL_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(EVAL_PAR_NAMES)

    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine EVAL_PARAMETER_FILEREAD....................runned correctly'

END SUBROUTINE EVAL_PARAMETER_FILEREAD

!========== SUBROUTINE READ_EVAL_LOCATION =======================================!
!This subroutines reads the location input files that are not simulation specific
!i.e. files not related to either the pre or post reform situation
!REMARK:    for all the othre modules, the read parameter location subroutine
!           is part of MIMOSIS_subroutines and uses variables that are declared public
!REMARK:    the variable EVAL_START is similar to MIMOSIS_PARLPRE and MIMOSIS_PARLPOST
!REMARK:    the log file for the evaluation module is used for both the log of the locations
!           read from EVAL_LOCATION as for the parameters. For the other modules this output
!           is spread over two files
!REMARK:    n_rec is now read by each "run" separatly, i.e. for pre reform, post reform and
!           evaluation. No testing is done on the number of records in the pre and post reform
!           although it is assumed that the pre and post reform files have the same number of records
!================================================================================!

SUBROUTINE EVAL_LOCATION_FILEREAD

    !open parameter file with locations of files other than the pre and post reform files
    OPEN(UNIT=3, FILE=EVAL_START, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_START
        PAUSE
        STOP
    END IF        

    READ (3,'(a150)')       EVALRUN_HEADING
    WRITE (*,'(1X,a150)')   EVALRUN_HEADING                     
    !read location of parameter-files of the evaluation module
    READ(3,'(a150)') EVAL_PARAMETERS
    READ(3,'(a150)') EVAL_LOG        

    !Read in parameters for path and name of input files
    DO i=1,n_lineval
        READ(3,'(a150)') CONTROL_EVAL(i)
    END DO

    !Read in parameters for path and name of input files
    DO i=1,n_lineval/2
        IF (CONTROL_EVAL(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_EVAL(2*i)
        ELSE IF (CONTROL_EVAL(2*i-1)=='EVAL_INPUT_FILE') THEN
            EVAL_INPUT_FILE = CONTROL_EVAL(2*i)
        ELSE IF (CONTROL_EVAL(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_EVAL(2*i)
        ELSE IF (CONTROL_EVAL(2*i-1)=='EVAL_OUTPUT_PATH') THEN
            EVAL_OUTPUT_PATH = CONTROL_EVAL(2*i)
        ELSE IF (CONTROL_EVAL(2*i-1)=='EVAL_OUTPUT_FILE') THEN
            EVAL_OUTPUT_FILE = CONTROL_EVAL(2*i)
        ELSE IF (CONTROL_EVAL(2*i-1)=='GLOBBUD_EMPL_FILE') THEN
            GLOBBUD_EMPL_FILE = CONTROL_EVAL(2*i)        
        ELSE IF (CONTROL_EVAL(2*i-1)=='GLOBBUD_SELFEMPL_FILE') THEN
            GLOBBUD_SELFEMPL_FILE = CONTROL_EVAL(2*i)        
        END IF
    END DO     

    !check for length of default path and file name parameters
    EVAL_PAR_BREAK_POINT(1) = INDEX(CONSTANT_INPUT_PATH,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(2) = INDEX(EVAL_INPUT_FILE,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(3) = INDEX(MIMOSIS_SAMPWEIG_FILE,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(4) = INDEX(EVAL_OUTPUT_PATH,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(5) = INDEX(EVAL_OUTPUT_FILE,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(6) = INDEX(GLOBBUD_EMPL_FILE,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(7) = INDEX(GLOBBUD_SELFEMPL_FILE,EVAL_PAR_BREAK)

    !merge information (path+name) to create default input file names
    EVAL_INPUT = CONSTANT_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &EVAL_INPUT_FILE(1:EVAL_PAR_BREAK_POINT(2)-1)
    EVAL_WEIGHT = CONSTANT_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:EVAL_PAR_BREAK_POINT(3)-1)
    EVAL_OUTPUT = EVAL_OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(4)-1)//&
    &EVAL_OUTPUT_FILE(1:EVAL_PAR_BREAK_POINT(5)-1)
    GLOBBUD_EMPL_REFY = CONSTANT_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &GLOBBUD_EMPL_FILE(1:EVAL_PAR_BREAK_POINT(6)-1)
    GLOBBUD_SELFEMPL_REFY = CONSTANT_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &GLOBBUD_SELFEMPL_FILE(1:EVAL_PAR_BREAK_POINT(7)-1)

    !Trim the result
    CALL TRIM@(EVAL_INPUT)
    CALL TRIM@(EVAL_WEIGHT)
    CALL TRIM@(EVAL_OUTPUT)
    CALL TRIM@(GLOBBUD_EMPL_REFY)
    CALL TRIM@(GLOBBUD_SELFEMPL_REFY)

    !Write read result to log file
    OPEN(unit=15, FILE=EVAL_LOG, STATUS='REPLACE', ACTION='WRITE')
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_LOG
        WRITE(15,'("Could not find ", a100)') EVAL_LOG
        PAUSE
        STOP
    END IF        

    WRITE (15,'(a150)') EVALRUN_HEADING
    WRITE (15,'(i10)')  N_REC
    WRITE (15,'(a150)') EVAL_PARAMETERS
    WRITE (15,'(a150)') EVAL_LOG        
    !Write parameters for path and name of input files
    DO i=1,n_lineval
        WRITE(15,'(a150)') CONTROL_EVAL(i)
    END DO
    !Write final results to log file
    WRITE(15,*) EVAL_START
    WRITE(15,*) EVAL_INPUT
    WRITE(15,*) EVAL_WEIGHT
    WRITE(15,*) EVAL_OUTPUT
    WRITE(15,*) GLOBBUD_EMPL_REFY
    WRITE(15,*) GLOBBUD_SELFEMPL_REFY
    CLOSE(UNIT=15)  

    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine EVAL_LOCATION_FILEREAD.....................runned correctly'
    
END SUBROUTINE EVAL_LOCATION_FILEREAD

!===============================================!
!Read in all lines that contain information on the location of paramter files of simulation run
!===============================================!

SUBROUTINE SIMUL_LOCATION_READ(LOCFILE,TYPERUN)

    CHARACTER(len=150)                                      :: LOCFILE                              !Location where the parameter file should be found 
    CHARACTER(len=14)                                       :: TYPERUN                              !Variable that contains description of the MIMOSIS parameter file that should be read

    !open parameter file after the location has been found            
    OPEN (UNIT=3, FILE=LOCFILE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    !check for status of parameter file
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') LOCFILE
        WRITE(15,'("Could not find ", a100)') LOCFILE
        PAUSE
        STOP
    END IF   
    
    !skip first five lines, they are not in control
    CALL RSKIP(3,5)
    !read in parameters for path and name of input files  
    DO i=1,N_LOCLINES
        !read in parameters for path and name of input files  
        READ(3,'(a150)') CONTROL_IO(i)  
    END DO
    CLOSE (UNIT=3)
    !Write to screen
    WRITE(*,*) 'FILE MIMOSIS_PARAMETERS.txt '//TYPERUN//'..............read correctly'  

END SUBROUTINE  SIMUL_LOCATION_READ

!===============================================!
!Determine location of PIT, CONTRIB and MINIMEX files that are produced by a simulation run
!===============================================!

SUBROUTINE MODOUTP_LOCATION_READ(RESULT_FAMREL,RESULT_WAGE,RESULT_WORKHOUR,RESULT_MINIMEX,RESULT_CONTRIB,RESULT_PIT,&
&GLOBBUD_EMPL,GLOBBUD_SELFEMPL,TYPERUN)

    CHARACTER(len=150)                                      :: RESULT_WORKHOUR                      !name of the file where the work-hour parameters are stored    
    CHARACTER(len=150)                                      :: EVAL_PIT_FILE                        !intermediate variable: file name of the file with the pit variables    
    CHARACTER(len=150)                                      :: RESULT_PIT                           !name that stores the final result with the pit variables    
    CHARACTER(len=150)                                      :: EVAL_CONTRIB_FILE                    !intermediate variable: file name of the file with the contrib variables
    CHARACTER(len=150)                                      :: RESULT_CONTRIB                       !name that stores the final result with the contrib variables
    CHARACTER(len=150)                                      :: EVAL_SOCBEN_FILE                     !intermediate variable: file name of the file with the minimex variables
    CHARACTER(len=150)                                      :: RESULT_MINIMEX                       !name that stores the final result with the minimex variables
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: RESULT_FAMREL                        !name of the file where the famrel information is stored   
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: RESULT_WAGE                          !name of the file where the wages are stored    
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL_FILE                    !file name of file with computed global budget figures for employees
    CHARACTER(len=150)                                      :: GLOBBUD_EMPL                         !location of file with computed global budget figures for employees
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL_FILE                !file name of file with computed global budget figures for self employed
    CHARACTER(len=150)                                      :: GLOBBUD_SELFEMPL                     !location of file with computed global budget figures for self employed
    CHARACTER(len=14)                                       :: TYPERUN                              !Variable that contains description of the MIMOSIS parameter file that should be read

    !Run through the vector CONTROL_IO
    DO i=1,n_loclines/2
        IF (CONTROL_IO(2*i-1)=='OUTPUT_PATH')THEN
            OUTPUT_PATH = CONTROL_IO(2*i)  
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_PIT_FILE')THEN
            EVAL_PIT_FILE = CONTROL_IO(2*i)   
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_CONTRIB_FILE') THEN
            EVAL_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_SOCBEN_FILE') THEN
            EVAL_SOCBEN_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='GLOBBUD_EMPL_FILE')THEN
            GLOBBUD_EMPL_FILE = CONTROL_IO(2*i)   
        ELSE IF (CONTROL_IO(2*i-1)=='GLOBBUD_SELFEMPL_FILE') THEN
            GLOBBUD_SELFEMPL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WORKHOUR_PARLOC') THEN 
            RESULT_WORKHOUR = CONTROL_IO(2*i)
        END IF 
    END DO
    !check for length of path and file name parameters 
    EVAL_PAR_BREAK_POINT(1)=INDEX(OUTPUT_PATH,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(2)=INDEX(EVAL_PIT_FILE,EVAL_PAR_BREAK)                          
    EVAL_PAR_BREAK_POINT(3)=INDEX(EVAL_CONTRIB_FILE,EVAL_PAR_BREAK)                          
    EVAL_PAR_BREAK_POINT(4)=INDEX(EVAL_SOCBEN_FILE,EVAL_PAR_BREAK)                          
    EVAL_PAR_BREAK_POINT(5)=INDEX(GLOBBUD_EMPL_FILE,EVAL_PAR_BREAK)                          
    EVAL_PAR_BREAK_POINT(6)=INDEX(GLOBBUD_SELFEMPL_FILE,EVAL_PAR_BREAK)                          
    EVAL_PAR_BREAK_POINT(7)=INDEX(FAMREL_INPUT_PATH,EVAL_PAR_BREAK)
    EVAL_PAR_BREAK_POINT(8)=INDEX(MIMOSIS_FAMREL_FILE,EVAL_PAR_BREAK)    
    EVAL_PAR_BREAK_POINT(9)=INDEX(WAGE_CONSTRUCT_FILE,EVAL_PAR_BREAK)
    !merge infomration (path+name) to create default output file name     
    RESULT_PIT=OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &EVAL_PIT_FILE(1:EVAL_PAR_BREAK_POINT(2)-1)
    RESULT_CONTRIB=OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &EVAL_CONTRIB_FILE(1:EVAL_PAR_BREAK_POINT(3)-1)
    RESULT_MINIMEX=OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &EVAL_SOCBEN_FILE(1:EVAL_PAR_BREAK_POINT(4)-1)
    GLOBBUD_EMPL=OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &GLOBBUD_EMPL_FILE(1:EVAL_PAR_BREAK_POINT(5)-1)
    GLOBBUD_SELFEMPL=OUTPUT_PATH(1:EVAL_PAR_BREAK_POINT(1)-1)//&
    &GLOBBUD_SELFEMPL_FILE(1:EVAL_PAR_BREAK_POINT(6)-1)
    RESULT_FAMREL = FAMREL_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(7)-1)//&
    &MIMOSIS_FAMREL_FILE(1:EVAL_PAR_BREAK_POINT(8)-1)    
    RESULT_WAGE = FAMREL_INPUT_PATH(1:EVAL_PAR_BREAK_POINT(7)-1)//&
    &WAGE_CONSTRUCT_FILE(1:EVAL_PAR_BREAK_POINT(9)-1)
    CALL TRIM@(RESULT_PIT)
    CALL TRIM@(RESULT_CONTRIB)
    CALL TRIM@(RESULT_MINIMEX)    
    CALL TRIM@(GLOBBUD_EMPL)
    CALL TRIM@(GLOBBUD_SELFEMPL)    
    CALL TRIM@(RESULT_FAMREL)
    CALL TRIM@(RESULT_WAGE)    
    !write names to log file
    WRITE (15,*) 'PIT FILE '//TYPERUN//'=', RESULT_PIT
    WRITE (15,*) 'CONTRIB FILE '//TYPERUN//'=', RESULT_CONTRIB
    WRITE (15,*) 'MINIMEX FILE '//TYPERUN//'=', RESULT_MINIMEX   
    WRITE (15,*) 'WORKHOUR FILE '//TYPERUN//'=', RESULT_WORKHOUR
    WRITE (15,*) 'GLOBBUD_EMPL FILE '//TYPERUN//'=', GLOBBUD_EMPL
    WRITE (15,*) 'GLOBBUD_SELFEMPL FILE '//TYPERUN//'=', GLOBBUD_SELFEMPL
    WRITE (15,*) 'RESULT_FAMREL '//TYPERUN//'=', RESULT_FAMREL
    WRITE (15,*) 'RESULT_WAGE '//TYPERUN//'=', RESULT_WAGE
    
    !Write to screen
    WRITE(*,*) 'FILENAMES OF SIMULATION files '//TYPERUN//'......have been determined'  

END SUBROUTINE  MODOUTP_LOCATION_READ

!==================================!
!Reads in data from the INPUT file !
!==================================!

SUBROUTINE EVAL_INPUT_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_INPUT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_INPUT
        WRITE(15,'("Could not find ", a100)') EVAL_INPUT
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,I10,I10,I10,I10,I10,I10,A7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
            &NAREGNIS_CIVIL(pers_j),NAREGNIS_COMMUNIT(pers_j),NAREGNIS_COLL(pers_j),&
            &NAREGNIS_HH_ID(pers_j),NAREGNIS_RND_SLCT(pers_j),NAREGNIS_DATBIRTH(pers_j),&
            &NAREGNIS_HEADOFHH(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_CIVIL(pers_j),NAREGNIS_COMMUNIT(pers_j),NAREGNIS_COLL(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_RND_SLCT(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_HEADOFHH(pers_j)'')')
                WRITE(15,'(A14,I10,I10,I10,I10,I10,I10,I10,A7,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_CIVIL(pers_j),NAREGNIS_COMMUNIT(pers_j),NAREGNIS_COLL(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_RND_SLCT(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_HEADOFHH(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,I10,I10,I10,I10,I10,I10,A7,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_CIVIL(pers_j),NAREGNIS_COMMUNIT(pers_j),NAREGNIS_COLL(pers_j),&
                &NAREGNIS_HH_ID(pers_j),NAREGNIS_RND_SLCT(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_HEADOFHH(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_INPUT_FILEREAD .......................runned correctly'
    WRITE (15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE EVAL_INPUT_FILEREAD 

!====================================!
!Reads in data from the EVAL_WEIGHT file !
!====================================!

SUBROUTINE EVAL_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_WEIGHT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_WEIGHT
        WRITE(15,'("Could not find ", a100)') EVAL_WEIGHT
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_WEIGHT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_WEIGHT_FILEREAD.......................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_WEIGHT")') pers_j-1 

END SUBROUTINE EVAL_WEIGHT_FILEREAD

!============================================================!
!Reads in data from the PIT file of PRE Reform               !
!============================================================!

SUBROUTINE EVAL_PIT_PRE_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_PIT_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_PIT_PRE
        WRITE(15,'("Could not find ", a100)') EVAL_PIT_PRE
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,I10,F15.2,F15.10)') GECODEERD_INSZ_PREPIT(pers_j),PIT_TURANK_ANN_PRE(pers_j),&
            &PIT_TUNUMB_ANN_PRE(pers_j),PIT_FINTAX_ANN_PRE(pers_j),PIT_TUWEIGHT_PRE(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_PIT_PRE
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_PIT_PRE_FILEREAD......................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_PIT_FILEREAD_PRE ")') pers_j-1

END SUBROUTINE EVAL_PIT_PRE_FILEREAD

!=============================================================!
!Reads in data from the PIT file of POST Reform               !
!=============================================================!

SUBROUTINE EVAL_PIT_POST_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_PIT_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_PIT_POST
        WRITE(15,'("Could not find ", a100)') EVAL_PIT_POST
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,I10,F15.2,F15.10)') GECODEERD_INSZ_POSTPIT(pers_j),PIT_TURANK_ANN_POST(pers_j),&
            &PIT_TUNUMB_ANN_POST(pers_j),PIT_FINTAX_ANN_POST(pers_j),PIT_TUWEIGHT_POST(pers_j) 
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_PIT_POST
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_PIT_POST_FILEREAD.....................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_PIT_FILEREAD_POST ")') pers_j-1

END SUBROUTINE EVAL_PIT_POST_FILEREAD

!=================================================!
!Reads in data from the CONTRIB file of PRE Reform!
!=================================================!

SUBROUTINE EVAL_CONTRIB_PRE_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_CONTRIB_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_CONTRIB_PRE
        WRITE(15,'("Could not find ", a100)') EVAL_CONTRIB_PRE
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
            &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
            &GECODEERD_INSZ_PRECONT(pers_j),CONTRIB_GTIPRIV_QT_PRE(pers_j),CONTRIB_GTIPUB_QT_PRE(pers_j),&
            &CONTRIB_GTISELF_QT_PRE(pers_j),CONTRIB_GTIPENS_QT_PRE(pers_j),CONTRIB_GTIEARNE_QT_PRE(pers_j),&
            &CONTRIB_GTIEAROL_QT_PRE(pers_j),CONTRIB_GTIOLDUN_QT_PRE(pers_j),CONTRIB_GTIOTHUN_QT_PRE(pers_j),&
            &CONTRIB_GTIDISAB_QT_PRE(pers_j),CONTRIB_GTINDOCC_QT_PRE(pers_j),CONTRIB_GTISICK_QT_PRE(pers_j),& 
            &CONTRIB_GTIFAMAL_QT_PRE(pers_j),CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j),CONTRIB_SSPUB_EMEE_QT_PRE(pers_j),&
            &CONTRIB_SSSELF_EMER_QT_PRE(pers_j),CONTRIB_SSPRIV_EMER_QT_PRE(pers_j),CONTRIB_SSPUB_EMER_QT_PRE(pers_j),&
            &CONTRIB_SSREPLAC_QT_PRE(pers_j),CONTRIB_SELFEXTR_QT_PRE(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_CONTRIB_PRE
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_CONTRIB_PRE_FILEREAD..................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_CONTRIB_PRE_FILEREAD ")') pers_j-1

END SUBROUTINE EVAL_CONTRIB_PRE_FILEREAD

!==================================================!
!Reads in data from the CONTRIB file of POST Reform!
!==================================================!

SUBROUTINE EVAL_CONTRIB_POST_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_CONTRIB_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_CONTRIB_POST
        WRITE(15,'("Could not find ", a100)') EVAL_CONTRIB_POST
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
            &F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
            &GECODEERD_INSZ_POSTCONT(pers_j),CONTRIB_GTIPRIV_QT_POST(pers_j),CONTRIB_GTIPUB_QT_POST(pers_j),&
            &CONTRIB_GTISELF_QT_POST(pers_j),CONTRIB_GTIPENS_QT_POST(pers_j),CONTRIB_GTIEARNE_QT_POST(pers_j),&
            &CONTRIB_GTIEAROL_QT_POST(pers_j),CONTRIB_GTIOLDUN_QT_POST(pers_j),CONTRIB_GTIOTHUN_QT_POST(pers_j),&
            &CONTRIB_GTIDISAB_QT_POST(pers_j),CONTRIB_GTINDOCC_QT_POST(pers_j),CONTRIB_GTISICK_QT_POST(pers_j),& 
            &CONTRIB_GTIFAMAL_QT_POST(pers_j),CONTRIB_SSPRIV_EMEE_QT_POST(pers_j),CONTRIB_SSPUB_EMEE_QT_POST(pers_j),&
            &CONTRIB_SSSELF_EMER_QT_POST(pers_j),CONTRIB_SSPRIV_EMER_QT_POST(pers_j),CONTRIB_SSPUB_EMER_QT_POST(pers_j),&
            &CONTRIB_SSREPLAC_QT_POST(pers_j),CONTRIB_SELFEXTR_QT_POST(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_CONTRIB_POST
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_CONTRIB_POST_FILEREAD.................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_CONTRIB_POST_FILEREAD ")') pers_j-1

END SUBROUTINE EVAL_CONTRIB_POST_FILEREAD

!=================================================!
!Reads in data from the MINIMEX file of PRE Reform!
!=================================================!

SUBROUTINE EVAL_SOCBEN_PRE_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_SOCBEN_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_SOCBEN_PRE
        WRITE(15,'("Could not find ", a100)') EVAL_SOCBEN_PRE
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2)',IOSTAT=ierror) GECODEERD_INSZ_PREMINM(pers_j),SOCBEN_MINIMEX_PRE(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_SOCBEN_PRE
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_SOCBEN_PRE_FILEREAD...................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_SOCBEN_FILEREAD_PRE ")') pers_j-1

END SUBROUTINE EVAL_SOCBEN_PRE_FILEREAD

!==================================================!
!Reads in data from the MINIMEX file of POST Reform!
!==================================================!

SUBROUTINE EVAL_SOCBEN_POST_FILEREAD

    OPEN (UNIT=5, FILE=EVAL_SOCBEN_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_SOCBEN_POST
        WRITE(15,'("Could not find ", a100)') EVAL_SOCBEN_POST
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2)',IOSTAT=ierror) GECODEERD_INSZ_POSTMINM(pers_j),SOCBEN_MINIMEX_POST(pers_j)
            IF (ierror > 0) THEN
                WRITE (15,*) 'An error occurred in line ', pers_j, 'of file ', EVAL_SOCBEN_POST
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_SOCBEN_POST_FILEREAD..................runned correctly'
    WRITE (15,'(I7, " records uploaded from EVAL_SOCBEN_FILEREAD_POST ")') pers_j-1

END SUBROUTINE EVAL_SOCBEN_POST_FILEREAD

!============================================================!
!Reads in data from the WORKHOUR parameter file of PRE Reform!
!============================================================!

SUBROUTINE EVAL_WORKHOUR_PRE_FILEREAD

    CHARACTER(len=150)                                      :: HEADER                               !read in header of the file (=first line)

    OPEN (UNIT=5, FILE=WORKHOUR_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') WORKHOUR_PRE
        WRITE(15,'("Could not find ", a100)') WORKHOUR_PRE
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
        READ (5,'(a150)')   HEADER
        !read location of file where net disposable income per individual needs to be stored
        READ(5,'(a150)')    NDISPINC_PARLOC_PRE
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_WORKHOUR_PRE_FILEREAD.................runned correctly'

END SUBROUTINE EVAL_WORKHOUR_PRE_FILEREAD

!=============================================================!
!Reads in data from the WORKHOUR parameter file of POST Reform!
!=============================================================!

SUBROUTINE EVAL_WORKHOUR_POST_FILEREAD

    CHARACTER(len=150)                                      :: HEADER                               !read in header of the file (=first line)

    OPEN (UNIT=5, FILE=WORKHOUR_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') WORKHOUR_POST
        WRITE(15,'("Could not find ", a100)') WORKHOUR_POST
        PAUSE
        STOP
    END IF   
    IF (ierror==0) THEN
         READ (5,'(a150)')   HEADER
        !read location of file where net disposable income per individual needs to be stored
        READ(5,'(a150)')    NDISPINC_PARLOC_POST
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_WORKHOUR_POST_FILEREAD................runned correctly'

END SUBROUTINE EVAL_WORKHOUR_POST_FILEREAD

!=====================================!
!Reads in data from the FAMREL file of the pre reform  !
!=====================================!

SUBROUTINE EVAL_FAMREL_PRE_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_FAMREL_PRE
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_FAMREL_PRE
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMPRE(pers_j),FAMREL_FAMILY_PRE(pers_j),FAMREL_RELATION_PRE(pers_j),&
            &FAMREL_FAMTYPE_PRE(pers_j),FAMREL_COUPLE_PRE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL_PRE
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMPRE(pers_j),FAMREL_FAMILY_PRE(pers_j),FAMREL_RELATION_PRE(pers_j),&
                &FAMREL_FAMTYPE_PRE(pers_j),FAMREL_COUPLE_PRE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMPRE(pers_j),FAMREL_FAMILY_PRE(pers_j),FAMREL_RELATION_PRE(pers_j),&
                &FAMREL_FAMTYPE_PRE(pers_j),FAMREL_COUPLE_PRE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMPRE(pers_j),FAMREL_FAMILY_PRE(pers_j),FAMREL_RELATION_PRE(pers_j),&
                &FAMREL_FAMTYPE_PRE(pers_j),FAMREL_COUPLE_PRE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_FAMREL_PRE_FILEREAD...................runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL_PRE")') pers_j-1

END SUBROUTINE EVAL_FAMREL_PRE_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file of the pre reform!
!=======================================!

SUBROUTINE EVAL_WAGE_PRE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') WAGE_CONSTRUCT_PRE
        WRITE (15,'("Could not find or open ", a100)') WAGE_CONSTRUCT_PRE
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT_PRE
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine EVAL_WAGE_PRE_FILEREAD.....................runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT_PRE")') pers_j-1 

END SUBROUTINE EVAL_WAGE_PRE_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE EVAL_CONSISTENCY_CHECK(n_size)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis
    INTEGER:: i

    DO i=1, n_size
        IF ((GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_WEIGHT(i))       .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_FAMPRE(i))       .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_WAGE(i))         .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_PRECONT(i))      .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_PREPIT(i))       .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_PREMINM(i))      .OR.&            
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_POSTCONT(i))     .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_POSTPIT(i))      .OR.&
            (GECODEERD_INSZ_INPUT(i).NE.GECODEERD_INSZ_POSTMINM(i))     .OR.&            
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_FAMPRE(i))      .OR.&
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_WAGE(i))        .OR.&
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_PRECONT(i))     .OR.&
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_PREPIT(i))      .OR.&
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_PREMINM(i))     .OR.&            
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_POSTCONT(i))    .OR.&            
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_POSTPIT(i))     .OR.&
            (GECODEERD_INSZ_WEIGHT(i).NE.GECODEERD_INSZ_POSTMINM(i))    .OR.&            
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_WAGE(i))        .OR.&
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_PRECONT(i))     .OR.&
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_PREPIT(i))      .OR.&
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_PREMINM(i))     .OR.&            
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_POSTCONT(i))    .OR.&            
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_POSTPIT(i))     .OR.&
            (GECODEERD_INSZ_FAMPRE(i).NE.GECODEERD_INSZ_POSTMINM(i))    .OR.&            
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_PRECONT(i))       .OR.&
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_PREPIT(i))        .OR.&
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_PREMINM(i))       .OR.&            
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_POSTCONT(i))      .OR.&            
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_POSTPIT(i))       .OR.&
            (GECODEERD_INSZ_WAGE(i).NE.GECODEERD_INSZ_POSTMINM(i))      .OR.&            
            (GECODEERD_INSZ_PRECONT(i).NE.GECODEERD_INSZ_PREPIT(i))     .OR.&
            (GECODEERD_INSZ_PRECONT(i).NE.GECODEERD_INSZ_PREMINM(i))    .OR.&
            (GECODEERD_INSZ_PRECONT(i).NE.GECODEERD_INSZ_POSTCONT(i))   .OR.&            
            (GECODEERD_INSZ_PRECONT(i).NE.GECODEERD_INSZ_POSTPIT(i))    .OR.&
            (GECODEERD_INSZ_PRECONT(i).NE.GECODEERD_INSZ_POSTMINM(i))   .OR.&
            (GECODEERD_INSZ_PREPIT(i).NE.GECODEERD_INSZ_PREMINM(i))     .OR.&            
            (GECODEERD_INSZ_PREPIT(i).NE.GECODEERD_INSZ_POSTCONT(i))    .OR.&            
            (GECODEERD_INSZ_PREPIT(i).NE.GECODEERD_INSZ_POSTPIT(i))     .OR.&
            (GECODEERD_INSZ_PREPIT(i).NE.GECODEERD_INSZ_POSTMINM(i))    .OR.&
            (GECODEERD_INSZ_POSTCONT(i).NE.GECODEERD_INSZ_POSTPIT(i))   .OR.&
            (GECODEERD_INSZ_POSTCONT(i).NE.GECODEERD_INSZ_POSTMINM(i))  .OR.&
            (GECODEERD_INSZ_POSTPIT(i).NE.GECODEERD_INSZ_POSTMINM(i))       )   THEN
            WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   

END SUBROUTINE EVAL_CONSISTENCY_CHECK 

!===============================================!
! HOUSEHOLD_SIZE computes household size      !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!===============================================!

SUBROUTINE EVAL_HOUSEHOLD_SIZE(n_size)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis

    I=1
    J=1
    K=0
    N=0
    
    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_size) EXIT    
        END DO
        EVAL_HH_SIZE(k)=n
        EVAL_START_HH(k)=j
        EVAL_END_HH(k)=j+n-1
        IF (i>n_size.OR.j>n_size)EXIT
    END DO
    N_HH_EVAL=K

END SUBROUTINE EVAL_HOUSEHOLD_SIZE

!===========================================
!Subroutine: to recode some variable values
!===========================================

SUBROUTINE EVAL_RECODE(n_size)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis
    INTEGER         :: pers_j

    DO pers_j=1,n_size
        !EVAL_INPUT_WRITE.SAS
        IF (INT(NAREGNIS_RELATION(pers_j)).EQ.9999999)           NAREGNIS_RELATION(pers_j)=0
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)                NAREGNIS_SEX(pers_j)=0
        IF (INT(NAREGNIS_CIVIL(pers_j)).EQ.9999999)              NAREGNIS_CIVIL(pers_j)=0
        IF (INT(NAREGNIS_COMMUNIT(pers_j)).EQ.9999999)           NAREGNIS_COMMUNIT(pers_j)=0
        IF (INT(NAREGNIS_COLL(pers_j)).EQ.9999999)               NAREGNIS_COLL(pers_j)=0
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)              NAREGNIS_HH_ID(pers_j)=0
        IF (INT(NAREGNIS_RND_SLCT(pers_j)).EQ.9999999)           NAREGNIS_RND_SLCT(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_HEADOFHH(pers_j)).EQ.9999999)           NAREGNIS_HEADOFHH(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)              MIMOSIS_WEIGHT(pers_j)=0
        !Sort variables read from the CONTRIB pre reform file according to the individual identification number
        IF (INT(CONTRIB_GTIPRIV_QT_PRE(pers_j)).EQ.9999999)      CONTRIB_GTIPRIV_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIPUB_QT_PRE(pers_j)).EQ.9999999)       CONTRIB_GTIPUB_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTISELF_QT_PRE(pers_j)).EQ.9999999)      CONTRIB_GTISELF_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIPENS_QT_PRE(pers_j)).EQ.9999999)      CONTRIB_GTIPENS_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIEARNE_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIEARNE_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIEAROL_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIEAROL_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIOLDUN_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIOLDUN_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIOTHUN_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIOTHUN_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIDISAB_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIDISAB_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTINDOCC_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTINDOCC_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTISICK_QT_PRE(pers_j)).EQ.9999999)      CONTRIB_GTISICK_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_GTIFAMAL_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_GTIFAMAL_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j)).EQ.9999999)  CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSPUB_EMEE_QT_PRE(pers_j)).EQ.9999999)   CONTRIB_SSPUB_EMEE_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSSELF_EMER_QT_PRE(pers_j)).EQ.9999999)  CONTRIB_SSSELF_EMER_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSPRIV_EMER_QT_PRE(pers_j)).EQ.9999999)  CONTRIB_SSPRIV_EMER_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSPUB_EMER_QT_PRE(pers_j)).EQ.9999999)   CONTRIB_SSPUB_EMER_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SSREPLAC_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_SSREPLAC_QT_PRE(pers_j) = 0
        IF (INT(CONTRIB_SELFEXTR_QT_PRE(pers_j)).EQ.9999999)     CONTRIB_SELFEXTR_QT_PRE(pers_j) = 0
        !Sort variables read from the PIT pre reform file according to the individual identification number    
        IF (INT(PIT_FINTAX_ANN_PRE(pers_j)).EQ.9999999)          PIT_FINTAX_ANN_PRE(pers_j) = 0
        !Sort variables read from the MINIMEX pre reform file according to the individual identification number    
        IF (INT(SOCBEN_MINIMEX_PRE(pers_j)).EQ.9999999)          SOCBEN_MINIMEX_PRE(pers_j) = 0
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)            SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)          MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)            MIMOSIS_GRINC_QT(pers_j)=0
        !Sort variables read from the CONTRIB post reform file according to the individual identification number    
        IF (INT(CONTRIB_GTIPRIV_QT_POST(pers_j)).EQ.9999999)     CONTRIB_GTIPRIV_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIPUB_QT_POST(pers_j)).EQ.9999999)      CONTRIB_GTIPUB_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTISELF_QT_POST(pers_j)).EQ.9999999)     CONTRIB_GTISELF_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIPENS_QT_POST(pers_j)).EQ.9999999)     CONTRIB_GTIPENS_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIEARNE_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIEARNE_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIEAROL_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIEAROL_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIOLDUN_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIOLDUN_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIOTHUN_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIOTHUN_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIDISAB_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIDISAB_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTINDOCC_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTINDOCC_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTISICK_QT_POST(pers_j)).EQ.9999999)     CONTRIB_GTISICK_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_GTIFAMAL_QT_POST(pers_j)).EQ.9999999)    CONTRIB_GTIFAMAL_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSPRIV_EMEE_QT_POST(pers_j)).EQ.9999999) CONTRIB_SSPRIV_EMEE_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSPUB_EMEE_QT_POST(pers_j)).EQ.9999999)  CONTRIB_SSPUB_EMEE_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSSELF_EMER_QT_POST(pers_j)).EQ.9999999) CONTRIB_SSSELF_EMER_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSPRIV_EMER_QT_POST(pers_j)).EQ.9999999) CONTRIB_SSPRIV_EMER_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSPUB_EMER_QT_POST(pers_j)).EQ.9999999)  CONTRIB_SSPUB_EMER_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SSREPLAC_QT_POST(pers_j)).EQ.9999999)    CONTRIB_SSREPLAC_QT_POST(pers_j) = 0
        IF (INT(CONTRIB_SELFEXTR_QT_POST(pers_j)).EQ.9999999)    CONTRIB_SELFEXTR_QT_POST(pers_j) = 0
        !Sort variables read from the PIT post reform file according to the individual identification number    
        IF (INT(PIT_FINTAX_ANN_POST(pers_j)).EQ.9999999)         PIT_FINTAX_ANN_POST(pers_j) = 0
        !Sort variables read from the MINIMEX POST reform file according to the individual identification number    
        IF (INT(SOCBEN_MINIMEX_POST(pers_j)).EQ.9999999)         SOCBEN_MINIMEX_POST(pers_j) = 0
        !Other weights
        IF (INT(PIT_TUWEIGHT_PRE(pers_j)).EQ.9999999)            PIT_TUWEIGHT_PRE(pers_j)=0
        IF (INT(PIT_TUWEIGHT_POST(pers_j)).EQ.9999999)           PIT_TUWEIGHT_POST(pers_j)=0
    END DO

END SUBROUTINE EVAL_RECODE

!============= SUBROUTINE EVAL_INFLATE===============================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE EVAL_INFLATE(n_size)

    INTEGER         :: n_size           !number of individuals over the households that are considered for analysis
    INTEGER         :: pers_j
 
    DO pers_j=1,n_size
        !==========================================================================
        !Define exogenous variables read from the input file 
        !Define exogenous variables read from the file with sample weights
        !Define variables exogenous for the EVAL module but produced by the CONTRIB module in the PRE REFORM run
        !Define variables exogenous for the EVAL module but produced by the PIT module in the PRE REFORM run
        !Define variables exogenous for the EVAL module but produced by the MINIMEX module in the PRE REFORM run
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !Variables read from file WAGE_CONSTRUCT.txt                                                     
        SALJRS_LAST_CORR(pers_j)    = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)    !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)  = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)  !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)    = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)    !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Define variables exogenous for the EVAL module but produced by the CONTRIB module in the POST REFORM run
        !Define variables exogenous for the EVAL module but produced by the PIT module in the POST REFORM run
        !Define variables exogenous for the EVAL module but produced by the MINIMEX module in the POST REFORM run
    END DO   

END SUBROUTINE EVAL_INFLATE

!**************************************************************************
!comment: construction of the variable EVAL_SUBGROUP
!This variable indicates wether the individuals should be kept for analysis later on
!yes or no
!Current rule:
! excluded all household members if 
! either there is a household member that is self employed
! or both head and partner have no income
!**************************************************************************
SUBROUTINE EVAL_SUBGROUP_DEFINITION

    INTEGER                 :: hh_i             !household identification index
    INTEGER                 :: pers_j,pers_k    !individual identification index
    REAL(KIND=KIND_REAL)    :: SumInc           !Construct intermediate variable that contains the sum of all income sources
    INTEGER,DIMENSION(2)    :: IncCond          !Contains indicator whether one of the spouses is without income
    INTEGER                 :: Partner          !indication that there is a partner of the head of the household

    DO hh_i=1,n_hh_eval
        !Check whether there is a self employed household member
!        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
!            IF ((HOEDRSVZ_QT(pers_j).EQ.'1'.OR.HOEDRSVZ_QT(pers_j).EQ.'2'.OR.&
!                &HOEDRSVZ_QT(pers_j).EQ.'3'.OR.HOEDRSVZ_QT(pers_j).EQ.'4').AND.&
!                &EVAL_SUBGROUP(pers_j).EQ.0) THEN 
!                DO pers_k=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
!                    EVAL_SUBGROUP(pers_k)=1
!                END DO  
!            END IF   
!        END DO
        !Check whether head of the household or spouse has no income apart from possible MINIMEX
        IncCond(1:2)=0
        Partner = 0
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            SumInc =                             &   
                &CONTRIB_GTIPRIV_QT_PRE(pers_j) +&
                &CONTRIB_GTIPUB_QT_PRE(pers_j)  +&
                &CONTRIB_GTISELF_QT_PRE(pers_j) +&
                &CONTRIB_GTIPENS_QT_PRE(pers_j) +&
                &CONTRIB_GTIEARNE_QT_PRE(pers_j)+&
                &CONTRIB_GTIEAROL_QT_PRE(pers_j)+&
                &CONTRIB_GTIOLDUN_QT_PRE(pers_j)+&
                &CONTRIB_GTIOTHUN_QT_PRE(pers_j)+&
                &CONTRIB_GTIDISAB_QT_PRE(pers_j)+&
                &CONTRIB_GTINDOCC_QT_PRE(pers_j)+&
                &CONTRIB_GTISICK_QT_PRE(pers_j) 
            IF (NAREGNIS_RELATION(pers_j).EQ.1.AND.NINT(SumInc*100.D0).EQ.0) THEN
                IncCond(1) = 1
            END IF
            IF (NAREGNIS_RELATION(pers_j).EQ.2) THEN
                Partner = 1
                IF (NINT(SumInc*100.D0).EQ.0) THEN
                    IncCond(2) = 1            
                END IF    
            END IF
        END DO
        !If both spouses are without income, the household is excluded from the analysis
        IF ((IncCond(1).EQ.1.AND.IncCond(2).EQ.1).OR.&
           &(IncCond(1).EQ.1.AND.Partner.EQ.0)) THEN
            DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
                EVAL_SUBGROUP(pers_j)=1
            END DO  
        END IF
    END DO

END SUBROUTINE EVAL_SUBGROUP_DEFINITION

!**************************************************************************
!Determine size of subgroup
!this part is separate from the subgropu definition to make the module work for
!full size if subgroup definition is commented out
!**************************************************************************

SUBROUTINE EVAL_SUBGROUP_SIZE
    
    !Determine value of n_rec_sub
    DO hh_i=1,n_hh_eval
        !Check whether either head of the household or partner is self employed in
        !main activity
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            IF (EVAL_SUBGROUP(pers_j).EQ.0) THEN
                n_rec_sub = n_rec_sub + 1
            END IF
        END DO
    END DO    

END SUBROUTINE EVAL_SUBGROUP_SIZE

!=================================================================
!Computes the age of each individual in terms of years
!=================================================================

!=============================================!
! Subroutine main input : EVAL_YEAR_BIRTH     ! 
!                         EVAL_MONTH_BIRTH    !
! Subroutine main output: EVAL_YEAR_AGE       !
!                         EVAL_MONTH_AGE      !  
!=============================================!

SUBROUTINE EVAL_AGE(n_size)

    INTEGER                 :: n_size   !number of individuals over the households that are considered for analysis
    INTEGER                                                 :: EVAL_YEAR_BIRTH                      !Intermediate construction variable that contains year of birth
    INTEGER                                                 :: EVAL_MONTH_BIRTH                     !Intermediate construction variable that contains month of birth
    INTEGER                                                 :: pers_j
    
    DO pers_j=1,n_size
        EVAL_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:EVAL_PAR_BREAK_POINT(1)-1),  '(I10)') EVAL_MONTH_BIRTH
        READ (NAREGNIS_DATBIRTH(pers_j)(EVAL_PAR_BREAK_POINT(1)+1:),   '(I10)') EVAL_YEAR_BIRTH         
        !determine age as number of years at the end of the year for which data are registered
        EVAL_YEAR_AGE(pers_j)  =YEAR-EVAL_YEAR_BIRTH
        !determine number of months as year age plus number of months passed until the end of the year
        EVAL_MONTH_AGE(pers_j) =EVAL_YEAR_AGE(pers_j)*12+(12-EVAL_MONTH_BIRTH)
    END DO

END SUBROUTINE EVAL_AGE

!====================================!
!Subroutine to create age class
!====================================!

SUBROUTINE EVAL_AGECLASS_CREATION(n_size)

    INTEGER                 :: n_size   !number of individuals over the households that are considered for analysis

    DO pers_j=1,n_size
        IF (EVAL_YEAR_AGE(pers_j).LE.4) THEN
            EVAL_AGECLASS(pers_j)=1
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.4.AND.EVAL_YEAR_AGE(pers_j).LE.9) THEN
            EVAL_AGECLASS(pers_j)=2
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.9.AND.EVAL_YEAR_AGE(pers_j).LE.14) THEN
            EVAL_AGECLASS(pers_j)=3
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.14.AND.EVAL_YEAR_AGE(pers_j).LE.19) THEN
            EVAL_AGECLASS(pers_j)=4
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.19.AND.EVAL_YEAR_AGE(pers_j).LE.24) THEN
            EVAL_AGECLASS(pers_j)=5
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.24.AND.EVAL_YEAR_AGE(pers_j).LE.29) THEN    
            EVAL_AGECLASS(pers_j)=6
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.29.AND.EVAL_YEAR_AGE(pers_j).LE.34) THEN
            EVAL_AGECLASS(pers_j)=7
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.34.AND.EVAL_YEAR_AGE(pers_j).LE.39) THEN
            EVAL_AGECLASS(pers_j)=8
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.39.AND.EVAL_YEAR_AGE(pers_j).LE.44) THEN
            EVAL_AGECLASS(pers_j)=9
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.44.AND.EVAL_YEAR_AGE(pers_j).LE.49) THEN
            EVAL_AGECLASS(pers_j)=10
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.49.AND.EVAL_YEAR_AGE(pers_j).LE.55) THEN
            EVAL_AGECLASS(pers_j)=11
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.55.AND.EVAL_YEAR_AGE(pers_j).LE.59) THEN
            EVAL_AGECLASS(pers_j)=12
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.59.AND.EVAL_YEAR_AGE(pers_j).LE.64) THEN
            EVAL_AGECLASS(pers_j)=13
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.64.AND.EVAL_YEAR_AGE(pers_j).LE.69) THEN
            EVAL_AGECLASS(pers_j)=14
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.69.AND.EVAL_YEAR_AGE(pers_j).LE.74) THEN
            EVAL_AGECLASS(pers_j)=15
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.74.AND.EVAL_YEAR_AGE(pers_j).LE.79) THEN
            EVAL_AGECLASS(pers_j)=16
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.79.AND.EVAL_YEAR_AGE(pers_j).LE.84) THEN
            EVAL_AGECLASS(pers_j)=17
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.84.AND.EVAL_YEAR_AGE(pers_j).LE.89) THEN
            EVAL_AGECLASS(pers_j)=18
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.89.AND.EVAL_YEAR_AGE(pers_j).LE.94) THEN
            EVAL_AGECLASS(pers_j)=19
        ELSE IF (EVAL_YEAR_AGE(pers_j).GT.94) THEN
            EVAL_AGECLASS(pers_j)=20
        END IF
    END DO

END SUBROUTINE EVAL_AGECLASS_CREATION

!====================================!
!Subroutine to create region class
!====================================!

SUBROUTINE EVAL_REGION_CREATION(n_size)

    INTEGER                 :: n_size   !number of individuals over the households that are considered for analysis

    DO pers_j=1,n_size
        IF      (NAREGNIS_COMMUNIT(pers_j).GE.11000.AND.NAREGNIS_COMMUNIT(pers_j).LE.13053) THEN 
            EVAL_REGION(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.21001.AND.NAREGNIS_COMMUNIT(pers_j).LE.21019) THEN 
            EVAL_REGION(pers_j) = 1
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.23002.AND.NAREGNIS_COMMUNIT(pers_j).LE.24137) THEN 
            EVAL_REGION(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.25005.AND.NAREGNIS_COMMUNIT(pers_j).LE.25124) THEN 
            EVAL_REGION(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.31003.AND.NAREGNIS_COMMUNIT(pers_j).LE.38025) THEN 
            EVAL_REGION(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.41002.AND.NAREGNIS_COMMUNIT(pers_j).LE.46025) THEN 
            EVAL_REGION(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.51004.AND.NAREGNIS_COMMUNIT(pers_j).LE.57095) THEN 
            EVAL_REGION(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.61003.AND.NAREGNIS_COMMUNIT(pers_j).LE.64076) THEN 
            EVAL_REGION(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.71002.AND.NAREGNIS_COMMUNIT(pers_j).LE.73109) THEN 
            EVAL_REGION(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.81001.AND.NAREGNIS_COMMUNIT(pers_j).LE.85047) THEN 
            EVAL_REGION(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.91005.AND.NAREGNIS_COMMUNIT(pers_j).LE.93090) THEN 
            EVAL_REGION(pers_j) = 3
        END IF
    END DO

END SUBROUTINE EVAL_REGION_CREATION

!======================================!
!Create variables with the number of adults and children in the household
!======================================!
SUBROUTINE EVAL_CREATE_ADULCHIL(hh_i)

    INTEGER                                                 :: hh_i                                 !index variable to loop over households   

    DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)                              
        IF (EVAL_YEAR_AGE(pers_j).GE.0.AND.EVAL_YEAR_AGE(pers_j).LT.14) THEN
            EVAL_NKIDS(hh_i) = EVAL_NKIDS(hh_i) + 1
        ELSE IF (EVAL_YEAR_AGE(pers_j).GE.14.AND.EVAL_YEAR_AGE(pers_j).LT.130) THEN
            EVAL_NADULT(hh_i) = EVAL_NADULT(hh_i) + 1        
        ELSE 
            WRITE(*,*) 'The age of the individual is lower than 0 or 130 or above'
            PAUSE
            STOP
        END IF
    END DO

END SUBROUTINE EVAL_CREATE_ADULCHIL

!===================================================================================!
!With subroutine EVAL_ECONINDI_POTENT we construct the variables                    !
!that link the potential members of independent economic units                      !
!Constructed variables                                                              !
!   EVAL_ECIRANK_ANN                                                                !
!   EVAL_ECINUMB_ANN                                                                !
!   EVAL_ECICOUP_ANN                                                                !
!===================================================================================!

SUBROUTINE EVAL_ECONINDI_POTENT

    INTEGER :: pers_j,pers_k
    INTEGER :: COUNT_HEAD,COUNT_SPOUSEA,COUNT_SPOUSEB,COUNT_DEPCHILD,COUNT_DEPOTH,COUNT_SUM

    !EVAL_ECIRANK_ANN: Rank of the individual within the economic independent unit the individual belongs to
    !1 Head of the unit
    !2 Spouse of the head of the unit
    !3 Dependent child
    !4 Dependent other
    !Remark: The FAMREL module produces married couples as output where the man is head of the
    !family and the wife is the married spouse. In case of equal sex the oldest one of the two is the head
    !EVAL_ECINUMB_ANN: Unique identification number of the economic independent unit
    !a unique number identifying the members of the same unit
    !EVAL_ECICOUP_ANN: Individual has a partner that is comprised in the economic independent unit as well
    !1 both partners are legally married
    !2 both partners form a de facto couple

    !Initialise count variables
    COUNT_HEAD = 0
    COUNT_SPOUSEA = 0
    COUNT_SPOUSEB = 0
    COUNT_DEPCHILD = 0
    COUNT_DEPOTH = 0
    !loop over all households
    DO hh_i=1,n_hh_eval
        !loop over all members of the sociological household
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i) 
            !Check whether the individual is the head of a family
            IF (FAMREL_RELATION_PRE(pers_j).EQ.1) THEN
                EVAL_ECIRANK_ANN(pers_j) = 1
                EVAL_ECINUMB_ANN(pers_j) = pers_j
                COUNT_HEAD = COUNT_HEAD + 1
                DO pers_k=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
                    !look for partner of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY_PRE(pers_j).EQ.FAMREL_FAMILY_PRE(pers_k).AND.&
                    &EVAL_ECIRANK_ANN(pers_k).EQ.0.AND.FAMREL_RELATION_PRE(pers_k).EQ.2) THEN
                        !if married
                        IF (FAMREL_COUPLE_PRE(pers_j).EQ.1.AND.FAMREL_COUPLE_PRE(pers_k).EQ.1) THEN
                            EVAL_ECIRANK_ANN(pers_k) = 2
                            EVAL_ECINUMB_ANN(pers_k) = pers_j
                            EVAL_ECICOUP_ANN(pers_j) = 1
                            EVAL_ECICOUP_ANN(pers_k) = 1
                            COUNT_SPOUSEA = COUNT_SPOUSEA + 1
                        !if de facto couple
                        ELSE IF (FAMREL_COUPLE_PRE(pers_j).EQ.2.AND.FAMREL_COUPLE_PRE(pers_k).EQ.2) THEN
                            EVAL_ECIRANK_ANN(pers_k) = 2
                            EVAL_ECINUMB_ANN(pers_k) = pers_j
                            EVAL_ECICOUP_ANN(pers_j) = 2
                            EVAL_ECICOUP_ANN(pers_k) = 2
                            COUNT_SPOUSEB = COUNT_SPOUSEB + 1
                        END IF
                    END IF
                    !look for children of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY_PRE(pers_j).EQ.FAMREL_FAMILY_PRE(pers_k).AND.&
                    &EVAL_ECIRANK_ANN(pers_k).EQ.0.AND.FAMREL_RELATION_PRE(pers_k).EQ.3) THEN
                        EVAL_ECIRANK_ANN(pers_k) = 3
                        EVAL_ECINUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPCHILD = COUNT_DEPCHILD + 1
                    END IF
                    !look for grandchildren of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY_PRE(pers_j).EQ.FAMREL_FAMILY_PRE(pers_k).AND.&
                    &EVAL_ECIRANK_ANN(pers_k).EQ.0.AND.FAMREL_RELATION_PRE(pers_k).EQ.6.AND.NAREGNIS_RELATION(pers_k).EQ.5) THEN
                        EVAL_ECIRANK_ANN(pers_k) = 3
                        EVAL_ECINUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPCHILD = COUNT_DEPCHILD + 1
                    END IF
                    !look for other dependents
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY_PRE(pers_j).EQ.FAMREL_FAMILY_PRE(pers_k).AND.&
                    &EVAL_ECIRANK_ANN(pers_k).EQ.0) THEN
                        EVAL_ECIRANK_ANN(pers_k) = 4
                        EVAL_ECINUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPOTH = COUNT_DEPOTH + 1
                    END IF
                END DO
            END IF    
        END DO
        !loop again over all members of the sociological household
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i) 
            IF (EVAL_ECIRANK_ANN(pers_j).EQ.0) THEN
                WRITE(15,'(''Individu '', i10,'' is nog niet gealloceerd. Relatie-type is '', i10)')&
                &pers_j,FAMREL_RELATION_PRE(pers_j)
            END IF
        END DO
    END DO
    
    COUNT_SUM = COUNT_HEAD + COUNT_SPOUSEA + COUNT_SPOUSEB + COUNT_DEPCHILD + COUNT_DEPOTH
    !Write results of construction of potential tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals after creation of potential independent economic units'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of heads                    '', i10)') COUNT_HEAD
    Write(15,'(''Number of spouses (married)        '', i10)') COUNT_SPOUSEA
    Write(15,'(''Number of spouses (unmarried)      '', i10)') COUNT_SPOUSEB
    Write(15,'(''Number of dependent children       '', i10)') COUNT_DEPCHILD
    Write(15,'(''Number of dependent others         '', i10)') COUNT_DEPOTH
    Write(15,'(''Sum of all allocated individuals   '', i10)') COUNT_SUM
    
END SUBROUTINE EVAL_ECONINDI_POTENT

!=====================================================!
!Compute disposable income from labour market activity
!=====================================================!

SUBROUTINE EVAL_DISPLAB_CREATE

    INTEGER                 :: pers_j,pers_k

    DO hh_i=1,n_hh_eval
        !Create disposable income from labour market activity
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !Pre reform quarterly
            EVAL_DISPLAB_PRE(pers_j)  =                 &
                &CONTRIB_GTIPRIV_QT_PRE(pers_j) +&
                &CONTRIB_GTIPUB_QT_PRE(pers_j)  +&
                &CONTRIB_GTISELF_QT_PRE(pers_j) 
            !annualise quarterly variable
            EVAL_DISPLAB_PRE(pers_j) = EVAL_DISPLAB_PRE(pers_j)*4.D0
            !Make equal to zero if negative
            !remark: rule is appropriate for 2001 but what if a reimbursable tax credit is introduced
            IF (EVAL_DISPLAB_PRE(pers_j).LT.0) THEN
                EVAL_DISPLAB_PRE(pers_j) = 0
            END IF
            !subtract annual amount of taxes that is registered for the head of the tax unit
            IF (PIT_TURANK_ANN_PRE(pers_j).EQ.1) THEN
                IF (PIT_FINTAX_ANN_PRE(pers_j).LT.EVAL_DISPLAB_PRE(pers_j)) THEN
                    EVAL_DISPLAB_PRE(pers_j) = EVAL_DISPLAB_PRE(pers_j) - PIT_FINTAX_ANN_PRE(pers_j)
                    EVAL_RESTLAB_PRE(pers_j) = 0
                ELSE
                    EVAL_DISPLAB_PRE(pers_j) = 0
                    EVAL_RESTLAB_PRE(pers_j) = PIT_FINTAX_ANN_PRE(pers_j) - EVAL_RESTLAB_PRE(pers_j) 
                END IF
            END IF
        END DO
    END DO

    DO hh_i=1,n_hh_eval
        !adapt disposable income of the TAX spouse with remainder of personal income taxes
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            DO pers_k=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
                !identify married spouse for pre reform adaptation
                IF (pers_j.NE.pers_k.AND.PIT_TUNUMB_ANN_PRE(pers_j).EQ.PIT_TUNUMB_ANN_PRE(pers_k).AND.&
                &PIT_TURANK_ANN_PRE(pers_j).EQ.2.AND.PIT_TURANK_ANN_PRE(pers_k).EQ.1) THEN
                    !subtract the remaining personal income taxes, stored for the head of the tax unit
                    IF (EVAL_RESTLAB_PRE(pers_k).GT.0) THEN
                        IF (EVAL_RESTLAB_PRE(pers_k).LT.EVAL_DISPLAB_PRE(pers_j)) THEN
                            EVAL_DISPLAB_PRE(pers_j) = EVAL_DISPLAB_PRE(pers_j) - EVAL_RESTLAB_PRE(pers_k)  
                        ELSE
                            EVAL_DISPLAB_PRE(pers_j) = 0
                        END IF
                    END IF
                END IF
            END DO
        END DO
    END DO
    
END SUBROUTINE EVAL_DISPLAB_CREATE

!===================================================================================!
!With subroutine EVAL_ECONINDI_FINAL we check the net own means conditions of       !
!potential members of economic independent units                                    !
!Variables adapted                                                                  !
!   EVAL_ECIRANK_ANN                                                                !
!   EVAL_ECINUMB_ANN                                                                !
!===================================================================================!

SUBROUTINE EVAL_ECONINDI_FINAL

    INTEGER                 :: pers_j
    INTEGER                 :: COUNT_SPLITINC,COUNT_SPLITUN,COUNT_NOTSPLIT,COUNT_SUM
    INTEGER                 :: COUNT_HEAD,COUNT_SPOUSEA,COUNT_SPOUSEB,COUNT_DEPCHILD,COUNT_DEPOTH
    REAL(KIND=KIND_REAL)    :: SumUNEMB                                    !compute the sum of all unemployment benefits

    !initialise EVAL_CHECK_ANN vector
    EVAL_CHECK_ANN(1:n_rec) = 0
    
    !Carry out check for dependents other than parents or grandparents that form a couple already
    !loop over all households
    COUNT_SPLITINC = 0
    COUNT_SPLITUN = 0
    COUNT_NOTSPLIT = 0
    DO hh_i=1,n_hh_eval
        !loop over all members of the sociological household
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i) 
            !Check only applied on dependent children or dependent others
            IF (ANY(EVAL_ECIRANK_ANN(pers_j).EQ.(/3,4/))) THEN
                !if individual has more net labour income than minimex and is above 18, make it an economic independent
                IF (EVAL_DISPLAB_PRE(pers_j).GT.EVAL_MINNINC.AND.EVAL_YEAR_AGE(pers_j).GE.18) THEN
                    EVAL_ECIRANK_ANN(pers_j) = 1
                    EVAL_ECINUMB_ANN(pers_j) = pers_j 
                    COUNT_SPLITINC = COUNT_SPLITINC + 1
                END IF
                !if individual is unemployed (gross taxable benefit is positive) then also split it of
                SumUNEMB = & 
                &CONTRIB_GTIEARNE_QT_PRE(pers_j)+&
                &CONTRIB_GTIEAROL_QT_PRE(pers_j)+&
                &CONTRIB_GTIOLDUN_QT_PRE(pers_j)+&
                &CONTRIB_GTIOTHUN_QT_PRE(pers_j)
                IF (SumUNEMB.GT.0) THEN
                    EVAL_ECIRANK_ANN(pers_j) = 1
                    EVAL_ECINUMB_ANN(pers_j) = pers_j 
                    COUNT_SPLITUN = COUNT_SPLITUN + 1
                END IF
                !count individuals that are not split off as indipendent but satisfy income condition
                IF (EVAL_DISPLAB_PRE(pers_j).GT.EVAL_MINNINC.AND.EVAL_YEAR_AGE(pers_j).LT.18) THEN
                    COUNT_NOTSPLIT = COUNT_NOTSPLIT + 1
                END IF
            END IF
        END DO            
    END DO

    !TEST
    !Some checks on the split procedure
    COUNT_SUM = COUNT_SPLITINC + COUNT_SPLITUN
    !Write results of construction independent units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals that are split off on the basis of their own means'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of dependents split off because of labour income condition                 '', i10)') COUNT_SPLITINC 
    Write(15,'(''Number of dependents split off because of unemployment condition                  '', i10)') COUNT_SPLITUN
    Write(15,'(''Sum of all individuals split off or transfered                                    '', i10)') COUNT_SUM
    Write(15,'(''Number of dependents not split off but who satisfy income condition               '', i10)') COUNT_NOTSPLIT

    !Initialise count variables for final check
    COUNT_HEAD = 0
    COUNT_SPOUSEA = 0
    COUNT_SPOUSEB = 0
    COUNT_DEPCHILD = 0
    COUNT_DEPOTH = 0

    DO pers_j=1,n_rec
        IF (EVAL_ECIRANK_ANN(pers_j).EQ.1) THEN
            COUNT_HEAD = COUNT_HEAD + 1
        ELSE IF (EVAL_ECIRANK_ANN(pers_j).EQ.2) THEN
            IF (EVAL_ECICOUP_ANN(pers_j).EQ.1) THEN
                COUNT_SPOUSEA = COUNT_SPOUSEA + 1 
            ELSE IF (EVAL_ECICOUP_ANN(pers_j).EQ.2) THEN
                COUNT_SPOUSEB = COUNT_SPOUSEB + 1 
            END IF
        ELSE IF (EVAL_ECIRANK_ANN(pers_j).EQ.3) THEN
            COUNT_DEPCHILD = COUNT_DEPCHILD + 1
        ELSE IF (EVAL_ECIRANK_ANN(pers_j).EQ.4) THEN
            COUNT_DEPOTH = COUNT_DEPOTH + 1
        END IF
    END DO

    !TEST
    COUNT_SUM = COUNT_HEAD + COUNT_SPOUSEA + COUNT_SPOUSEB + COUNT_DEPCHILD + COUNT_DEPOTH
    !Write results of construction of final tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals after split off on the basis of net own means'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of heads                    '', i10)') COUNT_HEAD
    Write(15,'(''Number of spouses (married)        '', i10)') COUNT_SPOUSEA
    Write(15,'(''Number of spouses (unmarried)      '', i10)') COUNT_SPOUSEB
    Write(15,'(''Number of dependent children       '', i10)') COUNT_DEPCHILD
    Write(15,'(''Number of dependent others         '', i10)') COUNT_DEPOTH
    Write(15,'(''Sum of all allocated individuals   '', i10)') COUNT_SUM

END SUBROUTINE EVAL_ECONINDI_FINAL

!======================================!
!Compute the poverty line that is to be used within this run
!======================================!
SUBROUTINE EVAL_POVLINE_COMPUTE(X,W,VecSize,PovLine,Percent_Pov)
! In this subroutine we calculate a poverty line 
! The vector X must NOT be ordered on input and remains UNCHANGED on output

    INTEGER                                                 :: VecSize                              !VecSize: size of the vectors x and w                                   - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the quantilenumber is calculated     - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x                           - input
    REAL(KIND=KIND_REAL)                                    :: Percent_Pov                          !Percentage to be used for the computation of the poverty line          - input
    REAL(KIND=DPRECI)                                       :: PovLine                              !PovLine:poverty line                                                   - output

    POVLINE = 0.D0
    IF (EVAL_TYPEPOV.EQ.1) THEN
        POVLINE = PERCENT_POV*MEANINC(X,W,VecSize)
    ELSE IF (EVAL_TYPEPOV.EQ.2) THEN
        POVLINE = PERCENT_POV*MEDINC(X,W,VecSize)    
    ELSE 
        WRITE(*,*) 'The requested type of poverty line is unknown'
        PAUSE
        STOP
    END IF

END SUBROUTINE EVAL_POVLINE_COMPUTE

!====================================!
!Create pre and post reform variables that are going to be analysed
!====================================!

SUBROUTINE EVAL_PREPOST_CREATE(n_size)

    INTEGER                                                 :: n_size,j                               !number of individuals over the households that are considered for analysis
    INTEGER                                                 :: pers_k                               !extra individual loop counter
    INTEGER                                                 :: FRAC                                 !Fraction used to devide population and compute fictitious incomes
    REAL(KIND=DPRECI)                                       :: SUMPRE                               !Sum of pre reform income
    REAL(KIND=DPRECI)                                       :: SUMPOST                              !Sum of post reform income
    REAL(KIND=DPRECI)                                       :: SUMDIF                               !Sum of post reform income
    REAL(KIND=DPRECI)                                       :: SUMPRE_BTRAN                         !Sum of pre reform income before transfers
    REAL(KIND=DPRECI)                                       :: SUMPOST_BTRAN                        !Sum of post reform income before transfers
    REAL(KIND=DPRECI)                                       :: SUMDIF_BTRAN                         !Sum of post reform income
    REAL(KIND=DPRECI)                                       :: SUMPRE_NPENS                         !Sum of pre reform income after pension transfers
    REAL(KIND=DPRECI)                                       :: SUMPOST_NPENS                        !Sum of post reform income after  pension transfers
    REAL(KIND=DPRECI)                                       :: SUMDIF_NPENS                         !Difference in pre and post reform income after pension transfers
    REAL(KIND=DPRECI)                                       :: ESHH_WELF                            !Household equivalence scale used to compute welfare concept
    REAL(KIND=DPRECI)                                       :: ESHH_ANAL                            !Household equivalence scale used to compute concept of analysis
    REAL(KIND=KIND_REAL),DIMENSION(17)                      :: GROSSREV                             !Vector to store intermediate results for output   
!commented part just used to develop eval-module before real data were read in from different modules
!    !Create pre reform disposable income    
!    FRAC = FLOOR(n_rec/10.0)
!    DO i=1,10
!        IF (i.LT.10) THEN
!            DO pers_j=(i-1)*frac+1,i*frac
!                EVAL_DISPPRE(pers_j) = i*200.D0 
!            END DO
!        ELSE
!            DO pers_j=(i-1)*frac+1,n_rec
!                EVAL_DISPPRE(pers_j) = i*200.D0 
!            END DO
!        END IF
!    END DO

!    !Create post reform disposable income and difference in disposable income
!    DO pers_j=1,n_rec
!        EVAL_DISPPOST(pers_j) = EVAL_DISPPRE(pers_j)*(1.05)
!        EVAL_D1DISP(pers_j)   = EVAL_DISPPOST(pers_j) - EVAL_DISPPRE(pers_j)
!    END DO

    DO hh_i=1,n_hh_eval
        !Create disposable income and difference in disposable income after all transfers
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !Concepts with social transfers
            !Pre reform quarterly
            EVAL_DISPPRE(pers_j)  =                 &
                &CONTRIB_GTIPRIV_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_GTIPUB_QT_PRE(pers_j)*4.00  +&
                &CONTRIB_GTISELF_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_GTIPENS_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_GTIEARNE_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTIEAROL_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTIOLDUN_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTIOTHUN_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTIDISAB_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTINDOCC_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTISICK_QT_PRE(pers_j)*4.00+&
                &CONTRIB_GTIFAMAL_QT_PRE(pers_j)*4.00                
            !Make equal to zero if negative
            !remark: rule is appropriate for 2001 but what if a reimbursable tax credit is introduced
            IF (EVAL_DISPPRE(pers_j).LT.0) THEN
                EVAL_DISPPRE(pers_j) = 0
            END IF
            !subtract annual amount of taxes that is registered for the head of the tax unit
            IF (PIT_TURANK_ANN_PRE(pers_j).EQ.1) THEN
                IF (PIT_FINTAX_ANN_PRE(pers_j).LT.EVAL_DISPPRE(pers_j)) THEN
                    EVAL_DISPPRE(pers_j) = EVAL_DISPPRE(pers_j) - PIT_FINTAX_ANN_PRE(pers_j)
                    EVAL_RESTPRE(pers_j) = 0
                ELSE
                    EVAL_RESTPRE(pers_j) = PIT_FINTAX_ANN_PRE(pers_j) - EVAL_DISPPRE(pers_j) 
                    EVAL_DISPPRE(pers_j) = 0
                END IF
            END IF
            !Add Minimex
            IF (EVAL_INCLMIN.EQ.1) THEN
                EVAL_DISPPRE(pers_j) = EVAL_DISPPRE(pers_j) + 4.D0*SOCBEN_MINIMEX_PRE(pers_j)
            END IF    
            EVAL_DISPPRE(pers_j) = EVAL_DISPPRE(pers_j) + 4.D0*CONTRIB_SELFEXTR_QT_PRE(pers_j)
            !Post reform quarterly
            EVAL_DISPPOST(pers_j)  =                 &
                &CONTRIB_GTIPRIV_QT_POST(pers_j)*4.00 +&
                &CONTRIB_GTIPUB_QT_POST(pers_j)*4.00  +&
                &CONTRIB_GTISELF_QT_POST(pers_j)*4.00 +&
                &CONTRIB_GTIPENS_QT_POST(pers_j)*4.00 +&
                &CONTRIB_GTIEARNE_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTIEAROL_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTIOLDUN_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTIOTHUN_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTIDISAB_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTINDOCC_QT_POST(pers_j)*4.00+&
                &CONTRIB_GTISICK_QT_POST(pers_j)*4.00 +& 
                &CONTRIB_GTIFAMAL_QT_POST(pers_j)*4.00                
            !Make equal to zero if negative
            !remark: rule is appropriate for 2001 but what if a reimbursable tax credit is introduced
            IF (EVAL_DISPPOST(pers_j).LT.0) THEN
                EVAL_DISPPOST(pers_j) = 0
            END IF    
            !subtract annual amount of taxes that is registered for the head of the tax unit
            IF (PIT_TURANK_ANN_POST(pers_j).EQ.1) THEN
                IF (PIT_FINTAX_ANN_POST(pers_j).LT.EVAL_DISPPOST(pers_j)) THEN
                    EVAL_DISPPOST(pers_j) = EVAL_DISPPOST(pers_j) - PIT_FINTAX_ANN_POST(pers_j)
                    EVAL_RESTPOST(pers_j) = 0
                ELSE
                    EVAL_RESTPOST(pers_j) = PIT_FINTAX_ANN_POST(pers_j) - EVAL_DISPPOST(pers_j) 
                    EVAL_DISPPOST(pers_j) = 0
                END IF
            END IF
            !Add Minimex
            IF (EVAL_INCLMIN.EQ.1) THEN
                EVAL_DISPPOST(pers_j) = EVAL_DISPPOST(pers_j) + 4.D0*SOCBEN_MINIMEX_POST(pers_j)
            END IF
            EVAL_DISPPOST(pers_j) = EVAL_DISPPOST(pers_j) + 4.D0*CONTRIB_SELFEXTR_QT_POST(pers_j)
        END DO
        !Create disposable income and difference in disposable income after pension transfers before all other transfers
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !Concepts with social transfers
            !Pre reform quarterly
            EVAL_DISPPRE_NPENS(pers_j)  =             &
                &CONTRIB_GTIPRIV_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_GTIPUB_QT_PRE(pers_j)*4.00  +&
                &CONTRIB_GTISELF_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_GTIPENS_QT_PRE(pers_j)*4.00 +&
                &CONTRIB_SELFEXTR_QT_PRE(pers_j)*4.00
            !Make equal to zero if negative
            !remark: rule is appropriate for 2001 but what if a reimbursable tax credit is introduced
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.0) THEN
                EVAL_DISPPRE_NPENS(pers_j) = 0
            END IF
            !Post reform quarterly
            EVAL_DISPPOST_NPENS(pers_j)  =                 &
                &CONTRIB_GTIPRIV_QT_POST(pers_j)*4.00 +&
                &CONTRIB_GTIPUB_QT_POST(pers_j)*4.00  +&
                &CONTRIB_GTISELF_QT_POST(pers_j)*4.00 +&
                &CONTRIB_GTIPENS_QT_POST(pers_j)*4.00 +&
                &CONTRIB_SELFEXTR_QT_POST(pers_j)*4.00
            !Make equal to zero if negative
            !remark: rule is appropriate for 2001 but what if a reimbursable tax credit is introduced
            IF (EVAL_DISPPOST_NPENS(pers_j).LT.0) THEN
                EVAL_DISPPOST_NPENS(pers_j) = 0
            END IF    
        END DO
    END DO

    DO hh_i=1,n_hh_eval
        !first adapt disposable income of the spouse with remainder of personal income taxes
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            DO pers_k=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
                !identify married spouse for pre reform adaptation
                IF (pers_j.NE.pers_k.AND.PIT_TUNUMB_ANN_PRE(pers_j).EQ.PIT_TUNUMB_ANN_PRE(pers_k).AND.&
                &PIT_TURANK_ANN_PRE(pers_j).EQ.2.AND.PIT_TURANK_ANN_PRE(pers_k).EQ.1) THEN
                    !subtract the remaining personal income taxes, stored for the head of the tax unit
                    IF (EVAL_RESTPRE(pers_k).GT.0) THEN
                        IF (EVAL_RESTPRE(pers_k).LT.EVAL_DISPPRE(pers_j)) THEN
                            EVAL_DISPPRE(pers_j) = EVAL_DISPPRE(pers_j) - EVAL_RESTPRE(pers_k)  
                        ELSE
                            EVAL_DISPPRE(pers_j) = 0
                        END IF
                    END IF
                END IF
                !identify married spouse for post reform adaptation
                IF (pers_j.NE.pers_k.AND.PIT_TUNUMB_ANN_POST(pers_j).EQ.PIT_TUNUMB_ANN_POST(pers_k).AND.&
                &PIT_TURANK_ANN_POST(pers_j).EQ.2.AND.PIT_TURANK_ANN_POST(pers_k).EQ.1) THEN
                    !subtract the remaining personal income taxes, stored for the head of the tax unit
                    IF (EVAL_RESTPOST(pers_k).GT.0) THEN
                        IF (EVAL_RESTPOST(pers_k).LT.EVAL_DISPPOST(pers_j)) THEN
                            EVAL_DISPPOST(pers_j) = EVAL_DISPPOST(pers_j) - EVAL_RESTPOST(pers_k)  
                        ELSE
                            EVAL_DISPPOST(pers_j) = 0
                        END IF
                    END IF
                END IF
            END DO
        END DO
        !create difference in disposable income and income concepts before transfers
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !compute the difference between pre and post reform variable
            EVAL_D1DISP(pers_j)   = EVAL_DISPPOST(pers_j) - EVAL_DISPPRE(pers_j)
            !Concepts before social transfers
            !REMARK: we construct this concept as gross income from labour activity
            !Pre reform quarterly
            EVAL_DISPPRE_BTRAN(pers_j)  =               &
                &(CONTRIB_GTIPRIV_QT_PRE(pers_j)        +&
                & CONTRIB_GTIPUB_QT_PRE(pers_j)         +&
                & CONTRIB_GTISELF_QT_PRE(pers_j))       +&
                &(CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j)    +&
                & CONTRIB_SSPUB_EMEE_QT_PRE(pers_j)     +&
                & CONTRIB_SSSELF_EMER_QT_PRE(pers_j)    +&
                & CONTRIB_SELFEXTR_QT_PRE(pers_j))
            !annualise quarterly variable        
            EVAL_DISPPRE_BTRAN(pers_j) = EVAL_DISPPRE_BTRAN(pers_j)*4.D0
            !Make equal to zero if negative
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.0) THEN
                EVAL_DISPPRE_BTRAN(pers_j) = 0
            END IF    
            !Post reform quarterly
            EVAL_DISPPOST_BTRAN(pers_j)  =              &
                &(CONTRIB_GTIPRIV_QT_POST(pers_j)       +&
                & CONTRIB_GTIPUB_QT_POST(pers_j)        +&
                & CONTRIB_GTISELF_QT_POST(pers_j))      +&
                &(CONTRIB_SSPRIV_EMEE_QT_POST(pers_j)   +&
                & CONTRIB_SSPUB_EMEE_QT_POST(pers_j)    +&
                & CONTRIB_SSSELF_EMER_QT_POST(pers_j)   +&
                & CONTRIB_SELFEXTR_QT_POST(pers_j))
            !annualise quarterly variable        
            EVAL_DISPPOST_BTRAN(pers_j) = EVAL_DISPPOST_BTRAN(pers_j)*4.D0
            !Make equal to zero if negative
            IF (EVAL_DISPPOST_BTRAN(pers_j).LT.0) THEN
                EVAL_DISPPOST_BTRAN(pers_j) = 0
            END IF    
            !compute the difference between pre and post reform variable
            EVAL_D1DISP_BTRAN(pers_j)   = EVAL_DISPPOST_BTRAN(pers_j) - EVAL_DISPPRE_BTRAN(pers_j)
        END DO
    END DO
    
    !Open pre and post reform files for writing net income per individual
    OPEN(UNIT=12, FILE=NDISPINC_PARLOC_PRE,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') NDISPINC_PARLOC_PRE
        WRITE(15,'("Could not find ", a100)') NDISPINC_PARLOC_PRE
        PAUSE
        STOP
    END IF   
    OPEN(UNIT=13, FILE=NDISPINC_PARLOC_POST,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') NDISPINC_PARLOC_POST
        WRITE(15,'("Could not find ", a100)') NDISPINC_PARLOC_POST
        PAUSE
        STOP
    END IF   
    DO hh_i=1,n_hh_eval
        !loop over all household members
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !first create gross revenues and social security contributions to write to pre reform file
            GROSSREV(1:17)=0.D0
            GROSSREV(1) =   &
                            & CONTRIB_SSPRIV_EMER_QT_PRE(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMER_QT_PRE(pers_j)*4.00
            GROSSREV(2) =   & 
                            & CONTRIB_GTIPRIV_QT_PRE(pers_j)*4.00        +&
                            & CONTRIB_GTIPUB_QT_PRE(pers_j)*4.00         +&
                            & CONTRIB_GTISELF_QT_PRE(pers_j)*4.00        +&
                            & CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMEE_QT_PRE(pers_j)*4.00     +&
                            & CONTRIB_SSSELF_EMER_QT_PRE(pers_j)*4.00 
            GROSSREV(3) =   CONTRIB_GTIPENS_QT_PRE(pers_j)*4.00        
            GROSSREV(4) =   CONTRIB_GTIEARNE_QT_PRE(pers_j)*4.00       
            GROSSREV(5) =   CONTRIB_GTIEAROL_QT_PRE(pers_j)*4.00       
            GROSSREV(6) =   CONTRIB_GTIOLDUN_QT_PRE(pers_j)*4.00       
            GROSSREV(7) =   CONTRIB_GTIOTHUN_QT_PRE(pers_j)*4.00       
            GROSSREV(8) =   CONTRIB_GTIDISAB_QT_PRE(pers_j)*4.00       
            GROSSREV(9) =   CONTRIB_GTINDOCC_QT_PRE(pers_j)*4.00       
            GROSSREV(10) =  CONTRIB_GTISICK_QT_PRE(pers_j)*4.00        
            GROSSREV(11) =  CONTRIB_GTIFAMAL_QT_PRE(pers_j)*4.00                       
            GROSSREV(12) =  SOCBEN_MINIMEX_PRE(pers_j)*4.00           
            GROSSREV(13) =  &
                            & CONTRIB_SSPRIV_EMEE_QT_PRE(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMEE_QT_PRE(pers_j)*4.00     +&
                            & CONTRIB_SSSELF_EMER_QT_PRE(pers_j)*4.00   
            GROSSREV(14) =  CONTRIB_SSREPLAC_QT_PRE(pers_j)*4.00
            GROSSREV(15) =  EVAL_DISPPRE(pers_j)
            GROSSREV(16) =  PIT_FINTAX_ANN_PRE(pers_j)
            GROSSREV(17) =  CONTRIB_SELFEXTR_QT_PRE(pers_j)*4.00
            !Write pre reform info
            IF (CONTROL_TYPESIM(1).EQ.'yes') THEN
                WRITE(12,'(A14,3i10,17f15.2)') GECODEERD_INSZ_CHAR(pers_j),EVAL_ECIRANK_ANN(pers_j),EVAL_ECINUMB_ANN(pers_j),&
                &EVAL_ECICOUP_ANN(pers_j),(GROSSREV(j),j=1,17)
            END IF
            !first create gross revenues and social security contributions to write to post reform file
            GROSSREV(1:17)=0.D0
            GROSSREV(1) =   &
                            & CONTRIB_SSPRIV_EMER_QT_POST(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMER_QT_POST(pers_j)*4.00
            GROSSREV(2) =   & 
                            & CONTRIB_GTIPRIV_QT_POST(pers_j)*4.00        +&
                            & CONTRIB_GTIPUB_QT_POST(pers_j)*4.00         +&
                            & CONTRIB_GTISELF_QT_POST(pers_j)*4.00        +&
                            & CONTRIB_SSPRIV_EMEE_QT_POST(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMEE_QT_POST(pers_j)*4.00     +&
                            & CONTRIB_SSSELF_EMER_QT_POST(pers_j)*4.00 
            GROSSREV(3) =   CONTRIB_GTIPENS_QT_POST(pers_j)*4.00        
            GROSSREV(4) =   CONTRIB_GTIEARNE_QT_POST(pers_j)*4.00       
            GROSSREV(5) =   CONTRIB_GTIEAROL_QT_POST(pers_j)*4.00       
            GROSSREV(6) =   CONTRIB_GTIOLDUN_QT_POST(pers_j)*4.00       
            GROSSREV(7) =   CONTRIB_GTIOTHUN_QT_POST(pers_j)*4.00       
            GROSSREV(8) =   CONTRIB_GTIDISAB_QT_POST(pers_j)*4.00       
            GROSSREV(9) =   CONTRIB_GTINDOCC_QT_POST(pers_j)*4.00       
            GROSSREV(10) =  CONTRIB_GTISICK_QT_POST(pers_j)*4.00        
            GROSSREV(11) =  CONTRIB_GTIFAMAL_QT_POST(pers_j)*4.00                       
            GROSSREV(12) =  SOCBEN_MINIMEX_POST(pers_j)*4.00           
            GROSSREV(13) =  &
                            & CONTRIB_SSPRIV_EMEE_QT_POST(pers_j)*4.00    +&
                            & CONTRIB_SSPUB_EMEE_QT_POST(pers_j)*4.00     +&
                            & CONTRIB_SSSELF_EMER_QT_POST(pers_j)*4.00    
            GROSSREV(14) =  CONTRIB_SSREPLAC_QT_POST(pers_j)*4.00
            GROSSREV(15) =  EVAL_DISPPOST(pers_j)
            GROSSREV(16) =  PIT_FINTAX_ANN_POST(pers_j)
            GROSSREV(17) =  CONTRIB_SELFEXTR_QT_POST(pers_j)*4.00
            !Write post reform info
            IF (CONTROL_TYPESIM(2).EQ.'yes') THEN
                WRITE(13,'(A14,3i10,17f15.2)') GECODEERD_INSZ_CHAR(pers_j),EVAL_ECIRANK_ANN(pers_j),EVAL_ECINUMB_ANN(pers_j),&
                &EVAL_ECICOUP_ANN(pers_j),(GROSSREV(j),j=1,17)
            END IF
        END DO
    END DO        
    CLOSE(12)
    CLOSE(13)

    !Convert all annual concepts to monthly amounts
    DO pers_j=1,n_size
        EVAL_DISPPRE(pers_j)        = EVAL_DISPPRE(pers_j)/12.D0  
        EVAL_DISPPOST(pers_j)       = EVAL_DISPPOST(pers_j)/12.D0  
        EVAL_D1DISP(pers_j)         = EVAL_D1DISP(pers_j)/12.D0  
        EVAL_D1DISP_NOES(pers_j)    = EVAL_D1DISP(pers_j)/12.D0  
        EVAL_DISPPRE_BTRAN(pers_j)  = EVAL_DISPPRE_BTRAN(pers_j)/12.D0  
        EVAL_DISPPOST_BTRAN(pers_j) = EVAL_DISPPOST_BTRAN(pers_j)/12.D0  
        EVAL_D1DISP_BTRAN(pers_j)   = EVAL_D1DISP_BTRAN(pers_j)/12.D0  
        EVAL_DISPPRE_NPENS(pers_j)  = EVAL_DISPPRE_NPENS(pers_j)/12.D0  
        EVAL_DISPPOST_NPENS(pers_j) = EVAL_DISPPOST_NPENS(pers_j)/12.D0  
        EVAL_D1DISP_NPENS(pers_j)   = EVAL_D1DISP_NPENS(pers_j)/12.D0  
    END DO    

    !Start creation of welfare concept and disposable income concepts by individual
    DO hh_i=1,n_hh_eval
        !Make sum of disposable incomes over individuals of the same household
        SUMPRE          = 0.D0
        SUMPOST         = 0.D0        
        SUMDIF          = 0.D0
        SUMPRE_BTRAN    = 0.D0
        SUMPOST_BTRAN   = 0.D0
        SUMDIF_BTRAN    = 0.D0        
        SUMPRE_NPENS    = 0.D0
        SUMPOST_NPENS   = 0.D0
        SUMDIF_NPENS    = 0.D0        
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)                              
            SUMPRE          = SUMPRE        + EVAL_DISPPRE(pers_j) 
            SUMPOST         = SUMPOST       + EVAL_DISPPOST(pers_j)             
            SUMDIF          = SUMDIF        + EVAL_D1DISP(pers_j)
            SUMPRE_BTRAN    = SUMPRE_BTRAN  + EVAL_DISPPRE_BTRAN(pers_j) 
            SUMPOST_BTRAN   = SUMPOST_BTRAN + EVAL_DISPPOST_BTRAN(pers_j)             
            SUMDIF_BTRAN    = SUMDIF_BTRAN  + EVAL_D1DISP_BTRAN(pers_j)            
            SUMPRE_NPENS    = SUMPRE_NPENS  + EVAL_DISPPRE_NPENS(pers_j) 
            SUMPOST_NPENS   = SUMPOST_NPENS + EVAL_DISPPOST_NPENS(pers_j)             
            SUMDIF_NPENS    = SUMDIF_NPENS  + EVAL_D1DISP_NPENS(pers_j)            
        END DO
        !Compute household equivalence scale for construction of welfare concept
        IF (EVAL_TYPEES_WELF.EQ.1) THEN
            IF ((EVAL_NADULT(hh_i)+EVAL_NKIDS(hh_i)).EQ.1) THEN
                ESHH_WELF = 1
            ELSE
                IF (EVAL_NADULT(hh_i).GE.1) THEN
                    ESHH_WELF = 1 + 0.5*(EVAL_NADULT(hh_i)-1) + 0.3*EVAL_NKIDS(hh_i)
                ELSE
                    ESHH_WELF = 1 + 0.3*(EVAL_NKIDS(hh_i)-1)
                END IF
            END IF    
        ELSE IF (EVAL_TYPEES_WELF.EQ.2) THEN
            IF ((EVAL_NADULT(hh_i)+EVAL_NKIDS(hh_i)).EQ.1) THEN
                ESHH_WELF = 1
            ELSE
                IF (EVAL_NADULT(hh_i).GE.1) THEN
                    ESHH_WELF = 1 + 0.7*(EVAL_NADULT(hh_i)-1) + 0.5*EVAL_NKIDS(hh_i)
                ELSE
                    ESHH_WELF = 1 + 0.5*(EVAL_NKIDS(hh_i)-1)
                END IF
            END IF    
        ELSE IF (EVAL_TYPEES_WELF.EQ.3) THEN
            ESHH_WELF = SQRT(DBLE(EVAL_NADULT(hh_i)) + DBLE(EVAL_NKIDS(hh_i)))
        ELSE IF (EVAL_TYPEES_WELF.EQ.4) THEN
            ESHH_WELF = DBLE(EVAL_NADULT(hh_i)) + DBLE(EVAL_NKIDS(hh_i))
        ELSE IF (EVAL_TYPEES_WELF.EQ.5) THEN
            ESHH_WELF = 1.D0
        ELSE 
            WRITE(*,*) 'Unknown type of equivalence scale'
            PAUSE
            STOP
        END IF
        !Compute household equivalence scale for construction of concept of analysis
        IF (EVAL_TYPEES_ANAL.EQ.1) THEN
            IF ((EVAL_NADULT(hh_i)+EVAL_NKIDS(hh_i)).EQ.1) THEN
                ESHH_ANAL = 1
            ELSE
                IF (EVAL_NADULT(hh_i).GE.1) THEN
                    ESHH_ANAL = 1 + 0.5*(EVAL_NADULT(hh_i)-1) + 0.3*EVAL_NKIDS(hh_i)
                ELSE
                    ESHH_ANAL = 1 + 0.3*(EVAL_NKIDS(hh_i)-1)
                END IF
            END IF    
        ELSE IF (EVAL_TYPEES_ANAL.EQ.2) THEN
            IF ((EVAL_NADULT(hh_i)+EVAL_NKIDS(hh_i)).EQ.1) THEN
                ESHH_ANAL = 1
            ELSE
                IF (EVAL_NADULT(hh_i).GE.1) THEN
                    ESHH_ANAL = 1 + 0.7*(EVAL_NADULT(hh_i)-1) + 0.5*EVAL_NKIDS(hh_i)
                ELSE
                    ESHH_ANAL = 1 + 0.5*(EVAL_NKIDS(hh_i)-1)
                END IF
            END IF    
        ELSE IF (EVAL_TYPEES_ANAL.EQ.3) THEN
            ESHH_ANAL = SQRT(DBLE(EVAL_NADULT(hh_i)) + DBLE(EVAL_NKIDS(hh_i)))
        ELSE IF (EVAL_TYPEES_ANAL.EQ.4) THEN
            ESHH_ANAL = DBLE(EVAL_NADULT(hh_i)) + DBLE(EVAL_NKIDS(hh_i))
        ELSE IF (EVAL_TYPEES_ANAL.EQ.5) THEN
            ESHH_ANAL = 1.D0
        ELSE 
            WRITE(*,*) 'Unknown type of equivalence scale'
            PAUSE
            STOP
        END IF
        !Compute pre and post reform welfare for each individual(i.e. equivalised household income)
        !and equal share of household disposable income for each individual
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)                              
            IF (INT(ESHH_WELF*1.D06).NE.0) THEN
                EVAL_WELFPRE(pers_j)  = SUMPRE/ESHH_WELF
                EVAL_WELFPOST(pers_j) = SUMPOST/ESHH_WELF            
            END IF
            IF (INT(ESHH_ANAL*1.D06).NE.0) THEN
                EVAL_DISPPRE(pers_j)        = SUMPRE/ESHH_ANAL
                EVAL_DISPPOST(pers_j)       = SUMPOST/ESHH_ANAL
                EVAL_D1DISP(pers_j)         = SUMDIF/ESHH_ANAL
                EVAL_DISPPRE_BTRAN(pers_j)  = SUMPRE_BTRAN/ESHH_ANAL
                EVAL_DISPPOST_BTRAN(pers_j) = SUMPOST_BTRAN/ESHH_ANAL
                EVAL_D1DISP_BTRAN(pers_j)   = SUMDIF_BTRAN/ESHH_ANAL
                EVAL_DISPPRE_NPENS(pers_j)  = SUMPRE_NPENS/ESHH_ANAL
                EVAL_DISPPOST_NPENS(pers_j) = SUMPOST_NPENS/ESHH_ANAL
                EVAL_D1DISP_NPENS(pers_j)   = SUMDIF_NPENS/ESHH_ANAL
            END IF
        END DO
    END DO    

!    !Write statements to check for unexpected values
!    DO pers_j=1,n_size
!        IF (EVAL_DISPPRE(pers_j).LT.0) THEN
!            write(15,*) 'A',gecodeerd_insz_char(pers_j),EVAL_DISPPRE(pers_j),HOEDRSVZ_QT(pers_j)
!        END IF       
!        IF (EVAL_DISPPOST(pers_j).LT.0) THEN
!            write(15,*) 'A',gecodeerd_insz_char(pers_j),EVAL_DISPPOST(pers_j),HOEDRSVZ_QT(pers_j)
!        END IF       
!    END DO    

END SUBROUTINE EVAL_PREPOST_CREATE

!====================================!
!Compute quantile numbers
!====================================!

SUBROUTINE EVAL_QUANTILENR(X,W,VecSize,K,QNR)
! In this subroutine we calculate the quantileNUMBER for the REAL(KIND=KIND_REAL) VecSize-vector X, with weights W
! The vector X must NOT be ordered on input and remains UNCHANGED on output
! The parameter K determines the number of quantiles
!       e.g. K=9 corresponds to deciles, K=99 are percentiles, K=4 are quintiles
! The output consists of the VecSize integer vector QNR with the quantile-numbers
! The subroutine essentially calls another subroutine QUANTILEK which produces the quantileVALUES,
!   which are then used to assign the quantilenumbers.
!   Since QUANTILEK is based on ordered vectors, we use two working vectors X2 and W2 
!

    INTEGER                                                 :: k                                    !k:(the number of quantiles -1)                                                             - input
    INTEGER                                                 :: VecSize                              !VecSize: size of the vectors x and w                                                       - input
    INTEGER,DIMENSION(VecSize)                              :: IRANK                                !Vector containing the rank of an observation, created to sort other variables later on     - local
    INTEGER,DIMENSION(VecSize)                              :: QNR                                  !qnr(VecSize):the quantile to which x(VecSize) belongs in the k+1-grouping                  - output
    INTEGER,DIMENSION(VecSize)                              :: OrigPos                              !Vector that contains the original position of an observation, used to resort later on      - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the quantilenumber is calculated,unsorted                - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x,unsorted                                      - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X2                                   !x(VecSize):the variable for which the quantilenumber is calculated,sorted                  - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W2                                   !w(VecSize):the weight attached to variable x,sorted                                        - local
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: QVAL                                 !qval(k+2):the quantile value                                                               - local

    ! PUT THE INPUTVECTORS X AND W INTO THE WORKING VECTORS
    X2(1:VecSize)=X(1:VecSize)
    W2(1:VecSize)=W(1:VecSize)

    !DETERMINE THE ORIGINAL POSITION OF THE INDIVIDUAL
    DO I=1,VecSize
        OrigPos(i) = i
    END DO

    !RANK AND SORT INPUT VECTORS TO DETERMINE QUANTILE NUMBER WITH QUANTILEK
    CALL RANKING(X2,IRANK)                                  ! INDEX VECTOR X2 IN IRANK
    CALL SORTING(X2,IRANK)                                  ! SORT VECTOR X2 ON THE INDEX IRANK
    CALL SORTING(W2,IRANK)                                  ! SORT VECTOR W2 ON THE INDEX IRANK
    CALL SORTING(OrigPos,IRANK)                             ! SORT VECTOR OrigPos ON THE INDEX IRANK

    CALL QUANTILEK(X2,W2,VecSize,K,QVAL,QNR)                ! CALCULATE QUANTILEVALUES FOR X2

    !DIFFERENT FROM DARTS USE QNR FROM QUANTILEK TO REORDER QUANTILE NUMBERS
    !RANK AND SORT DECILE NUMBER BACK TO ORIGINAL POSITION
    CALL RANKING(OrigPos,IRANK)                             ! INDEX VECTOR X2 IN IRANK
    CALL SORTING(QNR,IRANK)

END SUBROUTINE EVAL_QUANTILENR

!====================================!
!Compute quantile values
!====================================!

SUBROUTINE QUANTILEK(X,W,VecSize,K,Q,QNR)
!
! In this subroutine we calculate the quantilevalues for the REAL(KIND=KIND_REAL) VecSize-vector X, with weights W
! The vector must already be ordered 
! The parameter K determines the number of quantiles
!       e.g. K=9 corresponds to deciles, K=99 are percentiles, K=4 are quintiles
! The output consists of the K+2 vector Q with the quantile-values
!       and for each element of the vector x, the number of the quantile to which it belongs 
! The subroutine is inspired on Paarsch, but has been adapted in using weights for the observations

    INTEGER                                                 :: VecSize                              !VecSize:dimension of the vectors x and w - input
    INTEGER                                                 :: k                                    !k:the number of points where the ordinate is calculated (with exclusion of 0 and 1 on the abscissa) - input
    INTEGER,DIMENSION(VecSize)                              :: QNR                                  !Vector containing quantile number
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ITAG                                 !itag:contains the number of the observation where a given percentile ENDS, hence itag(j) means that the j-th quantile value is obtained as the value of V up to the itag(j)-th observation - local
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ICNT                                 !icnt:the number of observations in a given quantile - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the ordinate is calculated - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x - input
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: P                                    !vector with abscis values
    REAL(KIND=DPRECI),DIMENSION(0:VecSize)                  :: WCUM                                 !wcum:the cumulative sum of the weights, VecSize+1-vector - local
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: IWCUM                                !iwcum:the wcum-value corrsponding to the ex ante p-value - local
    REAL(KIND=DPRECI)                                       :: WSOM                                 !wsom:the sum of the weights - local
    REAL(KIND=DPRECI)                                       :: PROP                                 !Space between two abscis points
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: Q                                    !q:the quantile value (the value of x below which we find itag(j) of the observations) - local
    INTEGER                                                 :: start                                !Value to start construction loop                                  !counters
    
    !INITIALISE VARIABLES
    P(0)=0.D0
    Q(0)=0.D0
    WSOM=0.D0
    WCUM(0)=0.D0
    ITAG(0)=0
    ITAG(K+1)=VecSize
    ICNT(0)=0
    IWCUM(0)=0.D0

    !Compute sum weights
    DO  I=1,VecSize
        WSOM=WSOM+W(I)
        WCUM(I)=WCUM(I-1)+W(I)
    END DO

    !Compute P values and compare Wcum vector to determine ITAG
    PROP=1.D0/DBLE((K+1))
    start = 1
    DO J=1,K
        P(J)=PROP*J*WSOM
        DO I=start,VecSize
            IF(P(J).LE.WCUM(I)) THEN
                ITAG(J)=I
                start = I
                EXIT
            ENDIF
        END DO
    END DO
    P(K+1)=WSOM  

    !Compute quantile values and number of observations in each quantile
!    WRITE(15,'("Quantile Value(j) Numb Indivi in quantile Pop Weight Indiv")')
    DO J=1,K+1
        Q(J)=X(ITAG(J))
        ICNT(J)=ITAG(J)-ITAG(J-1)
        IWCUM(J)=WCUM(ITAG(J))
!        WRITE(15,'(2x,f15.2,12x,i10,2x,2f15.2)') Q(j),ICNT(j),WCUM(ITAG(J))-WCUM(ITAG(J-1))
    END DO

    !Compute the decilenumber for each observation
    !Change as compared to DARTS no double definition of individual in position N2
    DO J=1,K+1
        DO I=ITAG(J-1)+1,ITAG(J)
            QNR(I)=J
        END DO
    END DO
    
END SUBROUTINE QUANTILEK

!====================================!
!Compute median income
!====================================!

DOUBLE PRECISION FUNCTION MEDINC(X,W,VecSize)
!
! This function returns the median income of the unordered vector x, using weights w
! The vectors are given back unordered, since the routine runs on working vectors x2 and w2
! integer VecSize is the dimension of the vector
! the integer K used in QUANTILEK is set equal to 1 here

    INTEGER                                                 :: VecSize                              !VecSize:size of the vectors x and w                                    - input
    INTEGER,DIMENSION(VecSize)                              :: IRANK                                !Vector containing the rank of an observation, created to sort other variables later on     - local
    INTEGER,DIMENSION(VecSize)                              :: QNR                                  !qnr(VecSize):the quantile to which x(VecSize) belongs in the k+1-grouping   - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize): the variable for which the quantilenumber is calculated (unordered)        - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):  the weight attached to variable x (unordered)                             - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X2                                   !x2(VecSize): the variable for which the quantilenumber is calculated (to order)        - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W2                                   !w2(VecSize):  the weight attached to variable x (to order)                             - local
    REAL(KIND=DPRECI),DIMENSION(0:2)                        :: QVAL                                 !vector with quantil values, k=1 is median inc - output

    ! PUT THE INPUTVECTORS X AND W INTO THE WORKING VECTORS
    X2(1:VecSize)=X(1:VecSize)
    W2(1:VecSize)=W(1:VecSize)

    CALL RANKING(X2,IRANK)                                  ! INDEX VECTOR X2 IN IRANK
    CALL SORTING(X2,IRANK)                                  ! SORT VECTOR X2 ON THE INDEX IRANK
    CALL SORTING(W2,IRANK)                                  ! SORT VECTOR W2 ON THE INDEX IRANK

    !Call subroutine quantile values
    !Requires pre-ordered input, therefore separate function median income required
    CALL QUANTILEK(X2,W2,VecSize,1,QVAL,QNR)                ! CALCULATE QUANTILEVALUES FOR X2

    !WE DO NOT reorder QNR(VecSize) to original position here 

    MEDINC = QVAL(1)
!    write(15,'(''Median income'')')
!    write(15,'(f15.2)') MEDINC

END FUNCTION MEDINC

!====================================!
!Compute mean income
!====================================!

DOUBLE PRECISION FUNCTION MEANINC(X,W,VecSize)
!
! This function returns the mean income of the unordered vector x, using weights w
! integer VecSize is the dimension of the vector

    INTEGER                                                 :: VecSize                              !VecSize:size of the vectors x and w                                    - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X(VecSize)                           !x(VecSize):  the variable for which the quantilenumber is calculated        - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W(VecSize)                           !w(VecSize):  the weight attached to variable x                              - input
    REAL(KIND=DPRECI)                                       :: SumX                                 !Variable containing weighted sum of incomes    
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights
    
    SumX = 0.D0
    SumW = 0.D0
    MEANINC = 0.D0
    DO pers_j=1,VecSize
        SumX = SumX + W(pers_j)*X(pers_j)
        SumW = SumW + W(pers_j)
    END DO
    IF (INT(SumW).NE.0) THEN
        MEANINC = SumX/SumW    
    END IF
!    write(15,'(''Mean income'')')
!    write(15,'(f15.2)') MEANINC

END FUNCTION MEANINC

!====================================!
!DEFINE THE LABELS FOR THE OUTPUTTABLE 
!====================================!

SUBROUTINE LABELSDEF(TITLE,LABEL,VAL,N)

    ! THIS SUBROUTINE DEFINES THE LABELS FOR THE OUTPUTTABLE PER CHARACTERISTIC
    ! THE CHOICE IS DETERMINED ON THE BASIS OF THE TITLE

    INTEGER                                                 :: N                                    !Number of different values the characteristic can take
    CHARACTER(len=25)                                       :: TITLE                                !Title of the analysis being carried out
    CHARACTER(len=25),DIMENSION(N)                          :: LABEL                                !Vector with labels of the different values of the characterstic being analysed
    INTEGER,DIMENSION(N)                                    :: VAL                                  !Vector containing the value of the characteristic, helps to avoid strict increasing numbers

    DO I=1,N
        VAL(I)=I
    END DO
    IF(TITLE.EQ.'DECILE OF WELFARE CONCEPT') THEN
        LABEL(1)='DECILE 1                 '
        LABEL(2)='DECILE 2                 '
        LABEL(3)='DECILE 3                 '
        LABEL(4)='DECILE 4                 '
        LABEL(5)='DECILE 5                 '
        LABEL(6)='DECILE 6                 '
        LABEL(7)='DECILE 7                 '
        LABEL(8)='DECILE 8                 '
        LABEL(9)='DECILE 9                 '
        LABEL(10)='DECILE 10                 '
    ELSE IF(TITLE.EQ.'AGE CLASS                ') THEN
        LABEL(1)= '0 - 4 j                  ' 
        VAL(1)=1
        LABEL(2)= '5 - 9 j                  '
        VAL(2)=2
        LABEL(3)= '10 -14 j                 '    
        VAL(3)=3
        LABEL(4)= '15 - 19 j                '   
        VAL(4)=4
        LABEL(5)= '20 - 24 j                '   
        VAL(5)=5
        LABEL(6)= '25 - 29 j                '   
        VAL(6)=6
        LABEL(7)= '30 - 34 j                '   
        VAL(7)=7
        LABEL(8)= '35 - 39 j                '   
        VAL(8)=8
        LABEL(9)= '40 - 44 j                '   
        VAL(9)=9
        LABEL(10)='45 - 49 j                '   
        VAL(10)=10
        LABEL(11)='50 - 55 j                '   
        VAL(11)=11
        LABEL(12)='56 - 59 j                '   
        VAL(12)=12
        LABEL(13)='60 - 64 j                '   
        VAL(13)=13
        LABEL(14)='65 - 69 j                '   
        VAL(14)=14
        LABEL(15)='70 - 74 j                '   
        VAL(15)=15
        LABEL(16)='75 - 79 j                '   
        VAL(16)=16
        LABEL(17)='80 - 84 j                '   
        VAL(17)=17
        LABEL(18)='85 - 89 j                '   
        VAL(18)=18
        LABEL(19)='90 - 94 j                '   
        VAL(19)=19
        LABEL(20)='>= 95 j                  '
        VAL(20)=20
    ELSE IF(TITLE.EQ.'REGION                   ') THEN
        LABEL(1)='Brussels Capital City    '
        VAL(1)=1
        LABEL(2)='Flanders                 '
        VAL(2)=2
        LABEL(3)='Wallonia                 '
        VAL(3)=3
    ELSE IF(TITLE.EQ.'EMPLOYMENT STATUS        ') THEN        
        LABEL(1)='Individual not classified'
        VAL(1)=0
        LABEL(2)='Employee                 '
        VAL(2)=1
        LABEL(3)='Civil servant            '
        VAL(3)=2
        LABEL(4)='Self-employed            '
        VAL(4)=3
        LABEL(5)='Pensioner                '
        VAL(5)=4
        LABEL(6)='Sick/invalid             '
        VAL(6)=5
        LABEL(7)='Unemployed               '
        VAL(7)=6
        LABEL(8)='Student =inact. child <26'
        VAL(8)=7
        LABEL(9)='inactive child >=26      '
        VAL(9)=8
        LABEL(10)='inactive housewive       '
        VAL(10)=9
        LABEL(11)='inactive head of hh <65 '
        VAL(11)=10
        LABEL(12)='inactive head of hh >=65'
        VAL(12)=11
    ENDIF

END SUBROUTINE LABELSDEF

!==========================================!
!Draw a line of a give number of characters
!==========================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK
    INTEGER                                                 :: NUNIT
    
    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!======================================!
!Describe gains and losses per subgroup
!======================================!

SUBROUTINE EVAL_ANALCHAR(TITLE,ICHAR,VecSize,N,X1,X2,X3,X4,W,PRECIS)

! This subroutine describes the gains and losses per subgroup
! The subgroup is defined by the vector of characteristics in ichar(VecSize), which contains
!   n different values (e.g. 10 decilenumbers, or 5 ageclasses)
! For each subgroup is calculated: the mean of vector x1 to x2, taking the weights w into account (in matrix results)
!   the percentage of the group that wins, loses or is ex aequo on the basis of d1ls (in matrix gainloss), and
!   by using the treshold value in precis
!   and the FGT povertyindices for three values of the FGT-parameter alpha (in matrix poverty)
! The matrix with the FGT-indices has 3 times 3 cols:
!       col 1-3: alpha=0 (headcount)      , first pre reform, then post reform, then difference
!       col 4-6: alpha=1 (poverty deficit), first pre reform, then post reform, then difference
!       col 7-9: alpha=2                  , first pre reform, then post reform, then difference

! the input vectors coming from the calling program are:
!       x1: level of disposable income before social transfers before reform
!       x2: absolute change in disposable income before social transfers
!       x3: level of disposable income after social transfers before reform
!       x4: absolute change in disposable income after social transfers

    !Define a number of variables to be used later on in the module
    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    INTEGER                                                 :: N,I,IG                               !Number of different values for which results should be computed 
    INTEGER,DIMENSION(VecSize)                              :: ICHAR                                !Vector that contains the classification variable
    INTEGER,DIMENSION(N)                                    :: VAL                                  !Vector of size N that contains the N different values the classification variable can take                                  
    CHARACTER(len=25)                                       :: TITLE                                !Title of the analysis carried out with this call
    CHARACTER(len=25),DIMENSION(N)                          :: LABEL                                !Vector of length N containing the labels for the N different values of the characteristic
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X1                                   !Disposable income before social transfers
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X2                                   !Difference in disposable income before and after the reform in income before social transfers
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X3                                   !Disposable income after social transfers
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X4                                   !Difference in disposable income before and after the reform in income after social transfers
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !Vector of size VecSize with the weights that should be used to weight the vectors X1-X4
    REAL(KIND=KIND_REAL)                                    :: PRECIS                               !Precision level used to classify gainers and losers
    REAL(KIND=DPRECI)                                       :: GAMMA0                               !the normalised poverty gap before the reform
    REAL(KIND=DPRECI)                                       :: GAMMA1                               !the normalised poverty gap after the reform
    REAL(KIND=DPRECI)                                       :: WSUM                                 !Variable that contains the sum of the weights
    REAL(KIND=DPRECI),DIMENSION(N+1,5)                      :: RESULT                               !Matrix used to store some computational results, i.e. average disposable income etc
    REAL(KIND=DPRECI),DIMENSION(N+1,3)                      :: GAINLOSS                             !Matrix used to store the percentages of gainers and losers (in terms of concept of analysis)
    REAL(KIND=DPRECI),DIMENSION(N+1,4)                      :: GAINLOSS_AMOUNT                      !Matrix used to store the nominal gains and losses of gainers and losers (in terms of concept of analysis)
    REAL(KIND=DPRECI),DIMENSION(N+1,3)                      :: GAINLOSS_NOM                         !Matrix used to store the percentages of gainers and losers (in nominal terms)
    REAL(KIND=DPRECI),DIMENSION(N+1,4)                      :: GAINLOSS_NOM_AMOUNT                  !Matrix used to store the nominal gains and losses of gainers and losers  (in nominal terms)
    REAL(KIND=DPRECI),DIMENSION(N+1,3)                      :: INEQMEAS                             !Matrix containing the inequality measures by subroup 
    REAL(KIND=DPRECI),DIMENSION(EVAL_NUMPPERC,N+1,9)        :: POVERTY_BEFORE                       !Poverty statistics with income concepts before social transfers
    REAL(KIND=DPRECI),DIMENSION(EVAL_NUMPPERC,N+1,9)        :: POVERTY_AFTER                        !Poverty statistics with income concepts after social transfers
    REAL(KIND=DPRECI)                                       :: ALPHA                                !Power of the FGT index
    INTEGER                                                 :: pers_j                               !loop index
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: XINEQPRE                             !Vector with pre reform disposable income to compute inequality measure on subgroup
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: XINEQPOS                             !Vector with post reform disposable income to compute inequality measure on subgroup    
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: WINEQ                                !Vector with weights to compute inequality measure on subgroup
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: XPRESUB                              !Allocatable vector with pre reform disposable income to compute inequality measure on subgroup
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: XPOSSUB                              !Allocatable vector with post reform disposable income to compute inequality measure on subgroup
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: WSUB                                 !Allocatable vector with weights to compute inequality measure on subgroup    
    INTEGER                                                 :: COUNT                                !Counter to determine number of cases in a subgroup
    REAL(KIND=DPRECI),DIMENSION(EVAL_NUMPPERC,2)            :: EVAL_POVLINE_BEFORE                  !Vector that stores the poverty lines for the pre and post reform income before social transfers (1=pre, 2 = post)
    REAL(KIND=DPRECI),DIMENSION(EVAL_NUMPPERC,2)            :: EVAL_POVLINE_AFTER                   !Vector that stores the poverty lines for the pre and post reform income after social transfers (1=pre, 2 = post)
    REAL(KIND=DPRECI)                                       :: WSUMTEST                             !Variable that contains the sum of the weights to test whether individuals are observed yes or no
    REAL(KIND=DPRECI)                                       :: PRESUM                               !Variable that contains the sum of the pre reform values to test whether values of all individuals are equal to zero
    REAL(KIND=DPRECI)                                       :: POSTSUM                              !Variable that contains the sum of the post reform values to test whether values of all individuals are equal to zero
    CHARACTER(len=9)                                        :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    CHARACTER(len=7)                                        :: NCHARA                               !Variable containing the character equivalent of the converted value of the integer    
    CHARACTER(len=7)                                        :: NCHARB                               !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK
    INTEGER                                                 :: JCOL,IALPHA

    !Initialise a number of variables
    RESULT(1:N+1,1:5)=0.D0
    GAINLOSS(1:N+1,1:3)=0.D0
    GAINLOSS_AMOUNT(1:N+1,1:4)=0.D0
    GAINLOSS_NOM(1:N+1,1:3)=0.D0
    GAINLOSS_NOM_AMOUNT(1:N+1,1:4)=0.D0
    INEQMEAS(1:N+1,1:3)=0.D0
    POVERTY_BEFORE(1:EVAL_NUMPPERC,1:N+1,1:9)=0.D0
    POVERTY_AFTER(1:EVAL_NUMPPERC,1:N+1,1:9)=0.D0    
    WSUM=SUM(W(1:VecSize))

    ! DEFINE LABELS FOR THE GROUPS
    CALL LABELSDEF(TITLE,LABEL,VAL,N)

    !START computation of different poverty lines, necessary as input for computations in loop over individuals that follows
    !Computations for income before social transfers
    !Initialise vectors
    EVAL_POVLINE_BEFORE(1:EVAL_NUMPPERC,1:2)=0.D0
    !Poverty lines based on income before social transfers
    DO I=1,EVAL_NUMPPERC
        CALL EVAL_POVLINE_COMPUTE(X1,MIMOSIS_WEIGHT,n_rec_sub,EVAL_POVLINE_BEFORE(I,1),EVAL_POVPERC(I))
    END DO    
    ! CALCULATE THE POST REFORM income concept before social transfers
    XINEQPOS(1:VecSize)=0.D0        
    DO pers_j=1,VecSize
        XINEQPOS(pers_j) = X1(pers_j) + X2(pers_j)               
    END DO    
    DO I=1,EVAL_NUMPPERC
        CALL EVAL_POVLINE_COMPUTE(XINEQPOS,MIMOSIS_WEIGHT,n_rec_sub,EVAL_POVLINE_BEFORE(I,2),EVAL_POVPERC(I))
    END DO   

    !Computations for income after social transfers
    !Initialise vectors
    EVAL_POVLINE_AFTER(1:EVAL_NUMPPERC,1:2)=0.D0
    !Poverty lines based on income after social transfers
    DO I=1,EVAL_NUMPPERC
        CALL EVAL_POVLINE_COMPUTE(X3,MIMOSIS_WEIGHT,n_rec_sub,EVAL_POVLINE_AFTER(I,1),EVAL_POVPERC(I))
    END DO    
    ! CALCULATE THE POST REFORM income concept after social transfers
    XINEQPOS(1:VecSize)=0.D0        
    DO pers_j=1,VecSize
        XINEQPOS(pers_j) = X3(pers_j) + X4(pers_j)               
    END DO    
    DO I=1,EVAL_NUMPPERC
        CALL EVAL_POVLINE_COMPUTE(XINEQPOS,MIMOSIS_WEIGHT,n_rec_sub,EVAL_POVLINE_AFTER(I,2),EVAL_POVPERC(I))
    END DO    
    !Make all poverty lines equal to the baseline value after social transfers
    !kan tijdelijk zijn, daarom hier weinig orthodoxe manier van implementeren
    DO I=1,EVAL_NUMPPERC
        EVAL_POVLINE_AFTER(I,2)=EVAL_POVLINE_AFTER(I,1)
        EVAL_POVLINE_BEFORE(I,2)=EVAL_POVLINE_AFTER(I,1)
        EVAL_POVLINE_BEFORE(I,1)=EVAL_POVLINE_AFTER(I,1)
    END DO   
    
    ! START LOOP OVER UNITS OF observation to be analysed (for MIMOSIS normally individuals)
    DO pers_j=1,VecSize
        ! DETERMINE TO WHICH GROUP IG THE individual BELONGS
        DO IG=1,N
            IF(ICHAR(pers_j).EQ.VAL(IG)) EXIT       !IG NOW CONTAINS THE GROUP ID OF THIS individual
        END DO

        ! PUT THIS individual IN ROW IG OF THE OUTPUT TABLE
        RESULT(IG,1)=RESULT(IG,1)+W(pers_j)                                 ! SIZE OF THE GROUP
        RESULT(IG,2)=RESULT(IG,2)+W(pers_j)*X3(pers_j)                      ! DISPOSABLE INCOME
        RESULT(IG,3)=RESULT(IG,3)+W(pers_j)*X4(pers_j)                      ! D1DISP
        RESULT(IG,4)=RESULT(IG,4)+W(pers_j)*X4(pers_j)                      ! add D1DISP to compute percentage change later
        IF (INT(X3(pers_j)).NE.0) THEN
            RESULT(IG,5)=RESULT(IG,5)+W(pers_j)*100.D0*(X4(pers_j)/X3(pers_j))
        END IF

        !Compute statistics on gainers and losers in terms of concept of analysis
        IF(ABS(X4(pers_j)).LE.PRECIS) THEN      
            GAINLOSS(IG,3)=GAINLOSS(IG,3)+W(pers_j)                         ! EX AEQUO
        ELSE
            IF(X4(pers_j).GT.0.D0) THEN
                GAINLOSS(IG,1)=GAINLOSS(IG,1)+W(pers_j)                                         ! GAINERS
                GAINLOSS_AMOUNT(IG,1)=GAINLOSS_AMOUNT(IG,1)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE GAIN
                GAINLOSS_AMOUNT(IG,3)=GAINLOSS_AMOUNT(IG,3)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! GAIN IN NOMINAL TERMS
            END IF                
            IF(X4(pers_j).LT.0.D0) THEN
                GAINLOSS(IG,2)=GAINLOSS(IG,2)+W(pers_j)                                         ! LOSERS
                GAINLOSS_AMOUNT(IG,2)=GAINLOSS_AMOUNT(IG,2)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE LOSS
                GAINLOSS_AMOUNT(IG,4)=GAINLOSS_AMOUNT(IG,4)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! LOSS IN NOMINAL TERMS
            END IF                
        END IF

        !Compute statistics on gainers and losers in terms of nominal income concept
        IF(ABS(EVAL_D1DISP_NOES(pers_j)).LE.PRECIS) THEN      
            GAINLOSS_NOM(IG,3)=GAINLOSS_NOM(IG,3)+W(pers_j)                         ! EX AEQUO
        ELSE
            IF(EVAL_D1DISP_NOES(pers_j).GT.0.D0) THEN
                GAINLOSS_NOM(IG,1)=GAINLOSS_NOM(IG,1)+W(pers_j)                                         ! GAINERS
                GAINLOSS_NOM_AMOUNT(IG,1)=GAINLOSS_NOM_AMOUNT(IG,1)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE GAIN
                GAINLOSS_NOM_AMOUNT(IG,3)=GAINLOSS_NOM_AMOUNT(IG,3)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! GAIN IN NOMINAL TERMS
            END IF                
            IF(EVAL_D1DISP_NOES(pers_j).LT.0.D0) THEN
                GAINLOSS_NOM(IG,2)=GAINLOSS_NOM(IG,2)+W(pers_j)                                         ! LOSERS
                GAINLOSS_NOM_AMOUNT(IG,2)=GAINLOSS_NOM_AMOUNT(IG,2)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE LOSS
                GAINLOSS_NOM_AMOUNT(IG,4)=GAINLOSS_NOM_AMOUNT(IG,4)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! LOSS IN NOMINAL TERMS
            END IF                
        END IF

        ! AND ALSO PUT IT IN THE LAST ROW OF THE OUTPUT TABLE (ALL individualS)
        RESULT(N+1,1)=RESULT(N+1,1)+W(pers_j)                               ! SIZE OF THE GROUP
        RESULT(N+1,2)=RESULT(N+1,2)+W(pers_j)*X3(pers_j)                    ! DISPOSABLE INCOME
        RESULT(N+1,3)=RESULT(N+1,3)+W(pers_j)*X4(pers_j)                    ! D1DISP
        RESULT(N+1,4)=RESULT(N+1,4)+W(pers_j)*X4(pers_j)                    ! add D1DISP to compute percentage change later
        IF (INT(X3(pers_j)).NE.0) THEN
            RESULT(N+1,5)=RESULT(N+1,5)+W(pers_j)*100.D0*(X4(pers_j)/X3(pers_j))
        END IF
        
        !Compute statistics on gainers and losers in terms of concept of analysis
        IF(ABS(X4(pers_j)).LE.PRECIS) THEN      
            GAINLOSS(N+1,3)=GAINLOSS(N+1,3)+W(pers_j)                                             ! EX AEQUO
        ELSE
            IF(X4(pers_j).GT.0.D0) THEN
                GAINLOSS(N+1,1)=GAINLOSS(N+1,1)+W(pers_j)                                         ! GAINERS
                GAINLOSS_AMOUNT(N+1,1)=GAINLOSS_AMOUNT(N+1,1)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE GAIN
                GAINLOSS_AMOUNT(N+1,3)=GAINLOSS_AMOUNT(N+1,3)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! GAIN IN NOMINAL TERMS
            END IF                
            IF(X4(pers_j).LT.0.D0) THEN
                GAINLOSS(N+1,2)=GAINLOSS(N+1,2)+W(pers_j)                                         ! LOSERS
                GAINLOSS_AMOUNT(N+1,2)=GAINLOSS_AMOUNT(N+1,2)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE LOSS
                GAINLOSS_AMOUNT(N+1,4)=GAINLOSS_AMOUNT(N+1,4)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! LOSS IN NOMINAL TERMS
            END IF                
        END IF

        !Compute statistics on gainers and losers in terms of nominal income concept
        IF(ABS(EVAL_D1DISP_NOES(pers_j)).LE.PRECIS) THEN      
            GAINLOSS_NOM(N+1,3)=GAINLOSS_NOM(N+1,3)+W(pers_j)                                               ! EX AEQUO
        ELSE
            IF(EVAL_D1DISP_NOES(pers_j).GT.0.D0) THEN
                GAINLOSS_NOM(N+1,1)=GAINLOSS_NOM(N+1,1)+W(pers_j)                                         ! GAINERS
                GAINLOSS_NOM_AMOUNT(N+1,1)=GAINLOSS_NOM_AMOUNT(N+1,1)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE GAIN
                GAINLOSS_NOM_AMOUNT(N+1,3)=GAINLOSS_NOM_AMOUNT(N+1,3)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! GAIN IN NOMINAL TERMS
            END IF                
            IF(EVAL_D1DISP_NOES(pers_j).LT.0.D0) THEN
                GAINLOSS_NOM(N+1,2)=GAINLOSS_NOM(N+1,2)+W(pers_j)                                         ! LOSERS
                GAINLOSS_NOM_AMOUNT(N+1,2)=GAINLOSS_NOM_AMOUNT(N+1,2)+W(pers_j)*EVAL_D1DISP(pers_j)       ! WELFARE LOSS
                GAINLOSS_NOM_AMOUNT(N+1,4)=GAINLOSS_NOM_AMOUNT(N+1,4)+W(pers_j)*EVAL_D1DISP_NOES(pers_j)  ! LOSS IN NOMINAL TERMS
            END IF                
        END IF

        !Poverty computations on income before social transfers
        DO I=1,EVAL_NUMPPERC
            GAMMA0 = 0.D0
            IF(X1(pers_j).LT.EVAL_POVLINE_BEFORE(I,1).AND.INT(EVAL_POVLINE_BEFORE(I,1)*1.D06).NE.0) THEN
                GAMMA0=1.D0-(X1(pers_j)/EVAL_POVLINE_BEFORE(I,1))
            ENDIF
            GAMMA1 = 0.D0
            IF((X1(pers_j)+X2(pers_j)).LT.EVAL_POVLINE_BEFORE(I,2).AND.INT(EVAL_POVLINE_BEFORE(I,2)*1.D06).NE.0) THEN
                GAMMA1=1.D0-((X1(pers_j)+X2(pers_j))/EVAL_POVLINE_BEFORE(I,2))
            ENDIF
            DO IALPHA=1,3                                                       ! LOOP OVER 3 VALUES ALPHA FGT-INDEX
                ALPHA=REAL(IALPHA-1)                                            ! PARAMETER FOR FGT-INDEX
                JCOL=3*(IALPHA-1)                                               ! COLS IN OUTPUTMATRIX POVERTY FOR THIS ALPHA 
                !Compute poverty results for each subgroup
                IF(INT(GAMMA0*1.D06).NE.0)POVERTY_BEFORE(I,IG,JCOL+1)=POVERTY_BEFORE(I,IG,JCOL+1)+W(pers_j)*GAMMA0**ALPHA    !PRE REFORM POVERTY GAP
                IF(INT(GAMMA1*1.D06).NE.0)POVERTY_BEFORE(I,IG,JCOL+2)=POVERTY_BEFORE(I,IG,JCOL+2)+W(pers_j)*GAMMA1**ALPHA    !POST REFORM POVERTY GAP
                !Put each individual in the last row as well
                IF(INT(GAMMA0*1.D06).NE.0)POVERTY_BEFORE(I,N+1,JCOL+1)=POVERTY_BEFORE(I,N+1,JCOL+1)+W(pers_j)*GAMMA0**ALPHA  !PRE REFORM POVERTY GAP
                IF(INT(GAMMA1*1.D06).NE.0)POVERTY_BEFORE(I,N+1,JCOL+2)=POVERTY_BEFORE(I,N+1,JCOL+2)+W(pers_j)*GAMMA1**ALPHA  !POST REFORM POVERTY GAP
            END DO
        END DO

        !Poverty computations on income AFTER social transfers
        DO I=1,EVAL_NUMPPERC
            GAMMA0 = 0.D0
            IF(X3(pers_j).LT.EVAL_POVLINE_AFTER(I,1).AND.INT(EVAL_POVLINE_AFTER(I,1)*1.D06).NE.0) THEN
                GAMMA0=1.D0-(X3(pers_j)/EVAL_POVLINE_AFTER(I,1))
            ENDIF
            GAMMA1 = 0.D0            
            IF((X3(pers_j)+X4(pers_j)).LT.EVAL_POVLINE_AFTER(I,2).AND.INT(EVAL_POVLINE_AFTER(I,2)*1.D06).NE.0) THEN
                GAMMA1=1.D0-((X3(pers_j)+X4(pers_j))/EVAL_POVLINE_AFTER(I,2))
            ENDIF
            DO IALPHA=1,3                                                       ! LOOP OVER 3 VALUES ALPHA FGT-INDEX
                ALPHA=REAL(IALPHA-1)                                            ! PARAMETER FOR FGT-INDEX
                JCOL=3*(IALPHA-1)                                               ! COLS IN OUTPUTMATRIX POVERTY FOR THIS ALPHA 
                !Compute poverty results for each subgroup
                IF(INT(GAMMA0*1.D06).NE.0)POVERTY_AFTER(I,IG,JCOL+1)=POVERTY_AFTER(I,IG,JCOL+1)+W(pers_j)*GAMMA0**ALPHA    !PRE REFORM POVERTY GAP
                IF(INT(GAMMA1*1.D06).NE.0)POVERTY_AFTER(I,IG,JCOL+2)=POVERTY_AFTER(I,IG,JCOL+2)+W(pers_j)*GAMMA1**ALPHA    !POST REFORM POVERTY GAP
                !Put each individual in the last row as well
                IF(INT(GAMMA0*1.D06).NE.0)POVERTY_AFTER(I,N+1,JCOL+1)=POVERTY_AFTER(I,N+1,JCOL+1)+W(pers_j)*GAMMA0**ALPHA  !PRE REFORM POVERTY GAP
                IF(INT(GAMMA1*1.D06).NE.0)POVERTY_AFTER(I,N+1,JCOL+2)=POVERTY_AFTER(I,N+1,JCOL+2)+W(pers_j)*GAMMA1**ALPHA  !POST REFORM POVERTY GAP
            END DO
        END DO

    END DO

    ! CALCULATE THE AVERAGES OF THE TABLE
    DO IG=1,N+1                                                             ! DO FOR ALL ROWS
        !For Gainers and losers in terms of concept of analysis
        IF(INT(GAINLOSS(IG,1)).NE.0) THEN
            GAINLOSS_AMOUNT(IG,1)=GAINLOSS_AMOUNT(IG,1)/GAINLOSS(IG,1)
            GAINLOSS_AMOUNT(IG,3)=GAINLOSS_AMOUNT(IG,3)/GAINLOSS(IG,1)
        END IF                
        IF(INT(GAINLOSS(IG,2)).NE.0) THEN
            GAINLOSS_AMOUNT(IG,2)=GAINLOSS_AMOUNT(IG,2)/GAINLOSS(IG,2)
            GAINLOSS_AMOUNT(IG,4)=GAINLOSS_AMOUNT(IG,4)/GAINLOSS(IG,2)
        END IF                
        
        !For Gainers and losers in terms of nominal income concept
        IF(INT(GAINLOSS_NOM(IG,1)).NE.0) THEN
            GAINLOSS_NOM_AMOUNT(IG,1)=GAINLOSS_NOM_AMOUNT(IG,1)/GAINLOSS_NOM(IG,1)
            GAINLOSS_NOM_AMOUNT(IG,3)=GAINLOSS_NOM_AMOUNT(IG,3)/GAINLOSS_NOM(IG,1)
        END IF                
        IF(INT(GAINLOSS_NOM(IG,2)).NE.0) THEN
            GAINLOSS_NOM_AMOUNT(IG,2)=GAINLOSS_NOM_AMOUNT(IG,2)/GAINLOSS_NOM(IG,2)
            GAINLOSS_NOM_AMOUNT(IG,4)=GAINLOSS_NOM_AMOUNT(IG,4)/GAINLOSS_NOM(IG,2)
        END IF                

        IF(INT(RESULT(IG,2)*.1D-02).NE.0)RESULT(IG,4)=RESULT(IG,4)/RESULT(IG,2)    ! COL 4 is devided by total amount, i.e. result in column 2
        DO J=2,3                                                            ! COLS 2 TO 3 OF MATRIX RESULTS ARE DIVIDED BY POP SIZE
            IF(INT(RESULT(IG,1)).NE.0)RESULT(IG,J)=RESULT(IG,J)/RESULT(IG,1)
        END DO
        IF(INT(RESULT(IG,1)).NE.0)RESULT(IG,5)=RESULT(IG,5)/RESULT(IG,1)

        DO J=1,3                                                            ! COLS 1 TO 3 OF MATRIX GAINLOSS ARE DIVIDED BY POP SIZE
            IF(INT(RESULT(IG,1)).NE.0) THEN
                GAINLOSS(IG,J)=GAINLOSS(IG,J)/RESULT(IG,1)
                GAINLOSS_NOM(IG,J)=GAINLOSS_NOM(IG,J)/RESULT(IG,1)
            END IF    
        END DO

        !For poverty statistics before social transfers
        DO I=1,EVAL_NUMPPERC
            DO IALPHA=1,3                                                       ! LOOP OVER 3 VALUES ALPHA FGT-INDEX
                JCOL=3*(IALPHA-1)                                               ! COLS IN OUTPUTMATRIX POVERTY FOR THIS ALPHA 
                IF(INT(RESULT(IG,1)).NE.0)POVERTY_BEFORE(I,IG,JCOL+1)=POVERTY_BEFORE(I,IG,JCOL+1)/RESULT(IG,1)
                IF(INT(RESULT(IG,1)).NE.0)POVERTY_BEFORE(I,IG,JCOL+2)=POVERTY_BEFORE(I,IG,JCOL+2)/RESULT(IG,1)
            END DO

            DO J=3,9,3                                                          ! COLS WITH DIFFERENCES IN POVERTY RATES
                POVERTY_BEFORE(I,IG,J)=POVERTY_BEFORE(I,IG,J-1)-POVERTY_BEFORE(I,IG,J-2)
            END DO
        END DO

        !For poverty statistics AFTER social transfers
        DO I=1,EVAL_NUMPPERC
            DO IALPHA=1,3                                                       ! LOOP OVER 3 VALUES ALPHA FGT-INDEX
                JCOL=3*(IALPHA-1)                                               ! COLS IN OUTPUTMATRIX POVERTY FOR THIS ALPHA 
                IF(INT(RESULT(IG,1)).NE.0)POVERTY_AFTER(I,IG,JCOL+1)=POVERTY_AFTER(I,IG,JCOL+1)/RESULT(IG,1)
                IF(INT(RESULT(IG,1)).NE.0)POVERTY_AFTER(I,IG,JCOL+2)=POVERTY_AFTER(I,IG,JCOL+2)/RESULT(IG,1)
            END DO

            DO J=3,9,3                                                          ! COLS WITH DIFFERENCES IN POVERTY RATES
                POVERTY_AFTER(I,IG,J)=POVERTY_AFTER(I,IG,J-1)-POVERTY_AFTER(I,IG,J-2)
            END DO
        END DO
    
        RESULT(IG,1)=(RESULT(IG,1)/WSUM)*100.D0                             ! COL 1 OF RESULT IS ITSELF EXPRESSED AS PERCENTAGE
        RESULT(IG,4)=RESULT(IG,4)*100.D0                                    ! COL 3 OF RESULT IS ITSELF EXPRESSED AS PERCENTAGE        
        GAINLOSS(IG,1:3)=GAINLOSS(IG,1:3)*100.D0
        GAINLOSS_NOM(IG,1:3)=GAINLOSS_NOM(IG,1:3)*100.D0
        POVERTY_BEFORE(1:EVAL_NUMPPERC,IG,1:9)=POVERTY_BEFORE(1:EVAL_NUMPPERC,IG,1:9)*100.D0
        POVERTY_AFTER(1:EVAL_NUMPPERC,IG,1:9)=POVERTY_AFTER(1:EVAL_NUMPPERC,IG,1:9)*100.D0        
    END DO

    ! CALCULATE THE INEQUALITY MEASURES FOR EACH SUBGROUP
    !REMARK: idea was to allocate and deallocate following vectors time and time again within
    !the IG loop, but this failed with error 408: unable to acces memory
    !This error is likely to point at a memory leak produced by a forgotten deallocation of an allocatable variable
    !As far as I could see all allocatables were deallocated properly within the loop, but bringing 
    !the allocations outside the loop suppressed the error message
    !Another reason of getting this message might be that there is a wrong address accessing within the program
    !but debugging the full source code did not give any hint on this 
    !Therefore we keep the allocation and deallocation of the following three vectors outside the loop
    !As it currently is programmed, the use of these sub-vectors is of no use anymore. 
    !We keep them for possible future use if the reason of the error would become clear
    !Allocate sub-vector
    ALLOCATE(XPRESUB(VecSize))
    ALLOCATE(XPOSSUB(VecSize))
    ALLOCATE(WSUB(VecSize))        
    DO IG=1,N                                                               ! DO FOR ALL ROWS
        COUNT = 1
        XINEQPRE(1:VecSize)=0.D0
        XINEQPOS(1:VecSize)=0.D0        
        WINEQ(1:VecSize)=0.D0
        DO pers_j=1,VecSize
            IF(ICHAR(pers_j).EQ.VAL(IG)) THEN
                XINEQPRE(COUNT) = X3(pers_j)
                XINEQPOS(COUNT) = X3(pers_j) + X4(pers_j)               
                WINEQ(COUNT)    = W(pers_j)
                COUNT = COUNT + 1
            END IF
        END DO    
        COUNT = COUNT - 1  
        DO pers_j=1,COUNT
            XPRESUB(pers_j) = XINEQPRE(pers_j)
            XPOSSUB(pers_j) = XINEQPOS(pers_j)
            WSUB(pers_j)    = WINEQ(pers_j)            
        END DO
        WSUMTEST=SUM(WSUB(1:COUNT))
        PRESUM=SUM(XPRESUB(1:COUNT))
        POSTSUM=SUM(XPOSSUB(1:COUNT))
        IF (INT(WSUMTEST/100.D0).NE.0.AND.INT(PRESUM/100.D0).NE.0) THEN
            INEQMEAS(IG,1)= GENGINI(XPRESUB,WSUB,COUNT,2.D0,0)
        END IF    
        IF (INT(WSUMTEST/100.D0).NE.0.AND.INT(POSTSUM/100.D0).NE.0) THEN        
            INEQMEAS(IG,2)= GENGINI(XPOSSUB,WSUB,COUNT,2.D0,0)       
        END IF    
        INEQMEAS(IG,3)= INEQMEAS(IG,2) - INEQMEAS(IG,1)       
    END DO
   DEALLOCATE(XPRESUB)
   DEALLOCATE(XPOSSUB)
   DEALLOCATE(WSUB)
    ! CALCULATE THE OVERAL INEQUALITY MEASURES
    XINEQPOS(1:VecSize)=0.D0        
    DO pers_j=1,VecSize
        XINEQPOS(pers_j) = X3(pers_j) + X4(pers_j)               
    END DO    
    WSUMTEST=SUM(W(1:VecSize))
    PRESUM=SUM(X3(1:VecSize))
    POSTSUM=SUM(XINEQPOS(1:VecSize))
    IF (INT(WSUMTEST/100.D0).NE.0.AND.INT(PRESUM/100.D0).NE.0) THEN
        INEQMEAS(N+1,1)= GENGINI(X3,W,VecSize,2.D0,0)
    END IF    
    IF (INT(WSUMTEST/100.D0).NE.0.AND.INT(POSTSUM/100.D0).NE.0) THEN        
        INEQMEAS(N+1,2)= GENGINI(XINEQPOS,W,VecSize,2.D0,0)       
    END IF    
    INEQMEAS(N+1,3)= INEQMEAS(N+1,2) - INEQMEAS(N+1,1)         

    ! WRITE OUT THE TABLES TO THE TWO OUTPUT FILES
    DO IFILE=15,15
        WRITE(IFILE,'(A1)') ACHAR(12)
        CALL DRAWLINE(IFILE,53,'*')
        WRITE(IFILE,'(''* DISTRIBUTIONAL ANALYSIS BY SUBGROUP:              *'')')
        WRITE(IFILE,'(''* EFFECTS OF THE REFORM BY '',A25,''*'')')TITLE
        CALL DRAWLINE(IFILE,53,'*')
        IF(TITLE.EQ.'DECILE OF WELFARE CONCEPT') THEN
            WRITE(15,'(''WELFARE CONCEPT CONSTRUCTED AS: '')')
            WRITE(15,'(''   HOUSEHOLD DISPOSABLE INCOME AFTER SOCIAL TRANSFERS '')')
            WRITE(15,'(''   CORRECTED WITH EQUIVALENCE SCALE'')')
            WRITE(15,'(''   ATTRIBUTED TO EACH HOUSEHOLD MEMBER'')')
            WRITE(15,'(''   EQUIVALENCE SCALE USED IS'')')
            !write household equivalence scale used for construction of welfare concept
            IF (EVAL_TYPEES_WELF.EQ.1) THEN
                WRITE(15,'(''       FIRST HH-MEMBER IS 1, ADULTS WEIGHTED WITH 0.5 AND CHILDREN WITH 0.3 (=OECD modified scale)'')')
            ELSE IF (EVAL_TYPEES_WELF.EQ.2) THEN
                WRITE(15,'(''       FIRST HH-MEMBER IS 1, ADULTS WEIGHTED WITH 0.7 AND CHILDREN WITH 0.5 (=OECD scale)'')')
            ELSE IF (EVAL_TYPEES_WELF.EQ.3) THEN
                WRITE(15,'(''       SQUARE ROOT OF HOUSEHOLD SIZE'')')
            ELSE IF (EVAL_TYPEES_WELF.EQ.4) THEN
                WRITE(15,'(''       HOUSEHOLD SIZE WITHOUT SCALE CORRECTION'')')
            ELSE IF (EVAL_TYPEES_WELF.EQ.5) THEN
                WRITE(15,'(''       EQUAL TO 1 FOR EACH HOUSEHOLD'')')
            ELSE 
            END IF
        END IF

        ! TABLE WITH EFFECTS ON DISPOSABLE INCOME, TAXES, WELFARE
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': DISTRIBUTIONAL ANALYSIS'')')&
        &NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''CHANGES IN: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''            EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',2X,''BASELINE'',7X,''DIF'',7X,''DIF %'',7X,''AVG % DIF'')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,3F10.1,2x,F10.2,6x,F10.2)')LABEL(IG),(RESULT(IG,J),J=1,5)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,3F10.1,2x,F10.2,6x,F10.2)')(RESULT(N+1,J),J=1,5)

        ! TABLE WITH GAINERS AND LOSERS IN TERMS OF CONCEPT OF ANALYSIS
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': GAINERS AND LOSERS OF THE REFORM IN TERMS OF CONCEPT OF ANALYSIS'')')&
        &NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''ANALYSIS IS: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''             EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',2X,''%GAINERS'',3X,''%LOSERS'',9X,''% ='',&
        &8x,''GAINERS'',9X,''LOSERS'',14X,''='')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,3F10.1,2x,F10.1,3F15.0)')LABEL(IG),RESULT(IG,1),(GAINLOSS(IG,J),J=1,3),&
            &((GAINLOSS(IG,J)*RESULT(IG,1)*WSUM/10000.D0),J=1,3)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,3F10.1,2x,F10.1,3F15.0)')RESULT(N+1,1),(GAINLOSS(N+1,J),J=1,3),&
        &((GAINLOSS(N+1,J)*RESULT(IG,1)*WSUM/10000.D0),J=1,3)

        !TABLE WITH GAINS AND LOSSES IN TERMS OF CONCEPT OF ANALYSIS
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': AVERAGE GAINS AND LOSSES OF THOSE WHO GAIN OR LOOSE IN TERMS OF &
        &CONCEPT OF ANALYSIS (in Euro per month)'')') NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''ANALYSIS IS: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''             EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        WRITE(IFILE,'(''NOMINAL IS:  DISPOSABLE INCOME AFTER SOCIAL TRANSFERS OF THE INDIVIDUAL'')')        
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',4X,''AVG. GAIN'',1X,''AVG. LOSS'',11X,''AVG. GAIN'',1X,''AVG. LOSS'')')
        WRITE(IFILE,'(45X,''ANALYSIS'',19X,''NOMINAL AMOUNT'')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,3F10.1,10x,2F10.1)')LABEL(IG),RESULT(IG,1),(GAINLOSS_AMOUNT(IG,J),J=1,4)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,3F10.1,10x,2F10.1)')RESULT(N+1,1),(GAINLOSS_AMOUNT(N+1,J),J=1,4)

        ! TABLE WITH GAINERS AND LOSERS IN TERMS OF NOMINAL INCOME CONCEPT
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': GAINERS AND LOSERS OF THE REFORM IN TERMS OF NOMINAL INCOME CONCEPT'')')&
        &NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''NOMINAL IS:  DISPOSABLE INCOME AFTER SOCIAL TRANSFERS OF THE INDIVIDUAL'')')        
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',2X,''%GAINERS'',3X,''%LOSERS'',9X,''% ='',&
        &8x,''GAINERS'',9X,''LOSERS'',14X,''='')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,3F10.1,2x,F10.1,3F15.0)')LABEL(IG),RESULT(IG,1),(GAINLOSS_NOM(IG,J),J=1,3),&
            &((GAINLOSS_NOM(IG,J)*RESULT(IG,1)*WSUM/10000.D0),J=1,3)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,3F10.1,2x,F10.1,3F15.0)')RESULT(N+1,1),(GAINLOSS_NOM(N+1,J),J=1,3),&
        &((GAINLOSS_NOM(N+1,J)*RESULT(IG,1)*WSUM/10000.D0),J=1,3)

        !TABLE WITH GAINS AND LOSSES IN TERMS OF NOMINAL INCOME CONCEPT
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': AVERAGE GAINS AND LOSSES OF THOSE WHO GAIN OR LOOSE IN TERMS OF &
        &NOMINAL INCOME CONCEPT (in Euro per month)'')') NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''ANALYSIS IS: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''             EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        WRITE(IFILE,'(''NOMINAL IS:  DISPOSABLE INCOME AFTER SOCIAL TRANSFERS OF THE INDIVIDUAL'')')        
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',4X,''AVG. GAIN'',1X,''AVG. LOSS'',11X,''AVG. GAIN'',1X,''AVG. LOSS'')')
        WRITE(IFILE,'(45X,''ANALYSIS'',19X,''NOMINAL AMOUNT'')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,3F10.1,10x,2F10.1)')LABEL(IG),RESULT(IG,1),(GAINLOSS_NOM_AMOUNT(IG,J),J=1,4)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,3F10.1,10x,2F10.1)')RESULT(N+1,1),(GAINLOSS_NOM_AMOUNT(N+1,J),J=1,4)

        ! TABLE WITH FGT0 POVERTY INDICES (relative amounts)
        WRITE(IFILE,'(/,''EFFECT OF THE REFORM ON POVERTY INCIDENCE (MEASURE FGT0)'')')
        WRITE(IFILE,'(''CHANGES IN: DISPOSABLE INCOME BEFORE OR AFTER SOCIAL TRANSFERS (attributed to each &
        &household member)'')')
        WRITE(IFILE,'(''            EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB        
        DO I=1,EVAL_NUMPPERC
            WRITE (NCHARA, '(I7)') I
            OUTPUTFMT = 'A' // ACHAR(48 + LEN(NCHAR(NONBLK(NCHAR):)) + 1 + LEN(NCHARA(NONBLK(NCHARA):)) )
            NCHARB = NCHAR(NONBLK(NCHAR):) // ACHAR(46) // NCHARA(NONBLK(NCHARA):)
            IF (EVAL_TYPEPOV.EQ.1) THEN
                WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': THE POVERTY LINE IS '',F5.1,''% OF THE MEAN INCOME'')')&
                &NCHARB(NONBLK(NCHARB):),100.D0*EVAL_POVPERC(I)
            ELSE IF (EVAL_TYPEPOV.EQ.2) THEN
                WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': THE POVERTY LINE IS '',F5.1,''% OF THE MEDIAN INCOME'')')&
                &NCHARB(NONBLK(NCHARB):),100.D0*EVAL_POVPERC(I)
            END IF
            WRITE(IFILE,'(55X,''IN % OF THE POPULATION'')')
            WRITE(IFILE,'(43X,''BEFORE SOCIAL TRANSFERS'',7x,''AFTER SOCIAL TRANSFERS'')') 
            WRITE(IFILE,'(''VALUE OF POVERTY LINE'',13X,2(2X,2F10.1,10X))')&
            &EVAL_POVLINE_BEFORE(I,1),EVAL_POVLINE_BEFORE(I,2),EVAL_POVLINE_AFTER(I,1),EVAL_POVLINE_AFTER(I,2)
            WRITE(IFILE,'(''GROUP'',23X,''% OF POP'',2(2X,''BASELINE'',1X,''POST REF.'',7X,''DIFF.''))')
            DO IG=1,N
                WRITE(IFILE,'(A25,1x,F10.1,2(2F10.1,2x,F10.1))')LABEL(IG),RESULT(IG,1),(POVERTY_BEFORE(I,IG,J),J=1,3),&
                &(POVERTY_AFTER(I,IG,J),J=1,3)
            END DO 
            WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,F10.1,2(2F10.1,2x,F10.1))')RESULT(N+1,1),(POVERTY_BEFORE(I,N+1,J),J=1,3),&
            &(POVERTY_AFTER(I,N+1,J),J=1,3)
            
            IF (I.EQ.EVAL_NUMPPERC) THEN
                ! TABLE WITH FGT1 POVERTY INDICES (relative amounts)
                COUNT_TAB = COUNT_TAB + 1
                WRITE (NCHAR, '(I7)') COUNT_TAB
                OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
                WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': EFFECT OF THE REFORM ON POVERTY DEPTH (MEASURE FGT1)'')')&
                &NCHAR(NONBLK(NCHAR):)
                WRITE(IFILE,'(''CHANGES IN: DISPOSABLE INCOME BEFORE OR AFTER SOCIAL TRANSFERS (attributed to each &
                &household member)'')')
                WRITE(IFILE,'(''            EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
                IF (EVAL_TYPEPOV.EQ.1) THEN
                    WRITE(IFILE,'(''THE POVERTY LINE IS '',F5.1,''% OF THE MEAN INCOME'')')100.D0*EVAL_POVPERC(I)
                ELSE IF (EVAL_TYPEPOV.EQ.2) THEN
                    WRITE(IFILE,'(''THE POVERTY LINE IS '',F5.1,''% OF THE MEDIAN INCOME'')')100.D0*EVAL_POVPERC(I)            
                END IF
                WRITE(IFILE,'(43X,''BEFORE SOCIAL TRANSFERS'',7x,''AFTER SOCIAL TRANSFERS'')') 
                WRITE(IFILE,'(''VALUE OF POVERTY LINE'',13X,2(2X,2F10.1,10X))')&
                &EVAL_POVLINE_BEFORE(I,1),EVAL_POVLINE_BEFORE(I,2),EVAL_POVLINE_AFTER(I,1),EVAL_POVLINE_AFTER(I,2)
                WRITE(IFILE,'(''GROUP'',23X,''% OF POP'',2(2X,''BASELINE'',1X,''POST REF.'',7X,''DIFF.''))')
                DO IG=1,N
                    WRITE(IFILE,'(A25,1x,F10.1,2(2F10.1,2x,F10.1))')LABEL(IG),RESULT(IG,1),(POVERTY_BEFORE(I,IG,J),J=4,6),&
                    &(POVERTY_AFTER(I,IG,J),J=4,6)                    
                END DO 
                WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,F10.1,2(2F10.1,2x,F10.1))')RESULT(N+1,1),(POVERTY_BEFORE(I,N+1,J),J=4,6),&
                &(POVERTY_AFTER(I,N+1,J),J=4,6)
            END IF
        
        END DO
        
        ! TABLE WITH EFFECTS ON INEQUALITY OF THE REFORM
        COUNT_TAB = COUNT_TAB + 1
        WRITE (NCHAR, '(I7)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': EFFECT OF THE REFORM ON INEQUALITY (GINI COEFFICIENT)'')')&
        &NCHAR(NONBLK(NCHAR):)
        WRITE(IFILE,'(''CHANGES IN: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''            EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        WRITE(IFILE,'(/''GROUP'',23X,''% OF POP'',2X,''BASELINE'',1X,''POST REF.'',7X,''DIFF.'')')
        DO IG=1,N
            WRITE(IFILE,'(A25,1x,F10.1,2F10.1,2x,F10.1)')LABEL(IG),RESULT(IG,1),((INEQMEAS(IG,J)*1.D2),J=1,3)
        END DO 
        WRITE(IFILE,'(/''ALL INDIVIDUALS'',11X,F10.1,2F10.1,2x,F10.1)')RESULT(N+1,1),((INEQMEAS(N+1,J)*1.D2),J=1,3)

    END DO !END OF LOOP OVER THE OUTPUTFILE

END SUBROUTINE EVAL_ANALCHAR

!========================================!
!Compute the generalized Gini coefficient
!========================================!

DOUBLE PRECISION FUNCTION GENGINI(X,W,VecSize,RHO,K)
!
! This function returns the generalized Gini coefficient for the unordered vector x, using weights w
! The vectors are given back unordered, since the routine runs on working vectors x2 and w2
! integer VecSize is the dimension of the vector
! double precision rho is the parameter of the generalized Gini (=2 for the ordinary gini) 
! integer k determines how the Gini is calculated: see subroutines CONC, LOR and LORDINATE
!        k=4 is estimation on quintiles ordinates, k=9 on decile, k=0 on individual observations

! The function calls the function CONC, which needs an ordered vector as input
! Hence vectors x and w are first put in working vectors and these are ordered 

    INTEGER                                                 :: VECSIZE                              !Number of cases used to run the loop
    INTEGER                                                 :: K                                    !k:the number of points where the ordinate is calculated (with exclusion of 0 and 1 on the abscissa) - input
    INTEGER,DIMENSION(VecSize)                              :: IRANK                                !Vector containing the rank of an observation, created to sort other variables later on     - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the quantilenumber is calculated,unsorted                - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x,unsorted                                      - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X2                                   !x(VecSize):the variable for which the quantilenumber is calculated,sorted                  - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W2                                   !w(VecSize):the weight attached to variable x,sorted                                        - local
    REAL(KIND=DPRECI)                                       :: RHO                                  !rho: de parameter of the generalised Gini (2. giving the ordinary Gini)

    !PUT THE INPUTVECTORS X AND W INTO THE WORKING VECTORS
    X2(1:VECSIZE)=X(1:VECSIZE)
    W2(1:VECSIZE)=W(1:VECSIZE)
    
    !SORT THE WORKING VECTORS
    CALL RANKING(X2,IRANK)    ! PRODUCES RANKS FOR THE VECTOR X2 (ASCENDING) IN IRANK
    CALL SORTING(X2,IRANK)    ! USES THE VECTOR IRANK TO ORDER VECTOR X2 IN ASCENDING ORDER
    CALL SORTING(W2,IRANK)    ! USES THE VECTOR IRANK TO ORDER VECTOR W2 IN ASCENDING ORDER
    
    !CALL THE FUNCTION CONC TO CALCULATE THE GINI AS A CONCENTRATION COEFFICIENT
    GENGINI=CONC(X2,W2,VECSIZE,RHO,K,'L')

END FUNCTION GENGINI

!===============================================================!
!Compute the Ginicoefficient as a concentration coefficient C(a)
!===============================================================!

DOUBLE PRECISION FUNCTION CONC(a,w,VecSize,rho,k,gl)

! This function calculates the Ginicoefficient as a concentration coefficient C(a), in which 
!   a: the double precision vector of elements for which the coefficient has to be calculated
! The name CONC derives form an earlier version of the routine which allowed to calculate the
! more general concentration coefficients, where vector a could be orderd on another vector o.
! In this version the routine always orders vector a on itself (in ascending order).
! W.o.w. the CONC(a) here is a concentrationindex C(a,a). 
! The function  calls subroutine LOR (for k=0), or LORDINATE (for k/=0)
! The vectors a and w have to be ordered in ascending order when entering the routine

!REMARK: vectors P and L are allocated at maximal size here and then transfered to Lordinate and allocated with a smaller size there

    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    INTEGER                                                 :: k                                    !k: the number of quantiles, where the ordinates are calculated e.g. k=9 corresponds to deciles, k=99 are percentiles, k=4 are quintiles to work with ordinates at all individual observations, put k=0
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: A                                    !x(VecSize):the variable for which the quantilenumber is calculated,sorted                  - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x,sorted                                        - local
    REAL(KIND=KIND_REAL),DIMENSION(0:VecSize)               :: P                                    !Vector which is maximal of size VecSize+1 which stores the computed abscis points
    REAL(KIND=KIND_REAL),DIMENSION(0:VecSize)               :: L                                    !Vector which is maximal of size VecSize+1 which stores the computed ordinate points
    REAL(KIND=DPRECI)                                       :: KAPPA                                !intermediate variable used to compute the generalised gini as defined by Duclos
    REAL(KIND=DPRECI)                                       :: RHO                                  !rho: de parameter of the generalised Gini (2. giving the ordinary Gini)
    CHARACTER(Len=1)                                        :: GL                                   !gl: a character variable 'l' or 'g' used in the subroutines LOR and LORDINATE, which determines whether Lorens ('l') or generalized ('g') Lorenz curve ordinates are calculated
    REAL(KIND=KIND_REAL)                                    :: DP                                   !Difference in two abscissa points
    REAL(KIND=KIND_REAL)                                    :: SOM                                  !Sum of estimated elements on the ordinate
    INTEGER                                                 :: LOOPSTOP
    
    ! CALCULATE VALUES FOR ABSCISSE (VECTOR P) AND LORENZ ORDINATES (VECTOR L)
    
    IF(K.EQ.0) THEN   ! FOR K=0 WE CALL LOR WHICH WORKS WITH INDIVIDUAL DATA POINTS
        CALL LOR(A,W,VecSize,P,L,GL)
        LOOPSTOP=VecSize-1
    ELSE                  ! CASE OF GROUPED DATA IN K+1 GROUPS, DONE IN SUBROUTINE LORDINATE
        CALL LORDINATE(A,W,VecSize,K,GL,P,L)
        LOOPSTOP=K
    END IF

    ! USE THE ORDINATES IN VECTOR L IN THE INDEX FORMULA FOR THE GENERALISED GINI OF DUCLOS
    !REMARK: Do you have to take P(I+1)-P(I-1) or P(I+1)-P(I)? To check

    SOM=0.D0
    DO I=2,LOOPSTOP
        DP=P(I+1)-P(I-1)
        IF (P(i).GT.1.D0) THEN
            write(*,*) 'WARNING: the value P(i) in the function CONC became larger than 1'
            write(*,*) 'This might be because variables of a different precision are mixed, f.e. &
                       &single precision divided by double precision sum'
        end if    
        KAPPA=RHO*(RHO-1.)*(1-P(I))**(RHO-2.)
        SOM=SOM+(KAPPA/2.)*(P(I)-L(I))*DP
    END DO
    KAPPA=RHO*(RHO-1.)*(1-P(1))**(RHO-2.)   
    CONC=SOM+(KAPPA/2.)*(P(1)-L(1))*P(2)

END FUNCTION CONC

!====================================!
!Compute the abscisse and Lorenzordinates at individual points
!====================================!

SUBROUTINE LOR(a,w,VecSize,p,l,gl)

! For a real vector a, with weights w, both of length VecSize, this subroutine returns
!  the abscisse, and Lorenzordinates at all individual points
! This is done by simply cumulating the (weighted) values for the ordered vectors. 
! Hence no grouping of the data is done. For grouping in e.g. deciles, use the routine LORDINATE
! The vectors a and w have to be ordered in ascending order when entering the routine
! The output of the routine are the VecSize-vectors p (abscis-values) and l (ordinates)

    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: A                                    !x(VecSize):the variable for which the quantilenumber is calculated,sorted                  - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x,sorted                                        - local
    REAL(KIND=KIND_REAL),DIMENSION(0:VecSize)               :: P                                    !Vector which is maximal of size VecSize+1 which stores the computed abscis points
    REAL(KIND=KIND_REAL),DIMENSION(0:VecSize)               :: L                                    !Vector which is maximal of size VecSize+1 which stores the computed ordinate points
    REAL(KIND=KIND_REAL)                                    :: PCUM                                 !Variable that stores the sum of elements used to compute the abscis
    REAL(KIND=KIND_REAL)                                    :: LCUM                                 !Variable that stores the sum of elements used to compute the ordinate
    CHARACTER(Len=1)                                        :: GL                                   !gl: a character variable 'l' or 'g' used in the subroutines LOR and LORDINATE, which determines whether Lorens ('l') or generalized ('g') Lorenz curve ordinates are calculated
    REAL(KIND=KIND_REAL)                                    :: ASOM,WSOM                            !Sum of vector of analysis and vector of sample weights

    ! CALCULATE SUM OF THE WEIGHTS, AND SUM OF ELEMENTS IN VECTOR A
    P(0)=0.0
    L(0)=0.0
    WSOM=0.D0
    ASOM=0.D0
    DO  I=1,VecSize
        WSOM=WSOM+W(I)
        ASOM=ASOM+W(I)*A(I)
    END DO  

    ! CALCULATE ABSCIS- AND ORDINATE VALUES
    PCUM=0.D0
    LCUM=0.D0
    DO  I=1,VecSize
        PCUM=PCUM+W(I)
        P(I)=PCUM/WSOM
        LCUM=LCUM+W(I)*A(I)
        IF(GL.EQ.'L')L(I)=LCUM/ASOM
        IF(GL.EQ.'G')L(I)=LCUM/WSOM
    END DO

END SUBROUTINE LOR

!====================================!
!calculate the (generalised) Lorenzordinate 
!====================================!

SUBROUTINE LORDINATE(V,W,VecSize,K,GL,P,O)

! In this subroutine we calculate the (generalised) Lorenzordinate for the REAL(KIND=KIND_REAL) VecSize-vector V, with weights W,
!  based on a grouping of the data in K+1 groups
! The vector must already be ordered when entering the subroutine

! The parameter GL determines whether Lorenz (gl='l') or generalized (gl='g') curves are calculated
! The parameter K determines the number of points where the ordinate is estimated
!       e.g. K=9 corresponds to deciles, K=99 are percentiles, K=4 are quintiles
! The output consists of the K+2 vectors P (for the abscissa) and O (for the ordinates) 
! The subroutine is inspired on Paarsch, but has been adapted in using weights for the observations
!
! note that the last three variables are calculated but not used. They are available for later use, or for control
       
    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    INTEGER                                                 :: k                                    !k: the number of quantiles, where the ordinates are calculated e.g. k=9 corresponds to deciles, k=99 are percentiles, k=4 are quintiles to work with ordinates at all individual observations, put k=0                    
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ITAG                                 !itag:contains the number of the observation where a given percentile ENDS, hence itag(j) means that the j-th quantile value is obtained as the value of V up to the itag(j)-th observation - local
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ICNT                                 !icnt:the number of observations in a given quantile - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: V                                    !v(VecSize):the variable for which the ordinate is calculated - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable v - input
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: P                                    !p:the values for the abscissa, including 0 and 1 (hence k+2 elements) - output
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: O                                    !o:the ordinates, including 0 and 1 (hence k+2 elements) - output
    REAL(KIND=KIND_REAL),DIMENSION(0:VecSize)               :: WCUM                                 !wcum:the cumulative sum of the weights, VecSize+1-vector - local
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: Q                                    !q:the quantile value (the value of v below which we find itag(j) of the observations) - local
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: IWCUM                                !iwcum:the wcum-value corrsponding to the ex ante p-value - local
    REAL(KIND=KIND_REAL)                                    :: WSOM                                 !wsom:the sum of the weights - local
    CHARACTER(Len=1)                                        :: GL                                   !gl:a character variable 'l' or 'g' used in the subroutines LOR and LORDINATE, which determines whether Lorens ('l') or generalized ('g') Lorenz curve ordinates are calculated
    REAL(KIND=KIND_REAL)                                    :: PROP                                 !Proportion necessary to compute abscissa points

    !                       INITIALISATIONS
    P(0)=0.D0
    O(0)=0.D0
    WSOM=0.D0
    WCUM(0)=0.D0
    ITAG(0)=0
    ITAG(K+1)=VecSize
    Q(0)=0.D0
    ICNT(0)=0
    IWCUM(0)=0.D0
    
    !                       SUM OF THE WEIGHTS
    DO  I=1,VecSize
        WSOM=WSOM+W(I)
        WCUM(I)=WCUM(I-1)+W(I)
    END DO

    !                               CALCULATE P-VALUES ON THE BASIS OF K, AND COMPARE WITH VECTOR WCUM TO DETERMINE ITAG()
    PROP=1.D0/(K+1)
    DO J=1,K
        P(J)=PROP*J*WSOM
        DO I=1,VecSize
            IF(P(J).LE.WCUM(I)) THEN
                ITAG(J)=I
                EXIT
            ENDIF
        END DO
    END DO
    P(K+1)=WSOM  

    !                               CALCULATE QUANTILEVALUES, AND NUMBER OF OBSERVATIONS IN EACH QUANTILE      
    DO J=1,K+1
        Q(J)=V(ITAG(J))
        ICNT(J)=ITAG(J)-ITAG(J-1)
        IWCUM(J)=WCUM(ITAG(J))
    END DO
    
    !                       START CALCULATION OF ORDINATES
    ! FIRST CALCULATE THE INCOME MASS UP TO A GIVEN QUANTILEVALUE (INTGERAL OF H(X), LAMBERT P.25-26)
    DO J=1,K+1
        O(J)=0.D0
        DO I=1,ITAG(J)
            O(J)=O(J)+W(I)*V(I)
        END DO
    END DO
    
    ! FOR LORENZCURVE: DIVIDE BY TOTAL INCOMEMASS (INTEGRAL H(X) FROM 1 TO VecSize, HENCE O(K+1))
    ! FOR GENERALIZED: DIVIDE BY VecSize (IN THIS CASE: SUM OF WEIGHTS)
    DO J=1,K+1
        IF(GL.EQ.'L')O(J)=O(J)/O(K+1)
        IF(GL.EQ.'G')O(J)=O(J)/WSOM
        P(J)=P(J)/P(K+1)
    END DO
    
END SUBROUTINE LORDINATE

!====================================!
!calculates the Foster-Greer-Thorbecke poverty index 
!====================================!

DOUBLE PRECISION FUNCTION FGTINDEX(gamma,w,VecSize,alpha)
! This function calculates the Foster-Greer-Thorbecke poverty index for a vector of normalised

    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: GAMMA                                !poverty gaps gamma of length VecSize.
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !The weights w give the frequency weight of the observation
    REAL(KIND=KIND_REAL)                                    :: ALPHA                                !alpha is the FGT-parameter: 0 for the headcount, 1 for the normalised poverty deficit                            
    REAL(KIND=DPRECI)                                       :: WSUM                                 !Variable containing sum of weights in vector w
    
    WSUM=SUM(W(1:VecSize))
    FGTINDEX=0.D0
    
    DO I=1,VecSize
        IF(INT(GAMMA(I)*1.D06).NE.0)FGTINDEX=FGTINDEX+W(I)*GAMMA(I)**ALPHA
    END DO
    
    IF(INT(WSUM*1.D-03).NE.0) THEN
        FGTINDEX=FGTINDEX/WSUM
    ELSE
        WRITE(*,'(''/PROBLEM IN FUNCTION FGTINDEX: DENOMINATOR WSUM EQUALS '',F10.6)')WSUM
        WRITE(*,'('' WE CONTINUE WITH FGTINDEX PUT ON ZERO'')')
    END IF

END FUNCTION FGTINDEX

!======================================!
!subroutine to skip n lines on file k (reads as blanks)
!======================================!
SUBROUTINE RSKIP(k,n)

    INTEGER :: k,n

    DO i=1,n
        read(k,'(a)')
    END DO    

END SUBROUTINE RSKIP

!========================================================================================!
!Combine pre and post reform global budget results and compare them with official figures
!========================================================================================!
SUBROUTINE EVAL_GLOBBUD

    !Define type of variables used within this subroutine
    INTEGER                                                 :: N_ITEMS_EMPL                         !Number of global budget items analysed for global budget of employees
    INTEGER                                                 :: N_ITEMS_SELFEMPL                     !Number of global budget items analysed for global budget of self employed    
    CHARACTER(len=50),ALLOCATABLE,DIMENSION(:)              :: LAB_EMPL                             !labels of analysed global budget items of employees
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_EMPL_REFY                     !global budget figures of reference year for system of employees
    CHARACTER(len=50),ALLOCATABLE,DIMENSION(:)              :: LAB_SELFEMPL                         !labels of analysed global budget items of self employed
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_SELFEMPL_REFY                 !global budget figures of reference year for system of self employed
    INTEGER                                                 :: N_ITEMS_EMPL_COMPU                   !Number of items registered in computation file of employees
    INTEGER                                                 :: N_ITEMS_SELFEMPL_COMPU               !Number of items registered in computation file of self employed    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_EMPL_PRE                      !computed global budget figure for system of employees in pre reform
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_SELFEMPL_PRE                  !computed global budget figure for system of self employed in pre reform
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_EMPL_POST                     !computed global budget figure for system of employees in post reform
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: BUDGET_SELFEMPL_POST                 !computed global budget figure for system of self employed in post reform
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: ITEM_EMPL_PRE                        !number of global budget item to which the computed figure shoud be added for system of employees in pre reform
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: ITEM_SELFEMPL_PRE                    !number of global budget item to which the computed figure shoud be added for system of self employed in pre reform
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: ITEM_EMPL_POST                       !number of global budget item to which the computed figure shoud be added for system of employees in post reform
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: ITEM_SELFEMPL_POST                   !number of global budget item to which the computed figure shoud be added for system of self employed in post reform
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: BUDGET_EMPL                          !matrix with global budget figures (reference year, pre and post) for system of employees
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: BUDGET_SELFEMPL                      !matrix with global budget figures (reference year, pre and post) for system of self employed
    REAL(KIND=DPRECI),DIMENSION(2)                          :: PERC_BUD                             !vector to store intermediate computed percentages
    CHARACTER(len=9)                                        :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=3)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    

    !Set the number of computed items
    N_ITEMS_EMPL=12
    N_ITEMS_SELFEMPL=6
    N_ITEMS_EMPL_COMPU=14
    N_ITEMS_SELFEMPL_COMPU=7
    
    !Allocate allocatable variables
    ALLOCATE(LAB_EMPL(N_ITEMS_EMPL))
    ALLOCATE(BUDGET_EMPL_REFY(N_ITEMS_EMPL))    
    ALLOCATE(LAB_SELFEMPL(N_ITEMS_SELFEMPL))
    ALLOCATE(BUDGET_SELFEMPL_REFY(N_ITEMS_SELFEMPL))    
    ALLOCATE(BUDGET_EMPL_PRE(N_ITEMS_EMPL_COMPU))    
    ALLOCATE(BUDGET_SELFEMPL_PRE(N_ITEMS_SELFEMPL_COMPU))    
    ALLOCATE(BUDGET_EMPL_POST(N_ITEMS_EMPL_COMPU))       
    ALLOCATE(BUDGET_SELFEMPL_POST(N_ITEMS_SELFEMPL_COMPU))    
    ALLOCATE(ITEM_EMPL_PRE(N_ITEMS_EMPL_COMPU))    
    ALLOCATE(ITEM_SELFEMPL_PRE(N_ITEMS_SELFEMPL_COMPU))    
    ALLOCATE(ITEM_EMPL_POST(N_ITEMS_EMPL_COMPU))       
    ALLOCATE(ITEM_SELFEMPL_POST(N_ITEMS_SELFEMPL_COMPU))    
    ALLOCATE(BUDGET_EMPL(N_ITEMS_EMPL,6))    
    ALLOCATE(BUDGET_SELFEMPL(N_ITEMS_SELFEMPL,6))    

    !Read global budget figures of reference year for employees
    OPEN (UNIT=5, FILE=GLOBBUD_EMPL_REFY, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_EMPL_REFY
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_EMPL_REFY
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,1)
        DO i=1,N_ITEMS_EMPL
            READ (5,'(A50,F20.0)',IOSTAT=ierror) LAB_EMPL(i),BUDGET_EMPL_REFY(i)
        END DO
    END IF
    CLOSE (UNIT=5)
    !Read global budget figures of reference year for self employed
    OPEN (UNIT=5, FILE=GLOBBUD_SELFEMPL_REFY, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_REFY
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_REFY
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,1)
        DO i=1,N_ITEMS_SELFEMPL
            READ (5,'(A50,F20.0)',IOSTAT=ierror) LAB_SELFEMPL(i),BUDGET_SELFEMPL_REFY(i)
        END DO
    END IF
    CLOSE (UNIT=5)
    !Read global budget figures for employees computed for pre reform
    OPEN (UNIT=5, FILE=GLOBBUD_EMPL_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_EMPL_PRE
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_EMPL_PRE
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,2)
        DO i=1,N_ITEMS_EMPL_COMPU
            READ (5,'(100x,F20.2,40x,I10)',IOSTAT=ierror) BUDGET_EMPL_PRE(i),ITEM_EMPL_PRE(i)
        END DO
    END IF
    CLOSE (UNIT=5)
    !Read global budget figures for self employed computed for pre reform
    OPEN (UNIT=5, FILE=GLOBBUD_SELFEMPL_PRE, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_PRE
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_PRE
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,2)
        DO i=1,N_ITEMS_SELFEMPL_COMPU
            READ (5,'(100x,F20.2,40x,I10)',IOSTAT=ierror) BUDGET_SELFEMPL_PRE(i),ITEM_SELFEMPL_PRE(i)
        END DO
    END IF
    CLOSE (UNIT=5)
    !Read global budget figures for employees computed for POST reform
    OPEN (UNIT=5, FILE=GLOBBUD_EMPL_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_EMPL_POST
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_EMPL_POST
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,2)
        DO i=1,N_ITEMS_EMPL_COMPU
            READ (5,'(100x,F20.2,40x,I10)',IOSTAT=ierror) BUDGET_EMPL_POST(i),ITEM_EMPL_POST(i)
        END DO
    END IF
    CLOSE (UNIT=5)
    !Read global budget figures for self employed computed for POST reform
    OPEN (UNIT=5, FILE=GLOBBUD_SELFEMPL_POST, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror .NE.0) THEN
        WRITE (*,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_POST
        WRITE (15,'("Could not find or open ", a60)') GLOBBUD_SELFEMPL_POST
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        CALL RSKIP(5,2)
        DO i=1,N_ITEMS_SELFEMPL_COMPU
            READ (5,'(100x,F20.2,40x,I10)',IOSTAT=ierror) BUDGET_SELFEMPL_POST(i),ITEM_SELFEMPL_POST(i)
        END DO
    END IF
    CLOSE (UNIT=5)

    !Compute global budget pre and post reform
    BUDGET_EMPL(1:N_ITEMS_EMPL,1)=BUDGET_EMPL_REFY(1:N_ITEMS_EMPL)
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,1)=BUDGET_SELFEMPL_REFY(1:N_ITEMS_SELFEMPL)
    BUDGET_EMPL(1:N_ITEMS_EMPL,2:6)=0.D0
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,2:6)=0.D0
    DO I=1,N_ITEMS_EMPL_COMPU
        BUDGET_EMPL(ITEM_EMPL_PRE(i),2)=BUDGET_EMPL(ITEM_EMPL_PRE(i),2)+BUDGET_EMPL_PRE(i)        
        BUDGET_EMPL(ITEM_EMPL_POST(i),3)=BUDGET_EMPL(ITEM_EMPL_POST(i),3)+BUDGET_EMPL_POST(i)        
    END DO
    DO I=1,N_ITEMS_SELFEMPL_COMPU
        BUDGET_SELFEMPL(ITEM_SELFEMPL_PRE(i),2)=BUDGET_SELFEMPL(ITEM_SELFEMPL_PRE(i),2)+BUDGET_SELFEMPL_PRE(i)        
        BUDGET_SELFEMPL(ITEM_SELFEMPL_POST(i),3)=BUDGET_SELFEMPL(ITEM_SELFEMPL_POST(i),3)+BUDGET_SELFEMPL_POST(i)        
    END DO
    !Convert first three columns, expressed in 1000 Euros to Million Euros and compute difference in Euros
    BUDGET_EMPL(1:N_ITEMS_EMPL,6)=BUDGET_EMPL(1:N_ITEMS_EMPL,3)-BUDGET_EMPL(1:N_ITEMS_EMPL,2)
    BUDGET_EMPL(1:N_ITEMS_EMPL,1:3)=BUDGET_EMPL(1:N_ITEMS_EMPL,1:3)/1000
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,6)=BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,3)-BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,2)
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,1:3)=BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,1:3)/1000
    !column 4 contains difference between post reform and pre reform
    BUDGET_EMPL(1:N_ITEMS_EMPL,4)=BUDGET_EMPL(1:N_ITEMS_EMPL,3)-BUDGET_EMPL(1:N_ITEMS_EMPL,2)
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,4)=BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,3)-BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,2)
    !column 5 contains difference between pre reform and reference year
    BUDGET_EMPL(1:N_ITEMS_EMPL,5)=BUDGET_EMPL(1:N_ITEMS_EMPL,2)-BUDGET_EMPL(1:N_ITEMS_EMPL,1)
    BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,5)=BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,2)-BUDGET_SELFEMPL(1:N_ITEMS_SELFEMPL,1)

    !Write results to output file
    CALL DRAWLINE(15,133,'-')
    WRITE (NCHAR, '(I3)') COUNT_TAB
    OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
    WRITE(15,'(''TABLE ''' // OUTPUTFMT // ''': EFFECT OF THE REFORM IN TERMS OF GLOBAL BUDGET FIGURES'')')&
    &NCHAR(NONBLK(NCHAR):)
    CALL DRAWLINE(15,133,'-')
    WRITE(15,'(''TABLE ''' // OUTPUTFMT // '''.1: AGGREGATE STATISTICS OF GLOBAL BUDGET RESULTS PUBLISHED FOR SYSTEM OF &
    &EMPLOYEES'')')NCHAR(NONBLK(NCHAR):)
    WRITE(15,'(53X,1X,''REFERENCE YEAR'',7X,''BASELINE'',4X,''POST REFORM'',1X,''DIF POST - PRE'',2X,''DIF PRE - REF YEAR''&
    &,6X,''DIF POST - BAS'')')
    WRITE(15,'(53X,3X,''MILLION EURO'',3X,''MILLION EURO'',3X,''MILLION EURO'',4X,''AS % OF PRE'',4X,''AS % OF REF YEAR''&
    &,7X,''IN 1000 EUROS'')')
    DO I=1,N_ITEMS_EMPL
        PERC_BUD(1:2)=0.D0
        IF (INT(BUDGET_EMPL(i,2)).NE.0) THEN
            PERC_BUD(1) = 100.D0*BUDGET_EMPL(i,4)/BUDGET_EMPL(i,2)
        END IF
        IF (INT(BUDGET_EMPL(i,1)).NE.0) THEN
            PERC_BUD(2) = 100.D0*BUDGET_EMPL(i,5)/BUDGET_EMPL(i,1)
        END IF
        !convert the integer value I into a character value NCHAR with the Fortran internal Write function                                                        
        WRITE (NCHAR, '(I3)') I
        WRITE(15,'(A3,A50,3F15.1,F15.2,5x,F15.2,5x,F15.2)') ADJUSTL(NCHAR),LAB_EMPL(i),(BUDGET_EMPL(i,j),j=1,3),&
        &(PERC_BUD(j),j=1,2),BUDGET_EMPL(i,6)
    END DO
    CALL DRAWLINE(15,133,'-')
    WRITE (NCHAR, '(I3)') COUNT_TAB
    WRITE(15,'(''TABLE ''' // OUTPUTFMT // '''.2: AGGREGATE STATISTICS OF GLOBAL BUDGET RESULTS PUBLISHED FOR SYSTEM OF SELF &
    &EMPLOYED'')')NCHAR(NONBLK(NCHAR):)
    WRITE(15,'(53X,1X,''REFERENCE YEAR'',7X,''BASELINE'',4X,''POST REFORM'',1X,''DIF POST - BAS'',2X,''DIF BAS - REF YEAR''&
    &,6X,''DIF POST - BAS'')')
    WRITE(15,'(53X,3X,''MILLION EURO'',3X,''MILLION EURO'',3X,''MILLION EURO'',4X,''AS % OF BAS'',4X,''AS % OF REF YEAR''&
    &,7X,''IN 1000 EUROS'')')
    DO I=1,N_ITEMS_SELFEMPL
        PERC_BUD(1:2)=0.D0
        IF (INT(BUDGET_SELFEMPL(i,2)).NE.0) THEN
            PERC_BUD(1) = 100.D0*BUDGET_SELFEMPL(i,4)/BUDGET_SELFEMPL(i,2)
        END IF
        IF (INT(BUDGET_SELFEMPL(i,1)).NE.0) THEN
            PERC_BUD(2) = 100.D0*BUDGET_SELFEMPL(i,5)/BUDGET_SELFEMPL(i,1)
        END IF
        !convert the integer value I into a character value NCHAR with the Fortran internal Write function                                                        
        WRITE (NCHAR, '(I3)') I
        WRITE(15,'(A3,A50,3F15.1,F15.2,5x,F15.2,5x,F15.2)') ADJUSTL(NCHAR),LAB_SELFEMPL(i),(BUDGET_SELFEMPL(i,j),j=1,3),&
        &(PERC_BUD(j),j=1,2),BUDGET_SELFEMPL(i,6)
    END DO
    CALL DRAWLINE(15,133,'-')

    !Deallocate allocatable variables
    DEALLOCATE(LAB_EMPL)
    DEALLOCATE(BUDGET_EMPL)
    DEALLOCATE(LAB_SELFEMPL)
    DEALLOCATE(BUDGET_SELFEMPL)
    DEALLOCATE(BUDGET_EMPL_PRE)
    DEALLOCATE(BUDGET_SELFEMPL_PRE)
    DEALLOCATE(BUDGET_EMPL_POST)
    DEALLOCATE(BUDGET_SELFEMPL_POST)
    DEALLOCATE(ITEM_EMPL_PRE)
    DEALLOCATE(ITEM_SELFEMPL_PRE)
    DEALLOCATE(ITEM_EMPL_POST)
    DEALLOCATE(ITEM_SELFEMPL_POST)
    DEALLOCATE(BUDGET_EMPL)
    DEALLOCATE(BUDGET_SELFEMPL)

END SUBROUTINE EVAL_GLOBBUD

!======================================!
!Compute the impact of the reform on government revenues
!======================================!

SUBROUTINE EVAL_REVCALC

    REAL(KIND=DPRECI),DIMENSION(1:5,1:4)                    :: EVAL_INCTAB                          !table with summary on income statistics of the reform
    REAL(KIND=DPRECI),DIMENSION(5)                          :: SumInc                               !Vector with yearly amounts on income statistics
    REAL(KIND=DPRECI),DIMENSION(1:4,1:4)                    :: EVAL_REVTAB                          !table with summary on revenue statistics of the reform
    REAL(KIND=DPRECI),DIMENSION(4)                          :: SumRev                               !Vector with yearly amounts on revenues
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    CHARACTER(len=9)                                        :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=3)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    REAL(KIND=DPRECI),DIMENSION(4)                          :: NetEff                               !Vector with net effects

    !Construct the table with income statistics
    !Initialise the table
    NumAnal = 5    
    EVAL_INCTAB(1:NumAnal,1:4)=0.D0
    DO pers_j=1,n_rec
        !Compute amounts for pre reform on a yearly basis
        SumInc(1:NumAnal)=0.D0
        SumInc(1) = 0
        SumInc(2) = 0
        SumInc(3) = 0
        SumInc(4) = 0              
        SumInc(5) = SOCBEN_MINIMEX_PRE(pers_j)*4.00
        DO j=1,NumAnal
            IF (INT(SumInc(j)*100.D0).NE.0) THEN
                EVAL_INCTAB(j,1)=EVAL_INCTAB(j,1)+MIMOSIS_WEIGHT(pers_j)*SumInc(j)
            END IF
        END DO    
        !Compute amounts for post reform on a yearly basis
        SumInc(1:NumAnal)=0.D0
        SumInc(1) = 0
        SumInc(2) = 0
        SumInc(3) = 0
        SumInc(4) = 0
        SumInc(5) = SOCBEN_MINIMEX_POST(pers_j)*4.00
        DO j=1,NumAnal
            IF (INT(SumInc(j)*100.D0).NE.0) THEN
                EVAL_INCTAB(j,2)=EVAL_INCTAB(j,2)+MIMOSIS_WEIGHT(pers_j)*SumInc(j)
            END IF
        END DO    
    END DO
    !Compute absolute differences
    DO i=1,NumAnal
        EVAL_INCTAB(i,3) = EVAL_INCTAB(i,2) - EVAL_INCTAB(i,1) 
    END DO
    !Compute percentages and set euro values to billions
    DO i=1,NumAnal
        IF (INT(EVAL_INCTAB(i,1)*1.D-03).NE.0) THEN
            EVAL_INCTAB(i,4) = 100.D0*(EVAL_INCTAB(i,3)/EVAL_INCTAB(i,1))     
        END IF 
        EVAL_INCTAB(i,1:3) = EVAL_INCTAB(i,1:3)/1.D06 
    END DO

    !Store the sum of the expenses in the net revenue vector
    NetEff(1:4) = 0.D0
    NetEff(1) = -SUM(EVAL_INCTAB(1:NumAnal,1))
    NetEff(2) = -SUM(EVAL_INCTAB(1:NumAnal,2))
    
    !Construct the table with revenue statistics
    !Initialise the table
    NumAnal = 4    
    EVAL_REVTAB(1:NumAnal,1:4)=0.D0
    DO pers_j=1,n_rec
        !Compute amounts for pre reform on a yearly basis
        SumRev(1:NumAnal)=0.D0
        !multiplication factor 3.582 comes from contribution mass in RSZ in our sample 
        !for a full year devided by the contribution mass of the fourth quarter
        SumRev(1) = 0
        SumRev(2) = 0
        SumRev(3) = PIT_FINTAX_ANN_PRE(pers_j)
        SumRev(4) = 0.D0
        DO j=1,NumAnal
            IF (INT(SumRev(j)*100.D0).NE.0) THEN
                IF (j.EQ.3) THEN
                    !In principle all values of PIT_FINTAX should be zero for other members
                    !than the head of the household
                    !We always weight with the pre reform weights
                    IF (PIT_TURANK_ANN_PRE(pers_j).EQ.1) THEN
                        EVAL_REVTAB(j,1)=EVAL_REVTAB(j,1)+PIT_TUWEIGHT_PRE(pers_j)*SumRev(j)                
                    END IF    
                ELSE
                    EVAL_REVTAB(j,1)=EVAL_REVTAB(j,1)+MIMOSIS_WEIGHT(pers_j)*SumRev(j)                                
                END IF
            END IF
        END DO    
        !Compute amounts for post reform on a yearly basis
        SumRev(1:NumAnal)=0.D0
        SumRev(1) = 0
        SumRev(2) = 0
        SumRev(3) = PIT_FINTAX_ANN_POST(pers_j)
        SumRev(4) = 0.D0
        DO j=1,NumAnal
            IF (INT(SumRev(j)*100.D0).NE.0) THEN
                IF (j.EQ.3) THEN
                    !In principle all values of PIT_FINTAX should be zero for other members
                    !than the head of the household
                    !We always weight with the pre reform weights                    
                    IF (PIT_TURANK_ANN_POST(pers_j).EQ.1) THEN
                        EVAL_REVTAB(j,2)=EVAL_REVTAB(j,2)+PIT_TUWEIGHT_PRE(pers_j)*SumRev(j)                
                    END IF    
                ELSE
                    EVAL_REVTAB(j,2)=EVAL_REVTAB(j,2)+MIMOSIS_WEIGHT(pers_j)*SumRev(j)                                
                END IF
            END IF
        END DO    
    END DO
    !Compute absolute differences
    DO i=1,NumAnal
        EVAL_REVTAB(i,3) = EVAL_REVTAB(i,2) - EVAL_REVTAB(i,1) 
    END DO
    !Compute percentages and set euro values to billions
    DO i=1,NumAnal
        IF (INT(EVAL_REVTAB(i,1)*1.D-03).NE.0) THEN
            EVAL_REVTAB(i,4) = 100.D0*(EVAL_REVTAB(i,3)/EVAL_REVTAB(i,1))     
        END IF 
        EVAL_REVTAB(i,1:3) = EVAL_REVTAB(i,1:3)/1.D06 
    END DO

    !Create the first line of the summary table, i.e. last table of summary statistics
    SUMMARY_TABLE(1,1) = SUM(EVAL_REVTAB(1:NumAnal,1))
    SUMMARY_TABLE(1,2) = SUM(EVAL_REVTAB(1:NumAnal,2))
    !Add revenues to obtain net effect with expenses
    NetEff(1) = NetEff(1) + SUM(EVAL_REVTAB(1:NumAnal,1))
    NetEff(2) = NetEff(2) + SUM(EVAL_REVTAB(1:NumAnal,2))
    NetEff(3) = NetEff(2) - NetEff(1)
    IF (INT(NetEff(1)*1.D-03).NE.0) THEN
        NetEff(4) = 100.D0*(NetEff(3)/NetEff(1))     
    END IF 
    
    !Write the result to the log-file
    DO IFILE=15,15
        WRITE(15,'(A1)') ACHAR(12)
        WRITE (NCHAR, '(I3)') COUNT_TAB
        OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
        WRITE(IFILE,'(/,''TABLE ''' // OUTPUTFMT // ''': EFFECT OF THE REFORM ON GOVERNMENT REVENUES'')')&
        &NCHAR(NONBLK(NCHAR):)
        CALL DRAWLINE(IFILE,106,'-')
        WRITE(IFILE,'(31X,''       BASELINE         REFORM        CHANGES     CHANGES(%)        CHANGES'')')
        WRITE(IFILE,'(31X,''   MILLION EURO   MILLION EURO   MILLION EURO                     1000 EURO'')')
        CALL DRAWLINE(IFILE,106,'-')
        WRITE(IFILE,'(''EXPENDITURES:'')')
        WRITE(IFILE,'(''in terms of gross amounts: i.e. amounts before ss contributions of employees are subtracted'')')        
        WRITE(IFILE,'(''                    Pensions'',3x,5F15.2)') (EVAL_INCTAB(1,j),j=1,4),EVAL_INCTAB(1,3)*1000.D0
        WRITE(IFILE,'(''     Sickness and Invalidity'',3x,5F15.2)') (EVAL_INCTAB(2,j),j=1,4),EVAL_INCTAB(2,3)*1000.D0
        WRITE(IFILE,'(''           Family Allowances'',3x,5F15.2)') (EVAL_INCTAB(3,j),j=1,4),EVAL_INCTAB(3,3)*1000.D0
        WRITE(IFILE,'(''                 Unemployent'',3x,5F15.2)') (EVAL_INCTAB(4,j),j=1,4),EVAL_INCTAB(4,3)*1000.D0
        WRITE(IFILE,'(''          Guaranteed Minimum'',3x,5F15.2)') (EVAL_INCTAB(5,j),j=1,4),EVAL_INCTAB(5,3)*1000.D0        
        CALL DRAWLINE(IFILE,106,'-')
        WRITE(IFILE,'(''REVENUES:'')')
        WRITE(IFILE,'(''     Employers contributions'',3x,5F15.2)') (EVAL_REVTAB(1,j),j=1,4),EVAL_REVTAB(1,3)*1000.D0
        WRITE(IFILE,'(''    Employees contribuitions'',3x,5F15.2)') (EVAL_REVTAB(2,j),j=1,4),EVAL_REVTAB(2,3)*1000.D0
        WRITE(IFILE,'(''       Personal Income taxes'',3x,5F15.2)') (EVAL_REVTAB(3,j),j=1,4),EVAL_REVTAB(3,3)*1000.D0
        WRITE(IFILE,'(''              Indirect taxes'',3x,5F15.2)') (EVAL_REVTAB(4,j),j=1,4),EVAL_REVTAB(4,3)*1000.D0
        CALL DRAWLINE(IFILE,106,'-')
        WRITE(IFILE,'(''NET EFFECT (REVENUES - EXPEND.)'',5F15.2)') (NetEff(j),j=1,4),NetEff(3)*1000.D0
        CALL DRAWLINE(IFILE,106,'-')        
    END DO
    
END SUBROUTINE EVAL_REVCALC

!====================================!
!Subroutine that writes general remarks on the type of computation that is carried out
!====================================!

SUBROUTINE EVAL_WRITEANA

    REAL(KIND=DPRECI)                                       :: WSUMTOT  !Sum of all weights
    REAL(KIND=DPRECI)                                       :: WSUMSUB  !Sum of weights in subset
    REAL(KIND=DPRECI)                                       :: PROP     !Proportion of individuals in subset

    !write household equivalence scale used for construction of the concept of analysis
    IF (EVAL_TYPEES_ANAL.EQ.1) THEN
        LABES_ANAL ='FIRST HH-MEMBER IS 1, ADULTS WEIGHTED WITH 0.5 AND CHILDREN WITH 0.3 (=OECD modified scale)'
    ELSE IF (EVAL_TYPEES_ANAL.EQ.2) THEN
        LABES_ANAL ='FIRST HH-MEMBER IS 1, ADULTS WEIGHTED WITH 0.7 AND CHILDREN WITH 0.5 (=OECD scale)'
    ELSE IF (EVAL_TYPEES_ANAL.EQ.3) THEN
        LABES_ANAL ='SQUARE ROOT OF HOUSEHOLD SIZE'
    ELSE IF (EVAL_TYPEES_ANAL.EQ.4) THEN
        LABES_ANAL ='HOUSEHOLD SIZE WITHOUT SCALE CORRECTION'
    ELSE IF (EVAL_TYPEES_ANAL.EQ.5) THEN
        LABES_ANAL ='EQUAL TO 1 FOR EACH HOUSEHOLD'
    ELSE 
    END IF

    !Compute proportion of the individuals in subsample
    WSUMTOT=SUM(MIMOSIS_WEIGHT(1:n_rec))
    WSUMSUB=SUM(MIMOSIS_WEIGHT(1:n_rec_sub))
    PROP = 0.D0
    IF (EVAL_SUBSET.EQ.1) THEN    
        IF (INT(WSUMTOT*1.D-02).NE.0) THEN
            PROP = 100.D0*(WSUMSUB/WSUMTOT)
        END IF    
        WRITE(15,'(''ANALYSIS APPLIED ON SUBSET OF THE POPULATION'')')    
        WRITE(15,'(''THIS SUBSET COVERS '',F5.1,''% OF THE POPULATION'')') PROP       
        WRITE(15,'(''OR '',I10,'' INDIVIDUALS'')') NINT(WSUMSUB)               
    ELSE
        WRITE(15,'(''ANALYSIS APPLIED ON THE WHOLE POPULATION'')')        
        WRITE(15,'(''THERE ARE '',I10,'' INDIVIDUALS IN THE POPULATION'')') NINT(WSUMTOT)               
    END IF
    CALL DRAWLINE(15,100,'-')            
    WRITE(15,'(''GENERAL REMARKS ON CONCEPTS USED IN THE DIFFERENT TABLES OF DISTRIBUTIONAL ANALYSIS'')')    
    CALL DRAWLINE(15,100,'-')            
    WRITE(15,'(''HOUSEHOLD DISPOSABLE INCOME BEFORE TRANSFERS IS: '')')
    WRITE(15,'(''   SUM OF GROSS INCOMES OF ALL HOUSEHOLD MEMBERS FROM ACTIVITY AS EMPLOYEE OR AS SELF EMPLOYED'')')
    WRITE(15,'(''   GROSS INCOME OF EMPLOYEES INCLUDES OWN CONTRIBUTIONS BUT NOT THE CONTRIBUTIONS OF THE EMPLOYER'')')
    WRITE(15,'(''   EXPRESSED IN EUROS PER MONTH'')')            
    CALL DRAWLINE(15,100,'-')        
    WRITE(15,'(''HOUSEHOLD DISPOSABLE INCOME AFTER TRANSFERS IS: '')')
    WRITE(15,'(''   SUM OF GROSS INCOMES OF ALL HOUSEHOLD MEMBERS RECEIVED FROM DIFFERENT SOURCES'')')
    WRITE(15,'(''       SOURCES CONSIDERED: INCOME AS EMPLOYEE, AS SELF EMPLOYED, PENSIONS, UNEMPLOYMENT BENEFITS'')')
    WRITE(15,'(''                           BENEFITS OF SICKNESS AND INVALIDITY AND FAMILY ALLOWANCES'')')    
    WRITE(15,'(''   MINUS SUM OF SOCIAL SECURITY CONTRIBUTIONS PAID ON THESE INCOME COMPONENTS'')')    
    WRITE(15,'(''   MINUS SUM OF PERSONAL INCOME TAXES PAID BY ALL TAX UNITS WITHIN THE HOUSEHOLD'')')        
    WRITE(15,'(''   EXPRESSED IN EUROS PER MONTH'')')                
    CALL DRAWLINE(15,100,'-')        
    
END SUBROUTINE EVAL_WRITEANA

!====================================!
!Subroutine to compute the summary meausures of the distributional analysis
!====================================!

SUBROUTINE EVAL_SUMMARY(n_size)

    INTEGER                                                 :: n_size                               !number of individuals over the households that are considered for analysis
    REAL(KIND=DPRECI)                                       :: WSUMTEST                                 !Variable that contains the sum of the weights to test whether individuals are observed yes or no
    REAL(KIND=DPRECI)                                       :: PRESUM                                   !Variable that contains the sum of the pre reform values to test whether values of all individuals are equal to zero
    REAL(KIND=DPRECI)                                       :: POSTSUM                                  !Variable that contains the sum of the post reform values to test whether values of all individuals are equal to zero
    REAL(KIND=DPRECI),DIMENSION(3)                          :: GAINLOSS_GEN                             !Vector to store the gainers and losers computed with all individuals
    CHARACTER(len=9)                                        :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=3)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    !Compute average of the concept of analysis pre and post after social transfers
    SUMMARY_TABLE(2,1) = MEANINC(EVAL_DISPPRE,MIMOSIS_WEIGHT,n_size)
    SUMMARY_TABLE(2,2) = MEANINC(EVAL_DISPPOST,MIMOSIS_WEIGHT,n_size)
    
    !compute pre and post reform poverty lines
    !Only necessary for general summary computations of FGT index
    CALL EVAL_POVLINE_COMPUTE(EVAL_DISPPRE,MIMOSIS_WEIGHT,n_size,EVAL_POVLINE_PRE,EVAL_POVPERC(EVAL_NUMPPERC))
    !excluded for presentation FOD march 1, 2006
    !CALL EVAL_POVLINE_COMPUTE(EVAL_DISPPOST,MIMOSIS_WEIGHT,n_size,EVAL_POVLINE_POST,EVAL_POVPERC(EVAL_NUMPPERC))

    !compute vectors with gamma's, i.e. not necessarily equal to scalar gamma's used in ANALCHAR
    DO pers_j=1,n_size
        IF(EVAL_DISPPRE(pers_j).LT.EVAL_POVLINE_PRE.AND.INT(EVAL_POVLINE_PRE*1.D03).NE.0) THEN
            GAMMA0V(pers_j)=1.D0-(EVAL_DISPPRE(pers_j)/EVAL_POVLINE_PRE) 
        ENDIF
        IF(EVAL_DISPPOST(pers_j).LT.EVAL_POVLINE_PRE.AND.INT(EVAL_POVLINE_PRE*1.D03).NE.0) THEN
            GAMMA1V(pers_j)=1.D0-(EVAL_DISPPOST(pers_j)/EVAL_POVLINE_PRE) 
        ENDIF
    END DO

    !compute the sum of the sample weights
    WSUMTEST=SUM(MIMOSIS_WEIGHT(1:n_size))

    !CALCULATE THE FGT-INDICES PRE AND POST REFORM
    DO IPAR=0,2
        FGTPAR=REAL(IPAR)
        FGT0(IPAR)=FGTINDEX(GAMMA0V,MIMOSIS_WEIGHT,n_size,FGTPAR)
        FGT1(IPAR)=FGTINDEX(GAMMA1V,MIMOSIS_WEIGHT,n_size,FGTPAR)
        IF (IPAR.EQ.0) THEN
            SUMMARY_TABLE(IPAR+3,1) = FGT0(IPAR)*(WSUMTEST)
            SUMMARY_TABLE(IPAR+3,2) = FGT1(IPAR)*(WSUMTEST)
            SUMMARY_TABLE(IPAR+4,1) = FGT0(IPAR)
            SUMMARY_TABLE(IPAR+4,2) = FGT1(IPAR)       
        ELSE
            SUMMARY_TABLE(IPAR+4,1) = FGT0(IPAR)
            SUMMARY_TABLE(IPAR+4,2) = FGT1(IPAR)       
        END IF
    END DO

    !calculate the gini-coefficients pre and post reform if 
    ! a) sum of weights is not equal to zero and
    ! b) sum of incomes is not equal to zero
    PRESUM=SUM(EVAL_DISPPRE(1:n_size))
    POSTSUM=SUM(EVAL_DISPPOST(1:n_size))
    IF (INT(WSUMTEST/1000.D0).NE.0.AND.INT(PRESUM/1000.D0).NE.0) THEN    
        SUMMARY_TABLE(7,1)=GENGINI(EVAL_DISPPRE,MIMOSIS_WEIGHT,n_size,2.D0,0)
    END IF
    IF (INT(WSUMTEST/1000.D0).NE.0.AND.INT(POSTSUM/1000.D0).NE.0) THEN    
        SUMMARY_TABLE(7,2)=GENGINI(EVAL_DISPPOST,MIMOSIS_WEIGHT,n_size,2.D0,0)
    END IF

    !Compute column 3 and 4 of the summary table
    DO I=1,7
        IF (I.GT.3) THEN
            SUMMARY_TABLE(I,1:2)=100.D0*SUMMARY_TABLE(I,1:2)
        ENDIF
        SUMMARY_TABLE(I,3) = SUMMARY_TABLE(I,2) - SUMMARY_TABLE(I,1)       
        IF (INT(SUMMARY_TABLE(I,1)*100.D0).NE.0) THEN
            SUMMARY_TABLE(I,4) = 100.D0*SUMMARY_TABLE(I,3)/SUMMARY_TABLE(I,1)
        END IF
    END DO

    GAINLOSS_GEN(1:3) = 0.D0
    DO pers_j=1,n_size
        IF(ABS(EVAL_D1DISP(pers_j)).LE.EVAL_PRECIS) THEN      
            GAINLOSS_GEN(3)=GAINLOSS_GEN(3)+MIMOSIS_WEIGHT(pers_j)                       ! EX AEQUO
        ELSE
            IF(EVAL_D1DISP(pers_j).GT.0.D0)GAINLOSS_GEN(1)=GAINLOSS_GEN(1)+MIMOSIS_WEIGHT(pers_j) ! GAINERS
            IF(EVAL_D1DISP(pers_j).LT.0.D0)GAINLOSS_GEN(2)=GAINLOSS_GEN(2)+MIMOSIS_WEIGHT(pers_j) ! LOSERS
        ENDIF
    END DO
    IF (INT(WSUMTEST/1000.D0).NE.0) THEN    
        GAINLOSS_GEN(1:3) = 100.D0*GAINLOSS_GEN(1:3)/WSUMTEST 
    END IF

    !Write the result to the log-file
    COUNT_TAB = COUNT_TAB + 1    
    WRITE (NCHAR, '(I3)') COUNT_TAB
    OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
    DO IFILE=15,15
        WRITE(IFILE,'(''TABLE ''' // OUTPUTFMT // ''': SUMMARY OF THE REFORM'')')NCHAR(NONBLK(NCHAR):)
        CALL DRAWLINE(IFILE,108,'-')        
        WRITE(IFILE,'(''AGGREGATE BUDGET FIGURES'')')
        WRITE(IFILE,'(63X,''BASELINE'',6X,''REFORM'',5X,''ABS DIF'',7x,''% DIF'')')
        WRITE(IFILE,'(1x,''Revenues from taxes and s. sec. contributions (in bn Euro)'',4F12.1)')(SUMMARY_TABLE(1,j),j=1,4)
        CALL DRAWLINE(IFILE,108,'-')        
        WRITE(IFILE,'(''DISTRIBUTIONAL ANALYSIS'')')
        WRITE(IFILE,'(''CHANGES IN: DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''            EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
        IF (EVAL_TYPEPOV.EQ.1) THEN
            WRITE(IFILE,'(''THE POVERTY LINE IS '',F5.1,''% OF THE MEAN INCOME'')')100.D0*EVAL_POVPERC(EVAL_NUMPPERC)
        ELSE IF (EVAL_TYPEPOV.EQ.2) THEN
            WRITE(IFILE,'(''THE POVERTY LINE IS '',F5.1,''% OF THE MEDIAN INCOME'')')100.D0*EVAL_POVPERC(EVAL_NUMPPERC)
        END IF
        WRITE(IFILE,'(''AVERAGE OVER ALL INDIVIDUALS'')')
        WRITE(IFILE,'(63X,''BASELINE'',6X,''REFORM'',5X,''ABS DIF'',7x,''% DIF'')')
        WRITE(IFILE,'(52x,''Average'',4F12.1)')                         (SUMMARY_TABLE(2,j),j=1,4)
        WRITE(IFILE,'(38x,''Number of individuals'',2I12,2F12.1)')      (NINT(SUMMARY_TABLE(3,j)),j=1,2),(SUMMARY_TABLE(3,j),j=3,4)
        WRITE(IFILE,'(29x,''Poverty in % of the population'',4F12.1)')  (SUMMARY_TABLE(4,j),j=1,4)
        WRITE(IFILE,'(46x,''Poverty depth'',4F12.1)')                   (SUMMARY_TABLE(5,j),j=1,4)
        WRITE(IFILE,'(33x,''FGT poverty index, alpha=2'',4F12.1)')      (SUMMARY_TABLE(6,j),j=1,4)
        WRITE(IFILE,'(42x,''Inequality (Gini)'',4F12.1)')               (SUMMARY_TABLE(7,j),j=1,4)
        CALL DRAWLINE(IFILE,108,'-')        
        WRITE(IFILE,'(''ANALYSIS OF GAINERS AND LOSERS'')')
        WRITE(IFILE,'(''AVERAGE OVER ALL INDIVIDUALS'')')        
        WRITE(IFILE,'(29x,''% of the population that gains'',F12.1)')         GAINLOSS_GEN(1)    
        WRITE(IFILE,'(29x,''% of the population that loses'',F12.1)')         GAINLOSS_GEN(2)
        WRITE(IFILE,'(21x,''% of the population that is unaffected'',F12.1)') GAINLOSS_GEN(3)
        CALL DRAWLINE(IFILE,108,'-')        
    END DO

END SUBROUTINE EVAL_SUMMARY

!=========================================================================================!
!Subroutine to compute the summary meausures comparable with what is published for NAP_SI
!=========================================================================================!

SUBROUTINE EVAL_NAPSI(VecSize)

    INTEGER                                                 :: VecSize                              !Number of cases used to run the loop
    INTEGER                                                 :: K                                    !Number of cases used to run the loop
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: ASOM                                 !matrix containing sum of weighted amounts with which shares will be computed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: ASHA                                 !matrix containing shares
    INTEGER,DIMENSION(VecSize)                              :: QUANTNUM                             !Variable that contains the number of the quantile to which the individual belongs
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: SUBANALA                             !Vector with values to be analysed in subset
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: SUBANALB                             !Vector with values to be analysed in subset
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: SUBWEIGH                             !Vector with values of weight of vector to be analysed in subset
    INTEGER                                                 :: COUNT                                !Number of values for which 
    REAL(KIND=KIND_REAL)                                    :: MEDSING                              !Median income of households consisting of single adults
    REAL(KIND=KIND_REAL)                                    :: MEDPOV                               !Median income of households consisting of people below the poverty line
    REAL(KIND=DPRECI),DIMENSION(4,3)                        :: PovLine                              !PovLine:poverty line                                                   - output
    REAL(KIND=DPRECI),DIMENSION(3,4)                        :: PovMass                              !PovMass
    REAL(KIND=DPRECI),DIMENSION(3,4)                        :: IncMass                              !PovMass
    REAL(KIND=DPRECI),DIMENSION(3,4)                        :: PovSha                               !PovMass
    REAL(KIND=KIND_REAL)                                    :: MEDIAN                               !Median income of all households 
    REAL(KIND=KIND_REAL)                                    :: WSUM                                 !Sum of weights
    REAL(KIND=KIND_REAL)                                    :: GINI                                 !GINI
    CHARACTER(len=9)                                        :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=3)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    !Write the result to the log-file
    COUNT_TAB = COUNT_TAB + 1    
    WRITE (NCHAR, '(I3)') COUNT_TAB
    OUTPUTFMT = 'A' // ACHAR(48+LEN(NCHAR(NONBLK(NCHAR):)))
    DO IFILE=15,15
        WRITE(IFILE,'(''TABLE ''' // OUTPUTFMT // ''': NAP Social Inclusion indicators'')')NCHAR(NONBLK(NCHAR):)
        CALL DRAWLINE(IFILE,108,'-')        
        WRITE(IFILE,'(''MAIN ANALYSIS WITH DISPOSABLE INCOME AFTER SOCIAL TRANSFERS (attributed to each household member)'')')        
        WRITE(IFILE,'(''              EQUIVALISED WITH EQUIVALENCE SCALE: '',A100)') LABES_ANAL
    END DO

    !Determine quintile values of whole vector with 
    K = 4
    ALLOCATE(ASOM(1:K+1,1:3))
    ALLOCATE(ASHA(1:K+1,1:3))
    ASOM(1:K+1,1:3) = 0
    ASHA(1:K+1,1:3) = 0
    QUANTNUM(1:VecSize) = 0
    CALL EVAL_QUANTILENR(EVAL_DISPPRE,MIMOSIS_WEIGHT,VecSize,K,QUANTNUM)
    ! START LOOP OVER UNITS OF observation to be analysed (for MIMOSIS normally individuals)
    DO pers_j=1,VecSize
        DO i=1,K+1
            IF (QUANTNUM(pers_j).EQ.i) THEN
                ASOM(i,1) = ASOM(i,1) + MIMOSIS_WEIGHT(pers_j)
                ASOM(i,2) = ASOM(i,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE(pers_j))
                ASOM(i,3) = ASOM(i,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE(pers_j)+EVAL_D1DISP(pers_j))
            END IF
        END DO
    END DO
    !Write results
    WRITE(15,'(/,''SUM PER QUINTILE: RESULTS FOR TOTAL POPULATION'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASOM(i,j),j=1,3)
    END DO
    !Compute shares per quintile
    DO i=1,K+1
        DO j=1,3
            IF (INT(ASOM(1,j)).NE.0) THEN
                ASHA(i,j) = ASOM(i,j)/ASOM(1,j)
            END IF
        END DO            
    END DO
    WRITE(15,'(''SHARES OF FIRST QUINTILE PER QUINTILE: RESULTS FOR TOTAL POPULATION'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASHA(i,j),j=1,3)
    END DO
    DEALLOCATE(ASOM)
    DEALLOCATE(ASHA)

    !Put values in subvector, conditional on region-value
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBANALB(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO pers_j=1,VecSize
        IF (EVAL_REGION(pers_j).EQ.1) THEN
            SUBANALA(COUNT) = EVAL_DISPPRE(pers_j)
            SUBANALB(COUNT) = EVAL_DISPPRE(pers_j)+EVAL_D1DISP(pers_j)
            SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(pers_j)
            COUNT = COUNT + 1
        END IF
    END DO
    COUNT = COUNT - 1
    !Determine quintile values of whole vector with 
    K = 4
    ALLOCATE(ASOM(1:K+1,1:3))
    ALLOCATE(ASHA(1:K+1,1:3))
    ASOM(1:K+1,1:3) = 0
    ASHA(1:K+1,1:3) = 0
    QUANTNUM(1:VecSize) = 0
    CALL EVAL_QUANTILENR(SUBANALA,SUBWEIGH,COUNT,K,QUANTNUM)
    ! START LOOP OVER UNITS OF observation to be analysed (for MIMOSIS normally individuals)
    DO pers_j=1,COUNT
        DO i=1,K+1
            IF (QUANTNUM(pers_j).EQ.i) THEN
                ASOM(i,1) = ASOM(i,1) + SUBWEIGH(pers_j)
                ASOM(i,2) = ASOM(i,2) + SUBWEIGH(pers_j)*(SUBANALA(pers_j))
                ASOM(i,3) = ASOM(i,3) + SUBWEIGH(pers_j)*(SUBANALB(pers_j))
            END IF
        END DO
    END DO
    !Write results
    WRITE(15,'(''SUM PER QUINTILE: RESULTS FOR BRUSSELS'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASOM(i,j),j=1,3)
    END DO
    !Compute shares per quintile
    DO i=1,K+1
        DO j=1,3
            IF (INT(ASOM(1,j)).NE.0) THEN
                ASHA(i,j) = ASOM(i,j)/ASOM(1,j)
            END IF
        END DO            
    END DO
    WRITE(15,'(''SHARES OF FIRST QUINTILE PER QUINTILE: RESULTS FOR BRUSSELS'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASHA(i,j),j=1,3)
    END DO
    DEALLOCATE(ASOM)
    DEALLOCATE(ASHA)

    !Put values in subvector, conditional on region-value
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBANALB(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO pers_j=1,VecSize
        IF (EVAL_REGION(pers_j).EQ.2) THEN
            SUBANALA(COUNT) = EVAL_DISPPRE(pers_j)
            SUBANALB(COUNT) = EVAL_DISPPRE(pers_j)+EVAL_D1DISP(pers_j)
            SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(pers_j)
            COUNT = COUNT + 1
        END IF
    END DO
    COUNT = COUNT - 1
    !Determine quintile values of whole vector with 
    K = 4
    ALLOCATE(ASOM(1:K+1,1:3))
    ALLOCATE(ASHA(1:K+1,1:3))
    ASOM(1:K+1,1:3) = 0
    ASHA(1:K+1,1:3) = 0
    QUANTNUM(1:VecSize) = 0
    CALL EVAL_QUANTILENR(SUBANALA,SUBWEIGH,COUNT,K,QUANTNUM)
    ! START LOOP OVER UNITS OF observation to be analysed (for MIMOSIS normally individuals)
    DO pers_j=1,COUNT
        DO i=1,K+1
            IF (QUANTNUM(pers_j).EQ.i) THEN
                ASOM(i,1) = ASOM(i,1) + SUBWEIGH(pers_j)
                ASOM(i,2) = ASOM(i,2) + SUBWEIGH(pers_j)*(SUBANALA(pers_j))
                ASOM(i,3) = ASOM(i,3) + SUBWEIGH(pers_j)*(SUBANALB(pers_j))
            END IF
        END DO
    END DO
    !Write results
    WRITE(15,'(''SUM PER QUINTILE: RESULTS FOR FLANDERS'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASOM(i,j),j=1,3)
    END DO
    !Compute shares per quintile
    DO i=1,K+1
        DO j=1,3
            IF (INT(ASOM(1,j)).NE.0) THEN
                ASHA(i,j) = ASOM(i,j)/ASOM(1,j)
            END IF
        END DO            
    END DO
    WRITE(15,'(''SHARES OF FIRST QUINTILE PER QUINTILE: RESULTS FOR FLANDERS'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASHA(i,j),j=1,3)
    END DO
    DEALLOCATE(ASOM)
    DEALLOCATE(ASHA)

    !Put values in subvector, conditional on region-value
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBANALB(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO pers_j=1,VecSize
        IF (EVAL_REGION(pers_j).EQ.3) THEN
            SUBANALA(COUNT) = EVAL_DISPPRE(pers_j)
            SUBANALB(COUNT) = EVAL_DISPPRE(pers_j)+EVAL_D1DISP(pers_j)
            SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(pers_j)
            COUNT = COUNT + 1
        END IF
    END DO
    COUNT = COUNT - 1
    !Determine quintile values of whole vector with 
    K = 4
    ALLOCATE(ASOM(1:K+1,1:3))
    ALLOCATE(ASHA(1:K+1,1:3))
    ASOM(1:K+1,1:3) = 0
    ASHA(1:K+1,1:3) = 0
    QUANTNUM(1:VecSize) = 0
    CALL EVAL_QUANTILENR(SUBANALA,SUBWEIGH,COUNT,K,QUANTNUM)
    ! START LOOP OVER UNITS OF observation to be analysed (for MIMOSIS normally individuals)
    DO pers_j=1,COUNT
        DO i=1,K+1
            IF (QUANTNUM(pers_j).EQ.i) THEN
                ASOM(i,1) = ASOM(i,1) + SUBWEIGH(pers_j)
                ASOM(i,2) = ASOM(i,2) + SUBWEIGH(pers_j)*(SUBANALA(pers_j))
                ASOM(i,3) = ASOM(i,3) + SUBWEIGH(pers_j)*(SUBANALB(pers_j))
            END IF
        END DO
    END DO
    !Write results
    WRITE(15,'(''SUM PER QUINTILE: RESULTS FOR WALLONIA'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASOM(i,j),j=1,3)
    END DO
    !Compute shares per quintile
    DO i=1,K+1
        DO j=1,3
            IF (INT(ASOM(1,j)).NE.0) THEN
                ASHA(i,j) = ASOM(i,j)/ASOM(1,j)
            END IF
        END DO            
    END DO
    WRITE(15,'(''SHARES OF FIRST QUINTILE PER QUINTILE: RESULTS FOR WALLONIA'')')
    DO i=1,K+1
        WRITE(15,'(I10,3F20.2)') i,(ASHA(i,j),j=1,3)
    END DO
    DEALLOCATE(ASOM)
    DEALLOCATE(ASHA)

    !Compute median income of all households with a single member
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO hh_i=1,n_hh_eval
        IF (EVAL_HH_SIZE(hh_i).EQ.1) THEN
            SUBANALA(COUNT) = EVAL_DISPPRE(EVAL_START_HH(hh_i))
            SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(EVAL_START_HH(hh_i))
            COUNT = COUNT + 1
        END IF
    END DO
    COUNT = COUNT - 1
    MEDSING = MEDINC(SUBANALA,SUBWEIGH,COUNT)
    WRITE(15,'(''60% OF MEDIAN INCOME OF HOUSEHOLDS WITH SINGLE MEMBER'')')
    WRITE(15,'(F20.2)') 12.D0*(0.6D0*MEDSING)

    !Compute median income of all households with a single adult
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO hh_i=1,n_hh_eval
        IF (EVAL_NADULT(hh_i).EQ.1) THEN
            DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
                SUBANALA(COUNT) = EVAL_DISPPRE(pers_j)
                SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(pers_j)
                COUNT = COUNT + 1
            END DO
        END IF
    END DO
    COUNT = COUNT - 1
    MEDSING = MEDINC(SUBANALA,SUBWEIGH,COUNT)
    WRITE(15,'(''60% OF MEDIAN INCOME OF HOUSEHOLDS WITH SINGLE ADULT'')')
    WRITE(15,'(F20.2)') 12.D0*(0.6D0*MEDSING)

    !Compute poverty indices with equivalised income after all transfers
    Povline(1:4,1:3) = 0
    MEDIAN = MEDINC(EVAL_DISPPRE,MIMOSIS_WEIGHT,VecSize)
    Povline(1,1) = 0.4*MEDIAN
    Povline(2,1) = 0.5*MEDIAN
    Povline(3,1) = 0.6*MEDIAN
    Povline(4,1) = 0.7*MEDIAN
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            IF (EVAL_DISPPRE(pers_j).LT.Povline(1,1)) THEN
                Povline(1,2) = Povline(1,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE(pers_j).LT.Povline(2,1)) THEN
                Povline(2,2) = Povline(2,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE(pers_j).LT.Povline(3,1)) THEN
                Povline(3,2) = Povline(3,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE(pers_j).LT.Povline(4,1)) THEN
                Povline(4,2) = Povline(4,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
        END DO
    END DO
    
    WSUM=SUM(MIMOSIS_WEIGHT(1:VecSize))
    IF (INT(WSUM).NE.0) THEN
        Povline(1:4,3)=100.D0*Povline(1:4,2)/WSUM
    END IF        

    WRITE(15,'(''Poverty lines with equivalised income after all transfers'')')
    WRITE(15,'(3F20.2)') (Povline(1,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(2,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(3,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(4,j),j=1,3)

    !Compute poverty indices with equivalised income before all transfers
    Povline(1:4,1:3) = 0
!    MEDIAN = MEDINC(EVAL_DISPPRE_BTRAN,MIMOSIS_WEIGHT,VecSize)
    Povline(1,1) = 0.4*MEDIAN
    Povline(2,1) = 0.5*MEDIAN
    Povline(3,1) = 0.6*MEDIAN
    Povline(4,1) = 0.7*MEDIAN
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.Povline(1,1)) THEN
                Povline(1,2) = Povline(1,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.Povline(2,1)) THEN
                Povline(2,2) = Povline(2,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.Povline(3,1)) THEN
                Povline(3,2) = Povline(3,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.Povline(4,1)) THEN
                Povline(4,2) = Povline(4,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
        END DO
    END DO
    
    IF (INT(WSUM).NE.0) THEN
        Povline(1:4,3)=100.D0*Povline(1:4,2)/WSUM
    END IF        

    WRITE(15,'(''Poverty lines with equivalised income before all transfers'')')
    WRITE(15,'(3F20.2)') (Povline(1,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(2,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(3,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(4,j),j=1,3)

    !Compute poverty indices with equivalised income after pension transfers before all other transfers
    Povline(1:4,1:3) = 0
!    MEDIAN = MEDINC(EVAL_DISPPRE_NPENS,MIMOSIS_WEIGHT,VecSize)
    Povline(1,1) = 0.4*MEDIAN
    Povline(2,1) = 0.5*MEDIAN
    Povline(3,1) = 0.6*MEDIAN
    Povline(4,1) = 0.7*MEDIAN
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.Povline(1,1)) THEN
                Povline(1,2) = Povline(1,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.Povline(2,1)) THEN
                Povline(2,2) = Povline(2,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.Povline(3,1)) THEN
                Povline(3,2) = Povline(3,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.Povline(4,1)) THEN
                Povline(4,2) = Povline(4,2) + MIMOSIS_WEIGHT(pers_j)  
            END IF
        END DO
    END DO
    
    IF (INT(WSUM).NE.0) THEN
        Povline(1:4,3)=100.D0*Povline(1:4,2)/WSUM
    END IF        

    WRITE(15,'(''Poverty lines with equivalised income after pension transfers before all other transfers'')')
    WRITE(15,'(3F20.2)') (Povline(1,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(2,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(3,j),j=1,3)
    WRITE(15,'(3F20.2)') (Povline(4,j),j=1,3)

    !Select group of people below the normal porverty line, i.e. 60% of equivalised income after all transfers
    MEDIAN = MEDINC(EVAL_DISPPRE,MIMOSIS_WEIGHT,VecSize)
    COUNT = 1
    SUBANALA(1:VecSize) = 0
    SUBWEIGH(1:VecSize) = 0
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            IF (EVAL_DISPPRE(pers_j).LT.(0.6D0*MEDIAN)) THEN
                SUBANALA(COUNT) = EVAL_DISPPRE(pers_j)
                SUBWEIGH(COUNT) = MIMOSIS_WEIGHT(pers_j)
                COUNT = COUNT + 1
            END IF
        END DO
    END DO
    COUNT = COUNT - 1
    MEDPOV = MEDINC(SUBANALA,SUBWEIGH,COUNT)
    WRITE(15,'(''60% OF MEDIAN INCOME                                                   '', F20.2)') 0.6D0*MEDIAN
    WRITE(15,'(''MEDIAN INCOME OF PEOPLE BELOW THE POVERTY LINE                         '', F20.2)') MEDPOV
    IF (INT(MEDIAN).NE.0) THEN
        WRITE(15,'(''MEDIAN INCOME OF PEOPLE BELOW THE POVERTY LINE AS % OF POVERTY LINE  '', F20.2)') &
        &100.D0*(0.6D0*MEDIAN-MEDPOV)/(0.6D0*MEDIAN)
    END IF        

    GINI=GENGINI(EVAL_DISPPRE,MIMOSIS_WEIGHT,Vecsize,2.D0,0)
    WRITE(15,'(''GINI OF EQUIVALENT INCOME OF TOTAL POPULATION '', F20.2)') 100.D0*GINI

    !Compute income mass below the poverty line in baseline 
    PovMass(1:3,1:4) = 0
    IncMass(1:3,1:4) = 0
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !compute income mass below the poverty line
            IF (EVAL_DISPPRE(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(1,1) = PovMass(1,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(1,2) = PovMass(1,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(1,3) = PovMass(1,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE(pers_j))
                END IF
            END IF
            IF (EVAL_DISPPRE_BTRAN(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(2,1) = PovMass(2,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_BTRAN(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(2,2) = PovMass(2,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_BTRAN(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(2,3) = PovMass(2,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_BTRAN(pers_j))
                END IF
            END IF
            IF (EVAL_DISPPRE_NPENS(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(3,1) = PovMass(3,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_NPENS(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(3,2) = PovMass(3,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_NPENS(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(3,3) = PovMass(3,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPRE_NPENS(pers_j))
                END IF
            END IF
            !Compute total income mass
            IF (EVAL_REGION(pers_j).EQ.1) THEN
                IncMass(1,1) = IncMass(1,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE(pers_j))
                IncMass(2,1) = IncMass(2,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_BTRAN(pers_j))
                IncMass(3,1) = IncMass(3,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_NPENS(pers_j))
            ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                IncMass(1,2) = IncMass(1,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE(pers_j))
                IncMass(2,2) = IncMass(2,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_BTRAN(pers_j))
                IncMass(3,2) = IncMass(3,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_NPENS(pers_j))
            ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                IncMass(1,3) = IncMass(1,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE(pers_j))
                IncMass(2,3) = IncMass(2,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_BTRAN(pers_j))
                IncMass(3,3) = IncMass(3,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPRE_NPENS(pers_j))
            END IF
        END DO
    END DO
    DO i=1,3
        PovMass(i,4) = SUM(PovMass(i,1:3))
        IncMass(i,4) = SUM(IncMass(i,1:3))
    END DO

    PovSha(1:3,1:4) = 0
    DO i=1,3
        DO j=1,4
            IF (IncMass(i,j).NE.0) THEN
                PovSha(i,j) = PovMass(i,j)/IncMass(i,j)  
            END IF
        END DO
    END DO        
    
    WRITE(15,'(''Poverty mass baseline'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (PovMass(i,j),j=1,4)
    END DO
    WRITE(15,'(''Income mass baseline'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (IncMass(i,j),j=1,4)
    END DO
    WRITE(15,'(''Share baseline'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (100.D0*PovSha(i,j),j=1,4)
    END DO
    
    !Compute income mass below the poverty line after reform
    PovMass(1:3,1:4) = 0
    IncMass(1:3,1:4) = 0
    DO hh_i=1,n_hh_eval
        DO pers_j=EVAL_START_HH(hh_i),EVAL_END_HH(hh_i)
            !compute income mass below the poverty line
            IF (EVAL_DISPPOST(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(1,1) = PovMass(1,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(1,2) = PovMass(1,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(1,3) = PovMass(1,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST(pers_j))
                END IF
            END IF
            IF (EVAL_DISPPOST_BTRAN(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(2,1) = PovMass(2,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_BTRAN(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(2,2) = PovMass(2,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_BTRAN(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(2,3) = PovMass(2,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_BTRAN(pers_j))
                END IF
            END IF
            IF (EVAL_DISPPOST_NPENS(pers_j).LT.(0.6D0*MEDIAN)) THEN
                IF (EVAL_REGION(pers_j).EQ.1) THEN
                    PovMass(3,1) = PovMass(3,1) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_NPENS(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                    PovMass(3,2) = PovMass(3,2) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_NPENS(pers_j))
                ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                    PovMass(3,3) = PovMass(3,3) + MIMOSIS_WEIGHT(pers_j)*(0.6D0*MEDIAN-EVAL_DISPPOST_NPENS(pers_j))
                END IF
            END IF
            !Compute total income mass
            IF (EVAL_REGION(pers_j).EQ.1) THEN
                IncMass(1,1) = IncMass(1,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST(pers_j))
                IncMass(2,1) = IncMass(2,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_BTRAN(pers_j))
                IncMass(3,1) = IncMass(3,1) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_NPENS(pers_j))
            ELSE IF (EVAL_REGION(pers_j).EQ.2) THEN 
                IncMass(1,2) = IncMass(1,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST(pers_j))
                IncMass(2,2) = IncMass(2,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_BTRAN(pers_j))
                IncMass(3,2) = IncMass(3,2) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_NPENS(pers_j))
            ELSE IF (EVAL_REGION(pers_j).EQ.3) THEN 
                IncMass(1,3) = IncMass(1,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST(pers_j))
                IncMass(2,3) = IncMass(2,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_BTRAN(pers_j))
                IncMass(3,3) = IncMass(3,3) + MIMOSIS_WEIGHT(pers_j)*(EVAL_DISPPOST_NPENS(pers_j))
            END IF
        END DO
    END DO
    DO i=1,3
        PovMass(i,4) = SUM(PovMass(i,1:3))
        IncMass(i,4) = SUM(IncMass(i,1:3))
    END DO

    PovSha(1:3,1:4) = 0
    DO i=1,3
        DO j=1,4
            IF (IncMass(i,j).NE.0) THEN
                PovSha(i,j) = PovMass(i,j)/IncMass(i,j)  
            END IF
        END DO
    END DO        
    
    WRITE(15,'(''Poverty mass after reform'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (PovMass(i,j),j=1,4)
    END DO
    WRITE(15,'(''Income mass after reform'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (IncMass(i,j),j=1,4)
    END DO
    WRITE(15,'(''Share after reform'')')
    WRITE(15,'(''            Brussels            Flanders            Wallonia               Total'')')
    DO i=1,3
        WRITE(15,'(4F20.2)') (100.D0*PovSha(i,j),j=1,4)
    END DO
    
END SUBROUTINE EVAL_NAPSI

!=================================================================================!
!Routine to write EVAL variables for the computation of additional poverty measures
!=================================================================================!

SUBROUTINE WRITEIND_EVAL

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\EVAL_INDIVINF.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,9F15.2)') GECODEERD_INSZ_CHAR(pers_j),EVAL_DISPPRE(pers_j),EVAL_DISPPOST(pers_j),EVAL_D1DISP(pers_j),&
        &EVAL_DISPPRE_BTRAN(pers_j),EVAL_DISPPOST_BTRAN(pers_j),EVAL_D1DISP_BTRAN(pers_j),&
        &EVAL_DISPPRE_NPENS(pers_j),EVAL_DISPPOST_NPENS(pers_j),EVAL_D1DISP_NPENS(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEIND_EVAL..............................runned correctly'

END SUBROUTINE WRITEIND_EVAL

!=================================================================================!
!Routine to write EVAL variables for the computation of additional measures
!=================================================================================!

SUBROUTINE WRITEDEC_EVAL

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\EVAL_DECNR.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,I10)') GECODEERD_INSZ_CHAR(pers_j),EVAL_DECNR(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEIND_EVAL..............................runned correctly'

END SUBROUTINE WRITEDEC_EVAL

!====================================!
!Start calculation
!====================================!

SUBROUTINE EVAL_CALC

    !Program heading
    !=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS EVAL IS RUNNING '
    WRITE (*,*) ' '
    !=============================================================================
    !Read location parameters including value n_rec
    CALL EVAL_LOCATION_FILEREAD
    !open log file
    OPEN(unit=15,FILE=EVAL_LOG,POSITION='APPEND',ACTION='WRITE')
    !Allocate all allocatable variables with dimension n_rec
    CALL EVAL_ALLOC(n_rec)
    !Initialise endogenous variables
    CALL EVAL_INITENDO(n_rec)
    !Determine the file names of the pre and post reform CONTRIB and PIT files    
    CALL SIMUL_LOCATION_READ(MIMOSIS_PARLPRE,' of PRE REFORM')
    CALL MODOUTP_LOCATION_READ(MIMOSIS_FAMREL_PRE,WAGE_CONSTRUCT_PRE,WORKHOUR_PRE,EVAL_SOCBEN_PRE,EVAL_CONTRIB_PRE,&
    &EVAL_PIT_PRE,GLOBBUD_EMPL_PRE,GLOBBUD_SELFEMPL_PRE,' of PRE REFORM')
    CALL SIMUL_LOCATION_READ(MIMOSIS_PARLPOST,'of POST REFORM')    
    CALL MODOUTP_LOCATION_READ(MIMOSIS_FAMREL_POST,WAGE_CONSTRUCT_POST,WORKHOUR_POST,EVAL_SOCBEN_POST,EVAL_CONTRIB_POST,&
    &EVAL_PIT_POST,GLOBBUD_EMPL_POST,GLOBBUD_SELFEMPL_POST,'of POST REFORM')    
    !Read PRE reform input files
    CALL EVAL_PIT_PRE_FILEREAD
    CALL EVAL_CONTRIB_PRE_FILEREAD
    CALL EVAL_SOCBEN_PRE_FILEREAD
    CALL EVAL_WORKHOUR_PRE_FILEREAD
    CALL EVAL_FAMREL_PRE_FILEREAD
    CALL EVAL_WAGE_PRE_FILEREAD
    !Read POST reform input files
    CALL EVAL_PIT_POST_FILEREAD
    CALL EVAL_CONTRIB_POST_FILEREAD
    CALL EVAL_SOCBEN_POST_FILEREAD
    CALL EVAL_WORKHOUR_POST_FILEREAD
    !Read input files that are independent of a simulation run
    CALL EVAL_WEIGHT_FILEREAD
    CALL EVAL_INPUT_FILEREAD
    !Read in parameters
    CALL EVAL_PARAMETER_FILEREAD

    !Rank and Sort all exogenous variables on their full size
    CALL EVAL_RANKANDSORT(n_rec)   
    !Recode values
    CALL EVAL_RECODE(n_rec)
    !Inflate values
    CALL EVAL_INFLATE(n_rec)

    !Determine household size and number of households
    CALL EVAL_HOUSEHOLD_SIZE(n_rec)

    !Create the age of each household member
    CALL EVAL_AGE(n_rec)

    !Define economic independent units
    CALL EVAL_ECONINDI_POTENT
    CALL EVAL_DISPLAB_CREATE
    CALL EVAL_ECONINDI_FINAL

    !EVALUATION OF THE REVENUE EFFECTS OF THE REFORM
        !start with page break
        WRITE(15,'(A1)') ACHAR(12)
        CALL DRAWLINE(15,133,'=')
        WRITE(15,'(''START ANALYSIS OF AGGREGATE BUDGET FIGURES'')')
        CALL DRAWLINE(15,133,'=')    
        WRITE(15,'('' '')')
        !Compute the global budget results
        COUNT_TAB = 1
!        CALL EVAL_GLOBBUD
        !Call eval_revcalc
        COUNT_TAB = COUNT_TAB + 1
        CALL EVAL_REVCALC

    !EVALUATION OF THE DISTIRBUTIONAL EFFECTS OF THE REFORM
        !Exclude elements of subgroups
        !Create subgroup to exclude household with insufficient income information
        IF (EVAL_SUBSET.EQ.1) THEN
            CALL EVAL_SUBGROUP_DEFINITION
        END IF    
        CALL EVAL_SUBGROUP_SIZE    
        CALL EVAL_RANKANDSORT_SUB(n_rec,n_rec_sub)       
        CALL EVAL_HOUSEHOLD_SIZE(n_rec_sub)    

        !Determine the number of adults and children in each household
        DO hh_i=1,n_hh_eval
            CALL EVAL_CREATE_ADULCHIL(hh_i)
        END DO

        !Create the pre and post reform variables that are going to be analysed    
        CALL EVAL_PREPOST_CREATE(n_rec_sub)    

        !Write start of analysis
        WRITE(15,'(A1)') ACHAR(12)
        CALL DRAWLINE(15,100,'=')
        WRITE(15,'(''START DISTRIBUTIONAL ANALYSIS'')')
        CALL DRAWLINE(15,100,'=')    
        WRITE(15,'('' '')')

        !Call general description of the analysis that is carried out
        CALL EVAL_WRITEANA
        !Start specific analysis by variables of analysis
        !ANALYSIS BY DECILE
        CHARNAME(1)='DECILE OF WELFARE CONCEPT'
        !calculate the decilenumbers for EVAL_WELFPRE by calling subroutine EVAL_QUANTILENR, which gives the decilenumbers in decnr(VecSize)
        !the vectors EVAL_WELFPRE and MIMOSIS_WEIGHT are not changed by the call (it means: remain unordered)
        CALL EVAL_QUANTILENR(EVAL_WELFPRE,MIMOSIS_WEIGHT,n_rec_sub,9,EVAL_DECNR)
        CALL EVAL_ANALCHAR(CHARNAME(1),EVAL_DECNR,n_rec_sub,10,EVAL_DISPPRE_BTRAN,EVAL_D1DISP_BTRAN,EVAL_DISPPRE,EVAL_D1DISP,&
        &MIMOSIS_WEIGHT,EVAL_PRECIS)
        !ANALYSIS BY AGE CLASS
        CHARNAME(2)='AGE CLASS'
        CALL EVAL_AGECLASS_CREATION(n_rec_sub)
        CALL EVAL_ANALCHAR(CHARNAME(2),EVAL_AGECLASS,n_rec_sub,20,EVAL_DISPPRE_BTRAN,EVAL_D1DISP_BTRAN,EVAL_DISPPRE,EVAL_D1DISP,&
        &MIMOSIS_WEIGHT,EVAL_PRECIS)
        !ANALYSIS BY REGION
        CHARNAME(3)='REGION'
        CALL EVAL_REGION_CREATION(n_rec_sub)
        CALL EVAL_ANALCHAR(CHARNAME(3),EVAL_REGION,n_rec_sub,3,EVAL_DISPPRE_BTRAN,EVAL_D1DISP_BTRAN,EVAL_DISPPRE,EVAL_D1DISP,&
        &MIMOSIS_WEIGHT,EVAL_PRECIS)

    !CREATE SUMMARY TABLE WITH EFFECTS OF THE SIMULATION
        WRITE(15,'(A1)') ACHAR(12)
        CALL DRAWLINE(15,100,'=')
        WRITE(15,'(''START SUMMARY OF THE REFORM'')')
        CALL DRAWLINE(15,100,'=')    
        WRITE(15,'('' '')')
        !Call summary analysis
        CALL EVAL_SUMMARY(n_rec_sub)

    !CREATE SUMMARY TABLE WITH NAP_SI indicators
        WRITE(15,'(A1)') ACHAR(12)
        CALL DRAWLINE(15,100,'=')
        WRITE(15,'(''TABLE WITH NAP_SI INDICATORS'')')
        CALL DRAWLINE(15,100,'=')    
        WRITE(15,'('' '')')
        !Call summary analysis
!        CALL EVAL_NAPSI(n_rec_sub)

!    CALL WRITEIND_EVAL
!    CALL WRITEDEC_EVAL

    !Deallocate all allocatable variables, i.e. including allocated parameters
    CALL EVAL_DEALLOC
    !Close Log-file
    CLOSE(15)

END SUBROUTINE EVAL_CALC

END MODULE EVAL_SUBROUTINES 
