!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the program    !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE FAMAL_PRERUN_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=131                                            !
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008                                    !
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !========================================================================!

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: FAMAL_INPUT_PATH                     !Default path for all input files
    CHARACTER(len=150)                                      :: FAMAL_INPUT_FILE                     !Default name of input file (i.e. INPUT from NATIONAL REGISTER)
    CHARACTER(len=150)                                      :: FAMAL_INPUT                          !Path+Name of INPUT file
    CHARACTER(len=150)                                      :: FAMAL_SICK_FILE                      !Default name of SICK file (i.e. the OUTPUT FILE of the SICK module)
    CHARACTER(len=150)                                      :: FAMAL_SICK                           !Path+Name of SICK file
    CHARACTER(len=150)                                      :: FAMAL_UNEM_FILE                      !Default name of UNEM file (i.e. the OUTPUT FILE of the UNEM module)
    CHARACTER(len=150)                                      :: FAMAL_UNEM                           !Path+Name of UNEM file
    CHARACTER(len=150)                                      :: FAMAL_CONTRIB_FILE                   !Default name of CONTRIB file (i.e. the OUTPUT FILE of the CONTRIB module)
    CHARACTER(len=150)                                      :: FAMAL_CONTRIB                        !Path+Name of CONTRIB file
    CHARACTER(len=150)                                      :: FAMAL_PENSWELF_FILE                  !Default name of PENSWELF file (i.e. the OUTPUT FILE of the PENSWELF module)
    CHARACTER(len=150)                                      :: FAMAL_PENSWELF                       !Path+Name of PENSWELF file
    CHARACTER(len=150)                                      :: FAMAL_SOCBEN_FILE                    !Default name of SOCBEN file (i.e. the OUTPUT FILE of the SOCBEN module)
    CHARACTER(len=150)                                      :: FAMAL_SOCBEN                         !Path+Name of SOCBEN file
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: FAMAL_2008_FILE                      !File name of file with FAMAL reference values
    CHARACTER(len=150)                                      :: FAMAL_2008                           !Internal name to store Path+Name of file with FAMAL reference values
    !Variables for locating output (local variables)
    CHARACTER(len=150)                                      :: CONTRIB_FAMAL_FILE                   !Name of the pre run file to be created for the CONTRIB module
    CHARACTER(len=150)                                      :: CONTRIB_FAMAL                        !Path+Name of output for CONTRIB module   
    CHARACTER(len=150)                                      :: SICK_FAMAL_FILE                      !Name of the pre run file to be created for the SICK module
    CHARACTER(len=150)                                      :: SICK_FAMAL                           !Path+Name of output for SICK module
    CHARACTER(len=150)                                      :: UNEM_FAMAL_FILE                      !Name of the pre run file to be created for the UNEM module 
    CHARACTER(len=150)                                      :: UNEM_FAMAL                           !Path+Name of output for UNEM module 
    CHARACTER(len=150)                                      :: SOCBEN_FAMAL_FILE                    !Name of the pre run file to be created for the SOCBEN module 
    CHARACTER(len=150)                                      :: SOCBEN_FAMAL                         !Path+Name of output for SOCBEN module 
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: FAMAL_PAR_FORMAT                     !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: FAMAL_PAR_BREAK='@'                  !Character used to identify end of string
    CHARACTER(len=10)                                       :: FAMAL_PAR_CURR                       !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: FAMAL_PAR_NAMES                      !Vector of parameter names
    INTEGER                                                 :: FAMAL_PAR_L                          !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: FAMAL_PAR_C                          !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(18)                                   :: FAMAL_PAR_BREAK_POINT                !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the FAMAL module
    !Identification variables necessary to compute family allowances             
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: FAMAL_FAMREL_QT                      !Matrix used to convert relationships of family members
    REAL(KIND=KIND_REAL)                                    :: FAMAL_CHILUNEL_MONTH                 !Income ceiling, expressed in Euro per month, applied on unemployment benefits to see whether the child is still eligible for child benefits         
    REAL(KIND=KIND_REAL)                                    :: FAMAL_CHILINCL_MONTH                 !Income ceiling, expressed in Euro per month, applied on labour market income to see whether the child is still eligible for child benefits          
    REAL(KIND=KIND_REAL)                                    :: FAMAL_GUARINPC_QT                    !Percentage used to increase the income ceiling that is applied to test whether a beneficiary is eligible for guaranteed child allowances            
    REAL(KIND=KIND_REAL)                                    :: FAMAL_GUARINAB_QT                    !Basic income ceiling that is applied to test whether a beneficiary is eligible for guaranteed child allowances          
    !Computation of family allowances: basic allowances              
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_BALLAMOU_ANN                   !Lump sum amounts, differentiated according to birth rank of the child, granted as birth allowances          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ADOPAMOU_ANN                   !Lump sum amount granted as adoption allowance           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_OWEAAMOU_MONTH                 !Monthly amount, paid as child benefit if the beneficiary opened rights in the wage earner contribution scheme (amounts differ with the rank of the eligible child)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_OCIVAMOU_MONTH                 !Monthly amount, paid as child benefit if the beneficiary opened rights in the civil servant contribution scheme (amounts differ with the rank of the eligible child)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_OSELAMOU_MONTH                 !Monthly amount, paid as child benefit if the beneficiary opened rights in the self employed contribution scheme (amounts differ with the rank of the eligible child)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_OGUAAMOU_MONTH                 !Monthly amount, paid as child benefit if the beneficiary opened rights in the guaranteed child allowance scheme (amounts differ with the rank of the eligible child)            
    REAL(KIND=KIND_REAL)                                    :: FAMAL_OORPAMOU_MONTH                 !Monthly amount, paid as child benefit, if the beneficiary child is an orphan            
    !Computation of family allowances: supplements               
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: FAMAL_ASWEAR1_MONTH                  !Monthly amount, used to determine age supplements of children of rank 1         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: FAMAL_ASWEAR23_MONTH                 !Monthly amount, used to determine age supplements of children of rank 2 and 3           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_REPLEVEL_MONTH                 !Income ceiling, expressed in Euro per month, used to test whether the beneficiary lives in a household that satisfies the means test for social supplements         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SOCSUP1_MONTH                  !Monthly supplements granted to beneficiaries that are unemployed for more than 6 months or beneficiaries receiving pension benefits other than self employment pension benefits             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SOCSUP2_MONTH                  !Monthly supplements granted to beneficiaries that receive self employment pension benefits          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SOCSUP3_MONTH                  !Monthly supplements granted to beneficiaries who are disabled           
    !Extra for computation of family allowances of self employed               
    INTEGER                                                 :: FAMAL_ASSELFFS_ANN                   !Apply also the age supplement for the first or single child of self employed
    !Extra for computation of family allowances since 2006             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: FAMAL_SCHOOLPR_MONTH                 !Monthly supplement paid because of schooling cost
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SINGHHSP_MONTH                 !Monthly supplement for child benefits of single households
    REAL(KIND=KIND_REAL)                                    :: FAMAL_INCLIMSP_MONTH                 !Income ceiling used to test whether the single household is eligible for a child benefit supplement
    !Parameters for RKW simulations
    INTEGER                                                 :: FAMAL_SIMSURKW_QT                    !Start RKW simulations
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SUPRKW_MONTH                   !Monthly supplements granted for RKW simulation
    !Parameters for Handicap
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_VKBOLD_MONTH                   !Monthly supplements granted for children with disability (old system, law 1991)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_VKBNEW_MONTH                   !Monthly supplements granted for children with disability (new system, law 2003)
    !=======================================================================!  
    !Exogenous variables
    !FAMAL_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_CIVIL                       ! Burgerlijke staat van het individu op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    !/*RKW*/
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: gecodeerd_insz_rh                    ! Gecodeerd_insz rechthebbende
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: gecodeerd_insz_bt                    ! Gecodeerd_insz bijslagtrekkende
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRKW                                ! Individu geselecteerd uit basisdata RKW voor verwerking steekproef
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: BEGINRKW_2008                        ! Begindatum betaling kinderbijslag voor kind in een ander stelsel dan dat van de zelfstandigen (2008)
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: EINDRKW_2008                         ! Einddatum betaling kinderbijslag voor kind in een ander stelsel dan dat van de zelfstandigen (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RKW_GWB_2008                         ! Individu ontvangt kinderbijslag in stelsel van gewaarborgde kinderbijslag (2008)
    !/*RSVZ*/    
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: gecodeerd_insz_rh_rsvz               ! Gecodeerd_insz rechthebbende
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: gecodeerd_insz_bt_rsvz               ! Gecodeerd_insz bijslagtrekkende
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRSVZ_KB                            ! Individu geselecteerd uit basisdata RSVZ_KB voor verwerking steekproef
    CHARACTER(len=4),ALLOCATABLE,DIMENSION(:)               :: QUART_RSVZKB_2008                    ! Aanduiding van de kwartalen van het jaar 2008 waarin de gegeven combinatie geobserveerd wordt in RSVZ kinderbijslagbestanden
    !/*FOD SZ*/
    CHARACTER(len=2),ALLOCATABLE,DIMENSION(:)               :: code_leg                             ! Code wetgeving                                                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: pilier_un_conv                       ! pijler 1 omgezet voor VKB nieuwe wet                                                     
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: pilier_deux                          ! pijler 2                                                                                 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: pilier_trois                         ! pijler 3                                                                                 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PUNT_ZELFVKB                         ! Geschat aantal punten zelfredzaamheid voor bepaling VKB oude wet                         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InFODSZ                              ! Individu geselecteerd uit n van de drie basisbestanden FODSZ voor verwerking steekproef
    !/*HistorInfo*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PRIV                        ! Meest voorkomende arbeidsmarktstatus op de private arbeidsmarkt in de periode 1997 tot en met eindjaar 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: LABMSTAT_PUB                         ! Meest voorkomende arbeidsmarktstatus op de publieke arbeidsmarkt in de periode 1997 tot en met eindjaar
    !Variables read from file FAMAL_CONTRIB.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PRIV_QT             !Labour market status of a wage earner on the private labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PUB_QT              !Labour market status of a wage earner on the public labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_SELF_QT             !Labour market status of a self employed         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT               !Employee social security contributions paid for employee in the private labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT                !Employee social security contributions paid for employee in the public labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT               !Social security contributions net of deductions paid by self employed           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPRIV_QT                   !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the private labour market          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPUB_QT                    !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSELF_QT                   !Labour income, earned as self employed          
    !Variables read from file FAMAL_UNEM.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_TOTBEN_QT                       !Gross amount of unemployment benefits           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FULL6MON_QT                     !The number of months in unemployment if the unemployed is either full time unemployed or early retired (hence eligible for social supplement in child benefits)
    !Variables read from file FAMAL_SICK.txt     
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_PRIV_QT                !Insurance type of the sick or disabled in the wage earner regime on the private labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_PUB_QT                 !Insurance type of the sick or disabled in the wage earner regime on the public labour market            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_TYPEINSU_SELF_QT                !Insurance type of the sick or disabled as self employed         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_TOTBEN_QT                       !Sum of benefits for sickness, disability, industrial accidents and occupational diseases            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: SICK_DISABBEN_QT                     !Identification of the fact that the beneficiary receives disability benefits            
    !Variables read from file FAMAL_PENSWELF.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PENSWELF              !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PENSWELF_BENSCHEM_QT                 !Identification of the contribution scheme that generated the registered pension benefits.           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_AMOUNTP_QT                  !The gross amount of all pension benefits, paid because of legal Belgian pension laws            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_SURPENS_QT                  !The gross amount of all survival pension benefits, paid because of legal Belgian pension laws           
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !Variables read from file FAMAL_SOCBEN.txt                                                    
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SOCBEN                !Personal identification number in the file with minimex values
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SOCBEN_MINIMEX_QT                    !minimex received by individual
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_YEAR_AGE                       !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_MONTH_AGE                      !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_MONTH_BIRTH                    !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_YEAR_BIRTH                     !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_HH_SIZE                        !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_START_HH                       !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_END_HH                         !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_FAMAL                           !number of households analysed
    !To construct variables listed in the module note
    !Identification variables necessary to compute family allowances             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENEFID_QT                     !Individual identification number of the beneficiary of child benefits of the beneficiary child          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_RECIPID_QT                     !Individual identification number of the recipient of child benefits of the beneficiary child            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENSCHEM_QT                    !Contribution scheme of beneficiary          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_HANDICAP_ANN                   !The individual is handicapped (yes=1, no=0)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_LABMINC_QT                     !Sum of all gross labour market income sources obtained by the individual            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_STUDENT_ANN                    !The individual is a student (yes=1, no=0)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_STARTPAY_ANN                   !First month of the year for which child receives child benefits (number from 1 to 12)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_ENDPAY_ANN                     !Last month of the year for which child receives child benefits (number from 1 to 12)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_CHILELIG_QT                    !Individual is eligible for child benefits (yes=1, no=0)         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_INDINC_QT                      !Sum of all individual income components (including labour market income and replacement income but no social benefits)          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_NUMDEPCH_QT                    !Number of dependent children of the beneficiary         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENGUARI_QT                    !The beneficiary satisfies the conditions to receive guaranteed child allowances (yes=1, no=0)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_NMONTHBE_QT                    !Number of months for which benefits are paid during the quarter of analysis for the eligible child          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_WIDOWER_QT                     !Individual is considered to be a widow or widower (yes=1, no=0)         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_ORPHAN_QT                      !Individual is considered to be an orphan       
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BIRTRANK_QT                    !Birth rank of the eligible child            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_CBENRANK_QT                    !Rank of the eligible child to determine the ordinary child benefits         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_SINGCHIL_QT                    !The eligible child is the only eligible child in the sociological household         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_YOUNGEST_QT                    !The eligible child is the youngest child of all eligible children in the sociological household         
    !Computation of family allowances: basic allowances              
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_BIRTHALL_QT                    !Amount of birth allowances in Euro paid for an eligible child during the quarter of analysis            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ADOPALL_QT                     !Amount of adoption allowances in Euro paid for an eligible child during the quarter of analysis         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ORALLWEA_QT                    !Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the wage earner scheme            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ORALLCIV_QT                    !Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the civil servant scheme          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ORALLSEL_QT                    !Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the self employment scheme            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ORALLGUA_QT                    !Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the guaranteed child allowance scheme         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ORALLORP_QT                    !Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary child is an orphan         
    !Computation of family allowances: supplements             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ASUPWEA_QT                     !Amount of age supplements in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in another than the self employed contribution scheme          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_ASUPSEL_QT                     !Amount of age supplements in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the self employed contribution scheme           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_REPIBEN_QT                     !The beneficiary can be granted a social supplement because he receives on of the specified replacement  incomes         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_REPIREC_QT                     !The recipient can be granted a social supplement because he receives on of the specified replacement  incomes         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENIMEAN_QT                    !The beneficiary can be granted a social supplement because he lives in a household that satisfies the means test, applied when social supplements are granted           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_REPIMEAN_QT                    !The recipient can be granted a social supplement because he lives in a household that satisfies the means test, applied when social supplements are granted           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SOCSAMOU_QT                    !Amount of social supplements in Euro paid for an eligible child during the quarter of analysis if the beneficiary if the beneficiary receives some predefined type of replacement income and satisfies the necessary means test         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SUPLSING_QT                    !Child benefit supplement paid because the child is living with a single parent
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_DISA66PL_QT                    !The individual is disabled for more than 66%(yes=1, no=0)           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_HANDSCAL_QT                    !The individual is not self sustainable (scale from 0 to 9)          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_HANDAMOU_QT                    !Amount of supplements in Euro paid for an eligible child during the quarter of analysis if the eligible child is disabled for more than 66%         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_SCHOOL_QT                      !Amount of supplement paid per year for school expenses
    !Variables for computation of RKW simulations
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: FAMAL_MEANSRKW_QT                    !The recipient can be granted a social supplement because he satisfies the means imposed for additional RKW simulations
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_RKWSUPL_QT                     !Supplements for simulations of RKW

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RH_RKW_INPUT                         !Numeric part of the anonymised identification number of the individual that grants the right on child benefits (the qualifying child) in the system of salaried workers         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: BT_RKW_INPUT                         !Numeric part of the anonymised identification number of the individual that receives the child benefits (the beneficiary) in the system of salaried workers         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: RH_RSVZ_INPUT                        !Numeric part of the anonymised identification number of the individual that grants the right on child benefits (the qualifying child) in the system of self employed people         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: BT_RSVZ_INPUT                        !Numeric part of the anonymised identification number of the individual that receives the child benefits (the beneficiary) in the system of self employed people         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: HHMEMB_QT                            !Variable used to identify whether an identified individual is part of the household or not
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: INDISBEN_QT                          !Identifier saying that the individual is beneficiary in the given quarter (yes=1)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: INDISREC_QT                          !Identifier saying that the individual is recipient in the given quarter (yes=1)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENSCHIL_QT                    !Variable containing the contribution scheme of the beneficiary on the level of the beneficiary child
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_NEWBORN_QT                     !Indicator saying that the child is newborn child (yes=1, no=0)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_NUMRECIP_QT                    !Number of dependent children of the recipient
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_NUMNEWB_QT                     !Number of newborn children, depending on the recipient, and eligible for child benefits
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_ELIGCHHH_QT                    !Variable containing the sum of all eligible children within the household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENSCMAX_QT                    !Variable containing the contribution scheme that is observed most frequently within the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_AGECLASS                       !Individuals classified according to their age
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_BENSCHRP_QT                    !Variable containing the scheme of the beneficiary that produces a social supplement on the level of the beneficiary child
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_RECICHRP_QT                    !Variable containing the scheme of the recipient that produces a social supplement on the level of the beneficiary child
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMAL_RHBTKNOW                       !Identifier saying that beneficiary or right giving person is known

    !Constructed exchange variables
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_AMOUNTF_QT                     !Total amount of child allowances and supplements received by recipient
    
END MODULE FAMAL_PRERUN_DATA

!==========================================!
! This module contains all the subroutines !
! and functions used in the FAMAL MODULE   !
!==========================================!
!List of subroutine names

MODULE FAMAL_PRERUN_SUBROUTINES

    USE FAMAL_PRERUN_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC!
!=====================================================!

SUBROUTINE FAMAL_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !FAMAL_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    ALLOCATE(NAREGNIS_CIVIL(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    !/*RKW*/
    ALLOCATE(gecodeerd_insz_rh(DIM_ALLOC))
    ALLOCATE(gecodeerd_insz_bt(DIM_ALLOC))
    ALLOCATE(InRKW(DIM_ALLOC))
    ALLOCATE(BEGINRKW_2008(DIM_ALLOC))
    ALLOCATE(EINDRKW_2008(DIM_ALLOC))
    ALLOCATE(RKW_GWB_2008(DIM_ALLOC))
    !/*RSVZ*/    
    ALLOCATE(gecodeerd_insz_rh_rsvz(DIM_ALLOC))
    ALLOCATE(gecodeerd_insz_bt_rsvz(DIM_ALLOC))
    ALLOCATE(InRSVZ_KB(DIM_ALLOC))
    ALLOCATE(QUART_RSVZKB_2008(DIM_ALLOC))
    !/*FOD SZ*/
    ALLOCATE(code_leg(DIM_ALLOC))
    ALLOCATE(pilier_un_conv(DIM_ALLOC))
    ALLOCATE(pilier_deux(DIM_ALLOC))
    ALLOCATE(pilier_trois(DIM_ALLOC))
    ALLOCATE(PUNT_ZELFVKB(DIM_ALLOC))
    ALLOCATE(InFODSZ(DIM_ALLOC))
    !/*HistorInfo*/
    ALLOCATE(LABMSTAT_PRIV(DIM_ALLOC))
    ALLOCATE(LABMSTAT_PUB(DIM_ALLOC))
    !Variables read from file FAMAL_CONTRIB.txt
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))              
    ALLOCATE(CONTRIB_LABMSTAT_PRIV_QT(DIM_ALLOC))            
    ALLOCATE(CONTRIB_LABMSTAT_PUB_QT(DIM_ALLOC))             
    ALLOCATE(CONTRIB_LABMSTAT_SELF_QT(DIM_ALLOC))            
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT(DIM_ALLOC))              
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT(DIM_ALLOC))               
    ALLOCATE(CONTRIB_SSSELF_EMER_QT(DIM_ALLOC))              
    ALLOCATE(CONTRIB_REVPRIV_QT(DIM_ALLOC))                  
    ALLOCATE(CONTRIB_REVPUB_QT(DIM_ALLOC))                   
    ALLOCATE(CONTRIB_INCSELF_QT(DIM_ALLOC))                  
    !Variables read from file FAMAL_UNEM.txt
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))                 
    ALLOCATE(UNEM_TOTBEN_QT(DIM_ALLOC))                      
    ALLOCATE(UNEM_FULL6MON_QT(DIM_ALLOC))                               
    !Variables read from file FAMAL_SICK.txt     
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))                 
    ALLOCATE(SICK_TYPEINSU_PRIV_QT(DIM_ALLOC))               
    ALLOCATE(SICK_TYPEINSU_PUB_QT(DIM_ALLOC))                
    ALLOCATE(SICK_TYPEINSU_SELF_QT(DIM_ALLOC))               
    ALLOCATE(SICK_TOTBEN_QT(DIM_ALLOC))                      
    ALLOCATE(SICK_DISABBEN_QT(DIM_ALLOC))                    
    !Variables read from file FAMAL_PENSWELF.txt
    ALLOCATE(GECODEERD_INSZ_PENSWELF(DIM_ALLOC))                 
    ALLOCATE(PENSWELF_BENSCHEM_QT(DIM_ALLOC))                          
    ALLOCATE(PENSWELF_AMOUNTP_QT(DIM_ALLOC))                       
    ALLOCATE(PENSWELF_SURPENS_QT(DIM_ALLOC))                               
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !Variables read from file FAMAL_SOCBEN.txt                                                    
    ALLOCATE(GECODEERD_INSZ_SOCBEN(DIM_ALLOC))
    ALLOCATE(SOCBEN_MINIMEX_QT(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE FAMAL_ALLOC_EXOG

!======================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC!
!======================================================!

SUBROUTINE FAMAL_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(FAMAL_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(FAMAL_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(FAMAL_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(FAMAL_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(FAMAL_HH_SIZE(DIM_ALLOC))
    ALLOCATE(FAMAL_START_HH(DIM_ALLOC))
    ALLOCATE(FAMAL_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    !Identification variables necessary to compute family allowances             
    ALLOCATE(FAMAL_BENEFID_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_RECIPID_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_BENSCHEM_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_HANDICAP_ANN(DIM_ALLOC))                  
    ALLOCATE(FAMAL_LABMINC_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_STUDENT_ANN(DIM_ALLOC))                   
    ALLOCATE(FAMAL_STARTPAY_ANN(DIM_ALLOC))                  
    ALLOCATE(FAMAL_ENDPAY_ANN(DIM_ALLOC))                    
    ALLOCATE(FAMAL_CHILELIG_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_INDINC_QT(DIM_ALLOC))                             
    ALLOCATE(FAMAL_NUMDEPCH_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_BENGUARI_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_NMONTHBE_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_WIDOWER_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_ORPHAN_QT(DIM_ALLOC))                     
    ALLOCATE(FAMAL_BIRTRANK_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_CBENRANK_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_SINGCHIL_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_YOUNGEST_QT(DIM_ALLOC))                   
    !Computation of family allowances: basic allowances              
    ALLOCATE(FAMAL_BIRTHALL_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_ADOPALL_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_ORALLWEA_QT(DIM_ALLOC))                  
    ALLOCATE(FAMAL_ORALLCIV_QT(DIM_ALLOC))                  
    ALLOCATE(FAMAL_ORALLSEL_QT(DIM_ALLOC))                  
    ALLOCATE(FAMAL_ORALLGUA_QT(DIM_ALLOC))                        
    ALLOCATE(FAMAL_ORALLORP_QT(DIM_ALLOC))                  
    !Computation of family allowances: supplements             
    ALLOCATE(FAMAL_ASUPWEA_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_ASUPSEL_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_REPIBEN_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_REPIREC_QT(DIM_ALLOC))                    
    ALLOCATE(FAMAL_BENIMEAN_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_REPIMEAN_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_SOCSAMOU_QT(DIM_ALLOC))                          
    ALLOCATE(FAMAL_SUPLSING_QT(DIM_ALLOC))                          
    ALLOCATE(FAMAL_DISA66PL_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_HANDSCAL_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_HANDAMOU_QT(DIM_ALLOC))                   
    ALLOCATE(FAMAL_SCHOOL_QT(DIM_ALLOC))                   
    !Variables for computation of RKW simulations
    ALLOCATE(FAMAL_MEANSRKW_QT(DIM_ALLOC,3))
    ALLOCATE(FAMAL_RKWSUPL_QT(DIM_ALLOC))

    !Instruments
    ALLOCATE(RH_RKW_INPUT(DIM_ALLOC)) 
    ALLOCATE(BT_RKW_INPUT(DIM_ALLOC)) 
    ALLOCATE(RH_RSVZ_INPUT(DIM_ALLOC)) 
    ALLOCATE(BT_RSVZ_INPUT(DIM_ALLOC)) 
    ALLOCATE(HHMEMB_QT(DIM_ALLOC))
    ALLOCATE(INDISBEN_QT(DIM_ALLOC))
    ALLOCATE(INDISREC_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_BENSCHIL_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_NEWBORN_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_NUMRECIP_QT(DIM_ALLOC)) 
    ALLOCATE(FAMAL_NUMNEWB_QT(DIM_ALLOC)) 
    ALLOCATE(FAMAL_ELIGCHHH_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_BENSCMAX_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_AGECLASS(DIM_ALLOC))
    ALLOCATE(FAMAL_BENSCHRP_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_RECICHRP_QT(DIM_ALLOC))
    ALLOCATE(FAMAL_RHBTKNOW(DIM_ALLOC))

    !Constructed exchange variable
    ALLOCATE(FAMAL_AMOUNTF_QT(DIM_ALLOC))
    
END SUBROUTINE FAMAL_ALLOC_ENDO

SUBROUTINE FAMAL_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !FAMAL_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_RELATION)
    DEALLOCATE(NAREGNIS_CIVIL)
    DEALLOCATE(NAREGNIS_SEX)
    !/*RKW*/
    DEALLOCATE(gecodeerd_insz_rh)
    DEALLOCATE(gecodeerd_insz_bt)
    DEALLOCATE(InRKW)
    DEALLOCATE(BEGINRKW_2008)
    DEALLOCATE(EINDRKW_2008)
    DEALLOCATE(RKW_GWB_2008)
    !/*RSVZ*/    
    DEALLOCATE(gecodeerd_insz_rh_rsvz)
    DEALLOCATE(gecodeerd_insz_bt_rsvz)
    DEALLOCATE(InRSVZ_KB)
    DEALLOCATE(QUART_RSVZKB_2008)
    !/*FOD SZ*/
    DEALLOCATE(code_leg)
    DEALLOCATE(pilier_un_conv)
    DEALLOCATE(pilier_deux)
    DEALLOCATE(pilier_trois)
    DEALLOCATE(PUNT_ZELFVKB)
    DEALLOCATE(InFODSZ)
    !/*HistorInfo*/
    DEALLOCATE(LABMSTAT_PRIV)
    DEALLOCATE(LABMSTAT_PUB)
    !Variables read from file FAMAL_CONTRIB.txt
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_LABMSTAT_PRIV_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PUB_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_SELF_QT)
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT)
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT)
    DEALLOCATE(CONTRIB_REVPRIV_QT)
    DEALLOCATE(CONTRIB_REVPUB_QT)
    DEALLOCATE(CONTRIB_INCSELF_QT)
    !Variables read from file FAMAL_UNEM.txt
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_TOTBEN_QT)
    DEALLOCATE(UNEM_FULL6MON_QT)
    !Variables read from file FAMAL_SICK.txt     
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_TYPEINSU_PRIV_QT)
    DEALLOCATE(SICK_TYPEINSU_PUB_QT)
    DEALLOCATE(SICK_TYPEINSU_SELF_QT)
    DEALLOCATE(SICK_TOTBEN_QT)
    DEALLOCATE(SICK_DISABBEN_QT)
    !Variables read from file FAMAL_PENSWELF.txt
    DEALLOCATE(GECODEERD_INSZ_PENSWELF)
    DEALLOCATE(PENSWELF_BENSCHEM_QT)
    DEALLOCATE(PENSWELF_AMOUNTP_QT)
    DEALLOCATE(PENSWELF_SURPENS_QT)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !Variables read from file FAMAL_SOCBEN.txt                                                    
    DEALLOCATE(GECODEERD_INSZ_SOCBEN)
    DEALLOCATE(SOCBEN_MINIMEX_QT)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !=======================================================================!  
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(FAMAL_YEAR_AGE)
    DEALLOCATE(FAMAL_MONTH_AGE)
    DEALLOCATE(FAMAL_MONTH_BIRTH)
    DEALLOCATE(FAMAL_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(FAMAL_HH_SIZE)
    DEALLOCATE(FAMAL_START_HH)
    DEALLOCATE(FAMAL_END_HH)
    !To construct variables listed in the module note
    !Identification variables necessary to compute family allowances             
    DEALLOCATE(FAMAL_BENEFID_QT)
    DEALLOCATE(FAMAL_RECIPID_QT)
    DEALLOCATE(FAMAL_BENSCHEM_QT)
    DEALLOCATE(FAMAL_HANDICAP_ANN)
    DEALLOCATE(FAMAL_LABMINC_QT)
    DEALLOCATE(FAMAL_STUDENT_ANN)
    DEALLOCATE(FAMAL_STARTPAY_ANN)
    DEALLOCATE(FAMAL_ENDPAY_ANN)
    DEALLOCATE(FAMAL_CHILELIG_QT)
    DEALLOCATE(FAMAL_INDINC_QT)
    DEALLOCATE(FAMAL_NUMDEPCH_QT)
    DEALLOCATE(FAMAL_BENGUARI_QT)
    DEALLOCATE(FAMAL_NMONTHBE_QT)
    DEALLOCATE(FAMAL_WIDOWER_QT)
    DEALLOCATE(FAMAL_ORPHAN_QT)
    DEALLOCATE(FAMAL_BIRTRANK_QT)
    DEALLOCATE(FAMAL_CBENRANK_QT)
    DEALLOCATE(FAMAL_SINGCHIL_QT)
    DEALLOCATE(FAMAL_YOUNGEST_QT)
    !Computation of family allowances: basic allowances              
    DEALLOCATE(FAMAL_BIRTHALL_QT)
    DEALLOCATE(FAMAL_ADOPALL_QT)
    DEALLOCATE(FAMAL_ORALLWEA_QT)
    DEALLOCATE(FAMAL_ORALLCIV_QT)
    DEALLOCATE(FAMAL_ORALLSEL_QT)
    DEALLOCATE(FAMAL_ORALLGUA_QT)
    DEALLOCATE(FAMAL_ORALLORP_QT)
    !Computation of family allowances: supplements             
    DEALLOCATE(FAMAL_ASUPWEA_QT)
    DEALLOCATE(FAMAL_ASUPSEL_QT)
    DEALLOCATE(FAMAL_REPIBEN_QT)
    DEALLOCATE(FAMAL_REPIREC_QT)
    DEALLOCATE(FAMAL_BENIMEAN_QT)
    DEALLOCATE(FAMAL_REPIMEAN_QT)
    DEALLOCATE(FAMAL_SOCSAMOU_QT)
    DEALLOCATE(FAMAL_SUPLSING_QT)
    DEALLOCATE(FAMAL_DISA66PL_QT)
    DEALLOCATE(FAMAL_HANDSCAL_QT)
    DEALLOCATE(FAMAL_HANDAMOU_QT)
    DEALLOCATE(FAMAL_SCHOOL_QT)
    !Variables for computation of RKW simulations
    DEALLOCATE(FAMAL_MEANSRKW_QT)
    DEALLOCATE(FAMAL_RKWSUPL_QT)

    !Instruments
    DEALLOCATE(RH_RKW_INPUT)
    DEALLOCATE(BT_RKW_INPUT)
    DEALLOCATE(RH_RSVZ_INPUT)
    DEALLOCATE(BT_RSVZ_INPUT)
    DEALLOCATE(HHMEMB_QT)
    DEALLOCATE(INDISBEN_QT)
    DEALLOCATE(INDISREC_QT)
    DEALLOCATE(FAMAL_BENSCHIL_QT)
    DEALLOCATE(FAMAL_NEWBORN_QT)
    DEALLOCATE(FAMAL_NUMRECIP_QT)
    DEALLOCATE(FAMAL_NUMNEWB_QT)
    DEALLOCATE(FAMAL_ELIGCHHH_QT)
    DEALLOCATE(FAMAL_BENSCMAX_QT)
    DEALLOCATE(FAMAL_AGECLASS)
    DEALLOCATE(FAMAL_BENSCHRP_QT)
    DEALLOCATE(FAMAL_RECICHRP_QT)
    DEALLOCATE(FAMAL_RHBTKNOW)
    
    !Constructed exchange variable
    DEALLOCATE(FAMAL_AMOUNTF_QT)

    !=======================================================================!
    !Deallocate parameters
    !Identification variables necessary to compute family allowances             
    DEALLOCATE(FAMAL_FAMREL_QT)
    !Computation of family allowances: basic allowances              
    DEALLOCATE(FAMAL_BALLAMOU_ANN)
    DEALLOCATE(FAMAL_ADOPAMOU_ANN)
    DEALLOCATE(FAMAL_OWEAAMOU_MONTH)
    DEALLOCATE(FAMAL_OCIVAMOU_MONTH)    
    DEALLOCATE(FAMAL_OSELAMOU_MONTH)    
    DEALLOCATE(FAMAL_OGUAAMOU_MONTH)    
    !Computation of family allowances: supplements               
    DEALLOCATE(FAMAL_ASWEAR1_MONTH)
    DEALLOCATE(FAMAL_ASWEAR23_MONTH)
    DEALLOCATE(FAMAL_REPLEVEL_MONTH)
    DEALLOCATE(FAMAL_SOCSUP1_MONTH)
    DEALLOCATE(FAMAL_SOCSUP2_MONTH) 
    DEALLOCATE(FAMAL_SOCSUP3_MONTH) 
    !Deallocate extras
    DEALLOCATE(FAMAL_SCHOOLPR_MONTH)
    DEALLOCATE(FAMAL_SINGHHSP_MONTH)
    DEALLOCATE(FAMAL_SUPRKW_MONTH)
    !Variables for computation of disability benefits
    DEALLOCATE(FAMAL_VKBOLD_MONTH)
    DEALLOCATE(FAMAL_VKBNEW_MONTH)
    
END SUBROUTINE FAMAL_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE FAMAL_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                               !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        FAMAL_YEAR_AGE(pers_j) = 0
        FAMAL_MONTH_AGE(pers_j) = 0
        FAMAL_MONTH_BIRTH(pers_j) = 0
        FAMAL_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        FAMAL_HH_SIZE(pers_j) = 0
        FAMAL_START_HH(pers_j) = 0
        FAMAL_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        !Identification variables necessary to compute family allowances             
        FAMAL_BENEFID_QT(pers_j) = 0
        FAMAL_RECIPID_QT(pers_j) = 0
        FAMAL_BENSCHEM_QT(pers_j) = 0
        FAMAL_HANDICAP_ANN(pers_j) = 0
        FAMAL_LABMINC_QT(pers_j) = 0
        FAMAL_STUDENT_ANN(pers_j) = 0
        FAMAL_STARTPAY_ANN(pers_j) = 0
        FAMAL_ENDPAY_ANN(pers_j) = 0
        FAMAL_CHILELIG_QT(pers_j) = 0
        FAMAL_INDINC_QT(pers_j) = 0
        FAMAL_NUMDEPCH_QT(pers_j) = 0
        FAMAL_BENGUARI_QT(pers_j) = 0
        FAMAL_NMONTHBE_QT(pers_j) = 0
        FAMAL_WIDOWER_QT(pers_j) = 0
        FAMAL_ORPHAN_QT(pers_j) = 0
        FAMAL_BIRTRANK_QT(pers_j) = 0
        FAMAL_CBENRANK_QT(pers_j) = 0
        FAMAL_SINGCHIL_QT(pers_j) = 0
        FAMAL_YOUNGEST_QT(pers_j) = 0
        !Computation of family allowances: basic allowances              
        FAMAL_BIRTHALL_QT(pers_j) = 0
        FAMAL_ADOPALL_QT(pers_j) = 0
        FAMAL_ORALLWEA_QT(pers_j) = 0
        FAMAL_ORALLCIV_QT(pers_j) = 0
        FAMAL_ORALLSEL_QT(pers_j) = 0
        FAMAL_ORALLGUA_QT(pers_j) = 0
        FAMAL_ORALLORP_QT(pers_j) = 0
        !Computation of family allowances: supplements             
        FAMAL_ASUPWEA_QT(pers_j) = 0
        FAMAL_ASUPSEL_QT(pers_j) = 0
        FAMAL_REPIBEN_QT(pers_j) = 0
        FAMAL_REPIREC_QT(pers_j) = 0
        FAMAL_BENIMEAN_QT(pers_j) = 0
        FAMAL_REPIMEAN_QT(pers_j) = 0
        FAMAL_SOCSAMOU_QT(pers_j) = 0
        FAMAL_SUPLSING_QT(pers_j) = 0
        FAMAL_DISA66PL_QT(pers_j) = 0
        FAMAL_HANDSCAL_QT(pers_j) = 0
        FAMAL_HANDAMOU_QT(pers_j) = 0
        FAMAL_SCHOOL_QT(pers_j) = 0
        !Variables for computation of RKW simulations
        FAMAL_MEANSRKW_QT(pers_j,1:3) = 0
        FAMAL_RKWSUPL_QT(pers_j) = 0
        !Instruments
        RH_RKW_INPUT(pers_j) = 0
        BT_RKW_INPUT(pers_j) = 0
        RH_RSVZ_INPUT(pers_j) = 0
        BT_RSVZ_INPUT(pers_j) = 0
        HHMEMB_QT(pers_j) = 0
        INDISBEN_QT(pers_j) = 0
        INDISREC_QT(pers_j) = 0
        FAMAL_BENSCHIL_QT(pers_j) = 0
        FAMAL_NEWBORN_QT(pers_j) = 0
        FAMAL_NUMRECIP_QT(pers_j) = 0
        FAMAL_NUMNEWB_QT(pers_j) = 0
        FAMAL_ELIGCHHH_QT(pers_j) = 0
        FAMAL_BENSCMAX_QT(pers_j) = 0
        FAMAL_AGECLASS(pers_j) = 0
        FAMAL_BENSCHRP_QT(pers_j) = 0
        FAMAL_RECICHRP_QT(pers_j) = 0
        FAMAL_RHBTKNOW(pers_j) = 0
        !Constructed exchange variable
        FAMAL_AMOUNTF_QT(pers_j) = 0
    END DO
    N_HH_FAMAL=0

END SUBROUTINE FAMAL_INITENDO

!========== SUBROUTINE FAMAL_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!=========================================================!

SUBROUTINE FAMAL_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !FAMAL_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*RKW*/
    CALL SORTING(gecodeerd_insz_rh(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_bt(1:n_rec),NEW_RANK)
    CALL SORTING(InRKW(1:n_rec),NEW_RANK)
    CALL SORTING(BEGINRKW_2008(1:n_rec),NEW_RANK)
    CALL SORTING(EINDRKW_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RKW_GWB_2008(1:n_rec),NEW_RANK)
    !/*RSVZ*/    
    CALL SORTING(gecodeerd_insz_rh_rsvz(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_bt_rsvz(1:n_rec),NEW_RANK)
    CALL SORTING(InRSVZ_KB(1:n_rec),NEW_RANK)
    CALL SORTING(QUART_RSVZKB_2008(1:n_rec),NEW_RANK)
    !/*FOD SZ*/
    CALL SORTING(code_leg(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_un_conv(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_deux(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_trois(1:n_rec),NEW_RANK)
    CALL SORTING(PUNT_ZELFVKB(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !Rank individual identification number from the FAMAL_CONTRIB data file
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the FAMAL_UNEM data file
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_TOTBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_FULL6MON_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the FAMAL_SICK data file
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_SICK.txt     
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABBEN_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from the FAMAL_PENSWELF data file
    CALL RANKING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_BENSCHEM_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_SURPENS_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file FAMAL_SOCBEN.txt
    CALL RANKING(GECODEERD_INSZ_SOCBEN(1:n_rec),NEW_RANK)
    !Variables read from file FAMAL_SOCBEN.txt                                                    
    CALL SORTING(GECODEERD_INSZ_SOCBEN(1:n_rec),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL FAMAL_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !FAMAL_INPUT_Write.sas
    !/*Variabelen uit PopChar-bestand*/
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_CIVIL(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*RKW*/
    CALL SORTING(gecodeerd_insz_rh(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_bt(1:n_rec),NEW_RANK)
    CALL SORTING(InRKW(1:n_rec),NEW_RANK)
    CALL SORTING(BEGINRKW_2008(1:n_rec),NEW_RANK)
    CALL SORTING(EINDRKW_2008(1:n_rec),NEW_RANK)
    CALL SORTING(RKW_GWB_2008(1:n_rec),NEW_RANK)
    !/*RSVZ*/    
    CALL SORTING(gecodeerd_insz_rh_rsvz(1:n_rec),NEW_RANK)
    CALL SORTING(gecodeerd_insz_bt_rsvz(1:n_rec),NEW_RANK)
    CALL SORTING(InRSVZ_KB(1:n_rec),NEW_RANK)
    CALL SORTING(QUART_RSVZKB_2008(1:n_rec),NEW_RANK)
    !/*FOD SZ*/
    CALL SORTING(code_leg(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_un_conv(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_deux(1:n_rec),NEW_RANK)
    CALL SORTING(pilier_trois(1:n_rec),NEW_RANK)
    CALL SORTING(PUNT_ZELFVKB(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !/*HistorInfo*/
    CALL SORTING(LABMSTAT_PRIV(1:n_rec),NEW_RANK)
    CALL SORTING(LABMSTAT_PUB(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_TOTBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_FULL6MON_QT(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_SICK.txt     
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TYPEINSU_SELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_TOTBEN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_DISABBEN_QT(1:n_rec),NEW_RANK)
    !Sort variables read from file FAMAL_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_BENSCHEM_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_SURPENS_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Variables read from file FAMAL_SOCBEN.txt                                                    
    CALL SORTING(GECODEERD_INSZ_SOCBEN(1:n_rec),NEW_RANK)
    CALL SORTING(SOCBEN_MINIMEX_QT(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_WEIGHT.txt
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL FAMAL_CONSISTENCY_CHECK

END SUBROUTINE FAMAL_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from           !
!the parameter file                                      !
!========================================================!
SUBROUTINE FAMAL_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='FAMAL_INPUT_PATH') THEN
            FAMAL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_SICK_PRERUN_FILE') THEN
            FAMAL_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_UNEM_PRERUN_FILE') THEN
            FAMAL_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_CONTRIB_PRERUN_FILE') THEN
            FAMAL_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_PENSWELF_PRERUN_FILE') THEN
            FAMAL_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_SOCBEN_PRERUN_FILE') THEN
            FAMAL_SOCBEN_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_INPUT_FILE') THEN
            FAMAL_INPUT_FILE = CONTROL_IO(2*i)
        !Read material of other input files                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read material to read FAMAL reference information                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='FAMAL_2008_FILE') THEN
            FAMAL_2008_FILE = CONTROL_IO(2*i)
        !Read in names of output files 
        !output files in this run are input files of other modules
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_FAMAL_PRERUN_FILE') THEN
            CONTRIB_FAMAL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SICK_FAMAL_PRERUN_FILE') THEN
            SICK_FAMAL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_FAMAL_PRERUN_FILE') THEN
            UNEM_FAMAL_FILE = CONTROL_IO(2*i)            
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_FAMAL_PRERUN_FILE') THEN
            SOCBEN_FAMAL_FILE = CONTROL_IO(2*i)            
        END IF
    END DO     
    !check for length of default path and file name parameters
    FAMAL_PAR_BREAK_POINT(1) = INDEX(FAMAL_INPUT_PATH,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(2) = INDEX(FAMAL_INPUT_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(3) = INDEX(FAMAL_SICK_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(4) = INDEX(FAMAL_UNEM_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(12)= INDEX(FAMAL_CONTRIB_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(5) = INDEX(FAMAL_PENSWELF_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(17)= INDEX(FAMAL_SOCBEN_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(6) = INDEX(CONTRIB_FAMAL_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(7) = INDEX(SICK_FAMAL_FILE,FAMAL_PAR_BREAK)           
    FAMAL_PAR_BREAK_POINT(8) = INDEX(UNEM_FAMAL_FILE,FAMAL_PAR_BREAK) 
    FAMAL_PAR_BREAK_POINT(9) = INDEX(CONSTANT_INPUT_PATH,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(10)= INDEX(MIMOSIS_SAMPWEIG_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(13)= INDEX(FAMREL_INPUT_PATH,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(14)= INDEX(MIMOSIS_FAMREL_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(15)= INDEX(WAGE_CONSTRUCT_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(16)= INDEX(FAMAL_2008_FILE,FAMAL_PAR_BREAK)
    FAMAL_PAR_BREAK_POINT(18)= INDEX(SOCBEN_FAMAL_FILE,FAMAL_PAR_BREAK)
    !merge information (path+name) to create default input file names
    !of input files
    FAMAL_INPUT = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_INPUT_FILE(1:FAMAL_PAR_BREAK_POINT(2)-1)
    FAMAL_SICK = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_SICK_FILE(1:FAMAL_PAR_BREAK_POINT(3)-1)
    FAMAL_UNEM = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_UNEM_FILE(1:FAMAL_PAR_BREAK_POINT(4)-1)
    FAMAL_CONTRIB = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_CONTRIB_FILE(1:FAMAL_PAR_BREAK_POINT(12)-1)
    FAMAL_PENSWELF = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_PENSWELF_FILE(1:FAMAL_PAR_BREAK_POINT(5)-1)
    FAMAL_SOCBEN = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &FAMAL_SOCBEN_FILE(1:FAMAL_PAR_BREAK_POINT(17)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(13)-1)//&
    &MIMOSIS_FAMREL_FILE(1:FAMAL_PAR_BREAK_POINT(14)-1)    
    WAGE_CONSTRUCT = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:FAMAL_PAR_BREAK_POINT(15)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(9)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:FAMAL_PAR_BREAK_POINT(10)-1)
    FAMAL_2008 = CONSTANT_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(9)-1)//&
    &FAMAL_2008_FILE(1:FAMAL_PAR_BREAK_POINT(16)-1)    
    !of output files
    CONTRIB_FAMAL = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_FAMAL_FILE(1:FAMAL_PAR_BREAK_POINT(6)-1)        
    SICK_FAMAL = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &SICK_FAMAL_FILE(1:FAMAL_PAR_BREAK_POINT(7)-1)
    UNEM_FAMAL = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &UNEM_FAMAL_FILE(1:FAMAL_PAR_BREAK_POINT(8)-1)   
    SOCBEN_FAMAL = FAMAL_INPUT_PATH(1:FAMAL_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_FAMAL_FILE(1:FAMAL_PAR_BREAK_POINT(18)-1)   

    !Trim the result
    !of input files
    CALL TRIM@(FAMAL_SICK)
    CALL TRIM@(FAMAL_UNEM)
    CALL TRIM@(FAMAL_CONTRIB)
    CALL TRIM@(FAMAL_PENSWELF)
    CALL TRIM@(FAMAL_SOCBEN)
    CALL TRIM@(FAMAL_INPUT)
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(MIMOSIS_SAMPWEIG) 
    CALL TRIM@(FAMAL_2008)  
    !of output files
    CALL TRIM@(CONTRIB_FAMAL)
    CALL TRIM@(SICK_FAMAL)    
    CALL TRIM@(UNEM_FAMAL)  
    CALL TRIM@(SOCBEN_FAMAL)  

    !write names to log file
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')    
    !of input files
    WRITE(15,*)     'INPUT FILE=',      FAMAL_INPUT
    WRITE(15,*)     'SICK FILE=',       FAMAL_SICK
    WRITE(15,*)     'UNEM FILE=',       FAMAL_UNEM
    WRITE(15,*)     'CONTRIB FILE=',    FAMAL_CONTRIB
    WRITE(15,*)     'PENSWELF FILE=',   FAMAL_PENSWELF
    WRITE(15,*)     'SOCBEN FILE=',     FAMAL_SOCBEN
    WRITE(15,*)     'MIMOSIS_FAMREL= ', MIMOSIS_FAMREL
    WRITE(15,*)     'WAGE_CONSTRUCT= ', WAGE_CONSTRUCT
    WRITE(15,*)     'WEIGHT FILE= ',    MIMOSIS_SAMPWEIG    
    WRITE(15,*)     'FAMAL_2008= ',     FAMAL_2008  
    !of output files
    WRITE(15,*)     'CONTRIB_FAMAL=',   CONTRIB_FAMAL
    WRITE(15,*)     'SICK_FAMAL=',      SICK_FAMAL  
    WRITE(15,*)     'UNEM_FAMAL=',      UNEM_FAMAL 
    WRITE(15,*)     'SOCBEN_FAMAL=',    SOCBEN_FAMAL 
        
    !open parameter file        
    OPEN(UNIT=3, FILE=FAMAL_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_PARLOC
        WRITE(15,'("Could not find ", a100)') FAMAL_PARLOC
        PAUSE
        STOP
    END IF        

    ALLOCATE(FAMAL_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') FAMAL_PAR_NAMES(i), FAMAL_PAR_L, &
        &FAMAL_PAR_C, FAMAL_PAR_CURR

        IF (FAMAL_PAR_NAMES(i)=='FAMAL_FAMREL_QT') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_FAMREL_QT(FAMAL_PAR_L,FAMAL_PAR_C))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) ((FAMAL_FAMREL_QT(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) ((FAMAL_FAMREL_QT(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_CHILUNEL_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_CHILUNEL_MONTH
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_CHILUNEL_MONTH
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_CHILINCL_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_CHILINCL_MONTH
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_CHILINCL_MONTH
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_GUARINPC_QT') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_GUARINPC_QT
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_GUARINPC_QT
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_GUARINAB_QT') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_GUARINAB_QT
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_GUARINAB_QT
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_BALLAMOU_ANN') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_BALLAMOU_ANN(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_BALLAMOU_ANN(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_BALLAMOU_ANN(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_ADOPAMOU_ANN') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_ADOPAMOU_ANN(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_ADOPAMOU_ANN(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_ADOPAMOU_ANN(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_OWEAAMOU_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_OWEAAMOU_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_OWEAAMOU_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_OWEAAMOU_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_OCIVAMOU_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_OCIVAMOU_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_OCIVAMOU_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_OCIVAMOU_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_OSELAMOU_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_OSELAMOU_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_OSELAMOU_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_OSELAMOU_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_OGUAAMOU_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_OGUAAMOU_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_OGUAAMOU_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_OGUAAMOU_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_OORPAMOU_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_OORPAMOU_MONTH
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_OORPAMOU_MONTH
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_ASWEAR1_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_ASWEAR1_MONTH(FAMAL_PAR_L,FAMAL_PAR_C))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) ((FAMAL_ASWEAR1_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) ((FAMAL_ASWEAR1_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_ASWEAR23_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_ASWEAR23_MONTH(FAMAL_PAR_L,FAMAL_PAR_C))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) ((FAMAL_ASWEAR23_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) ((FAMAL_ASWEAR23_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_REPLEVEL_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_REPLEVEL_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_REPLEVEL_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_REPLEVEL_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SOCSUP1_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SOCSUP1_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP1_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP1_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SOCSUP2_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SOCSUP2_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP2_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP2_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SOCSUP3_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SOCSUP3_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP3_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_SOCSUP3_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_ASSELFFS_ANN') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_ASSELFFS_ANN
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_ASSELFFS_ANN
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SCHOOLPR_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SCHOOLPR_MONTH(FAMAL_PAR_L,FAMAL_PAR_C))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) ((FAMAL_SCHOOLPR_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) ((FAMAL_SCHOOLPR_MONTH(j,k),k=1,FAMAL_PAR_C),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SINGHHSP_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SINGHHSP_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_SINGHHSP_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_SINGHHSP_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_INCLIMSP_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_INCLIMSP_MONTH
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_INCLIMSP_MONTH
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SIMSURKW_QT') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) FAMAL_SIMSURKW_QT
            WRITE(15,FAMAL_PAR_FORMAT) FAMAL_SIMSURKW_QT
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_SUPRKW_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_SUPRKW_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_SUPRKW_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_SUPRKW_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_VKBOLD_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_VKBOLD_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_VKBOLD_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_VKBOLD_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='FAMAL_VKBNEW_MONTH') THEN
            WRITE(15,'(a40)') FAMAL_PAR_NAMES(i)
            ALLOCATE(FAMAL_VKBNEW_MONTH(FAMAL_PAR_L))
            READ(3,'(a20)') FAMAL_PAR_FORMAT
            READ(3,FAMAL_PAR_FORMAT) (FAMAL_VKBNEW_MONTH(j),j=1,FAMAL_PAR_L)
            WRITE(15,FAMAL_PAR_FORMAT) (FAMAL_VKBNEW_MONTH(j),j=1,FAMAL_PAR_L)
        ELSE IF (FAMAL_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(FAMAL_PAR_NAMES)
    
    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine FAMAL_PARAMETER_FILEREAD of PRE RUN........runned correctly'  

END SUBROUTINE FAMAL_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE FAMAL_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=FAMAL_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_INPUT
        WRITE(15,'("Could not find ", a100)') FAMAL_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,A7,I10,I10,I10,I10,A14,A14,I10,A7,A7,I10,A14,A14,I10,A4,&
            &A2,I10,I10,I10,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_HH_ID(pers_j),&
            &NAREGNIS_RELATION(pers_j),NAREGNIS_CIVIL(pers_j),NAREGNIS_SEX(pers_j),&
            &GECODEERD_INSZ_RH(pers_j),GECODEERD_INSZ_BT(pers_j),INRKW(pers_j),&
            &BEGINRKW_2008(pers_j),EINDRKW_2008(pers_j),RKW_GWB_2008(pers_j),&
            &GECODEERD_INSZ_RH_RSVZ(pers_j),GECODEERD_INSZ_BT_RSVZ(pers_j),INRSVZ_KB(pers_j),&
            &QUART_RSVZKB_2008(pers_j),CODE_LEG(pers_j),PILIER_UN_CONV(pers_j),&
            &PILIER_DEUX(pers_j),PILIER_TROIS(pers_j),PUNT_ZELFVKB(pers_j),&
            &INFODSZ(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_HH_ID(pers_j),&
                &NAREGNIS_RELATION(pers_j),NAREGNIS_CIVIL(pers_j),NAREGNIS_SEX(pers_j),&
                &GECODEERD_INSZ_RH(pers_j),GECODEERD_INSZ_BT(pers_j),INRKW(pers_j),&
                &BEGINRKW_2008(pers_j),EINDRKW_2008(pers_j),RKW_GWB_2008(pers_j),&
                &GECODEERD_INSZ_RH_RSVZ(pers_j),GECODEERD_INSZ_BT_RSVZ(pers_j),INRSVZ_KB(pers_j),&
                &QUART_RSVZKB_2008(pers_j),CODE_LEG(pers_j),PILIER_UN_CONV(pers_j),&
                &PILIER_DEUX(pers_j),PILIER_TROIS(pers_j),PUNT_ZELFVKB(pers_j),&
                &INFODSZ(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)'')')
                WRITE(15,'(A14,A7,I10,I10,I10,I10,A14,A14,I10,A7,A7,I10,A14,A14,I10,A4,&
                &A2,I10,I10,I10,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_HH_ID(pers_j),&
                &NAREGNIS_RELATION(pers_j),NAREGNIS_CIVIL(pers_j),NAREGNIS_SEX(pers_j),&
                &GECODEERD_INSZ_RH(pers_j),GECODEERD_INSZ_BT(pers_j),INRKW(pers_j),&
                &BEGINRKW_2008(pers_j),EINDRKW_2008(pers_j),RKW_GWB_2008(pers_j),&
                &GECODEERD_INSZ_RH_RSVZ(pers_j),GECODEERD_INSZ_BT_RSVZ(pers_j),INRSVZ_KB(pers_j),&
                &QUART_RSVZKB_2008(pers_j),CODE_LEG(pers_j),PILIER_UN_CONV(pers_j),&
                &PILIER_DEUX(pers_j),PILIER_TROIS(pers_j),PUNT_ZELFVKB(pers_j),&
                &INFODSZ(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,A7,I10,I10,I10,I10,A14,A14,I10,A7,A7,I10,A14,A14,I10,A4,&
                &A2,I10,I10,I10,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_DATBIRTH(pers_j),NAREGNIS_HH_ID(pers_j),&
                &NAREGNIS_RELATION(pers_j),NAREGNIS_CIVIL(pers_j),NAREGNIS_SEX(pers_j),&
                &GECODEERD_INSZ_RH(pers_j),GECODEERD_INSZ_BT(pers_j),INRKW(pers_j),&
                &BEGINRKW_2008(pers_j),EINDRKW_2008(pers_j),RKW_GWB_2008(pers_j),&
                &GECODEERD_INSZ_RH_RSVZ(pers_j),GECODEERD_INSZ_BT_RSVZ(pers_j),INRSVZ_KB(pers_j),&
                &QUART_RSVZKB_2008(pers_j),CODE_LEG(pers_j),PILIER_UN_CONV(pers_j),&
                &PILIER_DEUX(pers_j),PILIER_TROIS(pers_j),PUNT_ZELFVKB(pers_j),&
                &INFODSZ(pers_j),LABMSTAT_PRIV(pers_j),LABMSTAT_PUB(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_INPUT_FILEREAD of PRE RUN............runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE FAMAL_INPUT_FILEREAD

!============= SUBROUTINE SICK_FILEREAD ================!
!This subroutine  reads in data from the SICK file      !
!=======================================================!
SUBROUTINE FAMAL_SICK_FILEREAD

    OPEN(UNIT=5, FILE=FAMAL_SICK, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_SICK
        WRITE(15,'("Could not find ", a100)') FAMAL_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,I10,I10,F15.2,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),&
            &SICK_TYPEINSU_SELF_QT(pers_j),SICK_TOTBEN_QT(pers_j),SICK_DISABBEN_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),&
                &SICK_TYPEINSU_SELF_QT(pers_j),SICK_TOTBEN_QT(pers_j),SICK_DISABBEN_QT(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,F15.2,I10)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),&
                &SICK_TYPEINSU_SELF_QT(pers_j),SICK_TOTBEN_QT(pers_j),SICK_DISABBEN_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,F15.2,I10)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_TYPEINSU_PRIV_QT(pers_j),SICK_TYPEINSU_PUB_QT(pers_j),&
                &SICK_TYPEINSU_SELF_QT(pers_j),SICK_TOTBEN_QT(pers_j),SICK_DISABBEN_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_SICK_FILEREAD of PRE RUN.............runned correctly'
    WRITE(15,'(I7, " records uploaded from SICK_FILE")') pers_j-1

END SUBROUTINE FAMAL_SICK_FILEREAD

!============= SUBROUTINE UNEM_FILEREAD ================!
!This subroutine  reads in data from the UNEM file      !
!=======================================================!
SUBROUTINE FAMAL_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=FAMAL_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_UNEM
        WRITE(15,'("Could not find ", a100)') FAMAL_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j),UNEM_FULL6MON_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j),UNEM_FULL6MON_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j),UNEM_FULL6MON_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,I10)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_TOTBEN_QT(pers_j),UNEM_FULL6MON_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_UNEM_FILEREAD of PRE RUN.............runned correctly'
    WRITE(15,'(I7, " records uploaded from UNEM_FILE")') pers_j-1

END SUBROUTINE FAMAL_UNEM_FILEREAD

!============= SUBROUTINE CONTRIB_FILEREAD ================!
!This subroutine reads in data from the CONTRIB file       !
!==========================================================!
SUBROUTINE FAMAL_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=FAMAL_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_CONTRIB
        WRITE(15,'("Could not find ", a100)') FAMAL_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,I10,I10,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
            &CONTRIB_LABMSTAT_SELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&
            &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&
                &CONTRIB_INCSELF_QT(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_LABMSTAT_SELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),CONTRIB_INCSELF_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_CONTRIB_FILEREAD of PRE RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from CONTRIB_FILE")') pers_j-1

END SUBROUTINE FAMAL_CONTRIB_FILEREAD

!============= SUBROUTINE PENSWELF_FILEREAD ================!
!This subroutines reads in data from the PENSWELF file      !
!===========================================================!
SUBROUTINE FAMAL_PENSWELF_FILEREAD

    OPEN(UNIT=5, FILE=FAMAL_PENSWELF, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_PENSWELF
        WRITE(15,'("Could not find ", a100)') FAMAL_PENSWELF
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,I10,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_BENSCHEM_QT(pers_j),PENSWELF_AMOUNTP_QT(pers_j),&
            &PENSWELF_SURPENS_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_PENSWELF
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_BENSCHEM_QT(pers_j),PENSWELF_AMOUNTP_QT(pers_j),&
                &PENSWELF_SURPENS_QT(pers_j)'')')
                WRITE(15,'(7x,I7,I10,F15.2,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_BENSCHEM_QT(pers_j),PENSWELF_AMOUNTP_QT(pers_j),&
                &PENSWELF_SURPENS_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,F15.2,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_BENSCHEM_QT(pers_j),PENSWELF_AMOUNTP_QT(pers_j),&
                &PENSWELF_SURPENS_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_PENSWELF_FILEREAD of PRE RUN.........runned correctly'
    WRITE(15,'(I7, " records uploaded from PENSWELF_FILE")') pers_j-1

END SUBROUTINE FAMAL_PENSWELF_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE FAMAL_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_FAMREL_FILEREAD of PRE RUN...........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE FAMAL_FAMREL_FILEREAD

!=================================================!
!Reads in data from the SOCBEN file              !
!=================================================!

SUBROUTINE FAMAL_SOCBEN_FILEREAD

    OPEN (UNIT=5, FILE=FAMAL_SOCBEN, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') FAMAL_SOCBEN
        WRITE (15,'("Could not find or open ", a100)') FAMAL_SOCBEN
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,F15.2)',IOSTAT=ierror) &
            &GECODEERD_INSZ_SOCBEN(pers_j),SOCBEN_MINIMEX_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,FAMAL_SOCBEN
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SOCBEN(pers_j),SOCBEN_MINIMEX_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SOCBEN(pers_j),SOCBEN_MINIMEX_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_SOCBEN(pers_j),SOCBEN_MINIMEX_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_SOCBEN_FILEREAD of PRE RUN...........runned correctly'
    WRITE (15,'(I7, " records uploaded from FAMAL_SOCBEN")') pers_j-1 

END SUBROUTINE FAMAL_SOCBEN_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE FAMAL_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        WRITE (15,'("Could not find or open ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_WAGE_FILEREAD of PRE RUN.............runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE FAMAL_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE FAMAL_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE (*,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        WRITE (15,'("Could not find or open ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine FAMAL_WEIGHT_FILEREAD of PRE RUN...........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE FAMAL_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE FAMAL_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))   .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))  .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))    .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))       .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))    .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))   .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))    .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))   .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))     .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j)).OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))  .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j)) .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))   .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j)) .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j)) .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))   .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))   .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))     .OR.&
            &(GECODEERD_INSZ_WEIGHT(pers_j).NE.GECODEERD_INSZ_SOCBEN(pers_j))       &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   
   
END SUBROUTINE FAMAL_CONSISTENCY_CHECK 

!============= SUBROUTINE FAMAL_RECODE=========================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE FAMAL_RECODE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !FAMAL_INPUT_WRITE.SAS
        !/*VARIABELEN UIT POPCHAR-BESTAND*/
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)             NAREGNIS_HH_ID(pers_j)=0
        IF (INT(NAREGNIS_RELATION(pers_j)).EQ.9999999)          NAREGNIS_RELATION(pers_j)=0
        IF (INT(NAREGNIS_CIVIL(pers_j)).EQ.9999999)             NAREGNIS_CIVIL(pers_j)=0
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)               NAREGNIS_SEX(pers_j)=0
        !/*RKW*/
        CALL TRIM@(GECODEERD_INSZ_RH(pers_j))
        CALL TRIM@(GECODEERD_INSZ_BT(pers_j))
        IF (INT(INRKW(pers_j)).EQ.9999999)                      INRKW(pers_j)=0
        CALL TRIM@(BEGINRKW_2008(pers_j))
        CALL TRIM@(EINDRKW_2008(pers_j))
        IF (INT(RKW_GWB_2008(pers_j)).EQ.9999999)               RKW_GWB_2008(pers_j)=0
        !/*RSVZ*/
        CALL TRIM@(GECODEERD_INSZ_RH_RSVZ(pers_j))
        CALL TRIM@(GECODEERD_INSZ_BT_RSVZ(pers_j))
        IF (INT(INRSVZ_KB(pers_j)).EQ.9999999)                  INRSVZ_KB(pers_j)=0
        CALL TRIM@(QUART_RSVZKB_2008(pers_j))
        !/*FOD SZ*/
        CALL TRIM@(CODE_LEG(pers_j))
        IF (INT(PILIER_UN_CONV(pers_j)).EQ.9999999)             PILIER_UN_CONV(pers_j)=0
        IF (INT(PILIER_DEUX(pers_j)).EQ.9999999)                PILIER_DEUX(pers_j)=0
        IF (INT(PILIER_TROIS(pers_j)).EQ.9999999)               PILIER_TROIS(pers_j)=0
        IF (INT(PUNT_ZELFVKB(pers_j)).EQ.9999999)               PUNT_ZELFVKB(pers_j)=0
        IF (INT(INFODSZ(pers_j)).EQ.9999999)                    INFODSZ(pers_j)=0
        !/*HISTORINFO*/
        IF (INT(LABMSTAT_PRIV(pers_j)).EQ.9999999)              LABMSTAT_PRIV(pers_j)=0
        IF (INT(LABMSTAT_PUB(pers_j)).EQ.9999999)               LABMSTAT_PUB(pers_j)=0
        !Variables read from file FAMAL_CONTRIB.txt
        IF (INT(CONTRIB_SSPRIV_EMEE_QT(pers_j)).EQ.9999999)     CONTRIB_SSPRIV_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSPUB_EMEE_QT(pers_j)).EQ.9999999)      CONTRIB_SSPUB_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSSELF_EMER_QT(pers_j)).EQ.9999999)     CONTRIB_SSSELF_EMER_QT(pers_j)=0
        IF (INT(CONTRIB_REVPRIV_QT(pers_j)).EQ.9999999)         CONTRIB_REVPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_REVPUB_QT(pers_j)).EQ.9999999)          CONTRIB_REVPUB_QT(pers_j)=0.
        IF (INT(CONTRIB_INCSELF_QT(pers_j)).EQ.9999999)         CONTRIB_INCSELF_QT(pers_j)=0
        !Variables read from file FAMAL_UNEM.txt               
        IF (INT(UNEM_TOTBEN_QT(pers_j)).EQ.9999999)             UNEM_TOTBEN_QT(pers_j)=0
        IF (UNEM_FULL6MON_QT(pers_j).EQ.9999999)                UNEM_FULL6MON_QT(pers_j)=0
        !Variables read from file FAMAL_SICK.txt               
        IF (INT(SICK_TOTBEN_QT(pers_j)).EQ.9999999)             SICK_TOTBEN_QT(pers_j)=0
        !Variables read from file FAMAL_PENSWELF.txt               
        IF (INT(PENSWELF_AMOUNTP_QT(pers_j)).EQ.9999999)        PENSWELF_AMOUNTP_QT(pers_j)=0
        IF (INT(PENSWELF_SURPENS_QT(pers_j)).EQ.9999999)        PENSWELF_SURPENS_QT(pers_j)=0
        !Variables read from file FAMAL_SOCBEN.txt                                                    
        IF (INT(SOCBEN_MINIMEX_QT(pers_j)).EQ.9999999)          SOCBEN_MINIMEX_QT(pers_j)=0
        !Variables read from file MIMOSIS_FAMREL.txt                                         
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)           SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)         MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)           MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)             MIMOSIS_WEIGHT(pers_j)=0
    END DO   

END SUBROUTINE FAMAL_RECODE

!============= SUBROUTINE FAMAL_INFLATE==================================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE FAMAL_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file FAMAL_INPUT.txt                                            
        !Variables read from file FAMAL_CONTRIB.txt
        CONTRIB_SSPRIV_EMEE_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSPRIV_EMEE_QT(pers_j)            !Employee social security contributions paid for employee in the private labour market           
        CONTRIB_SSPUB_EMEE_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_SSPUB_EMEE_QT(pers_j)             !Employee social security contributions paid for employee in the public labour market            
        CONTRIB_SSSELF_EMER_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSSELF_EMER_QT(pers_j)            !Social security contributions net of deductions paid by self employed           
        CONTRIB_REVPRIV_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_REVPRIV_QT(pers_j)                !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the private labour market          
        CONTRIB_REVPUB_QT(pers_j)       = INFLATE_NOMVAL(1)*CONTRIB_REVPUB_QT(pers_j)                 !Sum of gross labour income, holiday earnings and other supplements, earned as wage earner on the public labour market           
        CONTRIB_INCSELF_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_INCSELF_QT(pers_j)                !Labour income, earned as self employed          
        !Variables read from file FAMAL_UNEM.txt
        UNEM_TOTBEN_QT(pers_j)          = INFLATE_NOMVAL(3)*UNEM_TOTBEN_QT(pers_j)                    !Gross amount of unemployment benefits           
        !Variables read from file FAMAL_SICK.txt     
        SICK_TOTBEN_QT(pers_j)          = INFLATE_NOMVAL(3)*SICK_TOTBEN_QT(pers_j)                    !Sum of benefits for sickness, disability, industrial accidents and occupational diseases            
        !Variables read from file FAMAL_PENSWELF.txt
!        PENSWELF_AMOUNTP_QT(pers_j)     = INFLATE_NOMVAL(2)*PENSWELF_AMOUNTP_QT(pers_j)               !The gross amount of all pension benefits, paid because of legal Belgian pension laws            
!        PENSWELF_SURPENS_QT(pers_j)     = INFLATE_NOMVAL(2)*PENSWELF_SURPENS_QT(pers_j)               !The gross amount of all survival pension benefits, paid because of legal Belgian pension laws           
        !Variables read from file FAMAL_SOCBEN.txt                                                    
        SOCBEN_MINIMEX_QT(pers_j)       = INFLATE_NOMVAL(3)*SOCBEN_MINIMEX_QT(pers_j) 
        !Variables read from file MIMOSIS_FAMREL.txt                                           
        !Variables read from file WAGE_CONSTRUCT.txt                                                     
        SALJRS_LAST_CORR(pers_j)        = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)                  !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)      = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)                !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)        = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)                  !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Variables read from file MIMOSIS_WEIGHT.txt                                                    
    END DO   

END SUBROUTINE FAMAL_INFLATE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   FAMAL_HH_SIZE                               !
!   FAMAL_START_HH                              !
!   FAMAL_END_HH                                !
!   N_HH_FAMAL                                  !
!===============================================!

SUBROUTINE FAMAL_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        FAMAL_HH_SIZE(k)=n
        FAMAL_START_HH(k)=j
        FAMAL_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_FAMAL=K

END SUBROUTINE FAMAL_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: FAMAL_YEAR_AGE     !
!                         FAMAL_MONTH_AGE    !
!                         FAMAL_YEAR_BIRTH   !
!                         FAMAL_MONTH_BIRTH  !   
!============================================!

SUBROUTINE FAMAL_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        FAMAL_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:FAMAL_PAR_BREAK_POINT(1)-1),  '(I10)') FAMAL_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(FAMAL_PAR_BREAK_POINT(1)+1:),   '(I10)') FAMAL_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        FAMAL_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-FAMAL_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        FAMAL_MONTH_AGE(pers_j) =FAMAL_YEAR_AGE(pers_j)*12+(12-FAMAL_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE FAMAL_AGE

!===================================================================================!
!Draw a line of a give number of characters
!===================================================================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!===================================================================================!
!With subroutine FAMAL_CHARAC we reconstruct variables 
!used troughout the different subroutines for computing family allowances
!Constructed variables
!   RH_RKW_INPUT
!   BT_RKW_INPUT
!   RH_RSVZ_INPUT
!   BT_RSVZ_INPUT
!   FAMAL_BENEFID_QT
!   FAMAL_RECIPID_QT
!   INDISBEN_QT
!   INDISREC_QT
!   FAMAL_BENSCHEM_QT
!   FAMAL_BENSCMAX_QT
!   FAMAL_HANDICAP_ANN
!   FAMAL_LABMINC_QT
!   FAMAL_STUDENT_ANN
!   FAMAL_BENSCHIL_QT
!   FAMAL_STARTPAY_ANN
!   FAMAL_ENDPAY_ANN
!   FAMAL_CHILELIG_QT
!   FAMAL_INDINC_QT
!   FAMAL_NUMDEPCH_QT
!   FAMAL_BENGUARI_QT
!   FAMAL_NMONTHBE_QT
!   FAMAL_WIDOWER_QT
!   FAMAL_ORPHAN_QT
!   FAMAL_NEWBORN_QT
!   FAMAL_NUMRECIP_QT
!   FAMAL_NUMNEWB_QT
!   FAMAL_BIRTRANK_QT
!   FAMAL_ELIGCHHH_QT
!   FAMAL_YOUNGEST_QT
!   FAMAL_SINGCHIL_QT 
!===================================================================================!

SUBROUTINE FAMAL_CHARAC

    INTEGER                             :: pers_j,pers_k,pers_l,i,lijn_j
    INTEGER                             :: COUNT_BT,COUNT_RH,COUNT_BTNA,COUNT_RHNA,COUNT_BTHEADEX,COUNT_RHHEADEX
    REAL(KIND=KIND_REAL)                :: POPU_BT,POPU_RH,POPU_BTNA,POPU_RHNA,POPU_BTHEADEX,POPU_RHHEADEX
    INTEGER                             :: COUNT_BENEF,COUNT_RECIP
    REAL(KIND=KIND_REAL)                :: POPU_BENEF,POPU_RECIP
    INTEGER,DIMENSION(5)                :: COUNT_BENSCHEM
    REAL(KIND=KIND_REAL),DIMENSION(5)   :: POPU_BENSCHEM
    INTEGER                             :: COUNT_STUDENT,COUNT_ELIG,COUNT_GUAR,COUNT_NEWB,COUNT_MULTB,TESYOUNG
    REAL(KIND=KIND_REAL)                :: POPU_STUDENT,POPU_ELIG,POPU_GUAR,POPU_NEWB,POPU_MULTB
    REAL(KIND=KIND_REAL)                :: INCSUM_QT,INCLIM_QT
    INTEGER                             :: DEPCHILD_QT
    INTEGER,DIMENSION(2)                :: COUNT_WIDO,COUNT_ORPH,COUNT_BIRR,COUNT_SING
    REAL(KIND=KIND_REAL),DIMENSION(2)   :: POPU_WIDO,POPU_ORPH,POPU_BIRR,POPU_SING
    INTEGER,DIMENSION(9)                :: COUNT_CBENR
    REAL(KIND=KIND_REAL),DIMENSION(9)   :: POPU_CBENR
    INTEGER                             :: THIRDEGR,TESTGUAR,MAXSCHEM,BTRH_KNOWN
    REAL(KIND=KIND_REAL)                :: MINIMEX_QT   
    INTEGER,ALLOCATABLE,DIMENSION(:)    :: SUBAGE_QT    !subvector with age of all household members
    INTEGER,ALLOCATABLE,DIMENSION(:)    :: SUBELIG_QT   !subvector with indicator that says whether individual is eligible for child benefits
    INTEGER,ALLOCATABLE,DIMENSION(:)    :: SUBORPH_QT   !subvector with indicator that says whether individual is orphan or not      
    INTEGER,ALLOCATABLE,DIMENSION(:)    :: SUBRANK_QT   !subvector with rank of individual
    INTEGER,ALLOCATABLE,DIMENSION(:)    :: SUBYOUNG_QT  !subvector with identification of the youngest child in the household eligible for child benefits
    
    !Read in the numeric part of the individual indentification character variable
    !RH_RKW_INPUT: Numeric part of the anonymised identification number of the individual that grants the right on child benefits (the qualifying child) in the system of salaried workers         
    !BT_RKW_INPUT: Numeric part of the anonymised identification number of the individual that receives the child benefits (the beneficiary) in the system of salaried workers         
    !RH_RSVZ_INPUT: Numeric part of the anonymised identification number of the individual that grants the right on child benefits (the qualifying child) in the system of self employed people         
    !BT_RSVZ_INPUT: Numeric part of the anonymised identification number of the individual that receives the child benefits (the beneficiary) in the system of self employed people         
    DO pers_j=1,n_rec
        IF (gecodeerd_insz_rh(pers_j).NE.'niet in stkprf') THEN
            READ (gecodeerd_insz_rh(pers_j)(8:), '(I10)') RH_RKW_INPUT(pers_j)
        END IF
        IF (gecodeerd_insz_bt(pers_j).NE.'niet in stkprf') THEN
            READ (gecodeerd_insz_bt(pers_j)(8:), '(I10)') BT_RKW_INPUT(pers_j)
        END IF
        IF (gecodeerd_insz_rh_RSVZ(pers_j).NE.'niet in stkprf') THEN
            READ (gecodeerd_insz_rh_RSVZ(pers_j)(8:), '(I10)') RH_RSVZ_INPUT(pers_j)
        END IF
        IF (gecodeerd_insz_bt_RSVZ(pers_j).NE.'niet in stkprf') THEN
            READ (gecodeerd_insz_bt_RSVZ(pers_j)(8:), '(I10)') BT_RSVZ_INPUT(pers_j)
        END IF
    END DO

    !FAMAL_BENEFID_QT: Individual identification number of the beneficiary of child benefits of the beneficiary child          
    !First loop over all households to use the exogenous info
    COUNT_RH = 0
    POPU_RH = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if person j is not head of the household
            IF (NAREGNIS_RELATION(pers_j).NE.1) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !check whether this individual is not partner of the head of the household
                    !and not possibly entitled to continued child benefits
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                    &((FAMREL_RELATION(pers_k).EQ.1.AND.FAMREL_RELATION(pers_j).EQ.2).OR.&               
                    &(FAMREL_RELATION(pers_k).EQ.2.AND.FAMREL_RELATION(pers_j).EQ.1)).AND.&
                    &(ANY(code_leg(pers_j).NE.(/'03', '04', '05'/))).AND.&                     
                    &(ANY(code_leg(pers_k).NE.(/'03', '04', '05'/)))) THEN
                        !skip this individual
                    !otherwise identify the beneficiary
                    ELSE
                        !first use the RKW identification variable
!Old condition
!                        IF (FAMAL_YEAR_AGE(pers_j).LT.26.AND.FAMAL_BENEFID_QT(pers_j).EQ.0.AND.&
!                        &RH_RKW_INPUT(pers_j).NE.9999999.AND.RH_RKW_INPUT(pers_j).NE.7777777.AND.RH_RKW_INPUT(pers_j).NE.0) THEN
!Aangepast bij herwerking september 2011
!If known in files, use the value, independent of the age condition
                        IF (FAMAL_BENEFID_QT(pers_j).EQ.0.AND.&
                        &RH_RKW_INPUT(pers_j).NE.9999999.AND.RH_RKW_INPUT(pers_j).NE.7777777.AND.RH_RKW_INPUT(pers_j).NE.0) THEN
                            FAMAL_BENEFID_QT(pers_j) = RH_RKW_INPUT(pers_j)
                            FAMAL_RHBTKNOW(pers_j) = 1
                            COUNT_RH = COUNT_RH + 1
                            POPU_RH = POPU_RH + MIMOSIS_WEIGHT(pers_j)
                        END IF                            
                        !if no RKW identification variable then use the RSVZ identification
!Old condition
!                        IF (FAMAL_YEAR_AGE(pers_j).LT.26.AND.FAMAL_BENEFID_QT(pers_j).EQ.0.AND.&
!                        &RH_RSVZ_INPUT(pers_j).NE.9999999.AND.RH_RSVZ_INPUT(pers_j).NE.7777777.AND.RH_RSVZ_INPUT(pers_j).NE.0) THEN
!Aangepast bij herwerking september 2011
!If known in files, use the value, independent of the age condition
                        IF (FAMAL_BENEFID_QT(pers_j).EQ.0.AND.&
                        &RH_RSVZ_INPUT(pers_j).NE.9999999.AND.RH_RSVZ_INPUT(pers_j).NE.7777777.AND.RH_RSVZ_INPUT(pers_j).NE.0) THEN
                            FAMAL_BENEFID_QT(pers_j) = RH_RSVZ_INPUT(pers_j)
                            FAMAL_RHBTKNOW(pers_j) = 2
                            COUNT_RH = COUNT_RH + 1
                            POPU_RH = POPU_RH + MIMOSIS_WEIGHT(pers_j)
                        END IF                            
                    END IF
                END DO
            END IF
        END DO        
    END DO  

    !Loop over all households to check whether the identified beneficiary is part of the household or not
    HHMEMB_QT(1:n_rec) = 0    
    COUNT_RHNA = 0
    POPU_RHNA = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if a beneficiary has been identified for person j
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                !check whether there is a household member with the existing identification number
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        HHMEMB_QT(pers_j) = 1 
                        EXIT
                    END IF
                END DO
                !If no such individual is present then switch the beneficiary again to 0
                IF (HHMEMB_QT(pers_j).EQ.0.AND.FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                    FAMAL_BENEFID_QT(pers_j) = 0
                    FAMAL_RHBTKNOW(pers_j) = 0
                    COUNT_RHNA = COUNT_RHNA + 1
                    POPU_RHNA = POPU_RHNA + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO        

    !Loop again over all households to make the head of the household the beneficiary of all remaining individuals
    COUNT_RHHEADEX = 0
    POPU_RHHEADEX = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual satisfies the age condition or is handicaped with "likely" right on child benefits 
            !and no beneficiary has been identified yet
            !identify the person who is most likely to be the beneficiary (the value can be switched back to zero conditional
            !on the eligibility conditions of the child
!Old condition
!            IF (FAMAL_BENEFID_QT(pers_j).EQ.0.AND.(FAMAL_YEAR_AGE(pers_j).LT.26.OR.ANY(code_leg(pers_j).EQ.(/'01', '02'/)))) THEN
!Aangepast bij herwerking september 2011
            IF (FAMAL_BENEFID_QT(pers_j).EQ.0.AND.(FAMAL_YEAR_AGE(pers_j).LT.26.OR.InRKW(pers_j).EQ.1.OR.InRSVZ_KB(pers_j).EQ.1))&
            & THEN
                !search the head of the household
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!Aangepast bij herwerking september 2011
                IF (NAREGNIS_RELATION(pers_k).EQ.1.OR.NAREGNIS_RELATION(pers_k).EQ.20) THEN
                        FAMAL_BENEFID_QT(pers_j) = GECODEERD_INSZ_INPUT(pers_k)
                        FAMAL_RHBTKNOW(pers_j) = 3
                        COUNT_RHHEADEX = COUNT_RHHEADEX + 1
                        POPU_RHHEADEX = POPU_RHHEADEX + MIMOSIS_WEIGHT(pers_j)
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !FAMAL_RECIPID_QT: Individual identification number of the recipient of child benefits of the beneficiary child            
    !First loop over all households to use the exogenous info
    COUNT_BT = 0
    POPU_BT = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if person j is not head of the household
            IF (NAREGNIS_RELATION(pers_j).NE.1) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !check whether this individual is not partner of the head of the household
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                    &((FAMREL_RELATION(pers_k).EQ.1.AND.FAMREL_RELATION(pers_j).EQ.2).OR.&               
                    &(FAMREL_RELATION(pers_k).EQ.2.AND.FAMREL_RELATION(pers_j).EQ.1)).AND.&
                    &(ANY(code_leg(pers_j).NE.(/'03', '04', '05'/))).AND.&                     
                    &(ANY(code_leg(pers_k).NE.(/'03', '04', '05'/)))) THEN
                        !skip this individual
                    !otherwise check whether this individual satisfies the age condition and if yes identify the recipient
                    ELSE
                        !first use the RKW identification variable
!Old
!                        IF (FAMAL_YEAR_AGE(pers_j).LT.26.AND.FAMAL_RECIPID_QT(pers_j).EQ.0.AND.&
!                        &BT_RKW_INPUT(pers_j).NE.9999999.AND.BT_RKW_INPUT(pers_j).NE.7777777.AND.BT_RKW_INPUT(pers_j).NE.0) THEN
!Aangepast bij herwerking september 2011
                        IF (FAMAL_RECIPID_QT(pers_j).EQ.0.AND.&
                        &BT_RKW_INPUT(pers_j).NE.9999999.AND.BT_RKW_INPUT(pers_j).NE.7777777.AND.BT_RKW_INPUT(pers_j).NE.0) THEN
                            FAMAL_RECIPID_QT(pers_j) = BT_RKW_INPUT(pers_j)
                            FAMAL_RHBTKNOW(pers_j) = 4
                            COUNT_BT = COUNT_BT + 1
                            POPU_BT = POPU_BT + MIMOSIS_WEIGHT(pers_j)
                        END IF                            
                        !if no RKW identification variable then use the RSVZ identification
!Old
!                        IF (FAMAL_YEAR_AGE(pers_j).LT.26.AND.FAMAL_RECIPID_QT(pers_j).EQ.0.AND.&
!                        &BT_RSVZ_INPUT(pers_j).NE.9999999.AND.BT_RSVZ_INPUT(pers_j).NE.7777777.AND.BT_RSVZ_INPUT(pers_j).NE.0) THEN
!Aangepast bij herwerking september 2011
                        IF (FAMAL_RECIPID_QT(pers_j).EQ.0.AND.&
                        &BT_RSVZ_INPUT(pers_j).NE.9999999.AND.BT_RSVZ_INPUT(pers_j).NE.7777777.AND.BT_RSVZ_INPUT(pers_j).NE.0) THEN
                            FAMAL_RECIPID_QT(pers_j) = BT_RSVZ_INPUT(pers_j)
                            FAMAL_RHBTKNOW(pers_j) = 5
                            COUNT_BT = COUNT_BT + 1
                            POPU_BT = POPU_BT + MIMOSIS_WEIGHT(pers_j)
                        END IF                            
                    END IF
                END DO
            END IF
        END DO        
    END DO  

    !Loop over all households to check whether the identified recipient is part of the household or not
    HHMEMB_QT(1:n_rec) = 0    
    COUNT_BTNA = 0
    POPU_BTNA = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if a recipient has been identified for person j
            IF (FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                !check whether there is a household member with the existing identification number
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        HHMEMB_QT(pers_j) = 1 
                        EXIT
                    END IF
                END DO
                !If no such individual is present then switch the recipient again to 0
                IF (HHMEMB_QT(pers_j).EQ.0.AND.FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                    FAMAL_RECIPID_QT(pers_j) = 0
                    FAMAL_RHBTKNOW(pers_j) = 0
                    COUNT_BTNA = COUNT_BTNA + 1
                    POPU_BTNA = POPU_BTNA + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO        

    !Loop again over all households to make the head of the household or his partner the recipient of all remaining individuals
    COUNT_BTHEADEX = 0
    POPU_BTHEADEX = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual satisfies the age condition and no recipient has been identified yet
            IF (FAMAL_RECIPID_QT(pers_j).EQ.0.AND.FAMAL_YEAR_AGE(pers_j).LT.26) THEN
                !search the head of the household
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (NAREGNIS_RELATION(pers_k).EQ.1) THEN
                        !if the head of the household has no partner he is the recipient
                        IF (FAMREL_COUPLE(pers_k).EQ.0) THEN
                            FAMAL_RECIPID_QT(pers_j) = GECODEERD_INSZ_INPUT(pers_k)
                            FAMAL_RHBTKNOW(pers_j) = 6
                            COUNT_BTHEADEX = COUNT_BTHEADEX + 1
                            POPU_BTHEADEX = POPU_BTHEADEX + MIMOSIS_WEIGHT(pers_j)
                        !otherwise look for the partner of the head of the household
                        !Note: in the FAMREL module, partners are made female in case of mixed sexes
                        !in couples of equal sex, the youngest one of the two is the partner
                        ELSE
                            DO pers_l=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                                !either k is partner 1
                                IF (pers_k.NE.pers_l.AND.FAMREL_FAMILY(pers_k).EQ.FAMREL_FAMILY(pers_l).AND.&
                                &(FAMREL_RELATION(pers_k).EQ.1.AND.FAMREL_RELATION(pers_l).EQ.2)) THEN
                                    FAMAL_RECIPID_QT(pers_j) = GECODEERD_INSZ_INPUT(pers_l)
                                    FAMAL_RHBTKNOW(pers_j) = 6
                                    COUNT_BTHEADEX = COUNT_BTHEADEX + 1
                                    POPU_BTHEADEX = POPU_BTHEADEX + MIMOSIS_WEIGHT(pers_j)
                                    EXIT
                                !or l is partner 1    
                                ELSE IF (pers_k.NE.pers_l.AND.FAMREL_FAMILY(pers_k).EQ.FAMREL_FAMILY(pers_l).AND.&
                                &(FAMREL_RELATION(pers_k).EQ.2.AND.FAMREL_RELATION(pers_l).EQ.1)) THEN
                                    FAMAL_RECIPID_QT(pers_j) = GECODEERD_INSZ_INPUT(pers_k)
                                    FAMAL_RHBTKNOW(pers_j) = 6
                                    COUNT_BTHEADEX = COUNT_BTHEADEX + 1
                                    POPU_BTHEADEX = POPU_BTHEADEX + MIMOSIS_WEIGHT(pers_j)
                                    EXIT
                                END IF
                            END DO
                        END IF
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Individuals after creation of beneficiaries and recipients'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''Number of beneficiaries initialy identified                                        '', i10)') COUNT_RH
    Write(15,'(''Number of beneficiaries not present in the household                               '', i10)') COUNT_RHNA
    Write(15,'(''Number of beneficiaries for which the head of hh has been used to fill gaps        '', i10)') COUNT_RHHEADEX
    Write(15,'(''Total                                                                              '', i10)') &
    &(COUNT_RH-COUNT_RHNA)+COUNT_RHHEADEX
    Write(15,'(''Number of recipients initialy identified                                           '', i10)') COUNT_BT
    Write(15,'(''Number of recipients not present in the household                                  '', i10)') COUNT_BTNA
    Write(15,'(''Number of recipients for which the head of hh or his partner has been used to fill '', i10)') COUNT_BTHEADEX
    Write(15,'(''Total                                                                              '', i10)') &
    &(COUNT_BT-COUNT_BTNA)+COUNT_BTHEADEX
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''Number of beneficiaries initialy identified                                        '', f10.0)') POPU_RH
    Write(15,'(''Number of beneficiaries not present in the household                               '', f10.0)') POPU_RHNA
    Write(15,'(''Number of beneficiaries for which the head of hh has been used to fill gaps        '', f10.0)') POPU_RHHEADEX
    Write(15,'(''Total                                                                              '', f10.0)') &
    &(POPU_RH-POPU_RHNA)+POPU_RHHEADEX
    Write(15,'(''Number of recipients initialy identified                                           '', f10.0)') POPU_BT
    Write(15,'(''Number of recipients not present in the household                                  '', f10.0)') POPU_BTNA
    Write(15,'(''Number of recipients for which the head of hh or his partner has been used to fill '', f10.0)') POPU_BTHEADEX
    Write(15,'(''Total                                                                              '', f10.0)') &
    &(POPU_BT-POPU_BTNA)+POPU_BTHEADEX

    !INDISBEN_QT: Identifier saying that the individual is beneficiary in the given quarter (yes=1)
    !INDISREC_QT: Identifier saying that the individual is recipient in the given quarter (yes=1)
    !Loop over all households 
    COUNT_BENEF = 0
    COUNT_RECIP = 0
    POPU_BENEF = 0
    POPU_RECIP = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at the level of the child look up the line of the beneficiary itself
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        INDISBEN_QT(pers_k) = 1
                        EXIT
                    END IF
                END DO
            END IF
            !if an individual has been identified to be recipient at the level of the child look up the line of the recipient itself
            IF (FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        INDISREC_QT(pers_k) = 1
                        EXIT
                    END IF
                END DO
            END IF
        END DO
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (INDISBEN_QT(pers_j).EQ.1) THEN
                COUNT_BENEF = COUNT_BENEF + 1
                POPU_BENEF = POPU_BENEF + MIMOSIS_WEIGHT(pers_j)
            END IF
            IF (INDISREC_QT(pers_j).EQ.1) THEN
                COUNT_RECIP = COUNT_RECIP + 1
                POPU_RECIP = POPU_RECIP + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Beneficiaries and recipients'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''Number of beneficiaries                                                            '', i10)') COUNT_BENEF
    Write(15,'(''Number of recipients                                                               '', i10)') COUNT_RECIP
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''Number of beneficiaries                                                            '', f10.0)') POPU_BENEF
    Write(15,'(''Number of recipients                                                               '', f10.0)') POPU_RECIP

    !FAMAL_BENSCHEM_QT: Contribution scheme of beneficiary
    !1 individual contributed as wage earner
    !2 individual contributed as civil servant
    !3 individual contributed as self employed
    !Loop over all households 
    COUNT_BENSCHEM(1:5) = 0
    POPU_BENSCHEM(1:5) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!Aangepast bij herwerking september 2011
!            !remark: at this stage we fill in the contribution scheme of each observed individual to be able to compute the maximum number
!            !of contribution schemes per household later on
!            !First check to which sector the individual paid the most contributions
!            IF      (CONTRIB_SSPRIV_EMEE_QT(pers_j).GE.CONTRIB_SSPUB_EMEE_QT(pers_j).AND.&
!                    &CONTRIB_SSPRIV_EMEE_QT(pers_j).GE.CONTRIB_SSSELF_EMER_QT(pers_j).AND.&
!                    &CONTRIB_SSPRIV_EMEE_QT(pers_j).GT.0) THEN
!                        FAMAL_BENSCHEM_QT(pers_j) = 1
!            ELSE IF (CONTRIB_SSPUB_EMEE_QT(pers_j).GT.CONTRIB_SSPRIV_EMEE_QT(pers_j).AND.&
!                    &CONTRIB_SSPUB_EMEE_QT(pers_j).GT.CONTRIB_SSSELF_EMER_QT(pers_j).AND.&
!                    &CONTRIB_SSPUB_EMEE_QT(pers_j).GT.0) THEN
!                        IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
!                            FAMAL_BENSCHEM_QT(pers_j) = 1
!                        ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
!                            FAMAL_BENSCHEM_QT(pers_j) = 2
!                        END IF                               
!            ELSE IF (CONTRIB_SSSELF_EMER_QT(pers_j).GT.CONTRIB_SSPUB_EMEE_QT(pers_j).AND.&
!                    &CONTRIB_SSSELF_EMER_QT(pers_j).GT.CONTRIB_SSPRIV_EMEE_QT(pers_j).AND.&
!                    &CONTRIB_SSSELF_EMER_QT(pers_j).GT.0) THEN
!                        FAMAL_BENSCHEM_QT(pers_j) = 3
!            END IF
            !If no contributions have been paid, but labour market status allows identification, then use labour market status
            !Indien geval verkeerdelijk geklasseerd
            !First use status on the private labour market
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 1
                END IF
            END IF
            !Then use public labour market status 
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 1
                ELSE IF (ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 2
                END IF                               
            END IF
            !Then use self employment status 
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(CONTRIB_LABMSTAT_SELF_QT(pers_j).EQ.(/1/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 3
                END IF
            END IF
            !If contribution scheme has not been identified yet, but if there are replacement incomes then use the type of replacement income
            !First check whether the individual receives unemployment benefits
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (UNEM_TOTBEN_QT(pers_j).NE.0) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 1
                END IF
            END IF
            !Then check the type of insurance for sickness and disability benefits
            !First use the private labour market insurance type
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(SICK_TYPEINSU_PRIV_QT(pers_j).EQ.(/1, 2/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 1
                END IF
            END IF
            !Then use public labour market insurance type
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/1, 2/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 1
                ELSE IF (ANY(SICK_TYPEINSU_PUB_QT(pers_j).EQ.(/3, 4/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 2
                END IF                               
            END IF
            !Then use self employment insurance type
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(SICK_TYPEINSU_SELF_QT(pers_j).EQ.(/1/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = 3
                END IF
            END IF
            !Check the system that generates pension benefits
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
                IF (ANY(PENSWELF_BENSCHEM_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    FAMAL_BENSCHEM_QT(pers_j) = PENSWELF_BENSCHEM_QT(pers_j)
                END IF
            END IF
!Aangepast bij herwerking september 2011
!            !If after this no status has been defined, use historic labour market info (most dominant labour market position
!            !First use status on the private labour market
!            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
!                IF (ANY(LABMSTAT_PRIV(pers_j).EQ.(/1, 2, 5/))) THEN
!                    FAMAL_BENSCHEM_QT(pers_j) = 1
!                END IF
!            END IF
!            !Then use public labour market status 
!            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
!                IF (ANY(LABMSTAT_PUB(pers_j).EQ.(/1, 2/))) THEN
!                    FAMAL_BENSCHEM_QT(pers_j) = 1
!                ELSE IF (LABMSTAT_PUB(pers_j).GT.2) THEN
!                    FAMAL_BENSCHEM_QT(pers_j) = 2
!                END IF                               
!            END IF
!Aangepast bij herwerking september 2011
!            !If after this individuals are known in the RKW database but classfied as self employed, correct them
!            IF (InRKW(pers_j).EQ.1.AND.ANY(FAMAL_BENSCHEM_QT(pers_j).EQ.(/0, 3/))) THEN
!                FAMAL_BENSCHEM_QT(pers_j) = 2
!            END IF
!            !Count the identified contribution schemes of those identified as beneficiary
            IF (INDISBEN_QT(pers_j).EQ.1) THEN
                DO i=0,3
                    IF (FAMAL_BENSCHEM_QT(pers_j).EQ.i) THEN
                        COUNT_BENSCHEM(i+1) = COUNT_BENSCHEM(i+1) + 1
                        POPU_BENSCHEM(i+1) = POPU_BENSCHEM(i+1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Contribution scheme of the beneficiaries'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''contribution scheme was not identified yet                                         '', i10)')COUNT_BENSCHEM(1)
    Write(15,'(''individual contributed as wage earner                                              '', i10)')COUNT_BENSCHEM(2)
    Write(15,'(''individual contributed as civil servant                                            '', i10)')COUNT_BENSCHEM(3)
    Write(15,'(''individual contributed as self employed                                            '', i10)')COUNT_BENSCHEM(4)
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_BENSCHEM(1:4))
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''contribution scheme was not identified yet                                         '', f10.0)')POPU_BENSCHEM(1)
    Write(15,'(''individual contributed as wage earner                                              '', f10.0)')POPU_BENSCHEM(2)
    Write(15,'(''individual contributed as civil servant                                            '', f10.0)')POPU_BENSCHEM(3)
    Write(15,'(''individual contributed as self employed                                            '', f10.0)')POPU_BENSCHEM(4)
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_BENSCHEM(1:4))

    !FAMAL_BENSCMAX_QT: Variable containing the contribution scheme that is observed most frequently within the sociological household
    DO hh_i=1,N_HH_FAMAL
        COUNT_BENSCHEM(1:5) = 0
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (FAMAL_BENSCHEM_QT(pers_j).EQ.1) THEN
                COUNT_BENSCHEM(1) = COUNT_BENSCHEM(1) + 1
            ELSE IF (FAMAL_BENSCHEM_QT(pers_j).EQ.2) THEN
                COUNT_BENSCHEM(2) = COUNT_BENSCHEM(2) + 1
            ELSE IF (FAMAL_BENSCHEM_QT(pers_j).EQ.3) THEN
                COUNT_BENSCHEM(3) = COUNT_BENSCHEM(3) + 1
            END IF
        END DO
        MAXSCHEM = MAX(COUNT_BENSCHEM(1),COUNT_BENSCHEM(2),COUNT_BENSCHEM(3))
        !loop over counted schemes and determine the maximum
        DO i=1,3
            IF (COUNT_BENSCHEM(i).EQ.MAXSCHEM.AND.MAXSCHEM.NE.0.AND.FAMAL_BENSCMAX_QT(hh_i).EQ.0) THEN
                FAMAL_BENSCMAX_QT(hh_i) = i
            END IF
        END DO
    END DO

    !FAMAL_BENSCHEM_QT: Contribution scheme of beneficiary (continued)
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual is beneficiary and no contribution scheme has been identified yet, then
            !assume that the rights are opened in the system that appears most in the household
!            IF (INDISBEN_QT(pers_j).EQ.1.AND.FAMAL_BENSCHEM_QT(pers_j).EQ.0) THEN
!                FAMAL_BENSCHEM_QT(pers_j) = FAMAL_BENSCMAX_QT(hh_i)
!            END IF
            !if an individual is not a beneficiary, then switch the value of the beneficiary scheme to 0
            IF (INDISBEN_QT(pers_j).EQ.0) THEN
                FAMAL_BENSCHEM_QT(pers_j) = 0
            END IF
        END DO
    END DO        
    
!Aangepast bij herwerking september 2011
    !FAMAL_HANDICAP_ANN: The individual is handicapped (yes=1, no=0)
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (InFODSZ(pers_j).EQ.1) THEN
                FAMAL_HANDICAP_ANN(pers_j) = 1
            END IF                
        END DO
    END DO        

    !FAMAL_LABMINC_QT: Sum of all gross labour market income sources obtained by the individual            
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            FAMAL_LABMINC_QT(pers_j) = CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j) + CONTRIB_INCSELF_QT(pers_j)
        END DO
    END DO        

    !FAMAL_STUDENT_ANN: The individual is a student (yes=1, no=0)                       
    COUNT_STUDENT = 0
    POPU_STUDENT = 0
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !the individual must be 25 or younger and be right giving for child benefits
!Aangepast bij herwerking september 2011
            IF (FAMAL_YEAR_AGE(pers_j).LT.26.AND.(InRKW(pers_j).EQ.1.OR.InRSVZ_KB(pers_j).EQ.1)) THEN
                !unemployment benefits or labour market income must be below a predefined ceiling 
!Aangepast bij herwerking september 2011
!Ook gehandicapten verliezen dus studentenstatuut indien ze een inkomen hebben boven de vooropgestelde grens
!In het andere geval kunnen gehandicapten wel de gewone kinderbijslag blijven ontvangen na de leeftijd van 21
!Het handicapsupplement laten we later wel wegvallen vanaf 21
                IF ((UNEM_TOTBEN_QT(pers_j).GT.(FAMAL_CHILUNEL_MONTH*3.D0).OR.FAMAL_LABMINC_QT(pers_j).GT.&
                &(FAMAL_CHILINCL_MONTH*3.D0))) THEN
                    FAMAL_STUDENT_ANN(pers_j) = 0
                ELSE
                    FAMAL_STUDENT_ANN(pers_j) = 1
                END IF
            END IF
            !count the number of students
            IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                COUNT_STUDENT = COUNT_STUDENT + 1
                POPU_STUDENT = POPU_STUDENT + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Number of students'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count of number of students                                                        '', i10)')COUNT_STUDENT
    Write(15,'(''Weighted amount of the number of students                                          '', f10.0)')POPU_STUDENT

    !FAMAL_BENSCHIL_QT(pers_j): variable containing the contribution scheme of the beneficiary on the level of the beneficiary child
    !1 individual contributed as wage earner
    !2 individual contributed as civil servant
    !3 individual contributed as self employed
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at the level of the child look up the line of the beneficiary itself
            !and identify the contribution scheme of the beneficiary at the level of the child
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_BENSCHIL_QT(pers_j) = FAMAL_BENSCHEM_QT(pers_k)
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO

    !FAMAL_STARTPAY_ANN: First month of the year for which child receives child benefits (number from 1 to 12)           
    !FAMAL_ENDPAY_ANN: Last month of the year for which child receives child benefits (number from 1 to 12)            
    !Loop over all households
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        !First check all conditions to determine start and end of the payments without checking the income conditions (see Table 10 and Table 11 of the text)
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !follow the identification procedure for individuals that have a beneficiary that either contributed
            !in the a) wage earner scheme, b) self employment scheme or c) for which the contribution scheme was not identified
            IF (ANY(FAMAL_BENSCHIL_QT(pers_j).EQ.(/0, 1, 3/)).AND.FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                !differentiate first according to age
                IF (FAMAL_YEAR_AGE(pers_j).EQ.0) THEN
                    !do not distinguish between not handicapped or handicapped
                    IF (ANY(FAMAL_HANDICAP_ANN(pers_j).EQ.(/0, 1/))) THEN
                        !do not distinguish between no student or student
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = FAMAL_MONTH_BIRTH(pers_j) + 1 !remark month can become 13
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.0.AND.FAMAL_YEAR_AGE(pers_j).LT.18) THEN
                    !do not distinguish between not handicapped or handicapped
                    IF (ANY(FAMAL_HANDICAP_ANN(pers_j).EQ.(/0, 1/))) THEN
                        !do not distinguish between no student or student
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1 
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).EQ.18) THEN
                    !distinguish between not handicapped or handicapped
                    IF (FAMAL_HANDICAP_ANN(pers_j).EQ.0) THEN
                        !distinguish between no student or student
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 8
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    ELSE IF (FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN 
                        !do not distinguish between no student or student
                        !alle personen met een handicap ontvangen kinderbijslag tot het einde van het jaar
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1 
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.18.AND.FAMAL_YEAR_AGE(pers_j).LT.21) THEN
                    !distinguish between not handicapped or handicapped
                    IF (FAMAL_HANDICAP_ANN(pers_j).EQ.0) THEN
                        !distinguish between no student or student
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    ELSE IF (FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN 
                        !do not distinguish between no student or student
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1 
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).EQ.21) THEN
!Aangepast bij herwerking september 2011
                    !distinguish between not handicapped or handicapped
                    IF (FAMAL_HANDICAP_ANN(pers_j).EQ.0) THEN
                        !distinguish between no student or student
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    ELSE IF (FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN 
                        !distinguish between no student or student
                        !en geen recht op verlengde kinderbijslag
                        IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = 12
                        ELSE
                            IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = FAMAL_MONTH_BIRTH(pers_j) 
                            ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = 12
                            END IF
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.21.AND.FAMAL_YEAR_AGE(pers_j).LT.25) THEN
                    !distinguish between no student or student
                    !en geen recht op verlengde kinderbijslag
                    IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                    ELSE
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).EQ.25) THEN
                    !distinguish between no student or student
                    !en geen recht op verlengde kinderbijslag
                    IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                    ELSE
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = FAMAL_MONTH_BIRTH(pers_j)
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.25.AND.FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN
                        FAMAL_STARTPAY_ANN(pers_j)  = 1
                        FAMAL_ENDPAY_ANN(pers_j)    = 12
                END IF
            END IF
            !follow the identification procedure for individuals that have a beneficiary that either contributed
            !in the a) civil servant scheme
            IF (ANY(FAMAL_BENSCHIL_QT(pers_j).EQ.(/2/)).AND.FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                !differentiate first according to age
                IF (FAMAL_YEAR_AGE(pers_j).EQ.0) THEN
                    !do not distinguish between not handicapped or handicapped
                    IF (ANY(FAMAL_HANDICAP_ANN(pers_j).EQ.(/0, 1/))) THEN
                        !do not distinguish between no student or student
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = FAMAL_MONTH_BIRTH(pers_j) + 1 !remark month can become 13
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.0.AND.FAMAL_YEAR_AGE(pers_j).LT.21) THEN
                    !do not distinguish between not handicapped or handicapped
                    IF (ANY(FAMAL_HANDICAP_ANN(pers_j).EQ.(/0, 1/))) THEN
                        !do not distinguish between no student or student
                        IF (ANY(FAMAL_STUDENT_ANN(pers_j).EQ.(/0, 1/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1 
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
                ELSE IF (FAMAL_YEAR_AGE(pers_j).EQ.21) THEN
!Aangepast bij herwerking september 2011
                    !distinguish between not handicapped or handicapped
                    IF (FAMAL_HANDICAP_ANN(pers_j).EQ.0) THEN
                        !distinguish between no student or student
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = FAMAL_MONTH_BIRTH(pers_j) 
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    ELSE IF (FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN 
                        !distinguish between no student or student
                        !en geen recht op verlengde kinderbijslag
                        IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = 12
                        ELSE
                            IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = FAMAL_MONTH_BIRTH(pers_j) 
                            ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                                FAMAL_STARTPAY_ANN(pers_j)  = 1
                                FAMAL_ENDPAY_ANN(pers_j)    = 12
                            END IF
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.21.AND.FAMAL_YEAR_AGE(pers_j).LT.25) THEN
                    !distinguish between no student or student
                    !en geen recht op verlengde kinderbijslag
                    IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                    ELSE
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).EQ.25) THEN
                    !distinguish between no student or student
                    !en geen recht op verlengde kinderbijslag
                    IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = 12
                    ELSE
                        IF (FAMAL_STUDENT_ANN(pers_j).EQ.0) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 0
                            FAMAL_ENDPAY_ANN(pers_j)    = 0
                        ELSE IF (FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
                            FAMAL_STARTPAY_ANN(pers_j)  = 1
                            FAMAL_ENDPAY_ANN(pers_j)    = FAMAL_MONTH_BIRTH(pers_j)
                        END IF
                    END IF
!Aangepast bij herwerking september 2011
                ELSE IF (FAMAL_YEAR_AGE(pers_j).GT.25.AND.FAMAL_HANDICAP_ANN(pers_j).EQ.1) THEN
                        FAMAL_STARTPAY_ANN(pers_j)  = 1
                        FAMAL_ENDPAY_ANN(pers_j)    = 12
                END IF
            END IF
        END DO
        !Loop over all persons within the household and check the necessary income conditions
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child satisfies the necessary age condtion, i.e. a beneificiary has been identified
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                !unemployment benefits or labour market income must be below a predefined ceiling and
                !person not handicaped
!Aangepast bij herwerking september 2011
                IF ((UNEM_TOTBEN_QT(pers_j).GT.(FAMAL_CHILUNEL_MONTH*3.D0).OR.FAMAL_LABMINC_QT(pers_j).GT.&
                &(FAMAL_CHILINCL_MONTH*3.D0)).AND.InFodSZ(pers_j).EQ.0) THEN
                    FAMAL_STARTPAY_ANN(pers_j) = 0
                    FAMAL_ENDPAY_ANN(pers_j) = 0
                END IF
            END IF
        END DO
    END DO

    !FAMAL_CHILELIG_QT: Individual is eligible for child benefits (yes=1, no=0)         
    COUNT_ELIG = 0
    POPU_ELIG = 0
    !Loop over all households
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household and check the necessary income conditions
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child satisfies the necessary age condtion, i.e. a beneificiary has been identified
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                !check whether the starting and ending month of payment are situated within the quarter of analysis
!Aangepast bij herwerking september 2011
!                IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                    IF ( ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3/)).AND.&
                        &ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN
                            FAMAL_CHILELIG_QT(pers_j) = 1
                    END IF
!                ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                    IF ( ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6/)).AND.&
                        &ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN
                            FAMAL_CHILELIG_QT(pers_j) = 1
                    END IF
!                ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                    IF ( ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 6, 9/)).AND.&
                        &ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/7, 6, 9, 10, 11, 12/))) THEN
                            FAMAL_CHILELIG_QT(pers_j) = 1
                    END IF
!                ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                    IF ( ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/)).AND.&
                        &ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/10, 11, 12/))) THEN
                            FAMAL_CHILELIG_QT(pers_j) = 1
                    END IF
!                END IF    
            END IF
            !count the number of children eligible for child benefits
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                COUNT_ELIG = COUNT_ELIG + 1
                POPU_ELIG = POPU_ELIG + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''Number of children eligible for child benefits'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count of number of children eligible for child benefits                            '', i10)')COUNT_ELIG
    Write(15,'(''Weighted amount of the number of children eligible for child benefits              '', f10.0)')POPU_ELIG

    !FAMAL_INDINC_QT: Sum of all individual income components (including labour market income and replacement income but no social benefits)          
    !Loop over all households 
!Aangepast bij herwerking september 2011
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            FAMAL_INDINC_QT(pers_j) =   CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j) + CONTRIB_INCSELF_QT(pers_j) + &
                                        &PENSWELF_AMOUNTP_QT(pers_j) + SICK_TOTBEN_QT(pers_j) + UNEM_TOTBEN_QT(pers_j) + &
                                        &SOCBEN_MINIMEX_QT(pers_j)
        END DO
    END DO        

    !FAMAL_NUMDEPCH_QT: Number of dependent children of the beneficiary         
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be a beneficiary at the level of the child 
            !and the child is eligible for child benefits then look up the line of the beneficiary itself and 1 to the variable
            IF (FAMAL_BENEFID_QT(pers_j).NE.0.AND.FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_NUMDEPCH_QT(pers_k) = FAMAL_NUMDEPCH_QT(pers_k) + 1
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !FAMAL_BENGUARI_QT: The beneficiary satisfies the conditions to receive guaranteed child allowances (yes=1, no=0)           
    !Loop over all households 
    COUNT_GUAR = 0
    POPU_GUAR = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at its own level
            IF (INDISBEN_QT(pers_j).EQ.1) THEN
                !initialise the intermediate income variable
                INCSUM_QT = 0
                DEPCHILD_QT = 0
                INCLIM_QT = 0
                BTRH_KNOWN = 0
                MINIMEX_QT = 0
                !check whether either the beneficiary or the recipient is known within the RKW or RSVZ source
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.&
                    &ANY(FAMAL_RHBTKNOW(pers_k).EQ.(/4/))) THEN
                        BTRH_KNOWN = 1
                        EXIT
                    END IF
                    IF (FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.&
                    &ANY(FAMAL_RHBTKNOW(pers_k).EQ.(/4/))) THEN
                        BTRH_KNOWN = 1
                        EXIT
                    END IF
                END DO
                !check whether this individual has a partner or not
                IF (FAMREL_COUPLE(pers_j).EQ.0) THEN
                    INCSUM_QT = FAMAL_INDINC_QT(pers_j)
                    DEPCHILD_QT = FAMAL_NUMDEPCH_QT(pers_j)
                    MINIMEX_QT = SOCBEN_MINIMEX_QT(pers_j)
                !otherwise look for the partner of the head of the household
                !Note: in the FAMREL module, partners are made female in case of mixed sexes
                !in couples of equal sex, the youngest one of the two is the partner
                ELSE
                    DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                        !either j is partner 1
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2)) THEN
                            INCSUM_QT = FAMAL_INDINC_QT(pers_j) + FAMAL_INDINC_QT(pers_k)
                            DEPCHILD_QT = FAMAL_NUMDEPCH_QT(pers_j) + FAMAL_NUMDEPCH_QT(pers_k)
                            MINIMEX_QT = SOCBEN_MINIMEX_QT(pers_j) 
                        !or k is partner 1    
                        ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                            INCSUM_QT = FAMAL_INDINC_QT(pers_j) + FAMAL_INDINC_QT(pers_k)
                            DEPCHILD_QT = FAMAL_NUMDEPCH_QT(pers_j) + FAMAL_NUMDEPCH_QT(pers_k)
                            MINIMEX_QT = SOCBEN_MINIMEX_QT(pers_j) 
                        END IF
                    END DO
                END IF
!Old rule for attributing guaranteed benefit                
!                !then compare the real income level with the relevant income ceiling 
!                IF ((DEPCHILD_QT-1).GT.0) THEN
!                    INCLIM_QT = FAMAL_GUARINAB_QT + (FAMAL_GUARINPC_QT/100.D0)*FAMAL_GUARINAB_QT*(DBLE(DEPCHILD_QT)-1.D0)
!                    IF (INCSUM_QT.GE.FAMAL_INCLIMSP_MONTH.AND.INCSUM_QT.LT.INCLIM_QT) THEN
!                        FAMAL_BENGUARI_QT(pers_j) = 1
!                    END IF
!                ELSE
!                    IF (INCSUM_QT.GE.FAMAL_INCLIMSP_MONTH.AND.INCSUM_QT.LT.FAMAL_GUARINAB_QT) THEN
!                        FAMAL_BENGUARI_QT(pers_j) = 1
!                    END IF
!                END IF
!            END IF
                !then compare the real income level with the relevant income ceiling 
                IF ((DEPCHILD_QT-1).GT.0) THEN
                    INCLIM_QT = FAMAL_GUARINAB_QT + (FAMAL_GUARINPC_QT/100.D0)*FAMAL_GUARINAB_QT*(DBLE(DEPCHILD_QT)-1.D0)
                    IF (MINIMEX_QT.GT.0.D0.AND.MINIMEX_QT.EQ.INCSUM_QT.AND.BTRH_KNOWN.EQ.1.AND.INCSUM_QT.LT.INCLIM_QT) THEN
                        FAMAL_BENGUARI_QT(pers_j) = 1
                    END IF
                ELSE
                    IF (MINIMEX_QT.GT.0.D0.AND.MINIMEX_QT.EQ.INCSUM_QT.AND.BTRH_KNOWN.EQ.1.AND.INCSUM_QT.LT.FAMAL_GUARINAB_QT) THEN
                        FAMAL_BENGUARI_QT(pers_j) = 1
                    END IF
                END IF
            END IF
            !count the number of beneficiaries eligible for guaranteed child allowances
            IF (FAMAL_BENGUARI_QT(pers_j).EQ.1) THEN
                COUNT_GUAR = COUNT_GUAR + 1
                POPU_GUAR = POPU_GUAR + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of eligibility for guaranteed child allowances
    Write(15,'('' '')')
    Write(15,'(''Number of beneficiaries eligible for guaranteed child allowances'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of beneficiaries eligible for guaranteed child allowances             '', i10)')COUNT_GUAR
    Write(15,'(''Weighted amount of the number of beneficiaries eligible for guaranteed child allowa'', f10.0)')POPU_GUAR

    !Continue with construction of variables constructed before
    !FAMAL_CHILELIG_QT: Individual is eligible for child benefits (yes=1, no=0)         
    !FAMAL_STARTPAY_ANN: First month of the year for which child receives child benefits (number from 1 to 12)           
    !FAMAL_ENDPAY_ANN: Last month of the year for which child receives child benefits (number from 1 to 12)            
    !FAMAL_BENSCHIL_QT(pers_j): variable containing the contribution scheme of the beneficiary on the level of the beneficiary child
    !Loop over all households 
    COUNT_BENSCHEM(1:5) = 0
    POPU_BENSCHEM(1:5) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at the level of the child look up the line of the beneficiary itself
            !and check whether this beneficiary is eligible for guaranteed child allowances
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        !Attribute some children to the self employed scheme
                        IF (FAMAL_BENSCHEM_QT(pers_k).EQ.0.AND.ANY(FAMAL_RHBTKNOW(pers_j).EQ.(/2, 5/))) THEN
                            FAMAL_BENSCHEM_QT(pers_k) = 3
                            FAMAL_BENSCHIL_QT(pers_j) = 3
                        END IF    
!Aangepast bij herwerking september 2011
                        IF (FAMAL_BENSCHEM_QT(pers_k).EQ.0.AND.InRSVZ_KB(pers_j).EQ.1) THEN
                            FAMAL_BENSCHEM_QT(pers_k) = 3
                            FAMAL_BENSCHIL_QT(pers_j) = 3
                        END IF    
                        !if the beneficiary did not contribute but is eligible for guaranteed allowances then create an extra value 
                        IF (FAMAL_BENSCHEM_QT(pers_k).EQ.0.AND.FAMAL_BENGUARI_QT(pers_k).EQ.1) THEN
                            FAMAL_BENSCHIL_QT(pers_j) = 4
                        END IF    
                        !If after all this no scheme has been filled in for those who are beneficiary
                        !then assume civil servant
!Aangepast bij herwerking september 2011
                        IF (FAMAL_BENSCHEM_QT(pers_k).EQ.0.AND.FAMAL_BENGUARI_QT(pers_k).EQ.0) THEN
                            FAMAL_BENSCHEM_QT(pers_k) = 2
                            FAMAL_BENSCHIL_QT(pers_j) = 2
                        END IF    
                        !If after this the scheme of the beneficiary would not be 0 but the scheme of the
                        !child is not identified yet then fill in this gap
                        IF (FAMAL_BENSCHEM_QT(pers_k).NE.0.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.0) THEN
                            FAMAL_BENSCHIL_QT(pers_j) = FAMAL_BENSCHEM_QT(pers_k)
                        END IF
                        EXIT
                    END IF
                END DO
            END IF
!Aangepast bij herwerking september 2011
            !Use appearance in RKW or RSVZ file right giving child as identifier to correct constructed values
            IF (InRKW(pers_j).EQ.0.AND.InRSVZ_KB(pers_j).EQ.0.AND.FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                !If Civil servant it still likely that the individual was not included in the RKW or RSVZ files
                IF (FAMAL_BENSCHIL_QT(pers_j).NE.2.AND.InFODSZ(pers_j).EQ.0) THEN
                    FAMAL_BENSCHIL_QT(pers_j) = 2
                END IF                    
            END IF
            !Count the identified contribution schemes of the beneficiaries at the level of the children
            IF (FAMAL_BENEFID_QT(pers_j).NE.0.AND.FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                DO i=0,4
                    IF (FAMAL_BENSCHIL_QT(pers_j).EQ.i) THEN
                        COUNT_BENSCHEM(i+1) = COUNT_BENSCHEM(i+1) + 1
                        POPU_BENSCHEM(i+1) = POPU_BENSCHEM(i+1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''The ontribution scheme of the beneficiary children eligible for child benefits'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''contribution scheme was not identified yet                                         '', i10)')COUNT_BENSCHEM(1)
    Write(15,'(''beneficiary contributed as wage earner                                             '', i10)')COUNT_BENSCHEM(2)
    Write(15,'(''beneficiary contributed as civil servant                                           '', i10)')COUNT_BENSCHEM(3)
    Write(15,'(''beneficiary contributed as self employed                                           '', i10)')COUNT_BENSCHEM(4)
    Write(15,'(''beneficiary did not contribute but was eligible for guaranteed allo                '', i10)')COUNT_BENSCHEM(5)
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_BENSCHEM(1:5))
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''contribution scheme was not identified yet                                         '', f10.0)')POPU_BENSCHEM(1)
    Write(15,'(''beneficiary contributed as wage earner                                             '', f10.0)')POPU_BENSCHEM(2)
    Write(15,'(''beneficiary contributed as civil servant                                           '', f10.0)')POPU_BENSCHEM(3)
    Write(15,'(''beneficiary contributed as self employed                                           '', f10.0)')POPU_BENSCHEM(4)
    Write(15,'(''beneficiary did not contribute but was eligible for guaranteed allo                '', f10.0)')POPU_BENSCHEM(5)
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_BENSCHEM(1:5))

    !FAMAL_NMONTHBE_QT: Number of months for which benefits are paid during the quarter of analysis for the eligible child          
    !Loop over all households
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household and check the necessary income conditions
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child satisfies the necessary age condtion, i.e. the child is eligible for child benefits
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                !check how many months of payment are situated within the quarter of analysis
!Aangepast bij herwerking september 2011
!                IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                    !if starting month of payments is within the quarter of analysis
                    IF (ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3/))) THEN
                        !if the ending month of payments in the quarter of analysis or later
                        IF (ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN
                            FAMAL_NMONTHBE_QT(pers_j) = MIN(FAMAL_ENDPAY_ANN(pers_j) - FAMAL_STARTPAY_ANN(pers_j) + 1,12)
                        END IF    
!                    ELSE
!                        FAMAL_NMONTHBE_QT(pers_j) = 0   
                    END IF
!                ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                    !if starting month of payments is within the quarter of analysis or earlier
                    IF (ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6/))) THEN
                        !if the ending month of payments in the quarter of analysis or later
                        IF (ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN
                            FAMAL_NMONTHBE_QT(pers_j) = MIN(FAMAL_ENDPAY_ANN(pers_j) - FAMAL_STARTPAY_ANN(pers_j) + 1,12)
                        END IF    
!                    ELSE
!                        FAMAL_NMONTHBE_QT(pers_j) = 0   
                    END IF
!                ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                    !if starting month of payments is within the quarter of analysis or earlier
                    IF (ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9/))) THEN
                        !if the ending month of payments in the quarter of analysis or later
                        IF (ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/7, 8, 9, 10, 11, 12/))) THEN
                            FAMAL_NMONTHBE_QT(pers_j) = MIN(FAMAL_ENDPAY_ANN(pers_j) - FAMAL_STARTPAY_ANN(pers_j) + 1,12)
                        END IF    
!                    ELSE
!                        FAMAL_NMONTHBE_QT(pers_j) = 0   
                    END IF
!                ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                    !if starting month of payments is within the quarter of analysis or earlier
                    IF (ANY(FAMAL_STARTPAY_ANN(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN
                        !if the ending month of payments in the quarter of analysis or later
                        IF (ANY(FAMAL_ENDPAY_ANN(pers_j).EQ.(/10, 11, 12/))) THEN
                            FAMAL_NMONTHBE_QT(pers_j) = MIN(FAMAL_ENDPAY_ANN(pers_j) - FAMAL_STARTPAY_ANN(pers_j) + 1,12)
                        END IF    
!                    ELSE
!                        FAMAL_NMONTHBE_QT(pers_j) = 0   
                    END IF
!                END IF    
            END IF
!            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_NMONTHBE_QT(pers_j).LT.3) THEN
!                WRITE(15,*) 'A',FAMAL_BENSCHIL_QT(pers_j),FAMAL_STARTPAY_ANN(pers_j),FAMAL_NMONTHBE_QT(pers_j)
!            END IF
        END DO
    END DO

    !FAMAL_WIDOWER_QT: Individual is considered to be a widow or widower (yes=1, no=0)         
    !Loop over all households
    COUNT_WIDO(1:2) = 0
    POPU_WIDO(1:2) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household and check the necessary income conditions
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the individual is widower according to National Register data
            IF (NAREGNIS_CIVIL(pers_j).EQ.3) THEN
                FAMAL_WIDOWER_QT(pers_j) = 1
            END IF
            !if the individual is widower according to pension data of Pensioenkadaster
            IF (PENSWELF_SURPENS_QT(pers_j).GT.0.AND.FAMAL_WIDOWER_QT(pers_j).EQ.0) THEN
                FAMAL_WIDOWER_QT(pers_j) = 1
            END IF
            !count the number of widowers in the sample
            IF (FAMAL_WIDOWER_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    COUNT_WIDO(1) = COUNT_WIDO(1) + 1
                    POPU_WIDO(1) = POPU_WIDO(1) + MIMOSIS_WEIGHT(pers_j) 
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    COUNT_WIDO(2) = COUNT_WIDO(2) + 1
                    POPU_WIDO(2) = POPU_WIDO(2) + MIMOSIS_WEIGHT(pers_j) 
                END IF
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of eligibility for guaranteed child allowances
    Write(15,'('' '')')
    Write(15,'(''Number of widows and widowers in the sample'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of widowers                                                           '', i10)')COUNT_WIDO(1) 
    Write(15,'(''Count number of widows                                                             '', i10)')COUNT_WIDO(2) 
    Write(15,'(''Weighted amount of the number of widowers                                          '', f10.0)')POPU_WIDO(1)
    Write(15,'(''Weighted amount of the number of widows                                            '', f10.0)')POPU_WIDO(2)

    !FAMAL_ORPHAN_QT: Individual is considered to be an orphan
    !1 an orphan who is not its own beneficiary
    !2 an orphan who is its own beneficiary
    !Loop over all households 
    COUNT_ORPH(1:2) = 0
    POPU_ORPH(1:2) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at its own level and if this beneficiary is a widow or widower
            IF (INDISBEN_QT(pers_j).EQ.1.AND.FAMAL_WIDOWER_QT(pers_j).EQ.1) THEN
                !check whether this individual has a partner or not (legally married or de facto)
                IF (FAMREL_COUPLE(pers_j).EQ.0) THEN
                    THIRDEGR = 0
                    !If there is no partner check whether there is a household member that is not related to the third degree
                    !First determine the line in the parameter FAMAL_FAMREL_QT to convert relationship codes
                    DO i = 1,SIZE(FAMAL_FAMREL_QT,1)
                        IF (FAMREL_RELATION(pers_j).EQ.FAMAL_FAMREL_QT(i,1)) THEN
                            lijn_j = i
                        END IF
                    END DO
                    !loop over all household members and check their relationship with individual j
                    DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                        !first check the relationship with members of the same family
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                            DO i = 1,SIZE(FAMAL_FAMREL_QT,2)
                                IF (FAMREL_RELATION(pers_k).EQ.FAMAL_FAMREL_QT(1,i)) THEN
                                    !If there is a family member that is not related to the third degree
                                    IF (FAMAL_FAMREL_QT(lijn_j,i).EQ.6) THEN
                                        THIRDEGR = 1
                                    END IF
                                END IF
                            END DO
                        !members of other families     
                        ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).NE.FAMREL_FAMILY(pers_k)) THEN
                            !if either j or k is living a collective household
                            IF (FAMREL_FAMTYPE(pers_j).EQ.8.OR.FAMREL_FAMTYPE(pers_k).EQ.8) THEN
                                THIRDEGR = 1
                            END IF
                            !if either j or k is not related to the third degree with its own head, j and k can not be related to the third degree neither
                            IF (FAMREL_RELATION(pers_j).EQ.6.OR.FAMREL_RELATION(pers_k).EQ.6) THEN
                                THIRDEGR = 1
                            END IF
                        END IF
                    END DO
                    !If the non married widower does not cohabitate with a person that is not related to the third degree
                    !make all dependent children orphans
                    IF (THIRDEGR.EQ.0) THEN
                        !look up all children for which the widower is beneficiary   
                        DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                            IF (GECODEERD_INSZ_INPUT(pers_j).EQ.FAMAL_BENEFID_QT(pers_k)) THEN
                                FAMAL_ORPHAN_QT(pers_k) = 1
                            END IF
                        END DO                            
                    END IF
                END IF
            END IF            
        END DO
        !Loop again over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !count the number of orphans in the sample
            IF (FAMAL_ORPHAN_QT(pers_j).EQ.1) THEN
                IF (NAREGNIS_SEX(pers_j).EQ.1) THEN
                    COUNT_ORPH(1) = COUNT_ORPH(1) + 1
                    POPU_ORPH(1) = POPU_ORPH(1) + MIMOSIS_WEIGHT(pers_j) 
                ELSE IF (NAREGNIS_SEX(pers_j).EQ.2) THEN
                    COUNT_ORPH(2) = COUNT_ORPH(2) + 1
                    POPU_ORPH(2) = POPU_ORPH(2) + MIMOSIS_WEIGHT(pers_j) 
                END IF
            END IF
            !if the orphan is its own beneficiary then idicate this by setting the ORPHAN variable equal to 2
            !remark: our way of identifying orphans (i.e. through a cohabitating widower) should make it almost impossible
            !to observe such cases (not checked)
            IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_ORPHAN_QT(pers_j).EQ.1) THEN
                FAMAL_ORPHAN_QT(pers_j) = 2
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of eligibility for guaranteed child allowances
    Write(15,'('' '')')
    Write(15,'(''Number of orphans in the sample'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of orphan boys                                                        '', i10)')COUNT_ORPH(1) 
    Write(15,'(''Count number of orphan girls                                                       '', i10)')COUNT_ORPH(2) 
    Write(15,'(''Weighted amount of the number of orphan boys                                       '', f10.0)')POPU_ORPH(1)
    Write(15,'(''Weighted amount of the number of orphan girls                                      '', f10.0)')POPU_ORPH(2)

    !FAMAL_NEWBORN_QT: Indicator saying that the child is a newborn child (yes=1, no=0)
    !Loop over all households 
    COUNT_NEWB = 0
    POPU_NEWB = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !check whether the child is newborn, i.e. number of months at the beginning of the last month
            !of the quarter of analysis is 3 at maximum
            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                IF (ANY((FAMAL_MONTH_AGE(pers_j)-10).EQ.(/1, 2, 3/))) THEN
                    FAMAL_NEWBORN_QT(pers_j) = 1
                END IF
            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                IF (ANY((FAMAL_MONTH_AGE(pers_j)-7).EQ.(/1, 2, 3/))) THEN
                    FAMAL_NEWBORN_QT(pers_j) = 1
                END IF
            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                IF (ANY((FAMAL_MONTH_AGE(pers_j)-4).EQ.(/1, 2, 3/))) THEN
                    FAMAL_NEWBORN_QT(pers_j) = 1
                END IF
            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                IF (ANY((FAMAL_MONTH_AGE(pers_j)-1).EQ.(/1, 2, 3/))) THEN
                    FAMAL_NEWBORN_QT(pers_j) = 1
                END IF
            END IF    
            !count the number of newborn children
            IF (FAMAL_NEWBORN_QT(pers_j).EQ.1) THEN
                COUNT_NEWB = COUNT_NEWB + 1
                POPU_NEWB = POPU_NEWB + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of newborn children
    Write(15,'('' '')')
    Write(15,'(''Number of newborn children'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of newborn children                                                   '', i10)')COUNT_NEWB
    Write(15,'(''Weighted amount of newborn children                                                '', f10.0)')POPU_NEWB

    !FAMAL_NUMRECIP_QT: Number of dependent children of the recipient         
    !FAMAL_NUMNEWB_QT: Number of newborn children, depending on the recipient, and eligible for child benefits     
    !Loop over all households 
    COUNT_MULTB = 0
    POPU_MULTB = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be a recipient at the level of the child 
            !then look up the line of the recipient itself and add 1 to the variable (hence both eligible as non eligible children are considered here)
            IF (FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_NUMRECIP_QT(pers_k) = FAMAL_NUMRECIP_QT(pers_k) + 1
                        !if a child is newborn and eligible for child benefits then count the individual to check multiple births later
                        IF (FAMAL_NEWBORN_QT(pers_j).EQ.1.AND.FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                            FAMAL_NUMNEWB_QT(pers_k) = FAMAL_NUMNEWB_QT(pers_k) + 1                      
                        END IF
                        EXIT
                    END IF
                END DO
            END IF
        END DO
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !count the number of multiple births on the level of the recipient
            IF (FAMAL_NUMNEWB_QT(pers_j).GT.1) THEN
                COUNT_MULTB = COUNT_MULTB + 1
                POPU_MULTB = POPU_MULTB + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of multiple births
    Write(15,'('' '')')
    Write(15,'(''Number of mulitple births of children eligible for child benefits (counted on the level of the recipient)'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of multiple births                                                    '', i10)')COUNT_MULTB 
    Write(15,'(''Weighted amount of multiple births                                                 '', f10.0)')POPU_MULTB

    !FAMAL_BIRTRANK_QT: Birth rank of the eligible child
    !1 if the newborn child is a first born child or part of a multiple birth
    !2 if the newborn child is not a first born and not part of a multiple birth
    COUNT_BIRR(1:2) = 0
    POPU_BIRR(1:2) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !check whether the child is a newborn child and eligible for child benefits
            IF (FAMAL_NEWBORN_QT(pers_j).EQ.1.AND.FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                !check the properties of the recipient of this newborn child
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        IF (FAMAL_NUMRECIP_QT(pers_k).EQ.1.OR.FAMAL_NUMNEWB_QT(pers_k).GT.1) THEN
                            FAMAL_BIRTRANK_QT(pers_j) = 1                            
                        ELSE
                            FAMAL_BIRTRANK_QT(pers_j) = 2                            
                        END IF
                        EXIT
                    END IF
                END DO
            END IF
            !count the number of newborn children of rank 1 and rank 2
            IF (FAMAL_BIRTRANK_QT(pers_j).GT.0) THEN
                DO i=1,2
                    IF (FAMAL_BIRTRANK_QT(pers_j).EQ.i) THEN
                        COUNT_BIRR(i) = COUNT_BIRR(i) + 1
                        POPU_BIRR(i) = POPU_BIRR(i) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END DO
            END IF
        END DO
    END DO

    !Write some test results
    !Write results of construction birth ranks of newborn children
    Write(15,'('' '')')
    Write(15,'(''Ranks of newborn children'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of newborn children of rank 1                                         '', i10)')COUNT_BIRR(1) 
    Write(15,'(''Count number of newborn children of rank 2                                         '', i10)')COUNT_BIRR(2) 
    Write(15,'(''Weighted amount of the number of newborn children of rank 1                        '', f10.0)')POPU_BIRR(1)
    Write(15,'(''Weighted amount of the number of newborn children of rank 2                        '', f10.0)')POPU_BIRR(2)

    !FAMAL_ELIGCHHH_QT: Variable containing the sum of all eligible children within the household
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                FAMAL_ELIGCHHH_QT(hh_i) = FAMAL_ELIGCHHH_QT(hh_i) + 1
            END IF
        END DO
    END DO
    
    !FAMAL_CBENRANK_QT: Rank of the eligible child to determine the ordinary child benefits         
    !FAMAL_YOUNGEST_QT: The eligible child is the youngest child of all eligible children in the sociological household         
    !Loop over all households 
    COUNT_CBENR(1:9) = 0
    POPU_CBENR(1:9) = 0
    DO hh_i=1,N_HH_FAMAL
!Aangepast
!        IF (hh_i.EQ.61) THEN
!            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!                WRITE(15,*) 'B',FAMAL_ELIGCHHH_QT(hh_i),pers_j,hh_i,FAMAL_CBENRANK_QT(pers_j),&
!                &FAMAL_BENSCHIL_QT(pers_j),NAREGNIS_RELATION(pers_j),FAMAL_CHILELIG_QT(pers_j),FAMAL_YEAR_AGE(pers_j)
!            END DO
!        END IF
        !If there is a child in the household, eligible for child benefits then determine the birth rank
        IF (FAMAL_ELIGCHHH_QT(hh_i).GT.0) THEN
            !allocate vectors with the necessary household information
            ALLOCATE(SUBAGE_QT(FAMAL_HH_SIZE(hh_i)))
            ALLOCATE(SUBELIG_QT(FAMAL_HH_SIZE(hh_i)))
            ALLOCATE(SUBORPH_QT(FAMAL_HH_SIZE(hh_i)))
            ALLOCATE(SUBRANK_QT(FAMAL_HH_SIZE(hh_i)))
            ALLOCATE(SUBYOUNG_QT(FAMAL_HH_SIZE(hh_i)))
            !Loop over all persons within the household and attribute the appropriate values to the subvectors
            i = 1
            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                SUBAGE_QT(i)   = FAMAL_YEAR_AGE(pers_j)
                SUBELIG_QT(i)  = FAMAL_CHILELIG_QT(pers_j)
                SUBORPH_QT(i)  = FAMAL_ORPHAN_QT(pers_j)
                SUBRANK_QT(i)  = 0
                SUBYOUNG_QT(i) = 0
                i = i + 1
            END DO
            !Call a subroutine that determines the rank of each child and whether the child is the youngest eligible child 
            !conditional on its age, whether the child is eligible and whether the child is an orphan or not
            CALL FAMAL_RANKCHIL(SUBAGE_QT,SUBELIG_QT,SUBORPH_QT,FAMAL_HH_SIZE(hh_i),SUBRANK_QT,SUBYOUNG_QT)
            !Loop over all persons within the household and attribute the appropriate rank values to the vector on sample size
            i = 1
            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                FAMAL_CBENRANK_QT(pers_j) = SUBRANK_QT(i)
                FAMAL_YOUNGEST_QT(pers_j) = SUBYOUNG_QT(i)
                i = i + 1
            END DO
            !count some control statistics
            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                !count the children by rank
                IF (FAMAL_CHILELIG_QT(pers_j).GT.0) THEN
                    DO i=0,7
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.i) THEN
                            COUNT_CBENR(i+1) = COUNT_CBENR(i+1) + 1
                            POPU_CBENR(i+1) = POPU_CBENR(i+1) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GT.7) THEN
                        COUNT_CBENR(9) = COUNT_CBENR(9) + 1
                        POPU_CBENR(9) = POPU_CBENR(9) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END IF
            END DO
            !deallocate vectors with the necessary household information
            DEALLOCATE(SUBAGE_QT)
            DEALLOCATE(SUBELIG_QT)
            DEALLOCATE(SUBORPH_QT)
            DEALLOCATE(SUBRANK_QT)
            DEALLOCATE(SUBYOUNG_QT)
        END IF
    END DO

    !Write some test results
    !Write results of construction of ranks of children
    Write(15,'('' '')')
    Write(15,'(''Ranks of children'')')
    CALL DRAWLINE(15,136,'-') 
    DO i=1,9
        Write(15,'(''Count number of children of rank                                         '', 2i10)')i,COUNT_CBENR(i) 
    END DO
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_CBENR(1:9))
    DO i=1,9
        Write(15,'(''Weighted amount of children of rank                                      '', i10,f10.0)')i,POPU_CBENR(i) 
    END DO
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_CBENR(1:9))

    !FAMAL_SINGCHIL_QT: The eligible child is the only eligible child in the sociological household         
    !Loop over all households 
    COUNT_SING = 0
    POPU_SING = 0
    DO hh_i=1,N_HH_FAMAL
        !If there is only one child in the household, eligible for child benefits then identify this at the level of the child
        IF (FAMAL_ELIGCHHH_QT(hh_i).EQ.1) THEN
            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                    FAMAL_SINGCHIL_QT(pers_j) = 1
                    COUNT_SING = COUNT_SING + 1
                    POPU_SING = POPU_SING + MIMOSIS_WEIGHT(pers_j) 
                END IF
            END DO
        END IF
    END DO       

    !Write some test results
    !Write results of construction of single children
    Write(15,'('' '')')
    Write(15,'(''Number of single children'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of single children                                                    '', i10)')COUNT_SING 
    Write(15,'(''Weighted amount of single children                                                '', f10.0)')POPU_SING

    !apply a test
    !in each household with an eligible child, one child must be indicated as the youngest
    DO hh_i=1,N_HH_FAMAL
        !If there is a child in the household, eligible for child benefits then check whether the variable youngest has been completed
        IF (FAMAL_ELIGCHHH_QT(hh_i).GT.0) THEN
            TESYOUNG = 0
            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                IF (FAMAL_YOUNGEST_QT(pers_j).EQ.1) THEN
                    TESYOUNG = 1
                    EXIT
                END IF
            END DO
            IF (TESYOUNG.EQ.0) THEN
                write(15,'(''Household '', i10,'' contains eligible children but none of them has been indicated &
                &as the youngest one'')')hh_i
            END IF
        END IF
    END DO

!    !Test whether a household member has been identified as beneficiary of guaranteed child allowances
!    !if yes, print some test statistics of all household members
!    DO hh_i=1,N_HH_FAMAL
!        TESTGUAR = 0
!        !Loop over all persons within the household
!        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!            IF (FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
!                TESTGUAR = 1
!            END IF
!        END DO
!        !if there is a beneficiary for guaranteed child allowances check whether another beneficiary could have been chosen
!        IF (TESTGUAR.EQ.1) THEN
!            write(15,'('' '')')
!            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!                Write(15,*) hh_i,pers_j,NAREGNIS_RELATION(pers_j),FAMAL_YEAR_AGE(pers_j),FAMAL_CHILELIG_QT(pers_j),&
!                &FAMAL_INDINC_QT(pers_j),FAMAL_LABMINC_QT(pers_j),FAMAL_BENSCHEM_QT(pers_j),FAMAL_BENGUARI_QT(pers_j)
!            END DO
!        END IF
!    END DO

!    !Test whether all household members without income source
!    !if yes, print some test statistics of all household members
!    DO hh_i=1,N_HH_FAMAL
!        TESTGUAR = 0
!        !Loop over all persons within the household
!        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!            IF (FAMAL_INDINC_QT(pers_j).NE.0) THEN
!                TESTGUAR = 1
!            END IF
!        END DO
!        !if there is a beneficiary for guaranteed child allowances check whether another beneficiary could have been chosen
!        IF (TESTGUAR.EQ.0) THEN
!            DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!                Write(15,'(A17)') GECODEERD_INSZ_CHAR(pers_j)
!            END DO
!        END IF
!    END DO

END SUBROUTINE FAMAL_CHARAC

!========================================!
!Subroutine to compute rank of each child
!========================================!

SUBROUTINE FAMAL_RANKCHIL(AGE,ELIG,ORPHAN,VecSize,RANK_FIN,YOUNGEST)

! In this subroutine we calculate the rank of each child that is eligible for child benefits
! The output consists of the VecSize integer vector RANK_FIN with the rank-numbers

    INTEGER                                                 :: i
    INTEGER                                                 :: VecSize                              !VecSize: size of the vectors x and w                                                                                                       - input
    INTEGER                                                 :: RANKVAL                              !Value of the rank                                                                                                                          - local
    INTEGER                                                 :: YOUNGVAL                             !Individual has the age of the youngest                                                                                                                          - local
    INTEGER,DIMENSION(VecSize)                              :: IRANK                                !Vector containing the rank of an observation, created to sort other variables later on                                                     - local
    INTEGER,DIMENSION(VecSize)                              :: RANK_FIN                             !RANK_FIN (VecSize):vector with the rank numbers                                                                                            - output
    INTEGER,DIMENSION(VecSize)                              :: YOUNGEST                             !YOUNGEST (VecSize):vector with identifier idicating which individual is the youngest child that is still eligible for family allowances    - output
    INTEGER,DIMENSION(VecSize)                              :: OrigPos                              !Vector that contains the original position of an observation, used to resort later on                                                      - local
    INTEGER,DIMENSION(VecSize)                              :: AGE,ELIG,ORPHAN                      !Vectors of size VecSize with the age, eligibility and orphan indicator                                                                     - input

    !DETERMINE THE ORIGINAL POSITION OF EACH INDIVIDUAL
    DO i=1,VecSize
        OrigPos(i) = i
    END DO

    !RANK AND SORT INPUT VECTORS ACCORDING TO AGE
    CALL RANKING(AGE,IRANK)                                  ! INDEX VECTOR AGE IN IRANK
    CALL SORTING(AGE,IRANK)                                  ! SORT VECTOR AGE ON THE INDEX IRANK
    CALL SORTING(ELIG,IRANK)                                 ! SORT VECTOR ELIG ON THE INDEX IRANK
    CALL SORTING(ORPHAN,IRANK)                               ! SORT VECTOR ORPHAN ON THE INDEX IRANK
    CALL SORTING(OrigPos,IRANK)                              ! SORT VECTOR OrigPos ON THE INDEX IRANK

    !LOOP OVER ALL HOUSEHOLD MEMBERS FROM OLD TO YOUNG TO DETERMINE RANK
    RANKVAL = 1
    DO i=VecSize,1,-1
        !the individual should be eligible for child benefits and not an orphan who is its own beneficiary
        IF (ELIG(i).EQ.1.AND.ORPHAN(i).NE.2) THEN
            RANK_FIN(i) = RANKVAL
            !Only increase rank if ages of two consecutive individuals differ
            !if the individual analysed is the youngest then no age is available for i-1
            IF (i.NE.1) THEN
                IF (AGE(i).NE.AGE(i-1)) THEN
                    RANKVAL = RANKVAL + 1
                END IF
            END IF
            !We do not limit rank here, but only in computation of benefits
            !!rank can not become larger than 3
            !IF (RANKVAL.GT.3) THEN
            !    RANKVAL = 3
            !END IF
        !if the child is an eligible orphan and his own beneficiary we give it rank 1
        ELSE IF (ELIG(i).EQ.1.AND.ORPHAN(i).EQ.2) THEN
            RANK_FIN(i) = 1
        END IF
    END DO

    !LOOP OVER ALL HOUSEHOLD MEMBERS FROM YOUNG TO OLD TO IDENTIFY THE YOUNGEST ONE
    YOUNGVAL = 0
    DO i=1,VecSize
        !the individual should be eligible for child benefits and have the age of the youngest eligible child
        IF (ELIG(i).EQ.1.AND.YOUNGVAL.EQ.0) THEN
            YOUNGEST(i) = 1
            !Only switch YOUNGVAL to 1  if ages of two consecutive individuals differ
            !if the individual analysed is the youngest then no age is available for i-1
            IF (i.NE.VecSize) THEN
                IF (AGE(i).NE.AGE(i+1)) THEN
                    YOUNGVAL = 1
                END IF
            END IF
        END IF
    END DO

!Write some test statistics to check production of rank
!     DO i=1,VecSize
!        write(15,*) AGE(i),ELIG(i),ORPHAN(i),RANK_FIN(i),YOUNGEST(i)
!    END DO
    
    !RANK AND SORT RANK NUMBER TO ORIGINAL POSITION OF THE INDIVIDUAL
    CALL RANKING(OrigPos,IRANK)                              ! INDEX VECTOR OrigPos IN IRANK
    CALL SORTING(RANK_FIN,IRANK)                             ! SORT VECTOR RANK_FIN ON THE INDEX IRANK
    CALL SORTING(YOUNGEST,IRANK)                             ! SORT VECTOR YOUNGEST ON THE INDEX IRANK

END SUBROUTINE FAMAL_RANKCHIL

!===================================================================================!
!With subroutine FAMAL_BASALLOW we compute the basic allowances
!Constructed variables
!FAMAL_BIRTHALL_QT
!FAMAL_ADOPALL_QT
!FAMAL_ORALLWEA_QT
!FAMAL_ORALLCIV_QT
!FAMAL_ORALLSEL_QT
!FAMAL_ORALLGUA_QT
!FAMAL_ORALLORP_QT
!===================================================================================!

SUBROUTINE FAMAL_BASALLOW

    REAL(KIND=KIND_REAL)                :: MONTH_AMOUNT
    
    !FAMAL_BIRTHALL_QT: Amount of birth allowances in Euro paid for an eligible child during the quarter of analysis            
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BIRTRANK_QT(pers_j).GT.0) THEN
                IF (FAMAL_BIRTRANK_QT(pers_j).EQ.1) THEN
                    FAMAL_BIRTHALL_QT(pers_j) = FAMAL_BALLAMOU_ANN(1)
                ELSE IF (FAMAL_BIRTRANK_QT(pers_j).EQ.2) THEN
                    FAMAL_BIRTHALL_QT(pers_j) = FAMAL_BALLAMOU_ANN(2)
                END IF
            END IF
        END DO
    END DO       

    !FAMAL_ADOPALL_QT: Amount of adoption allowances in Euro paid for an eligible child during the quarter of analysis         
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            FAMAL_ADOPALL_QT(pers_j) = 0
        END DO
    END DO       

    !FAMAL_ORALLWEA_QT: Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the wage earner scheme            
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            MONTH_AMOUNT = 0
            !check whether a child is eligible for child benefits and not an orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).EQ.0) THEN
                !if the beneficiary of this child generated benefits in the wage earner scheme, then proceed
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.1) THEN
                    !Determine the monthly benefit conditional on the rank of the child
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.1) THEN
                        MONTH_AMOUNT = FAMAL_OWEAAMOU_MONTH(1)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).EQ.2) THEN
                        MONTH_AMOUNT = FAMAL_OWEAAMOU_MONTH(2)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).GT.2) THEN
                        MONTH_AMOUNT = FAMAL_OWEAAMOU_MONTH(3)
                    END IF
                    !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                    FAMAL_ORALLWEA_QT(pers_j) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
                END IF
            END IF                
        END DO
    END DO       

    !FAMAL_ORALLCIV_QT: Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the civil servant scheme          
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            MONTH_AMOUNT = 0
            !check whether a child is eligible for child benefits and not an orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).EQ.0) THEN
                !if the beneficiary of this child generated benefits in the civil servant scheme, then proceed
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
                    !Determine the monthly benefit conditional on the rank of the child
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.1) THEN
                        MONTH_AMOUNT = FAMAL_OCIVAMOU_MONTH(1)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).EQ.2) THEN
                        MONTH_AMOUNT = FAMAL_OCIVAMOU_MONTH(2)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).GT.2) THEN
                        MONTH_AMOUNT = FAMAL_OCIVAMOU_MONTH(3)
                    END IF
                    !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                    FAMAL_ORALLCIV_QT(pers_j) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
                END IF
            END IF                
        END DO
    END DO       

    !FAMAL_ORALLSEL_QT: Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the self employment scheme            
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            MONTH_AMOUNT = 0
            !check whether a child is eligible for child benefits and not an orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).EQ.0) THEN
                !if the beneficiary of this child generated benefits in the self employment scheme, then proceed
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.3) THEN
                    !Determine the monthly benefit conditional on the rank of the child
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.1) THEN
                        MONTH_AMOUNT = FAMAL_OSELAMOU_MONTH(1)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).EQ.2) THEN
                        MONTH_AMOUNT = FAMAL_OSELAMOU_MONTH(2)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).GT.2) THEN
                        MONTH_AMOUNT = FAMAL_OSELAMOU_MONTH(3)
                    END IF
                    !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                    FAMAL_ORALLSEL_QT(pers_j) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
                END IF
            END IF                
        END DO
    END DO       

    !FAMAL_ORALLGUA_QT: Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the guaranteed child allowance scheme         
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            MONTH_AMOUNT = 0
            !check whether a child is eligible for child benefits and not an orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).EQ.0) THEN
                !if the beneficiary of this child generated benefits in the guaranteed child allowance scheme, then proceed
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
                    !Determine the monthly benefit conditional on the rank of the child
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.1) THEN
                        MONTH_AMOUNT = FAMAL_OGUAAMOU_MONTH(1)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).EQ.2) THEN
                        MONTH_AMOUNT = FAMAL_OGUAAMOU_MONTH(2)
                    ELSE IF (FAMAL_CBENRANK_QT(pers_j).GT.2) THEN
                        MONTH_AMOUNT = FAMAL_OGUAAMOU_MONTH(3)
                    END IF
                    !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                    FAMAL_ORALLGUA_QT(pers_j) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
                END IF
            END IF                
        END DO
    END DO       

    !FAMAL_ORALLORP_QT: Amount of ordinary child allowances in Euro paid for an eligible child during the quarter of analysis if the beneficiary child is an orphan         
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !check whether a child is eligible for child benefits and an orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).GT.0) THEN
                !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                FAMAL_ORALLORP_QT(pers_j) = FAMAL_OORPAMOU_MONTH*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
            END IF   
!Aangepast
!            IF (FAMAL_ORPHAN_QT(pers_j).GT.0.AND.FAMAL_CHILELIG_QT(pers_j).EQ.0.AND.FAMAL_STUDENT_ANN(pers_j).EQ.1) THEN
!                WRITE(15,*) 'Test',pers_j,FAMAL_CBENRANK_QT(pers_j),FAMAL_YEAR_AGE(pers_j),&
!                &InRKW(pers_j),InRSVZ_KB(pers_j),MIMOSIS_WEIGHT(pers_j),InFODSZ(pers_j),&
!                &FAMAL_STARTPAY_ANN(pers_j),FAMAL_ENDPAY_ANN(pers_j),PENSWELF_BENSCHEM_QT(pers_j)
!            END IF
        END DO
    END DO       

END SUBROUTINE FAMAL_BASALLOW

!===================================================================================!
!With subroutine FAMAL_SUPALLOW we compute the family allowance supplements
!Constructed variables
!FAMAL_ASUPWEA_QT 
!FAMAL_ASUPSEL_QT 
!FAMAL_REPIBEN_QT 
!FAMAL_REPIREC_QT 
!FAMAL_BENSCHRP_QT
!FAMAL_RECICHRP_QT
!FAMAL_BENIMEAN_QT
!FAMAL_REPIMEAN_QT
!FAMAL_SOCSAMOU_QT
!FAMAL_DISA66PL_QT
!FAMAL_HANDSCAL_QT
!FAMAL_HANDAMOU_QT
!FAMAL_MEANSRKW_QT
!FAMAL_RKWSUPL_QT
!===================================================================================!

SUBROUTINE FAMAL_SUPALLOW

    INTEGER                             :: pers_j,pers_k,pers_l
    REAL(KIND=KIND_REAL)                :: MONTH_AMOUNT,REPLACE_SUM,LABINC_PARTNER
    INTEGER                             :: MONTH_AGE,i,j,MONTH_START,MONTH_STOP
    INTEGER,DIMENSION(4)                :: COUNT_RPSCHEM
    REAL(KIND=KIND_REAL),DIMENSION(4)   :: POPU_RPSCHEM
    INTEGER                             :: COUNT_RPMEAN,COUNT_66PL
    REAL(KIND=KIND_REAL)                :: POPU_RPMEAN,POPU_66PL
    INTEGER,DIMENSION(10)               :: COUNT_SCAL
    REAL(KIND=KIND_REAL),DIMENSION(10)  :: POPU_SCAL
    INTEGER                             :: SINGHH
    REAL(KIND=KIND_REAL)                :: HHINC_SUM
    INTEGER                             :: TOTPIJL

    !FAMAL_REPIBEN_QT: The beneficiary can be granted a social supplement because he receives one of the specified replacement incomes         
    !1) if the beneficiary is unemployed for more than 6 months or receiving pension benefits other than self employment pension benefits
    !2) if the beneficiary receives self employment pension benefits 
    !3) if the beneficiary is disabled.
    !4) if the beneficiary is disabled with handicap.
    !Loop over all households 
    COUNT_RPSCHEM(1:4) = 0
    POPU_RPSCHEM(1:4) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at its own level
            IF (INDISBEN_QT(pers_j).EQ.1) THEN
                !Check the system that generates pension benefits
                IF (ANY(PENSWELF_BENSCHEM_QT(pers_j).EQ.(/1, 2/))) THEN
                    FAMAL_REPIBEN_QT(pers_j) = 1
                ELSE IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.3) THEN
                    FAMAL_REPIBEN_QT(pers_j) = 2
                END IF
                !Check the number of months the beneficiary is in unemployment
                IF (UNEM_FULL6MON_QT(pers_j).GT.6) THEN
                    FAMAL_REPIBEN_QT(pers_j) = 1
                END IF
!Aangepast bij herwerking september 2011
                !Check whether the beneficiary receives disability benefits
                IF (FAMAL_REPIBEN_QT(pers_j).EQ.0.AND.ANY(SICK_DISABBEN_QT(pers_j).EQ.(/1, 2, 3, 4/))) THEN
                    FAMAL_REPIBEN_QT(pers_j) = 3
                END IF
                !Check whether the beneficiary receives disability benefits
!Aangepast bij herwerking september 2011
!                IF (FAMAL_REPIBEN_QT(pers_j).EQ.0.AND.InFODSZ(pers_j).EQ.1) THEN
                IF (InFODSZ(pers_j).EQ.1) THEN
                    FAMAL_REPIBEN_QT(pers_j) = 4
                END IF
                !Count the identified schemes of the beneficiaries
                DO i=1,4
                    IF (FAMAL_REPIBEN_QT(pers_j).EQ.i) THEN
                        COUNT_RPSCHEM(i) = COUNT_RPSCHEM(i) + 1
                        POPU_RPSCHEM(i) = POPU_RPSCHEM(i) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END DO
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction of beneficiaries and recipients
    Write(15,'('' '')')
    Write(15,'(''The beneficiary scheme that generates social supplements'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''beneficiary is unemployed more than 6 months or pensioner in wea or civil          '', i10)')COUNT_RPSCHEM(1)
    Write(15,'(''beneficiary receives self employment pensions                                      '', i10)')COUNT_RPSCHEM(2)
    Write(15,'(''beneficiary is disabled                                                            '', i10)')COUNT_RPSCHEM(3)
    Write(15,'(''beneficiary is disabled with handicap                                              '', i10)')COUNT_RPSCHEM(4)
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_RPSCHEM(1:4))
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''beneficiary is unemployed more than 6 months or pensioner in wea or civil          '', f10.0)')POPU_RPSCHEM(1)
    Write(15,'(''beneficiary receives self employment pensions                                      '', f10.0)')POPU_RPSCHEM(2)
    Write(15,'(''beneficiary is disabled                                                            '', f10.0)')POPU_RPSCHEM(3)
    Write(15,'(''beneficiary is disabled with handicap                                              '', f10.0)')POPU_RPSCHEM(4)
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_RPSCHEM(1:4))

!Aangepast bij herwerking september 2011
    !FAMAL_REPIREC_QT: The recipient can be granted a social supplement because the beneficiary receives one of the specified replacement incomes         
    !1) if the recipient is unemployed for more than 6 months or receiving pension benefits other than self employment pension benefits
    !2) if the recipient receives self employment pension benefits 
    !3) if the recipient is disabled.
    !4) if the recipient is disabled with handicap
    !Loop over all households 
    COUNT_RPSCHEM(1:4) = 0
    POPU_RPSCHEM(1:4) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be a recipient at the level of the child 
            !and the child is eligible for child benefits then look up the line of the recipient itself and determine the status
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k).AND.FAMAL_REPIBEN_QT(pers_k).NE.0) THEN
                        DO pers_l=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                            IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_l)) THEN
                                FAMAL_REPIREC_QT(pers_l) = FAMAL_REPIBEN_QT(pers_k)        
                                EXIT
                            END IF
                        END DO
                    END IF
                END DO
            END IF
        END DO        
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !Count the identified schemes of the beneficiaries
            DO i=1,4
                IF (FAMAL_REPIREC_QT(pers_j).EQ.i) THEN
                    COUNT_RPSCHEM(i) = COUNT_RPSCHEM(i) + 1
                    POPU_RPSCHEM(i) = POPU_RPSCHEM(i) + MIMOSIS_WEIGHT(pers_j)                    
                END IF
            END DO
        END DO
    END DO

!    !FAMAL_REPIREC_QT: The recipient can be granted a social supplement because the beneficiary receives one of the specified replacement incomes         
!    !1) if the recipient is unemployed for more than 6 months or receiving pension benefits other than self employment pension benefits
!    !2) if the recipient receives self employment pension benefits 
!    !3) if the recipient is disabled.
!    !Loop over all households 
!    COUNT_RPSCHEM(1:3) = 0
!    POPU_RPSCHEM(1:3) = 0
!    DO hh_i=1,N_HH_FAMAL
!        !Loop over all persons within the household
!        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
!            !if an individual has been identified to be beneficiary at its own level
!            IF (INDISREC_QT(pers_j).EQ.1) THEN
!                !Check the system that generates pension benefits
!                IF (ANY(PENSWELF_BENSCHEM_QT(pers_j).EQ.(/1, 2/))) THEN
!                    FAMAL_REPIREC_QT(pers_j) = 1
!                ELSE IF (PENSWELF_BENSCHEM_QT(pers_j).EQ.3) THEN
!                    FAMAL_REPIREC_QT(pers_j) = 2
!                END IF
!                !Check the number of months the beneficiary is in unemployment
!                IF (UNEM_FULL6MON_QT(pers_j).GT.6) THEN
!                    FAMAL_REPIREC_QT(pers_j) = 1
!                END IF
!!Aangepast bij herwerking september 2011
!                !Check whether the beneficiary receives disability benefits
!                IF (ANY(SICK_DISABBEN_QT(pers_j).EQ.(/1, 2, 3/)).OR.InFODSZ(pers_j).EQ.1) THEN
!                    FAMAL_REPIREC_QT(pers_j) = 3
!                END IF
!                !Count the identified schemes of the beneficiaries
!                DO i=1,3
!                    IF (FAMAL_REPIREC_QT(pers_j).EQ.i) THEN
!                        COUNT_RPSCHEM(i) = COUNT_RPSCHEM(i) + 1
!                        POPU_RPSCHEM(i) = POPU_RPSCHEM(i) + MIMOSIS_WEIGHT(pers_j)                    
!                    END IF
!                END DO
!            END IF
!        END DO
!    END DO        

    !Write some test results
    !Write results of construction of recipients
    Write(15,'('' '')')
    Write(15,'(''The recipient scheme that generates social supplements'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Counts'')')
    Write(15,'(''recipient is unemployed more than 6 months or pensioner in wea or civil          '', i10)')COUNT_RPSCHEM(1)
    Write(15,'(''recipient receives self employment pensions                                      '', i10)')COUNT_RPSCHEM(2)
    Write(15,'(''recipient is disabled                                                            '', i10)')COUNT_RPSCHEM(3)
    Write(15,'(''recipient is disabled with handicap                                              '', i10)')COUNT_RPSCHEM(4)
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_RPSCHEM(1:3))
    Write(15,'(''Weighted amounts'')')
    Write(15,'(''recipient is unemployed more than 6 months or pensioner in wea or civil          '', f10.0)')POPU_RPSCHEM(1)
    Write(15,'(''recipient receives self employment pensions                                      '', f10.0)')POPU_RPSCHEM(2)
    Write(15,'(''recipient is disabled                                                            '', f10.0)')POPU_RPSCHEM(3)
    Write(15,'(''recipient is disabled with handicap                                              '', f10.0)')POPU_RPSCHEM(4)
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_RPSCHEM(1:3))

    !FAMAL_BENSCHRP_QT: variable containing the scheme of the beneficiary that produces a social supplement on the level of the beneficiary child
    !1) if the beneficiary is unemployed for more than 6 months or receiving pension benefits other than self employment pension benefits
    !2) if the beneficiary receives self employment pension benefits 
    !3) if the beneficiary is disabled.
    !4) if the beneficiary is disabled with handicap
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be beneficiary at the level of the child look up the line of the beneficiary itself
            !and identify the scheme of the beneficiary that produces a social supplement at the level of the child
            IF (FAMAL_BENEFID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_BENEFID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_BENSCHRP_QT(pers_j) = FAMAL_REPIBEN_QT(pers_k)
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO

    !FAMAL_RECICHRP_QT: variable containing the scheme of the recipient that produces a social supplement on the level of the beneficiary child
    !1) if the recipient is unemployed for more than 6 months or receiving pension benefits other than self employment pension benefits
    !2) if the recipient receives self employment pension benefits 
    !3) if the recipient is disabled.
    !4) if the recipient is disabled with handicap
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at the level of the child look up the line of the recipient itself
            !and identify the scheme of the recipient that produces a social supplement at the level of the child
            IF (FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_RECICHRP_QT(pers_j) = FAMAL_REPIREC_QT(pers_k)
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO

!Aangepast bij herwerking september 2011
    !FAMAL_BENIMEAN_QT: The beneficiary can be granted a social supplement because he lives in a household that satisfies the means test, applied when social supplements are granted           
    !Loop over all households 
    COUNT_RPMEAN = 0
    POPU_RPMEAN = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at its own level and if the individual has the appropriate replacement income
            IF (INDISBEN_QT(pers_j).EQ.1.AND.ANY(FAMAL_REPIBEN_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                !Compute sum of replacement income of both partners if present
                REPLACE_SUM = 0
                !Compute the labour income of the recipient and the partner of the recipient
                LABINC_PARTNER = 0
                !check whether this individual has a partner or not, if there is a partner, look up the partners labour income
                IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                    DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                        !either j is partner 1
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of replacement incomes of both partners (only these amounts count)
                            IF ((FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                            END IF
                            IF ((FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k))
                            END IF
                        !or k is partner 1    
                        ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of replacement incomes of both partners (only these amounts count)
                            IF ((FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                            END IF
                            IF ((FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k))
                            END IF
                        END IF
                    END DO
                !Otherwise the recipient has no partner
                ELSE
                    REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                END IF
                IF (SIMYEAR.LT.2005) THEN
                    IF (REPLACE_SUM.LT.(3.D0*FAMAL_REPLEVEL_MONTH(1)).AND.LABINC_PARTNER.LT.(3.D0*FAMAL_REPLEVEL_MONTH(2))) THEN
                        FAMAL_BENIMEAN_QT(pers_j) = 1    
                        COUNT_RPMEAN = COUNT_RPMEAN + 1
                        POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                    END IF
                ELSE
                    IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                        IF ((REPLACE_SUM+LABINC_PARTNER+FAMAL_LABMINC_QT(pers_j)).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)+&
                        &FAMAL_REPLEVEL_MONTH(2)))) THEN
                            FAMAL_BENIMEAN_QT(pers_j) = 1    
                            COUNT_RPMEAN = COUNT_RPMEAN + 1
                            POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                        END IF
                    ELSE
                        IF ((REPLACE_SUM+FAMAL_LABMINC_QT(pers_j)).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)))) THEN
                            FAMAL_BENIMEAN_QT(pers_j) = 1    
                            COUNT_RPMEAN = COUNT_RPMEAN + 1
                            POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                        END IF
                    END IF
                END IF
            END IF
!Aangepast bij herwerking september 2011
            !if an individual has been identified to be beneficiary and handicaped, no income test is necessary
            IF (INDISBEN_QT(pers_j).EQ.1.AND.ANY(FAMAL_REPIBEN_QT(pers_j).EQ.(/4/))) THEN
                FAMAL_BENIMEAN_QT(pers_j) = 1  
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction beneficiaries that generate social supplement
    Write(15,'('' '')')
    Write(15,'(''Number of beneficiaries that generate social supplements'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of beneficiaries                                                      '', i10)') COUNT_RPMEAN 
    Write(15,'(''Weighted amount of beneficiaries                                                   '', f10.0)') POPU_RPMEAN

    !FAMAL_REPIMEAN_QT: The recipient can be granted a social supplement because he lives in a household that satisfies the means test, applied when social supplements are granted           
    !Loop over all households 
    COUNT_RPMEAN = 0
    POPU_RPMEAN = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at its own level and if the individual has the appropriate replacement income
            IF (INDISREC_QT(pers_j).EQ.1.AND.ANY(FAMAL_REPIREC_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                !Compute sum of replacement income of both partners if present
                REPLACE_SUM = 0
                !Compute the labour income of the recipient and the partner of the recipient
                LABINC_PARTNER = 0
                !check whether this individual has a partner or not, if there is a partner, look up the partners labour income
                IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                    DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                        !either j is partner 1
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of replacement incomes of both partners (only these amounts count)
                            IF ((FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                            END IF
                            IF ((FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k))
                            END IF
                        !or k is partner 1    
                        ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of replacement incomes of both partners (only these amounts count)
                            IF ((FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                            END IF
                            IF ((FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k)).GT.0.D0) THEN
                                REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_k)-FAMAL_LABMINC_QT(pers_k))
                            END IF
                        END IF
                    END DO
                !Otherwise the recipient has no partner
                ELSE
                    REPLACE_SUM = REPLACE_SUM + (FAMAL_INDINC_QT(pers_j)-FAMAL_LABMINC_QT(pers_j))
                END IF
                IF (SIMYEAR.LT.2005) THEN
                    IF (REPLACE_SUM.LT.(3.D0*FAMAL_REPLEVEL_MONTH(1)).AND.LABINC_PARTNER.LT.(3.D0*FAMAL_REPLEVEL_MONTH(2))) THEN
                        FAMAL_REPIMEAN_QT(pers_j) = 1    
                        COUNT_RPMEAN = COUNT_RPMEAN + 1
                        POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                    END IF
                ELSE
                    IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                        IF ((REPLACE_SUM+LABINC_PARTNER+FAMAL_LABMINC_QT(pers_j)).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)+&
                        &FAMAL_REPLEVEL_MONTH(2)))) THEN
                            FAMAL_REPIMEAN_QT(pers_j) = 1    
                            COUNT_RPMEAN = COUNT_RPMEAN + 1
                            POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                        END IF
                    ELSE
                        IF ((REPLACE_SUM+FAMAL_LABMINC_QT(pers_j)).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)))) THEN
                            FAMAL_REPIMEAN_QT(pers_j) = 1    
                            COUNT_RPMEAN = COUNT_RPMEAN + 1
                            POPU_RPMEAN = POPU_RPMEAN + MIMOSIS_WEIGHT(pers_j) 
                        END IF
                    END IF
                END IF
            END IF
        END DO
    END DO        

    !Write some test results
    !Write results of construction recipients that generate social supplement
    Write(15,'('' '')')
    Write(15,'(''Number of recipients that generate social supplements'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of recipients                                                      '', i10)') COUNT_RPMEAN 
    Write(15,'(''Weighted amount of recipients                                                   '', f10.0)') POPU_RPMEAN

    !FAMAL_ASUPWEA_QT: Amount of age supplements in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in another than the self employed contribution scheme          
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !check whether a child is eligible for child benefits (orphan condition is not relevant for the application of supplements)
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                !If the beneficiary of this child generated benefits in the wage earner, civil servant or guaranteed allowance scheme, then proceed
                IF (ANY(FAMAL_BENSCHIL_QT(pers_j).EQ.(/1, 2, 4/))) THEN
                    !Loop over three possible months to compute the age supplement month by month
                    DO i=1,3
                        !initialise intermediate variables
                        MONTH_AGE = 0
                        MONTH_START = 0
                        MONTH_STOP = 0
                        !Determine the age of an individual in number of months, conditional on the quarter of analysis
                        IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 9)
                        ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 6)
                        ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 3)
                        ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - i
                        END IF
                        !If the child is not handicapped, does not generate social supplement and of rank 1 then apply normal age supplement
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.1.AND.FAMAL_HANDICAP_ANN(pers_j).EQ.0.AND.&
                        &ANY(FAMAL_BENSCHIL_QT(pers_j).EQ.(/1, 2/)).AND.FAMAL_RECICHRP_QT(pers_j).EQ.0) THEN
                            !Select the appropriate age conditions, conditional on the year of birth
!Oud systeem met mogelijke differentiatie: afgezet naar aaneleiding van RKW
!Systeem leeftijdsbijslagen is dus niet helemaal correct meer voor jaren voor 2010
!Voor 2010 is het min of meer correct omdat individu geboren in 1991, 17 jaar is in 2010
!                            IF (FAMAL_YEAR_BIRTH(pers_j).GE.1991) THEN
                                DO j=1,3
                                    !Convert the lower and upper age in years to values expressed in months
                                    MONTH_START = INT(12.D0*FAMAL_ASWEAR1_MONTH(j,1))
                                    MONTH_STOP = INT(12.D0*FAMAL_ASWEAR1_MONTH(j,2))
                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
                                        MONTH_STOP = MONTH_STOP + 11
                                    END IF
                                    !Check the age condition
                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                        FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
                                    END IF
                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).GE.1985.AND.FAMAL_YEAR_BIRTH(pers_j).LT.1991) THEN
!                                DO j=4,5
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).GE.1981.AND.FAMAL_YEAR_BIRTH(pers_j).LT.1985) THEN
!                                DO j=6,7
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).LT.1981) THEN
!                                DO j=8,8
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            END IF
                        END IF
                        !if the child is of rank 2 or larger 
                        IF (FAMAL_CBENRANK_QT(pers_j).NE.1) THEN
                            DO j=1,3
                                !Convert the lower and upper age in years to values expressed in months
                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                    MONTH_STOP = MONTH_STOP + 11
                                END IF
                                !Check the age condition
                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                    FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR23_MONTH(j,3)
                                END IF
                            END DO
                        END IF
                        !Add the schoolpremium for all children
                        IF (FAMAL_CBENRANK_QT(pers_j).GE.1) THEN
                            DO j=1,4
                                !Convert the lower and upper age in years to values expressed in months
                                MONTH_START = INT(12.D0*FAMAL_SCHOOLPR_MONTH(j,1))
                                MONTH_STOP = INT(12.D0*FAMAL_SCHOOLPR_MONTH(j,2))
                                IF (INT(FAMAL_SCHOOLPR_MONTH(j,2)).LT.25) THEN
                                    MONTH_STOP = MONTH_STOP + 11
                                END IF
                                !Check the age condition
                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                    FAMAL_SCHOOL_QT(pers_j) = FAMAL_SCHOOL_QT(pers_j) + FAMAL_SCHOOLPR_MONTH(j,3)
                                END IF
                            END DO
                        END IF
                    END DO
                    !if the child is of rank 1 but the beneficiary was in the guaranteed benefit system or the child is an orphan
                    !then adapt the age supplement
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.1.AND.(FAMAL_BENSCHIL_QT(pers_j).EQ.4.OR.FAMAL_ORPHAN_QT(pers_j).GT.0)) THEN
                        FAMAL_ASUPWEA_QT(pers_j) = 0.D0
                        !Loop over three possible months to compute the age supplement month by month
                        DO i=1,3
                            !initialise intermediate variables
                            MONTH_AGE = 0
                            MONTH_START = 0
                            MONTH_STOP = 0
                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 9)
                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 6)
                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 3)
                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - i
                            END IF
                            DO j=1,3
                                !Convert the lower and upper age in years to values expressed in months
                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                    MONTH_STOP = MONTH_STOP + 11
                                END IF
                                !Check the age condition
                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                    FAMAL_ASUPWEA_QT(pers_j) = FAMAL_ASUPWEA_QT(pers_j) + FAMAL_ASWEAR23_MONTH(j,3)
                                END IF
                            END DO
                        END DO
                    END IF
                END IF    
            END IF                
        END DO
    END DO       

    !FAMAL_ASUPSEL_QT: Amount of age supplements in Euro paid for an eligible child during the quarter of analysis if the beneficiary opened rights in the self employed contribution scheme           
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !check whether a child is eligible for child benefits (orphan condition is not relevant for the application of supplements)
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
                !If the beneficiary of this child generated benefits in the self employment scheme
                IF (ANY(FAMAL_BENSCHIL_QT(pers_j).EQ.(/3/))) THEN
                    !Loop over three possible months to compute the age supplement month by month
                    DO i=1,3
                        !initialise intermediate variables
                        MONTH_AGE = 0
                        MONTH_START = 0
                        MONTH_STOP = 0
                        !Determine the age of an individual in number of months, conditional on the quarter of analysis
                        IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 9)
                        ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 6)
                        ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - (i + 3)
                        ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                            MONTH_AGE = FAMAL_MONTH_AGE(pers_j) - i
                        END IF
                        !If the child is not handicapped and of rank 1 then apply age supplement
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.1.AND.FAMAL_HANDICAP_ANN(pers_j).EQ.0) THEN
                            !Select the appropriate age conditions, conditional on the year of birth
!Oud systeem met mogelijke differentiatie: afgezet naar aaneleiding van RKW
!Systeem leeftijdsbijslagen is dus niet helemaal correct meer voor jaren voor 2010
!Voor 2010 is het min of meer correct omdat individu geboren in 1991, 17 jaar is in 2010
!                            IF (FAMAL_YEAR_BIRTH(pers_j).GE.1991) THEN
                                DO j=1,3
                                    !Convert the lower and upper age in years to values expressed in months
                                    MONTH_START = INT(12.D0*FAMAL_ASWEAR1_MONTH(j,1))
                                    MONTH_STOP = INT(12.D0*FAMAL_ASWEAR1_MONTH(j,2))
                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
                                        MONTH_STOP = MONTH_STOP + 11
                                    END IF
                                    !Check the age condition
                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                        FAMAL_ASUPSEL_QT(pers_j) = FAMAL_ASUPSEL_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
                                    END IF
                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).GE.1985.AND.FAMAL_YEAR_BIRTH(pers_j).LT.1991) THEN
!                                DO j=4,5
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPSEL_QT(pers_j) = FAMAL_ASUPSEL_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).GE.1981.AND.FAMAL_YEAR_BIRTH(pers_j).LT.1985) THEN
!                                DO j=6,7
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPSEL_QT(pers_j) = FAMAL_ASUPSEL_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            ELSE IF (FAMAL_YEAR_BIRTH(pers_j).LT.1981) THEN
!                                DO j=8,8
!                                    !Convert the lower and upper age in years to values expressed in months
!                                    MONTH_START = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,1))
!                                    MONTH_STOP = INT(12.D0 * FAMAL_ASWEAR1_MONTH(j,2))
!                                    IF (INT(FAMAL_ASWEAR1_MONTH(j,2)).LT.25) THEN
!                                        MONTH_STOP = MONTH_STOP + 11
!                                    END IF
!                                    !Check the age condition
!                                    IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
!                                        FAMAL_ASUPSEL_QT(pers_j) = FAMAL_ASUPSEL_QT(pers_j) + FAMAL_ASWEAR1_MONTH(j,3)
!                                    END IF
!                                END DO
!                            END IF
                        END IF
                        !else if the child is of rank 2 or more then apply different age supplement
                        IF (FAMAL_CBENRANK_QT(pers_j).NE.1) THEN
                            DO j=1,3
                                !Convert the lower and upper age in years to values expressed in months
                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                    MONTH_STOP = MONTH_STOP + 11
                                END IF
                                !Check the age condition
                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                    FAMAL_ASUPSEL_QT(pers_j) = FAMAL_ASUPSEL_QT(pers_j) + FAMAL_ASWEAR23_MONTH(j,3)
                                END IF
                            END DO
                        END IF
                        !Add the schoolpremium for all children
                        IF (FAMAL_CBENRANK_QT(pers_j).GE.1) THEN
                            DO j=1,4
                                !Convert the lower and upper age in years to values expressed in months
                                MONTH_START = INT(12.D0*FAMAL_SCHOOLPR_MONTH(j,1))
                                MONTH_STOP = INT(12.D0*FAMAL_SCHOOLPR_MONTH(j,2))
                                IF (INT(FAMAL_SCHOOLPR_MONTH(j,2)).LT.25) THEN
                                    MONTH_STOP = MONTH_STOP + 11
                                END IF
                                !Check the age condition
                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                    FAMAL_SCHOOL_QT(pers_j) = FAMAL_SCHOOL_QT(pers_j) + FAMAL_SCHOOLPR_MONTH(j,3)
                                END IF
                            END DO
                        END IF
                    END DO
                    !Check whether the child is not single or the youngest of the household
                    !if so, switch age supplements equal to 0
                    IF (FAMAL_SINGCHIL_QT(pers_j).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_j).EQ.1) THEN
                        IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                            FAMAL_ASUPSEL_QT(pers_j) = 0
                        END IF    
                    END IF
                END IF    
            END IF                
        END DO
    END DO       

!Aangepast bij herwerking september 2011
    !FAMAL_SOCSAMOU_QT: Amount of social supplements in Euro paid for an eligible child during the quarter of analysis if the recipient receives some predefined type of replacement income and satisfies the necessary means test         
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at its own level and if the individual has the appropriate replacement income
            !and satisfies the means test then look up all the children for which he is recipient
            IF (INDISBEN_QT(pers_j).EQ.1.AND.((ANY(FAMAL_REPIBEN_QT(pers_j).EQ.(/1, 2, 3, 4/)).AND.FAMAL_BENIMEAN_QT(pers_j).EQ.1)&
            &.OR.(FAMAL_BENGUARI_QT(pers_j).EQ.1))) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !if the recipient is found and the child is eligible for child benefits then first determine the monthly amount
                    MONTH_AMOUNT = 0
                    IF (FAMAL_BENEFID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_CHILELIG_QT(pers_k).EQ.1&
                    &.AND.FAMAL_ORALLORP_QT(pers_k).EQ.0.D0) THEN
                        !Select the appropriate monthly supplement conditional on the type of replacement income
                        IF (FAMAL_REPIBEN_QT(pers_j).EQ.1.OR.&
                        &(FAMAL_BENGUARI_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_k).EQ.4)) THEN
                            !use the rank of the child to select the appropriate amount from the relevant parameter
                            IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP1_MONTH(1)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP1_MONTH(2)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP1_MONTH(3)
                            END IF
                        ELSE IF (FAMAL_REPIBEN_QT(pers_j).EQ.2) THEN
                            !use the rank of the child to select the appropriate amount from the relevant parameter
                            IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP2_MONTH(1)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP2_MONTH(2)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP2_MONTH(3)
                            END IF
                        ELSE IF (ANY(FAMAL_REPIBEN_QT(pers_j).EQ.(/3, 4/))) THEN
                            !use the rank of the child to select the appropriate amount from the relevant parameter
                            IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP3_MONTH(1)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP3_MONTH(2)
                            ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                MONTH_AMOUNT = FAMAL_SOCSUP3_MONTH(3)
                            END IF
                        END IF
                        !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                        FAMAL_SOCSAMOU_QT(pers_k) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                        !if the child is of rank 1 then adapt age supplement 
                        IF (FAMAL_CBENRANK_QT(pers_k).EQ.1.AND.FAMAL_SOCSAMOU_QT(pers_k).GT.0.D0) THEN
                            IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/1, 2, 4/))) THEN
                                FAMAL_ASUPWEA_QT(pers_k) = 0.D0
                                !Loop over three possible months to compute the age supplement month by month
                                DO i=1,3
                                    !initialise intermediate variables
                                    MONTH_AGE = 0
                                    MONTH_START = 0
                                    MONTH_STOP = 0
                                    !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                    ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                    END IF
                                    DO j=1,3
                                        !Convert the lower and upper age in years to values expressed in months
                                        MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                        MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                        IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                            MONTH_STOP = MONTH_STOP + 11
                                        END IF
                                        !Check the age condition
                                        IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                            FAMAL_ASUPWEA_QT(pers_k) = FAMAL_ASUPWEA_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                        END IF
                                    END DO
                                END DO
                            ELSE IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/3/))) THEN
                                FAMAL_ASUPSEL_QT(pers_k) = 0.D0
                                !Loop over three possible months to compute the age supplement month by month
                                DO i=1,3
                                    !initialise intermediate variables
                                    MONTH_AGE = 0
                                    MONTH_START = 0
                                    MONTH_STOP = 0
                                    !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                    ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                    END IF
                                    DO j=1,3
                                        !Convert the lower and upper age in years to values expressed in months
                                        MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                        MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                        IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                            MONTH_STOP = MONTH_STOP + 11
                                        END IF
                                        !Check the age condition
                                        IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                            FAMAL_ASUPSEL_QT(pers_k) = FAMAL_ASUPSEL_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                        END IF
                                    END DO
                                END DO
                                !Check whether the child is not single or the youngest of the household
                                !if so, switch age supplements equal to 0
                                IF (FAMAL_SINGCHIL_QT(pers_k).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_k).EQ.1) THEN
                                    IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                                        FAMAL_ASUPSEL_QT(pers_k) = 0
                                    END IF    
                                END IF
                            END IF
                        END IF                            
                    END IF
                END DO
            END IF
!Aangepast
!            IF (FAMAL_SOCSAMOU_QT(pers_j).EQ.0.D0.AND.FAMAL_BENSCHRP_QT(pers_j).GT.0) THEN
!Kan voor gevallen waarbij het kind niet eligible is voor het supplement of voor child benefits tout court
!                WRITE(15,*) 'A',FAMAL_ELIGCHHH_QT(hh_i),pers_j,hh_i,FAMAL_CBENRANK_QT(pers_j),FAMAL_BENSCHIL_QT(pers_j)
!            END IF
        END DO
    END DO

    !FAMAL_SUPLSING_QT: Child benefit supplement paid because the child is living with a single parent
    !Check whether children can be granted a supplement because they live in a household with a single parent    
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at its own level and if the individual satisfies the income condition
            !and the individual is single
            !then look up all the children for which he is recipient
            IF (INDISREC_QT(pers_j).EQ.1.AND.FAMAL_INDINC_QT(pers_j).LT.(3.D0*FAMAL_INCLIMSP_MONTH).AND.&
            &ANY(FAMREL_COUPLE(pers_j).EQ.(/0/))) THEN
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !if the recipient is found and the child is eligible for child benefits then first determine the monthly amount
                    !Look for children that are eligible for child benefits and who did not yet generate another social supplement   
                    IF (FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.FAMAL_SOCSAMOU_QT(pers_k).EQ.0.D0.AND.&
                    &FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_ORALLORP_QT(pers_k).EQ.0.D0) THEN
                        !use the rank of the child to select the appropriate amount from the relevant parameter
                        IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                            FAMAL_SUPLSING_QT(pers_k) = FAMAL_SINGHHSP_MONTH(1)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                        ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                            FAMAL_SUPLSING_QT(pers_k) = FAMAL_SINGHHSP_MONTH(2)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                        ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                            FAMAL_SUPLSING_QT(pers_k) = FAMAL_SINGHHSP_MONTH(3)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                        END IF
                        !if the child is of rank 1 then adapt age supplement 
                        IF (FAMAL_CBENRANK_QT(pers_k).EQ.1.AND.FAMAL_SOCSAMOU_QT(pers_k).GT.0.D0) THEN
                            IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/1, 2, 4/))) THEN
                                FAMAL_ASUPWEA_QT(pers_k) = 0.D0
                                !Loop over three possible months to compute the age supplement month by month
                                DO i=1,3
                                    !initialise intermediate variables
                                    MONTH_AGE = 0
                                    MONTH_START = 0
                                    MONTH_STOP = 0
                                    !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                    ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                    END IF
                                    DO j=1,3
                                        !Convert the lower and upper age in years to values expressed in months
                                        MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                        MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                        IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                            MONTH_STOP = MONTH_STOP + 11
                                        END IF
                                        !Check the age condition
                                        IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                            FAMAL_ASUPWEA_QT(pers_k) = FAMAL_ASUPWEA_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                        END IF
                                    END DO
                                END DO
                            ELSE IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/3/))) THEN
                                FAMAL_ASUPSEL_QT(pers_k) = 0.D0
                                !Loop over three possible months to compute the age supplement month by month
                                DO i=1,3
                                    !initialise intermediate variables
                                    MONTH_AGE = 0
                                    MONTH_START = 0
                                    MONTH_STOP = 0
                                    !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                    IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                    ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                    ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                    ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                        MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                    END IF
                                    DO j=1,3
                                        !Convert the lower and upper age in years to values expressed in months
                                        MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                        MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                        IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                            MONTH_STOP = MONTH_STOP + 11
                                        END IF
                                        !Check the age condition
                                        IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                            FAMAL_ASUPSEL_QT(pers_k) = FAMAL_ASUPSEL_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                        END IF
                                    END DO
                                END DO
                                !Check whether the child is not single or the youngest of the household
                                !if so, switch age supplements equal to 0
                                IF (FAMAL_SINGCHIL_QT(pers_k).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_k).EQ.1) THEN
                                    IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                                        FAMAL_ASUPSEL_QT(pers_k) = 0
                                    END IF    
                                END IF
                            END IF
                        END IF                            
                    END IF
                    !Look for children that are eligible for child benefits and who did generate another social supplement or who are
                    !eligible for guaranteerd allowances
                    IF (FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.(FAMAL_SOCSAMOU_QT(pers_k).NE.0.D0.OR.&
                    &FAMAL_ORALLGUA_QT(pers_k).NE.0.D0).AND.FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j)) THEN                    
                        !use the rank of the child to select the appropriate amount from the relevant parameter
                        IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                            FAMAL_SUPLSING_QT(pers_k) = FAMAL_SINGHHSP_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                        END IF
                    END IF
                END DO            
            END IF
        END DO
    END DO

    !FAMAL_DISA66PL_QT: The individual is disabled for more than 66%(yes=1, no=0)           
    !Loop over all households 
    COUNT_66PL = 0
    POPU_66PL = 0
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            FAMAL_DISA66PL_QT(pers_j) = 0
            IF (FAMAL_DISA66PL_QT(pers_j).EQ.1) THEN
                COUNT_66PL = COUNT_66PL + 1
                POPU_66PL = POPU_66PL + MIMOSIS_WEIGHT(pers_j) 
            END IF
        END DO
    END DO       

    !Write some test results
    !Write results of construction degree of handicap
    Write(15,'('' '')')
    Write(15,'(''Number of individuals handicapped for more than 66%'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Count number of individuals                                                        '', i10)')COUNT_66PL
    Write(15,'(''Weighted amount of individuals                                                     '', f10.0)')POPU_66PL

    !FAMAL_HANDSCAL_QT: The individual is not self sustainable (scale from 0 to 9)          
    !Loop over all households 
    COUNT_SCAL(1:10) = 0
    POPU_SCAL(1:10) = 0
    DO hh_i=1,N_HH_FAMAL
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            FAMAL_HANDSCAL_QT(pers_j) = 0
            IF (FAMAL_DISA66PL_QT(pers_j).EQ.1.AND.FAMAL_HANDSCAL_QT(pers_j).GT.0) THEN
                DO i=0,9
                    IF (FAMAL_HANDSCAL_QT(pers_j).EQ.i) THEN
                        COUNT_SCAL(i+1) = COUNT_SCAL(i+1) + 1
                        POPU_SCAL(i+1) = POPU_SCAL(i+1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF
                END DO
            END IF
        END DO
    END DO       
    
    !Write some test results
    !Write results on the degree of self sustainability of handicapped individuals 
    Write(15,'('' '')')
    Write(15,'(''Number of individuals handicapped for more than 66% according to degree of self sustainability'')')
    CALL DRAWLINE(15,136,'-') 
    DO i=1,10
        Write(15,'(''Count number of individuals according to degree of self sustainability   '', 2i10)')i,COUNT_SCAL(i) 
    END DO
    Write(15,'(''Total                                                                              '', i10)')&
    &SUM(COUNT_SCAL(1:10))
    DO i=1,10
        Write(15,'(''Weighted number of individuals according to degree of self sustainability'', i10,f10.0)')i,POPU_SCAL(i) 
    END DO
    Write(15,'(''Total                                                                              '', f10.0)')&
    &SUM(POPU_SCAL(1:10))

!Aangepast bij herwerking september 2011
    !FAMAL_HANDAMOU_QT: Amount of supplements in Euro paid for an eligible child during the quarter of analysis if the eligible child is disabled for more than 66%         
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual is eligible for child benefits then determine the benefit
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1) THEN
            !distinguish old and new law
                IF (code_leg(pers_j).EQ.'01'.AND.FAMAL_YEAR_AGE(pers_j).LT.21) THEN
                    !select monthly amount conditional on degree of self sustainability
                    IF (ANY(PUNT_ZELFVKB(pers_j).EQ.(/0, 1, 2, 3/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBOLD_MONTH(1)
                    ELSE IF (ANY(PUNT_ZELFVKB(pers_j).EQ.(/4, 5, 6/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBOLD_MONTH(2)
                    ELSE IF (ANY(PUNT_ZELFVKB(pers_j).EQ.(/7, 8, 9/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBOLD_MONTH(3)
                    END IF
                ELSE IF (code_leg(pers_j).EQ.'02'.AND.FAMAL_YEAR_AGE(pers_j).LT.21) THEN 
                    TOTPIJL = pilier_un_conv(pers_j) + pilier_deux(pers_j) + pilier_trois(pers_j)
                    !select monthly amount conditional on degree of self sustainability
                    IF (ANY(TOTPIJL.EQ.(/4, 5/)).AND.ANY(pilier_un_conv(pers_j).EQ.(/4, 5/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(1)
                    ELSE IF (ANY(TOTPIJL.EQ.(/6, 7, 8/)).AND.pilier_un_conv(pers_j).LT.4) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(2)
                    ELSE IF (ANY(TOTPIJL.EQ.(/6, 7, 8/)).AND.pilier_un_conv(pers_j).GE.4) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(3)
                    ELSE IF (ANY(TOTPIJL.EQ.(/9, 10, 11/)).AND.pilier_un_conv(pers_j).LT.4) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(4)
                    ELSE IF (ANY(TOTPIJL.EQ.(/9, 10, 11/)).AND.pilier_un_conv(pers_j).GE.4) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(5)
                    ELSE IF (ANY(TOTPIJL.EQ.(/12, 13, 14/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(6)
                    ELSE IF (ANY(TOTPIJL.EQ.(/15, 16, 17/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(7)
                    ELSE IF (ANY(TOTPIJL.EQ.(/18, 19, 20/))) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(8)
                    ELSE IF (TOTPIJL.GT.20) THEN
                        MONTH_AMOUNT = FAMAL_VKBNEW_MONTH(9)
                    END IF
!Aangepast bij herwerking september 2011                        
                ELSE IF (ANY(code_leg(pers_j).EQ.(/'03', '04', '05'/))) THEN
                    !Voor verlengde kinderbijslag kennen we maar 1 bedrag toe, met name de basisbijslag van een
                    !kind in het stelsel van de werknemers
                    IF (FAMAL_CBENRANK_QT(pers_j).LE.2) THEN
                        MONTH_AMOUNT = FAMAL_OWEAAMOU_MONTH(2)
                    ELSE
                        MONTH_AMOUNT = FAMAL_OWEAAMOU_MONTH(3)
                    END IF
                    !Zorg er voor dat alle mogelijke anderen uitkeringen hier op nul gezet worden.
                    !Voor verlengde bijslag wordt dus enkel een supplement ingevuld
                    FAMAL_ORALLWEA_QT(pers_j) = 0.D0
                    FAMAL_ASUPWEA_QT(pers_j) = 0.D0
                    FAMAL_SOCSAMOU_QT(pers_j) = 0.D0
                    FAMAL_ORALLORP_QT(pers_j) = 0.D0
                    FAMAL_BIRTHALL_QT(pers_j) = 0.D0
                    FAMAL_ADOPALL_QT(pers_j) = 0.D0
                    FAMAL_ORALLCIV_QT(pers_j) = 0.D0
                    FAMAL_ORALLSEL_QT(pers_j) = 0.D0
                    FAMAL_ASUPSEL_QT(pers_j) = 0.D0
                    FAMAL_ORALLGUA_QT(pers_j) = 0.D0
                    FAMAL_SUPLSING_QT(pers_j) = 0.D0
                    FAMAL_SCHOOL_QT(pers_j) = 0.D0
                END IF
!Aangepast bij herwerking september 2011                                        
                !Determine the quaterly amount by multiplying the monthly amount with the number of months for which a benefit is paid
                IF (ANY(code_leg(pers_j).EQ.(/'01', '02', '03', '04', '05'/))) THEN
                    FAMAL_HANDAMOU_QT(pers_j) = MONTH_AMOUNT*DBLE(FAMAL_NMONTHBE_QT(pers_j))/4.D0
!                    WRITE(15,*) 'Test',FAMAL_HANDAMOU_QT(pers_j),code_leg(pers_j)
                END IF
            END IF
!            IF (ANY(code_leg(pers_j).EQ.(/'01', '02'/))) THEN
!                IF (FAMAL_YEAR_AGE(pers_j).GT.25) THEN
!                    WRITE(15,*) GECODEERD_INSZ_INPUT(pers_j),FAMAL_YEAR_AGE(pers_j),MIMOSIS_WEIGHT(pers_j)
!                END IF
!            END IF
!            IF (InFODSZ(pers_j).EQ.1.AND.(InRKW(pers_j).EQ.1.OR.InRSVZ_KB(pers_j).EQ.1).AND.FAMAL_CHILELIG_QT(pers_j).EQ.0) THEN
!                WRITE(15,*) GECODEERD_INSZ_INPUT(pers_j),FAMAL_YEAR_AGE(pers_j),MIMOSIS_WEIGHT(pers_j),FAMAL_BENSCHIL_QT(pers_j),&
!                &code_leg(pers_j),NAREGNIS_RELATION(pers_j)
!            END IF
        END DO
    END DO    
    
    !FAMAL_MEANSRKW_QT: The recipient can be granted a social supplement because he satisfies the means imposed for additional RKW simulations
    !1 individual satisfies the income conditions for simualtion 1
    !2 individual satisfies the income conditions for simualtion 2
    !3 individual satisfies the income conditions for simualtion 3
    !Loop over all households 
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household to test whether a supplement can be granted
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if an individual has been identified to be recipient at its own level test additional conditions for supplement
            IF (INDISREC_QT(pers_j).EQ.1) THEN
                !Initialise household income
                HHINC_SUM = 0.D0
                !Compute the labour income of the partner of the recipient
                LABINC_PARTNER = 0
                !check whether this individual has a partner or not, if there is a partner, look up the partners labour income
                IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                    DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                        !either j is partner 1
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.1.AND.FAMREL_RELATION(pers_k).EQ.2)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of incomes of both partners (only these amounts count)
                            HHINC_SUM = HHINC_SUM + FAMAL_INDINC_QT(pers_j) + FAMAL_INDINC_QT(pers_k)                            
                        !or k is partner 1    
                        ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.&
                        &(FAMREL_RELATION(pers_j).EQ.2.AND.FAMREL_RELATION(pers_k).EQ.1)) THEN
                            LABINC_PARTNER = FAMAL_LABMINC_QT(pers_k)
                            !Make sum of incomes of both partners (only these amounts count)
                            HHINC_SUM = HHINC_SUM + FAMAL_INDINC_QT(pers_j) + FAMAL_INDINC_QT(pers_k)                            
                        END IF
                    END DO
                !Otherwise the recipient has no partner
                ELSE
                    HHINC_SUM = HHINC_SUM + FAMAL_INDINC_QT(pers_j)                            
                END IF
                !test the income conditions for simulation 1
                IF (FAMREL_COUPLE(pers_j).NE.0) THEN
                    IF (HHINC_SUM.LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)+FAMAL_REPLEVEL_MONTH(2))).AND.&
                    &(FAMAL_LABMINC_QT(pers_j)+LABINC_PARTNER).GT.0.D0) THEN
                        FAMAL_MEANSRKW_QT(pers_j,1) = 1    
                    END IF
                    !test the income conditions for simulation 2
                    IF (HHINC_SUM.LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)+FAMAL_REPLEVEL_MONTH(2)))) THEN
                        FAMAL_MEANSRKW_QT(pers_j,2) = 1    
                    END IF
                    !test the income conditions for simulation 3
                    IF ((FAMAL_LABMINC_QT(pers_j)+LABINC_PARTNER).GT.0.D0.AND.&
                    &(FAMAL_LABMINC_QT(pers_j)+LABINC_PARTNER).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)+FAMAL_REPLEVEL_MONTH(2)))) THEN
                        FAMAL_MEANSRKW_QT(pers_j,3) = 1    
                    END IF
                !Otherwise the recipient has no partner
                ELSE
                    IF (HHINC_SUM.LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1))).AND.FAMAL_LABMINC_QT(pers_j).GT.0.D0) THEN
                        FAMAL_MEANSRKW_QT(pers_j,1) = 1    
                    END IF
                    !test the income conditions for simulation 2
                    IF (HHINC_SUM.LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)))) THEN
                        FAMAL_MEANSRKW_QT(pers_j,2) = 1    
                    END IF
                    !test the income conditions for simulation 3
                    IF (FAMAL_LABMINC_QT(pers_j).GT.0.D0.AND.(FAMAL_LABMINC_QT(pers_j)).LT.(3.D0*(FAMAL_REPLEVEL_MONTH(1)))) THEN
                        FAMAL_MEANSRKW_QT(pers_j,3) = 1    
                    END IF
                END IF
            END IF
        END DO
    END DO        

    !FAMAL_RKWSUPL_QT: Supplements for simulations of RKW
    IF (FAMAL_SIMSURKW_QT.GT.0) THEN
        !Simulation 1
        IF (FAMAL_SIMSURKW_QT.EQ.1) THEN
            !Loop over all households
            DO hh_i=1,N_HH_FAMAL
                !Loop over all persons within the household
                DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !if an individual has been identified to be recipient at its own level and satisfies the means test 
                    !then look up all the children for which he is recipient
                    IF (INDISREC_QT(pers_j).EQ.1.AND.FAMAL_MEANSRKW_QT(pers_j,1).EQ.1) THEN
                        DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                            !if the recipient is found and the child is eligible for child benefits 
                            !and no other social supplement or supplement for single household has been granted yet 
                            !then use supplement
                            IF (FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.&
                            &FAMAL_SOCSAMOU_QT(pers_k).EQ.0.D0.AND.FAMAL_SUPLSING_QT(pers_k).EQ.0.D0.AND.&
                            &FAMAL_ORALLORP_QT(pers_k).EQ.0) THEN
                                !use the rank of the child to select the appropriate amount from the relevant parameter
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(1)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(2)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(3)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    IF (ANY(FAMREL_COUPLE(pers_j).EQ.(/0/))) THEN
                                        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    END IF
                                END IF
                                !if the child is of rank 1 then adapt age supplement 
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1.AND.FAMAL_RKWSUPL_QT(pers_k).GT.0.D0) THEN
                                    IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/1, 2, 4/))) THEN
                                        FAMAL_ASUPWEA_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPWEA_QT(pers_k) = FAMAL_ASUPWEA_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                    ELSE IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/3/))) THEN
                                        FAMAL_ASUPSEL_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPSEL_QT(pers_k) = FAMAL_ASUPSEL_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                        !Check whether the child is not single or the youngest of the household
                                        !if so, switch age supplements equal to 0
                                        IF (FAMAL_SINGCHIL_QT(pers_k).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_k).EQ.1) THEN
                                            IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                                                FAMAL_ASUPSEL_QT(pers_k) = 0
                                            END IF    
                                        END IF
                                    END IF
                                END IF
                            END IF
                            !First implementation
                            !!Look for children that are eligible for child benefits and who did generate another social supplement or who are
                            !!eligible for guaranteerd allowances and who live with single parents
                            !IF (FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.(FAMAL_SOCSAMOU_QT(pers_k).NE.0.D0.OR.&
                            !&FAMAL_ORALLGUA_QT(pers_k).NE.0.D0.OR.FAMAL_SUPLSING_QT(pers_k).NE.0.D0).AND.&
                            !&FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.ANY(FAMREL_COUPLE(pers_j).EQ.(/0, 2/)))&
                            !& THEN
                            !    !use the rank of the child to select the appropriate amount from the relevant parameter
                            !    IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                            !        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)**DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                            !    END IF
                            !END IF
                        END DO
                    END IF
                END DO
            END DO
        !Simulation 2
        ELSE IF (FAMAL_SIMSURKW_QT.EQ.2) THEN
            !Loop over all households
            DO hh_i=1,N_HH_FAMAL
                !Loop over all persons within the household
                DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !if an individual has been identified to be recipient at its own level and satisfies the means test 
                    !then look up all the children for which he is recipient
                    IF (INDISREC_QT(pers_j).EQ.1.AND.FAMAL_MEANSRKW_QT(pers_j,2).EQ.1) THEN
                        DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                            !if the recipient is found and the child is eligible for child benefits 
                            !and no other social supplement or supplement for single household has been granted yet 
                            !then use supplement
                            IF (FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.&
                            &FAMAL_SOCSAMOU_QT(pers_k).EQ.0.D0.AND.FAMAL_SUPLSING_QT(pers_k).EQ.0.D0.AND.&
                            &FAMAL_ORALLORP_QT(pers_k).EQ.0) THEN
                                !use the rank of the child to select the appropriate amount from the relevant parameter
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(1)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(2)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(3)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    IF (ANY(FAMREL_COUPLE(pers_j).EQ.(/0/))) THEN
                                        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    END IF
                                END IF
                                !if the child is of rank 1 then adapt age supplement 
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1.AND.FAMAL_RKWSUPL_QT(pers_k).GT.0.D0) THEN
                                    IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/1, 2, 4/))) THEN
                                        FAMAL_ASUPWEA_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPWEA_QT(pers_k) = FAMAL_ASUPWEA_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                    ELSE IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/3/))) THEN
                                        FAMAL_ASUPSEL_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPSEL_QT(pers_k) = FAMAL_ASUPSEL_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                        !Check whether the child is not single or the youngest of the household
                                        !if so, switch age supplements equal to 0
                                        IF (FAMAL_SINGCHIL_QT(pers_k).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_k).EQ.1) THEN
                                            IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                                                FAMAL_ASUPSEL_QT(pers_k) = 0
                                            END IF    
                                        END IF
                                    END IF
                                END IF
                            END IF
                            !!First implementation
                            !!Look for children that are eligible for child benefits and who did generate another social supplement or who are
                            !!eligible for guaranteerd allowances and who live with single parents
                            !IF (FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.(FAMAL_SOCSAMOU_QT(pers_k).NE.0.D0.OR.&
                            !&FAMAL_ORALLGUA_QT(pers_k).NE.0.D0.OR.FAMAL_SUPLSING_QT(pers_k).NE.0.D0).AND.&
                            !&FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.ANY(FAMREL_COUPLE(pers_j).EQ.(/0, 2/)))&
                            !& THEN
                            !    !use the rank of the child to select the appropriate amount from the relevant parameter
                            !    IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                            !        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                            !    END IF
                            !END IF
                        END DO
                    END IF
                END DO
            END DO
        !Simulation 3
        ELSE IF (FAMAL_SIMSURKW_QT.EQ.3) THEN
            !Loop over all households
            DO hh_i=1,N_HH_FAMAL
                !Loop over all persons within the household
                DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    !if an individual has been identified to be recipient at its own level and satisfies the means test 
                    !then look up all the children for which he is recipient
                    IF (INDISREC_QT(pers_j).EQ.1.AND.FAMAL_MEANSRKW_QT(pers_j,3).EQ.1) THEN
                        DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                            !if the recipient is found and the child is eligible for child benefits 
                            !and no other social supplement or supplement for single household has been granted yet 
                            !then use supplement
                            IF (FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.&
                            &FAMAL_SOCSAMOU_QT(pers_k).EQ.0.D0.AND.FAMAL_SUPLSING_QT(pers_k).EQ.0.D0.AND.&
                            &FAMAL_ORALLORP_QT(pers_k).EQ.0) THEN
                                !use the rank of the child to select the appropriate amount from the relevant parameter
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(1)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).EQ.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(2)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                ELSE IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                                    FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(3)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    IF (ANY(FAMREL_COUPLE(pers_j).EQ.(/0/))) THEN
                                        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                                    END IF
                                END IF
                                !if the child is of rank 1 then adapt age supplement 
                                IF (FAMAL_CBENRANK_QT(pers_k).EQ.1.AND.FAMAL_RKWSUPL_QT(pers_k).GT.0.D0) THEN
                                    IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/1, 2, 4/))) THEN
                                        FAMAL_ASUPWEA_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPWEA_QT(pers_k) = FAMAL_ASUPWEA_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                    ELSE IF (ANY(FAMAL_BENSCHIL_QT(pers_k).EQ.(/3/))) THEN
                                        FAMAL_ASUPSEL_QT(pers_k) = 0.D0
                                        !Loop over three possible months to compute the age supplement month by month
                                        DO i=1,3
                                            !initialise intermediate variables
                                            MONTH_AGE = 0
                                            MONTH_START = 0
                                            MONTH_STOP = 0
                                            !Determine the age of an individual in number of months, conditional on the quarter of analysis
                                            IF (ANY(month.EQ.(/1, 2, 3/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 9)
                                            ELSE IF (ANY(month.EQ.(/4, 5, 6/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 6)
                                            ELSE IF (ANY(month.EQ.(/7, 6, 9/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - (i + 3)
                                            ELSE IF (ANY(month.EQ.(/10, 11, 12/))) THEN
                                                MONTH_AGE = FAMAL_MONTH_AGE(pers_k) - i
                                            END IF
                                            DO j=1,3
                                                !Convert the lower and upper age in years to values expressed in months
                                                MONTH_START = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,1))
                                                MONTH_STOP = INT(12.D0*FAMAL_ASWEAR23_MONTH(j,2))
                                                IF (INT(FAMAL_ASWEAR23_MONTH(j,2)).LT.25) THEN
                                                    MONTH_STOP = MONTH_STOP + 11
                                                END IF
                                                !Check the age condition
                                                IF (MONTH_AGE.GE.MONTH_START.AND.MONTH_AGE.LE.MONTH_STOP) THEN
                                                    FAMAL_ASUPSEL_QT(pers_k) = FAMAL_ASUPSEL_QT(pers_k) + FAMAL_ASWEAR23_MONTH(j,3)
                                                END IF
                                            END DO
                                        END DO
                                        !Check whether the child is not single or the youngest of the household
                                        !if so, switch age supplements equal to 0
                                        IF (FAMAL_SINGCHIL_QT(pers_k).EQ.1.OR.FAMAL_YOUNGEST_QT(pers_k).EQ.1) THEN
                                            IF (FAMAL_ASSELFFS_ANN.NE.1) THEN
                                                FAMAL_ASUPSEL_QT(pers_k) = 0
                                            END IF    
                                        END IF
                                    END IF
                                END IF
                            END IF
                            !!First implementation
                            !!Look for children that are eligible for child benefits and who did generate another social supplement or who are
                            !!eligible for guaranteerd allowances and who live with single parents
                            !IF (FAMAL_CHILELIG_QT(pers_k).EQ.1.AND.(FAMAL_SOCSAMOU_QT(pers_k).NE.0.D0.OR.&
                            !&FAMAL_ORALLGUA_QT(pers_k).NE.0.D0.OR.FAMAL_SUPLSING_QT(pers_k).NE.0.D0).AND.&
                            !&FAMAL_RECIPID_QT(pers_k).EQ.GECODEERD_INSZ_INPUT(pers_j).AND.ANY(FAMREL_COUPLE(pers_j).EQ.(/0, 2/)))&
                            !& THEN
                            !    !use the rank of the child to select the appropriate amount from the relevant parameter
                            !    IF (FAMAL_CBENRANK_QT(pers_k).GT.2) THEN
                            !        FAMAL_RKWSUPL_QT(pers_k) = FAMAL_SUPRKW_MONTH(4)*DBLE(FAMAL_NMONTHBE_QT(pers_k))/4.D0
                            !    END IF
                            !END IF
                        END DO
                    END IF
                END DO
            END DO
        END IF
    END IF

END SUBROUTINE FAMAL_SUPALLOW

!====================================!
!Subroutine to create age class
!====================================!

SUBROUTINE FAMAL_AGECLASS_CREATION

    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            IF (FAMAL_YEAR_AGE(pers_j).LT.6) THEN
                FAMAL_AGECLASS(pers_j)=1
            ELSE IF (FAMAL_YEAR_AGE(pers_j).GE.6.AND.FAMAL_YEAR_AGE(pers_j).LT.12) THEN
                FAMAL_AGECLASS(pers_j)=2
            ELSE IF (FAMAL_YEAR_AGE(pers_j).GE.12.AND.FAMAL_YEAR_AGE(pers_j).LT.18) THEN
                FAMAL_AGECLASS(pers_j)=3
            ELSE IF (FAMAL_YEAR_AGE(pers_j).GE.18.AND.FAMAL_YEAR_AGE(pers_j).LT.25) THEN
                FAMAL_AGECLASS(pers_j)=4
            ELSE IF (FAMAL_YEAR_AGE(pers_j).GE.25) THEN
                FAMAL_AGECLASS(pers_j)=5
            END IF
        END DO
    END DO

END SUBROUTINE FAMAL_AGECLASS_CREATION

!======================================================!
!subroutine to skip n lines on file k (reads as blanks)
!======================================================!
SUBROUTINE RSKIP(k,n)

    INTEGER :: k,n

    DO i=1,n
        read(k,'(a)')
    END DO    

END SUBROUTINE RSKIP

!===================================================================================!
!With subroutine FAMAL_EXCHANGE we construct the variables                          !
!FAMAL_AMOUNTF_QT
!===================================================================================!

SUBROUTINE FAMAL_EXCHANGE

    INTEGER                                                 :: hh_i,pers_j,pers_k

    !Constructed exchange variable
    !FAMAL_AMOUNTF_QT: Total Level of child allowances and supplements by recipient

    !Loop over all households to accumulate the benefits of all children of a given recipient
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if a recipient has been identified for person j
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_RECIPID_QT(pers_j).NE.0) THEN
                !search for the line of the recipient
                DO pers_k=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
                    IF (FAMAL_RECIPID_QT(pers_j).EQ.GECODEERD_INSZ_INPUT(pers_k)) THEN
                        FAMAL_AMOUNTF_QT(pers_k) =   FAMAL_AMOUNTF_QT(pers_k)     + &
                                                    &FAMAL_ORALLWEA_QT(pers_j)  + &
                                                    &FAMAL_ASUPWEA_QT(pers_j)   + &
                                                    &FAMAL_SOCSAMOU_QT(pers_j)  + &
                                                    &FAMAL_HANDAMOU_QT(pers_j)  + &
                                                    &FAMAL_ORALLORP_QT(pers_j)  + &
                                                    &FAMAL_BIRTHALL_QT(pers_j)  + &
                                                    &FAMAL_ADOPALL_QT(pers_j)   + &
                                                    &FAMAL_ORALLCIV_QT(pers_j)  + &
                                                    &FAMAL_ORALLSEL_QT(pers_j)  + &
                                                    &FAMAL_ASUPSEL_QT(pers_j)   + &
                                                    &FAMAL_ORALLGUA_QT(pers_j)  + &
                                                    &FAMAL_SUPLSING_QT(pers_j)  + &
                                                    &FAMAL_SCHOOL_QT(pers_j)  + &
                                                    &FAMAL_RKWSUPL_QT(pers_j)
                        EXIT
                    END IF
                END DO
            END IF
        END DO
    END DO        

END SUBROUTINE FAMAL_EXCHANGE

!==========================================================!
!Produce some aggregate statistics to check the constuction
!==========================================================!

SUBROUTINE FAMAL_TEST

    INTEGER                                                 :: pers_j,j
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: FAMAL_RESULT_TEST                    !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: FAMAL_LAB_TEST                       !Labels of test variables
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)              :: SumAmount                            !Help vector to compute results
    REAL(KIND=DPRECI),DIMENSION(4)                          :: SumScheme                            !Help vector to compute results: contains sum per scheme

    WRITE(15,'(/,''Print of test statistics'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    
    !Compute aggregate statistics on child allowances
    NumAnal = 29

    !Allocate and initialise test result
    ALLOCATE(FAMAL_RESULT_TEST(NumAnal,4))
    ALLOCATE(FAMAL_LAB_TEST(NumAnal))
    ALLOCATE(SumAmount (NumAnal))
    FAMAL_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    FAMAL_LAB_TEST(1) = "Sum of basic child benefits in wage earner scheme                        "
    FAMAL_LAB_TEST(2) = "Sum of age supplements in wage earner scheme                             "
    FAMAL_LAB_TEST(3) = "Sum of social supplements in wage earner scheme                          "
    FAMAL_LAB_TEST(4) = "Sum of supplements for handicapped in wage earner scheme                 "
    FAMAL_LAB_TEST(5) = "Sum of orphan allowances in wage earner scheme                           "
    FAMAL_LAB_TEST(6) = "Sum of birth allowances in wage earner scheme                            "
    FAMAL_LAB_TEST(7) = "Sum of adoption allowances in wage earner scheme                         "
    FAMAL_LAB_TEST(8) = "Sum of basic child benefits in civil servant scheme                      "
    FAMAL_LAB_TEST(9) = "Sum of age supplements in civil servant scheme                           "
    FAMAL_LAB_TEST(10) = "Sum of social supplements in civil servant scheme                        "
    FAMAL_LAB_TEST(11) = "Sum of supplements for handicapped in civil servant scheme               "
    FAMAL_LAB_TEST(12) = "Sum of orphan allowances in civil servant scheme                         "
    FAMAL_LAB_TEST(13) = "Sum of birth allowances in civil servant scheme                          "
    FAMAL_LAB_TEST(14) = "Sum of adoption allowances in civil servant scheme                       "
    FAMAL_LAB_TEST(15) = "Sum of basic child benefits in self employment scheme                    "
    FAMAL_LAB_TEST(16) = "Sum of age supplements in self employment scheme                         "
    FAMAL_LAB_TEST(17) = "Sum of social supplements in self employment scheme                      "
    FAMAL_LAB_TEST(18) = "Sum of supplements for handicapped in self employment scheme             "
    FAMAL_LAB_TEST(19) = "Sum of orphan allowances in self employment scheme                       "
    FAMAL_LAB_TEST(20) = "Sum of birth allowances in self employment scheme                        "
    FAMAL_LAB_TEST(21) = "Sum of adoption allowances in self employment scheme                     "
    FAMAL_LAB_TEST(22) = "Sum of basic child benefits in guaranteed child allowance scheme         "
    FAMAL_LAB_TEST(23) = "Sum of age supplements in guaranteed child allowance scheme              "
    FAMAL_LAB_TEST(24) = "Sum of social supplements in guaranteed child allowance scheme           "
    FAMAL_LAB_TEST(25) = "Sum of supplements for handicapped in guaranteed child allowance scheme  "
    FAMAL_LAB_TEST(26) = "Sum of orphan allowances in guaranteed child allowance scheme            "
    FAMAL_LAB_TEST(27) = "Sum of birth allowances in guaranteed child allowance scheme             "
    FAMAL_LAB_TEST(28) = "Sum of adoption allowances in guaranteed child allowance scheme          "
    FAMAL_LAB_TEST(29) = "Sum of child benefit supplement paid to single parents                   "

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumAmount(1:NumAnal)=0.D0
        !Compute quarterly amounts
        IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.1) THEN
            SumAmount(1) = FAMAL_ORALLWEA_QT(pers_j)
            SumAmount(2) = FAMAL_ASUPWEA_QT(pers_j)
            SumAmount(3) = FAMAL_SOCSAMOU_QT(pers_j)
            SumAmount(4) = FAMAL_HANDAMOU_QT(pers_j)
            SumAmount(5) = FAMAL_ORALLORP_QT(pers_j)
            SumAmount(6) = FAMAL_BIRTHALL_QT(pers_j)
            SumAmount(7) = FAMAL_ADOPALL_QT(pers_j)
        ELSE IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
            SumAmount(8)  = FAMAL_ORALLCIV_QT(pers_j)
            SumAmount(9)  = FAMAL_ASUPWEA_QT(pers_j)
            SumAmount(10) = FAMAL_SOCSAMOU_QT(pers_j)
            SumAmount(11) = FAMAL_HANDAMOU_QT(pers_j)
            SumAmount(12) = FAMAL_ORALLORP_QT(pers_j)
            SumAmount(13) = FAMAL_BIRTHALL_QT(pers_j)
            SumAmount(14) = FAMAL_ADOPALL_QT(pers_j)
        ELSE IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.3) THEN
            SumAmount(15) = FAMAL_ORALLSEL_QT(pers_j)
            SumAmount(16) = FAMAL_ASUPSEL_QT(pers_j)
            SumAmount(17) = FAMAL_SOCSAMOU_QT(pers_j)
            SumAmount(18) = FAMAL_HANDAMOU_QT(pers_j)
            SumAmount(19) = FAMAL_ORALLORP_QT(pers_j)
            SumAmount(20) = FAMAL_BIRTHALL_QT(pers_j)
            SumAmount(21) = FAMAL_ADOPALL_QT(pers_j)
        ELSE IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
            SumAmount(22) = FAMAL_ORALLGUA_QT(pers_j)
            SumAmount(23) = FAMAL_ASUPWEA_QT(pers_j)
            SumAmount(24) = FAMAL_SOCSAMOU_QT(pers_j)
            SumAmount(25) = FAMAL_HANDAMOU_QT(pers_j)
            SumAmount(26) = FAMAL_ORALLORP_QT(pers_j)
            SumAmount(27) = FAMAL_BIRTHALL_QT(pers_j)
            SumAmount(28) = FAMAL_ADOPALL_QT(pers_j)
        END IF
        SumAmount(29) = FAMAL_SUPLSING_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumAmount(j)).NE.0) THEN
                FAMAL_RESULT_TEST(j,1)=FAMAL_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumAmount(j)
                FAMAL_RESULT_TEST(j,2)=FAMAL_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !convert total amount to anual values
    FAMAL_RESULT_TEST(1:NumAnal,1) = FAMAL_RESULT_TEST(1:NumAnal,1)*4.D0

    DO i=1,NumAnal
        IF (INT(FAMAL_RESULT_TEST(i,2)).NE.0) THEN
            FAMAL_RESULT_TEST(i,3)=FAMAL_RESULT_TEST(i,1)/FAMAL_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            FAMAL_RESULT_TEST(i,4)=100.D0*FAMAL_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    Write(15,'(''Annual amounts of child benefits according to scheme'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')FAMAL_LAB_TEST(i),(FAMAL_RESULT_TEST(i,j),j=1,4)
    END DO

    !convert existing labels
    FAMAL_LAB_TEST(1) = "Sum of basic child benefits                                              "
    FAMAL_LAB_TEST(2) = "Sum of age supplements                                                   "
    FAMAL_LAB_TEST(3) = "Sum of social supplements                                                "
    FAMAL_LAB_TEST(4) = "Sum of supplements for handicapped                                       "
    FAMAL_LAB_TEST(5) = "Sum of orphan allowances                                                 "
    FAMAL_LAB_TEST(6) = "Sum of birth allowances                                                  "
    FAMAL_LAB_TEST(7) = "Sum of adoption allowances                                               "

    !Compute the sum per scheme
    SumScheme(1) = SUM(FAMAL_RESULT_TEST(1:7,1))
    SumScheme(2) = SUM(FAMAL_RESULT_TEST(8:14,1))
    SumScheme(3) = SUM(FAMAL_RESULT_TEST(15:21,1))
    SumScheme(4) = SUM(FAMAL_RESULT_TEST(22:28,1))

    DO i=1,4
        IF (SumScheme(i).NE.0) THEN
            DO j=(i-1)*7+1,(i-1)*7+7
                FAMAL_RESULT_TEST(j,1)=100.D0*(FAMAL_RESULT_TEST(j,1)/SumScheme(i))
            END DO   
        ELSE
            DO j=(i-1)*7+1,(i-1)*7+7
                FAMAL_RESULT_TEST(j,1)=0
            END DO   
        END IF
    END DO

    !Write results
    Write(15,'(/,''                                                                                     Fraction'')')
    Write(15,'(''Annual amounts of child benefits according to scheme expressed as a percentage of the total amount of &
    &benefits in the scheme'')')
    Write(15,'(''                                                                                    &
    &Wage earner       Civil servant         Self employ          Guaranteed'')')
    DO i=1,7
        Write(15,'(a75,4F20.2)')FAMAL_LAB_TEST(i),FAMAL_RESULT_TEST(i,1),FAMAL_RESULT_TEST(i+7,1),&
        &FAMAL_RESULT_TEST(i+14,1),FAMAL_RESULT_TEST(i+21,1)
    END DO

    DEALLOCATE(FAMAL_RESULT_TEST)
    DEALLOCATE(FAMAL_LAB_TEST)
    DEALLOCATE(SumAmount)

!Aangepast bij herwerking september 2011
!Onderstaande tests hoeven niet meer voldaan te zijn, na opname gegevens gehandicapten
!i.e. gehandicapte rechthebbenden kunnen sociaal supplement openen zonder in een eerder bijdrageregime te
!vallen
    !Test: Is there a beneficiary for each orphan?
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_ORPHAN_QT(pers_j).GT.0) THEN
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.0) THEN
                    write(15,'(''Individual '', i10, '' is an orphan eligible for child benefits &
                    &but without contribution status for the beneficiary'')') pers_j
                END IF
            END IF
        END DO
    END DO
    
    !Test: Is each status for social supplement associated with a contribution scheme?
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and orphan
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHRP_QT(pers_j).GT.0) THEN
                IF (FAMAL_BENSCHIL_QT(pers_j).EQ.0) THEN
                    write(15,'(''Individual '', i10, '' has a beneficiary which is eligible for social supplements &
                    &but without contribution status for the beneficiary'')') pers_j
                END IF
            END IF
        END DO
    END DO

END SUBROUTINE FAMAL_TEST

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other sources
!==========================================================!

SUBROUTINE FAMAL_COMPARE

    !variables for computation of aggregates
    Integer                                                             :: NumLijn,NumCol,i,j,pers_j
    REAL(KIND=DPRECI)                                                   :: SumW                           !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: FAMAL_TESTINFO                 !Information read from external source and used to judge the quality of the reconstructed concepts     
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: FAMAL_TESTCOMP                 !Recomputed test statistics (comparable with content of TESTINFO file
    CHARACTER(len=100),ALLOCATABLE,DIMENSION(:)                         :: FAMAL_LAB_COMPARE              !Vector with labels of the different reconstructed aggregates                       
    CHARACTER(len=20)                                                   :: CHAR_SCAL                      !Store scalar value in character format
    CHARACTER(len=200)                                                  :: CHAR_LINE                      !Construct line with character values
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: RESULT_DIF                     !Compute the difference between the computed value and the real one
 
    WRITE(15,'(/,''Print statistics comparable with other sources'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

!Aangepast bij herwerking september 2011
!Start constructie Tabel 1
!Info over aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor werknemers (toestand op 30 juni 2008)

    NumLijn = 5
    NumCol  = 6
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    OPEN(UNIT=5, FILE=FAMAL_2008, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_2008
        WRITE(15,'("Could not find ", a100)') FAMAL_2008
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        !skip first 4 lines
        CALL RSKIP(5,4)
        DO i=1,NumLijn
            READ(5,'(a40,6f15.0)',IOSTAT=ierror) &  
            &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
            END IF
        END DO
    END IF    

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 1a: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor werknemers &
    &(toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 179'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.1) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.(FAMAL_BENSCHRP_QT(pers_j).EQ.0.OR.FAMAL_SOCSAMOU_QT(pers_j).EQ.0.D0)) THEN
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(1,j) = FAMAL_TESTCOMP(1,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(2,j) = FAMAL_TESTCOMP(2,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0.AND.FAMAL_ORALLORP_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(3,j) = FAMAL_TESTCOMP(3,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(4,j) = FAMAL_TESTCOMP(4,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                END IF
            END IF        
        END DO
    END DO  
    !compute total in last column or line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    DO i=1,NumLijn
        FAMAL_TESTCOMP(i,NumCol) = SUM(FAMAL_TESTCOMP(i,1:NumCol-1))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 1b: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor werknemers'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 1c: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor werknemers'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a130)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 2
!Info over aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor werknemers (toestand op 30 juni 2008)

    NumLijn = 5
    NumCol  = 6
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,6f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 2a: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor werknemers &
    &(toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 180'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.1) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.(FAMAL_BENSCHRP_QT(pers_j).EQ.0.OR.FAMAL_SOCSAMOU_QT(pers_j).EQ.0.D0)) THEN
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(1,j) = FAMAL_TESTCOMP(1,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(1,5) = FAMAL_TESTCOMP(1,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(2,j) = FAMAL_TESTCOMP(2,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(2,5) = FAMAL_TESTCOMP(2,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0.AND.FAMAL_ORALLORP_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(3,j) = FAMAL_TESTCOMP(3,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(3,5) = FAMAL_TESTCOMP(3,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(4,j) = FAMAL_TESTCOMP(4,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(4,5) = FAMAL_TESTCOMP(4,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END IF
            END IF        
        END DO
    END DO  
    !compute total in last column or line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    DO i=1,NumLijn
        FAMAL_TESTCOMP(i,NumCol) = SUM(FAMAL_TESTCOMP(i,1:NumCol-1))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 2b: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor werknemers '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 2c: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor werknemers '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a130)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 3
!Info over aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor zelfstandigen (toestand op 30 juni 2008)

    NumLijn = 5
    NumCol  = 6
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,6f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 3a: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor zelfstandigen &
    &(toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 295'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a self employed
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.3) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.(FAMAL_BENSCHRP_QT(pers_j).EQ.0.OR.FAMAL_SOCSAMOU_QT(pers_j).EQ.0.D0)) THEN
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(1,j) = FAMAL_TESTCOMP(1,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(2,j) = FAMAL_TESTCOMP(2,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0.AND.FAMAL_ORALLORP_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(3,j) = FAMAL_TESTCOMP(3,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                !if the beneficiary is entitled to pension or particular unemployment benefits  
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,5
                        IF (FAMAL_AGECLASS(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(4,j) = FAMAL_TESTCOMP(4,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                END IF
            END IF        
        END DO
    END DO  
    !compute total in last column or line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    DO i=1,NumLijn
        FAMAL_TESTCOMP(i,NumCol) = SUM(FAMAL_TESTCOMP(i,1:NumCol-1))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 3b: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor zelfstandigen'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 3c: Aantal rechtgevende kinderen naar schaal en leeftijdsklasse in het stelsel voor zelfstandigen'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''Klasse is gelijk aan of groter dan ondergrens en kleiner dan bovengrens'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                               0-6 jaar      6-12 jaar     12-18 jaar     18-25 jaar&
    &      + 25 jaar         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a130)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 4
!Info over aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor zelfstandigen (toestand op 30 juni 2008)

    NumLijn = 5
    NumCol  = 6
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,6f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 4a: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor zelfstandigen &
    &(toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 296'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a self employed
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.3) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.(FAMAL_BENSCHRP_QT(pers_j).EQ.0.OR.FAMAL_SOCSAMOU_QT(pers_j).EQ.0.D0)) THEN
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(1,j) = FAMAL_TESTCOMP(1,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(1,5) = FAMAL_TESTCOMP(1,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(2,j) = FAMAL_TESTCOMP(2,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(2,5) = FAMAL_TESTCOMP(2,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0.AND.FAMAL_ORALLORP_QT(pers_j).GT.0.D0) THEN 
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(3,j) = FAMAL_TESTCOMP(3,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(3,5) = FAMAL_TESTCOMP(3,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))&
                &.AND.FAMAL_SOCSAMOU_QT(pers_j).GT.0.D0) THEN  
                    DO j=1,4
                        IF (FAMAL_CBENRANK_QT(pers_j).EQ.j) THEN
                            FAMAL_TESTCOMP(4,j) = FAMAL_TESTCOMP(4,j) + MIMOSIS_WEIGHT(pers_j)                    
                        END IF    
                    END DO
                    IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                        FAMAL_TESTCOMP(4,5) = FAMAL_TESTCOMP(4,5) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END IF
            END IF        
        END DO
    END DO  
    !compute total in last column or line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    DO i=1,NumLijn
        FAMAL_TESTCOMP(i,NumCol) = SUM(FAMAL_TESTCOMP(i,1:NumCol-1))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 4b: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor zelfstandigen '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,6f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 4c: Aantal rechtgevende kinderen naar schaal en rang van het kind in het stelsel voor zelfstandigen '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                              1ste kind       2de kind       3de kind       4de kind &
    &5de en volgend         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a130)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 5
!Info over aantal rechtgevende kinderen naar schaal in het stelsel van de openbare sector (toestand op 31 december 2008)

    NumLijn = 5
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 5a: Aantal rechtgevende kinderen naar schaal in het stelsel van de openbare sector &
    &(toestand op 31 december 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 362'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a civil servant
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.FAMAL_BENSCHRP_QT(pers_j).EQ.0) THEN
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)                    
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))) THEN 
                    FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)                    
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0) THEN 
                    FAMAL_TESTCOMP(3,1) = FAMAL_TESTCOMP(3,1) + MIMOSIS_WEIGHT(pers_j)                    
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))) THEN 
                    FAMAL_TESTCOMP(4,1) = FAMAL_TESTCOMP(4,1) + MIMOSIS_WEIGHT(pers_j)                    
                END IF
            END IF        
        END DO
    END DO  
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 5b: Aantal rechtgevende kinderen naar schaal in het stelsel van de openbare sector '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 5c: Aantal rechtgevende kinderen naar schaal in het stelsel van de openbare sector '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 6
!Info over aantal rechtgevende kinderen naar rang van het kind in het stelsel van de openbare sector (toestand op 31 december 2008)

    NumLijn = 4
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 6a: Aantal rechtgevende kinderen naar rang van het kind in het stelsel van de openbare sector &
    &(toestand op 31 december 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 362'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a civil servant
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
                DO i=1,2
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.i) THEN
                        FAMAL_TESTCOMP(i,1) = FAMAL_TESTCOMP(i,1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END DO
                IF (FAMAL_CBENRANK_QT(pers_j).GE.3) THEN
                    FAMAL_TESTCOMP(3,1) = FAMAL_TESTCOMP(3,1) + MIMOSIS_WEIGHT(pers_j)                    
                END IF    
            END IF        
        END DO
    END DO  
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 6b: Aantal rechtgevende kinderen naar rang van het kind in het stelsel van de openbare sector '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 6c: Aantal rechtgevende kinderen naar rang van het kind in het stelsel van de openbare sector '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 7
!Info over aantal rechtgevende kinderen naar leeftijdsklasse in het stelsel van de openbare sector (toestand op 31 december 2008)

    NumLijn = 7
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 7a: Aantal rechtgevende kinderen naar leeftijdsklasse in het stelsel van de openbare sector &
    &(toestand op 31 december 2008))'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 362'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a civil servant
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
                DO i=1,5
                    IF (FAMAL_AGECLASS(pers_j).EQ.i) THEN
                        FAMAL_TESTCOMP(i,1) = FAMAL_TESTCOMP(i,1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END DO
            END IF        
        END DO
    END DO  
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 7b: Aantal rechtgevende kinderen naar leeftijdsklasse in het stelsel van de openbare sector'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 7c: Aantal rechtgevende kinderen naar leeftijdsklasse in het stelsel van de openbare sector'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 8
!Info over aantal rechtgevende kinderen naar rang van het kind in de gewaarborgde gezinsbijslag (toestand op 30 juni 2000)

    NumLijn = 6
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 8a: Aantal rechtgevende kinderen naar rang van het kind in de gewaarborgde gezinsbijslag &
    &(toestand op 30 juni 2000)'')')
    Write(15,'(''Bron: Statistisch jaarboek van de Sociale Zekerheid (editie 2000) &
    &p. 278'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a civil servant
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
                DO i=1,4
                    IF (FAMAL_CBENRANK_QT(pers_j).EQ.i) THEN
                        FAMAL_TESTCOMP(i,1) = FAMAL_TESTCOMP(i,1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END DO
                IF (FAMAL_CBENRANK_QT(pers_j).GE.5) THEN
                    FAMAL_TESTCOMP(5,1) = FAMAL_TESTCOMP(5,1) + MIMOSIS_WEIGHT(pers_j)                    
                END IF    
            END IF        
        END DO
    END DO  
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 8b: Aantal rechtgevende kinderen naar rang van het kind in de gewaarborgde gezinsbijslag '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 8c: Aantal rechtgevende kinderen naar rang van het kind in de gewaarborgde gezinsbijslag '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 9
!Info over aantal rechtgevende kinderen naar leeftijdsklasse in de gewaarborgde gezinsbijslag (toestand op 30 juni 2000)

    NumLijn = 5
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 9a: Aantal rechtgevende kinderen naar leeftijdsklasse in de gewaarborgde gezinsbijslag &
    &(toestand op 30 juni 2000)'')')
    Write(15,'(''Bron: Statistisch jaarboek van de Sociale Zekerheid (editie 2000) &
    &p. 278'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a civil servant
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
                DO i=1,4
                    IF (FAMAL_AGECLASS(pers_j).EQ.i) THEN
                        FAMAL_TESTCOMP(i,1) = FAMAL_TESTCOMP(i,1) + MIMOSIS_WEIGHT(pers_j)                    
                    END IF    
                END DO
            END IF        
        END DO
    END DO  
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO

    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 9b: Aantal rechtgevende kinderen naar leeftijdsklasse in de gewaarborgde gezinsbijslag '')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 9c: Aantal rechtgevende kinderen naar leeftijdsklasse in de gewaarborgde gezinsbijslag '')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 10
!Info over kinderbijslag in het stelsel van werknemers in Euro naar schaal (toestand op 30 juni 2008)

    NumLijn = 7
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 6 lijnen
    CALL RSKIP(5,6)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 10a: Kinderbijslag in het stelsel van werknemers in Euro naar schaal (toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 175-176 (combinatie van Tabel 5.1 en 5.2)'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

!Aangepast bij herwerking september 2011
    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.1) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.FAMAL_BENSCHRP_QT(pers_j).EQ.0) THEN
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLWEA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))) THEN 
                    FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLWEA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0) THEN 
                    FAMAL_TESTCOMP(3,1) = FAMAL_TESTCOMP(3,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLORP_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))) THEN 
                    FAMAL_TESTCOMP(4,1) = FAMAL_TESTCOMP(4,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLWEA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                END IF
                FAMAL_TESTCOMP(5,1) = FAMAL_TESTCOMP(5,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_BIRTHALL_QT(pers_j))
                FAMAL_TESTCOMP(6,1) = FAMAL_TESTCOMP(6,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_ADOPALL_QT(pers_j))
            END IF        
        END DO
    END DO 
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    !convert total amount to anual values
    FAMAL_TESTCOMP(1:NumLijn,NumCol) = FAMAL_TESTCOMP(1:NumLijn,NumCol)*4.D0
    
    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 10b: Kinderbijslag in het stelsel van werknemers in Euro naar schaal'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 10c: Kinderbijslag in het stelsel van werknemers in Euro naar schaal'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 11
!Info over kinderbijslag in het stelsel van zelfstandigen in Euro naar schaal (toestand op 30 juni 2008)

    NumLijn = 7
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 6 lijnen
    CALL RSKIP(5,6)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 11a: Kinderbijslag in het stelsel van zelfstandigen in Euro naar schaal (toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 292 (combinatie van Tabel 5.1 en 5.2)'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

!Aangepast bij herwerking september 2011
    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.3) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.FAMAL_BENSCHRP_QT(pers_j).EQ.0) THEN
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLSEL_QT(pers_j) + FAMAL_ASUPSEL_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))) THEN 
                    FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLSEL_QT(pers_j) + FAMAL_ASUPSEL_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0) THEN 
                    FAMAL_TESTCOMP(3,1) = FAMAL_TESTCOMP(3,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLORP_QT(pers_j) + FAMAL_ASUPSEL_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))) THEN 
                    FAMAL_TESTCOMP(4,1) = FAMAL_TESTCOMP(4,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLSEL_QT(pers_j) + FAMAL_ASUPSEL_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                END IF
                FAMAL_TESTCOMP(5,1) = FAMAL_TESTCOMP(5,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_BIRTHALL_QT(pers_j))
                FAMAL_TESTCOMP(6,1) = FAMAL_TESTCOMP(6,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_ADOPALL_QT(pers_j))
            END IF        
        END DO
    END DO 
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    !convert total amount to anual values
    FAMAL_TESTCOMP(1:NumLijn,NumCol) = FAMAL_TESTCOMP(1:NumLijn,NumCol)*4.D0
    
    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 11b: Kinderbijslag in het stelsel van zelfstandigen in Euro naar schaal'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 11c: Kinderbijslag in het stelsel van zelfstandigen in Euro naar schaal'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    WRITE(15,'(''Kinderbijslag'')')
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 12
!Info over kinderbijslag in het stelsel van de openbare sector in Euro naar schaal (toestand op 30 juni 2008)

    NumLijn = 3
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 12a: Kinderbijslag in het stelsel van de openbare sector in Euro (toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 361'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

!Aangepast bij herwerking september 2011
    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.2) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.FAMAL_BENSCHRP_QT(pers_j).EQ.0) THEN
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLCIV_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLCIV_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLORP_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLCIV_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                END IF
                FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_BIRTHALL_QT(pers_j))
                FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_ADOPALL_QT(pers_j))
            END IF        
        END DO
    END DO 
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    !convert total amount to anual values
    FAMAL_TESTCOMP(1:NumLijn,NumCol) = FAMAL_TESTCOMP(1:NumLijn,NumCol)*4.D0
    
    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 12b: Kinderbijslag in het stelsel van de openbare sector in Euro'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 12c: Kinderbijslag in het stelsel van de openbare sector in Euro'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF)     

!Start constructie Tabel 13
!Info over kinderbijslag in het stelsel van de gewaarborgde gezinsbijslag in Euro naar schaal (toestand op 30 juni 2008)

    NumLijn = 3
    NumCol  = 1
    ALLOCATE(FAMAL_TESTINFO(NumLijn,NumCol))       
    ALLOCATE(FAMAL_TESTCOMP(NumLijn,NumCol))
    ALLOCATE(FAMAL_LAB_COMPARE(NumLijn))     
    ALLOCATE(RESULT_DIF(NumCol))     
    
    !Read information from external source
    !skip volgende 5 lijnen
    CALL RSKIP(5,5)
    DO i=1,NumLijn
        READ(5,'(a40,f15.0)',IOSTAT=ierror) &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol) 
        IF (ierror > 0) THEN
            WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,FAMAL_2008
        END IF
    END DO

    !Write information from external file
    Write(15,'('' '')')
    Write(15,'(''Tabel 13a: Kinderbijslag in het stelsel van de gewaarborgde gezinsbijslag in Euro (toestand op 30 juni 2008)'')')
    Write(15,'(''Bron: Vademecum van de financile en statistische gegevens over de sociale bescherming in Belgi &
    &(periode 2000-2006) p. 175-176 (combinatie van Tabel 5.1 en 5.2)'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTINFO(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

!Aangepast bij herwerking september 2011
    !Recompute results
    !loop over all households
    FAMAL_TESTCOMP(1:NumLijn,1:NumCol) = 0
    DO hh_i=1,N_HH_FAMAL
        !Loop over all persons within the household
        DO pers_j=FAMAL_START_HH(hh_i),FAMAL_END_HH(hh_i)
            !if the child is eligible for child benefits and the beneficiary is a wage earner
            IF (FAMAL_CHILELIG_QT(pers_j).EQ.1.AND.FAMAL_BENSCHIL_QT(pers_j).EQ.4) THEN
                !if the child is not an orphan and the beneficiary is not eligible for a social supplement
                IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.FAMAL_BENSCHRP_QT(pers_j).EQ.0) THEN
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLGUA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is disabled
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/3, 4/))) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLGUA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the child is an orphan
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).GT.0) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLORP_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                !if the beneficiary is entitled to pension or particular unemployment benefits    
                ELSE IF (FAMAL_ORPHAN_QT(pers_j).EQ.0.AND.ANY(FAMAL_BENSCHRP_QT(pers_j).EQ.(/1, 2/))) THEN 
                    FAMAL_TESTCOMP(1,1) = FAMAL_TESTCOMP(1,1) + MIMOSIS_WEIGHT(pers_j)*&
                    &(FAMAL_ORALLGUA_QT(pers_j) + FAMAL_ASUPWEA_QT(pers_j) + FAMAL_SOCSAMOU_QT(pers_j) + FAMAL_HANDAMOU_QT(pers_j)&
                    &+ FAMAL_SUPLSING_QT(pers_j) + FAMAL_SCHOOL_QT(pers_j))
                END IF
                FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_BIRTHALL_QT(pers_j))
                FAMAL_TESTCOMP(2,1) = FAMAL_TESTCOMP(2,1) + MIMOSIS_WEIGHT(pers_j)*(FAMAL_ADOPALL_QT(pers_j))
            END IF        
        END DO
    END DO 
    !compute total in last line
    DO j=1,NumCol
        FAMAL_TESTCOMP(NumLijn,j) = SUM(FAMAL_TESTCOMP(1:NumLijn-1,j))    
    END DO
    !convert total amount to anual values
    FAMAL_TESTCOMP(1:NumLijn,NumCol) = FAMAL_TESTCOMP(1:NumLijn,NumCol)*4.D0
    
    !Write recomputed results
    Write(15,'('' '')')
    Write(15,'(''Tabel 13b: Kinderbijslag in het stelsel van de gewaarborgde gezinsbijslag in Euro'')')
    Write(15,'(''Bron: resultaten berekend met de FAMAL module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        WRITE(15,'(a40,f15.0)') &  
        &FAMAL_LAB_COMPARE(i),(FAMAL_TESTCOMP(i,j),j=1,NumCol)            
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results
    Write(15,'('' '')')
    Write(15,'(''Tabel 13c: Kinderbijslag in het stelsel van de gewaarborgde gezinsbijslag in Euro'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = getal in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''                                                 Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,NumLijn
        RESULT_DIF(1:NumCol) = 0
        CHAR_LINE(1:40) = FAMAL_LAB_COMPARE(i)
        DO j=1,NumCol
            CHAR_SCAL = "                    "
            IF (FAMAL_TESTINFO(i,j).NE.0) THEN
                RESULT_DIF(j) = 100.D0*(FAMAL_TESTCOMP(i,j)-FAMAL_TESTINFO(i,j))/FAMAL_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') RESULT_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:40+j*15) = CHAR_LINE(1:40+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a55)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

    DEALLOCATE(FAMAL_TESTINFO)      
    DEALLOCATE(FAMAL_TESTCOMP)
    DEALLOCATE(FAMAL_LAB_COMPARE)    
    DEALLOCATE(RESULT_DIF) 
    
    CLOSE(5)

END SUBROUTINE FAMAL_COMPARE

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=======================================================!

SUBROUTINE FAMAL_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN(UNIT=12, FILE=CONTRIB_FAMAL,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_FAMAL
        WRITE(15,'("Could not find ", a100)') CONTRIB_FAMAL
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=13, FILE=SICK_FAMAL,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SICK_FAMAL
        WRITE(15,'("Could not find ", a100)') SICK_FAMAL
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=14, FILE=UNEM_FAMAL,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_FAMAL
        WRITE(15,'("Could not find ", a100)') UNEM_FAMAL
        PAUSE
        STOP
    END IF        
    OPEN(UNIT=16, FILE=SOCBEN_FAMAL,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_FAMAL
        WRITE(15,'("Could not find ", a100)') SOCBEN_FAMAL
        PAUSE
        STOP
    END IF        

    DO pers_j=1,n_rec
        WRITE(12,'(A14,f15.2)') GECODEERD_INSZ_CHAR(pers_j),FAMAL_AMOUNTF_QT(pers_j)
        WRITE(13,'(A14,f15.2)') GECODEERD_INSZ_CHAR(pers_j),FAMAL_AMOUNTF_QT(pers_j)
        WRITE(14,'(A14,f15.2)') GECODEERD_INSZ_CHAR(pers_j),FAMAL_AMOUNTF_QT(pers_j) 
        WRITE(16,'(A14,2I10)')  GECODEERD_INSZ_CHAR(pers_j),FAMAL_CHILELIG_QT(pers_j),FAMAL_ORPHAN_QT(pers_j)
    END DO

    CLOSE(UNIT=12)
    CLOSE(UNIT=13)    
    CLOSE(UNIT=14)    
    CLOSE(UNIT=16)    

    WRITE(*,*) 'Subroutine FAMAL_OUTPUT_FILEWRIT of PRE RUN...........runned correctly'

END SUBROUTINE FAMAL_OUTPUT_FILEWRIT

!==========================================================!
!The subroutine FAMAL_CALC is used to call the other       ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE FAMAL_PRERUN_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS FAMAL PRE RUN IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=FAMAL_PRERUN_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') FAMAL_LOG
        WRITE(15,'("Could not find ", a100)') FAMAL_LOG
        PAUSE
        STOP
    END IF

    !Allocate all exogenous vectors with dimension n_rec
    CALL FAMAL_ALLOC_EXOG(n_rec)
    
    !Reads in PARAMETER file
    CALL FAMAL_PARAMETER_FILEREAD
    !Read in files with input data
    CALL FAMAL_INPUT_FILEREAD
    CALL FAMAL_SICK_FILEREAD
    CALL FAMAL_UNEM_FILEREAD
    CALL FAMAL_CONTRIB_FILEREAD
    CALL FAMAL_PENSWELF_FILEREAD
    CALL FAMAL_FAMREL_FILEREAD
    CALL FAMAL_SOCBEN_FILEREAD
    CALL FAMAL_WAGE_FILEREAD
    CALL FAMAL_WEIGHT_FILEREAD

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec
    CALL FAMAL_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL FAMAL_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL FAMAL_RANKANDSORT

    !Recode missing values
    CALL FAMAL_RECODE
    !Inflate values
    CALL FAMAL_INFLATE

    !Determine household size  
    CALL FAMAL_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL FAMAL_AGE

    !Create age classes
    CALL FAMAL_AGECLASS_CREATION
    
    !Run routine to construct identification variables necessary to compute family allowances             
    CALL FAMAL_CHARAC

    !Compute basic allowances
    CALL FAMAL_BASALLOW

    !Compute suplements
    CALL FAMAL_SUPALLOW    

    !Construct exchange concepts
    CALL FAMAL_EXCHANGE

    !test construction of different contriubtion concepts
    CALL FAMAL_TEST
    CALL FAMAL_COMPARE

    !Write output to output file
    CALL FAMAL_OUTPUT_FILEWRIT

    !Deallocate allocated variables
    CALL FAMAL_DEALLOC
    CLOSE(15)

END SUBROUTINE FAMAL_PRERUN_CALC

END MODULE FAMAL_PRERUN_SUBROUTINES
