!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declares all variables used in the program   !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE PIT_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=178
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !Define number of deciles that have to be constructed                    !
    INTEGER, PARAMETER :: NUMBDEC = 100                                      !   
    !==========================================================================

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: PIT_INPUT_PATH                       !Path name where file with PIT input data is stored
    CHARACTER(len=150)                                      :: PIT_INPUT_FILE                       !File name of file with PIT input data
    CHARACTER(len=150)                                      :: PIT_INPUT                            !Internal name to store Path+Name of file with PIT input data
    CHARACTER(len=150)                                      :: PIT_CONTRIB_FILE                     !File name of file with CONTRIB input data
    CHARACTER(len=150)                                      :: PIT_CONTRIB                          !Internal name to store Path+Name of file with CONTRIB input data
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: PITTNTI_2008_FILE                    !File name of file with PIT reference values
    CHARACTER(len=150)                                      :: PITTNTI_2008                         !Internal name to store Path+Name of file with PIT reference values
    !Parameters for locating output (local variables)
    CHARACTER(len=150)                                      :: SOCBEN_PIT_FILE                      !Name of file with PIT variables for MINIMEX module
    CHARACTER(len=150)                                      :: SOCBEN_PIT                           !Path+Name of file with PIT variables for MINIMEX module
    CHARACTER(len=150)                                      :: EVAL_PIT_FILE                        !Name of file with PIT variables for EVAL module
    CHARACTER(len=150)                                      :: EVAL_PIT                             !Path+Name of file with PIT variables for EVAL module
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: PIT_PAR_FORMAT                       !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: PIT_PAR_BREAK='@'                    !Character used to identify end of string
    CHARACTER(len=10)                                       :: PIT_PAR_CURR                         !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: PIT_PAR_NAMES                        !Vector of parameter names
    INTEGER                                                 :: PIT_PAR_L                            !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: PIT_PAR_C                            !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(12)                                   :: PIT_PAR_BREAK_POINT                  !Number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the PIT module
    !Reconstruction of tax units 
    REAL(KIND=KIND_REAL)                                    :: PIT_CRATEREP_ANN                     !Fraction used to determine net own replacement income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RALUMPEM_ANN                     !Brackets and rates applied on gross taxable income of employees to determine a lump sum amount of expenses made to earn this income amount
    REAL(KIND=KIND_REAL)                                    :: PIT_RATEEMP_ANN                      !Fixed fraction used to determine expenses made to earn employment income
    REAL(KIND=KIND_REAL)                                    :: PIT_MINCOST_ANN                      !Minimal amount of costs that can be made to earn employment income
    REAL(KIND=KIND_REAL)                                    :: PIT_RALUMPSE_ANN                     !Rate applied on gross taxable income of self employed to determine a lump sum amount of expenses made to earn this income amount
    REAL(KIND=KIND_REAL)                                    :: PIT_MAXSELF_ANN                      !Maximal amount of costs that can be made to earn self employment income
    REAL(KIND=KIND_REAL)                                    :: PIT_RATESEL_ANN                      !Fixed fraction used to determine expenses made to earn self employment income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_MAXOWNNG_ANN                     !Income ceilings used to compare net own means of potential tax dependents (other than parents and grandparents)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_MAXOWNG_ANN                      !Income ceilings used to compare net own means of parents and grandparents
    !Determination of net taxable income     
    REAL(KIND=KIND_REAL)                                    :: PIT_PERMASPL_ANN                     !Percentage of net taxable income that can be redistributed from one spouse to the other in case of the marital splitting rule
    REAL(KIND=KIND_REAL)                                    :: PIT_ABSMASPL_ANN                     !Maximal absolute amount that can be redistributed between spouses in case of the marital splitting rule
    !Determination of gross personal income taxes    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RATSCHEM_ANN                     !Tax rate scheme applied on net taxable income to determine gross personal income taxes
    !Determination of tax credits    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMSCH_ANN                      !Tax exemption levels for children that vary with the rank of the child
    REAL(KIND=KIND_REAL)                                    :: PIT_EXECHLA_ANN                      !Tax exemption for children below the age of three for which the taxpayer does not declare deductible costs
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEMPS_ANN                       !Basic tax exemption in case the taxpayer is single
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEMPC_ANN                       !Basic tax exemption for each partner if the taxpayers are a couple
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEOTH_ANN                       !Basic tax exemption for tax dependent persons other than children
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEHANSP_ANN                     !Basic tax exemption if spouse is handicaped
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEHANOT_ANN                     !Basic tax exemption for other people than spouse and children
    REAL(KIND=KIND_REAL)                                    :: PIT_EXEOLD_ANN                       !Basic tax exemption for tax dependent ascendants older than 65
    REAL(KIND=KIND_REAL)                                    :: PIT_EXESDEP_ANN                      !Basic tax exemption when tax payer is single and has dependent children
    !Credits in case of replacement income   
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_MAXREPL_ANN                      !Maximal amounts of tax credits that can be applied on different types of replacement income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RREDUCE_ANN                      !Rates used to determine reduced tax credits on replacement income other than normal unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RUNEMPL_ANN                      !Rates used to determine reduced tax credits on replacement income of normal unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_UNIQREPL_ANN                     !Levels of net taxable income to determine tax credits in case of a unique source of replacement income
    !Other interventions to determine the final personal income tax bill 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RATECRIS_ANN                     !Rates used to determine the crisis surcharge
    !Tax units that are not liable to enter a tax form   
    REAL(KIND=KIND_REAL)                                    :: PIT_BESINGLE_ANN                     !Tax exempted level of income of the tax unit if taxpayer is single
    REAL(KIND=KIND_REAL)                                    :: PIT_BECOUPLE_ANN                     !Tax exempted level of income of the tax unit if taxpayer is married
    !New parameters introduced to model 2008 system
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_TAXFREEB_ANN                     !Values required to determine the new basic exemption from the bottom up
    REAL(KIND=KIND_REAL)                                    :: PIT_CREDREDU_ANN                     !rate used to reduce tax reductions after adaptation of basic exemptions in 2007
    REAL(KIND=KIND_REAL)                                    :: PIT_REBTACR_ANN                      !maximal amount that can be rebated per child in charge
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_SPECCON_ANN                      !elements required to compute the special social security contribution
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_JOBREDU_ANN                      !matrix necessary to compute the Flemish reduction on labour income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_TAXCREWE_ANN                     !Elements to compute the tax credit for low labour income of employees
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_TAXCRESE_ANN                     !Elements to compute the tax credit for low labour income of self employed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_TAXCRECS_ANN                     !Elements to compute the tax credit for low labour income of civil servants with statutory service
    REAL(KIND=KIND_REAL)                                    :: PIT_COMUNTAX_ANN                     !fixed rate used to compute community taxes
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_TAXFREHA_ANN                     !Values required to determine the new basic exemption from the bottom up in case of a taxpayer or spouse that is disabled
    !New parameters simulation HQ
    INTEGER                                                 :: PIT_SIMHQ                            !Extra parameter die onderscheid maakt tussen scenario's
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_ABSHQ_ANN                        !Extra bedragen voor toepassing huwelijksquotint
    !=======================================================================!  
    !Exogenous variables
    !PIT_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_RELATION                    ! Verband tussen het individu en het huishoudhoofd op 1 januari 2009
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_COMMUNIT                    ! NIS gemeentecode van de woonplaats waar het individu op 1 januari 2009 woonde
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InFODSZ                              ! Individu geselecteerd uit n van de drie basisbestanden FODSZ voor verwerking steekproef
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: COSTRATE                             ! Fractie waarmee geobserveerd belastbaar netto inkomen vermenigvuldigd wordt om niet geobserveerde kosten en inkomens te benaderen
    !Variables read from file PIT_CONTRIB.txt                                            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Numerical part of the Personal identification number read from the CONTRIB data set    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPRIV_QT                   !Gross taxable labour income earned as wage earner on the private labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPUB_QT                    !Gross taxable labour income earned as wage earner on the public labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISELF_QT                   !Gross taxable labour income earned as self employed
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIPENS_QT                   !Gross taxable pension benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEARNE_QT                  !Gross taxable early retirement benefits of the new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIEAROL_QT                  !Gross taxable early retirement benefits of the old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOLDUN_QT                  !Gross taxable unemployment benefits of older unemployed with seniority supplement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIOTHUN_QT                  !Gross taxable unemployment benefits, other than early retirement benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIDISAB_QT                  !Gross taxable disability benefits of wage earners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTINDOCC_QT                  !Gross taxable benefits in case of industrial accidents or occupational diseases
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTISICK_QT                   !Gross taxable benefits in case of sickness other than disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_GTIFAMAL_QT                  !Gross taxable income from family allowances
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PRIV_QT             !Labour market status of a wage earner on the private labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PUB_QT              !Labour market status of a wage earner on the public labour market           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_SELFPCR_QT                   !Status of the self employed         
    !Variables read from file MIMOSIS_FAMREL.txt                                                  
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_YEAR_AGE                         !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_MONTH_AGE                        !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_MONTH_BIRTH                      !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_YEAR_BIRTH                       !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_HH_SIZE                          !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_START_HH                         !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_END_HH                           !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_PIT                             !number of households analysed
    !To construct variables listed in the module note
    !Reconstruction of tax units 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TURANK_ANN                       !Rank of the individual within the tax unit the individual belongs to
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TUNUMB_ANN                       !Unique identification number of the tax unit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_PARTID_ANN                       !Identifier to identify partner of head of the tax unit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_NUMTAXU_ANN                      !Variable that contains the number of tax units in the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_HQOLDNEW                         !Variable that contains value that allows to distinguish old and new HQ beneficiary
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWN_ANN                       !Net own means of the individual (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNOC_ANN                     !Net own means of occupational income sources (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNRE_ANN                     !Net own replacement income of the individual (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNEM_ANN                     !Net own means of activity as employee (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNSE_ANN                     !Net own means of activity as self employed (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTRTEM_ANN                     !Estimate of costs made to earn income as employee, estimated with cost rates (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTEMP_ANN                      !Final estimate of costs made to earn income as employee (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTRTSE_ANN                     !Estimate of costs made to earn income as self employed, estimated with cost rates (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTSELF_ANN                     !Final estimate of costs made to earn income as self employed (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNOT_ANN                     !Net own means from taxable income sources other than occupational income (variable for testing tax dependency)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETOWNNT_ANN                     !Net own means from non-taxable income sources (variable for testing tax dependency)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_HANDICAP_ANN                     !Identification of having an handicap (yes = 1, no = 0)
    !Variables at tax unit level before personal income tax computation  
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_COUPLE_ANN                       !Head of the tax unit is legally married with a partner that is comprised by the tax unit as well
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHILD_ANN                     !Number of children that are dependent according to the personal income tax legislation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPOTHER_ANN                     !Number of individuals, other than children, that are dependent according to the personal income tax legislation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPOLDER_ANN                     !Number of dependent ascendants up to the second degree older than 65
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHIL3_ANN                     !Number of children below the age of 3 that are dependent according to the personal income tax legislation
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHIHA_ANN                     !Number of children that are dependent according to the personal income tax legislation AND are disabled
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPOTHHA_ANN                     !Number of individuals, other than children, that are dependent according to the personal income tax legislation AND are disabled
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPRIV_ANN_SP1                  !Gross taxable labour income earned as wage earner on the private labour market by spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPUB_ANN_SP1                   !Gross taxable labour income earned as wage earner on the public labour market by spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTISELF_ANN_SP1                  !Gross taxable labour income earned as self employed by spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPENS_ANN_SP1                  !Gross taxable pension benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIEARNE_ANN_SP1                 !Gross taxable early retirement benefits of the new type of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIEAROL_ANN_SP1                 !Gross taxable early retirement benefits of the old type of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIOLDUN_ANN_SP1                 !Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIOTHUN_ANN_SP1                 !Gross taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIDISAB_ANN_SP1                 !Gross taxable disability benefits of wage earners of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTINDOCC_ANN_SP1                 !Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTISICK_ANN_SP1                  !Gross taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIFAMAL_ANN_SP1                 !Gross taxable income from family allowances of spouse 1 of tax unit
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: PIT_PROFSTAT_ANN_SP1                 !Professional status of spouse 1 on the private, public and self employed labour market
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPRIV_ANN_SP2                  !Gross taxable labour income earned as wage earner on the private labour market by spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPUB_ANN_SP2                   !Gross taxable labour income earned as wage earner on the public labour market by spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTISELF_ANN_SP2                  !Gross taxable labour income earned as self employed by spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIPENS_ANN_SP2                  !Gross taxable pension benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIEARNE_ANN_SP2                 !Gross taxable early retirement benefits of the new type of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIEAROL_ANN_SP2                 !Gross taxable early retirement benefits of the old type of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIOLDUN_ANN_SP2                 !Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIOTHUN_ANN_SP2                 !Gross taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIDISAB_ANN_SP2                 !Gross taxable disability benefits of wage earners of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTINDOCC_ANN_SP2                 !Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTISICK_ANN_SP2                  !Gross taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GTIFAMAL_ANN_SP2                 !Gross taxable income from family allowances of spouse 2 of tax unit
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: PIT_PROFSTAT_ANN_SP2                 !Professional status of spouse 2 on the private, public and self employed labour market
    !Determination of net taxable income     
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTOI_ANN_SP1                     !Net taxable occupational income of spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIPENS_ANN_SP1                  !Net taxable pension benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEARNE_ANN_SP1                 !Net taxable early retirement benefits of the new type of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEAROL_ANN_SP1                 !Net taxable early retirement benefits of the old type of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIOLDUN_ANN_SP1                 !Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIOTHUN_ANN_SP1                 !Net taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIDISAB_ANN_SP1                 !Net taxable disability benefits of wage earners of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTINDOCC_ANN_SP1                 !Net taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTISICK_ANN_SP1                  !Net taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTRATE_ANN_SP1                 !Lump sum costs of spouse 1 deductible of employment income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEMPL_ANN_SP1                  !Net taxable employment income of spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTISELF_ANN_SP1                  !Net taxable self employment income of spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTI_ANN_SP1                      !Total amount of net taxable income of spouse 1 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTOI_ANN_SP2                     !Net taxable occupational income of spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIPENS_ANN_SP2                  !Net taxable pension benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEARNE_ANN_SP2                 !Net taxable early retirement benefits of the new type of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEAROL_ANN_SP2                 !Net taxable early retirement benefits of the old type of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIOLDUN_ANN_SP2                 !Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIOTHUN_ANN_SP2                 !Net taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIDISAB_ANN_SP2                 !Net taxable disability benefits of wage earners of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTINDOCC_ANN_SP2                 !Net taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTISICK_ANN_SP2                  !Net taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_COSTRATE_ANN_SP2                 !Lump sum costs of spouse 2 deductible of employment income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTIEMPL_ANN_SP2                  !Net taxable employment income of spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTISELF_ANN_SP2                  !Net taxable self employment income of spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NTI_ANN_SP2                      !Total amount of net taxable income of spouse 2 of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_MARSPLIT_ANN                     !Amount of taxable income that can be transferred from one spouse to the other because of marital splitting rule
    !Determination of gross personal income taxes    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GPIT_ANN_SP1                     !Gross amount of personal income taxes of spouse 1 after applying the global tax rate scheme on net taxable income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_GPIT_ANN_SP2                     !Gross amount of personal income taxes of spouse 2 after applying the global tax rate scheme on net taxable income
    !Determination of tax credits    
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMP_ANN_SP1                    !Income amount of spouse 1 that is used to compute the tax credit for family composition
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMP_ANN_SP2                    !Income amount of spouse 2 that is used to compute the tax credit for family composition
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMPCH_ANN_SP1                  !Income amount of spouse 1 that is used to compute the tax credit for childeren composition (required to compute the rebatable tax credit)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMPCH_ANN_SP2                  !Income amount of spouse 2 that is used to compute the tax credit for childeren composition (required to compute the rebatable tax credit)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_EXEMTRAN_ANN                     !Amount of exemptions not taken up by one of the partners
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_PITFAMC_ANN_SP1                  !Personal income taxes of spouse 1 net of tax credits for family composition
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_PITFAMC_ANN_SP2                  !Personal income taxes of spouse 2 net of tax credits for family composition
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_PITFAMC_ANN                      !Personal income taxes at the tax unit level net of tax credits for family composition
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_REBATCH_ANN                      !Amount of exemptions rebatable in the form of tax credit for children
    !Credits in case of replacement income   
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_PITREP_ANN                       !Personal income taxes at the tax unit level net of tax credits for family composition and credits for replacement income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN1                     !Fraction applied on pension benefits and early retirement benefits of new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN2                     !Fraction applied on early retirement benefits of old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN3                     !Fraction applied on normal unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN4                     !Fraction applied on unemployment benefits of older unemployed (age of 58 or above)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN5                     !Fraction applied on sickness and disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_FRACREP_ANN6                     !Fraction applied on other replacement income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN1                      !Basic tax credit granted for pension benefits and early retirement benefits of new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN2                      !Basic tax credit granted for early retirement benefits of old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN3                      !Basic tax credit granted for normal unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN4                      !Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN5                      !Basic tax credit granted for sickness and disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_BTCREP_ANN6                      !Basic tax credit granted for other replacement income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_TNTI_ANN                         !Total net taxable income of the tax unit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN1                      !Reduced tax credit granted for pension benefits and early retirement benefits of new type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN2                      !Reduced tax credit granted for early retirement benefits of old type
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN3                      !Reduced tax credit granted for normal unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN4                      !Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN5                      !Reduced tax credit granted for sickness and disability benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: PIT_RTCREP_ANN6                      !Reduced tax credit granted for other replacement income
    !Other interventions to determine the final personal income tax bill 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETPITSC_ANN                     !Personal income taxes at the tax unit level net of all tax credits and obtained after application of a tax rate scheme on an aggregate income concept
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETPITSE_ANN                     !Sum of all personal income taxes due to tax rates applied on separate income components
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_NETPIT_ANN                       !Sum of all personal incomes taxes: rate scheme taxes and separate taxes
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_STATEPIT_ANN                     !Personal income taxes to be paid to the federal state
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_REGIOTAX_ANN                     !Personal income taxes to be paid to other entities than the federal state
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_CSAMOUNT_ANN                     !Amount of crisis surcharge
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_SPECSSCO_ANN                     !Amount of special social security contribution
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_FLEMJOBR_ANN                     !Amount of reduction installed by the Flemish government for those who work
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_TAXCRLOW_ANN_SP1                 !Tax credit for low labour income of spouse 1 (in case of self employed or civil servant income)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_TAXCRLOW_ANN_SP2                 !Tax credit for low labour income of spouse 2 (in case of self employed or civil servant income)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PIT_FINTAX_ANN                       !Sum of all personal incomes taxes paid by the tax unit
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_REGIO                            !Indicator of region where the tax unit is living
    !Tax units that are not liable to enter a tax form   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_TUNOLAB_ANN                      !Indicator whether the tax unit is liable to enter a tax form yes or no (no = 1, yes = 0)
    !Reconstructed exchange concepts             

    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_CHECK_ANN                        !Vector to store identification of a check
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: TAXUNIT_WEIGHT                       !Vector with weights for tax units
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: QNR_TNTI                             !Vector with quantile numbers based on total net taxable income
    REAL(KIND=KIND_REAL),DIMENSION(0:NUMBDEC)               :: QVAL_TNTI                            !Vector with quantile values based on total net taxable income
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_AGECLASS                         !Age of the individual in classes

END MODULE PIT_DATA

!==========================================!
! This module contains all the subroutines !
! and functions used in the PIT MODULE     !
!==========================================!
!List of subroutine names

MODULE PIT_SUBROUTINES

    USE PIT_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC!
!=====================================================!

SUBROUTINE PIT_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Exogenous variables
    !PIT_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_RELATION(DIM_ALLOC))
    ALLOCATE(NAREGNIS_COMMUNIT(DIM_ALLOC))
    ALLOCATE(InFODSZ(DIM_ALLOC))
    ALLOCATE(COSTRATE(DIM_ALLOC))
    !Variables read from file PIT_CONTRIB.txt                                            
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIPENS_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEARNE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIEAROL_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOLDUN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIOTHUN_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIDISAB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTINDOCC_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTISICK_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_GTIFAMAL_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PRIV_QT(DIM_ALLOC))          
    ALLOCATE(CONTRIB_LABMSTAT_PUB_QT(DIM_ALLOC))           
    ALLOCATE(CONTRIB_SELFPCR_QT(DIM_ALLOC))        
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE PIT_ALLOC_EXOG

!======================================================!
!Allocate all endogenous vectors of dimension DIM_ALLOC!
!======================================================!

SUBROUTINE PIT_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(PIT_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(PIT_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(PIT_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(PIT_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(PIT_HH_SIZE(DIM_ALLOC))
    ALLOCATE(PIT_START_HH(DIM_ALLOC))
    ALLOCATE(PIT_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    !Reconstruction of tax units 
    ALLOCATE(PIT_TURANK_ANN(DIM_ALLOC))
    ALLOCATE(PIT_TUNUMB_ANN(DIM_ALLOC))
    ALLOCATE(PIT_PARTID_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NUMTAXU_ANN(DIM_ALLOC))
    ALLOCATE(PIT_HQOLDNEW(DIM_ALLOC))
    ALLOCATE(PIT_NETOWN_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNOC_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNRE_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNEM_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNSE_ANN(DIM_ALLOC))
    ALLOCATE(PIT_COSTRTEM_ANN(DIM_ALLOC))
    ALLOCATE(PIT_COSTEMP_ANN(DIM_ALLOC))
    ALLOCATE(PIT_COSTRTSE_ANN(DIM_ALLOC))
    ALLOCATE(PIT_COSTSELF_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNOT_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETOWNNT_ANN(DIM_ALLOC))
    ALLOCATE(PIT_HANDICAP_ANN(DIM_ALLOC))
    !Variables at tax unit level before personal income tax computation  
    ALLOCATE(PIT_COUPLE_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHILD_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPOTHER_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPOLDER_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHIL3_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHIHA_ANN(DIM_ALLOC))
    ALLOCATE(PIT_DEPOTHHA_ANN(DIM_ALLOC))
    ALLOCATE(PIT_GTIPRIV_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIPUB_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTISELF_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIPENS_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIEARNE_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIEAROL_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIOLDUN_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIOTHUN_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIDISAB_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTINDOCC_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTISICK_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GTIFAMAL_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_PROFSTAT_ANN_SP1(DIM_ALLOC,3))
    ALLOCATE(PIT_GTIPRIV_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIPUB_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTISELF_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIPENS_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIEARNE_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIEAROL_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIOLDUN_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIOTHUN_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIDISAB_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTINDOCC_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTISICK_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_GTIFAMAL_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_PROFSTAT_ANN_SP2(DIM_ALLOC,3))
    !Determination of net taxable income     
    ALLOCATE(PIT_NTOI_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIPENS_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIEARNE_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIEAROL_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIOLDUN_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIOTHUN_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIDISAB_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTINDOCC_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTISICK_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_COSTRATE_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTIEMPL_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTISELF_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTI_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_NTOI_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIPENS_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIEARNE_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIEAROL_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIOLDUN_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIOTHUN_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIDISAB_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTINDOCC_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTISICK_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_COSTRATE_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTIEMPL_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTISELF_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_NTI_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_MARSPLIT_ANN(DIM_ALLOC))
    !Determination of gross personal income taxes    
    ALLOCATE(PIT_GPIT_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_GPIT_ANN_SP2(DIM_ALLOC))
    !Determination of tax credits    
    ALLOCATE(PIT_EXEMP_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_EXEMP_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_EXEMPCH_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_EXEMPCH_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_EXEMTRAN_ANN(DIM_ALLOC))
    ALLOCATE(PIT_PITFAMC_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_PITFAMC_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_PITFAMC_ANN(DIM_ALLOC))
    ALLOCATE(PIT_REBATCH_ANN(DIM_ALLOC))
    !Credits in case of replacement income   
    ALLOCATE(PIT_PITREP_ANN(DIM_ALLOC))
    ALLOCATE(PIT_FRACREP_ANN1(DIM_ALLOC,2))
    ALLOCATE(PIT_FRACREP_ANN2(DIM_ALLOC,2))
    ALLOCATE(PIT_FRACREP_ANN3(DIM_ALLOC,2))
    ALLOCATE(PIT_FRACREP_ANN4(DIM_ALLOC,2))
    ALLOCATE(PIT_FRACREP_ANN5(DIM_ALLOC,2))
    ALLOCATE(PIT_FRACREP_ANN6(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN1(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN2(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN3(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN4(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN5(DIM_ALLOC,2))
    ALLOCATE(PIT_BTCREP_ANN6(DIM_ALLOC,2))
    ALLOCATE(PIT_TNTI_ANN(DIM_ALLOC))
    ALLOCATE(PIT_RTCREP_ANN1(DIM_ALLOC,2))
    ALLOCATE(PIT_RTCREP_ANN2(DIM_ALLOC,2))
    ALLOCATE(PIT_RTCREP_ANN3(DIM_ALLOC,2))
    ALLOCATE(PIT_RTCREP_ANN4(DIM_ALLOC,2))
    ALLOCATE(PIT_RTCREP_ANN5(DIM_ALLOC,2))
    ALLOCATE(PIT_RTCREP_ANN6(DIM_ALLOC,2))
    !Other interventions to determine the final personal income tax bill 
    ALLOCATE(PIT_NETPITSC_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETPITSE_ANN(DIM_ALLOC))
    ALLOCATE(PIT_NETPIT_ANN(DIM_ALLOC))
    ALLOCATE(PIT_STATEPIT_ANN(DIM_ALLOC))
    ALLOCATE(PIT_REGIOTAX_ANN(DIM_ALLOC))
    ALLOCATE(PIT_CSAMOUNT_ANN(DIM_ALLOC))
    ALLOCATE(PIT_SPECSSCO_ANN(DIM_ALLOC))
    ALLOCATE(PIT_FLEMJOBR_ANN(DIM_ALLOC))
    ALLOCATE(PIT_TAXCRLOW_ANN_SP1(DIM_ALLOC))
    ALLOCATE(PIT_TAXCRLOW_ANN_SP2(DIM_ALLOC))
    ALLOCATE(PIT_FINTAX_ANN(DIM_ALLOC))
    ALLOCATE(PIT_REGIO(DIM_ALLOC))
    !Tax units that are not liable to enter a tax form   
    ALLOCATE(PIT_TUNOLAB_ANN(DIM_ALLOC))
    !Reconstructed exchange concepts             

    !Instruments
    ALLOCATE(PIT_CHECK_ANN(DIM_ALLOC))
    ALLOCATE(TAXUNIT_WEIGHT(DIM_ALLOC))
    ALLOCATE(QNR_TNTI(DIM_ALLOC))
    ALLOCATE(PIT_AGECLASS(DIM_ALLOC))
    
END SUBROUTINE PIT_ALLOC_ENDO

SUBROUTINE PIT_DEALLOC

    !================================================
    !Exogenous variables
    !PIT_INPUT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_SEX)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_RELATION)
    DEALLOCATE(NAREGNIS_COMMUNIT)
    DEALLOCATE(InFODSZ)
    DEALLOCATE(COSTRATE)
    !Variables read from file PIT_CONTRIB.txt                                            
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_GTIPRIV_QT)
    DEALLOCATE(CONTRIB_GTIPUB_QT)
    DEALLOCATE(CONTRIB_GTISELF_QT)
    DEALLOCATE(CONTRIB_GTIPENS_QT)
    DEALLOCATE(CONTRIB_GTIEARNE_QT)
    DEALLOCATE(CONTRIB_GTIEAROL_QT)
    DEALLOCATE(CONTRIB_GTIOLDUN_QT)
    DEALLOCATE(CONTRIB_GTIOTHUN_QT)
    DEALLOCATE(CONTRIB_GTIDISAB_QT)
    DEALLOCATE(CONTRIB_GTINDOCC_QT)
    DEALLOCATE(CONTRIB_GTISICK_QT)
    DEALLOCATE(CONTRIB_GTIFAMAL_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PRIV_QT)          
    DEALLOCATE(CONTRIB_LABMSTAT_PUB_QT)           
    DEALLOCATE(CONTRIB_SELFPCR_QT)        
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)
    !================================================
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(PIT_YEAR_AGE)
    DEALLOCATE(PIT_MONTH_AGE)
    DEALLOCATE(PIT_MONTH_BIRTH)
    DEALLOCATE(PIT_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(PIT_HH_SIZE)
    DEALLOCATE(PIT_START_HH)
    DEALLOCATE(PIT_END_HH)
    !To construct variables listed in the module note
    !Reconstruction of tax units 
    DEALLOCATE(PIT_TURANK_ANN)
    DEALLOCATE(PIT_TUNUMB_ANN)
    DEALLOCATE(PIT_PARTID_ANN)
    DEALLOCATE(PIT_HQOLDNEW)
    DEALLOCATE(PIT_NUMTAXU_ANN)
    DEALLOCATE(PIT_NETOWN_ANN)
    DEALLOCATE(PIT_NETOWNOC_ANN)
    DEALLOCATE(PIT_NETOWNRE_ANN)
    DEALLOCATE(PIT_NETOWNEM_ANN)
    DEALLOCATE(PIT_NETOWNSE_ANN)
    DEALLOCATE(PIT_COSTRTEM_ANN)
    DEALLOCATE(PIT_COSTEMP_ANN)
    DEALLOCATE(PIT_COSTRTSE_ANN)
    DEALLOCATE(PIT_COSTSELF_ANN)
    DEALLOCATE(PIT_NETOWNOT_ANN)
    DEALLOCATE(PIT_NETOWNNT_ANN)
    DEALLOCATE(PIT_HANDICAP_ANN)
    !Variables at tax unit level before personal income tax computation  
    DEALLOCATE(PIT_COUPLE_ANN)
    DEALLOCATE(PIT_DEPCHILD_ANN)
    DEALLOCATE(PIT_DEPOTHER_ANN)
    DEALLOCATE(PIT_DEPOLDER_ANN)
    DEALLOCATE(PIT_DEPCHIL3_ANN)
    DEALLOCATE(PIT_DEPCHIHA_ANN)
    DEALLOCATE(PIT_DEPOTHHA_ANN)
    DEALLOCATE(PIT_GTIPRIV_ANN_SP1)
    DEALLOCATE(PIT_GTIPUB_ANN_SP1)
    DEALLOCATE(PIT_GTISELF_ANN_SP1)
    DEALLOCATE(PIT_GTIPENS_ANN_SP1)
    DEALLOCATE(PIT_GTIEARNE_ANN_SP1)
    DEALLOCATE(PIT_GTIEAROL_ANN_SP1)
    DEALLOCATE(PIT_GTIOLDUN_ANN_SP1)
    DEALLOCATE(PIT_GTIOTHUN_ANN_SP1)
    DEALLOCATE(PIT_GTIDISAB_ANN_SP1)
    DEALLOCATE(PIT_GTINDOCC_ANN_SP1)
    DEALLOCATE(PIT_GTISICK_ANN_SP1)
    DEALLOCATE(PIT_GTIFAMAL_ANN_SP1)
    DEALLOCATE(PIT_PROFSTAT_ANN_SP1)
    DEALLOCATE(PIT_GTIPRIV_ANN_SP2)
    DEALLOCATE(PIT_GTIPUB_ANN_SP2)
    DEALLOCATE(PIT_GTISELF_ANN_SP2)
    DEALLOCATE(PIT_GTIPENS_ANN_SP2)
    DEALLOCATE(PIT_GTIEARNE_ANN_SP2)
    DEALLOCATE(PIT_GTIEAROL_ANN_SP2)
    DEALLOCATE(PIT_GTIOLDUN_ANN_SP2)
    DEALLOCATE(PIT_GTIOTHUN_ANN_SP2)
    DEALLOCATE(PIT_GTIDISAB_ANN_SP2)
    DEALLOCATE(PIT_GTINDOCC_ANN_SP2)
    DEALLOCATE(PIT_GTISICK_ANN_SP2)
    DEALLOCATE(PIT_GTIFAMAL_ANN_SP2)
    DEALLOCATE(PIT_PROFSTAT_ANN_SP2)
    !Determination of net taxable income     
    DEALLOCATE(PIT_NTOI_ANN_SP1)
    DEALLOCATE(PIT_NTIPENS_ANN_SP1)
    DEALLOCATE(PIT_NTIEARNE_ANN_SP1)
    DEALLOCATE(PIT_NTIEAROL_ANN_SP1)
    DEALLOCATE(PIT_NTIOLDUN_ANN_SP1)
    DEALLOCATE(PIT_NTIOTHUN_ANN_SP1)
    DEALLOCATE(PIT_NTIDISAB_ANN_SP1)
    DEALLOCATE(PIT_NTINDOCC_ANN_SP1)
    DEALLOCATE(PIT_NTISICK_ANN_SP1)
    DEALLOCATE(PIT_COSTRATE_ANN_SP1)
    DEALLOCATE(PIT_NTIEMPL_ANN_SP1)
    DEALLOCATE(PIT_NTISELF_ANN_SP1)
    DEALLOCATE(PIT_NTI_ANN_SP1)
    DEALLOCATE(PIT_NTOI_ANN_SP2)
    DEALLOCATE(PIT_NTIPENS_ANN_SP2)
    DEALLOCATE(PIT_NTIEARNE_ANN_SP2)
    DEALLOCATE(PIT_NTIEAROL_ANN_SP2)
    DEALLOCATE(PIT_NTIOLDUN_ANN_SP2)
    DEALLOCATE(PIT_NTIOTHUN_ANN_SP2)
    DEALLOCATE(PIT_NTIDISAB_ANN_SP2)
    DEALLOCATE(PIT_NTINDOCC_ANN_SP2)
    DEALLOCATE(PIT_NTISICK_ANN_SP2)
    DEALLOCATE(PIT_COSTRATE_ANN_SP2)
    DEALLOCATE(PIT_NTIEMPL_ANN_SP2)
    DEALLOCATE(PIT_NTISELF_ANN_SP2)
    DEALLOCATE(PIT_NTI_ANN_SP2)
    DEALLOCATE(PIT_MARSPLIT_ANN)
    !Determination of gross personal income taxes    
    DEALLOCATE(PIT_GPIT_ANN_SP1)
    DEALLOCATE(PIT_GPIT_ANN_SP2)
    !Determination of tax credits    
    DEALLOCATE(PIT_EXEMP_ANN_SP1)
    DEALLOCATE(PIT_EXEMP_ANN_SP2)
    DEALLOCATE(PIT_EXEMPCH_ANN_SP1)
    DEALLOCATE(PIT_EXEMPCH_ANN_SP2)
    DEALLOCATE(PIT_EXEMTRAN_ANN)
    DEALLOCATE(PIT_PITFAMC_ANN_SP1)
    DEALLOCATE(PIT_PITFAMC_ANN_SP2)
    DEALLOCATE(PIT_PITFAMC_ANN)
    DEALLOCATE(PIT_REBATCH_ANN)
    !Credits in case of replacement income   
    DEALLOCATE(PIT_PITREP_ANN)
    DEALLOCATE(PIT_FRACREP_ANN1)
    DEALLOCATE(PIT_FRACREP_ANN2)
    DEALLOCATE(PIT_FRACREP_ANN3)
    DEALLOCATE(PIT_FRACREP_ANN4)
    DEALLOCATE(PIT_FRACREP_ANN5)
    DEALLOCATE(PIT_FRACREP_ANN6)
    DEALLOCATE(PIT_BTCREP_ANN1)
    DEALLOCATE(PIT_BTCREP_ANN2)
    DEALLOCATE(PIT_BTCREP_ANN3)
    DEALLOCATE(PIT_BTCREP_ANN4)
    DEALLOCATE(PIT_BTCREP_ANN5)
    DEALLOCATE(PIT_BTCREP_ANN6)
    DEALLOCATE(PIT_TNTI_ANN)
    DEALLOCATE(PIT_RTCREP_ANN1)
    DEALLOCATE(PIT_RTCREP_ANN2)
    DEALLOCATE(PIT_RTCREP_ANN3)
    DEALLOCATE(PIT_RTCREP_ANN4)
    DEALLOCATE(PIT_RTCREP_ANN5)
    DEALLOCATE(PIT_RTCREP_ANN6)
    !Other interventions to determine the final personal income tax bill 
    DEALLOCATE(PIT_NETPITSC_ANN)
    DEALLOCATE(PIT_NETPITSE_ANN)
    DEALLOCATE(PIT_NETPIT_ANN)
    DEALLOCATE(PIT_STATEPIT_ANN)
    DEALLOCATE(PIT_REGIOTAX_ANN)
    DEALLOCATE(PIT_CSAMOUNT_ANN)
    DEALLOCATE(PIT_SPECSSCO_ANN)
    DEALLOCATE(PIT_FLEMJOBR_ANN)
    DEALLOCATE(PIT_TAXCRLOW_ANN_SP1)
    DEALLOCATE(PIT_TAXCRLOW_ANN_SP2)
    DEALLOCATE(PIT_FINTAX_ANN)
    DEALLOCATE(PIT_REGIO)
    !Tax units that are not liable to enter a tax form   
    DEALLOCATE(PIT_TUNOLAB_ANN)
    !Reconstructed exchange concepts             

    !Instruments
    DEALLOCATE(PIT_CHECK_ANN)
    DEALLOCATE(TAXUNIT_WEIGHT)
    DEALLOCATE(QNR_TNTI)
    DEALLOCATE(PIT_AGECLASS)
    
    !Parameters
    DEALLOCATE(PIT_RALUMPEM_ANN)                     
    DEALLOCATE(PIT_MAXOWNNG_ANN)                     
    DEALLOCATE(PIT_MAXOWNG_ANN)                      
    DEALLOCATE(PIT_RATSCHEM_ANN)                     
    DEALLOCATE(PIT_EXEMSCH_ANN)                      
    DEALLOCATE(PIT_MAXREPL_ANN)                      
    DEALLOCATE(PIT_RREDUCE_ANN)                      
    DEALLOCATE(PIT_RUNEMPL_ANN)                      
    DEALLOCATE(PIT_UNIQREPL_ANN)                     
    DEALLOCATE(PIT_RATECRIS_ANN)                     
    DEALLOCATE(PIT_TAXFREEB_ANN) 
    DEALLOCATE(PIT_SPECCON_ANN) 
    DEALLOCATE(PIT_JOBREDU_ANN) 
    DEALLOCATE(PIT_TAXCREWE_ANN) 
    DEALLOCATE(PIT_TAXCRESE_ANN) 
    DEALLOCATE(PIT_TAXCRECS_ANN) 
    DEALLOCATE(PIT_TAXFREHA_ANN) 
    DEALLOCATE(PIT_ABSHQ_ANN) 

END SUBROUTINE PIT_DEALLOC

!======================================================!
!subroutine to skip n lines on file k (reads as blanks)
!======================================================!
SUBROUTINE RSKIP(k,n)

    INTEGER :: k,n

    DO i=1,n
        read(k,'(a)')
    END DO    

END SUBROUTINE RSKIP

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE PIT_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                               !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        PIT_YEAR_AGE(pers_j) = 0
        PIT_MONTH_AGE(pers_j) = 0
        PIT_MONTH_BIRTH(pers_j) = 0
        PIT_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        PIT_HH_SIZE(pers_j) = 0
        PIT_START_HH(pers_j) = 0
        PIT_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        !Reconstruction of tax units 
        PIT_TURANK_ANN(pers_j) = 0
        PIT_TUNUMB_ANN(pers_j) = 0
        PIT_PARTID_ANN(pers_j) = 0
        PIT_HQOLDNEW(pers_j) = 0
        PIT_NUMTAXU_ANN(pers_j) = 0
        PIT_NETOWN_ANN(pers_j) = 0
        PIT_NETOWNOC_ANN(pers_j) = 0
        PIT_NETOWNRE_ANN(pers_j) = 0
        PIT_NETOWNEM_ANN(pers_j) = 0
        PIT_NETOWNSE_ANN(pers_j) = 0
        PIT_COSTRTEM_ANN(pers_j) = 0
        PIT_COSTEMP_ANN(pers_j) = 0
        PIT_COSTRTSE_ANN(pers_j) = 0
        PIT_COSTSELF_ANN(pers_j) = 0
        PIT_NETOWNOT_ANN(pers_j) = 0
        PIT_NETOWNNT_ANN(pers_j) = 0
        PIT_HANDICAP_ANN(pers_j) = 0
        !Variables at tax unit level before personal income tax computation  
        PIT_COUPLE_ANN(pers_j) = 0
        PIT_DEPCHILD_ANN(pers_j) = 0
        PIT_DEPOTHER_ANN(pers_j) = 0
        PIT_DEPOLDER_ANN(pers_j) = 0
        PIT_DEPCHIL3_ANN(pers_j) = 0
        PIT_DEPCHIHA_ANN(pers_j) = 0
        PIT_DEPOTHHA_ANN(pers_j) = 0
        PIT_GTIPRIV_ANN_SP1(pers_j) = 0
        PIT_GTIPUB_ANN_SP1(pers_j) = 0
        PIT_GTISELF_ANN_SP1(pers_j) = 0
        PIT_GTIPENS_ANN_SP1(pers_j) = 0
        PIT_GTIEARNE_ANN_SP1(pers_j) = 0
        PIT_GTIEAROL_ANN_SP1(pers_j) = 0
        PIT_GTIOLDUN_ANN_SP1(pers_j) = 0
        PIT_GTIOTHUN_ANN_SP1(pers_j) = 0
        PIT_GTIDISAB_ANN_SP1(pers_j) = 0
        PIT_GTINDOCC_ANN_SP1(pers_j) = 0
        PIT_GTISICK_ANN_SP1(pers_j) = 0
        PIT_GTIFAMAL_ANN_SP1(pers_j) = 0
        PIT_PROFSTAT_ANN_SP1(pers_j,1:3) = 0
        PIT_GTIPRIV_ANN_SP2(pers_j) = 0
        PIT_GTIPUB_ANN_SP2(pers_j) = 0
        PIT_GTISELF_ANN_SP2(pers_j) = 0
        PIT_GTIPENS_ANN_SP2(pers_j) = 0
        PIT_GTIEARNE_ANN_SP2(pers_j) = 0
        PIT_GTIEAROL_ANN_SP2(pers_j) = 0
        PIT_GTIOLDUN_ANN_SP2(pers_j) = 0
        PIT_GTIOTHUN_ANN_SP2(pers_j) = 0
        PIT_GTIDISAB_ANN_SP2(pers_j) = 0
        PIT_GTINDOCC_ANN_SP2(pers_j) = 0
        PIT_GTISICK_ANN_SP2(pers_j) = 0
        PIT_GTIFAMAL_ANN_SP2(pers_j) = 0
        PIT_PROFSTAT_ANN_SP2(pers_j,1:3) = 0
        !Determination of net taxable income     
        PIT_NTOI_ANN_SP1(pers_j) = 0
        PIT_NTIPENS_ANN_SP1(pers_j) = 0
        PIT_NTIEARNE_ANN_SP1(pers_j) = 0
        PIT_NTIEAROL_ANN_SP1(pers_j) = 0
        PIT_NTIOLDUN_ANN_SP1(pers_j) = 0
        PIT_NTIOTHUN_ANN_SP1(pers_j) = 0
        PIT_NTIDISAB_ANN_SP1(pers_j) = 0
        PIT_NTINDOCC_ANN_SP1(pers_j) = 0
        PIT_NTISICK_ANN_SP1(pers_j) = 0
        PIT_COSTRATE_ANN_SP1(pers_j) = 0
        PIT_NTIEMPL_ANN_SP1(pers_j) = 0
        PIT_NTISELF_ANN_SP1(pers_j) = 0
        PIT_NTI_ANN_SP1(pers_j) = 0
        PIT_NTOI_ANN_SP2(pers_j) = 0
        PIT_NTIPENS_ANN_SP2(pers_j) = 0
        PIT_NTIEARNE_ANN_SP2(pers_j) = 0
        PIT_NTIEAROL_ANN_SP2(pers_j) = 0
        PIT_NTIOLDUN_ANN_SP2(pers_j) = 0
        PIT_NTIOTHUN_ANN_SP2(pers_j) = 0
        PIT_NTIDISAB_ANN_SP2(pers_j) = 0
        PIT_NTINDOCC_ANN_SP2(pers_j) = 0
        PIT_NTISICK_ANN_SP2(pers_j) = 0
        PIT_COSTRATE_ANN_SP2(pers_j) = 0
        PIT_NTIEMPL_ANN_SP2(pers_j) = 0
        PIT_NTISELF_ANN_SP2(pers_j) = 0
        PIT_NTI_ANN_SP2(pers_j) = 0
        PIT_MARSPLIT_ANN(pers_j) = 0
        !Determination of gross personal income taxes    
        PIT_GPIT_ANN_SP1(pers_j) = 0
        PIT_GPIT_ANN_SP2(pers_j) = 0
        !Determination of tax credits    
        PIT_EXEMP_ANN_SP1(pers_j) = 0
        PIT_EXEMP_ANN_SP2(pers_j) = 0
        PIT_EXEMPCH_ANN_SP1(pers_j) = 0
        PIT_EXEMPCH_ANN_SP2(pers_j) = 0
        PIT_EXEMTRAN_ANN(pers_j) = 0
        PIT_PITFAMC_ANN_SP1(pers_j) = 0
        PIT_PITFAMC_ANN_SP2(pers_j) = 0
        PIT_PITFAMC_ANN(pers_j) = 0
        PIT_REBATCH_ANN(pers_j) = 0
        !Credits in case of replacement income   
        PIT_PITREP_ANN(pers_j) = 0
        PIT_FRACREP_ANN1(pers_j,1:2) = 0
        PIT_FRACREP_ANN2(pers_j,1:2) = 0
        PIT_FRACREP_ANN3(pers_j,1:2) = 0
        PIT_FRACREP_ANN4(pers_j,1:2) = 0
        PIT_FRACREP_ANN5(pers_j,1:2) = 0
        PIT_FRACREP_ANN6(pers_j,1:2) = 0
        PIT_BTCREP_ANN1(pers_j,1:2) = 0
        PIT_BTCREP_ANN2(pers_j,1:2) = 0
        PIT_BTCREP_ANN3(pers_j,1:2) = 0
        PIT_BTCREP_ANN4(pers_j,1:2) = 0
        PIT_BTCREP_ANN5(pers_j,1:2) = 0
        PIT_BTCREP_ANN6(pers_j,1:2) = 0
        PIT_TNTI_ANN(pers_j) = 0
        PIT_RTCREP_ANN1(pers_j,1:2) = 0
        PIT_RTCREP_ANN2(pers_j,1:2) = 0
        PIT_RTCREP_ANN3(pers_j,1:2) = 0
        PIT_RTCREP_ANN4(pers_j,1:2) = 0
        PIT_RTCREP_ANN5(pers_j,1:2) = 0
        PIT_RTCREP_ANN6(pers_j,1:2) = 0
        !Other interventions to determine the final personal income tax bill 
        PIT_NETPITSC_ANN(pers_j) = 0
        PIT_NETPITSE_ANN(pers_j) = 0
        PIT_NETPIT_ANN(pers_j) = 0
        PIT_STATEPIT_ANN(pers_j) = 0
        PIT_REGIOTAX_ANN(pers_j) = 0
        PIT_CSAMOUNT_ANN(pers_j) = 0
        PIT_SPECSSCO_ANN(pers_j) = 0
        PIT_FLEMJOBR_ANN(pers_j) = 0
        PIT_TAXCRLOW_ANN_SP1(pers_j) = 0
        PIT_TAXCRLOW_ANN_SP2(pers_j) = 0
        PIT_FINTAX_ANN(pers_j) = 0
        PIT_REGIO(pers_j) = 0
        !Tax units that are not liable to enter a tax form   
        PIT_TUNOLAB_ANN(pers_j) = 0
        !Reconstructed exchange concepts             

        !Instruments
        PIT_CHECK_ANN(pers_j) = 0
        TAXUNIT_WEIGHT(pers_j) = 0
        QNR_TNTI(pers_j) = 0
        PIT_AGECLASS(pers_j) = 0
    END DO
    N_HH_PIT=0
    QVAL_TNTI(0:NUMBDEC) = 0

END SUBROUTINE PIT_INITENDO

!========== SUBROUTINE PIT_RANKANDSORT ======================================!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!============================================================================!

SUBROUTINE PIT_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !PIT_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    CALL SORTING(COSTRATE(1:n_rec),NEW_RANK)
    !Rank individual identification number from the CONTRIB data file
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Variables read from file PIT_CONTRIB.txt                                            
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SELFPCR_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL PIT_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !PIT_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_COMMUNIT(1:n_rec),NEW_RANK)
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    CALL SORTING(COSTRATE(1:n_rec),NEW_RANK)
    !Variables read from file PIT_CONTRIB.txt                                            
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIPENS_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEARNE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIEAROL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOLDUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIOTHUN_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIDISAB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTINDOCC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTISICK_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_GTIFAMAL_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SELFPCR_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_WEIGHT.txt
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL PIT_CONSISTENCY_CHECK

END SUBROUTINE PIT_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from           !
!the parameter file                                      !
!========================================================!
SUBROUTINE PIT_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='PIT_INPUT_PATH') THEN
            PIT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PIT_INPUT_FILE') THEN
            PIT_INPUT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='PIT_CONTRIB_FILE') THEN
            PIT_CONTRIB_FILE = CONTROL_IO(2*i)
        !Read material from family compostion file
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read material to read PIT reference information                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='PITTNTI_2008_FILE') THEN
            PITTNTI_2008_FILE = CONTROL_IO(2*i)
        !Read material from wage construct file
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        !Read the file names of the intermediate output files
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_PIT_FILE') THEN
            SOCBEN_PIT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='EVAL_PIT_FILE') THEN
            EVAL_PIT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='OUTPUT_PATH')THEN
            OUTPUT_PATH = CONTROL_IO(2*i)   
        END IF    
    END DO

    !check for length of default path and file names 
    PIT_PAR_BREAK_POINT(1)  = INDEX(PIT_INPUT_PATH,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(2)  = INDEX(PIT_INPUT_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(3)  = INDEX(PIT_CONTRIB_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(4)  = INDEX(FAMREL_INPUT_PATH,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(5)  = INDEX(MIMOSIS_FAMREL_FILE,PIT_PAR_BREAK)    
    PIT_PAR_BREAK_POINT(6)  = INDEX(CONSTANT_INPUT_PATH,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(7)  = INDEX(MIMOSIS_SAMPWEIG_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(8)  = INDEX(WAGE_CONSTRUCT_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(12) = INDEX(PITTNTI_2008_FILE,PIT_PAR_BREAK)
    !check for length of path and file names of intermediate output files
    PIT_PAR_BREAK_POINT(9)  = INDEX(SOCBEN_PIT_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(10) = INDEX(EVAL_PIT_FILE,PIT_PAR_BREAK)
    PIT_PAR_BREAK_POINT(11) = INDEX(OUTPUT_PATH,PIT_PAR_BREAK)

    !merge information (path+name) to create default input file names
    PIT_INPUT = PIT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(1)-1)//&
    &PIT_INPUT_FILE(1:PIT_PAR_BREAK_POINT(2)-1)
    PIT_CONTRIB = PIT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(1)-1)//&
    &PIT_CONTRIB_FILE(1:PIT_PAR_BREAK_POINT(3)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:PIT_PAR_BREAK_POINT(4)-1)//&
    &MIMOSIS_FAMREL_FILE(1:PIT_PAR_BREAK_POINT(5)-1)    
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(6)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:PIT_PAR_BREAK_POINT(7)-1)    
    WAGE_CONSTRUCT = PIT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:PIT_PAR_BREAK_POINT(8)-1)
    PITTNTI_2008 = CONSTANT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(6)-1)//&
    &PITTNTI_2008_FILE(1:PIT_PAR_BREAK_POINT(12)-1)    
    !construct names of output files
    SOCBEN_PIT = PIT_INPUT_PATH(1:PIT_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_PIT_FILE(1:PIT_PAR_BREAK_POINT(9)-1)
    EVAL_PIT = OUTPUT_PATH(1:PIT_PAR_BREAK_POINT(11)-1)//&
    &EVAL_PIT_FILE(1:PIT_PAR_BREAK_POINT(10)-1)

    !Trim the result
    !of input files
    CALL TRIM@(PIT_INPUT)
    CALL TRIM@(PIT_CONTRIB)
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(PITTNTI_2008)  
    !of output files
    CALL TRIM@(SOCBEN_PIT)
    CALL TRIM@(EVAL_PIT)

    !write names to log file
    !of input files
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')
    WRITE(15,*) 'PIT_INPUT= ',          PIT_INPUT
    WRITE(15,*) 'PIT_CONTRIB= ',        PIT_CONTRIB
    WRITE(15,*) 'MIMOSIS_FAMREL= ',     MIMOSIS_FAMREL
    WRITE(15,*) 'MIMOSIS_SAMPWEIG= ',   MIMOSIS_SAMPWEIG
    WRITE(15,*) 'WAGE_CONSTRUCT= ',     WAGE_CONSTRUCT
    WRITE(15,*) 'PITTNTI_2008= ',       PITTNTI_2008
    !of output files
    WRITE(15,*) 'SOCBEN_PIT= ',         SOCBEN_PIT
    WRITE(15,*) 'EVAL_PIT= ',           EVAL_PIT

    !open parameter file        
    OPEN(UNIT=3, FILE=PIT_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PIT_PARLOC
        WRITE(15,'("Could not find ", a100)') PIT_PARLOC
        PAUSE
        STOP
    END IF

    ALLOCATE(PIT_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') PIT_PAR_NAMES(i),PIT_PAR_L, &
        &PIT_PAR_C,PIT_PAR_CURR
        !Reconstruction of tax units 
        IF (PIT_PAR_NAMES(i)=='PIT_CRATEREP_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_CRATEREP_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_CRATEREP_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RALUMPEM_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_RALUMPEM_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_RALUMPEM_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_RALUMPEM_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RATEEMP_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_RATEEMP_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_RATEEMP_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_MINCOST_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_MINCOST_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_MINCOST_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RALUMPSE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_RALUMPSE_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_RALUMPSE_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_MAXSELF_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_MAXSELF_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_MAXSELF_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RATESEL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_RATESEL_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_RATESEL_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_MAXOWNNG_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_MAXOWNNG_ANN(PIT_PAR_L))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) (PIT_MAXOWNNG_ANN(j),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) (PIT_MAXOWNNG_ANN(j),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_MAXOWNG_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_MAXOWNG_ANN(PIT_PAR_L))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) (PIT_MAXOWNG_ANN(j),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) (PIT_MAXOWNG_ANN(j),j=1,PIT_PAR_L)        
        !Determination of net taxable income     
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_PERMASPL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_PERMASPL_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_PERMASPL_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_ABSMASPL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_ABSMASPL_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_ABSMASPL_ANN
        !Determination of gross personal income taxes    
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RATSCHEM_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_RATSCHEM_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_RATSCHEM_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_RATSCHEM_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        !Determination of tax credits    
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEMSCH_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_EXEMSCH_ANN(PIT_PAR_L))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) (PIT_EXEMSCH_ANN(j),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) (PIT_EXEMSCH_ANN(j),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXECHLA_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXECHLA_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXECHLA_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEMPS_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEMPS_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEMPS_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEMPC_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEMPC_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEMPC_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEOTH_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEOTH_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEOTH_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEHANSP_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEHANSP_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEHANSP_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEHANOT_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEHANOT_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEHANOT_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXEOLD_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXEOLD_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXEOLD_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_EXESDEP_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_EXESDEP_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_EXESDEP_ANN
        !Credits in case of replacement income   
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_MAXREPL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_MAXREPL_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_MAXREPL_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_MAXREPL_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RREDUCE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_RREDUCE_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_RREDUCE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_RREDUCE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RUNEMPL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_RUNEMPL_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_RUNEMPL_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_RUNEMPL_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_UNIQREPL_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_UNIQREPL_ANN(PIT_PAR_L))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) (PIT_UNIQREPL_ANN(j),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) (PIT_UNIQREPL_ANN(j),j=1,PIT_PAR_L)        
        !Other interventions to determine the final personal income tax bill 
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_RATECRIS_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_RATECRIS_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_RATECRIS_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_RATECRIS_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        !Tax units that are not liable to enter a tax form   
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_BESINGLE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_BESINGLE_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_BESINGLE_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_BECOUPLE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_BECOUPLE_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_BECOUPLE_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_TAXFREEB_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_TAXFREEB_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_TAXFREEB_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_TAXFREEB_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            !Make sure value PIT_EXEHANSP_ANN is known before PIT_TAXFREEB_ANN 
            WRITE(15,'(a16)') 'PIT_TAXFREHA_ANN'
            ALLOCATE(PIT_TAXFREHA_ANN(PIT_PAR_L,PIT_PAR_C))
            PIT_TAXFREHA_ANN(1:PIT_PAR_L,1:PIT_PAR_C-1) = PIT_TAXFREEB_ANN(1:PIT_PAR_L,1:PIT_PAR_C-1)
            PIT_TAXFREHA_ANN(1:PIT_PAR_L,PIT_PAR_C)     = PIT_TAXFREEB_ANN(1:PIT_PAR_L,PIT_PAR_C) + PIT_EXEHANSP_ANN
            WRITE(15,PIT_PAR_FORMAT) ((PIT_TAXFREHA_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_CREDREDU_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_CREDREDU_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_CREDREDU_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_REBTACR_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_REBTACR_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_REBTACR_ANN
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_SPECCON_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_SPECCON_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_SPECCON_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_SPECCON_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_JOBREDU_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_JOBREDU_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_JOBREDU_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_JOBREDU_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_TAXCREWE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_TAXCREWE_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_TAXCREWE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_TAXCREWE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_TAXCRESE_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_TAXCRESE_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_TAXCRESE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_TAXCRESE_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_TAXCRECS_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_TAXCRECS_ANN(PIT_PAR_L,PIT_PAR_C))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) ((PIT_TAXCRECS_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) ((PIT_TAXCRECS_ANN(j,k),k=1,PIT_PAR_C),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_COMUNTAX_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_COMUNTAX_ANN
            WRITE(15,PIT_PAR_FORMAT) PIT_COMUNTAX_ANN
        !New parameters simulation HQ
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_SIMHQ') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) PIT_SIMHQ
            WRITE(15,PIT_PAR_FORMAT) PIT_SIMHQ
        ELSE IF (PIT_PAR_NAMES(i)=='PIT_ABSHQ_ANN') THEN
            WRITE(15,'(a40)') PIT_PAR_NAMES(i)
            ALLOCATE(PIT_ABSHQ_ANN(PIT_PAR_L))
            READ(3,'(a20)') PIT_PAR_FORMAT
            READ(3,PIT_PAR_FORMAT) (PIT_ABSHQ_ANN(j),j=1,PIT_PAR_L)
            WRITE(15,PIT_PAR_FORMAT) (PIT_ABSHQ_ANN(j),j=1,PIT_PAR_L)        
        ELSE IF (PIT_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(PIT_PAR_NAMES)
    
    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine PIT_PARAMETER_FILEREAD of NORMAL RUN.......runned correctly'  

END SUBROUTINE PIT_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE PIT_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=PIT_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PIT_INPUT
        WRITE(15,'("Could not find ", a100)') PIT_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,I10,A7,I10,I10,I10,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
            &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_COMMUNIT(pers_j),&
            &INFODSZ(pers_j),COSTRATE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PIT_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_COMMUNIT(pers_j),&
                &INFODSZ(pers_j),COSTRATE(pers_j)'')')
                WRITE(15,'(A14,I10,I10,A7,I10,I10,I10,F15.10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_COMMUNIT(pers_j),&
                &INFODSZ(pers_j),COSTRATE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,I10,A7,I10,I10,I10,F15.10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_SEX(pers_j),&
                &NAREGNIS_DATBIRTH(pers_j),NAREGNIS_RELATION(pers_j),NAREGNIS_COMMUNIT(pers_j),&
                &INFODSZ(pers_j),COSTRATE(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PIT_INPUT_FILEREAD of NORMAL RUN...........runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE PIT_INPUT_FILEREAD

!============= SUBROUTINE CONTRIB_FILEREAD ================!
!This subroutines reads in data from the CONTRIB file         !
!==========================================================!
SUBROUTINE PIT_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=PIT_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PIT_CONTRIB
        WRITE(15,'("Could not find ", a100)') PIT_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
            &F15.2,F15.2,F15.2,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
            &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
            &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
            &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
            &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
            &CONTRIB_SELFPCR_QT(pers_j)
            !write line if error
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,PIT_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(''&
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_SELFPCR_QT(pers_j)'')')               
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2,I10,I10,I10)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_SELFPCR_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,&
                &F15.2,F15.2,F15.2,I10,I10,I10)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_GTIPRIV_QT(pers_j),CONTRIB_GTIPUB_QT(pers_j),&
                &CONTRIB_GTISELF_QT(pers_j),CONTRIB_GTIPENS_QT(pers_j),CONTRIB_GTIEARNE_QT(pers_j),&
                &CONTRIB_GTIEAROL_QT(pers_j),CONTRIB_GTIOLDUN_QT(pers_j),CONTRIB_GTIOTHUN_QT(pers_j),&
                &CONTRIB_GTIDISAB_QT(pers_j),CONTRIB_GTINDOCC_QT(pers_j),CONTRIB_GTISICK_QT(pers_j),& 
                &CONTRIB_GTIFAMAL_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_SELFPCR_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine PIT_CONTRIB_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE(15,'(I7, " records uploaded from CONTRIB_FILE")') pers_j-1

END SUBROUTINE PIT_CONTRIB_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE PIT_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_FAMREL
        WRITE(15,'("Could not find ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PIT_FAMREL_FILEREAD of NORMAL RUN..........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE PIT_FAMREL_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE PIT_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') WAGE_CONSTRUCT
        WRITE(15,'("Could not find ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PIT_WAGE_FILEREAD of NORMAL RUN............runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE PIT_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE PIT_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        WRITE(15,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine PIT_WEIGHT_FILEREAD of NORMAL RUN..........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE PIT_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE PIT_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))   .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))  .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))   .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   
   
END SUBROUTINE PIT_CONSISTENCY_CHECK 

!============= SUBROUTINE PIT_RECODE==========================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE PIT_RECODE

    DO pers_j=1,n_rec
        !Exogenous variables
        !PIT_INPUT_WRITE.SAS
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)         NAREGNIS_HH_ID(pers_j)=0
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)           NAREGNIS_SEX(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_RELATION(pers_j)).EQ.9999999)      NAREGNIS_RELATION(pers_j)=0
        IF (INT(NAREGNIS_COMMUNIT(pers_j)).EQ.9999999)      NAREGNIS_COMMUNIT(pers_j)=0
        IF (INT(INFODSZ(pers_j)).EQ.9999999)                INFODSZ(pers_j)=0
        IF (INT(COSTRATE(pers_j)).EQ.9999999)               COSTRATE(pers_j)=0
        !Variables read from file PIT_CONTRIB.txt
        IF (INT(CONTRIB_GTIPRIV_QT(pers_j)).EQ.9999999)     CONTRIB_GTIPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_GTIPUB_QT(pers_j)).EQ.9999999)      CONTRIB_GTIPUB_QT(pers_j)=0
        IF (INT(CONTRIB_GTISELF_QT(pers_j)).EQ.9999999)     CONTRIB_GTISELF_QT(pers_j)=0
        IF (INT(CONTRIB_GTIPENS_QT(pers_j)).EQ.9999999)     CONTRIB_GTIPENS_QT(pers_j)=0
        IF (INT(CONTRIB_GTIEARNE_QT(pers_j)).EQ.9999999)    CONTRIB_GTIEARNE_QT(pers_j)=0
        IF (INT(CONTRIB_GTIEAROL_QT(pers_j)).EQ.9999999)    CONTRIB_GTIEAROL_QT(pers_j)=0
        IF (INT(CONTRIB_GTIOLDUN_QT(pers_j)).EQ.9999999)    CONTRIB_GTIOLDUN_QT(pers_j)=0
        IF (INT(CONTRIB_GTIOTHUN_QT(pers_j)).EQ.9999999)    CONTRIB_GTIOTHUN_QT(pers_j)=0
        IF (INT(CONTRIB_GTIDISAB_QT(pers_j)).EQ.9999999)    CONTRIB_GTIDISAB_QT(pers_j)=0
        IF (INT(CONTRIB_GTINDOCC_QT(pers_j)).EQ.9999999)    CONTRIB_GTINDOCC_QT(pers_j)=0
        IF (INT(CONTRIB_GTISICK_QT(pers_j)).EQ.9999999)     CONTRIB_GTISICK_QT(pers_j)=0
        IF (INT(CONTRIB_GTIFAMAL_QT(pers_j)).EQ.9999999)    CONTRIB_GTIFAMAL_QT(pers_j)=0
        !Variables read from file MIMOSIS_FAMREL.txt                                                  
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)       SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)     MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)       MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)         MIMOSIS_WEIGHT(pers_j)=0
        !Trim leading blanks in exogenous variables 
        !exemptions: NAREGNIS_DATBIRTH and GECODEERD_INSZ_CHAR
   END DO   

END SUBROUTINE PIT_RECODE

!============= SUBROUTINE PIT_INFLATE==================================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE PIT_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file PIT_INPUT.txt                                            
        !Variables read from file PIT_CONTRIB.txt                                            
!        CONTRIB_GTIPRIV_QT(pers_j)                  = INFLATE_NOMVAL(1)*CONTRIB_GTIPRIV_QT(pers_j)                !Gross taxable labour income earned as wage earner on the private labour market
!        CONTRIB_GTIPUB_QT(pers_j)                   = INFLATE_NOMVAL(1)*CONTRIB_GTIPUB_QT(pers_j)                 !Gross taxable labour income earned as wage earner on the public labour market
!        CONTRIB_GTISELF_QT(pers_j)                  = INFLATE_NOMVAL(1)*CONTRIB_GTISELF_QT(pers_j)                !Gross taxable labour income earned as self employed
!        CONTRIB_GTIPENS_QT(pers_j)                  = INFLATE_NOMVAL(1)*CONTRIB_GTIPENS_QT(pers_j)                !Gross taxable pension benefits
!        CONTRIB_GTIEARNE_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIEARNE_QT(pers_j)               !Gross taxable early retirement benefits of the new type
!        CONTRIB_GTIEAROL_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIEAROL_QT(pers_j)               !Gross taxable early retirement benefits of the old type
!        CONTRIB_GTIOLDUN_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIOLDUN_QT(pers_j)               !Gross taxable unemployment benefits of older unemployed with seniority supplement
!        CONTRIB_GTIOTHUN_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIOTHUN_QT(pers_j)               !Gross taxable unemployment benefits, other than early retirement benefits
!        CONTRIB_GTIDISAB_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIDISAB_QT(pers_j)               !Gross taxable disability benefits of wage earners
!        CONTRIB_GTINDOCC_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTINDOCC_QT(pers_j)               !Gross taxable benefits in case of industrial accidents or occupational diseases
!        CONTRIB_GTISICK_QT(pers_j)                  = INFLATE_NOMVAL(1)*CONTRIB_GTISICK_QT(pers_j)                !Gross taxable benefits in case of sickness other than disability benefits
!        CONTRIB_GTIFAMAL_QT(pers_j)                 = INFLATE_NOMVAL(1)*CONTRIB_GTIFAMAL_QT(pers_j)               !Gross taxable income from family allowances
        !Variables read from file MIMOSIS_FAMREL.txt                                                  
        !Variables read from file WAGE_CONSTRUCT.txt                                                     
        SALJRS_LAST_CORR(pers_j)                    = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)                  !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)                  = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)                !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)                    = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)                  !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Variables read from file MIMOSIS_WEIGHT.txt
    END DO   

END SUBROUTINE PIT_INFLATE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   PIT_HH_SIZE                                 !
!   PIT_START_HH                                !
!   PIT_END_HH                                  !
!   N_HH_PIT                                    !
!===============================================!

SUBROUTINE PIT_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        PIT_HH_SIZE(k)=n
        PIT_START_HH(k)=j
        PIT_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_PIT=K

END SUBROUTINE PIT_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: PIT_YEAR_AGE       !
!                         PIT_MONTH_AGE      !
!                         PIT_YEAR_BIRTH     !
!                         PIT_MONTH_BIRTH    !   
!============================================!

SUBROUTINE PIT_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        PIT_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:PIT_PAR_BREAK_POINT(1)-1),  '(I10)') PIT_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(PIT_PAR_BREAK_POINT(1)+1:),   '(I10)') PIT_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        PIT_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-PIT_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        PIT_MONTH_AGE(pers_j) =PIT_YEAR_AGE(pers_j)*12+(12-PIT_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE PIT_AGE

!==========================================!
!Draw a line of a give number of characters
!==========================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of different rates applied on an input amount.             !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The rate is in column 3 of this matrix.              !
!An example of this procedure is the social security contribution of    ! 
!self employed in principal activity before retirement age              !
!In this module to be applied with parameters                           !
!   PIT_RALUMPEM_ANN                                                    !
!   PIT_RATSCHEM_ANN                                                    !
!=======================================================================!

FUNCTION PIT_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        !otherwise the amount is higher than the upper bracket
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                                                                     
            !apply rate to the difference between upper and lower limit (and add it up to the amount computed with previous brackets) 
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACRATE_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_BRACRATE_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of a comparison with some income levels.                   !
!An example of this procedure is the computation of the basic exemption !
!from the bottom up, introduced in 2007                                 !
!In this module to be applied with parameter                            !
!   PIT_TAXFREEB_ANN                                                    !
!=======================================================================!

FUNCTION PIT_BAMOUNT_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1 
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            IF (i.EQ.1) THEN
                OUTPUT_AMOUNT=MAT_BRACRATE(i,3)
            ELSE
                !prevent negative values
                IF (MAT_BRACRATE(i,3).GT.(INPUT_AMOUNT-MAT_BRACRATE(i,1))) THEN
                    OUTPUT_AMOUNT=MAT_BRACRATE(i,3)-(INPUT_AMOUNT-MAT_BRACRATE(i,1))
                END IF
            END IF
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(i,3)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BAMOUNT_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_BAMOUNT_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as the sum of different amounts conditional on the value of a status   !
!variable                                                               !
!The amounts are stored in the vector MAT_AMOUNT                        !
!In this module to be applied with parameters                           !
!   PIT_EXEMSCH_ANN                                                     !
!=======================================================================!

FUNCTION PIT_SUMAMOUNT_1COLUMN(STATUS1,NLINES_AMOUNT,MAT_AMOUNT,INDIV) RESULT (OUTPUT_AMOUNT)

    INTEGER                                             :: STATUS1                                  !Status variable used to compute the sum of nominal amounts 
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_AMOUNT                            !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_AMOUNT)      :: MAT_AMOUNT                               !Matrix with status variable and nominal amounts
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first value to the second last value
    DO i=1,NLINES_AMOUNT-1                                                                          
        !if the status is larger than the value in column 1 add the amount
        IF (STATUS1.GE.i) THEN
            OUTPUT_AMOUNT = OUTPUT_AMOUNT + MAT_AMOUNT(i)
            FUNCSELE = 1
        END IF
        !if no selection is necessary still keep track of running procedure
        IF (STATUS1.EQ.0) THEN
            FUNCSELE = 1
        END IF
    END DO 
    !in case of the last line a special manipulation is required
    IF (STATUS1.GE.NLINES_AMOUNT) THEN
        OUTPUT_AMOUNT = OUTPUT_AMOUNT+(STATUS1 + 1 - NLINES_AMOUNT)*MAT_AMOUNT(NLINES_AMOUNT)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_SUMAMOUNT_1COLUMN has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_SUMAMOUNT_1COLUMN

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as the sum of                                                          !
!a) a fraction of input_amount1                                         !
!b) a fraction of input_amount1 which is computed with input_amount2    !
!The fractions to be applied vary with the level of the input amount2,  !
!the lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix.                                                      !
!The rates that should be applied are in column 3 and 4.                !
!In this module to be applied with parameters                           !
!   PIT_RREDUCE_ANN                                                     !
!   PIT_RUNEMPL_ANN                                                     !
!   PIT_RATECRIS_ANN                                                    ! 
!=======================================================================!

FUNCTION PIT_BRACRATE_4COLUMNS_RATES(INPUT_AMOUNT1,INPUT_AMOUNT2,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT1                            !Amount1 on which the rates in column 3 will be applied
    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT2                            !Amount2 on which the rates in column 4 will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT2).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT2).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN
            IF (INT(100.D0*(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))).NE.0) THEN
                !apply rate1 and rate2 to the appropriate value
                OUTPUT_AMOUNT=&
                &INPUT_AMOUNT1*(MAT_BRACRATE(i,3)/100.D0)+& 
                &INPUT_AMOUNT1*(MAT_BRACRATE(i,4)/100.D0)*((INPUT_AMOUNT2-MAT_BRACRATE(i,1))/(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1)))
                FUNCSELE = 1
                !exit loop                                                                   
                EXIT
            END IF
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT2).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        IF (INT(100.D0*(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))).NE.0) THEN
            !apply rate1 and rate2 to the appropriate value
            OUTPUT_AMOUNT=&
            &INPUT_AMOUNT1*(MAT_BRACRATE(i,3)/100.D0)+& 
            &INPUT_AMOUNT1*(MAT_BRACRATE(i,4)/100.D0)*((INPUT_AMOUNT2-MAT_BRACRATE(i,1))/(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1)))
            FUNCSELE = 1
        END IF
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACRATE_4COLUMNS_RATES has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF
    
END FUNCTION PIT_BRACRATE_4COLUMNS_RATES

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount PLUS                                         !
!b) the result of a rate applied on part of the input amount            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the rate part is to be  !
!added are in column 3 of the matrix, the rate is in column 4 of this   !
!matrix.                                                                !
!An example of this procedure is the computation of the special         !
!social security contribution                                           !
!   PIT_SPECCON_ANN                                                     !
!=======================================================================!

FUNCTION PIT_BRACRATE_4COLUMNS_PLUS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,4)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)+(INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,1))*&
        &(MAT_BRACRATE(NLINES_BRACRATE,4)/100.D0)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACRATE_4COLUMNS_PLUS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_BRACRATE_4COLUMNS_PLUS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount MINUS                                        !
!b) the result of a rate applied on part of the input amount            !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the rate part is to be  !
!added are in column 3 of the matrix, the rate is in column 4 of this   !
!matrix.                                                                !
!An example of this procedure is the computation of the special         !
!social security contribution                                           !
!   PIT_JOBREDU_ANN                                                     !
!=======================================================================!

FUNCTION PIT_BRACRATE_4COLUMNS_MINUS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)-(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,4)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)-(INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,1))*&
        &(MAT_BRACRATE(NLINES_BRACRATE,4)/100.D0)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_BRACRATE_4COLUMNS_MINUS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_BRACRATE_4COLUMNS_MINUS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as                                                                     !
!a) a fixed nominal amount PLUS                                         !
!b) the result of a nominal amount multiplied with the result of a      !
!computation                                                            !
!The nominal amounts vary with the level of the input amount and are    !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The nominal amounts to which the multiplication part !
!is to be added are in column 3 of the matrix, the multiplication part  !
!is in column 4 of this matrix                                          !
!An example of this procedure is the computation of the rebatable tax   !
!credit for low incomes                                                 !
!   PIT_TAXCREWE_ANN                                                    !
!   PIT_TAXCRESE_ANN                                                    !
!   PIT_TAXCRECS_ANN                                                    !
!=======================================================================!

FUNCTION PIT_MULTIPL_4COLUMNS_PLUS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,4)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2)).AND.&
        &(i.LT.NLINES_BRACRATE-1)) THEN                                             
            !mulitply the nominal amount in column 4 with the difference between income and lower limit divided by the difference between
            !the upper and lower limit
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)+((INPUT_AMOUNT-MAT_BRACRATE(i,1))/(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1)))&
            &*(MAT_BRACRATE(i,4))
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2)).AND.&
        &(i.EQ.NLINES_BRACRATE-1)) THEN    
            !mulitply the nominal amount in column 4 with the difference between income and lower limit divided by the difference between
            !the upper and lower limit
            OUTPUT_AMOUNT=MAT_BRACRATE(i,3)+((MAT_BRACRATE(i,2)-INPUT_AMOUNT)/(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1)))&
            &*(MAT_BRACRATE(i,4))
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=MAT_BRACRATE(NLINES_BRACRATE,3)+((INPUT_AMOUNT-MAT_BRACRATE(NLINES_BRACRATE,1))/&
        &(MAT_BRACRATE(NLINES_BRACRATE,2)-MAT_BRACRATE(NLINES_BRACRATE,1)))*(MAT_BRACRATE(NLINES_BRACRATE,4))
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function PIT_MULTIPL_4COLUMNS_PLUS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION PIT_MULTIPL_4COLUMNS_PLUS

!===================================================================================!
!With subroutine PIT_TUSTRUCT_POTENT we construct the variables                     !
!that link the potential tax unit members                                           !
!Constructed variables                                                              !
!   PIT_TURANK_ANN                                                                  !
!   PIT_TUNUMB_ANN                                                                  !
!   PIT_PARTID_ANN                                                                  !
!   PIT_COUPLE_ANN                                                                  !
!===================================================================================!

SUBROUTINE PIT_TUSTRUCT_POTENT

    INTEGER :: pers_j,pers_k
    INTEGER :: COUNT_HEAD,COUNT_SPOUSE,COUNT_DEPCHILD,COUNT_DEPOTH,COUNT_SUM

    !PIT_TURANK_ANN: Rank of the individual within the tax unit the individual belongs to
    !1 Head of the tax unit
    !2 Spouse of the head of the tax unit
    !3 Dependent child
    !4 Dependent ascendent
    !5 Dependent other
    !Remark: The FAMREL module produces married couples as output where the man is head of the
    !family and the wife is the married spouse. In case of equal sex the oldest one of the two is the head
    !PIT_TUNUMB_ANN: Unique identification number of the tax unit
    !a unique number identifying the members of the same tax unit
    !PIT_COUPLE_ANN: Head of the tax unit is legally married with a partner that is comprised by the tax unit as well

    !Initialise count variables
    COUNT_HEAD = 0
    COUNT_SPOUSE = 0
    COUNT_DEPCHILD = 0
    COUNT_DEPOTH = 0
    !loop over all households
    DO hh_i=1,N_HH_PIT
        !loop over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            !Check whether the individual is the head of a family
            IF (FAMREL_RELATION(pers_j).EQ.1) THEN
                PIT_TURANK_ANN(pers_j) = 1
                PIT_TUNUMB_ANN(pers_j) = pers_j
                COUNT_HEAD = COUNT_HEAD + 1
                DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                    !look for married partner of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &FAMREL_RELATION(pers_k).EQ.2.AND.FAMREL_COUPLE(pers_j).EQ.1.AND.FAMREL_COUPLE(pers_k).EQ.1) THEN
                        PIT_TURANK_ANN(pers_k) = 2
                        PIT_TUNUMB_ANN(pers_k) = pers_j
                        PIT_PARTID_ANN(pers_j) = pers_k
                        PIT_COUPLE_ANN(pers_j) = 1
                        COUNT_SPOUSE = COUNT_SPOUSE + 1
                    END IF
                    !look for unmarried partner of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &FAMREL_RELATION(pers_k).EQ.2.AND.FAMREL_COUPLE(pers_j).EQ.2.AND.FAMREL_COUPLE(pers_k).EQ.2) THEN
                        PIT_TURANK_ANN(pers_k) = 1
                        PIT_TUNUMB_ANN(pers_k) = pers_k                                                                 
                        COUNT_HEAD = COUNT_HEAD + 1
                    END IF
                    !look for children of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &FAMREL_RELATION(pers_k).EQ.3) THEN
                        PIT_TURANK_ANN(pers_k) = 3
                        PIT_TUNUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPCHILD = COUNT_DEPCHILD + 1
                    END IF
                    !look for grandchildren of the head of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &FAMREL_RELATION(pers_k).EQ.6.AND.NAREGNIS_RELATION(pers_k).EQ.5) THEN
                        PIT_TURANK_ANN(pers_k) = 3
                        PIT_TUNUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPCHILD = COUNT_DEPCHILD + 1
                    END IF
                    !look for ascendants of the head of the family that are already part of the family
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &(FAMREL_RELATION(pers_k).EQ.4.OR.FAMREL_RELATION(pers_k).EQ.5)) THEN
                        PIT_TURANK_ANN(pers_k) = 4
                        PIT_TUNUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPOTH = COUNT_DEPOTH + 1
                    END IF
                    !look for family members of the second degree
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k).AND.PIT_TURANK_ANN(pers_k).EQ.0.AND.&
                    &FAMREL_RELATION(pers_k).EQ.6.AND.(NAREGNIS_RELATION(pers_k).EQ.9.OR.NAREGNIS_RELATION(pers_k).EQ.10)) THEN
                        PIT_TURANK_ANN(pers_k) = 5
                        PIT_TUNUMB_ANN(pers_k) = pers_j                                                                 
                        COUNT_DEPOTH = COUNT_DEPOTH + 1
                    END IF
                END DO
            END IF    
        END DO
        !loop again over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            IF (PIT_TURANK_ANN(pers_j).EQ.0) THEN
                !make each non assigned member head of its own tax unit
                PIT_TURANK_ANN(pers_j) = 1
                PIT_TUNUMB_ANN(pers_j) = pers_j
                COUNT_HEAD = COUNT_HEAD + 1
                !These individuals should only be of relation type 6 or 7
                IF (FAMREL_RELATION(pers_j).NE.6.AND.FAMREL_RELATION(pers_j).NE.7) THEN
                    WRITE(15,'(''Individu '', i10,'' is niet van relatie-type 6 of 7. Relatie-type is '', i10)')&
                    &pers_j,FAMREL_RELATION(pers_j)
                END IF
            END IF
        END DO
    END DO
    
    COUNT_SUM = COUNT_HEAD + COUNT_SPOUSE + COUNT_DEPCHILD + COUNT_DEPOTH
    !Write results of construction of potential tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals after creation of potential tax units'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of heads                    '', i10)') COUNT_HEAD
    Write(15,'(''Number of spouses                  '', i10)') COUNT_SPOUSE
    Write(15,'(''Number of dependent children       '', i10)') COUNT_DEPCHILD
    Write(15,'(''Number of dependent others         '', i10)') COUNT_DEPOTH
    Write(15,'(''Sum of all allocated individuals   '', i10)') COUNT_SUM
    
END SUBROUTINE PIT_TUSTRUCT_POTENT

!=====================================================================================!
!With subroutine PIT_NETOWN_CONSTRUC we construct the variables necessary to check the!
!net own means conditions of potential dependent tax unit members                     !  
!Constructed variables                                                                !
!   PIT_NETOWN_ANN                                                                    !
!   PIT_NETOWNOC_ANN                                                                  !
!   PIT_NETOWNRE_ANN                                                                  !
!   PIT_NETOWNEM_ANN                                                                  !
!   PIT_NETOWNSE_ANN                                                                  !
!   PIT_COSTRTEM_ANN                                                                  !
!   PIT_COSTEMP_ANN                                                                   !
!   PIT_COSTRTSE_ANN                                                                  !
!   PIT_COSTSELF_ANN                                                                  !
!   PIT_NETOWNOT_ANN                                                                  !
!   PIT_NETOWNNT_ANN                                                                  !
!   PIT_HANDICAP_ANN                                                                  !
!=====================================================================================!

SUBROUTINE PIT_NETOWN_CONSTRUC

    INTEGER:: pers_j
    REAL(KIND=KIND_REAL)          :: PIT_GTINC_ANN                       !Gross taxable income per year to call function for later computations           

    !PIT_NETOWNRE_ANN: Net own replacement income of the individual (variable for testing tax dependency)
    DO pers_j=1,n_rec
        PIT_NETOWNRE_ANN(pers_j) = (PIT_CRATEREP_ANN/100.D0)*&
            &(CONTRIB_GTIPENS_QT(pers_j) + CONTRIB_GTIEARNE_QT(pers_j)  + &
            & CONTRIB_GTIEAROL_QT(pers_j) + CONTRIB_GTIOLDUN_QT(pers_j) + &
            & CONTRIB_GTIOTHUN_QT(pers_j) + CONTRIB_GTIDISAB_QT(pers_j) + &
            & CONTRIB_GTINDOCC_QT(pers_j) + CONTRIB_GTISICK_QT(pers_j)  )*4.D0
    END DO

    !PIT_COSTRTEM_ANN: Estimate of costs made to earn income as employee, estimated with cost rates (variable for testing tax dependency)
    !PIT_COSTEMP_ANN: Final estimate of costs made to earn income as employee (variable for testing tax dependency)
    !PIT_NETOWNEM_ANN: Net own means of activity as employee (variable for testing tax dependency)
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = (CONTRIB_GTIPRIV_QT(pers_j) + CONTRIB_GTIPUB_QT(pers_j))*4.0D0
        PIT_COSTRTEM_ANN(pers_j) = PIT_BRACRATE_3COLUMNS(PIT_GTINC_ANN,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
        PIT_COSTEMP_ANN(pers_j) = MAX(PIT_COSTRTEM_ANN(pers_j),(PIT_RATEEMP_ANN/100.D0)*PIT_GTINC_ANN)
        IF (PIT_COSTEMP_ANN(pers_j).LT.PIT_MINCOST_ANN) THEN
            PIT_COSTEMP_ANN(pers_j) = PIT_MINCOST_ANN
        END IF
        !determination of net own means
        PIT_NETOWNEM_ANN(pers_j) = PIT_GTINC_ANN - PIT_COSTEMP_ANN(pers_j)
        IF (PIT_NETOWNEM_ANN(pers_j).LT.0.D0) THEN
            PIT_NETOWNEM_ANN(pers_j) = 0.D0
        END IF
    END DO

    !PIT_COSTRTSE_ANN: Estimate of costs made to earn income as self employed, estimated with cost rates (variable for testing tax dependency)
    !PIT_COSTSELF_ANN: Final estimate of costs made to earn income as self employed (variable for testing tax dependency)
    !PIT_NETOWNSE_ANN: Net own means of activity as self employed (variable for testing tax dependency)
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = (CONTRIB_GTISELF_QT(pers_j))*4.0D0
        PIT_COSTRTSE_ANN(pers_j) = MIN((PIT_RALUMPSE_ANN/100.D0)*PIT_GTINC_ANN,PIT_MAXSELF_ANN)
        PIT_COSTSELF_ANN(pers_j) = MAX(PIT_COSTRTSE_ANN(pers_j),(PIT_RATESEL_ANN/100.D0)*PIT_GTINC_ANN)
        IF (PIT_COSTSELF_ANN(pers_j).LT.PIT_MINCOST_ANN) THEN
            PIT_COSTSELF_ANN(pers_j) = PIT_MINCOST_ANN
        END IF
        !determination of net own means
        PIT_NETOWNSE_ANN(pers_j) = PIT_GTINC_ANN - PIT_COSTSELF_ANN(pers_j)
        IF (PIT_NETOWNSE_ANN(pers_j).LT.0.D0) THEN
            PIT_NETOWNSE_ANN(pers_j) = 0.D0
        END IF
    END DO

    !PIT_NETOWNOC_ANN: Net own means of occupational income sources (variable for testing tax dependency)
    !PIT_NETOWNOT_ANN: Net own means from taxable income sources other than occupational income (variable for testing tax dependency)
    !PIT_NETOWNNT_ANN: Net own means from non-taxable income sources (variable for testing tax dependency)
    !PIT_NETOWN_ANN: Net own means of the individual (variable for testing tax dependency)
    DO pers_j=1,n_rec
        PIT_NETOWNOC_ANN(pers_j) = &
            &PIT_NETOWNRE_ANN(pers_j) + PIT_NETOWNEM_ANN(pers_j) + PIT_NETOWNSE_ANN(pers_j)
        PIT_NETOWNOT_ANN(pers_j) = 0.D0
        PIT_NETOWNNT_ANN(pers_j) = 0.D0
        PIT_NETOWN_ANN(pers_j) = &
            &PIT_NETOWNOC_ANN(pers_j) + PIT_NETOWNOT_ANN(pers_j) + PIT_NETOWNNT_ANN(pers_j)
    END DO

    !PIT_HANDICAP_ANN: Identification of having an handicap (yes = 1, no = 0)
    DO pers_j=1,n_rec
        IF (InFODSZ(pers_j).EQ.1) THEN
            PIT_HANDICAP_ANN(pers_j) = 1
        END IF
    END DO
    
END SUBROUTINE PIT_NETOWN_CONSTRUC

!===================================================================================!
!With subroutine PIT_TUSTRUCT_FINAL we check the net own means conditions of        !
!potential tax unit members.                                                        !
!Variables adapted                                                                  !
!   PIT_TURANK_ANN                                                                  !
!   PIT_TUNUMB_ANN                                                                  !
!===================================================================================!

SUBROUTINE PIT_TUSTRUCT_FINAL

    INTEGER :: pers_j,pers_k,pers_l
    INTEGER :: COUNT_SPLITC,COUNT_SPLITSNH,COUNT_SPLITSH,COUNT_SPLITCGP,COUNT_SPLITSGP,COUNT_SUM
    INTEGER :: COUNT_HEAD,COUNT_SPOUSE,COUNT_DEPCHILD,COUNT_DEPOTH

    !initialise PIT_CHECK_ANN vector
    PIT_CHECK_ANN(1:n_rec) = 0
    
    !Carry out check for dependents other than parents or grandparents that form a couple already
    !loop over all households
    COUNT_SPLITC = 0
    COUNT_SPLITSNH = 0
    COUNT_SPLITSH = 0
    DO hh_i=1,N_HH_PIT
        !loop over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            !Check only applied on dependent children or dependent others
            IF (ANY(PIT_TURANK_ANN(pers_j).EQ.(/3,4,5/))) THEN
                !The necessary checks differ for married and unmarried couples
                !PIT_TUNUMB_ANN(pers_j) should contain the line number of the head of the tax unit
                IF (PIT_COUPLE_ANN(PIT_TUNUMB_ANN(pers_j)).EQ.1) THEN
                    IF (PIT_NETOWN_ANN(pers_j).GT.PIT_MAXOWNNG_ANN(1)) THEN
                        PIT_TURANK_ANN(pers_j) = 1
                        PIT_TUNUMB_ANN(pers_j) = pers_j 
                        COUNT_SPLITC = COUNT_SPLITC + 1
                    END IF
                ELSE IF (PIT_COUPLE_ANN(PIT_TUNUMB_ANN(pers_j)).EQ.0) THEN
                    IF (PIT_NETOWN_ANN(pers_j).GT.PIT_MAXOWNNG_ANN(2).AND.PIT_HANDICAP_ANN(pers_j).EQ.0) THEN
                        PIT_TURANK_ANN(pers_j) = 1
                        PIT_TUNUMB_ANN(pers_j) = pers_j
                        COUNT_SPLITSNH = COUNT_SPLITSNH + 1
                    END IF
                    IF (PIT_NETOWN_ANN(pers_j).GT.PIT_MAXOWNNG_ANN(3).AND.PIT_HANDICAP_ANN(pers_j).EQ.1) THEN
                        PIT_TURANK_ANN(pers_j) = 1
                        PIT_TUNUMB_ANN(pers_j) = pers_j
                        COUNT_SPLITSH = COUNT_SPLITSH + 1
                    END IF
                END IF
            END IF
        END DO            
    END DO

    !Carry out check for dependent parents or grandparents that form a couple already
    !loop over all households
    COUNT_SPLITCGP = 0 
    COUNT_SPLITSGP = 0
    DO hh_i=1,N_HH_PIT
        !loop over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
            !Check for couples of parents
            !look for parent 1
            IF (FAMREL_FAMTYPE(pers_j).EQ.3) THEN
                DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                    !look for parent 2
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMTYPE(pers_k).EQ.3.AND.&
                    &FAMREL_COUPLE(pers_j).EQ.1.AND.FAMREL_COUPLE(pers_k).EQ.1) THEN
                        !if joint net own means are below threshold 1 
                        IF ((PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).LT.PIT_MAXOWNG_ANN(1).AND.&
                        &PIT_CHECK_ANN(pers_j).EQ.0) THEN
                            COUNT_SPLITCGP = COUNT_SPLITCGP + 1
                            PIT_CHECK_ANN(pers_j) = 1
                            PIT_CHECK_ANN(pers_k) = 1
                            !both individuals are unmarried in the fiscal sense since they become dependent
                            PIT_COUPLE_ANN(pers_j) = 0
                            PIT_COUPLE_ANN(pers_k) = 0
                            !both individuals have rank 4 in the tax unit
                            PIT_TURANK_ANN(pers_j) = 4
                            PIT_TURANK_ANN(pers_k) = 4
                            !they both belong to the tax unit of the head of the sociological household
                            DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                    PIT_TUNUMB_ANN(pers_j) = PIT_TUNUMB_ANN(pers_l)
                                    PIT_TUNUMB_ANN(pers_k) = PIT_TUNUMB_ANN(pers_l)
                                END IF
                            END DO
                        !if joint net own means are between threshold 1 and 2                            
                        ELSE IF ((PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).GE.PIT_MAXOWNG_ANN(1).AND.&
                        &(PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).LT.PIT_MAXOWNG_ANN(2).AND.&
                        &PIT_CHECK_ANN(pers_j).EQ.0) THEN
                            COUNT_SPLITSGP = COUNT_SPLITSGP + 1
                            PIT_CHECK_ANN(pers_j) = 1
                            PIT_CHECK_ANN(pers_k) = 1
                            !both individuals are unmarried in the fiscal sense since one becomes dependent
                            PIT_COUPLE_ANN(pers_j) = 0
                            PIT_COUPLE_ANN(pers_k) = 0
                            !the individual with the lowest income has rank 4 in the tax unit of the head of the sociological household
                            IF (PIT_NETOWN_ANN(pers_j).GE.PIT_NETOWN_ANN(pers_k)) THEN
                                PIT_TURANK_ANN(pers_j) = 1
                                PIT_TUNUMB_ANN(pers_j) = pers_j
                                PIT_TURANK_ANN(pers_k) = 4
                                !only the individual with the lowest income belongs to the tax unit of the head of the household
                                DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                    IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                        PIT_TUNUMB_ANN(pers_k) = PIT_TUNUMB_ANN(pers_l)
                                    END IF
                                END DO
                            ELSE
                                PIT_TURANK_ANN(pers_j) = 4
                                PIT_TURANK_ANN(pers_k) = 1
                                PIT_TUNUMB_ANN(pers_k) = pers_k
                                !only the individual with the lowest income belongs to the tax unit of the head of the household
                                DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                    IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                        PIT_TUNUMB_ANN(pers_j) = PIT_TUNUMB_ANN(pers_l)
                                    END IF
                                END DO
                            END IF
                        END IF
                    END IF    
                END DO
            END IF
            !Check for couples of grandparents
            !look for grandparent 1
            IF (FAMREL_FAMTYPE(pers_j).EQ.4) THEN
                DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                    !look for grandparent 2
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMTYPE(pers_k).EQ.4.AND.&
                    &FAMREL_COUPLE(pers_j).EQ.1.AND.FAMREL_COUPLE(pers_k).EQ.1) THEN
                        !if joint net own means are below threshold 1 
                        IF ((PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).LT.PIT_MAXOWNG_ANN(1).AND.&
                        &PIT_CHECK_ANN(pers_j).EQ.0) THEN
                            COUNT_SPLITCGP = COUNT_SPLITCGP + 1
                            PIT_CHECK_ANN(pers_j) = 1
                            PIT_CHECK_ANN(pers_k) = 1
                            !both individuals are unmarried in the fiscal sense since they becomes dependent
                            PIT_COUPLE_ANN(pers_j) = 0
                            PIT_COUPLE_ANN(pers_k) = 0
                            !both individuals have rank 4 in the tax unit
                            PIT_TURANK_ANN(pers_j) = 4
                            PIT_TURANK_ANN(pers_k) = 4
                            !they both belong to the tax unit of the head of the sociological household
                            DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                    PIT_TUNUMB_ANN(pers_j) = PIT_TUNUMB_ANN(pers_l)
                                    PIT_TUNUMB_ANN(pers_k) = PIT_TUNUMB_ANN(pers_l)
                                END IF
                            END DO
                        !if joint net own means are between threshold 1 and 2                            
                        ELSE IF ((PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).GE.PIT_MAXOWNG_ANN(1).AND.&
                        &(PIT_NETOWN_ANN(pers_j)+PIT_NETOWN_ANN(pers_k)).LT.PIT_MAXOWNG_ANN(2).AND.&
                        &PIT_CHECK_ANN(pers_j).EQ.0) THEN
                            COUNT_SPLITSGP = COUNT_SPLITSGP + 1
                            PIT_CHECK_ANN(pers_j) = 1
                            PIT_CHECK_ANN(pers_k) = 1
                            !both individuals are unmarried in the fiscal sense since one becomes dependent
                            PIT_COUPLE_ANN(pers_j) = 0
                            PIT_COUPLE_ANN(pers_k) = 0
                            !the individual with the lowest income has rank 4 in the tax unit of the head of the sociological household
                            IF (PIT_NETOWN_ANN(pers_j).GE.PIT_NETOWN_ANN(pers_k)) THEN
                                PIT_TURANK_ANN(pers_j) = 1
                                PIT_TUNUMB_ANN(pers_j) = pers_j
                                PIT_TURANK_ANN(pers_k) = 4
                                !only the individual with the lowest income belongs to the tax unit of the head of the household
                                DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                    IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                        PIT_TUNUMB_ANN(pers_k) = PIT_TUNUMB_ANN(pers_l)
                                    END IF
                                END DO
                            ELSE
                                PIT_TURANK_ANN(pers_j) = 4
                                PIT_TURANK_ANN(pers_k) = 1
                                PIT_TUNUMB_ANN(pers_k) = pers_k
                                !only the individual with the lowest income belongs to the tax unit of the head of the household
                                DO pers_l=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                                    IF (NAREGNIS_RELATION(pers_l).EQ.1) THEN
                                        PIT_TUNUMB_ANN(pers_j) = PIT_TUNUMB_ANN(pers_l)
                                    END IF
                                END DO
                            END IF
                        END IF
                    END IF    
                END DO
            END IF
        END DO
    END DO        

    !TEST
    !Some checks on the split procedure
    COUNT_SUM = COUNT_SPLITC + COUNT_SPLITSNH + COUNT_SPLITSH + 2*COUNT_SPLITCGP + COUNT_SPLITSGP
    !Write results of construction of potential tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals that are split of on the basis of their net own means'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of dependents split of from couple                                      '', i10)') COUNT_SPLITC 
    Write(15,'(''Number of dependents, not handicapped, split of from singles                   '', i10)') COUNT_SPLITSNH
    Write(15,'(''Number of dependents, handicapped, split of from singles                       '', i10)') COUNT_SPLITSH
    Write(15,'(''Number of parents or grandparents jointly transfered to other TU               '', i10)') 2*COUNT_SPLITCGP
    Write(15,'(''Number of parents or grandparents split of from partner                        '', i10)') COUNT_SPLITSGP
    Write(15,'(''Sum of all individuals split of or transfered                                  '', i10)') COUNT_SUM

    !Initialise count variables for final check
    COUNT_HEAD = 0
    COUNT_SPOUSE = 0
    COUNT_DEPCHILD = 0
    COUNT_DEPOTH = 0

    DO pers_j=1,n_rec
        IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
            COUNT_HEAD = COUNT_HEAD + 1
        ELSE IF (PIT_TURANK_ANN(pers_j).EQ.2) THEN
            COUNT_SPOUSE = COUNT_SPOUSE + 1 
        ELSE IF (PIT_TURANK_ANN(pers_j).EQ.3) THEN
            COUNT_DEPCHILD = COUNT_DEPCHILD + 1
        ELSE IF (PIT_TURANK_ANN(pers_j).EQ.4) THEN
            COUNT_DEPOTH = COUNT_DEPOTH + 1
        ELSE IF (PIT_TURANK_ANN(pers_j).EQ.5) THEN
            COUNT_DEPOTH = COUNT_DEPOTH + 1
        END IF
    END DO

    !TEST
    COUNT_SUM = COUNT_HEAD + COUNT_SPOUSE + COUNT_DEPCHILD + COUNT_DEPOTH
    !Write results of construction of final tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals after split of on the basis of net own means'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of heads                    '', i10)') COUNT_HEAD
    Write(15,'(''Number of spouses                  '', i10)') COUNT_SPOUSE
    Write(15,'(''Number of dependent children       '', i10)') COUNT_DEPCHILD
    Write(15,'(''Number of dependent others         '', i10)') COUNT_DEPOTH
    Write(15,'(''Sum of all allocated individuals   '', i10)') COUNT_SUM

END SUBROUTINE PIT_TUSTRUCT_FINAL

!===================================================================================!
!With subroutine PIT_TUVAR_CREATE we create the variables at the level of the tax   !
!unit                                                                               !
!Variables created                                                                  !
!   PIT_DEPCHILD_ANN                                                                !
!   PIT_DEPOTHER_ANN                                                                !
!   PIT_DEPOLDER_ANN                                                                !
!   PIT_DEPCHIL3_ANN                                                                !
!   PIT_DEPCHIHA_ANN                                                                !
!   PIT_DEPOTHHA_ANN                                                                !
!   PIT_GTIPRIV_ANN_SP1                                                             !
!   PIT_GTIPUB_ANN_SP1                                                              !
!   PIT_GTISELF_ANN_SP1                                                             !
!   PIT_GTIPENS_ANN_SP1                                                             !
!   PIT_GTIEARNE_ANN_SP1                                                            !
!   PIT_GTIEAROL_ANN_SP1                                                            !
!   PIT_GTIOLDUN_ANN_SP1                                                            !
!   PIT_GTIOTHUN_ANN_SP1                                                            !
!   PIT_GTIDISAB_ANN_SP1                                                            !
!   PIT_GTINDOCC_ANN_SP1                                                            !
!   PIT_GTISICK_ANN_SP1                                                             !
!   PIT_GTIFAMAL_ANN_SP1                                                            !
!   PIT_PROFSTAT_ANN_SP1                                                            !
!   PIT_GTIPRIV_ANN_SP2                                                             !
!   PIT_GTIPUB_ANN_SP2                                                              !
!   PIT_GTISELF_ANN_SP2                                                             !
!   PIT_GTIPENS_ANN_SP2                                                             !
!   PIT_GTIEARNE_ANN_SP2                                                            !
!   PIT_GTIEAROL_ANN_SP2                                                            !
!   PIT_GTIOLDUN_ANN_SP2                                                            !
!   PIT_GTIOTHUN_ANN_SP2                                                            !
!   PIT_GTIDISAB_ANN_SP2                                                            !
!   PIT_GTINDOCC_ANN_SP2                                                            !
!   PIT_GTISICK_ANN_SP2                                                             !
!   PIT_GTIFAMAL_ANN_SP2                                                            ! 
!   PIT_PROFSTAT_ANN_SP2                                                            !
!   PIT_NUMTAXU_ANN                                                                 !
!===================================================================================!

SUBROUTINE PIT_TUVAR_CREATE

    INTEGER :: pers_j,pers_k
    INTEGER :: COUNT_HEAD,COUNT_SPOUSE,COUNT_DEPCHILD,COUNT_DEPOTH,COUNT_SUM

    !PIT_DEPCHILD_ANN:     Number of children that are dependent according to the personal income tax legislation
    !PIT_DEPOTHER_ANN:     Number of individuals, other than children, that are dependent according to the personal income tax legislation
    !PIT_DEPOLDER_ANN:     Number of dependent ascendants up to the second degree older than 65
    !PIT_DEPCHIL3_ANN:     Number of children below the age of 3 that are dependent according to the personal income tax legislation
    !PIT_DEPCHIHA_ANN:     Number of children that are dependent according to the personal income tax legislation AND that are disabled
    !PIT_DEPOTHHA_ANN:     Number of individuals, other than children, that are dependent according to the personal income tax legislation AND disabled
    !PIT_GTIPRIV_ANN_SP1:  Gross taxable labour income earned as wage earner on the private labour market by spouse 1 of the tax unit
    !PIT_GTIPUB_ANN_SP1:   Gross taxable labour income earned as wage earner on the public labour market by spouse 1 of the tax unit
    !PIT_GTISELF_ANN_SP1:  Gross taxable labour income earned as self employed by spouse 1 of the tax unit
    !PIT_GTIPENS_ANN_SP1:  Gross taxable pension benefits of spouse 1 of tax unit
    !PIT_GTIEARNE_ANN_SP1: Gross taxable early retirement benefits of the new type of spouse 1 of tax unit
    !PIT_GTIEAROL_ANN_SP1: Gross taxable early retirement benefits of the old type of spouse 1 of tax unit
    !PIT_GTIOLDUN_ANN_SP1: Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit
    !PIT_GTIOTHUN_ANN_SP1: Gross taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit
    !PIT_GTIDISAB_ANN_SP1: Gross taxable disability benefits of wage earners of spouse 1 of tax unit
    !PIT_GTINDOCC_ANN_SP1: Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit
    !PIT_GTISICK_ANN_SP1:  Gross taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit
    !PIT_GTIFAMAL_ANN_SP1: Gross taxable income from family allowances of spouse 1 of tax unit
    !PIT_PROFSTAT_ANN_SP1: Professional status of spouse 1 on the private, public and self employed labour market
    !PIT_GTIPRIV_ANN_SP2:  Gross taxable labour income earned as wage earner on the private labour market by spouse 2 of the tax unit
    !PIT_GTIPUB_ANN_SP2:   Gross taxable labour income earned as wage earner on the public labour market by spouse 2 of the tax unit
    !PIT_GTISELF_ANN_SP2:  Gross taxable labour income earned as self employed by spouse 2 of the tax unit
    !PIT_GTIPENS_ANN_SP2:  Gross taxable pension benefits of spouse 2 of tax unit
    !PIT_GTIEARNE_ANN_SP2: Gross taxable early retirement benefits of the new type of spouse 2 of tax unit
    !PIT_GTIEAROL_ANN_SP2: Gross taxable early retirement benefits of the old type of spouse 2 of tax unit
    !PIT_GTIOLDUN_ANN_SP2: Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit
    !PIT_GTIOTHUN_ANN_SP2: Gross taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit
    !PIT_GTIDISAB_ANN_SP2: Gross taxable disability benefits of wage earners of spouse 2 of tax unit
    !PIT_GTINDOCC_ANN_SP2: Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit
    !PIT_GTISICK_ANN_SP2:  Gross taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit
    !PIT_GTIFAMAL_ANN_SP2: Gross taxable income from family allowances of spouse 2 of tax unit
    !PIT_PROFSTAT_ANN_SP2: Professional status of spouse 2 on the private, public and self employed labour market

    !loop over all households
    DO hh_i=1,N_HH_PIT
        COUNT_HEAD = 0
        !loop over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            !Check whether the individual is the head of a family
            IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
                COUNT_HEAD = COUNT_HEAD + 1
                !Define income variables of the head of the household
                PIT_GTIPRIV_ANN_SP1(pers_j)     = CONTRIB_GTIPRIV_QT(pers_j)*4.D0
                PIT_GTIPUB_ANN_SP1(pers_j)      = CONTRIB_GTIPUB_QT(pers_j)*4.D0  
                PIT_GTISELF_ANN_SP1(pers_j)     = CONTRIB_GTISELF_QT(pers_j)*4.D0 
                PIT_GTIPENS_ANN_SP1(pers_j)     = CONTRIB_GTIPENS_QT(pers_j)*4.D0 
                PIT_GTIEARNE_ANN_SP1(pers_j)    = CONTRIB_GTIEARNE_QT(pers_j)*4.D0
                PIT_GTIEAROL_ANN_SP1(pers_j)    = CONTRIB_GTIEAROL_QT(pers_j)*4.D0
                PIT_GTIOLDUN_ANN_SP1(pers_j)    = CONTRIB_GTIOLDUN_QT(pers_j)*4.D0
                PIT_GTIOTHUN_ANN_SP1(pers_j)    = CONTRIB_GTIOTHUN_QT(pers_j)*4.D0
                PIT_GTIDISAB_ANN_SP1(pers_j)    = CONTRIB_GTIDISAB_QT(pers_j)*4.D0
                PIT_GTINDOCC_ANN_SP1(pers_j)    = CONTRIB_GTINDOCC_QT(pers_j)*4.D0
                PIT_GTISICK_ANN_SP1(pers_j)     = CONTRIB_GTISICK_QT(pers_j)*4.D0 
                PIT_GTIFAMAL_ANN_SP1(pers_j)    = CONTRIB_GTIFAMAL_QT(pers_j)*4.D0
                PIT_PROFSTAT_ANN_SP1(pers_j,1)  = CONTRIB_LABMSTAT_PRIV_QT(pers_j)
                PIT_PROFSTAT_ANN_SP1(pers_j,2)  = CONTRIB_LABMSTAT_PUB_QT(pers_j)
                PIT_PROFSTAT_ANN_SP1(pers_j,3)  = CONTRIB_SELFPCR_QT(pers_j)
                DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i)
                    !Define income variables of the spouse of the head
                    IF (pers_j.NE.pers_k.AND.PIT_TURANK_ANN(pers_k).EQ.2.AND.&
                    &PIT_TUNUMB_ANN(pers_j).EQ.PIT_TUNUMB_ANN(pers_k)) THEN
                        PIT_GTIPRIV_ANN_SP2(pers_j)     = CONTRIB_GTIPRIV_QT(pers_k)*4.D0 
                        PIT_GTIPUB_ANN_SP2(pers_j)      = CONTRIB_GTIPUB_QT(pers_k)*4.D0  
                        PIT_GTISELF_ANN_SP2(pers_j)     = CONTRIB_GTISELF_QT(pers_k)*4.D0 
                        PIT_GTIPENS_ANN_SP2(pers_j)     = CONTRIB_GTIPENS_QT(pers_k)*4.D0 
                        PIT_GTIEARNE_ANN_SP2(pers_j)    = CONTRIB_GTIEARNE_QT(pers_k)*4.D0
                        PIT_GTIEAROL_ANN_SP2(pers_j)    = CONTRIB_GTIEAROL_QT(pers_k)*4.D0
                        PIT_GTIOLDUN_ANN_SP2(pers_j)    = CONTRIB_GTIOLDUN_QT(pers_k)*4.D0
                        PIT_GTIOTHUN_ANN_SP2(pers_j)    = CONTRIB_GTIOTHUN_QT(pers_k)*4.D0
                        PIT_GTIDISAB_ANN_SP2(pers_j)    = CONTRIB_GTIDISAB_QT(pers_k)*4.D0
                        PIT_GTINDOCC_ANN_SP2(pers_j)    = CONTRIB_GTINDOCC_QT(pers_k)*4.D0
                        PIT_GTISICK_ANN_SP2(pers_j)     = CONTRIB_GTISICK_QT(pers_k)*4.D0 
                        PIT_GTIFAMAL_ANN_SP2(pers_j)    = CONTRIB_GTIFAMAL_QT(pers_k)*4.D0
                        PIT_PROFSTAT_ANN_SP2(pers_j,1)  = CONTRIB_LABMSTAT_PRIV_QT(pers_j)
                        PIT_PROFSTAT_ANN_SP2(pers_j,2)  = CONTRIB_LABMSTAT_PUB_QT(pers_j)
                        PIT_PROFSTAT_ANN_SP2(pers_j,3)  = CONTRIB_SELFPCR_QT(pers_j)
                    END IF
                    IF (pers_j.NE.pers_k.AND.PIT_TURANK_ANN(pers_k).EQ.3.AND.&
                    &PIT_TUNUMB_ANN(pers_j).EQ.PIT_TUNUMB_ANN(pers_k)) THEN
                        PIT_DEPCHILD_ANN(pers_j) = PIT_DEPCHILD_ANN(pers_j) + 1
                        IF (PIT_YEAR_AGE(pers_k).LT.3) THEN
                            PIT_DEPCHIL3_ANN(pers_j) = PIT_DEPCHIL3_ANN(pers_j) + 1
                        END IF    
                        IF (InFODSZ(pers_k).EQ.1) THEN
                            PIT_DEPCHIHA_ANN(pers_j) = PIT_DEPCHIHA_ANN(pers_j) + 1
                        END IF    
                    END IF
                    IF (pers_j.NE.pers_k.AND.ANY(PIT_TURANK_ANN(pers_k).EQ.(/4,5/)).AND.&
                    &PIT_TUNUMB_ANN(pers_j).EQ.PIT_TUNUMB_ANN(pers_k)) THEN
                        PIT_DEPOTHER_ANN(pers_j) = PIT_DEPOTHER_ANN(pers_j) + 1
                        IF (PIT_TURANK_ANN(pers_k).EQ.4.AND.PIT_YEAR_AGE(pers_k).GT.65) THEN
                            PIT_DEPOLDER_ANN(pers_j) = PIT_DEPOLDER_ANN(pers_j) + 1
                        END IF
                        IF (InFODSZ(pers_k).EQ.1) THEN
                            PIT_DEPOTHHA_ANN(pers_j) = PIT_DEPOTHHA_ANN(pers_j) + 1
                        END IF    
                    END IF
                END DO
            END IF
        END DO
        !loop again over all members of the sociological household
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            !Check whether the individual is the head of a family
            IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
                PIT_NUMTAXU_ANN(pers_j) = COUNT_HEAD 
            END IF
        END DO
    END DO

    !TEST
    !Initialise count variables for final check
    COUNT_HEAD = 0
    COUNT_SPOUSE = 0
    COUNT_DEPCHILD = 0
    COUNT_DEPOTH = 0

    DO pers_j=1,n_rec
        IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
            COUNT_HEAD = COUNT_HEAD + 1
        END IF
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            COUNT_SPOUSE = COUNT_SPOUSE + 1 
        END IF
        IF (PIT_DEPCHILD_ANN(pers_j).GT.0) THEN
            COUNT_DEPCHILD = COUNT_DEPCHILD + PIT_DEPCHILD_ANN(pers_j)
        END IF
        IF (PIT_DEPOTHER_ANN(pers_j).GT.0) THEN
            COUNT_DEPOTH = COUNT_DEPOTH + PIT_DEPOTHER_ANN(pers_j)
        END IF
    END DO

    COUNT_SUM = COUNT_HEAD + COUNT_SPOUSE + COUNT_DEPCHILD + COUNT_DEPOTH
    !Write results of construction of final tax units
    Write(15,'('' '')')
    Write(15,'(''Counts of individuals after construction of tax units'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of heads                    '', i10)') COUNT_HEAD
    Write(15,'(''Number of spouses                  '', i10)') COUNT_SPOUSE
    Write(15,'(''Number of dependent children       '', i10)') COUNT_DEPCHILD
    Write(15,'(''Number of dependent others         '', i10)') COUNT_DEPOTH
    Write(15,'(''Sum of all allocated individuals   '', i10)') COUNT_SUM

END SUBROUTINE PIT_TUVAR_CREATE

!===================================================================================!
!With subroutine PIT_TUNTI_CREATE we create the net taxable income concepts at the  !
!level of the tax unit                                                              !
!Variables created                                                                  !
!   PIT_NTOI_ANN_SP1                                                                !
!   PIT_NTIPENS_ANN_SP1                                                             !
!   PIT_NTIEARNE_ANN_SP1                                                            !
!   PIT_NTIEAROL_ANN_SP1                                                            !
!   PIT_NTIOLDUN_ANN_SP1                                                            !
!   PIT_NTIOTHUN_ANN_SP1                                                            !
!   PIT_NTIDISAB_ANN_SP1                                                            !
!   PIT_NTINDOCC_ANN_SP1                                                            !
!   PIT_NTISICK_ANN_SP1                                                             !
!   PIT_COSTRATE_ANN_SP1                                                            !
!   PIT_NTIEMPL_ANN_SP1                                                             !
!   PIT_NTISELF_ANN_SP1                                                             !
!   PIT_NTI_ANN_SP1                                                                 !
!   PIT_NTOI_ANN_SP2                                                                !
!   PIT_NTIPENS_ANN_SP2                                                             !
!   PIT_NTIEARNE_ANN_SP2                                                            !
!   PIT_NTIEAROL_ANN_SP2                                                            !
!   PIT_NTIOLDUN_ANN_SP2                                                            !
!   PIT_NTIOTHUN_ANN_SP2                                                            !
!   PIT_NTIDISAB_ANN_SP2                                                            !
!   PIT_NTINDOCC_ANN_SP2                                                            !
!   PIT_NTISICK_ANN_SP2                                                             !
!   PIT_COSTRATE_ANN_SP2                                                            !
!   PIT_NTIEMPL_ANN_SP2                                                             !
!   PIT_NTISELF_ANN_SP2                                                             !
!   PIT_NTI_ANN_SP2                                                                 !
!   PIT_MARSPLIT_ANN                                                                !
!   PIT_HQOLDNEW                                                                    !
!===================================================================================!

SUBROUTINE PIT_TUNTI_CREATE

    INTEGER :: pers_j
    REAL(KIND=KIND_REAL)            :: PIT_GTINC_ANN                       !Gross taxable income per year to call function for later computations           
    REAL(KIND=KIND_REAL)            :: PIT_NTINC_ANN                       !Net taxable income per year to call function for later computations           
    REAL(KIND=KIND_REAL)            :: PIT_PCTNTOI_ANN                     !Store percentage of net taxable income for application of marital splitting rule
    INTEGER                         :: COUNT_MTSPPERC,COUNT_MTSPABS,COUNT_MTSPNEG,J,pers_k

    !SPOUSE 1
    !Compute net taxable replacement incomes of spouse 1
    !PIT_NTIPENS_ANN_SP1:  Net taxable pension benefits of spouse 1 of tax unit
    !PIT_NTIEARNE_ANN_SP1: Net taxable early retirement benefits of the new type of spouse 1 of tax unit
    !PIT_NTIEAROL_ANN_SP1: Net taxable early retirement benefits of the old type of spouse 1 of tax unit
    !PIT_NTIOLDUN_ANN_SP1: Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit
    !PIT_NTIOTHUN_ANN_SP1: Net taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit
    !PIT_NTIDISAB_ANN_SP1: Net taxable disability benefits of wage earners of spouse 1 of tax unit
    !PIT_NTINDOCC_ANN_SP1: Net taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit
    !PIT_NTISICK_ANN_SP1:  Net taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit
    DO pers_j=1,n_rec
        PIT_NTIPENS_ANN_SP1(pers_j) =   PIT_GTIPENS_ANN_SP1(pers_j) 
        PIT_NTIEARNE_ANN_SP1(pers_j) =  PIT_GTIEARNE_ANN_SP1(pers_j)
        PIT_NTIEAROL_ANN_SP1(pers_j) =  PIT_GTIEAROL_ANN_SP1(pers_j)
        PIT_NTIOLDUN_ANN_SP1(pers_j) =  PIT_GTIOLDUN_ANN_SP1(pers_j)
        PIT_NTIOTHUN_ANN_SP1(pers_j) =  PIT_GTIOTHUN_ANN_SP1(pers_j)
        PIT_NTIDISAB_ANN_SP1(pers_j) =  PIT_GTIDISAB_ANN_SP1(pers_j)
        PIT_NTINDOCC_ANN_SP1(pers_j) =  PIT_GTINDOCC_ANN_SP1(pers_j)
        PIT_NTISICK_ANN_SP1(pers_j) =   PIT_GTISICK_ANN_SP1(pers_j) 
    END DO

    !Compute net taxable income of spouse 1 earned as employee
    !PIT_COSTRATE_ANN_SP1: Lump sum costs of spouse 1 deductible of employment income
    !PIT_NTIEMPL_ANN_SP1: Net taxable employment income of spouse 1 of the tax unit
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = PIT_GTIPRIV_ANN_SP1(pers_j) + PIT_GTIPUB_ANN_SP1(pers_j)
        PIT_COSTRATE_ANN_SP1(pers_j) = PIT_BRACRATE_3COLUMNS(PIT_GTINC_ANN,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
        IF (PIT_COSTRATE_ANN_SP1(pers_j).LT.0.D0) THEN
            PIT_COSTRATE_ANN_SP1(pers_j) = 0.D0
        END IF
        !determination of net taxable income
        PIT_NTIEMPL_ANN_SP1(pers_j) = PIT_GTINC_ANN - PIT_COSTRATE_ANN_SP1(pers_j)
        IF (PIT_NTIEMPL_ANN_SP1(pers_j).LT.0.D0) THEN
            PIT_NTIEMPL_ANN_SP1(pers_j) = 0.D0
        END IF
    END DO

    !Compute net taxable income of spouse 1 earned as self employed
    !PIT_NTISELF_ANN_SP1: Net taxable self employment income of spouse 1 of the tax unit
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = PIT_GTISELF_ANN_SP1(pers_j)
        IF (MIN((PIT_RALUMPSE_ANN/100.D0)*PIT_GTINC_ANN,PIT_MAXSELF_ANN).GT.0) THEN
            PIT_NTISELF_ANN_SP1(pers_j) = PIT_GTINC_ANN - MIN((PIT_RALUMPSE_ANN/100.D0)*PIT_GTINC_ANN,PIT_MAXSELF_ANN)
        ELSE
            PIT_NTISELF_ANN_SP1(pers_j) = PIT_GTINC_ANN 
        END IF
    END DO

    !Compute net taxable income of spouse 1 before appliction of marital split
    !PIT_NTOI_ANN_SP1: Net taxable occupational income of spouse 1 of the tax unit
    DO pers_j=1,n_rec
        PIT_NTOI_ANN_SP1(pers_j) = &
            &PIT_NTIPENS_ANN_SP1(pers_j)  + &
            &PIT_NTIEARNE_ANN_SP1(pers_j) + &
            &PIT_NTIEAROL_ANN_SP1(pers_j) + &
            &PIT_NTIOLDUN_ANN_SP1(pers_j) + &
            &PIT_NTIOTHUN_ANN_SP1(pers_j) + &
            &PIT_NTIDISAB_ANN_SP1(pers_j) + &
            &PIT_NTINDOCC_ANN_SP1(pers_j) + &
            &PIT_NTISICK_ANN_SP1(pers_j)  + &
            &PIT_NTIEMPL_ANN_SP1(pers_j)  + &
            &PIT_NTISELF_ANN_SP1(pers_j)  
    END DO

    !SPOUSE 2
    !Compute net taxable replacement incomes of spouse 2
    !PIT_NTIPENS_ANN_SP2:  Net taxable pension benefits of spouse 2 of tax unit
    !PIT_NTIEARNE_ANN_SP2: Net taxable early retirement benefits of the new type of spouse 2 of tax unit
    !PIT_NTIEAROL_ANN_SP2: Net taxable early retirement benefits of the old type of spouse 2 of tax unit
    !PIT_NTIOLDUN_ANN_SP2: Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit
    !PIT_NTIOTHUN_ANN_SP2: Net taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit
    !PIT_NTIDISAB_ANN_SP2: Net taxable disability benefits of wage earners of spouse 2 of tax unit
    !PIT_NTINDOCC_ANN_SP2: Net taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit
    !PIT_NTISICK_ANN_SP2:  Net taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit
    DO pers_j=1,n_rec
        PIT_NTIPENS_ANN_SP2(pers_j) =   PIT_GTIPENS_ANN_SP2(pers_j) 
        PIT_NTIEARNE_ANN_SP2(pers_j) =  PIT_GTIEARNE_ANN_SP2(pers_j)
        PIT_NTIEAROL_ANN_SP2(pers_j) =  PIT_GTIEAROL_ANN_SP2(pers_j)
        PIT_NTIOLDUN_ANN_SP2(pers_j) =  PIT_GTIOLDUN_ANN_SP2(pers_j)
        PIT_NTIOTHUN_ANN_SP2(pers_j) =  PIT_GTIOTHUN_ANN_SP2(pers_j)
        PIT_NTIDISAB_ANN_SP2(pers_j) =  PIT_GTIDISAB_ANN_SP2(pers_j)
        PIT_NTINDOCC_ANN_SP2(pers_j) =  PIT_GTINDOCC_ANN_SP2(pers_j)
        PIT_NTISICK_ANN_SP2(pers_j) =   PIT_GTISICK_ANN_SP2(pers_j) 
    END DO

    !Compute net taxable income of spouse 2 earned as employee
    !PIT_COSTRATE_ANN_SP2: Lump sum costs of spouse 2 deductible of employment income
    !PIT_NTIEMPL_ANN_SP2: Net taxable employment income of spouse 2 of the tax unit
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = PIT_GTIPRIV_ANN_SP2(pers_j) + PIT_GTIPUB_ANN_SP2(pers_j)
        PIT_COSTRATE_ANN_SP2(pers_j) = PIT_BRACRATE_3COLUMNS(PIT_GTINC_ANN,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
        IF (PIT_COSTRATE_ANN_SP2(pers_j).LT.0.D0) THEN
            PIT_COSTRATE_ANN_SP2(pers_j) = 0.D0
        END IF
        !determination of net taxable income
        PIT_NTIEMPL_ANN_SP2(pers_j) = PIT_GTINC_ANN - PIT_COSTRATE_ANN_SP2(pers_j)
        IF (PIT_NTIEMPL_ANN_SP2(pers_j).LT.0.D0) THEN
            PIT_NTIEMPL_ANN_SP2(pers_j) = 0.D0
        END IF
    END DO

    !Compute net taxable income of spouse 2 earned as self employed
    !PIT_NTISELF_ANN_SP2: Net taxable self employment income of spouse 2 of the tax unit
    DO pers_j=1,n_rec
        !determination of lump sum costs
        PIT_GTINC_ANN = PIT_GTISELF_ANN_SP2(pers_j)
        IF (MIN((PIT_RALUMPSE_ANN/100.D0)*PIT_GTINC_ANN,PIT_MAXSELF_ANN).GT.0) THEN
            PIT_NTISELF_ANN_SP2(pers_j) = PIT_GTINC_ANN - MIN((PIT_RALUMPSE_ANN/100.D0)*PIT_GTINC_ANN,PIT_MAXSELF_ANN)
        ELSE
            PIT_NTISELF_ANN_SP2(pers_j) = PIT_GTINC_ANN 
        END IF
    END DO

    !Compute net taxable income of spouse 2 before appliction of marital split
    !PIT_NTOI_ANN_SP2: Net taxable occupational income of spouse 2 of the tax unit
    DO pers_j=1,n_rec
        PIT_NTOI_ANN_SP2(pers_j) = &
            &PIT_NTIPENS_ANN_SP2(pers_j)  + &
            &PIT_NTIEARNE_ANN_SP2(pers_j) + &
            &PIT_NTIEAROL_ANN_SP2(pers_j) + &
            &PIT_NTIOLDUN_ANN_SP2(pers_j) + &
            &PIT_NTIOTHUN_ANN_SP2(pers_j) + &
            &PIT_NTIDISAB_ANN_SP2(pers_j) + &
            &PIT_NTINDOCC_ANN_SP2(pers_j) + &
            &PIT_NTISICK_ANN_SP2(pers_j)  + &
            &PIT_NTIEMPL_ANN_SP2(pers_j)  + &
            &PIT_NTISELF_ANN_SP2(pers_j)  
    END DO

!Korte omschrijving scenario's
!Eerste scenario is gewoon aanpassing van bestaande regeling maar dan enkel voor gezinshoofden minder dan 50
!Tweede scenario is aanpassing voor alleenstaanden maar dan alleen als ze kinderen hebben en ook de enige
!fiscale eenheid in het gezin zijn
!Derde scenario is combinatie van de twee maar met gedifferentieerd maximum
!we passen daar de leeftijdsgrens enkel toe voor de oude gevallen, niet bij de nieuwe (i.e. alleenstaanden)

    !Apply the marital splitting rule
    !PIT_MARSPLIT_ANN: Amount of taxable income that can be transferred from one spouse to the other because of marital splitting rule    
    COUNT_MTSPPERC = 0
    COUNT_MTSPABS = 0
    COUNT_MTSPNEG = 0
    DO hh_i=1,N_HH_PIT
        !Doe hier loop per huishouden want je hebt huishoudnummer later nodig
        DO pers_j=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
            !Simulate baseline scenario: normal marital splitting rule
            IF (PIT_SIMHQ.EQ.0) THEN    
                !apply splitting rule in the line of the head of the tax unit
                IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
                    PIT_PCTNTOI_ANN = (PIT_PERMASPL_ANN/100.D0)*(PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j))   
                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                        !Compute amount to be transfered if spouse 1 has lowest income
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE 
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        END IF
                        !Correct if amount would become negative
                        !Would apply if net taxable income of the spouse with the lowest amount is already
                        !larger than the possbile maximal amount after transfer
                        IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                            PIT_MARSPLIT_ANN(pers_j) = 0.D0
                            COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                        END IF
                        !Correct net taxable income components
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                        END IF
                    END IF
                END IF
            !Simulate scenario 1: limitation in function of age head of the tax unit
            ELSE IF (PIT_SIMHQ.EQ.1) THEN    
                !apply splitting rule in the line of the head of the tax unit
                !only if the head is 50 or above
                IF (PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_YEAR_AGE(pers_j).GE.50) THEN
                    PIT_PCTNTOI_ANN = (PIT_PERMASPL_ANN/100.D0)*(PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j))   
                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                        !Compute amount to be transfered if spouse 1 has lowest income
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE 
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        END IF
                        !Correct if amount would become negative
                        !Would apply if net taxable income of the spouse with the lowest amount is already
                        !larger than the possbile maximal amount after transfer
                        IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                            PIT_MARSPLIT_ANN(pers_j) = 0.D0
                            COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                        END IF
                        !Correct net taxable income components
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                        END IF
                    END IF
                END IF
            !Simulate scenario 2: give extra HQ to singles
            ELSE IF (PIT_SIMHQ.EQ.2) THEN    
                !Check whether there is only one non married tax head with children in the household
                IF (PIT_NUMTAXU_ANN(pers_j).EQ.1.AND.PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_DEPCHILD_ANN(pers_j).GT.0) THEN
                    !If so, make the first child the partner
                    !Look up first child in the tax unit, loop over all members of the sociological household
                    !Het volstaat hier om over alle gezinsleden te lopen want er is maar n fiscale eenheid in het gezin
                    DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
                        IF (PIT_TURANK_ANN(pers_k).EQ.3) THEN
                            PIT_TURANK_ANN(pers_k) = 2
                            PIT_PARTID_ANN(pers_j) = pers_k
                            PIT_COUPLE_ANN(pers_j) = 1
                            PIT_HQOLDNEW(pers_j) = 1
                            PIT_COUPLE_ANN(pers_j) = 1
                            PIT_DEPCHILD_ANN(pers_j) = PIT_DEPCHILD_ANN(pers_j) - 1
                            !PIT_DEPCHIL3_ANN:     Number of children below the age of 3 that are dependent according to the personal income tax legislation
                            IF (PIT_YEAR_AGE(pers_k).LT.3) THEN
                                PIT_DEPCHIL3_ANN(pers_j) = PIT_DEPCHIL3_ANN(pers_j) - 1
                            END IF    
                            !PIT_DEPCHIHA_ANN:     Number of children that are dependent according to the personal income tax legislation AND that are disabled
                            IF (InFODSZ(pers_k).EQ.1) THEN
                                PIT_DEPCHIHA_ANN(pers_j) = PIT_DEPCHIHA_ANN(pers_j) - 1
                            END IF    
                            !If you find one child, exit the do loop
                            EXIT
                        END IF
                    END DO
                END IF
                !execute marital splitting for both old and new types independent of the age of the head
                !apply splitting rule in the line of the head of the tax unit
                IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
                    PIT_PCTNTOI_ANN = (PIT_PERMASPL_ANN/100.D0)*(PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j))   
                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                        !Compute amount to be transfered if spouse 1 has lowest income
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE 
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        END IF
                        !Correct if amount would become negative
                        !Would apply if net taxable income of the spouse with the lowest amount is already
                        !larger than the possbile maximal amount after transfer
                        IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                            PIT_MARSPLIT_ANN(pers_j) = 0.D0
                            COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                        END IF
                        !Correct net taxable income components
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                        END IF
                    END IF
                END IF
            !Simulate scenario 3: limitation in function of age head of the tax unit and give extra HQ to singles
            !but limit the amount of the benefit
            ELSE IF (PIT_SIMHQ.EQ.3) THEN    
                !Check whether there is only one non married tax head with children in the household
                IF (PIT_NUMTAXU_ANN(pers_j).EQ.1.AND.PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_DEPCHILD_ANN(pers_j).GT.0) THEN
!                    WRITE(15,*) 'TestA',pers_j
                    !If so, make the first child the partner
                    !Look up first child in the tax unit, loop over all members of the sociological household
                    !Het volstaat hier om over alle gezinsleden te lopen want er is maar n fiscale eenheid in het gezin
                    DO pers_k=PIT_START_HH(hh_i),PIT_END_HH(hh_i) 
                        IF (PIT_TURANK_ANN(pers_k).EQ.3) THEN
!                            WRITE(15,*) 'TestB',pers_j,pers_k
                            PIT_TURANK_ANN(pers_k) = 2
                            PIT_PARTID_ANN(pers_j) = pers_k
                            PIT_COUPLE_ANN(pers_j) = 1
                            PIT_HQOLDNEW(pers_j) = 1
                            PIT_COUPLE_ANN(pers_j) = 1
                            PIT_DEPCHILD_ANN(pers_j) = PIT_DEPCHILD_ANN(pers_j) - 1
                            !PIT_DEPCHIL3_ANN:     Number of children below the age of 3 that are dependent according to the personal income tax legislation
                            IF (PIT_YEAR_AGE(pers_k).LT.3) THEN
                                PIT_DEPCHIL3_ANN(pers_j) = PIT_DEPCHIL3_ANN(pers_j) - 1
                            END IF    
                            !PIT_DEPCHIHA_ANN:     Number of children that are dependent according to the personal income tax legislation AND that are disabled
                            IF (InFODSZ(pers_k).EQ.1) THEN
                                PIT_DEPCHIHA_ANN(pers_j) = PIT_DEPCHIHA_ANN(pers_j) - 1
                            END IF    
                            !If you find one child, exit the do loop
                            EXIT
                        END IF
                    END DO
                END IF
                !execute marital splitting for both old and new types 
                !treat cases differently
                !apply splitting rule in the line of the head of the tax unit
                !For old type, simple age limitation
                IF (PIT_HQOLDNEW(pers_j).EQ.0.AND.PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_YEAR_AGE(pers_j).GE.50) THEN
                    PIT_PCTNTOI_ANN = (PIT_PERMASPL_ANN/100.D0)*(PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j))   
                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                        !Compute amount to be transfered if spouse 1 has lowest income
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            !Correct if amount would exceed the legal transfer ceiling
                            IF (PIT_PCTNTOI_ANN.LT.PIT_ABSMASPL_ANN) THEN
                                PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                            ELSE 
                                PIT_MARSPLIT_ANN(pers_j) = PIT_ABSMASPL_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                COUNT_MTSPABS = COUNT_MTSPABS + 1 
                            END IF
                        END IF
                        !Correct if amount would become negative
                        !Would apply if net taxable income of the spouse with the lowest amount is already
                        !larger than the possbile maximal amount after transfer
                        IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                            PIT_MARSPLIT_ANN(pers_j) = 0.D0
                            COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                        END IF
                        !Correct net taxable income components
                        IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                        ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                            PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                            PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                        END IF
                    END IF
                END IF
                !In case of new type, use decreasing amounts
                IF (PIT_HQOLDNEW(pers_j).EQ.1.AND.PIT_COUPLE_ANN(pers_j).EQ.1) THEN
!                    WRITE(15,*) 'TestC',pers_j,pers_k
                    PIT_PCTNTOI_ANN = (PIT_PERMASPL_ANN/100.D0)*(PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j))   
                    PIT_NTINC_ANN = PIT_NTOI_ANN_SP1(pers_j) + PIT_NTOI_ANN_SP2(pers_j)
                    DO j=1,6
                        IF (j.LE.5) THEN
                            !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
                            IF (INT(100.D0*PIT_NTINC_ANN).GE.INT(100.D0*PIT_RATSCHEM_ANN(j,1)).AND.&
                            &INT(100.D0*PIT_NTINC_ANN).LT.INT(100.D0*PIT_RATSCHEM_ANN(j,2))) THEN                                             
                                IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) &
                                &THEN
                                    !Compute amount to be transfered if spouse 1 has lowest income
                                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        !Correct if amount would exceed the legal transfer ceiling
                                        IF (PIT_PCTNTOI_ANN.LT.PIT_ABSHQ_ANN(j)) THEN
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                            COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                                        ELSE
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_ABSHQ_ANN(j) - PIT_NTOI_ANN_SP1(pers_j)
                                            COUNT_MTSPABS = COUNT_MTSPABS + 1 
                                        END IF
                                    ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        !Correct if amount would exceed the legal transfer ceiling
                                        IF (PIT_PCTNTOI_ANN.LT.PIT_ABSHQ_ANN(j)) THEN
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                            COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                                        ELSE 
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_ABSHQ_ANN(j) - PIT_NTOI_ANN_SP2(pers_j)
                                            COUNT_MTSPABS = COUNT_MTSPABS + 1 
                                        END IF
                                    END IF
                                    !Correct if amount would become negative
                                    !Would apply if net taxable income of the spouse with the lowest amount is already
                                    !larger than the possbile maximal amount after transfer
                                    IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                                        PIT_MARSPLIT_ANN(pers_j) = 0.D0
                                        COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                                    END IF
                                    !Correct net taxable income components
                                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                                        PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                                    ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                                        PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                                    END IF
                                    EXIT
                                END IF
                            END IF
                        ELSE
                            !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
                            IF (INT(100.D0*PIT_NTINC_ANN).GE.INT(100.D0*PIT_RATSCHEM_ANN(j,1))) THEN                                             
                                IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN.OR.PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) &
                                &THEN
                                    !Compute amount to be transfered if spouse 1 has lowest income
                                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        !Correct if amount would exceed the legal transfer ceiling
                                        IF (PIT_PCTNTOI_ANN.LT.PIT_ABSHQ_ANN(j)) THEN
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP1(pers_j)
                                            COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                                        ELSE
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_ABSHQ_ANN(j) - PIT_NTOI_ANN_SP1(pers_j)
                                            COUNT_MTSPABS = COUNT_MTSPABS + 1 
                                        END IF
                                    ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        !Correct if amount would exceed the legal transfer ceiling
                                        IF (PIT_PCTNTOI_ANN.LT.PIT_ABSHQ_ANN(j)) THEN
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_PCTNTOI_ANN - PIT_NTOI_ANN_SP2(pers_j)
                                            COUNT_MTSPPERC = COUNT_MTSPPERC + 1 
                                        ELSE 
                                            PIT_MARSPLIT_ANN(pers_j) = PIT_ABSHQ_ANN(j) - PIT_NTOI_ANN_SP2(pers_j)
                                            COUNT_MTSPABS = COUNT_MTSPABS + 1 
                                        END IF
                                    END IF
                                    !Correct if amount would become negative
                                    !Would apply if net taxable income of the spouse with the lowest amount is already
                                    !larger than the possbile maximal amount after transfer
                                    IF (PIT_MARSPLIT_ANN(pers_j).LT.0.D0) THEN
                                        PIT_MARSPLIT_ANN(pers_j) = 0.D0
                                        COUNT_MTSPNEG = COUNT_MTSPNEG + 1
                                    END IF
                                    !Correct net taxable income components
                                    IF (PIT_NTOI_ANN_SP1(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) + PIT_MARSPLIT_ANN(pers_j)
                                        PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) - PIT_MARSPLIT_ANN(pers_j)      
                                    ELSE IF (PIT_NTOI_ANN_SP2(pers_j).LT.PIT_PCTNTOI_ANN) THEN
                                        PIT_NTOI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j) - PIT_MARSPLIT_ANN(pers_j)
                                        PIT_NTOI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j) + PIT_MARSPLIT_ANN(pers_j)      
                                    END IF
                                    EXIT
                                END IF
                            END IF
                        END IF
                    END DO
                END IF
            END IF
        END DO        
    END DO        

    !Correct net taxable income if negative
    DO pers_j=1,n_rec
        IF (PIT_NTOI_ANN_SP1(pers_j).LT.0.D0) THEN
            PIT_NTOI_ANN_SP1(pers_j) = 0.D0
        END IF
        IF (PIT_NTOI_ANN_SP2(pers_j).LT.0.D0) THEN
            PIT_NTOI_ANN_SP2(pers_j) = 0.D0
        END IF
    END DO
    
    !TEST
    !Write results of construction of final tax units
    Write(15,'('' '')')
    Write(15,'(''Counts on execution of marital splitting rule'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times percentage splitting is applied        '', i10)') COUNT_MTSPPERC
    Write(15,'(''Number of times absolute amount is applied             '', i10)') COUNT_MTSPABS
    Write(15,'(''Number of times lowest income exceeds absolute amount  '', i10)') COUNT_MTSPNEG

    !Compute net taxable income of spouse 1
    !PIT_NTI_ANN_SP1: Total amount of net taxable income of spouse 1 of the tax unit
    DO pers_j=1,n_rec
        PIT_NTI_ANN_SP1(pers_j) = PIT_NTOI_ANN_SP1(pers_j)
    END DO

    !Compute net taxable income of spouse 2
    !PIT_NTI_ANN_SP2: Total amount of net taxable income of spouse 2 of the tax unit
    DO pers_j=1,n_rec
        PIT_NTI_ANN_SP2(pers_j) = PIT_NTOI_ANN_SP2(pers_j)
    END DO

END SUBROUTINE PIT_TUNTI_CREATE

!===================================================================================!
!With subroutine PIT_GPIT_CREATE we create the gross amount of personal income taxes!
!of each spouse                                                                     !
!Variables created                                                                  !
!   PIT_GPIT_ANN_SP1                                                                !
!   PIT_GPIT_ANN_SP2                                                                !
!===================================================================================!

SUBROUTINE PIT_GPIT_CREATE

    INTEGER :: pers_j

    !Determination of gross personal income taxes of spouse 1   
    !PIT_GPIT_ANN_SP1: Gross amount of personal income taxes of spouse 1 after applying the global tax rate scheme on net taxable income
    DO pers_j=1,n_rec
        IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
            PIT_GPIT_ANN_SP1(pers_j) = PIT_BRACRATE_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_RATSCHEM_ANN,1),&
            &PIT_RATSCHEM_ANN,pers_j)
        END IF            
    END DO

    !Determination of gross personal income taxes of spouse 2   
    !PIT_GPIT_ANN_SP2: Gross amount of personal income taxes of spouse 2 after applying the global tax rate scheme on net taxable income
    DO pers_j=1,n_rec
        IF (PIT_TURANK_ANN(pers_j).EQ.1) THEN
            PIT_GPIT_ANN_SP2(pers_j) = PIT_BRACRATE_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_RATSCHEM_ANN,1),&
            &PIT_RATSCHEM_ANN,pers_j)
        END IF 
    END DO

END SUBROUTINE PIT_GPIT_CREATE

!===================================================================================!
!With subroutine PIT_TCFAMCOM_CREATE we compute the tax credit that compensates for !
!family composition                                                                 !
!Variables created                                                                  !
!   PIT_EXEMP_ANN_SP1                                                               !
!   PIT_EXEMP_ANN_SP2                                                               !
!   PIT_EXEMPCH_ANN_SP1                                                             !
!   PIT_EXEMPCH_ANN_SP2                                                             !
!   PIT_PITFAMC_ANN_SP1                                                             !
!   PIT_PITFAMC_ANN_SP2                                                             !
!   PIT_PITFAMC_ANN                                                                 !
!   PIT_REBATCH_ANN                                                                 !
!===================================================================================!

SUBROUTINE PIT_TCFAMCOM_CREATE

    INTEGER :: pers_j
    INTEGER                         :: COUNT_EXEMSP1C,COUNT_EXEMSP2C,COUNT_EXEMSP1S
    INTEGER                         :: COUNT_CREDSP1C,COUNT_CREDSP2C,COUNT_CREDSP1S
    REAL(KIND=KIND_REAL)            :: AMOUNT1,AMOUNT2,PIT_TAXCRCH_ANN                  !Instruments used to compute the rebatable tax credit for children
    INTEGER                         :: SUMDEPCH                                         !Number of dependent children (children and double count for children that are disabled)

    !Compute exempted amounts
    !PIT_EXEMP_ANN_SP1: Income amount of spouse 1 that is used to compute the tax credit for family composition
    !PIT_EXEMP_ANN_SP2: Income amount of spouse 2 that is used to compute the tax credit for family composition
    COUNT_EXEMSP1C = 0 
    COUNT_EXEMSP2C = 0 
    COUNT_EXEMSP1S = 0
    DO pers_j=1,n_rec
        SUMDEPCH = 0
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            !Compute exemption
            IF (SIMYEAR.LT.2008) THEN
                !Basic exemption is attributed to each spouse
                IF (InFODSZ(pers_j).EQ.0) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMPC_ANN   
                ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMPC_ANN + PIT_EXEHANSP_ANN
                END IF
                IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.0) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMPC_ANN   
                ELSE IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.1) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMPC_ANN + PIT_EXEHANSP_ANN
                END IF
            ELSE
                !Basic exemption is attributed to each spouse
                !Different amount is applied in case of disability
                IF (InFODSZ(pers_j).EQ.0) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                    &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                    &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                END IF
                IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.0) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                    &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                ELSE IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.1) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                    &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                END IF
            END IF
            !Distribute basic exemptions over both spouses
            IF (PIT_NTI_ANN_SP1(pers_j).GE.PIT_NTI_ANN_SP2(pers_j)) THEN
                IF (PIT_EXEMP_ANN_SP2(pers_j).GT.PIT_NTI_ANN_SP2(pers_j)) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + (PIT_EXEMP_ANN_SP2(pers_j) - PIT_NTI_ANN_SP2(pers_j))
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_NTI_ANN_SP2(pers_j)
                    IF (PIT_EXEMP_ANN_SP1(pers_j).GT.PIT_NTI_ANN_SP1(pers_j)) THEN
                        PIT_EXEMP_ANN_SP1(pers_j) = PIT_NTI_ANN_SP1(pers_j)
                    END IF
                END IF 
            ELSE
                IF (PIT_EXEMP_ANN_SP1(pers_j).GT.PIT_NTI_ANN_SP1(pers_j)) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + (PIT_EXEMP_ANN_SP1(pers_j) - PIT_NTI_ANN_SP1(pers_j))
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_NTI_ANN_SP1(pers_j)
                    IF (PIT_EXEMP_ANN_SP2(pers_j).GT.PIT_NTI_ANN_SP2(pers_j)) THEN
                        PIT_EXEMP_ANN_SP2(pers_j) = PIT_NTI_ANN_SP2(pers_j)
                    END IF
                END IF 
            END IF
            !Other exemptions are first attributed to the tax unit with the highest income
            IF (PIT_NTI_ANN_SP1(pers_j).GE.PIT_NTI_ANN_SP2(pers_j)) THEN
                SUMDEPCH = PIT_DEPCHILD_ANN(pers_j) + PIT_DEPCHIHA_ANN(pers_j)
                !Exemptions for dependent children
                PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                            &PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                            &PIT_EXEMSCH_ANN,pers_j)
                PIT_EXEMPCH_ANN_SP1(pers_j) = PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                            &PIT_EXEMSCH_ANN,pers_j)
                !part that is rebatable for children in charge
                PIT_REBATCH_ANN(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) - PIT_NTI_ANN_SP1(pers_j)
                !Exemptions for dependent children below the age of 3
                PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                            &DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN 
                PIT_EXEMPCH_ANN_SP1(pers_j) = PIT_EXEMPCH_ANN_SP1(pers_j) + DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN
                !Exemptions for dependent others
                PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                            &DBLE(PIT_DEPOTHER_ANN(pers_j))*PIT_EXEOTH_ANN + &
                                            &DBLE(PIT_DEPOTHHA_ANN(pers_j))*PIT_EXEHANOT_ANN 
                !Exemption for dependent ascendants
                IF (PIT_DEPOLDER_ANN(pers_j).GT.0) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + PIT_EXEOLD_ANN 
                END IF                    
            ELSE
                SUMDEPCH = PIT_DEPCHILD_ANN(pers_j) + PIT_DEPCHIHA_ANN(pers_j)
                !Exemptions for dependent children
                PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + &
                                            &PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                            &PIT_EXEMSCH_ANN,pers_j)
                PIT_EXEMPCH_ANN_SP2(pers_j) = PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                            &PIT_EXEMSCH_ANN,pers_j)
                !part that is rebatable for children in charge
                PIT_REBATCH_ANN(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) - PIT_NTI_ANN_SP2(pers_j)
                !Exemptions for dependent children below the age of 3
                PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + &
                                            &DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN 
                PIT_EXEMPCH_ANN_SP2(pers_j) = PIT_EXEMPCH_ANN_SP2(pers_j) + DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN
                !Exemptions for dependent others
                PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + &
                                            &DBLE(PIT_DEPOTHER_ANN(pers_j))*PIT_EXEOTH_ANN + &
                                            &DBLE(PIT_DEPOTHHA_ANN(pers_j))*PIT_EXEHANOT_ANN
                !Exemption for dependent ascendants
                IF (PIT_DEPOLDER_ANN(pers_j).GT.0) THEN
                    PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + PIT_EXEOLD_ANN 
                END IF                    
            END IF
            !Transfer exemption if there is no full take up
            PIT_EXEMTRAN_ANN(pers_j) = 0
            IF (PIT_EXEMP_ANN_SP1(pers_j).GT.PIT_NTI_ANN_SP1(pers_j)) THEN
                PIT_EXEMTRAN_ANN(pers_j)    = PIT_EXEMP_ANN_SP1(pers_j) - PIT_NTI_ANN_SP1(pers_j)
                PIT_EXEMP_ANN_SP1(pers_j)   = PIT_NTI_ANN_SP1(pers_j)
                COUNT_EXEMSP1C = COUNT_EXEMSP1C + 1 
            END IF
            PIT_EXEMP_ANN_SP2(pers_j) = PIT_EXEMP_ANN_SP2(pers_j) + PIT_EXEMTRAN_ANN(pers_j)
            IF (PIT_EXEMP_ANN_SP2(pers_j).GT.PIT_NTI_ANN_SP2(pers_j)) THEN
                PIT_EXEMTRAN_ANN(pers_j)    = PIT_EXEMP_ANN_SP2(pers_j) - PIT_NTI_ANN_SP2(pers_j)
                PIT_EXEMP_ANN_SP2(pers_j)   = PIT_NTI_ANN_SP2(pers_j)
                COUNT_EXEMSP2C = COUNT_EXEMSP2C + 1 
            ELSE
                PIT_EXEMTRAN_ANN(pers_j)    = 0.D0
                COUNT_EXEMSP2C = COUNT_EXEMSP2C + 1 
            END IF
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
            !Compute exemption
            IF (SIMYEAR.LT.2008) THEN
                !Basic exemption is attributed to the single tax-payer
                IF (InFODSZ(pers_j).EQ.0) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMPS_ANN   
                ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMPS_ANN + PIT_EXEHANSP_ANN
                END IF
            ELSE
                !Basic exemption is attributed to the single tax-payer
                IF (InFODSZ(pers_j).EQ.0) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                    &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j)) 
                ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                    PIT_EXEMP_ANN_SP1(pers_j) = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                    &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j)) 
                END IF
            END IF
            !Other exemptions are first attributed to the head of the tax unit
            !Exemptions for dependent children
            SUMDEPCH = PIT_DEPCHILD_ANN(pers_j) + PIT_DEPCHIHA_ANN(pers_j)
            PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                        &PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                        &PIT_EXEMSCH_ANN,pers_j)
            PIT_EXEMPCH_ANN_SP1(pers_j) = PIT_SUMAMOUNT_1COLUMN(SUMDEPCH,SIZE(PIT_EXEMSCH_ANN,1),&
                                        &PIT_EXEMSCH_ANN,pers_j)
            !part that is rebatable for children in charge
            PIT_REBATCH_ANN(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) - PIT_NTI_ANN_SP1(pers_j)
            !Exemptions for dependent children below the age of 3
            PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                        &DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN 
            PIT_EXEMPCH_ANN_SP1(pers_j) = PIT_EXEMPCH_ANN_SP1(pers_j) + DBLE(PIT_DEPCHIL3_ANN(pers_j))*PIT_EXECHLA_ANN
            !Exemptions for dependent others
            PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + &
                                        &DBLE(PIT_DEPOTHER_ANN(pers_j))*PIT_EXEOTH_ANN + &
                                        &DBLE(PIT_DEPOTHHA_ANN(pers_j))*PIT_EXEHANOT_ANN 
            !Exemption for dependent ascendants
            IF (PIT_DEPOLDER_ANN(pers_j).GT.0) THEN
                PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + PIT_EXEOLD_ANN 
            END IF                    
            !Additional exemption if the taxpayer is single and has dependent children
            IF (PIT_DEPCHILD_ANN(pers_j).GT.0) THEN
                PIT_EXEMP_ANN_SP1(pers_j) = PIT_EXEMP_ANN_SP1(pers_j) + PIT_EXESDEP_ANN
            END IF                    
            !Limit exemption if there is no full take up
            IF (PIT_EXEMP_ANN_SP1(pers_j).GT.PIT_NTI_ANN_SP1(pers_j)) THEN
                PIT_EXEMTRAN_ANN(pers_j)    = PIT_EXEMP_ANN_SP1(pers_j) - PIT_NTI_ANN_SP1(pers_j)
                PIT_EXEMP_ANN_SP1(pers_j)   = PIT_NTI_ANN_SP1(pers_j)
                COUNT_EXEMSP1S = COUNT_EXEMSP1S + 1 
            END IF
        END IF
    END DO

    !TEST
    !Write results of construction of tax exemptions for family composition
    Write(15,'('' '')')
    Write(15,'(''Counts on exemptions that exceed taxable income'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times exemption is larger than TI of spouse1 in couple   '', i10)') COUNT_EXEMSP1C
    Write(15,'(''Number of times exemption is larger than TI of spouse2 in couple   '', i10)') COUNT_EXEMSP2C
    Write(15,'(''Number of times exemption is larger than TI of single              '', i10)') COUNT_EXEMSP1S

    !Compute exempted amounts
    !PIT_PITFAMC_ANN_SP1: Personal income taxes of spouse 1 net of tax credits for family composition
    !PIT_PITFAMC_ANN_SP2: Personal income taxes of spouse 2 net of tax credits for family composition
    !PIT_PITFAMC_ANN: Personal income taxes at the tax unit level net of tax credits for family composition
    COUNT_CREDSP1C = 0 
    COUNT_CREDSP2C = 0 
    COUNT_CREDSP1S = 0
    DO pers_j=1,n_rec
        SUMDEPCH = 0
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            !Subtract tax equivalent of exempted amount from personal income taxes
            PIT_PITFAMC_ANN_SP1(pers_j) = PIT_GPIT_ANN_SP1(pers_j) - PIT_BRACRATE_3COLUMNS(PIT_EXEMP_ANN_SP1(pers_j),&
            &SIZE(PIT_RATSCHEM_ANN,1),PIT_RATSCHEM_ANN,pers_j)
            !correct if negative
            IF (PIT_PITFAMC_ANN_SP1(pers_j).LT.0.D0) THEN
                PIT_PITFAMC_ANN_SP1(pers_j) = 0.D0
                COUNT_EXEMSP1C = COUNT_EXEMSP1C + 1 
            END IF
            !Subtract tax equivalent of exempted amount from personal income taxes
            PIT_PITFAMC_ANN_SP2(pers_j) = PIT_GPIT_ANN_SP2(pers_j) - PIT_BRACRATE_3COLUMNS(PIT_EXEMP_ANN_SP2(pers_j),&
            &SIZE(PIT_RATSCHEM_ANN,1),PIT_RATSCHEM_ANN,pers_j)
            !correct if negative
            IF (PIT_PITFAMC_ANN_SP2(pers_j).LT.0.D0) THEN
                PIT_PITFAMC_ANN_SP2(pers_j) = 0.D0
                COUNT_EXEMSP2C = COUNT_EXEMSP2C + 1 
            END IF
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
            !Subtract tax equivalent of exempted amount from personal income taxes
            PIT_PITFAMC_ANN_SP1(pers_j) = PIT_GPIT_ANN_SP1(pers_j) - PIT_BRACRATE_3COLUMNS(PIT_EXEMP_ANN_SP1(pers_j),&
            &SIZE(PIT_RATSCHEM_ANN,1),PIT_RATSCHEM_ANN,pers_j)
            !correct if negative
            IF (PIT_PITFAMC_ANN_SP1(pers_j).LT.0.D0) THEN
                PIT_PITFAMC_ANN_SP1(pers_j) = 0.D0
                COUNT_EXEMSP1S = COUNT_EXEMSP1S + 1 
            END IF
        END IF
        !Compute total amount of personal income taxes at the level of the tax unit
        PIT_PITFAMC_ANN(pers_j) = PIT_PITFAMC_ANN_SP1(pers_j) + PIT_PITFAMC_ANN_SP2(pers_j)
        !Compute exemption
        IF (SIMYEAR.GT.2001) THEN
            !If there is still a part that can be rebated because of family composition compute the relevant amount
            IF (INT(100.D0*PIT_REBATCH_ANN(pers_j)).GT.0) THEN
                AMOUNT1         = PIT_REBATCH_ANN(pers_j)
                AMOUNT2         = PIT_EXEMPCH_ANN_SP1(pers_j)+PIT_EXEMPCH_ANN_SP2(pers_j)
                PIT_TAXCRCH_ANN = 0.D0
                IF (AMOUNT1.GT.AMOUNT2) THEN
                    PIT_TAXCRCH_ANN = PIT_BRACRATE_3COLUMNS(AMOUNT2,&
                    &SIZE(PIT_RATSCHEM_ANN,1),PIT_RATSCHEM_ANN,pers_j)
                ELSE
                    PIT_TAXCRCH_ANN = PIT_BRACRATE_3COLUMNS(AMOUNT1,&
                    &SIZE(PIT_RATSCHEM_ANN,1),PIT_RATSCHEM_ANN,pers_j)
                END IF
                !limit the computed amount in function of the maximum set by the legislator
                SUMDEPCH = PIT_DEPCHILD_ANN(pers_j) + PIT_DEPCHIHA_ANN(pers_j)
                IF (PIT_TAXCRCH_ANN.GT.PIT_REBTACR_ANN*SUMDEPCH) THEN
                    PIT_TAXCRCH_ANN = PIT_REBTACR_ANN*SUMDEPCH    
                END IF
                !Adapt the computed amount of personal income taxes at the level of the tax unit
                PIT_PITFAMC_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) - PIT_TAXCRCH_ANN
            END IF
        END IF            
    END DO        
        
    !TEST
    !Write results of construction of tax credits for family composition
    Write(15,'('' '')')
    Write(15,'(''Counts on credits for family composition that exceed personal income taxes'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times credit is larger than PIT of spouse1 in couple   '', i10)') COUNT_CREDSP1C
    Write(15,'(''Number of times credit is larger than PIT of spouse2 in couple   '', i10)') COUNT_CREDSP2C
    Write(15,'(''Number of times credit is larger than PIT of single              '', i10)') COUNT_CREDSP1S

END SUBROUTINE PIT_TCFAMCOM_CREATE

!===================================================================================!
!With subroutine PIT_TCREPINC_CREATE_2001 we compute the tax credits that compensates for!
!replacement income                                                                 !
!Variables created                                                                  !
!   PIT_PITREP_ANN                                                                  !
!   PIT_FRACREP_ANN1                                                                !
!   PIT_FRACREP_ANN2                                                                !
!   PIT_FRACREP_ANN3                                                                !
!   PIT_FRACREP_ANN4                                                                !
!   PIT_FRACREP_ANN5                                                                !
!   PIT_FRACREP_ANN6                                                                !
!   PIT_BTCREP_ANN1                                                                 !
!   PIT_BTCREP_ANN2                                                                 !
!   PIT_BTCREP_ANN3                                                                 !
!   PIT_BTCREP_ANN4                                                                 !
!   PIT_BTCREP_ANN5                                                                 !
!   PIT_BTCREP_ANN6                                                                 !
!   PIT_TNTI_ANN                                                                    !
!   PIT_RTCREP_ANN1                                                                 !
!   PIT_RTCREP_ANN2                                                                 !
!   PIT_RTCREP_ANN3                                                                 !
!   PIT_RTCREP_ANN4                                                                 !
!   PIT_RTCREP_ANN5                                                                 !
!   PIT_RTCREP_ANN6                                                                 !
!===================================================================================!

SUBROUTINE PIT_TCREPINC_CREATE_2001

    INTEGER :: pers_j,i
    REAL(KIND=KIND_REAL),DIMENSION(6)               :: FRAC_REP       !Intermediate value of fractions
    REAL(KIND=KIND_REAL),DIMENSION(6)               :: QUANT_REP      !Amount of replacement income of a certain kind
    INTEGER                                         :: COUNT_CREDRE1,COUNT_CREDRE2

    !PIT_TNTI_ANN: Total net taxable income of the tax unit
    DO pers_j=1,n_rec
        PIT_TNTI_ANN(pers_j) = PIT_NTI_ANN_SP1(pers_j) + PIT_NTI_ANN_SP2(pers_j)
    END DO

    !Set maximal basic tax credit for replacement income
    !PIT_BTCREP_ANN1: Basic tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_BTCREP_ANN2: Basic tax credit granted for early retirement benefits of old type
    !PIT_BTCREP_ANN3: Basic tax credit granted for normal unemployment benefits
    !PIT_BTCREP_ANN4: Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_BTCREP_ANN5: Basic tax credit granted for sickness and disability benefits
    !PIT_BTCREP_ANN6: Basic tax credit granted for other replacement income
    DO pers_j=1,n_rec
        !Different basic credits apply for couples and singles
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1) = PIT_MAXREPL_ANN(1,2)
            PIT_BTCREP_ANN2(pers_j,1) = PIT_MAXREPL_ANN(2,2)
            PIT_BTCREP_ANN3(pers_j,1) = PIT_MAXREPL_ANN(3,2)
            PIT_BTCREP_ANN4(pers_j,1) = PIT_MAXREPL_ANN(4,2)
            PIT_BTCREP_ANN5(pers_j,1) = PIT_MAXREPL_ANN(5,2)
            PIT_BTCREP_ANN6(pers_j,1) = PIT_MAXREPL_ANN(6,2)
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1) = PIT_MAXREPL_ANN(1,1)
            PIT_BTCREP_ANN2(pers_j,1) = PIT_MAXREPL_ANN(2,1)
            PIT_BTCREP_ANN3(pers_j,1) = PIT_MAXREPL_ANN(3,1)
            PIT_BTCREP_ANN4(pers_j,1) = PIT_MAXREPL_ANN(4,1)
            PIT_BTCREP_ANN5(pers_j,1) = PIT_MAXREPL_ANN(5,1)
            PIT_BTCREP_ANN6(pers_j,1) = PIT_MAXREPL_ANN(6,1)
        END IF
    END DO

    !PIT_FRACREP_ANN1: Fraction applied on pension benefits and early retirement benefits of new type
    !PIT_FRACREP_ANN2: Fraction applied on early retirement benefits of old type
    !PIT_FRACREP_ANN3: Fraction applied on normal unemployment benefits
    !PIT_FRACREP_ANN4: Fraction applied on unemployment benefits of older unemployed (age of 58 or above)
    !PIT_FRACREP_ANN5: Fraction applied on sickness and disability benefits
    !PIT_FRACREP_ANN6: Fraction applied on other replacement income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            FRAC_REP(1:6) = 0.D0
            !Compute fractions
            IF (INT(100.D0*PIT_TNTI_ANN(pers_j)).NE.0) THEN
                FRAC_REP(1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j) + &
                              &PIT_NTIPENS_ANN_SP2(pers_j) + PIT_NTIEARNE_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(2) = (PIT_NTIEAROL_ANN_SP1(pers_j) + PIT_NTIEAROL_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(3) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(4) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(5) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTIDISAB_ANN_SP2(pers_j) &
                              &+ PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j) +&
                              & PIT_NTINDOCC_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(6) = 0
            END IF
            !Correct fractions if necessary
            DO i=1,6
                IF (FRAC_REP(i).LT.0.D0) THEN
                    FRAC_REP(i) = 0.D0
                END IF
                IF (FRAC_REP(i).GT.1.D0) THEN
                    FRAC_REP(i) = 1.D0
                END IF
            END DO
            !Store fractions at individual level
            PIT_FRACREP_ANN1(pers_j,1) = FRAC_REP(1)
            PIT_FRACREP_ANN2(pers_j,1) = FRAC_REP(2)
            PIT_FRACREP_ANN3(pers_j,1) = FRAC_REP(3)
            PIT_FRACREP_ANN4(pers_j,1) = FRAC_REP(4)
            PIT_FRACREP_ANN5(pers_j,1) = FRAC_REP(5)
            PIT_FRACREP_ANN6(pers_j,1) = FRAC_REP(6)
        END IF
    END DO

    !Continue computation of basic tax credits
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_BTCREP_ANN1(pers_j,1) = PIT_BTCREP_ANN1(pers_j,1)*PIT_FRACREP_ANN1(pers_j,1)
            PIT_BTCREP_ANN2(pers_j,1) = PIT_BTCREP_ANN2(pers_j,1)*PIT_FRACREP_ANN2(pers_j,1)
            PIT_BTCREP_ANN3(pers_j,1) = PIT_BTCREP_ANN3(pers_j,1)*PIT_FRACREP_ANN3(pers_j,1)
            PIT_BTCREP_ANN4(pers_j,1) = PIT_BTCREP_ANN4(pers_j,1)*PIT_FRACREP_ANN4(pers_j,1)
            PIT_BTCREP_ANN5(pers_j,1) = PIT_BTCREP_ANN5(pers_j,1)*PIT_FRACREP_ANN5(pers_j,1)
            PIT_BTCREP_ANN6(pers_j,1) = PIT_BTCREP_ANN6(pers_j,1)*PIT_FRACREP_ANN6(pers_j,1)
        END IF
    END DO
    
    !PIT_RTCREP_ANN1: Reduced tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_RTCREP_ANN2: Reduced tax credit granted for early retirement benefits of old type
    !PIT_RTCREP_ANN3: Reduced tax credit granted for normal unemployment benefits
    !PIT_RTCREP_ANN4: Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_RTCREP_ANN5: Reduced tax credit granted for sickness and disability benefits
    !PIT_RTCREP_ANN6: Reduced tax credit granted for other replacement income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_RTCREP_ANN1(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN1(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)
            PIT_RTCREP_ANN2(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN2(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)
            PIT_RTCREP_ANN3(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN3(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RUNEMPL_ANN,1),PIT_RUNEMPL_ANN,pers_j)
            PIT_RTCREP_ANN4(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN4(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)
            PIT_RTCREP_ANN5(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN5(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)
            PIT_RTCREP_ANN6(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN6(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                        &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)
        END IF
    END DO

    !Limit tax credits in function of the personal income tax fraction
    !A tax credit can not exceed the fraction of personal income taxes generated by the income source
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            IF (PIT_RTCREP_ANN1(pers_j,1).GT.(PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN1(pers_j,1) = (PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
            IF (PIT_RTCREP_ANN2(pers_j,1).GT.(PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN2(pers_j,1) = (PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
            IF (PIT_RTCREP_ANN3(pers_j,1).GT.(PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN3(pers_j,1) = (PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
            IF (PIT_RTCREP_ANN4(pers_j,1).GT.(PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN4(pers_j,1) = (PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
            IF (PIT_RTCREP_ANN5(pers_j,1).GT.(PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN5(pers_j,1) = (PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
            IF (PIT_RTCREP_ANN6(pers_j,1).GT.(PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                PIT_RTCREP_ANN6(pers_j,1) = (PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
            END IF
        END IF
    END DO

    !PIT_PITREP_ANN: Personal income taxes at the tax unit level net of tax credits for family composition and credits for replacement income
    COUNT_CREDRE1 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            IF (PIT_PITFAMC_ANN(pers_j).GT.0.D0) THEN
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) - (PIT_RTCREP_ANN1(pers_j,1) + PIT_RTCREP_ANN2(pers_j,1) + &
                                        &PIT_RTCREP_ANN3(pers_j,1) + PIT_RTCREP_ANN4(pers_j,1) + PIT_RTCREP_ANN5(pers_j,1) + &
                                        &PIT_RTCREP_ANN6(pers_j,1))
                IF (PIT_PITREP_ANN(pers_j).LT.0.D0) THEN
                    PIT_PITREP_ANN(pers_j) = 0.D0
                    COUNT_CREDRE1 = COUNT_CREDRE1 + 1 
                END IF
            ELSE
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) 
            END IF
        END IF
    END DO

    !Check whether replacement income is of a unique type
    COUNT_CREDRE2 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            QUANT_REP(1:6) = 0.D0
            QUANT_REP(1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j) + &
                           &PIT_NTIPENS_ANN_SP2(pers_j) + PIT_NTIEARNE_ANN_SP2(pers_j))
            QUANT_REP(2) = (PIT_NTIEAROL_ANN_SP1(pers_j) + PIT_NTIEAROL_ANN_SP2(pers_j))
            QUANT_REP(3) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))
            QUANT_REP(4) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))
            QUANT_REP(5) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTIDISAB_ANN_SP2(pers_j) &
                           &+ PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j) +&
                           & PIT_NTINDOCC_ANN_SP2(pers_j))
            QUANT_REP(6) = 0
            !check whether replacement income is of a unique type and taxes still positive
            DO i=1,6
                IF (INT(QUANT_REP(i)).EQ.INT(PIT_TNTI_ANN(pers_j)).AND.&
                    &INT(100.D0*QUANT_REP(i)).NE.0.AND.QUANT_REP(i).LT.PIT_UNIQREPL_ANN(i).AND.&
                    &PIT_PITREP_ANN(pers_j).GT.0.D0) THEN
                        PIT_PITREP_ANN(pers_j) = 0.D0
                        COUNT_CREDRE2 = COUNT_CREDRE2 + 1 
                        EXIT
                END IF                    
            END DO
        END IF
    END DO

    !TEST
    !Write results of construction of tax credits for replacement income
    Write(15,'('' '')')
    Write(15,'(''Counts on credits for replacement income'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times credit is larger than PIT of the TU            '', i10)') COUNT_CREDRE1
    Write(15,'(''Number of times income is of unique source and below ceiling   '', i10)') COUNT_CREDRE2
    
END SUBROUTINE PIT_TCREPINC_CREATE_2001

!===================================================================================!
!With subroutine PIT_TCREPINC_CREATE_2004 we compute the tax credits that           !
!compensates for replacement income                                                 !
!Variables created                                                                  !
!   PIT_PITREP_ANN                                                                  !
!   PIT_FRACREP_ANN1                                                                !
!   PIT_FRACREP_ANN2                                                                !
!   PIT_FRACREP_ANN3                                                                !
!   PIT_FRACREP_ANN4                                                                !
!   PIT_FRACREP_ANN5                                                                !
!   PIT_FRACREP_ANN6                                                                !
!   PIT_BTCREP_ANN1                                                                 !
!   PIT_BTCREP_ANN2                                                                 !
!   PIT_BTCREP_ANN3                                                                 !
!   PIT_BTCREP_ANN4                                                                 !
!   PIT_BTCREP_ANN5                                                                 !
!   PIT_BTCREP_ANN6                                                                 !
!   PIT_TNTI_ANN                                                                    !
!   PIT_RTCREP_ANN1                                                                 !
!   PIT_RTCREP_ANN2                                                                 !
!   PIT_RTCREP_ANN3                                                                 !
!   PIT_RTCREP_ANN4                                                                 !
!   PIT_RTCREP_ANN5                                                                 !
!   PIT_RTCREP_ANN6                                                                 !
!===================================================================================!

SUBROUTINE PIT_TCREPINC_CREATE_2004

    INTEGER :: pers_j,i,j,NR,NC
    REAL(KIND=KIND_REAL),DIMENSION(6,2)             :: FRAC_REP                                 !Intermediate value of fractions
    REAL(KIND=KIND_REAL),DIMENSION(6)               :: QUANT_REP                                !Amount of replacement income of a certain kind
    INTEGER                                         :: COUNT_CREDRE1,COUNT_CREDRE2
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN                           !Amount of exemptions used to compute reduction in tax credit
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN_SP1                       !Amount of exemptions used to compute reduction in tax credit for spouse 1
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN_SP2                       !Amount of exemptions used to compute reduction in tax credit for spouse 2
    REAL(KIND=KIND_REAL)                            :: PIT_SUMCRED_ANN                          !Total amount of credits subtracted computed for replacement income
    REAL(KIND=KIND_REAL)                            :: PIT_SUMCRED_ANN_SP1                      !Total amount of credits subtracted computed for replacement income for spouse 1
    REAL(KIND=KIND_REAL)                            :: PIT_SUMCRED_ANN_SP2                      !Total amount of credits subtracted computed for replacement income for spouse 2
    REAL(KIND=KIND_REAL)                            :: COMPAMOUNT,COMPAMOUNT_SP1,COMPAMOUNT_SP2 !Amounts that have to be compensated

    NR=SIZE(PIT_TAXFREEB_ANN,1)
    NC=SIZE(PIT_TAXFREEB_ANN,2)
    
    !PIT_TNTI_ANN: Total net taxable income of the tax unit
    DO pers_j=1,n_rec
        PIT_TNTI_ANN(pers_j) = PIT_NTI_ANN_SP1(pers_j) + PIT_NTI_ANN_SP2(pers_j)
    END DO

    !Set maximal basic tax credit for replacement income
    !PIT_BTCREP_ANN1: Basic tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_BTCREP_ANN2: Basic tax credit granted for early retirement benefits of old type
    !PIT_BTCREP_ANN3: Basic tax credit granted for normal unemployment benefits
    !PIT_BTCREP_ANN4: Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_BTCREP_ANN5: Basic tax credit granted for sickness and disability benefits
    !PIT_BTCREP_ANN6: Basic tax credit granted for other replacement income
    DO pers_j=1,n_rec
        !Different basic credits apply for couples and singles
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1:2) = PIT_MAXREPL_ANN(1,2)
            PIT_BTCREP_ANN2(pers_j,1:2) = PIT_MAXREPL_ANN(2,2)
            PIT_BTCREP_ANN3(pers_j,1:2) = PIT_MAXREPL_ANN(3,2)
            PIT_BTCREP_ANN4(pers_j,1:2) = PIT_MAXREPL_ANN(4,2)
            PIT_BTCREP_ANN5(pers_j,1:2) = PIT_MAXREPL_ANN(5,2)
            PIT_BTCREP_ANN6(pers_j,1:2) = PIT_MAXREPL_ANN(6,2)
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1:2) = PIT_MAXREPL_ANN(1,1)
            PIT_BTCREP_ANN2(pers_j,1:2) = PIT_MAXREPL_ANN(2,1)
            PIT_BTCREP_ANN3(pers_j,1:2) = PIT_MAXREPL_ANN(3,1)
            PIT_BTCREP_ANN4(pers_j,1:2) = PIT_MAXREPL_ANN(4,1)
            PIT_BTCREP_ANN5(pers_j,1:2) = PIT_MAXREPL_ANN(5,1)
            PIT_BTCREP_ANN6(pers_j,1:2) = PIT_MAXREPL_ANN(6,1)
        END IF
    END DO

    !PIT_FRACREP_ANN1: Fraction applied on pension benefits and early retirement benefits of new type
    !PIT_FRACREP_ANN2: Fraction applied on early retirement benefits of old type
    !PIT_FRACREP_ANN3: Fraction applied on normal unemployment benefits
    !PIT_FRACREP_ANN4: Fraction applied on unemployment benefits of older unemployed (age of 58 or above)
    !PIT_FRACREP_ANN5: Fraction applied on sickness and disability benefits
    !PIT_FRACREP_ANN6: Fraction applied on other replacement income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            FRAC_REP(1:6,1:2) = 0.D0
            !Compute fractions
            !if applied per spouse, for spouse 1
            IF (INT(100.D0*PIT_NTI_ANN_SP1(pers_j)).NE.0) THEN
                FRAC_REP(1,1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(2,1) = (PIT_NTIEAROL_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(5,1) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j))&
                                &/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(6,1) = 0
            END IF
            !if applied per spouse, for spouse 2
            IF (INT(100.D0*PIT_NTI_ANN_SP2(pers_j)).NE.0) THEN
                FRAC_REP(1,2) = (PIT_NTIPENS_ANN_SP2(pers_j)  + PIT_NTIEARNE_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(2,2) = (PIT_NTIEAROL_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(5,2) = (PIT_NTIDISAB_ANN_SP2(pers_j) + PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP2(pers_j))&
                                &/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(6,2) = 0
            END IF
            !if applied per couple, put value in column 1
            IF (INT(100.D0*PIT_TNTI_ANN(pers_j)).NE.0) THEN
                FRAC_REP(3,1) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
                FRAC_REP(4,1) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))/PIT_TNTI_ANN(pers_j)
            END IF
            !Correct fractions if necessary
            DO i=1,6
                DO j=1,2
                    IF (FRAC_REP(i,j).LT.0.D0) THEN
                        FRAC_REP(i,j) = 0.D0
                    END IF
                    IF (FRAC_REP(i,j).GT.1.D0) THEN
                        FRAC_REP(i,j) = 1.D0
                    END IF
                END DO
            END DO
            !Store fractions at individual level
            PIT_FRACREP_ANN1(pers_j,1) = FRAC_REP(1,1)
            PIT_FRACREP_ANN2(pers_j,1) = FRAC_REP(2,1)
            PIT_FRACREP_ANN3(pers_j,1) = FRAC_REP(3,1)
            PIT_FRACREP_ANN4(pers_j,1) = FRAC_REP(4,1)
            PIT_FRACREP_ANN5(pers_j,1) = FRAC_REP(5,1)
            PIT_FRACREP_ANN6(pers_j,1) = FRAC_REP(6,1)
            PIT_FRACREP_ANN1(pers_j,2) = FRAC_REP(1,2)
            PIT_FRACREP_ANN2(pers_j,2) = FRAC_REP(2,2)
            PIT_FRACREP_ANN3(pers_j,2) = FRAC_REP(3,2)
            PIT_FRACREP_ANN4(pers_j,2) = FRAC_REP(4,2)
            PIT_FRACREP_ANN5(pers_j,2) = FRAC_REP(5,2)
            PIT_FRACREP_ANN6(pers_j,2) = FRAC_REP(6,2)
        END IF
    END DO

    !Continue computation of basic tax credits
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_BTCREP_ANN1(pers_j,1) = PIT_BTCREP_ANN1(pers_j,1)*PIT_FRACREP_ANN1(pers_j,1)
            PIT_BTCREP_ANN2(pers_j,1) = PIT_BTCREP_ANN2(pers_j,1)*PIT_FRACREP_ANN2(pers_j,1)
            PIT_BTCREP_ANN3(pers_j,1) = PIT_BTCREP_ANN3(pers_j,1)*PIT_FRACREP_ANN3(pers_j,1)
            PIT_BTCREP_ANN4(pers_j,1) = PIT_BTCREP_ANN4(pers_j,1)*PIT_FRACREP_ANN4(pers_j,1)
            PIT_BTCREP_ANN5(pers_j,1) = PIT_BTCREP_ANN5(pers_j,1)*PIT_FRACREP_ANN5(pers_j,1)
            PIT_BTCREP_ANN6(pers_j,1) = PIT_BTCREP_ANN6(pers_j,1)*PIT_FRACREP_ANN6(pers_j,1)
            PIT_BTCREP_ANN1(pers_j,2) = PIT_BTCREP_ANN1(pers_j,2)*PIT_FRACREP_ANN1(pers_j,2)
            PIT_BTCREP_ANN2(pers_j,2) = PIT_BTCREP_ANN2(pers_j,2)*PIT_FRACREP_ANN2(pers_j,2)
            PIT_BTCREP_ANN3(pers_j,2) = PIT_BTCREP_ANN3(pers_j,2)*PIT_FRACREP_ANN3(pers_j,2)
            PIT_BTCREP_ANN4(pers_j,2) = PIT_BTCREP_ANN4(pers_j,2)*PIT_FRACREP_ANN4(pers_j,2)
            PIT_BTCREP_ANN5(pers_j,2) = PIT_BTCREP_ANN5(pers_j,2)*PIT_FRACREP_ANN5(pers_j,2)
            PIT_BTCREP_ANN6(pers_j,2) = PIT_BTCREP_ANN6(pers_j,2)*PIT_FRACREP_ANN6(pers_j,2)
        END IF
    END DO
    
    !PIT_RTCREP_ANN1: Reduced tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_RTCREP_ANN2: Reduced tax credit granted for early retirement benefits of old type
    !PIT_RTCREP_ANN3: Reduced tax credit granted for normal unemployment benefits
    !PIT_RTCREP_ANN4: Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_RTCREP_ANN5: Reduced tax credit granted for sickness and disability benefits
    !PIT_RTCREP_ANN6: Reduced tax credit granted for other replacement income
    DO pers_j=1,n_rec
        COMPAMOUNT = 0.D0
        COMPAMOUNT_SP1 = 0.D0
        COMPAMOUNT_SP2 = 0.D0 
        
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN

            !Compute the basic exemption applied on the income of the first spouse for couple credits. Is a choice. Probably the real system is more refined
            IF (InFODSZ(pers_j).EQ.0) THEN
                PIT_EXEMPT_ANN = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j)) 
                COMPAMOUNT = PIT_TAXFREEB_ANN(NR,NC)
            ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                PIT_EXEMPT_ANN = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT = PIT_TAXFREHA_ANN(NR,NC)
            END IF
            !Compute the basic exemption applied on the income of the first spouse for single credits
            IF (InFODSZ(pers_j).EQ.0) THEN
                PIT_EXEMPT_ANN_SP1 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT_SP1 = PIT_TAXFREEB_ANN(NR,NC)
            ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                PIT_EXEMPT_ANN_SP1 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT_SP1 = PIT_TAXFREHA_ANN(NR,NC)
            END IF
            !Compute the basic exemption applied on the income of the second spouse for single credits
            IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
                IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.0) THEN
                    PIT_EXEMPT_ANN_SP2 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                    &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))
                    COMPAMOUNT_SP2 = PIT_TAXFREEB_ANN(NR,NC)
                ELSE IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.1) THEN
                    PIT_EXEMPT_ANN_SP2 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                    &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))
                    COMPAMOUNT_SP2 = PIT_TAXFREHA_ANN(NR,NC)
                END IF
            END IF
            
            !Compute tax credit including the possible reduction for credits applied per individual: spouse 1
            PIT_RTCREP_ANN1(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN1(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN1(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN1(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN1(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN2(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN2(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN2(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN2(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN2(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN5(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN5(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN5(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN5(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN5(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN6(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN6(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN6(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN6(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN6(pers_j,1) = 0.D0
            END IF

            !Compute tax credit including the possible reduction for credits applied per individual: spouse 2
            PIT_RTCREP_ANN1(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN1(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN1(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN1(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN1(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN2(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN2(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN2(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN2(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN2(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN5(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN5(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN5(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN5(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN5(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN6(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN6(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN6(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN6(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN6(pers_j,2) = 0.D0
            END IF

            !Compute tax credit including the possible reduction for credits applied per couple
            PIT_RTCREP_ANN3(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN3(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                          &SIZE(PIT_RUNEMPL_ANN,1),PIT_RUNEMPL_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN3(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN3(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN3(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN4(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN4(pers_j,1),PIT_TNTI_ANN(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN4(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN4(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN4(pers_j,1) = 0.D0
            END IF

        END IF
    END DO

    !Limit tax credits in function of the personal income tax fraction
    !A tax credit can not exceed the fraction of personal income taxes generated by the income source
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !for credits applied per spouse: spouse 1
            IF (100.D0*PIT_PITFAMC_ANN_SP1(pers_j).GT.0.D0) THEN
                IF (PIT_RTCREP_ANN1(pers_j,1).GT.(PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN1(pers_j,1) = (PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN2(pers_j,1).GT.(PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN2(pers_j,1) = (PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN5(pers_j,1).GT.(PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN5(pers_j,1) = (PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN6(pers_j,1).GT.(PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN6(pers_j,1) = (PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
            END IF
            
            !for credits applied per spouse: spouse 2
            IF (100.D0*PIT_PITFAMC_ANN_SP2(pers_j).GT.0.D0) THEN
                IF (PIT_RTCREP_ANN1(pers_j,2).GT.(PIT_FRACREP_ANN1(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN1(pers_j,2) = (PIT_FRACREP_ANN1(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN2(pers_j,2).GT.(PIT_FRACREP_ANN2(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN2(pers_j,2) = (PIT_FRACREP_ANN2(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN5(pers_j,2).GT.(PIT_FRACREP_ANN5(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN5(pers_j,2) = (PIT_FRACREP_ANN5(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN6(pers_j,2).GT.(PIT_FRACREP_ANN6(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN6(pers_j,2) = (PIT_FRACREP_ANN6(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
            END IF
            
            !for credits applied per couple
            IF (100.D0*PIT_PITFAMC_ANN(pers_j).GT.0.D0) THEN
                IF (PIT_RTCREP_ANN3(pers_j,1).GT.(PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                    PIT_RTCREP_ANN3(pers_j,1) = (PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
                END IF
                IF (PIT_RTCREP_ANN4(pers_j,1).GT.(PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN(pers_j))) THEN
                    PIT_RTCREP_ANN4(pers_j,1) = (PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN(pers_j))
                END IF
            END IF
        END IF
    END DO

    !PIT_PITREP_ANN: Personal income taxes at the tax unit level net of tax credits for family composition and credits for replacement income
    COUNT_CREDRE1 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_SUMCRED_ANN_SP1 = (PIT_RTCREP_ANN1(pers_j,1) + PIT_RTCREP_ANN2(pers_j,1) + PIT_RTCREP_ANN5(pers_j,1) + &
                                  &PIT_RTCREP_ANN6(pers_j,1))
            PIT_SUMCRED_ANN_SP2 = (PIT_RTCREP_ANN1(pers_j,2) + PIT_RTCREP_ANN2(pers_j,2) + PIT_RTCREP_ANN5(pers_j,2) + &
                                  &PIT_RTCREP_ANN6(pers_j,2))
            PIT_SUMCRED_ANN =     (PIT_RTCREP_ANN3(pers_j,1) + PIT_RTCREP_ANN4(pers_j,1))

            !Only subtract credits in case of a postive amount of taxes
            IF (PIT_PITFAMC_ANN(pers_j).GT.0.D0) THEN
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) - PIT_SUMCRED_ANN - PIT_SUMCRED_ANN_SP1 - PIT_SUMCRED_ANN_SP2
                IF (PIT_PITREP_ANN(pers_j).LT.0.D0) THEN
                    PIT_PITREP_ANN(pers_j) = 0.D0
                    COUNT_CREDRE1 = COUNT_CREDRE1 + 1 
                END IF
            ELSE
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) 
            END IF
        END IF
    END DO

    !Check whether replacement income is of a unique type: apply the reduction on the tax unit level
    COUNT_CREDRE2 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !Only limit taxes in case of a positive amount
            IF (PIT_PITREP_ANN(pers_j).GT.0.D0) THEN
                QUANT_REP(1:6) = 0.D0
                QUANT_REP(1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j) + &
                               &PIT_NTIPENS_ANN_SP2(pers_j) + PIT_NTIEARNE_ANN_SP2(pers_j))
                QUANT_REP(2) = (PIT_NTIEAROL_ANN_SP1(pers_j) + PIT_NTIEAROL_ANN_SP2(pers_j))
                QUANT_REP(3) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))
                QUANT_REP(4) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))
                QUANT_REP(5) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTIDISAB_ANN_SP2(pers_j) &
                               &+ PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j) +&
                               & PIT_NTINDOCC_ANN_SP2(pers_j))
                QUANT_REP(6) = 0
                !check whether replacement income is of a unique type
                DO i=1,6
                    IF (INT(QUANT_REP(i)).EQ.INT(PIT_TNTI_ANN(pers_j)).AND.&
                        &INT(100.D0*QUANT_REP(i)).NE.0.AND.QUANT_REP(i).LT.PIT_UNIQREPL_ANN(i).AND.&
                        &PIT_PITREP_ANN(pers_j).GT.0.D0) THEN
                            PIT_PITREP_ANN(pers_j) = 0.D0
                            COUNT_CREDRE2 = COUNT_CREDRE2 + 1 
                            EXIT
                    END IF                    
                END DO
            END IF
        END IF
    END DO

    !TEST
    !Write results of construction of tax credits for replacement income
    Write(15,'('' '')')
    Write(15,'(''Counts on credits for replacement income'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times credit is larger than PIT of the TU            '', i10)') COUNT_CREDRE1
    Write(15,'(''Number of times income is of unique source and below ceiling   '', i10)') COUNT_CREDRE2
    
END SUBROUTINE PIT_TCREPINC_CREATE_2004

!===================================================================================!
!With subroutine PIT_TCREPINC_CREATE_2010 we compute the tax credits that           !
!compensates for replacement income                                                 !
!Variables created                                                                  !
!   PIT_PITREP_ANN                                                                  !
!   PIT_FRACREP_ANN1                                                                !
!   PIT_FRACREP_ANN2                                                                !
!   PIT_FRACREP_ANN3                                                                !
!   PIT_FRACREP_ANN4                                                                !
!   PIT_FRACREP_ANN5                                                                !
!   PIT_FRACREP_ANN6                                                                !
!   PIT_BTCREP_ANN1                                                                 !
!   PIT_BTCREP_ANN2                                                                 !
!   PIT_BTCREP_ANN3                                                                 !
!   PIT_BTCREP_ANN4                                                                 !
!   PIT_BTCREP_ANN5                                                                 !
!   PIT_BTCREP_ANN6                                                                 !
!   PIT_TNTI_ANN                                                                    !
!   PIT_RTCREP_ANN1                                                                 !
!   PIT_RTCREP_ANN2                                                                 !
!   PIT_RTCREP_ANN3                                                                 !
!   PIT_RTCREP_ANN4                                                                 !
!   PIT_RTCREP_ANN5                                                                 !
!   PIT_RTCREP_ANN6                                                                 !
!===================================================================================!

SUBROUTINE PIT_TCREPINC_CREATE_2010

    INTEGER :: pers_j,i,j,NR,NC
    REAL(KIND=KIND_REAL),DIMENSION(6,2)             :: FRAC_REP             !Intermediate value of fractions
    REAL(KIND=KIND_REAL),DIMENSION(6)               :: QUANT_REP            !Amount of replacement income of a certain kind
    INTEGER                                         :: COUNT_CREDRE1,COUNT_CREDRE2
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN       !Amount of exemptions used to compute reduction in tax credit
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN_SP1   !Amount of exemptions used to compute reduction in tax credit for spouse 1
    REAL(KIND=KIND_REAL)                            :: PIT_EXEMPT_ANN_SP2   !Amount of exemptions used to compute reduction in tax credit for spouse 2
    REAL(KIND=KIND_REAL)                            :: PIT_SUMCRED_ANN_SP1  !Total amount of credits subtracted computed for replacement income for spouse 1
    REAL(KIND=KIND_REAL)                            :: PIT_SUMCRED_ANN_SP2  !Total amount of credits subtracted computed for replacement income for spouse 2
    REAL(KIND=KIND_REAL)                            :: COMPAMOUNT,COMPAMOUNT_SP1,COMPAMOUNT_SP2 !Amounts that have to be compensated

    NR=SIZE(PIT_TAXFREEB_ANN,1)
    NC=SIZE(PIT_TAXFREEB_ANN,2)
    
    !PIT_TNTI_ANN: Total net taxable income of the tax unit
    DO pers_j=1,n_rec
        PIT_TNTI_ANN(pers_j) = PIT_NTI_ANN_SP1(pers_j) + PIT_NTI_ANN_SP2(pers_j)
    END DO

    !Set maximal basic tax credit for replacement income
    !PIT_BTCREP_ANN1: Basic tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_BTCREP_ANN2: Basic tax credit granted for early retirement benefits of old type
    !PIT_BTCREP_ANN3: Basic tax credit granted for normal unemployment benefits
    !PIT_BTCREP_ANN4: Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_BTCREP_ANN5: Basic tax credit granted for sickness and disability benefits
    !PIT_BTCREP_ANN6: Basic tax credit granted for other replacement income
    DO pers_j=1,n_rec
        !Different basic credits apply for couples and singles
        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1:2) = PIT_MAXREPL_ANN(1,2)
            PIT_BTCREP_ANN2(pers_j,1:2) = PIT_MAXREPL_ANN(2,2)
            PIT_BTCREP_ANN3(pers_j,1:2) = PIT_MAXREPL_ANN(3,2)
            PIT_BTCREP_ANN4(pers_j,1:2) = PIT_MAXREPL_ANN(4,2)
            PIT_BTCREP_ANN5(pers_j,1:2) = PIT_MAXREPL_ANN(5,2)
            PIT_BTCREP_ANN6(pers_j,1:2) = PIT_MAXREPL_ANN(6,2)
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
            PIT_BTCREP_ANN1(pers_j,1:2) = PIT_MAXREPL_ANN(1,1)
            PIT_BTCREP_ANN2(pers_j,1:2) = PIT_MAXREPL_ANN(2,1)
            PIT_BTCREP_ANN3(pers_j,1:2) = PIT_MAXREPL_ANN(3,1)
            PIT_BTCREP_ANN4(pers_j,1:2) = PIT_MAXREPL_ANN(4,1)
            PIT_BTCREP_ANN5(pers_j,1:2) = PIT_MAXREPL_ANN(5,1)
            PIT_BTCREP_ANN6(pers_j,1:2) = PIT_MAXREPL_ANN(6,1)
        END IF
    END DO

    !PIT_FRACREP_ANN1: Fraction applied on pension benefits and early retirement benefits of new type
    !PIT_FRACREP_ANN2: Fraction applied on early retirement benefits of old type
    !PIT_FRACREP_ANN3: Fraction applied on normal unemployment benefits
    !PIT_FRACREP_ANN4: Fraction applied on unemployment benefits of older unemployed (age of 58 or above)
    !PIT_FRACREP_ANN5: Fraction applied on sickness and disability benefits
    !PIT_FRACREP_ANN6: Fraction applied on other replacement income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            FRAC_REP(1:6,1:2) = 0.D0
            !Compute fractions
            !if applied per spouse, for spouse 1
            IF (INT(100.D0*PIT_NTI_ANN_SP1(pers_j)).NE.0) THEN
                FRAC_REP(1,1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(2,1) = (PIT_NTIEAROL_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(3,1) = (PIT_NTIOTHUN_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(4,1) = (PIT_NTIOLDUN_ANN_SP1(pers_j))/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(5,1) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j))&
                                &/PIT_NTI_ANN_SP1(pers_j)
                FRAC_REP(6,1) = 0
            END IF
            !if applied per spouse, for spouse 2
            IF (INT(100.D0*PIT_NTI_ANN_SP2(pers_j)).NE.0) THEN
                FRAC_REP(1,2) = (PIT_NTIPENS_ANN_SP2(pers_j)  + PIT_NTIEARNE_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(2,2) = (PIT_NTIEAROL_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(3,2) = (PIT_NTIOTHUN_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(4,2) = (PIT_NTIOLDUN_ANN_SP2(pers_j))/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(5,2) = (PIT_NTIDISAB_ANN_SP2(pers_j) + PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP2(pers_j))&
                                &/PIT_NTI_ANN_SP2(pers_j)
                FRAC_REP(6,2) = 0
            END IF
            !Correct fractions if necessary
            DO i=1,6
                DO j=1,2
                    IF (FRAC_REP(i,j).LT.0.D0) THEN
                        FRAC_REP(i,j) = 0.D0
                    END IF
                    IF (FRAC_REP(i,j).GT.1.D0) THEN
                        FRAC_REP(i,j) = 1.D0
                    END IF
                END DO
            END DO
            !Store fractions at individual level
            PIT_FRACREP_ANN1(pers_j,1) = FRAC_REP(1,1)
            PIT_FRACREP_ANN2(pers_j,1) = FRAC_REP(2,1)
            PIT_FRACREP_ANN3(pers_j,1) = FRAC_REP(3,1)
            PIT_FRACREP_ANN4(pers_j,1) = FRAC_REP(4,1)
            PIT_FRACREP_ANN5(pers_j,1) = FRAC_REP(5,1)
            PIT_FRACREP_ANN6(pers_j,1) = FRAC_REP(6,1)
            PIT_FRACREP_ANN1(pers_j,2) = FRAC_REP(1,2)
            PIT_FRACREP_ANN2(pers_j,2) = FRAC_REP(2,2)
            PIT_FRACREP_ANN3(pers_j,2) = FRAC_REP(3,2)
            PIT_FRACREP_ANN4(pers_j,2) = FRAC_REP(4,2)
            PIT_FRACREP_ANN5(pers_j,2) = FRAC_REP(5,2)
            PIT_FRACREP_ANN6(pers_j,2) = FRAC_REP(6,2)
        END IF
    END DO

    !Continue computation of basic tax credits
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_BTCREP_ANN1(pers_j,1) = PIT_BTCREP_ANN1(pers_j,1)*PIT_FRACREP_ANN1(pers_j,1)
            PIT_BTCREP_ANN2(pers_j,1) = PIT_BTCREP_ANN2(pers_j,1)*PIT_FRACREP_ANN2(pers_j,1)
            PIT_BTCREP_ANN3(pers_j,1) = PIT_BTCREP_ANN3(pers_j,1)*PIT_FRACREP_ANN3(pers_j,1)
            PIT_BTCREP_ANN4(pers_j,1) = PIT_BTCREP_ANN4(pers_j,1)*PIT_FRACREP_ANN4(pers_j,1)
            PIT_BTCREP_ANN5(pers_j,1) = PIT_BTCREP_ANN5(pers_j,1)*PIT_FRACREP_ANN5(pers_j,1)
            PIT_BTCREP_ANN6(pers_j,1) = PIT_BTCREP_ANN6(pers_j,1)*PIT_FRACREP_ANN6(pers_j,1)
            PIT_BTCREP_ANN1(pers_j,2) = PIT_BTCREP_ANN1(pers_j,2)*PIT_FRACREP_ANN1(pers_j,2)
            PIT_BTCREP_ANN2(pers_j,2) = PIT_BTCREP_ANN2(pers_j,2)*PIT_FRACREP_ANN2(pers_j,2)
            PIT_BTCREP_ANN3(pers_j,2) = PIT_BTCREP_ANN3(pers_j,2)*PIT_FRACREP_ANN3(pers_j,2)
            PIT_BTCREP_ANN4(pers_j,2) = PIT_BTCREP_ANN4(pers_j,2)*PIT_FRACREP_ANN4(pers_j,2)
            PIT_BTCREP_ANN5(pers_j,2) = PIT_BTCREP_ANN5(pers_j,2)*PIT_FRACREP_ANN5(pers_j,2)
            PIT_BTCREP_ANN6(pers_j,2) = PIT_BTCREP_ANN6(pers_j,2)*PIT_FRACREP_ANN6(pers_j,2)
        END IF
    END DO
    
    !PIT_RTCREP_ANN1: Reduced tax credit granted for pension benefits and early retirement benefits of new type
    !PIT_RTCREP_ANN2: Reduced tax credit granted for early retirement benefits of old type
    !PIT_RTCREP_ANN3: Reduced tax credit granted for normal unemployment benefits
    !PIT_RTCREP_ANN4: Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above)
    !PIT_RTCREP_ANN5: Reduced tax credit granted for sickness and disability benefits
    !PIT_RTCREP_ANN6: Reduced tax credit granted for other replacement income
    DO pers_j=1,n_rec
        COMPAMOUNT = 0.D0
        COMPAMOUNT_SP1 = 0.D0
        COMPAMOUNT_SP2 = 0.D0 

        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
        
            !Compute the basic exemption applied on the income of the first spouse for couple credits. Is a choice. Probably the real system is more refined
            IF (InFODSZ(pers_j).EQ.0) THEN
                PIT_EXEMPT_ANN = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j)) 
                COMPAMOUNT = PIT_TAXFREEB_ANN(NR,NC)
            ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                PIT_EXEMPT_ANN = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT = PIT_TAXFREHA_ANN(NR,NC)
            END IF
            !Compute the basic exemption applied on the income of the first spouse for single credits
            IF (InFODSZ(pers_j).EQ.0) THEN
                PIT_EXEMPT_ANN_SP1 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT_SP1 = PIT_TAXFREEB_ANN(NR,NC)
            ELSE IF (InFODSZ(pers_j).EQ.1) THEN
                PIT_EXEMPT_ANN_SP1 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP1(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))    
                COMPAMOUNT_SP1 = PIT_TAXFREHA_ANN(NR,NC)
            END IF
            !Compute the basic exemption applied on the income of the second spouse for single credits
            IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
                IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.0) THEN
                    PIT_EXEMPT_ANN_SP2 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREEB_ANN,1),&
                    &PIT_TAXFREEB_ANN,GECODEERD_INSZ_INPUT(pers_j))
                    COMPAMOUNT_SP2 = PIT_TAXFREEB_ANN(NR,NC)
                ELSE IF (InFODSZ(PIT_PARTID_ANN(pers_j)).EQ.1) THEN
                    PIT_EXEMPT_ANN_SP2 = PIT_BAMOUNT_3COLUMNS(PIT_NTI_ANN_SP2(pers_j),SIZE(PIT_TAXFREHA_ANN,1),&
                    &PIT_TAXFREHA_ANN,GECODEERD_INSZ_INPUT(pers_j))
                    COMPAMOUNT_SP2 = PIT_TAXFREHA_ANN(NR,NC)
                END IF
            END IF
            
            !Compute tax credit including the possible reduction for credits applied per individual: spouse 1
            PIT_RTCREP_ANN1(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN1(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN1(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN1(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN1(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN2(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN2(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN2(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN2(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN2(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN3(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN3(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RUNEMPL_ANN,1),PIT_RUNEMPL_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN3(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN3(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN3(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN4(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN4(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN4(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN4(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN4(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN5(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN5(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN5(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN5(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN5(pers_j,1) = 0.D0
            END IF
            PIT_RTCREP_ANN6(pers_j,1) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN6(pers_j,1),PIT_NTI_ANN_SP1(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN6(pers_j,1)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP1-COMPAMOUNT_SP1))
            IF (100.D0*PIT_RTCREP_ANN6(pers_j,1).LT.0.D0) THEN
                PIT_RTCREP_ANN6(pers_j,1) = 0.D0
            END IF

            !Compute tax credit including the possible reduction for credits applied per individual: spouse 2
            PIT_RTCREP_ANN1(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN1(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN1(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN1(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN1(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN2(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN2(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN2(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN2(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN2(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN3(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN3(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RUNEMPL_ANN,1),PIT_RUNEMPL_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN3(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN3(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN3(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN4(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN4(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN4(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN4(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN4(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN5(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN5(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN5(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN-COMPAMOUNT))
            IF (100.D0*PIT_RTCREP_ANN5(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN5(pers_j,2) = 0.D0
            END IF
            PIT_RTCREP_ANN6(pers_j,2) =   PIT_BRACRATE_4COLUMNS_RATES(PIT_BTCREP_ANN6(pers_j,2),PIT_NTI_ANN_SP2(pers_j),&
                                          &SIZE(PIT_RREDUCE_ANN,1),PIT_RREDUCE_ANN,pers_j)&
                                          &-(PIT_FRACREP_ANN6(pers_j,2)*(PIT_CREDREDU_ANN/100.D0)*&
                                          &(PIT_EXEMPT_ANN_SP2-COMPAMOUNT_SP2))
            IF (100.D0*PIT_RTCREP_ANN6(pers_j,2).LT.0.D0) THEN
                PIT_RTCREP_ANN6(pers_j,2) = 0.D0
            END IF

        END IF
    END DO

    !Limit tax credits in function of the personal income tax fraction
    !A tax credit can not exceed the fraction of personal income taxes generated by the income source
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !for credits applied per spouse: spouse 1
            IF (100.D0*PIT_PITFAMC_ANN_SP1(pers_j).GT.0.D0) THEN
                IF (PIT_RTCREP_ANN1(pers_j,1).GT.(PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN1(pers_j,1) = (PIT_FRACREP_ANN1(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN2(pers_j,1).GT.(PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN2(pers_j,1) = (PIT_FRACREP_ANN2(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN3(pers_j,1).GT.(PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN3(pers_j,1) = (PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN4(pers_j,1).GT.(PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN4(pers_j,1) = (PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN5(pers_j,1).GT.(PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN5(pers_j,1) = (PIT_FRACREP_ANN5(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
                IF (PIT_RTCREP_ANN6(pers_j,1).GT.(PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))) THEN
                    PIT_RTCREP_ANN6(pers_j,1) = (PIT_FRACREP_ANN6(pers_j,1)*PIT_PITFAMC_ANN_SP1(pers_j))
                END IF
            END IF
            
            !for credits applied per spouse: spouse 2
            IF (100.D0*PIT_PITFAMC_ANN_SP2(pers_j).GT.0.D0) THEN
                IF (PIT_RTCREP_ANN1(pers_j,2).GT.(PIT_FRACREP_ANN1(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN1(pers_j,2) = (PIT_FRACREP_ANN1(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN2(pers_j,2).GT.(PIT_FRACREP_ANN2(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN2(pers_j,2) = (PIT_FRACREP_ANN2(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN3(pers_j,2).GT.(PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN3(pers_j,2) = (PIT_FRACREP_ANN3(pers_j,1)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN4(pers_j,2).GT.(PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN4(pers_j,2) = (PIT_FRACREP_ANN4(pers_j,1)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN5(pers_j,2).GT.(PIT_FRACREP_ANN5(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN5(pers_j,2) = (PIT_FRACREP_ANN5(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
                IF (PIT_RTCREP_ANN6(pers_j,2).GT.(PIT_FRACREP_ANN6(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))) THEN
                    PIT_RTCREP_ANN6(pers_j,2) = (PIT_FRACREP_ANN6(pers_j,2)*PIT_PITFAMC_ANN_SP2(pers_j))
                END IF
            END IF
            
        END IF
    END DO

    !PIT_PITREP_ANN: Personal income taxes at the tax unit level net of tax credits for family composition and credits for replacement income
    COUNT_CREDRE1 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            PIT_SUMCRED_ANN_SP1 = ( PIT_RTCREP_ANN1(pers_j,1) + PIT_RTCREP_ANN2(pers_j,1) + &
                                    PIT_RTCREP_ANN3(pers_j,1) + PIT_RTCREP_ANN4(pers_j,1) + &
                                    PIT_RTCREP_ANN5(pers_j,1) + PIT_RTCREP_ANN6(pers_j,1))
            PIT_SUMCRED_ANN_SP2 = ( PIT_RTCREP_ANN1(pers_j,2) + PIT_RTCREP_ANN2(pers_j,2) + &
                                    PIT_RTCREP_ANN3(pers_j,2) + PIT_RTCREP_ANN4(pers_j,2) + &
                                    PIT_RTCREP_ANN5(pers_j,2) + PIT_RTCREP_ANN6(pers_j,2))
            !Only subtract credits in case of a postive amount of taxes
            IF (PIT_PITFAMC_ANN(pers_j).GT.0.D0) THEN
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) - PIT_SUMCRED_ANN_SP1 - PIT_SUMCRED_ANN_SP2
                IF (PIT_PITREP_ANN(pers_j).LT.0.D0) THEN
                    PIT_PITREP_ANN(pers_j) = 0.D0
                    COUNT_CREDRE1 = COUNT_CREDRE1 + 1 
                END IF
            ELSE
                PIT_PITREP_ANN(pers_j) = PIT_PITFAMC_ANN(pers_j) 
            END IF
        END IF
    END DO

    !Check whether replacement income is of a unique type: apply the reduction on the tax unit level
    COUNT_CREDRE2 = 0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !Only limit taxes in case of a positive amount
            IF (PIT_PITREP_ANN(pers_j).GT.0.D0) THEN
                QUANT_REP(1:6) = 0.D0
                QUANT_REP(1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j) + &
                               &PIT_NTIPENS_ANN_SP2(pers_j) + PIT_NTIEARNE_ANN_SP2(pers_j))
                QUANT_REP(2) = (PIT_NTIEAROL_ANN_SP1(pers_j) + PIT_NTIEAROL_ANN_SP2(pers_j))
                QUANT_REP(3) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))
                QUANT_REP(4) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))
                QUANT_REP(5) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + PIT_NTIDISAB_ANN_SP2(pers_j) &
                               &+ PIT_NTISICK_ANN_SP2(pers_j) + PIT_NTINDOCC_ANN_SP1(pers_j) +&
                               & PIT_NTINDOCC_ANN_SP2(pers_j))
                QUANT_REP(6) = 0
                !check whether replacement income is of a unique type
                DO i=1,6
                    IF (INT(QUANT_REP(i)).EQ.INT(PIT_TNTI_ANN(pers_j)).AND.&
                        &INT(100.D0*QUANT_REP(i)).NE.0.AND.QUANT_REP(i).LT.PIT_UNIQREPL_ANN(i).AND.&
                        &PIT_PITREP_ANN(pers_j).GT.0.D0) THEN
                            PIT_PITREP_ANN(pers_j) = 0.D0
                            COUNT_CREDRE2 = COUNT_CREDRE2 + 1 
                            EXIT
                    END IF                    
                END DO
            END IF
        END IF
    END DO

    !TEST
    !Write results of construction of tax credits for replacement income
    Write(15,'('' '')')
    Write(15,'(''Counts on credits for replacement income'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times credit is larger than PIT of the TU            '', i10)') COUNT_CREDRE1
    Write(15,'(''Number of times income is of unique source and below ceiling   '', i10)') COUNT_CREDRE2
    
END SUBROUTINE PIT_TCREPINC_CREATE_2010

!===================================================================================!
!With subroutine PIT_PITLIAB_CREATE we determine whether the tax unit is liable to  !
!pay income taxes yes or no                                                         !
!Variables created                                                                  !
!   PIT_TUNOLAB_ANN                                                                 !
!===================================================================================!

SUBROUTINE PIT_PITLIAB_CREATE

    INTEGER :: pers_j
    INTEGER :: COUNT_LIABC,COUNT_LIABS

    !PIT_TUNOLAB_ANN: Indicator whether the tax unit is liable to enter a tax form yes or no (no = 1, yes = 0)
    COUNT_LIABC = 0
    COUNT_LIABS = 0
    DO pers_j=1,n_rec
!        IF (PIT_COUPLE_ANN(pers_j).EQ.1) THEN
!            IF (INT(100.D0*PIT_NTOI_ANN_SP1(pers_j)).EQ.0.AND.INT(100.D0*PIT_NTOI_ANN_SP2(pers_j)).EQ.0) THEN
!                IF (PIT_TNTI_ANN(pers_j).LT.PIT_BECOUPLE_ANN) THEN
!                    PIT_TUNOLAB_ANN(pers_j) = 1
!                    COUNT_LIABC = COUNT_LIABC + 1
!                END IF
!            END IF                
!        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1) THEN
!            IF (INT(100.D0*PIT_NTOI_ANN_SP1(pers_j)).EQ.0) THEN
!                IF (PIT_TNTI_ANN(pers_j).LT.PIT_BESINGLE_ANN) THEN
!                    PIT_TUNOLAB_ANN(pers_j) = 1
!                    COUNT_LIABS = COUNT_LIABS + 1
!                END IF
!            END IF                
!        END IF
!Gegeven werd aangemaakt voor analyse 2001. Vermoeden lijkt gerechtvaardigd dat er voor gegevens 2008 geen
!onderscheid meer gemaakt wordt tussen belastingplichtigen en niet belastingplichtigen
        PIT_TUNOLAB_ANN(pers_j) = 0
    END DO        

    !TEST
    !Write results of tests on liability to pay income taxes
    Write(15,'('' '')')
    Write(15,'(''Counts on liability to pay income taxes'')')
    CALL DRAWLINE(15,136,'-')    
    Write(15,'(''Number of times a couple is not liable to pay income taxes   '', i10)') COUNT_LIABC 
    Write(15,'(''Number of times a single is not liable to pay income taxes   '', i10)') COUNT_LIABS

END SUBROUTINE PIT_PITLIAB_CREATE

!===================================================================================!
!With subroutine PIT_OTHPIT_CREATE we determine the final tax bill                  !
!Variables created                                                                  !
!   PIT_NETPITSC_ANN                                                                !
!   PIT_NETPITSE_ANN                                                                !
!   PIT_NETPIT_ANN                                                                  !
!   PIT_STATEPIT_ANN                                                                !
!   PIT_REGIOTAX_ANN                                                                !
!   PIT_CSAMOUNT_ANN                                                                !
!   PIT_SPECSSCO_ANN                                                                !    
!   PIT_FLEMJOBR_ANN                                                                !
!   PIT_FINTAX_ANN                                                                  !
!   PIT_REGIO                                                                  !
!===================================================================================!

SUBROUTINE PIT_OTHPIT_CREATE

    INTEGER                                         :: pers_j
    REAL(KIND=KIND_REAL)                            :: SELF     !Self employed income of the tax unit
    REAL(KIND=KIND_REAL)                            :: OTHER    !Other income sources of the tax unit
    REAL(KIND=KIND_REAL)                            :: BASEAM   !Basic amount to compute rebatable tax credit for low income
    REAL(KIND=KIND_REAL)                            :: COST     !Cost to be subtracted from the basic amount in computation of rebatable tax credit for low income
    REAL(KIND=KIND_REAL)                            :: NETAM    !Net amount to compute rebatable tax credit for low income
    REAL(KIND=KIND_REAL)                            :: CREDIT   !Rebatable tax credit for low income

    !PIT_REGIO: Indicator of region where the tax unit is living
    DO pers_j=1,n_rec
        IF      (NAREGNIS_COMMUNIT(pers_j).GE.11000.AND.NAREGNIS_COMMUNIT(pers_j).LE.13053) THEN 
            PIT_REGIO(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.21001.AND.NAREGNIS_COMMUNIT(pers_j).LE.21019) THEN 
            PIT_REGIO(pers_j) = 1
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.23002.AND.NAREGNIS_COMMUNIT(pers_j).LE.24137) THEN 
            PIT_REGIO(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.25005.AND.NAREGNIS_COMMUNIT(pers_j).LE.25124) THEN 
            PIT_REGIO(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.31003.AND.NAREGNIS_COMMUNIT(pers_j).LE.38025) THEN 
            PIT_REGIO(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.41002.AND.NAREGNIS_COMMUNIT(pers_j).LE.46025) THEN 
            PIT_REGIO(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.51004.AND.NAREGNIS_COMMUNIT(pers_j).LE.57095) THEN 
            PIT_REGIO(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.61003.AND.NAREGNIS_COMMUNIT(pers_j).LE.64076) THEN 
            PIT_REGIO(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.71002.AND.NAREGNIS_COMMUNIT(pers_j).LE.73109) THEN 
            PIT_REGIO(pers_j) = 2
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.81001.AND.NAREGNIS_COMMUNIT(pers_j).LE.85047) THEN 
            PIT_REGIO(pers_j) = 3
        ELSE IF (NAREGNIS_COMMUNIT(pers_j).GE.91005.AND.NAREGNIS_COMMUNIT(pers_j).LE.93090) THEN 
            PIT_REGIO(pers_j) = 3
        END IF
    END DO

    !PIT_TAXCRLOW_ANN_SP1: Tax credit for low labour income of spouse 1 (in case of self employed or civil servant income)
    !PIT_TAXCRLOW_ANN_SP2: Tax credit for low labour income of spouse 2 (in case of self employed or civil servant income)
    DO pers_j=1,n_rec
        !Execute computation at the level of the head of the tax unit
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN

            !Execute computations for spouse 1
            !Execute computation if spouse 1 is employee and tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.(ANY(PIT_PROFSTAT_ANN_SP1(pers_j,1).EQ.(/1,2/)).OR.&
            &ANY(PIT_PROFSTAT_ANN_SP1(pers_j,2).EQ.(/1,2/)))) THEN
                BASEAM = PIT_GTIPRIV_ANN_SP1(pers_j)+PIT_GTIPUB_ANN_SP1(pers_j)
                COST   = PIT_BRACRATE_3COLUMNS(BASEAM,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
                IF (COST.LT.0.D0) THEN
                    COST = 0.D0
                END IF
                NETAM  = BASEAM - COST
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP1(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP1(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCREWE_ANN,1),PIT_TAXCREWE_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP1(pers_j) = PIT_TAXCRLOW_ANN_SP1(pers_j) + CREDIT
            END IF
            !Execute computation if spouse 1 is civil servant with statutory service and tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.ANY(PIT_PROFSTAT_ANN_SP1(pers_j,2).EQ.(/3,4/))) THEN
                BASEAM = PIT_GTIPUB_ANN_SP1(pers_j)
                COST   = PIT_BRACRATE_3COLUMNS(BASEAM,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
                IF (COST.LT.0.D0) THEN
                    COST = 0.D0
                END IF
                NETAM  = BASEAM - COST
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP1(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP1(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCRECS_ANN,1),PIT_TAXCRECS_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP1(pers_j) = PIT_TAXCRLOW_ANN_SP1(pers_j) + CREDIT
            END IF
            !Execute computation if spouse 1 is self employed and tax unit is liable to pay income taxes
            !Remark: self employed after the age of retirement are assumed to be in principal activity
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.ANY(PIT_PROFSTAT_ANN_SP1(pers_j,3).EQ.(/1,3,4,6/))) THEN
                NETAM  = PIT_NTISELF_ANN_SP1(pers_j)
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP1(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP1(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCRESE_ANN,1),PIT_TAXCRESE_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP1(pers_j) = PIT_TAXCRLOW_ANN_SP1(pers_j) + CREDIT
            END IF

            !Execute computations for spouse 2
            !Execute computation if spouse 2 is employee and tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.(ANY(PIT_PROFSTAT_ANN_SP2(pers_j,1).EQ.(/1,2/)).OR.&
            &ANY(PIT_PROFSTAT_ANN_SP2(pers_j,2).EQ.(/1,2/)))) THEN
                BASEAM = PIT_GTIPRIV_ANN_SP2(pers_j)+PIT_GTIPUB_ANN_SP2(pers_j)
                COST   = PIT_BRACRATE_3COLUMNS(BASEAM,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
                IF (COST.LT.0.D0) THEN
                    COST = 0.D0
                END IF
                NETAM  = BASEAM - COST
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP2(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP2(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCREWE_ANN,1),PIT_TAXCREWE_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP2(pers_j) = PIT_TAXCRLOW_ANN_SP2(pers_j) + CREDIT
            END IF
            !Execute computation if spouse 2 is civil servant with statutory service and tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.ANY(PIT_PROFSTAT_ANN_SP2(pers_j,2).EQ.(/3,4/))) THEN
                BASEAM = PIT_GTIPUB_ANN_SP2(pers_j)
                COST   = PIT_BRACRATE_3COLUMNS(BASEAM,SIZE(PIT_RALUMPEM_ANN,1),PIT_RALUMPEM_ANN,pers_j)
                IF (COST.LT.0.D0) THEN
                    COST = 0.D0
                END IF
                NETAM  = BASEAM - COST
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP2(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP2(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCRECS_ANN,1),PIT_TAXCRECS_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP2(pers_j) = PIT_TAXCRLOW_ANN_SP2(pers_j) + CREDIT
            END IF
            !Execute computation if spouse 2 is self employed and tax unit is liable to pay income taxes
            !Remark: self employed after the age of retirement are assumed to be in principal activity
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.ANY(PIT_PROFSTAT_ANN_SP2(pers_j,3).EQ.(/1,3,4,6/))) THEN
                NETAM  = PIT_NTISELF_ANN_SP2(pers_j)
                CREDIT = 0.D0
                !limit the credit in proportion to its income part in net occupational income
                IF (PIT_NTOI_ANN_SP2(pers_j).NE.0.D0) THEN
                    CREDIT = (NETAM/PIT_NTOI_ANN_SP2(pers_j))*&
                    &PIT_MULTIPL_4COLUMNS_PLUS(NETAM,SIZE(PIT_TAXCRESE_ANN,1),PIT_TAXCRESE_ANN,pers_j)  
                END IF  
                PIT_TAXCRLOW_ANN_SP2(pers_j) = PIT_TAXCRLOW_ANN_SP2(pers_j) + CREDIT
            END IF
        END IF
    END DO

    !PIT_NETPITSC_ANN: Personal income taxes at the tax unit level net of all tax credits and obtained after application of a tax rate scheme on an aggregate income concept
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_NETPITSC_ANN(pers_j) = PIT_PITREP_ANN(pers_j) - PIT_TAXCRLOW_ANN_SP1(pers_j) - PIT_TAXCRLOW_ANN_SP2(pers_j)
            END IF
        END IF
    END DO

    !PIT_NETPITSE_ANN: Sum of all personal income taxes due to tax rates applied on separate income components
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_NETPITSE_ANN(pers_j) = 0.D0
            END IF
        END IF
    END DO
    
    !PIT_NETPIT_ANN: Sum of all personal incomes taxes: rate scheme taxes and separate taxes
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_NETPIT_ANN(pers_j) = PIT_NETPITSC_ANN(pers_j) + PIT_NETPITSE_ANN(pers_j)
            END IF
        END IF
    END DO

    !PIT_STATEPIT_ANN: Personal income taxes to be paid to the federal state
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_STATEPIT_ANN(pers_j) = PIT_NETPIT_ANN(pers_j)
            END IF
        END IF
    END DO

    !PIT_REGIOTAX_ANN: Personal income taxes to be paid to other entities than the federal state
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.PIT_STATEPIT_ANN(pers_j).GT.0.D0) THEN
                PIT_REGIOTAX_ANN(pers_j) = (PIT_COMUNTAX_ANN/100.D0)*PIT_STATEPIT_ANN(pers_j)
            END IF
        END IF
    END DO
    
    !PIT_CSAMOUNT_ANN: Amount of crisis surcharge
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_CSAMOUNT_ANN(pers_j) = PIT_BRACRATE_4COLUMNS_RATES(PIT_STATEPIT_ANN(pers_j),PIT_TNTI_ANN(pers_j),&
                &SIZE(PIT_RATECRIS_ANN,1),PIT_RATECRIS_ANN,pers_j)
            END IF
        END IF
    END DO

    !PIT_SPECSSCO_ANN: Amount of special social security contribution
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes and does not exist about pure self employed
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                SELF    =    PIT_GTISELF_ANN_SP1(pers_j) + PIT_GTISELF_ANN_SP2(pers_j)  
                OTHER   =    PIT_GTIPRIV_ANN_SP1(pers_j) +PIT_GTIPUB_ANN_SP1(pers_j)  +&        
                            &PIT_GTIPENS_ANN_SP1(pers_j) +PIT_GTIEARNE_ANN_SP1(pers_j)+&        
                            &PIT_GTIEAROL_ANN_SP1(pers_j)+PIT_GTIOLDUN_ANN_SP1(pers_j)+&   
                            &PIT_GTIOTHUN_ANN_SP1(pers_j)+PIT_GTIDISAB_ANN_SP1(pers_j)+&
                            &PIT_GTINDOCC_ANN_SP1(pers_j)+PIT_GTISICK_ANN_SP1(pers_j) +&
                            &PIT_GTIFAMAL_ANN_SP1(pers_j)+&
                            &PIT_GTIPRIV_ANN_SP2(pers_j) +PIT_GTIPUB_ANN_SP2(pers_j)  +&        
                            &PIT_GTIPENS_ANN_SP2(pers_j) +PIT_GTIEARNE_ANN_SP2(pers_j)+&        
                            &PIT_GTIEAROL_ANN_SP2(pers_j)+PIT_GTIOLDUN_ANN_SP2(pers_j)+&   
                            &PIT_GTIOTHUN_ANN_SP2(pers_j)+PIT_GTIDISAB_ANN_SP2(pers_j)+&
                            &PIT_GTINDOCC_ANN_SP2(pers_j)+PIT_GTISICK_ANN_SP2(pers_j) +&
                            &PIT_GTIFAMAL_ANN_SP2(pers_j)
                !check whether the tax unit exist of self employed only
                IF (SELF.EQ.0.D0.OR.(SELF.NE.0.D0.AND.OTHER.NE.0.D0)) THEN
                    PIT_SPECSSCO_ANN(pers_j) = PIT_BRACRATE_4COLUMNS_PLUS(PIT_TNTI_ANN(pers_j),SIZE(PIT_SPECCON_ANN,1),&
                    &PIT_SPECCON_ANN,pers_j) 
                END IF                    
            END IF
        END IF
    END DO

    !PIT_FLEMJOBR_ANN: Amount of reduction installed by the Flemish government for those who work
    DO pers_j=1,n_rec
        !Execute computation at the level of the head of the tax unit
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0.AND.PIT_REGIO(pers_j).EQ.2) THEN
                !reduction is determined per spouse on the obtained occupational income
                IF (PIT_NTOI_ANN_SP1(pers_j).GT.0.D0) THEN
                    PIT_FLEMJOBR_ANN(pers_j) = PIT_BRACRATE_4COLUMNS_MINUS(PIT_NTOI_ANN_SP1(pers_j),SIZE(PIT_JOBREDU_ANN,1),&
                    &PIT_JOBREDU_ANN,pers_j) 
                END IF
                IF (PIT_NTOI_ANN_SP2(pers_j).GT.0.D0) THEN
                    PIT_FLEMJOBR_ANN(pers_j) = PIT_FLEMJOBR_ANN(pers_j) + &
                    &PIT_BRACRATE_4COLUMNS_MINUS(PIT_NTOI_ANN_SP2(pers_j),SIZE(PIT_JOBREDU_ANN,1),PIT_JOBREDU_ANN,pers_j) 
                END IF
            END IF
        END IF
    END DO

    !PIT_FINTAX_ANN: Total amount of taxes paid by the tax unit
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            !if the tax unit is liable to pay income taxes
            IF (PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                PIT_FINTAX_ANN(pers_j) = PIT_STATEPIT_ANN(pers_j)+PIT_REGIOTAX_ANN(pers_j)+PIT_CSAMOUNT_ANN(pers_j)&
                &+PIT_SPECSSCO_ANN(pers_j)
                IF (PIT_FINTAX_ANN(pers_j).GT.PIT_FLEMJOBR_ANN(pers_j)) THEN
                    PIT_FINTAX_ANN(pers_j) = PIT_FINTAX_ANN(pers_j)-PIT_FLEMJOBR_ANN(pers_j)
                ELSE 
                    IF (PIT_FINTAX_ANN(pers_j).GT.0.D0) THEN
                        PIT_FINTAX_ANN(pers_j) = 0.D0
                    END IF
                END IF
                !Further deduct the rebatable tax credit for low income
                PIT_FINTAX_ANN(pers_j) = PIT_FINTAX_ANN(pers_j)
            END IF
        END IF
    END DO

END SUBROUTINE PIT_OTHPIT_CREATE

!===================================================================!
!Subroutine PIT_WEIGHT_CREATE                                       !
!Construct weights to scale up the observations on the tax units    !
!at the sample level to the level of population of tax units        !
!===================================================================!

SUBROUTINE PIT_WEIGHT_CREATE(TU_ID,TU_POS,TU_COUP,TU_LIAB,Vecsize,W_OUT)

    INTEGER                                                 :: i,j,k,n,pers_j,hh_i                  !loop indexes                                                                                                   - local
    INTEGER                                                 :: VecSize                              !VecSize: size of the vector x                                                                                  - input
    INTEGER,DIMENSION(VecSize)                              :: TU_ID                                !x(VecSize):the variable with original tax unit numbers, unsorted                                               - input
    INTEGER,DIMENSION(VecSize)                              :: TU_POS                               !Position of the individual within the tax unit, used to identify the head of the tax unit                      - input
    INTEGER,DIMENSION(VecSize)                              :: TU_COUP                              !Identifier to indicate whether tax unit consists of a married couple or of a single                            - input
    INTEGER,DIMENSION(VecSize)                              :: TU_LIAB                              !Identifier to indicate whether tax unit is liable to enter a tax form, if equal 1 for head, then not liable    - input
    INTEGER,DIMENSION(VecSize)                              :: X2                                   !x2(VecSize):the variable with original tax unit numbers, sorted                                                - local
    INTEGER,DIMENSION(VecSize)                              :: Y2                                   !y2(VecSize):the variable with original position in the tax unit, sorted                                        - local    
    INTEGER,DIMENSION(VecSize)                              :: Z1                                   !z2(VecSize):the variable with original position in the couples indicator, sorted                                        - local    
    INTEGER,DIMENSION(VecSize)                              :: Z2                                   !z2(VecSize):the variable with original position in the liability indicator, sorted                                        - local    
    INTEGER                                                 :: MaxTU                                !Largest number in the original vector                                                                          - local
    INTEGER,DIMENSION(VecSize)                              :: OrigPos                              !Vector that contains the original position of an observation, used to resort later on                          - local
    INTEGER,DIMENSION(VecSize)                              :: TAXU_SIZE                            !Vector storing the size of the tax unit at the tax unit level                                                  - local
    INTEGER,DIMENSION(VecSize)                              :: TAXU_START                           !Vector storing the first line of the tax unit                                                                  - local
    INTEGER,DIMENSION(VecSize)                              :: TAXU_END                             !Vector storing the last line of the tax unit                                                                   - local
    INTEGER                                                 :: TotTU                                !Total number of tax units in the sample                                                                        - local
    INTEGER,DIMENSION(VecSize)                              :: TAXU_SIZE_SUM                        !Vector storing the possibility that the head of the tax unit is in the sample                                  - local    
    INTEGER                                                 :: TotSumTU                             !sum of all possiblities of selection of tax unit heads                                                         - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !W(VecSize):the variable with tax unit weights at the tax unit level                                            - local
    REAL(KIND=DPRECI)                                       :: SumW                                 !Sum of the weights computed in first step                                                                      - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W_OUT                                !W(VecSize):the variable with tax unit weights at the level of the head of the tax unit                         - ouput    
    INTEGER                                                 :: CountTU                              !Count the number of tax units in the sociological household loop
    REAL(KIND=DPRECI)                                       :: Size_SING,Size_COUP                  !Scalars used to store the size of the group of singles and couples

    !PUT THE INPUT VECTORS INTO WORKING VECTORS SO THAT THEY
    !CAN BE SORTED WITHOUT INFLUENCING THE ORIGINAL VECTORS
    X2(1:VecSize)=TU_ID(1:VecSize)
    Y2(1:VecSize)=TU_POS(1:VecSize)    
    Z1(1:VecSize)=TU_COUP(1:VecSize)    
    Z2(1:VecSize)=TU_LIAB(1:VecSize)    

    !determine maximal observed tax unit number
    MaxTU=MAX(TU_ID(1:VecSize))

    DO i=1,VecSize
        !convert tax unit number of people not assigned to a tax unit to MaxTU + 1
        IF (X2(i).LE.0) THEN
            X2(i) = MaxTU + 1
        END IF
        !DETERMINE THE ORIGINAL POSITION OF THE INDIVIDUAL
        OrigPos(i) = i
    END DO

    !RANK AND SORT INPUT VECTOR TO DETERMINE TAX UNIT SIZE
    CALL RANKING(X2,NEW_RANK)                                  ! INDEX VECTOR X2 IN NEW_RANK
    CALL SORTING(X2,NEW_RANK)                                  ! SORT VECTOR X2 ON THE INDEX NEW_RANK
    CALL SORTING(Y2,NEW_RANK)                                  ! SORT VECTOR Y2 ON THE INDEX NEW_RANK
    CALL SORTING(Z1,NEW_RANK)                                  ! SORT VECTOR Z1 ON THE INDEX NEW_RANK
    CALL SORTING(Z2,NEW_RANK)                                  ! SORT VECTOR Z2 ON THE INDEX NEW_RANK
    CALL SORTING(OrigPos,NEW_RANK)                             ! SORT VECTOR OrigPos ON THE INDEX NEW_RANK

    !Initialise local vectors
    TAXU_SIZE(1:VecSize)=0
    TAXU_START(1:VecSize)=0
    TAXU_END(1:VecSize)=0    

    !Loop over all tax units and determine their stop and starting position and size of the tax unit
    i=1
    j=1
    k=0
    n=0
    DO
        IF (X2(j).LE.MaxTU) THEN
            j=j+n
            n=0
            k=k+1
            DO WHILE (X2(i)==X2(j))
                n=n+1
                i=i+1
                IF (i>VecSize) EXIT    
            END DO
            TAXU_SIZE(k)=n
            TAXU_START(k)=j
            TAXU_END(k)=j+n-1
            IF (i>VecSize.OR.j>VecSize)EXIT
        ELSE
            EXIT
        END IF    
    END DO
    TotTU=k

    !CONSTRUCTION OF TAX UNIT WEIGHTS
    !principle is the same as individual weights, i.e. 
    !weights to weigh the head of the tax unit have to be constructed
    !so, we start with the possibility that a head is sampled. This sampling possibility
    !is the same as the one for any other household member
    !With these possibilities we construct the factor gi/sum(gi's)
    !At this stage the sum(gi's) might still be biased
    !a better estimate would be to use gi*random sampling indicator and then sum the result

    !In order to determine the tax unit weights, we first have to determine
    !the possibility that the head of the tax unit is selected
    !This possibility is equal to the number of individuals in the tax unit
    !Hence for each tax unit we make the variable TAXU_SIZE_SUM equal to PIT_HH_SIZE
    !Only make weights for tax units that are liable to pay income taxes
    TAXU_SIZE_SUM(1:VecSize)=0
    CountTU  = 1
    DO hh_i=1,TotTU
        DO pers_j=TAXU_START(hh_i),TAXU_END(hh_i)    
            IF (Y2(pers_j).EQ.1.AND.Z2(pers_j).EQ.0) THEN
                TAXU_SIZE_SUM(pers_j) = TAXU_SIZE(hh_i)
                CountTU = CountTU + 1
            END IF    
        END DO    
    END DO

    !Compute the sum of sizes of the tax units with the sample observations only
    TotSumTU = SUM(TAXU_SIZE_SUM(1:VecSize))       

    !Determine weights that scale up to the sample size only
    W(1:VecSize)=0        
    DO hh_i=1,TotTU
        DO pers_j=TAXU_START(hh_i),TAXU_END(hh_i)    
            IF (TAXU_SIZE_SUM(pers_j).NE.0.AND.CountTU.NE.0) THEN
                W(pers_j) = DBLE(TotSumTU)/(DBLE(TAXU_SIZE_SUM(pers_j))*DBLE(CountTU))       
            END IF
        END DO
    END DO
    
    !Determine weights that scale up to the size of population of tax units
    !Total number of tax units in the population in tax year 2002:4905623
    !Remark that conisderable change in total size is also due to change in policy of NIS to count all fiscal units
    !including "les non enrles"
    SumW = SUM(W(1:VecSize))           
    DO hh_i=1,TotTU
        DO pers_j=TAXU_START(hh_i),TAXU_END(hh_i)    
            IF (INT(SumW*1.D-03).NE.0) THEN
                W(pers_j) = W(pers_j)*(6143173.D0/SumW)       
            END IF
        END DO
    END DO

    !Determine the size of the group of singles and couples
    Size_SING = 0
    Size_COUP = 0
    DO hh_i=1,TotTU
        DO pers_j=TAXU_START(hh_i),TAXU_END(hh_i)
            !If person j is part of a tax couple
            IF (Z1(pers_j).EQ.1) THEN
                !and liable to pay income taxes
                IF (Z2(pers_j).EQ.0) THEN
                    Size_COUP = Size_COUP + W(pers_j)
!                    Size_COUP = Size_COUP + MIMOSIS_WEIGHT(pers_j)
                END IF
            !If person j is a tax single
            ELSE IF (Z1(pers_j).EQ.0.AND.Y2(pers_j).EQ.1) THEN
                !and liable to pay income taxes
                IF (Z2(pers_j).EQ.0) THEN
                    Size_SING = Size_SING + W(pers_j)
!                    Size_SING = Size_SING + MIMOSIS_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO

    !Adapt the weights of singles and couples such that they scale up to the size of their group
    DO hh_i=1,TotTU
        DO pers_j=TAXU_START(hh_i),TAXU_END(hh_i)
            !If person j is part of a tax couple
            IF (Z1(pers_j).EQ.1) THEN
                !and liable to pay income taxes
                IF (Z2(pers_j).EQ.0) THEN
                    IF (INT(Size_COUP*1.D-03).NE.0) THEN
!                        W_OUT(pers_j) = W(pers_j)
!                        W_OUT(pers_j) = W(pers_j)*1.42
!                        W_OUT(pers_j) = W(pers_j)*(2140717.D0/Size_COUP)       
!                        W_OUT(pers_j) = (2140717.D0/Size_COUP)   
                        W_OUT(pers_j) = MIMOSIS_WEIGHT(pers_j) 
                    END IF
                END IF
            !If person j is a tax single
            ELSE IF (Z1(pers_j).EQ.0.AND.Y2(pers_j).EQ.1) THEN
                !and liable to pay income taxes
                IF (Z2(pers_j).EQ.0) THEN
                    IF (INT(Size_SING*1.D-03).NE.0) THEN
!                        W_OUT(pers_j) = W(pers_j)
!                        W_OUT(pers_j) = W(pers_j)*(4002456.D0/Size_SING)       
!                        W_OUT(pers_j) = (4002456.D0/Size_SING)      
!Vele varianten zijn mogelijk, maar gebruik van individueel gewicht lijkt het zuiverste, ook met het oog
!op de latere behandeling van het gegeven in de EVAL module. Dit resulteert wel in een lichte onderschatting van
!totaal aantal belasting-eenheden, maar dat ligt een beetje in de lijn der verwachting aangezien bij het totaal
!aantal eenheden allicht ook studenten geteld geweest zijn die in voorkomen als zowel ten laste als met een aparte
!eenheid
                        W_OUT(pers_j) = MIMOSIS_WEIGHT(pers_j)   
                    END IF
                END IF
            END IF
        END DO
    END DO

    !Set weights in their original position for output
    CALL RANKING(OrigPos,NEW_RANK)                             
    CALL SORTING(W_OUT,NEW_RANK)

END SUBROUTINE PIT_WEIGHT_CREATE

!====================================!
!Compute quantile numbers
!====================================!

SUBROUTINE PIT_QUANTILENR(X,W,TU_POS,TU_LIAB,VecSize,K,QVAL,QNR)
! In this subroutine we calculate the quantileNUMBER for the REAL(KIND=KIND_REAL) VecSize-vector X, with weights W
! The vector X must NOT be ordered on input and remains UNCHANGED on output
! The parameter K determines the number of quantiles
!       e.g. K=9 corresponds to deciles, K=99 are percentiles, K=4 are quintiles
! The output consists of the VecSize integer vector QNR with the quantile-numbers
! The subroutine essentially calls another subroutine QUANTILEK which produces the quantileVALUES,
!   which are then used to assign the quantilenumbers.
!   Since QUANTILEK is based on ordered vectors, we use two working vectors X2 and W2 
!

    INTEGER                                                 :: k                                    !k:(the number of quantiles -1)                                                             - input
    INTEGER                                                 :: VecSize                              !VecSize: size of the vectors x and w                                                       - input
    INTEGER,DIMENSION(VecSize)                              :: IRANK                                !Vector containing the rank of an observation, created to sort other variables later on     - local
    INTEGER,DIMENSION(VecSize)                              :: QNR                                  !qnr(VecSize):the quantile to which x(VecSize) belongs in the k+1-grouping                  - output
    INTEGER,DIMENSION(VecSize)                              :: OrigPos                              !Vector that contains the original position of an observation, used to resort later on      - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the quantilenumber is calculated,unsorted                - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x,unsorted                                      - input
    INTEGER,DIMENSION(VecSize)                              :: TU_POS                               !Position of the individual within the tax unit, used to identify the head of the tax unit                      - input
    INTEGER,DIMENSION(VecSize)                              :: TU_LIAB                              !Identifier to indicate whether tax unit is liable to enter a tax form, if equal 1 for head, then not liable    - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X2                                   !x(VecSize):the variable for which the quantilenumber is calculated,sorted                  - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W2                                   !w(VecSize):the weight attached to variable x,sorted                                        - local
    INTEGER,DIMENSION(VecSize)                              :: Y2                                   !y2(VecSize):the variable with original position in the tax unit, sorted                                        - local    
    INTEGER,DIMENSION(VecSize)                              :: Z2                                   !z2(VecSize):the variable with original position in the liability indicator, sorted                                        - local    
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: QVAL                                 !qval(k+2):the quantile value                                                               - local
    INTEGER                                                 :: pers_j

    ! PUT THE INPUTVECTORS X AND W INTO THE WORKING VECTORS
    X2(1:VecSize)=X(1:VecSize)
    W2(1:VecSize)=W(1:VecSize)
    Y2(1:VecSize)=TU_POS(1:VecSize)    
    Z2(1:VecSize)=TU_LIAB(1:VecSize)    

    !DETERMINE THE ORIGINAL POSITION OF THE INDIVIDUAL
    DO pers_j=1,VecSize
        OrigPos(pers_j) = pers_j
    END DO

    !RANK AND SORT INPUT VECTORS TO DETERMINE QUANTILE NUMBER WITH QUANTILEK
    CALL RANKING(X2,IRANK)                                  ! INDEX VECTOR X2 IN IRANK
    CALL SORTING(X2,IRANK)                                  ! SORT VECTOR X2 ON THE INDEX IRANK
    CALL SORTING(W2,IRANK)                                  ! SORT VECTOR W2 ON THE INDEX IRANK
    CALL SORTING(Y2,IRANK)                                  ! SORT VECTOR Y2 ON THE INDEX IRANK
    CALL SORTING(Z2,IRANK)                                  ! SORT VECTOR Z2 ON THE INDEX IRANK
    CALL SORTING(OrigPos,IRANK)                             ! SORT VECTOR OrigPos ON THE INDEX IRANK

    CALL QUANTILEK(X2,W2,Y2,Z2,VecSize,K,QVAL,QNR)          ! CALCULATE QUANTILEVALUES FOR X2

    !DIFFERENT FROM DARTS USE QNR FROM QUANTILEK TO REORDER QUANTILE NUMBERS
    !RANK AND SORT DECILE NUMBER BACK TO ORIGINAL POSITION
    CALL RANKING(OrigPos,IRANK)                             ! INDEX VECTOR X2 IN IRANK
    CALL SORTING(QNR,IRANK)

END SUBROUTINE PIT_QUANTILENR

!====================================!
!Compute quantile values
!====================================!

SUBROUTINE QUANTILEK(X,W,Y,Z,VecSize,K,Q,QNR)
!
! In this subroutine we calculate the quantilevalues for the REAL(KIND=KIND_REAL) VecSize-vector X, with weights W
! The vector must already be ordered 
! The parameter K determines the number of quantiles
!       e.g. K=9 corresponds to deciles, K=99 are percentiles, K=4 are quintiles
! The output consists of the K+2 vector Q with the quantile-values
!       and for each element of the vector x, the number of the quantile to which it belongs 
! The subroutine is inspired on Paarsch, but has been adapted in using weights for the observations

    INTEGER                                                 :: VecSize                              !VecSize:dimension of the vectors x and w - input
    INTEGER                                                 :: k                                    !k:the number of points where the ordinate is calculated (with exclusion of 0 and 1 on the abscissa) - input
    INTEGER,DIMENSION(VecSize)                              :: QNR                                  !Vector containing quantile number
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ITAG                                 !itag:contains the number of the observation where a given percentile ENDS, hence itag(j) means that the j-th quantile value is obtained as the value of V up to the itag(j)-th observation - local
    INTEGER(KIND=INT4),DIMENSION(0:K+1)                     :: ICNT                                 !icnt:the number of observations in a given quantile - local
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: X                                    !x(VecSize):the variable for which the ordinate is calculated - input
    REAL(KIND=KIND_REAL),DIMENSION(VecSize)                 :: W                                    !w(VecSize):the weight attached to variable x - input
    INTEGER,DIMENSION(VecSize)                              :: Y                                    !y(VecSize):the variable with original position in the tax unit, sorted                                        - local    
    INTEGER,DIMENSION(VecSize)                              :: Z                                    !z(VecSize):the variable with original position in the liability indicator, sorted                                        - local    
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: P                                    !vector with abscis values
    REAL(KIND=DPRECI),DIMENSION(0:VecSize)                  :: WCUM                                 !wcum:the cumulative sum of the weights, VecSize+1-vector - local
    REAL(KIND=DPRECI),DIMENSION(0:K+1)                      :: IWCUM                                !iwcum:the wcum-value corrsponding to the ex ante p-value - local
    REAL(KIND=DPRECI)                                       :: WSOM                                 !wsom:the sum of the weights - local
    REAL(KIND=DPRECI)                                       :: PROP                                 !Space between two abscis points
    REAL(KIND=KIND_REAL),DIMENSION(0:K+1)                   :: Q                                    !q:the quantile value (the value of x below which we find itag(j) of the observations) - local
    INTEGER                                                 :: PosTU                                !Position of tax unit last met
    INTEGER                                                 :: pers_j,j                             !counters
    INTEGER                                                 :: start                                !Value to start construction loop                                  !counters
    
    !INITIALISE VARIABLES
    P(0:K+1)=0.D0
    Q(0:K+1)=0.D0
    WSOM=0.D0
    WCUM(0:VecSize)=0.D0
    ITAG(0:K)=0
    ITAG(K+1)=VecSize
    ICNT(0:K+1)=0
    IWCUM(0:K+1)=0.D0

    !Compute sum weights
    PosTU  = 0
    DO  pers_j=1,VecSize
        IF (Y(pers_j).EQ.1.AND.Z(pers_j).EQ.0) THEN
            WSOM=WSOM+W(pers_j)
            WCUM(pers_j)=WCUM(PosTU)+W(pers_j)
            PosTU = pers_j
        END IF
    END DO

    !Compute P values and compare Wcum vector to determine ITAG
    PROP=1.D0/DBLE((K+1))
    start = 1
    DO j=1,K
        P(j)=PROP*j*WSOM
        DO pers_j=start,VecSize
            IF(P(j).LE.WCUM(pers_j).AND.Y(pers_j).EQ.1.AND.Z(pers_j).EQ.0) THEN
                ITAG(j)=pers_j
                start = pers_j
                EXIT
            ENDIF
        END DO
    END DO
    P(K+1)=WSOM  

    !Compute quantile values and number of observations in each quantile
!    WRITE(15,'("Quantile Value(j) Numb Indivi in quantile Pop Weight Indiv")')
    DO j=1,K+1
        Q(j)=X(ITAG(j))
        ICNT(j)=ITAG(j)-ITAG(j-1)
        IWCUM(j)=WCUM(ITAG(j))
!        WRITE(15,'(2x,f15.2,12x,i10,2x,2f15.2)') Q(j),ICNT(j),WCUM(ITAG(j))-WCUM(ITAG(j-1))
    END DO

    !Compute the decilenumber for each observation
    !Change as compared to DARTS no double definition of individual in position N2
    DO j=1,K+1
        DO pers_j=ITAG(j-1)+1,ITAG(j)
            IF(Y(pers_j).EQ.1.AND.Z(pers_j).EQ.0) THEN
                QNR(pers_j)=j
            END IF                
        END DO
    END DO
    
END SUBROUTINE QUANTILEK

!==========================================================!
!Produce some aggregate statistics to check the constuction!
!==========================================================!

SUBROUTINE PIT_TEST

    INTEGER                                                 :: pers_j,i
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI)                                       :: SumW_TU                              !Variable containing sum of weights of tax units
    REAL(KIND=DPRECI)                                       :: SumW_HEADS                           !Variable containing sum of weights of the heads of the tax units
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: PIT_RESULT_TEST                      !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: PIT_LAB_TEST                         !Labels of test variables
    REAL(KIND=DPRECI),DIMENSION(43)                         :: SumGross                             !Vector with quarterly amounts on benefits
    REAL(KIND=DPRECI)                                       :: TOT_TAX,TOT_INC,TAX_RATE
    REAL(KIND=DPRECI),DIMENSION(NUMBDEC,2)                  :: TEST_DECSIZE                         !Test size of deciles of net taxable income

    WRITE(15,'(/,''Print of test statistics'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights and some weight statistics
    SumW        =SUM(MIMOSIS_WEIGHT(1:n_rec))
    SumW_TU     =SUM(TAXUNIT_WEIGHT(1:n_rec))
    SumW_HEADS  =0.D0
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)) THEN
            SumW_HEADS = SumW_HEADS + MIMOSIS_WEIGHT(pers_j)
        ELSE
            IF (INT(100.D0*TAXUNIT_WEIGHT(pers_j)).NE.0) THEN
                write(15,'(''Individual is not head of a tax unit and still has value in weighting variable '', i10)')pers_j
            END IF
        END IF
    END DO        
    Write(15,'(/,''Total amount of individuals in population                            '', F15.0)') SumW
    Write(15,'(''Total amount of heads of the tax unit (including tax units not liable  '', F15.0)') SumW_HEADS
    Write(15,'(''Total amount of tax units in population                                '', F15.0)') SumW_TU

    !Compute aggregate statistics on gross taxable income concepts
    NumAnal = 25

    !Allocate and initialise test result
    ALLOCATE(PIT_RESULT_TEST(NumAnal,4))
    ALLOCATE(PIT_LAB_TEST(NumAnal))
    PIT_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PIT_LAB_TEST(1) = "Gross taxable labour income earned as wage earner on the private labour market by spouse 1 of the tax unit"
    PIT_LAB_TEST(2) = "Gross taxable labour income earned as wage earner on the public labour market by spouse 1 of the tax unit"
    PIT_LAB_TEST(3) = "Gross taxable labour income earned as self employed by spouse 1 of the tax unit"
    PIT_LAB_TEST(4) = "Gross taxable pension benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(5) = "Gross taxable early retirement benefits of the new type of spouse 1 of tax unit"
    PIT_LAB_TEST(6) = "Gross taxable early retirement benefits of the old type of spouse 1 of tax unit"
    PIT_LAB_TEST(7) = "Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit"
    PIT_LAB_TEST(8) = "Gross taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(9) = "Gross taxable disability benefits of wage earners of spouse 1 of tax unit"
    PIT_LAB_TEST(10) = "Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit"
    PIT_LAB_TEST(11) = "Gross taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(12) = "Gross taxable income from family allowances of spouse 1 of tax unit"
    PIT_LAB_TEST(13) = "Gross taxable labour income earned as wage earner on the private labour market by spouse 2 of the tax unit"
    PIT_LAB_TEST(14) = "Gross taxable labour income earned as wage earner on the public labour market by spouse 2 of the tax unit"
    PIT_LAB_TEST(15) = "Gross taxable labour income earned as self employed by spouse 2 of the tax unit"
    PIT_LAB_TEST(16) = "Gross taxable pension benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(17) = "Gross taxable early retirement benefits of the new type of spouse 2 of tax unit"
    PIT_LAB_TEST(18) = "Gross taxable early retirement benefits of the old type of spouse 2 of tax unit"
    PIT_LAB_TEST(19) = "Gross taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit"
    PIT_LAB_TEST(20) = "Gross taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(21) = "Gross taxable disability benefits of wage earners of spouse 2 of tax unit"
    PIT_LAB_TEST(22) = "Gross taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit"
    PIT_LAB_TEST(23) = "Gross taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(24) = "Gross taxable income from family allowances of spouse 2 of tax unit"
    PIT_LAB_TEST(25) = "Total amount of gross taxable income"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute annual amounts
        SumGross(1) = PIT_GTIPRIV_ANN_SP1(pers_j) 
        SumGross(2) = PIT_GTIPUB_ANN_SP1(pers_j) 
        SumGross(3) = PIT_GTISELF_ANN_SP1(pers_j) 
        SumGross(4) = PIT_GTIPENS_ANN_SP1(pers_j) 
        SumGross(5) = PIT_GTIEARNE_ANN_SP1(pers_j) 
        SumGross(6) = PIT_GTIEAROL_ANN_SP1(pers_j) 
        SumGross(7) = PIT_GTIOLDUN_ANN_SP1(pers_j) 
        SumGross(8) = PIT_GTIOTHUN_ANN_SP1(pers_j) 
        SumGross(9) = PIT_GTIDISAB_ANN_SP1(pers_j) 
        SumGross(10) =PIT_GTINDOCC_ANN_SP1(pers_j) 
        SumGross(11) =PIT_GTISICK_ANN_SP1(pers_j)
        SumGross(12) =PIT_GTIFAMAL_ANN_SP1(pers_j) 
        SumGross(13) =PIT_GTIPRIV_ANN_SP2(pers_j) 
        SumGross(14) =PIT_GTIPUB_ANN_SP2(pers_j) 
        SumGross(15) =PIT_GTISELF_ANN_SP2(pers_j) 
        SumGross(16) =PIT_GTIPENS_ANN_SP2(pers_j)
        SumGross(17) =PIT_GTIEARNE_ANN_SP2(pers_j) 
        SumGross(18) =PIT_GTIEAROL_ANN_SP2(pers_j) 
        SumGross(19) =PIT_GTIOLDUN_ANN_SP2(pers_j) 
        SumGross(20) =PIT_GTIOTHUN_ANN_SP2(pers_j) 
        SumGross(21) =PIT_GTIDISAB_ANN_SP2(pers_j) 
        SumGross(22) =PIT_GTINDOCC_ANN_SP2(pers_j) 
        SumGross(23) =PIT_GTISICK_ANN_SP2(pers_j) 
        SumGross(24) =PIT_GTIFAMAL_ANN_SP2(pers_j)  
        SumGross(25) =SUM(SumGross(1:24))
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
                &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                    PIT_RESULT_TEST(j,1)=PIT_RESULT_TEST(j,1)+TAXUNIT_WEIGHT(pers_j)*SumGross(j)
                    PIT_RESULT_TEST(j,2)=PIT_RESULT_TEST(j,2)+TAXUNIT_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PIT_RESULT_TEST(i,2)).NE.0) THEN
            PIT_RESULT_TEST(i,3)=PIT_RESULT_TEST(i,1)/PIT_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PIT_RESULT_TEST(i,4)=100.D0*PIT_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''Gross taxable income amounts of spouse1 and spouse2'')')
    Write(15,'(''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PIT_LAB_TEST(i),(PIT_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PIT_LAB_TEST)
    DEALLOCATE(PIT_RESULT_TEST) 

    !Compute aggregate statistics on net taxable income concepts
    NumAnal = 27

    !Allocate and initialise test result
    ALLOCATE(PIT_RESULT_TEST(NumAnal,4))
    ALLOCATE(PIT_LAB_TEST(NumAnal))
    PIT_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PIT_LAB_TEST(1)  = "Net taxable occupational income of spouse 1 of the tax unit"
    PIT_LAB_TEST(2)  = "Net taxable pension benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(3)  = "Net taxable early retirement benefits of the new type of spouse 1 of tax unit"
    PIT_LAB_TEST(4)  = "Net taxable early retirement benefits of the old type of spouse 1 of tax unit"
    PIT_LAB_TEST(5)  = "Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 1 of tax unit"
    PIT_LAB_TEST(6)  = "Net taxable unemployment benefits, other than early retirement benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(7)  = "Net taxable disability benefits of wage earners of spouse 1 of tax unit"
    PIT_LAB_TEST(8)  = "Net taxable benefits in case of industrial accidents or occupational diseases of spouse 1 of tax unit"
    PIT_LAB_TEST(9)  = "Net taxable benefits in case of sickness other than disability benefits of spouse 1 of tax unit"
    PIT_LAB_TEST(10) = "Lump sum costs of spouse 1 deductible of employment income"
    PIT_LAB_TEST(11) = "Net taxable employment income of spouse 1 of the tax unit"
    PIT_LAB_TEST(12) = "Net taxable self employment income of spouse 1 of the tax unit"
    PIT_LAB_TEST(13) = "Total amount of net taxable income of spouse 1 of the tax unit"
    PIT_LAB_TEST(14) = "Net taxable occupational income of spouse 2 of the tax unit"
    PIT_LAB_TEST(15) = "Net taxable pension benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(16) = "Net taxable early retirement benefits of the new type of spouse 2 of tax unit"
    PIT_LAB_TEST(17) = "Net taxable early retirement benefits of the old type of spouse 2 of tax unit"
    PIT_LAB_TEST(18) = "Net taxable unemployment benefits of older unemployed with seniority supplement of spouse 2 of tax unit"
    PIT_LAB_TEST(19) = "Net taxable unemployment benefits, other than early retirement benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(20) = "Net taxable disability benefits of wage earners of spouse 2 of tax unit"
    PIT_LAB_TEST(21) = "Net taxable benefits in case of industrial accidents or occupational diseases of spouse 2 of tax unit"
    PIT_LAB_TEST(22) = "Net taxable benefits in case of sickness other than disability benefits of spouse 2 of tax unit"
    PIT_LAB_TEST(23) = "Lump sum costs of spouse 2 deductible of employment income"
    PIT_LAB_TEST(24) = "Net taxable employment income of spouse 2 of the tax unit"
    PIT_LAB_TEST(25) = "Net taxable self employment income of spouse 2 of the tax unit"
    PIT_LAB_TEST(26) = "Total amount of net taxable income of spouse 2 of the tax unit"
    PIT_LAB_TEST(27) = "Amount of taxable income that can be transferred from one spouse to the other because of mar split"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute annual amounts
        SumGross(1) = PIT_NTOI_ANN_SP1(pers_j)
        SumGross(2) = PIT_NTIPENS_ANN_SP1(pers_j)  
        SumGross(3) = PIT_NTIEARNE_ANN_SP1(pers_j) 
        SumGross(4) = PIT_NTIEAROL_ANN_SP1(pers_j) 
        SumGross(5) = PIT_NTIOLDUN_ANN_SP1(pers_j)  
        SumGross(6) = PIT_NTIOTHUN_ANN_SP1(pers_j)  
        SumGross(7) = PIT_NTIDISAB_ANN_SP1(pers_j)  
        SumGross(8) = PIT_NTINDOCC_ANN_SP1(pers_j)  
        SumGross(9) = PIT_NTISICK_ANN_SP1(pers_j)  
        SumGross(10) =PIT_COSTRATE_ANN_SP1(pers_j)  
        SumGross(11) =PIT_NTIEMPL_ANN_SP1(pers_j)  
        SumGross(12) =PIT_NTISELF_ANN_SP1(pers_j)   
        SumGross(13) =PIT_NTI_ANN_SP1(pers_j)      
        SumGross(14) =PIT_NTOI_ANN_SP2(pers_j)     
        SumGross(15) =PIT_NTIPENS_ANN_SP2(pers_j)  
        SumGross(16) =PIT_NTIEARNE_ANN_SP2(pers_j) 
        SumGross(17) =PIT_NTIEAROL_ANN_SP2(pers_j)  
        SumGross(18) =PIT_NTIOLDUN_ANN_SP2(pers_j)  
        SumGross(19) =PIT_NTIOTHUN_ANN_SP2(pers_j)  
        SumGross(20) =PIT_NTIDISAB_ANN_SP2(pers_j)  
        SumGross(21) =PIT_NTINDOCC_ANN_SP2(pers_j)  
        SumGross(22) =PIT_NTISICK_ANN_SP2(pers_j)   
        SumGross(23) =PIT_COSTRATE_ANN_SP2(pers_j) 
        SumGross(24) =PIT_NTIEMPL_ANN_SP2(pers_j)   
        SumGross(25) =PIT_NTISELF_ANN_SP2(pers_j)   
        SumGross(26) =PIT_NTI_ANN_SP2(pers_j)       
        SumGross(27) =PIT_MARSPLIT_ANN(pers_j)   
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
                &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                    PIT_RESULT_TEST(j,1)=PIT_RESULT_TEST(j,1)+TAXUNIT_WEIGHT(pers_j)*SumGross(j)
                    PIT_RESULT_TEST(j,2)=PIT_RESULT_TEST(j,2)+TAXUNIT_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PIT_RESULT_TEST(i,2)).NE.0) THEN
            PIT_RESULT_TEST(i,3)=PIT_RESULT_TEST(i,1)/PIT_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PIT_RESULT_TEST(i,4)=100.D0*PIT_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''Net taxable income amounts of spouse1 and spouse2'')')
    Write(15,'(''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PIT_LAB_TEST(i),(PIT_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PIT_LAB_TEST)
    DEALLOCATE(PIT_RESULT_TEST) 

    !Compute aggregate statistics on final tax computations
    NumAnal = 43

    !Allocate and initialise test result
    ALLOCATE(PIT_RESULT_TEST(NumAnal,4))
    ALLOCATE(PIT_LAB_TEST(NumAnal))
    PIT_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    PIT_LAB_TEST(1)  = "Gross amount of personal income taxes of spouse 1 after applying the global tax rate scheme on net &
    &taxable income"
    PIT_LAB_TEST(2)  = "Gross amount of personal income taxes of spouse 2 after applying the global tax rate scheme on net &
    &taxable income"
    PIT_LAB_TEST(3)  = "Income amount of spouse 1 that is used to compute the tax credit for family composition"
    PIT_LAB_TEST(4)  = "Income amount of spouse 2 that is used to compute the tax credit for family composition"
    PIT_LAB_TEST(5)  = "Personal income taxes of spouse 1 net of tax credits for family composition"
    PIT_LAB_TEST(6)  = "Personal income taxes of spouse 2 net of tax credits for family composition"
    PIT_LAB_TEST(7)  = "Personal income taxes at the tax unit level net of tax credits for family composition"
    PIT_LAB_TEST(8)  = "Personal income taxes at the tax unit level net of tax credits for family composition and credits for&
    & replacement income"
    PIT_LAB_TEST(9)  = "Basic tax credit granted for pension benefits and early retirement benefits of new type: spouse 1"
    PIT_LAB_TEST(10) = "Basic tax credit granted for early retirement benefits of old type: spouse 1"
    PIT_LAB_TEST(11) = "Basic tax credit granted for normal unemployment benefits: spouse 1"
    PIT_LAB_TEST(12) = "Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above): spouse 1"
    PIT_LAB_TEST(13) = "Basic tax credit granted for sickness and disability benefits: spouse 1"
    PIT_LAB_TEST(14) = "Basic tax credit granted for other replacement income: spouse 1"
    PIT_LAB_TEST(15) = "Basic tax credit granted for pension benefits and early retirement benefits of new type: spouse 2"
    PIT_LAB_TEST(16) = "Basic tax credit granted for early retirement benefits of old type: spouse 2"
    PIT_LAB_TEST(17) = "Basic tax credit granted for normal unemployment benefits: spouse 2"
    PIT_LAB_TEST(18) = "Basic tax credit granted for unemployment benefits of older unemployed (age of 58 or above): spouse 2"
    PIT_LAB_TEST(19) = "Basic tax credit granted for sickness and disability benefits: spouse 2"
    PIT_LAB_TEST(20) = "Basic tax credit granted for other replacement income: spouse 2"
    PIT_LAB_TEST(21) = "Total net taxable income of the tax unit"
    PIT_LAB_TEST(22) = "Reduced tax credit granted for pension benefits and early retirement benefits of new type: spouse 1"
    PIT_LAB_TEST(23) = "Reduced tax credit granted for early retirement benefits of old type: spouse 1"
    PIT_LAB_TEST(24) = "Reduced tax credit granted for normal unemployment benefits: spouse 1"
    PIT_LAB_TEST(25) = "Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above): spouse 1"
    PIT_LAB_TEST(26) = "Reduced tax credit granted for sickness and disability benefits: spouse 1"
    PIT_LAB_TEST(27) = "Reduced tax credit granted for other replacement income: spouse 1"
    PIT_LAB_TEST(28) = "Reduced tax credit granted for pension benefits and early retirement benefits of new type: spouse 2"
    PIT_LAB_TEST(29) = "Reduced tax credit granted for early retirement benefits of old type: spouse 2"
    PIT_LAB_TEST(30) = "Reduced tax credit granted for normal unemployment benefits: spouse 2"
    PIT_LAB_TEST(31) = "Reduced tax credit granted for unemployment benefits of older unemployed (age of 58 or above): spouse 2"
    PIT_LAB_TEST(32) = "Reduced tax credit granted for sickness and disability benefits: spouse 2"
    PIT_LAB_TEST(33) = "Reduced tax credit granted for other replacement income: spouse 2"
    PIT_LAB_TEST(34) = "Personal income taxes at the tax unit level net of all tax credits and obtained after application of a&
    & tax rate scheme on an aggregate income concept"
    PIT_LAB_TEST(35) = "Sum of all personal income taxes due to tax rates applied on separate income components"
    PIT_LAB_TEST(36) = "Sum of all personal incomes taxes: rate scheme taxes and separate taxes"
    PIT_LAB_TEST(37) = "Personal income taxes to be paid to the federal state"
    PIT_LAB_TEST(38) = "Personal income taxes to be paid to other entities than the federal state"
    PIT_LAB_TEST(39) = "Amount of crisis surcharge"
    PIT_LAB_TEST(40) = "Special contribution for social security"
    PIT_LAB_TEST(41) = "Tax credit for low labour income of spouse 1 (in case of self employed or civil servant income)"
    PIT_LAB_TEST(42) = "Tax credit for low labour income of spouse 2 (in case of self employed or civil servant income)"
    PIT_LAB_TEST(43) = "Amount of reduction, granted by the Flemish government for those who work"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute annual amounts
        SumGross(1) = PIT_GPIT_ANN_SP1(pers_j)   
        SumGross(2) = PIT_GPIT_ANN_SP2(pers_j)    
        SumGross(3) = PIT_EXEMP_ANN_SP1(pers_j)   
        SumGross(4) = PIT_EXEMP_ANN_SP2(pers_j)   
        SumGross(5) = PIT_PITFAMC_ANN_SP1(pers_j)  
        SumGross(6) = PIT_PITFAMC_ANN_SP2(pers_j)  
        SumGross(7) = PIT_PITFAMC_ANN(pers_j)      
        SumGross(8) = PIT_PITREP_ANN(pers_j)       
        SumGross(9) = PIT_BTCREP_ANN1(pers_j,1)     
        SumGross(10) =PIT_BTCREP_ANN2(pers_j,1)      
        SumGross(11) =PIT_BTCREP_ANN3(pers_j,1)     
        SumGross(12) =PIT_BTCREP_ANN4(pers_j,1)      
        SumGross(13) =PIT_BTCREP_ANN5(pers_j,1)     
        SumGross(14) =PIT_BTCREP_ANN6(pers_j,1)     
        SumGross(15) =PIT_BTCREP_ANN1(pers_j,2)     
        SumGross(16) =PIT_BTCREP_ANN2(pers_j,2)      
        SumGross(17) =PIT_BTCREP_ANN3(pers_j,2)     
        SumGross(18) =PIT_BTCREP_ANN4(pers_j,2)      
        SumGross(19) =PIT_BTCREP_ANN5(pers_j,2)     
        SumGross(20) =PIT_BTCREP_ANN6(pers_j,2)     
        SumGross(21) =PIT_TNTI_ANN(pers_j)        
        SumGross(22) =PIT_RTCREP_ANN1(pers_j,1)     
        SumGross(23) =PIT_RTCREP_ANN2(pers_j,1)      
        SumGross(24) =PIT_RTCREP_ANN3(pers_j,1)      
        SumGross(25) =PIT_RTCREP_ANN4(pers_j,1)      
        SumGross(26) =PIT_RTCREP_ANN5(pers_j,1)      
        SumGross(27) =PIT_RTCREP_ANN6(pers_j,1)      
        SumGross(28) =PIT_RTCREP_ANN1(pers_j,2)     
        SumGross(29) =PIT_RTCREP_ANN2(pers_j,2)      
        SumGross(30) =PIT_RTCREP_ANN3(pers_j,2)      
        SumGross(31) =PIT_RTCREP_ANN4(pers_j,2)      
        SumGross(32) =PIT_RTCREP_ANN5(pers_j,2)      
        SumGross(33) =PIT_RTCREP_ANN6(pers_j,2)      
        SumGross(34) =PIT_NETPITSC_ANN(pers_j)     
        SumGross(35) =PIT_NETPITSE_ANN(pers_j)    
        SumGross(36) =PIT_NETPIT_ANN(pers_j)       
        SumGross(37) =PIT_STATEPIT_ANN(pers_j)     
        SumGross(38) =PIT_REGIOTAX_ANN(pers_j)     
        SumGross(39) =PIT_CSAMOUNT_ANN(pers_j)     
        SumGross(40) =PIT_SPECSSCO_ANN(pers_j)
        SumGross(41) =PIT_TAXCRLOW_ANN_SP1(pers_j)
        SumGross(42) =PIT_TAXCRLOW_ANN_SP2(pers_j)
        SumGross(43) =PIT_FLEMJOBR_ANN(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
                &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
                    PIT_RESULT_TEST(j,1)=PIT_RESULT_TEST(j,1)+TAXUNIT_WEIGHT(pers_j)*SumGross(j)
                    PIT_RESULT_TEST(j,2)=PIT_RESULT_TEST(j,2)+TAXUNIT_WEIGHT(pers_j)
                END IF
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(PIT_RESULT_TEST(i,2)).NE.0) THEN
            PIT_RESULT_TEST(i,3)=PIT_RESULT_TEST(i,1)/PIT_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            PIT_RESULT_TEST(i,4)=100.D0*PIT_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''Personal income taxes of the tax unit'')')
    Write(15,'(''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')PIT_LAB_TEST(i),(PIT_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(PIT_LAB_TEST)
    DEALLOCATE(PIT_RESULT_TEST) 

    !Compute aggregate statistics on final tax bill (not weighted)
    TOT_TAX = 0.D0
    TOT_INC = 0.D0
    TAX_RATE = 0.D0
    DO pers_j=1,n_rec                                                                                 
        IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
        &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TOT_TAX=TOT_TAX+PIT_STATEPIT_ANN(pers_j)+PIT_REGIOTAX_ANN(pers_j)+PIT_CSAMOUNT_ANN(pers_j)
            TOT_INC=TOT_INC+PIT_TNTI_ANN(pers_j)
        END IF
    END DO
    IF (TOT_INC.GT.0.5D-15) THEN 
        TAX_RATE=(TOT_TAX/TOT_INC)*100
    END IF        
    Write(15,'(/,''Total taxes, taxable income and tax rate: not weighted'')')
    Write(15,'(''        TOT_INC        TOT_TAX       TAX_RATE'')')
    WRITE(15,'(3F15.2)') TOT_INC,TOT_TAX,TAX_RATE

    !Compute aggregate statistics on final tax bill (weighted with tax unit weights)
    TOT_TAX = 0.D0
    TOT_INC = 0.D0
    TAX_RATE = 0.D0
    DO pers_j=1,n_rec                                                                                 
        IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
        &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TOT_TAX=TOT_TAX+(PIT_STATEPIT_ANN(pers_j)+PIT_REGIOTAX_ANN(pers_j)+PIT_CSAMOUNT_ANN(pers_j))*TAXUNIT_WEIGHT(pers_j)
            TOT_INC=TOT_INC+PIT_TNTI_ANN(pers_j)*TAXUNIT_WEIGHT(pers_j)
        END IF
    END DO
    IF (TOT_INC.GT.0.5D-15) THEN 
        TAX_RATE=(TOT_TAX/TOT_INC)*100
    END IF        
    Write(15,'(/,''Total taxes, taxable income and tax rate: weighted with tax unit weights'')')
    Write(15,'(''        TOT_INC        TOT_TAX       TAX_RATE'')')
    WRITE(15,'(3F15.2)') TOT_INC,TOT_TAX,TAX_RATE

    !Compute aggregate statistics on final tax bill (weighted with individual sample weights)
    TOT_TAX = 0.D0
    TOT_INC = 0.D0
    TAX_RATE = 0.D0
    DO pers_j=1,n_rec                                                                                 
        IF ((PIT_COUPLE_ANN(pers_j).EQ.1.OR.(PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1)).AND.&
        &PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TOT_TAX=TOT_TAX+(PIT_STATEPIT_ANN(pers_j)+PIT_REGIOTAX_ANN(pers_j)+PIT_CSAMOUNT_ANN(pers_j))*MIMOSIS_WEIGHT(pers_j)
            TOT_INC=TOT_INC+PIT_TNTI_ANN(pers_j)*MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO
    IF (TOT_INC.GT.0.5D-15) THEN 
        TAX_RATE=(TOT_TAX/TOT_INC)*100
    END IF        
    Write(15,'(/,''Total taxes, taxable income and tax rate: weighted with individual sample weights'')')
    Write(15,'(''        TOT_INC        TOT_TAX       TAX_RATE'')')
    WRITE(15,'(3F15.2)') TOT_INC,TOT_TAX,TAX_RATE

    TEST_DECSIZE(1:NUMBDEC,1:2) = 0
    DO pers_j=1,n_rec
        DO j=1,NUMBDEC
            IF (QNR_TNTI(pers_j).EQ.j) THEN
                TEST_DECSIZE(j,1) = TEST_DECSIZE(j,1) + 1.D0
                TEST_DECSIZE(j,2) = TEST_DECSIZE(j,2) + TAXUNIT_WEIGHT(pers_j)
            END IF
        END DO
    END DO
    Write(15,'(/,''Size of deciles constructed with net taxable income'')')
    Write(15,'(''    Decile  Quantile Valu       Size Unw    Size Weight'')')
    DO j=1,NUMBDEC
        write(15,'(i10,3F15.0)') j,QVAL_TNTI(j),(TEST_DECSIZE(j,i),i=1,2)
    END DO
    
END SUBROUTINE PIT_TEST

!====================================!
!Subroutine to create age classes
!====================================!

SUBROUTINE PIT_AGECLASS_CREATION

    DO pers_j=1,n_rec
        IF (PIT_YEAR_AGE(pers_j).LT.15) THEN
            PIT_AGECLASS(pers_j)=2
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.14.AND.PIT_YEAR_AGE(pers_j).LE.19) THEN
            PIT_AGECLASS(pers_j)=3
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.19.AND.PIT_YEAR_AGE(pers_j).LE.24) THEN
            PIT_AGECLASS(pers_j)=4
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.24.AND.PIT_YEAR_AGE(pers_j).LE.29) THEN    
            PIT_AGECLASS(pers_j)=5
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.29.AND.PIT_YEAR_AGE(pers_j).LE.34) THEN
            PIT_AGECLASS(pers_j)=6
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.34.AND.PIT_YEAR_AGE(pers_j).LE.39) THEN
            PIT_AGECLASS(pers_j)=7
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.39.AND.PIT_YEAR_AGE(pers_j).LE.44) THEN
            PIT_AGECLASS(pers_j)=8
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.44.AND.PIT_YEAR_AGE(pers_j).LE.49) THEN
            PIT_AGECLASS(pers_j)=9 
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.49.AND.PIT_YEAR_AGE(pers_j).LE.55) THEN
            PIT_AGECLASS(pers_j)=10
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.55.AND.PIT_YEAR_AGE(pers_j).LE.59) THEN
            PIT_AGECLASS(pers_j)=11
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.59.AND.PIT_YEAR_AGE(pers_j).LE.64) THEN
            PIT_AGECLASS(pers_j)=12
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.64.AND.PIT_YEAR_AGE(pers_j).LE.69) THEN
            PIT_AGECLASS(pers_j)=13
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.69.AND.PIT_YEAR_AGE(pers_j).LE.74) THEN
            PIT_AGECLASS(pers_j)=14
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.74.AND.PIT_YEAR_AGE(pers_j).LE.79) THEN
            PIT_AGECLASS(pers_j)=15
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.79.AND.PIT_YEAR_AGE(pers_j).LE.84) THEN
            PIT_AGECLASS(pers_j)=16
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.84.AND.PIT_YEAR_AGE(pers_j).LE.89) THEN
            PIT_AGECLASS(pers_j)=17
        ELSE IF (PIT_YEAR_AGE(pers_j).GT.89) THEN
            PIT_AGECLASS(pers_j)=18
        END IF
    END DO

END SUBROUTINE PIT_AGECLASS_CREATION

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other external sources
!==========================================================!

SUBROUTINE PIT_COMPARE

    INTEGER                                             :: pers_j,i,j,k
    !Data on taxable income and personal income taxes
    REAL(KIND=DPRECI),DIMENSION(21,6)                   :: PITTNTI_TESTINFO                          !Information read from external source and used to judge the quality of the reconstructed tax concepts     
    REAL(KIND=DPRECI),DIMENSION(21,6)                   :: PITTNTI_TESTCOMP                          !Recomputed test statistics (comparable with content of TESTINFO file
    CHARACTER(len=100),DIMENSION(21)                    :: PIT_LAB_COMPARE                           !Vector with labels of the different reconstructed aggregates                       
    REAL(KIND=DPRECI),DIMENSION(6)                      :: PITTNTI_DIF                               !Vector to store difference between computed value and external value
    CHARACTER(len=7)                                    :: NCHAR                                     !Variable containing the character equivalent of the converted value of the integer    
    CHARACTER(len=20)                                   :: CHAR_SCAL                                 !Store scalar value in character format
    CHARACTER(len=200)                                  :: CHAR_LINE                                 !Construct line with character values
    INTEGER,EXTERNAL                                    :: NONBLK
    INTEGER                                             :: TELCHAR                                   !scalar to count the position of the character element that has to be added to the reconstructed character-line
    !Other data, added later (on age classes and tax forms by decile and household type
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TAXFCOUP_AGECLASS_DECILE_TESTINFO         !Number of tax files of couples by age class and decile of taxable income
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TAXFSING_AGECLASS_DECILE_TESTINFO         !Number of tax files of singles by age class and decile of taxable income
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TINCCOUP_AGECLASS_DECILE_TESTINFO         !Taxable income of couples by age class and decile of taxable income
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TINCSING_AGECLASS_DECILE_TESTINFO         !Taxable income of singles by age class and decile of taxable income
    REAL(KIND=DPRECI),DIMENSION(11,7)                   :: TINCSING_DECILE_SIZE_TESTINFO             !Taxable income of singles by decile of taxable income and size of the household
    REAL(KIND=DPRECI),DIMENSION(11,7)                   :: TAXFSING_DECILE_SIZE_TESTINFO             !Number of tax files of singles by decile of taxable income and size of the household
    REAL(KIND=DPRECI),DIMENSION(11,7)                   :: TINCCOUP_DECILE_SIZE_TESTINFO             !Taxable income of couples by decile of taxable income and size of the household
    REAL(KIND=DPRECI),DIMENSION(11,7)                   :: TAXFCOUP_DECILE_SIZE_TESTINFO             !Number of tax files of couples by decile of taxable income and size of the household
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TAXFTINC_COUPLE_TESTCOMP                  !Recomputed test statistics (comparable with content of TESTINFO file on taxable income and tax forms)
    REAL(KIND=DPRECI),DIMENSION(19,11)                  :: TAXFTINC_SINGLE_TESTCOMP                  !Recomputed test statistics (comparable with content of TESTINFO file on taxable income and tax forms)
    REAL(KIND=DPRECI),DIMENSION(11)                     :: TAXFTINC_DIF                              !Compute the difference between the computed value and the real one
    REAL(KIND=DPRECI),DIMENSION(11)                     :: TAXFTINC_CRED                             !Compute the credit granted for the given line
    INTEGER                                             :: TAXUNIT_SIZE                              !scalar to store the size of the tax unit
    REAL(KIND=DPRECI)                                   :: TAXAMOUNT                                 !Tax amount to compare with ADSEI

    WRITE(15,'(/,''Print statistics comparable with other sources'')')
    CALL DRAWLINE(15,110,'=')    

    !Read information from external source
    OPEN(UNIT=5, FILE=PITTNTI_2008, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PITTNTI_2008
        WRITE(15,'("Could not find ", a100)') PITTNTI_2008
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        !skip first 7 lines
        CALL RSKIP(5,7)
        DO i=1,21
            READ(5,'(10x,F10.0,F20.0,F10.1,F20.0,F10.1,F10.2)',IOSTAT=ierror) &  
            &(PITTNTI_TESTINFO(i,j),j=1,6) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip again 7 lines
        CALL RSKIP(5,7)
        DO i=1,19
            READ(5,'(35x,11F15.0)',IOSTAT=ierror) &  
            &(TAXFCOUP_AGECLASS_DECILE_TESTINFO(i,j),j=1,11) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip 3 lines
        CALL RSKIP(5,3)
        DO i=1,19
            READ(5,'(35x,11F15.0)',IOSTAT=ierror) &  
            &(TAXFSING_AGECLASS_DECILE_TESTINFO(i,j),j=1,11) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip again 7 lines
        CALL RSKIP(5,7)
        DO i=1,19
            READ(5,'(35x,11F15.0)',IOSTAT=ierror) &  
            &(TINCCOUP_AGECLASS_DECILE_TESTINFO(i,j),j=1,11) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip 3 lines
        CALL RSKIP(5,3)
        DO i=1,19
            READ(5,'(35x,11F15.0)',IOSTAT=ierror) &  
            &(TINCSING_AGECLASS_DECILE_TESTINFO(i,j),j=1,11) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip again 7 lines
        CALL RSKIP(5,7)
        DO i=1,11
            READ(5,'(35x,7F15.0)',IOSTAT=ierror) &  
            &(TINCSING_DECILE_SIZE_TESTINFO(i,j),j=1,7) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip 3 lines
        CALL RSKIP(5,3)
        DO i=1,11
            READ(5,'(35x,7F15.0)',IOSTAT=ierror) &  
            &(TAXFSING_DECILE_SIZE_TESTINFO(i,j),j=1,7) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip 3 lines
        CALL RSKIP(5,3)
        DO i=1,11
            READ(5,'(35x,7F15.0)',IOSTAT=ierror) &  
            &(TINCCOUP_DECILE_SIZE_TESTINFO(i,j),j=1,7) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
        !skip 3 lines
        CALL RSKIP(5,3)
        DO i=1,11
            READ(5,'(35x,7F15.0)',IOSTAT=ierror) &  
            &(TAXFCOUP_DECILE_SIZE_TESTINFO(i,j),j=1,7) 
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') i,PITTNTI_2008
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)

!Start constructie Tabel 1

    !Construct labels for decile and percentile constructions
    PIT_LAB_COMPARE(1)  = "1                        "
    PIT_LAB_COMPARE(2)  = "2                        "
    PIT_LAB_COMPARE(3)  = "3                        "
    PIT_LAB_COMPARE(4)  = "4                        "
    PIT_LAB_COMPARE(5)  = "5                        "
    PIT_LAB_COMPARE(6)  = "6                        "
    PIT_LAB_COMPARE(7)  = "7                        "
    PIT_LAB_COMPARE(8)  = "8                        "
    PIT_LAB_COMPARE(9)  = "9                        "
    PIT_LAB_COMPARE(10) = "         91              "
    PIT_LAB_COMPARE(11) = "         92              "
    PIT_LAB_COMPARE(12) = "         93              "
    PIT_LAB_COMPARE(13) = "         94              "
    PIT_LAB_COMPARE(14) = "         95              "
    PIT_LAB_COMPARE(15) = "         96              "
    PIT_LAB_COMPARE(16) = "         97              "
    PIT_LAB_COMPARE(17) = "         98              "
    PIT_LAB_COMPARE(18) = "         99              "
    PIT_LAB_COMPARE(19) = "         100             "
    PIT_LAB_COMPARE(20) = "10                       "
    PIT_LAB_COMPARE(21) = "Totaal                   "

    !Write information on taxable income and taxes by decile and percentile
    Write(15,'('' '')')
    Write(15,'(''Tabel 1a: Totaal belastbaar netto inkomen en belasting van het Aanslagjaar 2009 - Belgi'')')
    Write(15,'(''Decielenverdeling op basis van het totaal belastbaar netto-inkomen'')')
    Write(15,'(''Gegevens overgenomen uit fiscale statistieken van het NIS'')')
    CALL DRAWLINE(15,110,'-')                
    Write(15,'(''                                                                                                    Gemiddelde'')')
    Write(15,'(''                            Totaal belastbaar netto-inkomen                Totale belasting        &
    &aanslagvoet'')')
    Write(15,'(''                                                                                                        &
    &(in %)'')')
    Write(15,'(''Decielen Percentielen    Bovengrens       Totaal bedrag  In % van       Totaal bedrag  In % van'')')
    Write(15,'(''                                                       het totaal                    het totaal'')')
    Write(15,'(''                          (in Euro)           (in Euro)                     (in Euro)'')')
    CALL DRAWLINE(15,110,'=')                
    DO i=1,21
        IF (i.LT.10) THEN
            WRITE (NCHAR, '(I7)') i
            CALL TRIM@(NCHAR)
            WRITE(15,'(a7,18x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTINFO(i,j),j=1,6) 
        ELSE IF (i.LT.20) THEN
            WRITE (NCHAR, '(I7)') i+81
            CALL TRIM@(NCHAR)
            WRITE(15,'(9x,a7,9x,,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTINFO(i,j),j=1,6) 
        ELSE IF (i.EQ.20) THEN
            WRITE (NCHAR, '(I7)') i-10
            CALL TRIM@(NCHAR)
            WRITE(15,'(a7,18x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTINFO(i,j),j=1,6) 
        ELSE IF (i.EQ.21) THEN
            WRITE(15,'(''Totaal'',19x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &(PITTNTI_TESTINFO(i,j),j=1,6) 
        END IF
    END DO

    !Recompute test statistics comparable with content of the file PITTNTI_2008 on taxable income and taxes by decile and percentile
    PITTNTI_TESTCOMP(1:21,1:6) = 0

    !identify quantile values
    DO i=1,NUMBDEC
        IF (i.LE.90) THEN
            DO j=1,9
                IF (i.EQ.(j*10)) THEN
                    PITTNTI_TESTCOMP(j,1) = QVAL_TNTI(i)
                END IF
            END DO
        ELSE IF (i.GT.90) THEN
            DO j=1,10
                IF (i.EQ.(j+90)) THEN
                    PITTNTI_TESTCOMP(j+9,1) = QVAL_TNTI(i)
                END IF
            END DO
        END IF
    END DO

    !compute taxable income and taxes first
    DO pers_j=1,n_rec
        TAXAMOUNT = PIT_STATEPIT_ANN(pers_j)
        !compute values for decile 1 to 9
        IF (QNR_TNTI(pers_j).LE.90.AND.QNR_TNTI(pers_j).NE.0) THEN
            DO j=0,8
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    PITTNTI_TESTCOMP(j+1,2) = PITTNTI_TESTCOMP(j+1,2) + TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    PITTNTI_TESTCOMP(j+1,4) = PITTNTI_TESTCOMP(j+1,4) + TAXUNIT_WEIGHT(pers_j)*TAXAMOUNT
                    !statistics for total
                    PITTNTI_TESTCOMP(21,2)  = PITTNTI_TESTCOMP(21,2) + TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    PITTNTI_TESTCOMP(21,4)  = PITTNTI_TESTCOMP(21,4) + TAXUNIT_WEIGHT(pers_j)*TAXAMOUNT
                    EXIT
                END IF
            END DO
        !compute percentiles 91 until 100 and decile 10
        ELSE IF (QNR_TNTI(pers_j).GT.90.AND.QNR_TNTI(pers_j).NE.0) THEN
            DO j=1,10
                IF (QNR_TNTI(pers_j).EQ.(j+90)) THEN
                    PITTNTI_TESTCOMP(j+9,2) = PITTNTI_TESTCOMP(j+9,2) + TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    PITTNTI_TESTCOMP(j+9,4) = PITTNTI_TESTCOMP(j+9,4) + TAXUNIT_WEIGHT(pers_j)*TAXAMOUNT
                    PITTNTI_TESTCOMP(20,2)  = PITTNTI_TESTCOMP(20,2) + TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    PITTNTI_TESTCOMP(20,4)  = PITTNTI_TESTCOMP(20,4) + TAXUNIT_WEIGHT(pers_j)*TAXAMOUNT
                    !statistics for total
                    PITTNTI_TESTCOMP(21,2)  = PITTNTI_TESTCOMP(21,2) + TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    PITTNTI_TESTCOMP(21,4)  = PITTNTI_TESTCOMP(21,4) + TAXUNIT_WEIGHT(pers_j)*TAXAMOUNT
                    EXIT
                END IF
            END DO
        END IF
    END DO

    !compute derived statistics
    DO i=1,21
        !Share of taxable income as percentage of total amount of taxable income
        IF (PITTNTI_TESTCOMP(21,2).NE.0.D0) THEN
            PITTNTI_TESTCOMP(i,3) = 100.D0*PITTNTI_TESTCOMP(i,2)/PITTNTI_TESTCOMP(21,2)    
        END IF
        !Share of taxes as percentage of total amount of taxes
        IF (PITTNTI_TESTCOMP(21,4).NE.0.D0) THEN
            PITTNTI_TESTCOMP(i,5) = 100.D0*PITTNTI_TESTCOMP(i,4)/PITTNTI_TESTCOMP(21,4)    
        END IF
        !average tax rate
        IF (PITTNTI_TESTCOMP(i,2).NE.0.D0) THEN
            PITTNTI_TESTCOMP(i,6) = 100.D0*PITTNTI_TESTCOMP(i,4)/PITTNTI_TESTCOMP(i,2)    
        END IF
    END DO

    Write(15,'('' '')')
    Write(15,'(''Tabel 1b: Totaal belastbaar netto inkomen en belasting van het Aanslagjaar 2009 - Belgi'')')
    Write(15,'(''Decielenverdeling op basis van het totaal belastbaar netto-inkomen'')')
    Write(15,'(''Gegevens berekend met de PIT module'')')
    CALL DRAWLINE(15,110,'-')                
    Write(15,'(''                                                                                                    Gemiddelde'')')
    Write(15,'(''                            Totaal belastbaar netto-inkomen                Totale belasting        &
    &aanslagvoet'')')
    Write(15,'(''                                                                                                        &
    &(in %)'')')
    Write(15,'(''Decielen Percentielen    Bovengrens       Totaal bedrag  In % van       Totaal bedrag  In % van'')')
    Write(15,'(''                                                       het totaal                    het totaal'')')
    Write(15,'(''                          (in Euro)           (in Euro)                     (in Euro)'')')
    CALL DRAWLINE(15,110,'=')                
    DO i=1,21
        IF (i.LT.10) THEN
            WRITE (NCHAR, '(I7)') i
            CALL TRIM@(NCHAR)
            WRITE(15,'(a7,18x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTCOMP(i,j),j=1,6) 
        ELSE IF (i.LT.20) THEN
            WRITE (NCHAR, '(I7)') i+81
            CALL TRIM@(NCHAR)
            WRITE(15,'(9x,a7,9x,,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTCOMP(i,j),j=1,6) 
        ELSE IF (i.EQ.20) THEN
            WRITE (NCHAR, '(I7)') i-10
            CALL TRIM@(NCHAR)
            WRITE(15,'(a7,18x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &NCHAR,(PITTNTI_TESTCOMP(i,j),j=1,6) 
        ELSE IF (i.EQ.21) THEN
            WRITE(15,'(''Totaal'',19x,F10.0,F20.0,F10.1,F20.0,F10.1,F15.2)') &  
            &(PITTNTI_TESTCOMP(i,j),j=1,6) 
        END IF
    END DO

    Write(15,'('' '')')
    Write(15,'(''Tabel 1c: Totaal belastbaar netto inkomen en belasting van het Aanslagjaar 2009 - Belgi'')')
    Write(15,'(''Decielenverdeling op basis van het totaal belastbaar netto-inkomen'')')
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = bedrag in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,110,'-')                
    Write(15,'(''                                                                                                    Gemiddelde'')')
    Write(15,'(''                            Totaal belastbaar netto-inkomen                Totale belasting        &
    &aanslagvoet'')')
    Write(15,'(''                                                                                                        &
    &(in %)'')')
    Write(15,'(''Decielen Percentielen    Bovengrens       Totaal bedrag  In % van       Totaal bedrag  In % van'')')
    Write(15,'(''                                                       het totaal                    het totaal'')')
    Write(15,'(''                          (in Euro)           (in Euro)                     (in Euro)'')')
    CALL DRAWLINE(15,110,'=')                
    DO i=1,21
        PITTNTI_DIF(1:6) = 0.D0
        CHAR_LINE(1:25) = PIT_LAB_COMPARE(i)
        TELCHAR = 25
        DO j=1,6
            CHAR_SCAL = "                    "
            IF (ANY(j.EQ.(/1,3,5/))) THEN
                IF (PITTNTI_TESTINFO(i,j).NE.0) THEN
                    PITTNTI_DIF(j) = 100.D0*(PITTNTI_TESTCOMP(i,j)-PITTNTI_TESTINFO(i,j))/PITTNTI_TESTINFO(i,j)
                    WRITE (CHAR_SCAL, '(F10.2)') PITTNTI_DIF(j)
                ELSE
                    CHAR_SCAL = "       EF0"
                END IF
                CHAR_LINE(1:TELCHAR+10) = CHAR_LINE(1:TELCHAR) // CHAR_SCAL 
                TELCHAR = TELCHAR + 10
            ELSE IF (ANY(j.EQ.(/2,4/))) THEN
                IF (PITTNTI_TESTINFO(i,j).NE.0) THEN
                    PITTNTI_DIF(j) = 100.D0*(PITTNTI_TESTCOMP(i,j)-PITTNTI_TESTINFO(i,j))/PITTNTI_TESTINFO(i,j)
                    WRITE (CHAR_SCAL, '(F20.2)') PITTNTI_DIF(j)
                ELSE
                    CHAR_SCAL = "                 EF0"
                END IF
                CHAR_LINE(1:TELCHAR+20) = CHAR_LINE(1:TELCHAR) // CHAR_SCAL 
                TELCHAR = TELCHAR + 20
            ELSE IF (j.EQ.6) THEN
                IF (PITTNTI_TESTINFO(i,j).NE.0) THEN
                    PITTNTI_DIF(j) = 100.D0*(PITTNTI_TESTCOMP(i,j)-PITTNTI_TESTINFO(i,j))/PITTNTI_TESTINFO(i,j)
                    WRITE (CHAR_SCAL, '(F15.2)') PITTNTI_DIF(j)
                ELSE
                    CHAR_SCAL = "            EF0"
                END IF
                CHAR_LINE(1:TELCHAR+15) = CHAR_LINE(1:TELCHAR) // CHAR_SCAL 
                TELCHAR = TELCHAR + 15
            END IF            
        END DO
        WRITE(15,'(a110)') CHAR_LINE 
    END DO

!Start constructie Tabel 2

    !Construct labels for age class constructions
    PIT_LAB_COMPARE(1)  = "Onbekend                           "
    PIT_LAB_COMPARE(2)  = "Minder dan 15 jaar                 "
    PIT_LAB_COMPARE(3)  = "Van 15 tot 19 jaar                 "
    PIT_LAB_COMPARE(4)  = "Van 20 tot 24 jaar                 "
    PIT_LAB_COMPARE(5)  = "Van 25 tot 29 jaar                 "
    PIT_LAB_COMPARE(6)  = "Van 30 tot 34 jaar                 "
    PIT_LAB_COMPARE(7)  = "Van 35 tot 39 jaar                 "
    PIT_LAB_COMPARE(8)  = "Van 40 tot 44 jaar                 "
    PIT_LAB_COMPARE(9)  = "Van 45 tot 49 jaar                 "
    PIT_LAB_COMPARE(10) = "Van 50 tot 54 jaar                 "
    PIT_LAB_COMPARE(11) = "Van 55 tot 59 jaar                 "
    PIT_LAB_COMPARE(12) = "Van 60 tot 64 jaar                 "
    PIT_LAB_COMPARE(13) = "Van 65 tot 69 jaar                 "
    PIT_LAB_COMPARE(14) = "Van 70 tot 74 jaar                 "
    PIT_LAB_COMPARE(15) = "Van 75 tot 79 jaar                 "
    PIT_LAB_COMPARE(16) = "Van 80 tot 84 jaar                 "
    PIT_LAB_COMPARE(17) = "Van 85 tot 89 jaar                 "
    PIT_LAB_COMPARE(18) = "90 jaar en meer                    "
    PIT_LAB_COMPARE(19) = "Totaal                             "

    !Write information on tax forms by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 2a: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Gegevens overgenomen uit fiscale statistieken van het NIS'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFCOUP_AGECLASS_DECILE_TESTINFO(i,j),j=1,11)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFSING_AGECLASS_DECILE_TESTINFO(i,j),j=1,11)             
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute test statistics comparable with content of the file PITTNTI_2008 on enterred tax forms
    TAXFTINC_COUPLE_TESTCOMP(1:19,1:11) = 0
    TAXFTINC_SINGLE_TESTCOMP(1:19,1:11) = 0

    !count tax froms
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    DO k=1,18
                        IF (PIT_AGECLASS(pers_j).EQ.k) THEN
                            TAXFTINC_COUPLE_TESTCOMP(k,j+1) = TAXFTINC_COUPLE_TESTCOMP(k,j+1) + TAXUNIT_WEIGHT(pers_j)
                            TAXFTINC_COUPLE_TESTCOMP(19,j+1) = TAXFTINC_COUPLE_TESTCOMP(19,j+1) + TAXUNIT_WEIGHT(pers_j)
                            EXIT
                        END IF
                    END DO
                    EXIT
                END IF
            END DO                
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    DO k=1,18
                        IF (PIT_AGECLASS(pers_j).EQ.k) THEN
                            TAXFTINC_SINGLE_TESTCOMP(k,j+1) = TAXFTINC_SINGLE_TESTCOMP(k,j+1) + TAXUNIT_WEIGHT(pers_j)
                            TAXFTINC_SINGLE_TESTCOMP(19,j+1) = TAXFTINC_SINGLE_TESTCOMP(19,j+1) + TAXUNIT_WEIGHT(pers_j)
                            EXIT
                        END IF
                    END DO
                    EXIT
                END IF
            END DO                
        END IF    
    END DO
    !compute total and store it in the last column
    DO i=1,19
        TAXFTINC_COUPLE_TESTCOMP(i,11) = SUM(TAXFTINC_COUPLE_TESTCOMP(i,1:10))
        TAXFTINC_SINGLE_TESTCOMP(i,11) = SUM(TAXFTINC_SINGLE_TESTCOMP(i,1:10))
    END DO

    !Write recomputed results on tax forms by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 2b: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Gegevens berekend met de PIT module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_COUPLE_TESTCOMP(i,j),j=1,11)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_SINGLE_TESTCOMP(i,j),j=1,11)             
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results on tax forms by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 2c: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = bedrag in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,11
            CHAR_SCAL = "                    "
            IF (TAXFCOUP_AGECLASS_DECILE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_COUPLE_TESTCOMP(i,j)-TAXFCOUP_AGECLASS_DECILE_TESTINFO(i,j))/&
                &TAXFCOUP_AGECLASS_DECILE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a200)') CHAR_LINE 
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,11
            CHAR_SCAL = "                    "
            IF (TAXFSING_AGECLASS_DECILE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_SINGLE_TESTCOMP(i,j)-TAXFSING_AGECLASS_DECILE_TESTINFO(i,j))/&
                &TAXFSING_AGECLASS_DECILE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a200)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                

!Start constructie Tabel 3

     !Write information on taxable income by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 3a: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Gegevens overgenomen uit fiscale statistieken van het NIS'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TINCCOUP_AGECLASS_DECILE_TESTINFO(i,j),j=1,11)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TINCSING_AGECLASS_DECILE_TESTINFO(i,j),j=1,11)             
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Recompute test statistics comparable with content of the file PITTNTI_2008 on enterred tax forms
    TAXFTINC_COUPLE_TESTCOMP(1:19,1:11) = 0
    TAXFTINC_SINGLE_TESTCOMP(1:19,1:11) = 0

    !compute taxable income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    DO k=1,18
                        IF (PIT_AGECLASS(pers_j).EQ.k) THEN
                            TAXFTINC_COUPLE_TESTCOMP(k,j+1) = TAXFTINC_COUPLE_TESTCOMP(k,j+1) + &
                            &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                            TAXFTINC_COUPLE_TESTCOMP(19,j+1) = TAXFTINC_COUPLE_TESTCOMP(19,j+1) + &
                            &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                            EXIT
                        END IF
                    END DO
                    EXIT
                END IF
            END DO                
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    DO k=1,18
                        IF (PIT_AGECLASS(pers_j).EQ.k) THEN
                            TAXFTINC_SINGLE_TESTCOMP(k,j+1) = TAXFTINC_SINGLE_TESTCOMP(k,j+1) + &
                            &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                            TAXFTINC_SINGLE_TESTCOMP(19,j+1) = TAXFTINC_SINGLE_TESTCOMP(19,j+1) + &
                            &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                            EXIT
                        END IF
                    END DO
                    EXIT
                END IF
            END DO                
        END IF    
    END DO
    !compute total and store it in the last column
    DO i=1,19
        TAXFTINC_COUPLE_TESTCOMP(i,11) = SUM(TAXFTINC_COUPLE_TESTCOMP(i,1:10))
        TAXFTINC_SINGLE_TESTCOMP(i,11) = SUM(TAXFTINC_SINGLE_TESTCOMP(i,1:10))
    END DO

    !Write reconstructed results on taxable income by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 3b: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Gegevens berekend met de PIT module'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_COUPLE_TESTCOMP(i,j),j=1,11)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        WRITE(15,'(a35,11F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_SINGLE_TESTCOMP(i,j),j=1,11)             
    END DO
    CALL DRAWLINE(15,200,'=')                

    !Compute the difference between the recomputed and external results on taxable income by age class and decile
    Write(15,'('' '')')
    Write(15,'(''Tabel 3c: Leeftijdcategorie van de aangever: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per leeftijdscategorie, per deciel'')') 
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = bedrag in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,200,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,11
            CHAR_SCAL = "                    "
            IF (TINCCOUP_AGECLASS_DECILE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_COUPLE_TESTCOMP(i,j)-TINCCOUP_AGECLASS_DECILE_TESTINFO(i,j))/&
                &TINCCOUP_AGECLASS_DECILE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a200)') CHAR_LINE 
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden per deciel'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5              6              7              8              9             10         Totaal'')')
    CALL DRAWLINE(15,200,'-')                
    DO i=1,19
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,11
            CHAR_SCAL = "                    "
            IF (TINCSING_AGECLASS_DECILE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_SINGLE_TESTCOMP(i,j)-TINCSING_AGECLASS_DECILE_TESTINFO(i,j))/&
                &TINCSING_AGECLASS_DECILE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a200)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,200,'=')                
   
!Start constructie Tabel 4

    !Construct labels for decile constructions
    PIT_LAB_COMPARE(1)  = "1                                  "
    PIT_LAB_COMPARE(2)  = "2                                  "
    PIT_LAB_COMPARE(3)  = "3                                  "
    PIT_LAB_COMPARE(4)  = "4                                  "
    PIT_LAB_COMPARE(5)  = "5                                  "
    PIT_LAB_COMPARE(6)  = "6                                  "
    PIT_LAB_COMPARE(7)  = "7                                  "
    PIT_LAB_COMPARE(8)  = "8                                  "
    PIT_LAB_COMPARE(9)  = "9                                  "
    PIT_LAB_COMPARE(10) = "10                                 "
    PIT_LAB_COMPARE(11) = "Totaal                             "

    !Write information on tax forms by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 4a: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Gegevens overgenomen uit fiscale statistieken van het NIS'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels:volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFCOUP_DECILE_SIZE_TESTINFO(i,j),j=1,7)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fisaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFSING_DECILE_SIZE_TESTINFO(i,j),j=1,7)             
    END DO
    CALL DRAWLINE(15,155,'=')                

    !Recompute test statistics comparable with content of the file PITTNTI_2008 on enterred tax forms
    TAXFTINC_COUPLE_TESTCOMP(1:19,1:11) = 0
    TAXFTINC_SINGLE_TESTCOMP(1:19,1:11) = 0

    !count tax froms
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TAXUNIT_SIZE = 2 + PIT_DEPCHILD_ANN(pers_j) + PIT_DEPOTHER_ANN(pers_j)
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    IF (TAXUNIT_SIZE.LT.7) THEN
                        DO k=2,6
                            IF (TAXUNIT_SIZE.EQ.k) THEN
                                TAXFTINC_COUPLE_TESTCOMP(j+1,k-1) = TAXFTINC_COUPLE_TESTCOMP(j+1,k-1) + TAXUNIT_WEIGHT(pers_j)
                                TAXFTINC_COUPLE_TESTCOMP(11,k-1) = TAXFTINC_COUPLE_TESTCOMP(11,k-1) + TAXUNIT_WEIGHT(pers_j)
                                EXIT
                            END IF
                        END DO
                    ELSE IF (TAXUNIT_SIZE.GE.7) THEN
                        TAXFTINC_COUPLE_TESTCOMP(j+1,6) = TAXFTINC_COUPLE_TESTCOMP(j+1,6) + TAXUNIT_WEIGHT(pers_j)
                        TAXFTINC_COUPLE_TESTCOMP(11,6) = TAXFTINC_COUPLE_TESTCOMP(11,6) + TAXUNIT_WEIGHT(pers_j)
                    END IF
                    EXIT
                END IF
            END DO                
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TAXUNIT_SIZE = 1 + PIT_DEPCHILD_ANN(pers_j) + PIT_DEPOTHER_ANN(pers_j)
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    IF (TAXUNIT_SIZE.LT.6) THEN
                        DO k=1,5
                            IF (TAXUNIT_SIZE.EQ.k) THEN
                                TAXFTINC_SINGLE_TESTCOMP(j+1,k) = TAXFTINC_SINGLE_TESTCOMP(j+1,k) + TAXUNIT_WEIGHT(pers_j)
                                TAXFTINC_SINGLE_TESTCOMP(11,k) = TAXFTINC_SINGLE_TESTCOMP(11,k) + TAXUNIT_WEIGHT(pers_j)
                                EXIT
                            END IF
                        END DO
                    ELSE IF (TAXUNIT_SIZE.GE.6) THEN
                        TAXFTINC_SINGLE_TESTCOMP(j+1,6) = TAXFTINC_COUPLE_TESTCOMP(j+1,6) + TAXUNIT_WEIGHT(pers_j)
                        TAXFTINC_SINGLE_TESTCOMP(11,6) = TAXFTINC_COUPLE_TESTCOMP(11,6) + TAXUNIT_WEIGHT(pers_j)
                    END IF
                    EXIT
                END IF
            END DO                
        END IF    
    END DO
    !compute total and store it in the last column
    DO i=1,11
        TAXFTINC_COUPLE_TESTCOMP(i,7) = SUM(TAXFTINC_COUPLE_TESTCOMP(i,1:6))
        TAXFTINC_SINGLE_TESTCOMP(i,7) = SUM(TAXFTINC_SINGLE_TESTCOMP(i,1:6))
    END DO

    !Write reconstructed results on tax forms by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 4b: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Gegevens berekend met de PIT module'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels:volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_COUPLE_TESTCOMP(i,j),j=1,7)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fisaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_SINGLE_TESTCOMP(i,j),j=1,7)             
    END DO
    CALL DRAWLINE(15,155,'=')                

    !Compute the difference between the recomputed and external results on tax forms by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 4c: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Aantal aangiften per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = bedrag in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Aantal aangiften van fiscale koppels:volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,7
            CHAR_SCAL = "                    "
            IF (TAXFCOUP_DECILE_SIZE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_COUPLE_TESTCOMP(i,j)-TAXFCOUP_DECILE_SIZE_TESTINFO(i,j))/&
                &TAXFCOUP_DECILE_SIZE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a140)') CHAR_LINE 
    END DO
    Write(15,'('' '')')
    Write(15,'(''Aantal aangiften van fisaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,7
            CHAR_SCAL = "                    "
            IF (TAXFSING_DECILE_SIZE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_SINGLE_TESTCOMP(i,j)-TAXFSING_DECILE_SIZE_TESTINFO(i,j))/&
                &TAXFSING_DECILE_SIZE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a140)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,155,'=')                

    !Compute tax credits granted for size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 4d: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Afwijking in geschatte en werkelijke gezinsomvang per deciel omgezet naar corresponderende belastingkrediet &
    &voor gezinsomvang in Euro'')') 
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              &
    &4              5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-') 
    !start with couples
    DO i=1,11
        TAXFTINC_DIF(1:11) = 0
        TAXFTINC_CRED(1:11) = 0
        DO j=1,6
            !compute tax credit
            IF (j.EQ.1) THEN
                TAXFTINC_CRED(j) = 2.D0*PIT_EXEMPC_ANN 
            ELSE 
                TAXFTINC_CRED(j) = 2.D0*PIT_EXEMPC_ANN + PIT_SUMAMOUNT_1COLUMN(j-1,SIZE(PIT_EXEMSCH_ANN,1),&
                                   &PIT_EXEMSCH_ANN,400000)
            END IF
            !store difference in cell with corresponding amount of people
            IF (j.LT.5) THEN
                TAXFTINC_DIF(j+1) = (TAXFTINC_COUPLE_TESTCOMP(i,j)-TAXFCOUP_DECILE_SIZE_TESTINFO(i,j))*TAXFTINC_CRED(j)
            ELSE
                TAXFTINC_DIF(6) = (TAXFTINC_COUPLE_TESTCOMP(i,j)-TAXFCOUP_DECILE_SIZE_TESTINFO(i,j))*TAXFTINC_CRED(j)
            END IF
        END DO
    END DO
    !continue with singles
    DO i=1,11
        TAXFTINC_CRED(1:11) = 0
        DO j=1,6
            !compute tax credit
            IF (j.EQ.1) THEN
                TAXFTINC_CRED(j) = PIT_EXEMPS_ANN 
            ELSE 
                TAXFTINC_CRED(j) = PIT_EXEMPS_ANN + PIT_SUMAMOUNT_1COLUMN(j-1,SIZE(PIT_EXEMSCH_ANN,1),&
                                   &PIT_EXEMSCH_ANN,400000)
            END IF
            !store difference in cell with corresponding amount of people
            TAXFTINC_DIF(j) = (TAXFTINC_SINGLE_TESTCOMP(i,j)-TAXFSING_DECILE_SIZE_TESTINFO(i,j))*TAXFTINC_CRED(j)
        END DO
        TAXFTINC_DIF(7) = SUM(TAXFTINC_DIF(1:6))
        WRITE(15,'(a35,8F15.0)') PIT_LAB_COMPARE(i),(TAXFTINC_DIF(j),j=1,7)
    END DO
    CALL DRAWLINE(15,155,'=')                

!Start constructie Tabel 5

    !Write information on taxable income by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 5a: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Gegevens overgenomen uit fiscale statistieken van het NIS'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TINCCOUP_DECILE_SIZE_TESTINFO(i,j),j=1,7)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TINCSING_DECILE_SIZE_TESTINFO(i,j),j=1,7)             
    END DO
    CALL DRAWLINE(15,155,'=')                

    !Recompute test statistics comparable with content of the file PITTNTI_2008 on enterred tax forms
    TAXFTINC_COUPLE_TESTCOMP(1:19,1:11) = 0
    TAXFTINC_SINGLE_TESTCOMP(1:19,1:11) = 0

    !compute taxable income
    DO pers_j=1,n_rec
        IF (PIT_COUPLE_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TAXUNIT_SIZE = 2 + PIT_DEPCHILD_ANN(pers_j) + PIT_DEPOTHER_ANN(pers_j)
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    IF (TAXUNIT_SIZE.LT.7) THEN
                        DO k=2,6
                            IF (TAXUNIT_SIZE.EQ.k) THEN
                                TAXFTINC_COUPLE_TESTCOMP(j+1,k-1) = TAXFTINC_COUPLE_TESTCOMP(j+1,k-1) + &
                                &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                                TAXFTINC_COUPLE_TESTCOMP(11,k-1) = TAXFTINC_COUPLE_TESTCOMP(11,k-1) + &
                                &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                                EXIT
                            END IF
                        END DO
                    ELSE IF (TAXUNIT_SIZE.GE.7) THEN
                        TAXFTINC_COUPLE_TESTCOMP(j+1,6) = TAXFTINC_COUPLE_TESTCOMP(j+1,6) + &
                        &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                        TAXFTINC_COUPLE_TESTCOMP(11,6) = TAXFTINC_COUPLE_TESTCOMP(11,6) + &
                        &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    END IF
                    EXIT
                END IF
            END DO                
        ELSE IF (PIT_COUPLE_ANN(pers_j).EQ.0.AND.PIT_TURANK_ANN(pers_j).EQ.1.AND.PIT_TUNOLAB_ANN(pers_j).EQ.0) THEN
            TAXUNIT_SIZE = 1 + PIT_DEPCHILD_ANN(pers_j) + PIT_DEPOTHER_ANN(pers_j)
            DO j=0,9
                IF (QNR_TNTI(pers_j).GT.(j*10).AND.QNR_TNTI(pers_j).LE.((j+1)*10)) THEN
                    IF (TAXUNIT_SIZE.LT.6) THEN
                        DO k=1,5
                            IF (TAXUNIT_SIZE.EQ.k) THEN
                                TAXFTINC_SINGLE_TESTCOMP(j+1,k) = TAXFTINC_SINGLE_TESTCOMP(j+1,k) + &
                                &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                                TAXFTINC_SINGLE_TESTCOMP(11,k) = TAXFTINC_SINGLE_TESTCOMP(11,k) + &
                                &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                                EXIT
                            END IF
                        END DO
                    ELSE IF (TAXUNIT_SIZE.GE.6) THEN
                        TAXFTINC_SINGLE_TESTCOMP(j+1,6) = TAXFTINC_COUPLE_TESTCOMP(j+1,6) + &
                        &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                        TAXFTINC_SINGLE_TESTCOMP(11,6) = TAXFTINC_COUPLE_TESTCOMP(11,6) + &
                        &TAXUNIT_WEIGHT(pers_j)*PIT_TNTI_ANN(pers_j)
                    END IF
                    EXIT
                END IF
            END DO                
        END IF    
    END DO
    !compute total and store it in the last column
    DO i=1,11
        TAXFTINC_COUPLE_TESTCOMP(i,7) = SUM(TAXFTINC_COUPLE_TESTCOMP(i,1:6))
        TAXFTINC_SINGLE_TESTCOMP(i,7) = SUM(TAXFTINC_SINGLE_TESTCOMP(i,1:6))
    END DO

    !Write reconstructed results on taxable income by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 5b: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Gegevens berekend met de PIT module'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_COUPLE_TESTCOMP(i,j),j=1,7)             
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        WRITE(15,'(a35,7F15.0)') &  
        &PIT_LAB_COMPARE(i),(TAXFTINC_SINGLE_TESTCOMP(i,j),j=1,7)             
    END DO
    CALL DRAWLINE(15,155,'=')                

    !Compute the difference between the recomputed and external results on taxable income by decile and size of the family
    Write(15,'('' '')')
    Write(15,'(''Tabel 5c: Aantal personen in het fiscaal gezin: gegevens aanslagjaar 2009 en voor Belgi'')')
    Write(15,'(''Totaal netto belastbaar inkomen per type aangifte, per deciel volgens het aantal personen in het gezin'')') 
    Write(15,'(''Verschil berekende gegevens en externe gegevens in procentuele termen van het extern gegeven'')')
    Write(15,'(''EF0 = bedrag in externe bron is gelijk aan 0'')')
    CALL DRAWLINE(15,155,'=')                
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscale koppels: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 2              3              4              &
    &5              6      7 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,7
            CHAR_SCAL = "                    "
            IF (TINCCOUP_DECILE_SIZE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_COUPLE_TESTCOMP(i,j)-TINCCOUP_DECILE_SIZE_TESTINFO(i,j))/&
                &TINCCOUP_DECILE_SIZE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a140)') CHAR_LINE 
    END DO
    Write(15,'('' '')')
    Write(15,'(''Totaal netto belastbaar inkomen (in Euro) van fiscaal alleenstaanden: volgens het aantal personen in het gezin'')')
    Write(15,'(''                                                 1              2              3              4              &
    &5      6 of meer         Totaal'')')
    CALL DRAWLINE(15,155,'-')                
    DO i=1,11
        TAXFTINC_DIF(1:11) = 0
        CHAR_LINE(1:35) = PIT_LAB_COMPARE(i)
        DO j=1,7
            CHAR_SCAL = "                    "
            IF (TINCSING_DECILE_SIZE_TESTINFO(i,j).NE.0) THEN
                TAXFTINC_DIF(j) = 100.D0*(TAXFTINC_SINGLE_TESTCOMP(i,j)-TINCSING_DECILE_SIZE_TESTINFO(i,j))/&
                &TINCSING_DECILE_SIZE_TESTINFO(i,j)
                WRITE (CHAR_SCAL, '(F15.2)') TAXFTINC_DIF(j)
            ELSE
                CHAR_SCAL = "            EF0"
            END IF                
            CHAR_LINE(1:35+j*15) = CHAR_LINE(1:35+(j-1)*15) // CHAR_SCAL 
        END DO
        WRITE(15,'(a140)') CHAR_LINE 
    END DO
    CALL DRAWLINE(15,155,'=')                
    
END SUBROUTINE PIT_COMPARE

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=======================================================!

SUBROUTINE PIT_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN(UNIT=12, FILE=SOCBEN_PIT,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_PIT
        WRITE(15,'("Could not find ", a100)') SOCBEN_PIT
        PAUSE
        STOP
    END IF
    OPEN(UNIT=13, FILE=EVAL_PIT,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') EVAL_PIT
        WRITE(15,'("Could not find ", a100)') EVAL_PIT
        PAUSE
        STOP
    END IF

    DO pers_j=1,n_rec
        WRITE(12,'(A14,F15.2,F15.2,F15.2,F15.2,F15.2)') GECODEERD_INSZ_CHAR(pers_j),PIT_FINTAX_ANN(pers_j),&
        &PIT_NTOI_ANN_SP1(pers_j),PIT_NTOI_ANN_SP2(pers_j),PIT_NTI_ANN_SP1(pers_j),PIT_NTI_ANN_SP2(pers_j) 
        WRITE(13,'(A14,I10,I10,F15.2,F15.10)') GECODEERD_INSZ_CHAR(pers_j),PIT_TURANK_ANN(pers_j),PIT_TUNUMB_ANN(pers_j),&
        &PIT_FINTAX_ANN(pers_j),TAXUNIT_WEIGHT(pers_j) 
    END DO

    CLOSE(UNIT=12)
    CLOSE(UNIT=13)

    WRITE(*,*) 'Subroutine PIT_OUTPUT_FILEWRIT of NORMAL RUN..........runned correctly'

END SUBROUTINE PIT_OUTPUT_FILEWRIT

!=================================================================================!
!Routine to write PIT variables for pre run
!=================================================================================!

SUBROUTINE WRITEPRE_PIT

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_PRERUN.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,I10)') GECODEERD_INSZ_CHAR(pers_j),PIT_DEPCHILD_ANN(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_PIT...............................runned correctly'

END SUBROUTINE WRITEPRE_PIT

!=================================================================================!
!Routine to write PIT variables for labsupply
!=================================================================================!

SUBROUTINE WRITELAB_PIT

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_LABSUPPLY.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,I10,I10,F15.10)') GECODEERD_INSZ_CHAR(pers_j),PIT_TURANK_ANN(pers_j),PIT_TUNUMB_ANN(pers_j),&
        &TAXUNIT_WEIGHT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_PIT...............................runned correctly'

END SUBROUTINE WRITELAB_PIT

!===================================================================================!
!With subroutine WRITE_ESSPROS we write out the data required for part of the
!computation of the requested ESSPROS data for EUROSTAT
!===================================================================================!

SUBROUTINE WRITE_ESSPROS

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCTOT  
    REAL(KIND=KIND_REAL),DIMENSION(6)                                   :: PIT_REP       !sum of replacement incomes
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_ESSPROS.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        !Compute total amount of gross taxable income
        INCTOT =  PIT_GTIPENS_ANN_SP1(pers_j)+PIT_GTIEARNE_ANN_SP1(pers_j)+PIT_GTIEAROL_ANN_SP1(pers_j)&
                &+PIT_GTIOLDUN_ANN_SP1(pers_j)+PIT_GTIOTHUN_ANN_SP1(pers_j)+PIT_GTIDISAB_ANN_SP1(pers_j)&
                &+PIT_GTINDOCC_ANN_SP1(pers_j)+PIT_GTISICK_ANN_SP1(pers_j)+PIT_GTIPRIV_ANN_SP1(pers_j)&
                &+PIT_GTIPUB_ANN_SP1(pers_j)+PIT_GTISELF_ANN_SP1(pers_j)&
                &+PIT_GTIPENS_ANN_SP2(pers_j)+PIT_GTIEARNE_ANN_SP2(pers_j)+PIT_GTIEAROL_ANN_SP2(pers_j)&
                &+PIT_GTIOLDUN_ANN_SP2(pers_j)+PIT_GTIOTHUN_ANN_SP2(pers_j)+PIT_GTIDISAB_ANN_SP2(pers_j)&
                &+PIT_GTINDOCC_ANN_SP2(pers_j)+PIT_GTISICK_ANN_SP2(pers_j)+PIT_GTIPRIV_ANN_SP2(pers_j)&
                &+PIT_GTIPUB_ANN_SP2(pers_j)+PIT_GTISELF_ANN_SP2(pers_j)

        !Compute whether there are taxable replacement incomes
        !PIT_REP_ANN1: pension benefits and early retirement benefits of new type
        !PIT_REP_ANN2: early retirement benefits of old type
        !PIT_REP_ANN3: normal unemployment benefits
        !PIT_REP_ANN4: unemployment benefits of older unemployed (age of 58 or above)
        !PIT_REP_ANN5: sickness and disability benefits
        !PIT_REP_ANN6: other replacement income
        PIT_REP(1:6) = 0.D0
        PIT_REP(1) = (PIT_NTIPENS_ANN_SP1(pers_j)  + PIT_NTIEARNE_ANN_SP1(pers_j) + &
                     &PIT_NTIPENS_ANN_SP2(pers_j)  + PIT_NTIEARNE_ANN_SP2(pers_j))
        PIT_REP(2) = (PIT_NTIEAROL_ANN_SP1(pers_j) + PIT_NTIEAROL_ANN_SP2(pers_j))
        PIT_REP(3) = (PIT_NTIOTHUN_ANN_SP1(pers_j) + PIT_NTIOTHUN_ANN_SP2(pers_j))
        PIT_REP(4) = (PIT_NTIOLDUN_ANN_SP1(pers_j) + PIT_NTIOLDUN_ANN_SP2(pers_j))
        PIT_REP(5) = (PIT_NTIDISAB_ANN_SP1(pers_j) + PIT_NTISICK_ANN_SP1(pers_j) + &
                     &PIT_NTIDISAB_ANN_SP2(pers_j) + PIT_NTISICK_ANN_SP2(pers_j) + &
                     &PIT_NTINDOCC_ANN_SP1(pers_j) + PIT_NTINDOCC_ANN_SP2(pers_j))
        PIT_REP(6) = 0

        !Select a number of additional variables required for the computations
        !PIT_TUNOLAB_ANN    !Indicator whether the tax unit is liable to enter a tax form yes or no (no = 1, yes = 0)
        !PIT_TURANK_ANN     !Rank of the individual within the tax unit the individual belongs to
        !TAXUNIT_WEIGHT     !Vector with weights for tax units
        !PIT_FINTAX_ANN     !Sum of all personal incomes taxes paid by the tax unit   
        
        WRITE(12,'(A14,7F15.2,2I10,F15.10,F15.2)') GECODEERD_INSZ_CHAR(pers_j),INCTOT,PIT_REP(1:6),&
        &PIT_TUNOLAB_ANN(pers_j),PIT_TURANK_ANN(pers_j),TAXUNIT_WEIGHT(pers_j),PIT_FINTAX_ANN(pers_j) 
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_ESSPROS..............................runned correctly'

END SUBROUTINE WRITE_ESSPROS

!===================================================================================!
!With subroutine 
!===================================================================================!

SUBROUTINE WRITE_SPECONT

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_SPECCONT.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,F15.2)') GECODEERD_INSZ_CHAR(pers_j),PIT_SPECSSCO_ANN(pers_j) 
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_SPECONT..............................runned correctly'

END SUBROUTINE WRITE_SPECONT

!===================================================================================!
!With subroutine 
!===================================================================================!

SUBROUTINE WRITE_PITRKW

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCTOT  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_PITRKW.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        INCTOT = PIT_GPIT_ANN_SP1(pers_j)+PIT_GPIT_ANN_SP2(pers_j)
        WRITE(12,'(A14,3F15.2)') GECODEERD_INSZ_CHAR(pers_j),PIT_FINTAX_ANN(pers_j),INCTOT,PIT_PITFAMC_ANN(pers_j) 
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_PITRKW..............................runned correctly'

END SUBROUTINE WRITE_PITRKW

!===================================================================================!
!With subroutine 
!===================================================================================!

SUBROUTINE WRITE_PITSOCB

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCTOT  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_SOCBEL.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,2F15.2)') GECODEERD_INSZ_CHAR(pers_j),PIT_NTI_ANN_SP1(pers_j),PIT_NTI_ANN_SP2(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_PITRKW..............................runned correctly'

END SUBROUTINE WRITE_PITSOCB

!===================================================================================!
!With subroutine 
!===================================================================================!

SUBROUTINE WRITE_QUANTNUM

    INTEGER :: pers_j
    REAL(KIND=DPRECI)                                                   :: INCTOT  
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\PIT_QUANTNUM.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        WRITE(12,'(A14,2I10)') GECODEERD_INSZ_CHAR(pers_j),PIT_TUNUMB_ANN(pers_j),QNR_TNTI(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_WRITE_QUANTNUM..........................runned correctly'

END SUBROUTINE WRITE_QUANTNUM

!==========================================================!
!The subroutine PIT_CALC is used to call the other         ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE PIT_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS PIT IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=PIT_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') PIT_LOG
        WRITE(15,'("Could not find ", a100)') PIT_LOG
        PAUSE
        STOP
    END IF

    !Allocate all exogenous vectors with dimension n_rec 
    CALL PIT_ALLOC_EXOG(n_rec)
    !Reads in PARAMETER file
    CALL PIT_PARAMETER_FILEREAD
    !Read in files with input data
    CALL PIT_INPUT_FILEREAD
    CALL PIT_CONTRIB_FILEREAD
    CALL PIT_FAMREL_FILEREAD
    CALL PIT_WAGE_FILEREAD
    CALL PIT_WEIGHT_FILEREAD

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec 
    CALL PIT_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL PIT_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL PIT_RANKANDSORT

    !Recode missing values
    CALL PIT_RECODE
    !Inflate values
    CALL PIT_INFLATE

    !Determine household size  
    CALL PIT_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL PIT_AGE

    !Construction of tax units
    CALL PIT_TUSTRUCT_POTENT    !Construct potential tax units
    CALL PIT_NETOWN_CONSTRUC    !Construct net own means of each individual
    CALL PIT_TUSTRUCT_FINAL     !Check net own means conditions
    CALL PIT_TUVAR_CREATE       !Create tax units

    !Computation of personal income tax
    CALL PIT_TUNTI_CREATE       !Compute net taxable income for each spouse
    CALL PIT_GPIT_CREATE        !Compute gross amount of personal income taxes
    CALL PIT_TCFAMCOM_CREATE    !Compute tax credits because of family composition
    IF (SIMYEAR.LE.2003) THEN
        CALL PIT_TCREPINC_CREATE_2001    !Compute tax credits because of replacement income
    ELSE IF (SIMYEAR.GT.2003.AND.SIMYEAR.LT.2010) THEN
        CALL PIT_TCREPINC_CREATE_2004    !Compute tax credits because of replacement income
    ELSE
        CALL PIT_TCREPINC_CREATE_2010    !Compute tax credits because of replacement income
    END IF
    CALL PIT_PITLIAB_CREATE     !Check whether the tax unit is liable to pay income taxes (!! should come before PIT_OTHPIT_CREATE)
    CALL PIT_OTHPIT_CREATE      !Other interventions to determine final tax bill

    !Compute weights to weight tax units
    CALL PIT_WEIGHT_CREATE(PIT_TUNUMB_ANN,PIT_TURANK_ANN,PIT_COUPLE_ANN,PIT_TUNOLAB_ANN,n_rec,TAXUNIT_WEIGHT)

    !calculate the decilenumbers on the basis of taxable income by calling subroutine PIT_QUANTILENR, which gives the decilenumbers in decnr(VecSize)
    !the vectors PIT_TNTI_ANN, TAXUNIT_WEIGHT, PIT_TURANK_ANN and PIT_TUNOLAB_ANN are not changed by the call (it means: remain unordered)
    CALL PIT_QUANTILENR(PIT_TNTI_ANN,TAXUNIT_WEIGHT,PIT_TURANK_ANN,PIT_TUNOLAB_ANN,n_rec,NUMBDEC-1,QVAL_TNTI,QNR_TNTI)

    !test construction of PIT constructions
    CALL PIT_TEST
    CALL PIT_AGECLASS_CREATION !Create age in classes
    CALL PIT_COMPARE

    !Write output to output file
    CALL PIT_OUTPUT_FILEWRIT
    !Write file with PIT variables for pre run
!    CALL WRITEPRE_PIT
!    CALL WRITELAB_PIT
!    CALL WRITE_ESSPROS
!    CALL WRITE_SPECONT
!    CALL WRITE_PITRKW
!    CALL WRITE_PITSOCB
!    CALL WRITE_QUANTNUM

    !Deallocate allocated variables
    CALL PIT_DEALLOC
    CLOSE(15)

END SUBROUTINE PIT_CALC

END MODULE PIT_SUBROUTINES
