!===================================================================!
!  Record of revisions:                                             !
!                                                                   !
!    Date      Programmer               Description of changes      !
!  ========    ====================     ======================      !
!  01/03/06    Consortium AG/01/086     Original code               !
!  01/10/11    FPS Social Security      Revision                    !
!===================================================================!

!==========================================================!
! This module declare all variables used in the program    !
!  - Parameters                                            !
!  - Exogenous Variables                                   !
!  - Endogeneous Variables                                 !
!==========================================================!

MODULE UNEM_DATA

    !========================================================================!
    ! Database specific parameters:                                          !
    ! number of lines in parameters file                                     !
    INTEGER, PARAMETER::n_par=269
    ! year of database                                                       !
    INTEGER, PARAMETER::MIMOSIS_YEAR=2008
    !(first quarter=1, second quarter=4, third quarter=9, fourth quarter=12) !
    INTEGER, PARAMETER::month=12                                             !
    !Define kind of Integer*4 in a portable way                              !                       
    INTEGER, PARAMETER :: INT4 = SELECTED_INT_KIND(18)                       !                       
    !Define Double precision, i.e. REAL*8 in a portable way                  !
    INTEGER, PARAMETER :: DPRECI = SELECTED_REAL_KIND(15,307)                !
    !Define Single precision, i.e. REAL*4 in a portable way                  !
    INTEGER, PARAMETER :: SPRECI = SELECTED_REAL_KIND(6,37)                  !
    !Define parameter that stores the precision value used for large vectors !
    INTEGER, PARAMETER :: KIND_REAL = SPRECI                                 !   
    !==========================================================================

    !==========================================================================
    !Define variables to read locations and input parameters
    !Variables for locating input 
    CHARACTER(len=150)                                      :: UNEM_INPUT_PATH                      !Path name where file with UNEM input data is stored
    CHARACTER(len=150)                                      :: UNEM_INPUT_FILE                      !File name of file with UNEM input data
    CHARACTER(len=150)                                      :: UNEM_INPUT                           !Internal name to store Path+Name of file with UNEM input data
    CHARACTER(len=150)                                      :: UNEM_CONTRIB_FILE                    !File name of file with CONTRIB data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_CONTRIB                         !Internal name to store Path+Name of file with CONTRIB data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_SICK_FILE                       !File name of file with SICK data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_SICK                            !Internal name to store Path+Name of file with SICK data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_PIT_FILE                        !File name of file with PIT data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_PIT                             !Internal name to store Path+Name of file with PIT data for the UNEM module 
    CHARACTER(len=150)                                      :: UNEM_PENSWELF_FILE                   !File name of file with PENSWELF data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_PENSWELF                        !Internal name to store Path+Name of file with PENSWELF data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_FAMAL_FILE                      !File name of file with FAMAL data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_FAMAL                           !Internal name to store Path+Name of file with FAMAL data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_UNEM_FILE                       !File name of file with UNEM data for the UNEM module
    CHARACTER(len=150)                                      :: UNEM_UNEM                            !Internal name to store Path+Name of file with UNEM data for the UNEM module
    CHARACTER(len=150)                                      :: FAMREL_INPUT_PATH                    !Path name where file with FAMREL input data is stored
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL_FILE                  !File name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: MIMOSIS_FAMREL                       !Internal name to store Path+Name of file with FAMREL data (=output of the FAMREL module)
    CHARACTER(len=150)                                      :: CONSTANT_INPUT_PATH                  !Path name where file with MIMOSIS Weights is stored
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG_FILE                !File name of file with MIMOSIS Weights                    
    CHARACTER(len=150)                                      :: MIMOSIS_SAMPWEIG                     !Internal name to store Path+Name of file with MIMOSIS Weights
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT_FILE                  !File name of file with MIMOSIS gross wages
    CHARACTER(len=150)                                      :: WAGE_CONSTRUCT                       !Internal name to store Path+Name of file with MIMOSIS gross wages
    !Parameters for locating output (local variables)
    CHARACTER(len=150)                                      :: CONTRIB_UNEM_FILE                    !File name of file with UNEM data for the CONTRIB module
    CHARACTER(len=150)                                      :: CONTRIB_UNEM                         !Internal name to store Path+Name of file with UNEM data for the CONTRIB module
    CHARACTER(len=150)                                      :: SOCBEN_UNEM_FILE                     !File name of file with UNEM data for the SOCBEN module
    CHARACTER(len=150)                                      :: SOCBEN_UNEM                          !Internal name to store Path+Name of file with UNEM data for the SOCBEN module
    !Tools to read input, output and parameters    
    CHARACTER(len=20)                                       :: UNEM_PAR_FORMAT                      !Format of parameter read in from parameter file
    CHARACTER(len=1)                                        :: UNEM_PAR_BREAK='@'                   !Character used to identify end of string
    CHARACTER(len=10)                                       :: UNEM_PAR_CURR                        !Currency of the parameter read in from the parameter file
    CHARACTER(len=40),ALLOCATABLE,DIMENSION(:)              :: UNEM_PAR_NAMES                       !Vector of parameter names
    INTEGER                                                 :: UNEM_PAR_L                           !Number of lines (if parameter is a matrix)_
    INTEGER                                                 :: UNEM_PAR_C                           !Number of columnss (if parameter is a matrix)
    INTEGER,DIMENSION(15)                                   :: UNEM_PAR_BREAK_POINT                 !number of break points to be determined when combining path names and file names
    !=======================================================================!
    !Parameters of the UNEM module
    INTEGER,ALLOCATABLE,DIMENSION(:,:)                      :: UNEM_FAMREL_QT                       !Matrix used to convert relationships of family members
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ1_QT                     !Ceiling used to exempt net wages of spouse
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ2_QT                     !Ceiling used to exempt gross professional income of children
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ3_QT                     !Ceiling used to exempt unemployment benefits and sickness and disability benefits, other than benefits obtained because of industrial accidents or occupational diseases of children
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ4_QT                     !Ceiling used to exempt pensions of antecedents in case the unemployed has cohabitating children
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ5_QT                     !Ceiling used to exempt pensions of antecedents in case the unemployed has no cohabitating children
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ6_QT                     !Ceiling used to exempt other pensions and benefits obtained because of industrial accidents or occupational diseases
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ7_QT                     !Ceiling used to exempt not else classified replacement or professional income
    REAL(KIND=KIND_REAL)                                    :: UNEM_WAGEDFQ8_QT                     !Ceiling used to determine whether the partner is a privileged partner (i.e. low income)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_LUSUWB_DAY                      !Lump sum benefits paid per day as waiting benefit for unemployed entitled to unemployment benefits on the basis of studies
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_LIMLOSW_DAY                     !Ceiling used to limit lost daily wages for unemployed entitled to unemployment benefits on the basis of employment
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_LIMLOSW_REF_DAY                 !Ceiling used to limit lost daily wages to compute reference unemployment benefits
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_PERLIMUB_DAY                    !Percentages and ceilings used to determine daily benefits for unemployed entitled to unemployment benefits on the basis of employment
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_PERLIMUB_REF_DAY                !Percentages and ceilings used to determine daily benefits for unemployed for which a reference unemployment benefit is computed
    INTEGER                                                 :: UNEM_DUURP                           !Maximum number of months in unemployment for which the unemployed can receive a benefit
    REAL(KIND=KIND_REAL)                                    :: UNEM_LSCUB1_DAY                      !Lump sum benefit of cohabitating unemployed who does not have a privileged partner
    REAL(KIND=KIND_REAL)                                    :: UNEM_LSCUB2_DAY                      !Additional lump sum benefit if the cohabitating unemployed has a privileged partner
    REAL(KIND=KIND_REAL)                                    :: UNEM_LIMWAGE_QT                      !Ceiling used to limit the lost gross wages per quarter in the computation of early retirement benefits for the RVA benefit
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_EARPAR_QT                       !Percentage and ceilings used to determine early retirement benefits
    REAL(KIND=KIND_REAL)                                    :: UNEM_LIMEARR_QT                      !Ceiling used to limit the lost gross wages per quarter in the computation of early retirement benefits paid by the employer
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENCCB_MONTH                    !Monthly lump sum benefits used to determine common career break benefits for employees who leave a full time job
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPCARE_MONTH                  !Monthly lump sum benefits used to determine career break benefits in case of palliative care for employees who leave a full time job
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENMCARE_MONTH                  !Monthly lump sum benefits used to determine career break benefits in case of medical assistance for employees who leave a full time job
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPLEAV_MONTH                  !Monthly lump sum benefits used to determine career break benefits in case of parental leave for employees who leave a full time job
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENSSUP_DAY                     !Percentages and ceilings used to determine daily unemployment benefits of unemployed entitled to a seniority supplement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_EXEMPSF_DAY                     !Lump sum benefits paid per day in case of exemption from certain obligations because of social and familial reasons
    REAL(KIND=KIND_REAL)                                    :: UNEM_LSPEAR_DAY                      !sum benefit paid per day in case of part time early retirement
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENCCM50_MONTH                  !Monthly lump sum benefits used to determine common part time career break benefits for employees younger than 50
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENCCP50_MONTH                  !Monthly lump sum benefits used to determine common part time career break benefits for employees who are 50 or older
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPCM50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of palliative care for employees younger than 50
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPCP50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of palliative care for employees who are 50 or older
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENMCM50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of medical assistance for employees younger than 50
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENMCP50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of medical assistance for employees who are 50 or older
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPLM50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of parental leave for employees younger than 50
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_BENPLP50_MONTH                  !Monthly lump sum benefits used to determine part time career break benefits in case of parental leave for employees who are 50 or older
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_GUARINC_MONTH                   !Monthly lump sum amounts and Percentages used to compute the guaranteed income benefits in case of unemployment
    REAL(KIND=KIND_REAL)                                    :: UNEM_PWAMIN_HOUR                     !Hourly lump sum amount used to reduce the common unemployment benefit in case of PWA activation
    REAL(KIND=KIND_REAL)                                    :: UNEM_PWAPLUS_HOUR                    !Hourly lump sum amount used to determine the PWA supplement 
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_TRANSBEN_MONTH                  !Monthly lump sum benefits for employees in a transition program
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_REINBEN_MONTH                   !Monthly lump sum benefits for employees in a reinsertion program
    REAL(KIND=KIND_REAL)                                    :: UNEM_SSCONT_QT                       !Percentage used to determine social security contributions paid on gross income
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_COSTSCAL_ANN                    !Percentages and thresholds used to determine professional costs in the prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:,:)         :: UNEM_TAXRATES_ANN                    !Tax rates and thresholds used to determine annual taxes in the prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_REDCHILD_ANN                    !Tax credit for children in prepayment computation
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_LSINGUAR_HOUR                   !Lump sum amount paid per hour for those combining their unemployment benefit with work
    !=======================================================================!  
    !Exogenous variables
    !UNEM_INPUT_Write.sas
    CHARACTER(len=14),ALLOCATABLE,DIMENSION(:)              :: GECODEERD_INSZ_CHAR                  ! Geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_INPUT                 ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_HH_ID                       ! Geanonimiseerd identificatienummer van het gezin waartoe het individu behoort
    CHARACTER(len=7),ALLOCATABLE,DIMENSION(:)               :: NAREGNIS_DATBIRTH                    ! Geboortedatum van het individu in jaar en maand
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NAREGNIS_SEX                         ! Geslacht van het individu op 1 januari 2009
    !/*RVA*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FICHE7_2008                          ! Typering werlkoosheid (statuut persoon bij RVA) (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DAGEN_2008                           ! Aantal dagen waarvoor een werkloosheidsuitkering wordt ontvangen (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: DGNDMND_2008                         ! Bedrag van de dagvergoeding die aan de werkloze toegekend wordt (2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: BEDRAGEN_2008                        ! bedrag van de werkloosheidsuitkering ontvangen tijdens het kwartaal (2008)
    CHARACTER(len=1),ALLOCATABLE,DIMENSION(:)               :: REDENLO_2008                         ! Reden van loopbaanonderbreking (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: DUUR_2008                            ! Werkloosheidsduur (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: URENPWA_2008                         ! Aantal uren gewerkt in het kader van PWA gedurende het kwartaal (2008)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InRVA                                ! Individu geselecteerd uit basisdata RVA voor verwerking steekproef
    !/*FOD SZ*/
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: InFODSZ                              ! Individu geselecteerd uit n van de drie basisbestanden FODSZ voor verwerking steekproef
    !Variables read from file UNEM_CONTRIB.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_CONTRIB               !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPRIV_QT                   !Sum of gross labour income and holiday earnings, earned as wage earner on the private labour market in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_REVPUB_QT                    !Sum of gross labour income and holiday earnings, earned as wage earner on the public labour market in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_INCSELF_QT                   !Gross income earned as self employed in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPRIV_EMEE_QT               !Social security contributions paid on CONTRIB_REVPRIV_QT in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSPUB_EMEE_QT                !Social security contributions paid on CONTRIB_REVPUB_QT in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_SSSELF_EMER_QT               !Social security contributions paid on CONTRIB_INCSELF_QT in a given quater
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PRIV_QT             !Labour market status of a wage earner on the private labour market          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: CONTRIB_LABMSTAT_PUB_QT              !Labour market status of a wage earner on the public labour market           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: CONTRIB_WORKHOUR                     !Vector with hours that the individual is assumed to work per week
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: MIMOSIS_SIMNUMMT                     !Unique identification number identifying the position of the individual in the household for the computations of the marginal taxes
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WAGEPRED                     !Wages predicted with wage equation (estimated with data based on real hours worked)
    !Variables read from file UNEM_SICK.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_SICK                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_BENINOC_QT                      !Gross amount of benefits for industrial accidents and occupational diseases in a given quater
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SICK_NONEINOC_QT                     !Gross amount of sickness and disability benefits other than benefits for industrial accidents and occupational diseases in a given quater
    !Variables read from file UNEM_PIT.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PIT                   !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: PIT_DEPCHILD_ANN                     !Number of dependent children in the tax unit of the individual in a given year
    !Variables read from file UNEM_PENSWELF.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_PENSWELF              !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: PENSWELF_AMOUNTP_QT                  !Gross amount of pensions received in a given quater
    !Variables read from file UNEM_FAMAL.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMAL                 !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: FAMAL_AMOUNTF_QT                     !Gross amount of family allowances received in a given quater by the recipient
    !Variables read from file UNEM_UNEM.txt
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_UNEM                  !Anonymised identification number of the individual
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNPR_QT                      !Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage) estimated before unemployment benefits are computed
    !Variables read from file MIMOSIS_FAMREL.txt                                                  
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_FAMREL                !Anonymised identification number of the individual
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMILY                        !Unique identifier identifying the family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_RELATION                      !Relation of the individual with respect to the head of the family
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_FAMTYPE                       !Type of family to which the individual belongs
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: FAMREL_COUPLE                        !Type of couple to which the individual belongs
    !WAGE_CONSTRUCT_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WAGE                  ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: SALJRS_LAST_CORR                     ! Dagloon in laatste jaar met positieve waarde gecorrigeerd dagloon (in prijzen van 2008)
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_HOUR                   ! Uurloon gecorrigeerd voor minimum onder minimumgrens
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_GRINC_QT                     ! Raming van het kwartaalinkomen in het laatst geobserveerde kwartaal
    !MIMOSIS_SAMPWEIG_Write.sas
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: GECODEERD_INSZ_WEIGHT                ! Numeriek deel van het geanonimiseerd identificatienummer van het individu
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: MIMOSIS_WEIGHT                       ! Gewicht dat corrigeert voor niet random selectie van de observatie-eenheden
    !=======================================================================!
    !Endogeneous variables
    !to rank variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: NEW_RANK                             !New rank after sorting and re-ranking       
    !to construct age variables
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_YEAR_AGE                        !Age of the individual expressed in years
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_MONTH_AGE                       !Age of the individual expressed in moths
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_MONTH_BIRTH                     !Month of birth(mm)
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_YEAR_BIRTH                      !Year of birth (yyyy)
    !to construct information at the level of the sociological household
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_HH_SIZE                         !Variable stores the size of the household (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_START_HH                        !Variable stores an identification of the first household member (maximal size vector = number of individuals)   
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_END_HH                          !Variable stores an identification of the last household member (maximal size vector = number of individuals)   
    INTEGER                                                 :: N_HH_UNEM                            !number of households analysed
    !To construct variables listed in the module note
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_STATUS_QT                       !Indicator whether an individual is entitled to any sort of benefit paid by the RVA          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FAMSIT_QT                       !Family charge position of the unemployed            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_REL_QT                          !Relationship of the individual with respect to other family members         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_NETWEARN_QT                     !Net income earned as wage earner            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_NETSELF_QT                      !Net income earned as self employed          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_DEPINC_QT                       !Variable containing an indication whether an individual is income dependent yes or no           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_CHILINHH_QT                     !Unemployed cohabits with children yes or no         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_PRIVICOH_QT                     !The unemployed has a privileged partner yes or no           
    !Unemployment benefits of unemployed in search of work           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_PERMIS_QT                       !Unemployed is permitted to the system of unemployment benefits on the basis of studies or on the basis of previous employment           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGUIS_QT                      !Individual satisfies the eligibility conditions for unemployment benefits on the basis of studies           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENSTUD_DAY                     !Unemployment benefits per day for unemployed entitled to benefits after studies         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENSTUD_QT                      !Unemployment benefits per quarter for unemployed entitled to benefits after studies         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMPL_DAY                     !Unemployment benefits per day for unemployed entitled to benefits after employment          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMPL_QT                      !Unemployment benefits per quarter for unemployed entitled to benefits after employment          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_DISABLED_QT                     !Unemployed is disabled yes or no            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_NUMMINU_QT                      !Number of months the unemployed is already in unemployment          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_NUMYWAGE_QT                     !The number of years the unemployed has worked as wage earner            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_DECREARN_QT                     !The decrease in earning capacity if the unemployed would be on the labour market            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_AVGLWAGE_DAY                    !The last daily wage earned before entering unemployment         
    !Reference unemployment benefits of unemployed not in search of work or employees entitled to RVA benefits         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENREF_DAY                      !Reference unemployment benefits per day for unemployed not in search of work or employees entitled to RVA benefits computed          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENREF_QT                       !Reference unemployment benefits per quarter for unemployed not in search of work or employees entitled to RVA benefits
    !Unemployment benefits of unemployed not in search of work who are entitled to RVA-ONEM benefits             
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGUNS1_QT                     !Unemployed is not in search of work and is entitled to early retirement benefits either of the old or the new type          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_LOSWAGE_QT                      !The last quarterly wage earned before entering unemployment         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS1_QT                      !Quarterly early retirement benefit paid by the RVA to an unemployed not in search of work           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_ADDUNS_QT                       !Quarterly early retirement benefit paid by the previous employer to an unemployed not in search of work         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_PREPAY_QT                       !Prepayments paid on labour market income            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGUNS2_QT                     !Unemployed is not in search of work and is entitled to career break benefits            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FPUNS_QT                        !Type of job left by the unemployed not in search of work (i.e. either full or part time)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_CARBCON_QT                      !Qualification of the career break of the unemployed not in search of work (i.e. number of months and supplement)            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS2_QT                      !Quarterly career break benefit paid to unemployed not in search of work         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGUNS3_QT                     !Unemployed is not in search of work and is entitled to benefits for older unemployed with seniority supplement          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS3_DAY                     !Benefit per day for unemployed not in search of work who is entitled to benefits for older unemployed with seniority supplement         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS3_QT                      !Benefit per quarter for unemployed not in search of work who is entitled to benefits for older unemployed with seniority supplement         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGUNS4_QT                     !Unemployed is not in search of work and is entitled to unemployment benefits in case of exemptions          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUNS4_QT                      !Benefit per quarter for unemployed not in search of work who is entitled to benefits in case of exemption           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_EXEMP_QT                        !Number of months the unemployed is already exempted from certain obligations            
    !Unemployment benefits of employees entitled to benefits paid by the RVA-ONEM                                 
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP1_QT                     !Employee is entitled to part time early retirement benefits         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP1_QT                      !Quarterly early retirement benefit paid by the RVA to an employee entitled to part time early retirement benefits           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_ADDEMP_QT                       !Quarterly early retirement benefit paid by the employer to an employee entitled to part time early retirement benefits          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP2_QT                     !Employee is entitled to part time career break benefits         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP2_QT                      !Quarterly career break benefits paid to employee entitled to part time career break benefits            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FPEMP_QT                        !Type of job left by the employee entitled to part time career break benefits (i.e. either full or part time)            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_TIMEUNS_QT                      !Reduction in work time of the employee entitled to part time career break benefits          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP3_QT                     !Employee is entitled to guaranteed income benefits paid by the RVA          
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP3_DAY                     !Guaranteed income benefit for a day paid by the RVA to an employee entitled to these benefits           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP3_QT                      !Guaranteed income benefit per quarter paid by the RVA to an employee entitled to these benefits         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP4_QT                     !Employee is entitled to temporary unemployment benefits         
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP4_QT                      !Quarterly benefits paid to employee entitled to temporary unemployment benefits         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP5_QT                     !Employee is entitled to PWA benefits            
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP5_DAY                     !PWA benefits paid per day           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP5_QT                      !Quarterly amount of PWA benefits            
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_ELIGEMP6_QT                     !Employee is entitled to activation benefits for other than PWA program          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FRACTRAN_QT                     !Time performed by the employee in an activation program, other than PWA, expressed as a fraction of a full time         
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_TOWNRATE_QT                     !Degree of unemployment in the town the employee in an activation program is living in           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_PWAHOURS_QT                     !Number of hours performed in PWA activity           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENEMP6_QT                      !Quarterly benefits paid to employee entitled to benefits for activation program other than PWA program          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_DUURREIN_QT                     !Number of months the employee is in a reinsertion program           
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FRACREIN_QT                     !Time performed by the employee in a reinsertion program, expressed as a fraction of a full time         
    !Prepayment variables                
    !Reconstructed exchange concepts             
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_OLDUNSS_QT                      !Unemployment benefits of older unemployed with seniority supplement, paid in the given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_OTHERUN_QT                      !Unemployment benefits, other than early retirement benefits and unemployment benefits for older unemployed with seniority supplement received in a given quarter
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_TOTBEN_QT                       !Gross amount of unemployment benefits           
    REAL(KIND=KIND_REAL),ALLOCATABLE,DIMENSION(:)           :: UNEM_BENUN_QT                        !Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage)          
    INTEGER,ALLOCATABLE,DIMENSION(:)                        :: UNEM_FULL6MON_QT                     !The number of months in unemployment if the unemployed is either full time unemployed or early retired (hence eligible for social supplement in child benefits)
    
    !Instruments
    INTEGER                                                 :: pers_j,hh_i,i,j,k
    INTEGER                                                 :: ierror

END MODULE UNEM_DATA

!==========================================!
! This module contains all the subroutines !
! and functions used in the UNEM MODULE    !
!==========================================!
!List of subroutine names
!SUBROUTINE UNEM_ALLOC_EXOG(DIM_ALLOC)
!SUBROUTINE UNEM_ALLOC_ENDO(DIM_ALLOC)
!SUBROUTINE UNEM_DEALLOC
!SUBROUTINE UNEM_INITENDO(DIM_ALLOC)
!SUBROUTINE UNEM_RANKANDSORT
!SUBROUTINE UNEM_PARAMETER_FILEREAD
!SUBROUTINE UNEM_INPUT_FILEREAD
!SUBROUTINE UNEM_CONTRIB_FILEREAD
!SUBROUTINE UNEM_SICK_FILEREAD
!SUBROUTINE UNEM_PIT_FILEREAD
!SUBROUTINE UNEM_PENSWELF_FILEREAD
!SUBROUTINE UNEM_FAMAL_FILEREAD
!SUBROUTINE UNEM_FAMREL_FILEREAD
!SUBROUTINE UNEM_WAGE_FILEREAD
!SUBROUTINE UNEM_WEIGHT_FILEREAD
!SUBROUTINE UNEM_CONSISTENCY_CHECK
!SUBROUTINE UNEM_HOUSEHOLD_SIZE
!SUBROUTINE UNEM_AGE
!SUBROUTINE UNEM_OUTPUT_FILEWRIT
!SUBROUTINE UNEM_TEST
!SUBROUTINE UNEM_RECODE
!List of functions
!FUNCTION UNEM_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV)         RESULT (OUTPUT_AMOUNT)
!FUNCTION UNEM_SUMAMOUNT_1COLUMN(STATUS1,NLINES_AMOUNT,MAT_AMOUNT,INDIV)                 RESULT (OUTPUT_AMOUNT)
!FUNCTION UNEM_PREPAY_COMPUTE(INPUT_AMOUNT,CHILDREN,INDIV)                                RESULT (OUTPUT_AMOUNT)
!FUNCTION UNEM_SELAMOUNT_2COLUMNS(STATUS1,STATUS2,STATUS3,NLINES_AMOUNT,MAT_AMOUNT,INDIV)  RESULT (OUTPUT_AMOUNT)

MODULE UNEM_SUBROUTINES

    USE UNEM_DATA
    USE MIMOSIS_DATA    
    !use routine for sorting and ranking data
    USE RANKING_SORTING_ROUTINES

    IMPLICIT NONE
    CONTAINS

!=====================================================!
!Allocate all exogenous vectors of dimension DIM_ALLOC
!=====================================================!

SUBROUTINE UNEM_ALLOC_EXOG(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !=======================================================================!  
    !Exogenous variables
    !UNEM_INPUT_Write.sas
    ALLOCATE(GECODEERD_INSZ_CHAR(DIM_ALLOC))
    ALLOCATE(GECODEERD_INSZ_INPUT(DIM_ALLOC))
    ALLOCATE(NAREGNIS_HH_ID(DIM_ALLOC))
    ALLOCATE(NAREGNIS_DATBIRTH(DIM_ALLOC))
    ALLOCATE(NAREGNIS_SEX(DIM_ALLOC))
    !/*RVA*/
    ALLOCATE(FICHE7_2008(DIM_ALLOC))
    ALLOCATE(DAGEN_2008(DIM_ALLOC))
    ALLOCATE(DGNDMND_2008(DIM_ALLOC))
    ALLOCATE(BEDRAGEN_2008(DIM_ALLOC))
    ALLOCATE(REDENLO_2008(DIM_ALLOC))
    ALLOCATE(DUUR_2008(DIM_ALLOC))
    ALLOCATE(URENPWA_2008(DIM_ALLOC))
    ALLOCATE(InRVA(DIM_ALLOC))
    !/*FOD SZ*/
    ALLOCATE(InFODSZ(DIM_ALLOC))
    !Variables read from file UNEM_CONTRIB.txt
    ALLOCATE(GECODEERD_INSZ_CONTRIB(DIM_ALLOC))
    ALLOCATE(CONTRIB_REVPRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_REVPUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_INCSELF_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPRIV_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSPUB_EMEE_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_SSSELF_EMER_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PRIV_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_LABMSTAT_PUB_QT(DIM_ALLOC))
    ALLOCATE(CONTRIB_WORKHOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_SIMNUMMT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WAGEPRED(DIM_ALLOC))
    !Variables read from file UNEM_SICK.txt
    ALLOCATE(GECODEERD_INSZ_SICK(DIM_ALLOC))
    ALLOCATE(SICK_BENINOC_QT(DIM_ALLOC))
    ALLOCATE(SICK_NONEINOC_QT(DIM_ALLOC))
    !Variables read from file UNEM_PIT.txt
    ALLOCATE(GECODEERD_INSZ_PIT(DIM_ALLOC))
    ALLOCATE(PIT_DEPCHILD_ANN(DIM_ALLOC))
    !Variables read from file UNEM_PENSWELF.txt
    ALLOCATE(GECODEERD_INSZ_PENSWELF(DIM_ALLOC))
    ALLOCATE(PENSWELF_AMOUNTP_QT(DIM_ALLOC))
    !Variables read from file UNEM_FAMAL.txt
    ALLOCATE(GECODEERD_INSZ_FAMAL(DIM_ALLOC))
    ALLOCATE(FAMAL_AMOUNTF_QT(DIM_ALLOC))
    !Variables read from file UNEM_UNEM.txt
    ALLOCATE(GECODEERD_INSZ_UNEM(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNPR_QT(DIM_ALLOC))
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    ALLOCATE(GECODEERD_INSZ_FAMREL(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMILY(DIM_ALLOC))
    ALLOCATE(FAMREL_RELATION(DIM_ALLOC))
    ALLOCATE(FAMREL_FAMTYPE(DIM_ALLOC))
    ALLOCATE(FAMREL_COUPLE(DIM_ALLOC))
    !WAGE_CONSTRUCT_Write.sas
    ALLOCATE(GECODEERD_INSZ_WAGE(DIM_ALLOC))
    ALLOCATE(SALJRS_LAST_CORR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_HOUR(DIM_ALLOC))
    ALLOCATE(MIMOSIS_GRINC_QT(DIM_ALLOC))
    !MIMOSIS_SAMPWEIG_Write.sas
    ALLOCATE(GECODEERD_INSZ_WEIGHT(DIM_ALLOC))
    ALLOCATE(MIMOSIS_WEIGHT(DIM_ALLOC))

END SUBROUTINE UNEM_ALLOC_EXOG

SUBROUTINE UNEM_ALLOC_ENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors

    !Endogeneous variables
    !to rank variables
    ALLOCATE(NEW_RANK(DIM_ALLOC))
    !to construct age variables
    ALLOCATE(UNEM_YEAR_AGE(DIM_ALLOC))
    ALLOCATE(UNEM_MONTH_AGE(DIM_ALLOC))
    ALLOCATE(UNEM_MONTH_BIRTH(DIM_ALLOC))
    ALLOCATE(UNEM_YEAR_BIRTH(DIM_ALLOC))
    !to construct information at the level of the sociological household
    ALLOCATE(UNEM_HH_SIZE(DIM_ALLOC))
    ALLOCATE(UNEM_START_HH(DIM_ALLOC))
    ALLOCATE(UNEM_END_HH(DIM_ALLOC))
    !To construct variables listed in the module note
    ALLOCATE(UNEM_STATUS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FAMSIT_QT(DIM_ALLOC))
    ALLOCATE(UNEM_REL_QT(DIM_ALLOC))
    ALLOCATE(UNEM_NETWEARN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_NETSELF_QT(DIM_ALLOC))
    ALLOCATE(UNEM_DEPINC_QT(DIM_ALLOC))
    ALLOCATE(UNEM_CHILINHH_QT(DIM_ALLOC))
    ALLOCATE(UNEM_PRIVICOH_QT(DIM_ALLOC))
    !Unemployment benefits of unemployed in search of work           
    ALLOCATE(UNEM_PERMIS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGUIS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENSTUD_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENSTUD_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMPL_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMPL_QT(DIM_ALLOC))
    ALLOCATE(UNEM_DISABLED_QT(DIM_ALLOC))
    ALLOCATE(UNEM_NUMMINU_QT(DIM_ALLOC))
    ALLOCATE(UNEM_NUMYWAGE_QT(DIM_ALLOC))
    ALLOCATE(UNEM_DECREARN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_AVGLWAGE_DAY(DIM_ALLOC))
    !Reference unemployment benefits of unemployed not in search of work or employees entitled to RVA benefits         
    ALLOCATE(UNEM_BENREF_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENREF_QT(DIM_ALLOC))
    !Unemployment benefits of unemployed not in search of work who are entitled to RVA-ONEM benefits             
    ALLOCATE(UNEM_ELIGUNS1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_LOSWAGE_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ADDUNS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_PREPAY_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGUNS2_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FPUNS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_CARBCON_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS2_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGUNS3_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS3_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS3_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGUNS4_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENUNS4_QT(DIM_ALLOC))
    ALLOCATE(UNEM_EXEMP_QT(DIM_ALLOC))
    !Unemployment benefits of employees entitled to benefits paid by the RVA-ONEM                                 
    ALLOCATE(UNEM_ELIGEMP1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP1_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ADDEMP_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGEMP2_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP2_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FPEMP_QT(DIM_ALLOC))
    ALLOCATE(UNEM_TIMEUNS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGEMP3_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP3_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP3_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGEMP4_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP4_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGEMP5_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP5_DAY(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP5_QT(DIM_ALLOC))
    ALLOCATE(UNEM_ELIGEMP6_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FRACTRAN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_TOWNRATE_QT(DIM_ALLOC))
    ALLOCATE(UNEM_PWAHOURS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENEMP6_QT(DIM_ALLOC))
    ALLOCATE(UNEM_DUURREIN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FRACREIN_QT(DIM_ALLOC))
    !Prepayment variables                
    !Reconstructed exchange concepts             
    ALLOCATE(UNEM_OLDUNSS_QT(DIM_ALLOC))
    ALLOCATE(UNEM_OTHERUN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_TOTBEN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_BENUN_QT(DIM_ALLOC))
    ALLOCATE(UNEM_FULL6MON_QT(DIM_ALLOC))

END SUBROUTINE UNEM_ALLOC_ENDO

SUBROUTINE UNEM_DEALLOC

    !=======================================================================!  
    !Exogenous variables
    !UNEM_INPUT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_CHAR)
    DEALLOCATE(GECODEERD_INSZ_INPUT)
    DEALLOCATE(NAREGNIS_HH_ID)
    DEALLOCATE(NAREGNIS_DATBIRTH)
    DEALLOCATE(NAREGNIS_SEX)
    !/*RVA*/
    DEALLOCATE(FICHE7_2008)
    DEALLOCATE(DAGEN_2008)
    DEALLOCATE(DGNDMND_2008)
    DEALLOCATE(BEDRAGEN_2008)
    DEALLOCATE(REDENLO_2008)
    DEALLOCATE(DUUR_2008)
    DEALLOCATE(URENPWA_2008)
    DEALLOCATE(InRVA)
    !/*FOD SZ*/
    DEALLOCATE(InFODSZ)
    !Variables read from file UNEM_CONTRIB.txt
    DEALLOCATE(GECODEERD_INSZ_CONTRIB)
    DEALLOCATE(CONTRIB_REVPRIV_QT)
    DEALLOCATE(CONTRIB_REVPUB_QT)
    DEALLOCATE(CONTRIB_INCSELF_QT)
    DEALLOCATE(CONTRIB_SSPRIV_EMEE_QT)
    DEALLOCATE(CONTRIB_SSPUB_EMEE_QT)
    DEALLOCATE(CONTRIB_SSSELF_EMER_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PRIV_QT)
    DEALLOCATE(CONTRIB_LABMSTAT_PUB_QT)
    DEALLOCATE(CONTRIB_WORKHOUR)
    DEALLOCATE(MIMOSIS_SIMNUMMT)
    DEALLOCATE(MIMOSIS_WAGEPRED)
    !Variables read from file UNEM_SICK.txt
    DEALLOCATE(GECODEERD_INSZ_SICK)
    DEALLOCATE(SICK_BENINOC_QT)
    DEALLOCATE(SICK_NONEINOC_QT)
    !Variables read from file UNEM_PIT.txt
    DEALLOCATE(GECODEERD_INSZ_PIT)
    DEALLOCATE(PIT_DEPCHILD_ANN)
    !Variables read from file UNEM_PENSWELF.txt
    DEALLOCATE(GECODEERD_INSZ_PENSWELF)
    DEALLOCATE(PENSWELF_AMOUNTP_QT)
    !Variables read from file UNEM_FAMAL.txt
    DEALLOCATE(GECODEERD_INSZ_FAMAL)
    DEALLOCATE(FAMAL_AMOUNTF_QT)
    !Variables read from file UNEM_UNEM.txt
    DEALLOCATE(GECODEERD_INSZ_UNEM)
    DEALLOCATE(UNEM_BENUNPR_QT)
    !Variables read from file MIMOSIS_FAMREL.txt                                           
    DEALLOCATE(GECODEERD_INSZ_FAMREL)
    DEALLOCATE(FAMREL_FAMILY)
    DEALLOCATE(FAMREL_RELATION)
    DEALLOCATE(FAMREL_FAMTYPE)
    DEALLOCATE(FAMREL_COUPLE)
    !WAGE_CONSTRUCT_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WAGE)
    DEALLOCATE(SALJRS_LAST_CORR)
    DEALLOCATE(MIMOSIS_GRINC_HOUR)
    DEALLOCATE(MIMOSIS_GRINC_QT)
    !MIMOSIS_SAMPWEIG_Write.sas
    DEALLOCATE(GECODEERD_INSZ_WEIGHT)
    DEALLOCATE(MIMOSIS_WEIGHT)

    !=======================================================================!  
    !Endogeneous variables
    !to rank variables
    DEALLOCATE(NEW_RANK)
    !to construct age variables
    DEALLOCATE(UNEM_YEAR_AGE)
    DEALLOCATE(UNEM_MONTH_AGE)
    DEALLOCATE(UNEM_MONTH_BIRTH)
    DEALLOCATE(UNEM_YEAR_BIRTH)
    !to construct information at the level of the sociological household
    DEALLOCATE(UNEM_HH_SIZE)
    DEALLOCATE(UNEM_START_HH)
    DEALLOCATE(UNEM_END_HH)
    !To construct variables listed in the module note
    DEALLOCATE(UNEM_STATUS_QT)
    DEALLOCATE(UNEM_FAMSIT_QT)
    DEALLOCATE(UNEM_REL_QT)
    DEALLOCATE(UNEM_NETWEARN_QT)
    DEALLOCATE(UNEM_NETSELF_QT)
    DEALLOCATE(UNEM_DEPINC_QT)
    DEALLOCATE(UNEM_CHILINHH_QT)
    DEALLOCATE(UNEM_PRIVICOH_QT)
    !Unemployment benefits of unemployed in search of work           
    DEALLOCATE(UNEM_PERMIS_QT)
    DEALLOCATE(UNEM_ELIGUIS_QT)
    DEALLOCATE(UNEM_BENSTUD_DAY)
    DEALLOCATE(UNEM_BENSTUD_QT)
    DEALLOCATE(UNEM_BENEMPL_DAY)
    DEALLOCATE(UNEM_BENEMPL_QT)
    DEALLOCATE(UNEM_DISABLED_QT)
    DEALLOCATE(UNEM_NUMMINU_QT)
    DEALLOCATE(UNEM_NUMYWAGE_QT)
    DEALLOCATE(UNEM_DECREARN_QT)
    DEALLOCATE(UNEM_AVGLWAGE_DAY)
    !Reference unemployment benefits of unemployed not in search of work or employees entitled to RVA benefits         
    DEALLOCATE(UNEM_BENREF_DAY)
    DEALLOCATE(UNEM_BENREF_QT)
    !Unemployment benefits of unemployed not in search of work who are entitled to RVA-ONEM benefits             
    DEALLOCATE(UNEM_ELIGUNS1_QT)
    DEALLOCATE(UNEM_LOSWAGE_QT)
    DEALLOCATE(UNEM_BENUNS1_QT)
    DEALLOCATE(UNEM_ADDUNS_QT)
    DEALLOCATE(UNEM_PREPAY_QT)
    DEALLOCATE(UNEM_ELIGUNS2_QT)
    DEALLOCATE(UNEM_FPUNS_QT)
    DEALLOCATE(UNEM_CARBCON_QT)
    DEALLOCATE(UNEM_BENUNS2_QT)
    DEALLOCATE(UNEM_ELIGUNS3_QT)
    DEALLOCATE(UNEM_BENUNS3_DAY)
    DEALLOCATE(UNEM_BENUNS3_QT)
    DEALLOCATE(UNEM_ELIGUNS4_QT)
    DEALLOCATE(UNEM_BENUNS4_QT)
    DEALLOCATE(UNEM_EXEMP_QT)
    !Unemployment benefits of employees entitled to benefits paid by the RVA-ONEM                                 
    DEALLOCATE(UNEM_ELIGEMP1_QT)
    DEALLOCATE(UNEM_BENEMP1_QT)
    DEALLOCATE(UNEM_ADDEMP_QT)
    DEALLOCATE(UNEM_ELIGEMP2_QT)
    DEALLOCATE(UNEM_BENEMP2_QT)
    DEALLOCATE(UNEM_FPEMP_QT)
    DEALLOCATE(UNEM_TIMEUNS_QT)
    DEALLOCATE(UNEM_ELIGEMP3_QT)
    DEALLOCATE(UNEM_BENEMP3_DAY)
    DEALLOCATE(UNEM_BENEMP3_QT)
    DEALLOCATE(UNEM_ELIGEMP4_QT)
    DEALLOCATE(UNEM_BENEMP4_QT)
    DEALLOCATE(UNEM_ELIGEMP5_QT)
    DEALLOCATE(UNEM_BENEMP5_DAY)
    DEALLOCATE(UNEM_BENEMP5_QT)
    DEALLOCATE(UNEM_ELIGEMP6_QT)
    DEALLOCATE(UNEM_FRACTRAN_QT)
    DEALLOCATE(UNEM_TOWNRATE_QT)
    DEALLOCATE(UNEM_PWAHOURS_QT)
    DEALLOCATE(UNEM_BENEMP6_QT)
    DEALLOCATE(UNEM_DUURREIN_QT)
    DEALLOCATE(UNEM_FRACREIN_QT)
    !Prepayment variables                
    !Reconstructed exchange concepts             
    DEALLOCATE(UNEM_OLDUNSS_QT)
    DEALLOCATE(UNEM_OTHERUN_QT)
    DEALLOCATE(UNEM_TOTBEN_QT)
    DEALLOCATE(UNEM_BENUN_QT)
    DEALLOCATE(UNEM_FULL6MON_QT)

    !=======================================================================!
    !Deallocate parameters
    DEALLOCATE(UNEM_FAMREL_QT)
    DEALLOCATE(UNEM_LUSUWB_DAY)
    DEALLOCATE(UNEM_LIMLOSW_DAY)
    DEALLOCATE(UNEM_LIMLOSW_REF_DAY)
    DEALLOCATE(UNEM_PERLIMUB_DAY)
    DEALLOCATE(UNEM_PERLIMUB_REF_DAY)
    DEALLOCATE(UNEM_EARPAR_QT)
    DEALLOCATE(UNEM_BENCCB_MONTH)
    DEALLOCATE(UNEM_BENPCARE_MONTH)
    DEALLOCATE(UNEM_BENMCARE_MONTH)
    DEALLOCATE(UNEM_BENPLEAV_MONTH)
    DEALLOCATE(UNEM_BENSSUP_DAY)
    DEALLOCATE(UNEM_EXEMPSF_DAY)
    DEALLOCATE(UNEM_BENCCM50_MONTH)
    DEALLOCATE(UNEM_BENCCP50_MONTH)
    DEALLOCATE(UNEM_BENPCM50_MONTH)
    DEALLOCATE(UNEM_BENPCP50_MONTH)
    DEALLOCATE(UNEM_BENMCM50_MONTH)
    DEALLOCATE(UNEM_BENMCP50_MONTH)
    DEALLOCATE(UNEM_BENPLM50_MONTH)
    DEALLOCATE(UNEM_BENPLP50_MONTH)
    DEALLOCATE(UNEM_GUARINC_MONTH)
    DEALLOCATE(UNEM_TRANSBEN_MONTH)
    DEALLOCATE(UNEM_REINBEN_MONTH)
    DEALLOCATE(UNEM_COSTSCAL_ANN)
    DEALLOCATE(UNEM_TAXRATES_ANN)
    DEALLOCATE(UNEM_REDCHILD_ANN)
    DEALLOCATE(UNEM_LSINGUAR_HOUR)
    
END SUBROUTINE UNEM_DEALLOC

!===============================================!
!Initialise endogenous variables                        
!===============================================!

SUBROUTINE UNEM_INITENDO(DIM_ALLOC)

    INTEGER                                                 :: DIM_ALLOC                            !dimension of allocated vectors
    INTEGER                                                 :: pers_j                                    !index for do loop

    DO pers_j=1,DIM_ALLOC
        !to rank variables
        NEW_RANK(pers_j) = 0
        !to construct age variables
        UNEM_YEAR_AGE(pers_j) = 0
        UNEM_MONTH_AGE(pers_j) = 0
        UNEM_MONTH_BIRTH(pers_j) = 0
        UNEM_YEAR_BIRTH(pers_j) = 0
        !to construct information at the level of the sociological household
        UNEM_HH_SIZE(pers_j) = 0
        UNEM_START_HH(pers_j) = 0
        UNEM_END_HH(pers_j) = 0
        !To construct variables listed in the module note
        UNEM_STATUS_QT(pers_j) = 0
        UNEM_FAMSIT_QT(pers_j) = 0
        UNEM_REL_QT(pers_j) = 0
        UNEM_NETWEARN_QT(pers_j) = 0
        UNEM_NETSELF_QT(pers_j) = 0
        UNEM_DEPINC_QT(pers_j) = 0
        UNEM_CHILINHH_QT(pers_j) = 0
        UNEM_PRIVICOH_QT(pers_j) = 0
        !Unemployment benefits of unemployed in search of work           
        UNEM_PERMIS_QT(pers_j) = 0
        UNEM_ELIGUIS_QT(pers_j) = 0
        UNEM_BENSTUD_DAY(pers_j) = 0
        UNEM_BENSTUD_QT(pers_j) = 0
        UNEM_BENEMPL_DAY(pers_j) = 0
        UNEM_BENEMPL_QT(pers_j) = 0
        UNEM_DISABLED_QT(pers_j) = 0
        UNEM_NUMMINU_QT(pers_j) = 0
        UNEM_NUMYWAGE_QT(pers_j) = 0
        UNEM_DECREARN_QT(pers_j) = 0
        UNEM_AVGLWAGE_DAY(pers_j) = 0
        !Reference unemployment benefits of unemployed not in search of work or employees entitled to RVA benefits         
        UNEM_BENREF_DAY(pers_j) = 0
        UNEM_BENREF_QT(pers_j) = 0
        !Unemployment benefits of unemployed not in search of work who are entitled to RVA-ONEM benefits             
        UNEM_ELIGUNS1_QT(pers_j) = 0
        UNEM_LOSWAGE_QT(pers_j) = 0
        UNEM_BENUNS1_QT(pers_j) = 0
        UNEM_ADDUNS_QT(pers_j) = 0
        UNEM_PREPAY_QT(pers_j) = 0
        UNEM_ELIGUNS2_QT(pers_j) = 0
        UNEM_FPUNS_QT(pers_j) = 0
        UNEM_CARBCON_QT(pers_j) = 0
        UNEM_BENUNS2_QT(pers_j) = 0
        UNEM_ELIGUNS3_QT(pers_j) = 0
        UNEM_BENUNS3_DAY(pers_j) = 0
        UNEM_BENUNS3_QT(pers_j) = 0
        UNEM_ELIGUNS4_QT(pers_j) = 0
        UNEM_BENUNS4_QT(pers_j) = 0
        UNEM_EXEMP_QT(pers_j) = 0
        !Unemployment benefits of employees entitled to benefits paid by the RVA-ONEM                                 
        UNEM_ELIGEMP1_QT(pers_j) = 0
        UNEM_BENEMP1_QT(pers_j) = 0
        UNEM_ADDEMP_QT(pers_j) = 0
        UNEM_ELIGEMP2_QT(pers_j) = 0
        UNEM_BENEMP2_QT(pers_j) = 0
        UNEM_FPEMP_QT(pers_j) = 0
        UNEM_TIMEUNS_QT(pers_j) = 0
        UNEM_ELIGEMP3_QT(pers_j) = 0
        UNEM_BENEMP3_DAY(pers_j) = 0
        UNEM_BENEMP3_QT(pers_j) = 0
        UNEM_ELIGEMP4_QT(pers_j) = 0
        UNEM_BENEMP4_QT(pers_j) = 0
        UNEM_ELIGEMP5_QT(pers_j) = 0
        UNEM_BENEMP5_DAY(pers_j) = 0
        UNEM_BENEMP5_QT(pers_j) = 0
        UNEM_ELIGEMP6_QT(pers_j) = 0
        UNEM_FRACTRAN_QT(pers_j) = 0
        UNEM_TOWNRATE_QT(pers_j) = 0
        UNEM_PWAHOURS_QT(pers_j) = 0
        UNEM_BENEMP6_QT(pers_j) = 0
        UNEM_DUURREIN_QT(pers_j) = 0
        UNEM_FRACREIN_QT(pers_j) = 0
        !Prepayment variables                
        !Reconstructed exchange concepts             
        UNEM_OLDUNSS_QT(pers_j) = 0
        UNEM_OTHERUN_QT(pers_j) = 0
        UNEM_TOTBEN_QT(pers_j) = 0
        UNEM_BENUN_QT(pers_j) = 0
        UNEM_FULL6MON_QT(pers_j) = 0
    END DO
    N_HH_UNEM=0

END SUBROUTINE UNEM_INITENDO

!========== SUBROUTINE UNEM_RANKANDSORT ===============!
!RANK and SORT exogenous variables 
!   Step 1: rank and sort all variables according to their 
!           individual identification number which is read in as an integer
!   Step 2: rank and sort all variables according to the 
!           household identification number
!======================================================!

SUBROUTINE UNEM_RANKANDSORT

!STEP1: ranking and sorting with the individuals identification number
    !Rank individual identification number from the INPUT data file
    CALL RANKING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    !UNEM_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*RVA*/
    CALL SORTING(FICHE7_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DGNDMND_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BEDRAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(REDENLO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DUUR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(URENPWA_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRVA(1:n_rec),NEW_RANK)
    !/*FOD SZ*/
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_CONTRIB.txt
    CALL RANKING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_WORKHOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WAGEPRED(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_SICK.txt
    CALL RANKING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_BENINOC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_NONEINOC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_PIT.txt
    CALL RANKING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_PENSWELF.txt
    CALL RANKING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_FAMAL.txt
    CALL RANKING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file UNEM_UNEM.txt
    CALL RANKING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUNPR_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_FAMREL.txt                                           
    CALL RANKING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !Rank individual identification number from file WAGE_CONSTRUCT.txt                                           
    CALL RANKING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !Rank individual identification number from file MIMOSIS_WEIGHT.txt
    CALL RANKING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL UNEM_CONSISTENCY_CHECK
   
!STEP2: ranking and sorting with the household identification number
    !Rank household number
    CALL RANKING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    !Sort variables on rank of household number
    !UNEM_INPUT_Write.sas
    CALL SORTING(GECODEERD_INSZ_CHAR(1:n_rec),NEW_RANK)
    CALL SORTING(GECODEERD_INSZ_INPUT(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_HH_ID(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_DATBIRTH(1:n_rec),NEW_RANK)
    CALL SORTING(NAREGNIS_SEX(1:n_rec),NEW_RANK)
    !/*RVA*/
    CALL SORTING(FICHE7_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DGNDMND_2008(1:n_rec),NEW_RANK)
    CALL SORTING(BEDRAGEN_2008(1:n_rec),NEW_RANK)
    CALL SORTING(REDENLO_2008(1:n_rec),NEW_RANK)
    CALL SORTING(DUUR_2008(1:n_rec),NEW_RANK)
    CALL SORTING(URENPWA_2008(1:n_rec),NEW_RANK)
    CALL SORTING(InRVA(1:n_rec),NEW_RANK)
    !/*FOD SZ*/
    CALL SORTING(InFODSZ(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_CONTRIB.txt
    CALL SORTING(GECODEERD_INSZ_CONTRIB(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_REVPUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_INCSELF_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPRIV_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSPUB_EMEE_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_SSSELF_EMER_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PRIV_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_LABMSTAT_PUB_QT(1:n_rec),NEW_RANK)
    CALL SORTING(CONTRIB_WORKHOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_SIMNUMMT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WAGEPRED(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_SICK.txt
    CALL SORTING(GECODEERD_INSZ_SICK(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_BENINOC_QT(1:n_rec),NEW_RANK)
    CALL SORTING(SICK_NONEINOC_QT(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_PIT.txt
    CALL SORTING(GECODEERD_INSZ_PIT(1:n_rec),NEW_RANK)
    CALL SORTING(PIT_DEPCHILD_ANN(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_PENSWELF.txt
    CALL SORTING(GECODEERD_INSZ_PENSWELF(1:n_rec),NEW_RANK)
    CALL SORTING(PENSWELF_AMOUNTP_QT(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_FAMAL.txt
    CALL SORTING(GECODEERD_INSZ_FAMAL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMAL_AMOUNTF_QT(1:n_rec),NEW_RANK)
    !Sort variables from file UNEM_UNEM.txt
    CALL SORTING(GECODEERD_INSZ_UNEM(1:n_rec),NEW_RANK)
    CALL SORTING(UNEM_BENUNPR_QT(1:n_rec),NEW_RANK)
    !Sort variables from file MIMOSIS_FAMREL.txt                                           
    CALL SORTING(GECODEERD_INSZ_FAMREL(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMILY(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_RELATION(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_FAMTYPE(1:n_rec),NEW_RANK)
    CALL SORTING(FAMREL_COUPLE(1:n_rec),NEW_RANK)
    !WAGE_CONSTRUCT_Write.sas
    CALL SORTING(GECODEERD_INSZ_WAGE(1:n_rec),NEW_RANK)
    CALL SORTING(SALJRS_LAST_CORR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_HOUR(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_GRINC_QT(1:n_rec),NEW_RANK)
    !MIMOSIS_SAMPWEIG_Write.sas
    CALL SORTING(GECODEERD_INSZ_WEIGHT(1:n_rec),NEW_RANK)
    CALL SORTING(MIMOSIS_WEIGHT(1:n_rec),NEW_RANK)

!Intermediate step: check consistentcy of individual identification numbers of different files
    CALL UNEM_CONSISTENCY_CHECK

END SUBROUTINE UNEM_RANKANDSORT

!========== SUBROUTINE PARAMETER_FILEREAD ===============!
!This subroutines reads in all parameters from           !
!the parameter file                                      !
!========================================================!
SUBROUTINE UNEM_PARAMETER_FILEREAD

    !Read in parameters for path and name of input files
    DO i=1,n_loclines/2
        !Read default path names
        !Start reading the normal file names
        IF (CONTROL_IO(2*i-1)=='UNEM_INPUT_PATH') THEN
            UNEM_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_INPUT_FILE') THEN
            UNEM_INPUT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_CONTRIB_PRERUN_FILE') THEN
            UNEM_CONTRIB_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_SICK_FILE') THEN
            UNEM_SICK_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_PIT_PRERUN_FILE') THEN
            UNEM_PIT_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_PENSWELF_PRERUN_FILE') THEN
            UNEM_PENSWELF_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_FAMAL_FILE') THEN
            UNEM_FAMAL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='UNEM_UNEM_FILE') THEN
            UNEM_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='FAMREL_INPUT_PATH') THEN
            FAMREL_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_FAMREL_FILE') THEN
            MIMOSIS_FAMREL_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='WAGE_CONSTRUCT_FILE') THEN
            WAGE_CONSTRUCT_FILE = CONTROL_IO(2*i)
        !Read material to read sample weights                                                    
        ELSE IF (CONTROL_IO(2*i-1)=='CONSTANT_INPUT_PATH') THEN
            CONSTANT_INPUT_PATH = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='MIMOSIS_SAMPWEIG_FILE') THEN
            MIMOSIS_SAMPWEIG_FILE = CONTROL_IO(2*i)
        !Read the file names of the intermediate output files
        ELSE IF (CONTROL_IO(2*i-1)=='CONTRIB_UNEM_FILE') THEN
            CONTRIB_UNEM_FILE = CONTROL_IO(2*i)
        ELSE IF (CONTROL_IO(2*i-1)=='SOCBEN_UNEM_FILE') THEN
            SOCBEN_UNEM_FILE = CONTROL_IO(2*i)
        !Read information necessary to construct file for evaluation module
        END IF    
    END DO

    !check for length of default path and file names 
    UNEM_PAR_BREAK_POINT(1)  = INDEX(UNEM_INPUT_PATH,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(2)  = INDEX(UNEM_INPUT_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(3)  = INDEX(UNEM_CONTRIB_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(4)  = INDEX(UNEM_SICK_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(5)  = INDEX(UNEM_PIT_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(6)  = INDEX(UNEM_PENSWELF_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(7)  = INDEX(UNEM_FAMAL_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(14) = INDEX(UNEM_UNEM_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(8)  = INDEX(FAMREL_INPUT_PATH,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(9)  = INDEX(MIMOSIS_FAMREL_FILE,UNEM_PAR_BREAK)    
    UNEM_PAR_BREAK_POINT(10) = INDEX(WAGE_CONSTRUCT_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(11) = INDEX(CONSTANT_INPUT_PATH,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(12) = INDEX(MIMOSIS_SAMPWEIG_FILE,UNEM_PAR_BREAK)
    !check for length of path and file names of intermediate output files
    UNEM_PAR_BREAK_POINT(13) = INDEX(CONTRIB_UNEM_FILE,UNEM_PAR_BREAK)
    UNEM_PAR_BREAK_POINT(15) = INDEX(SOCBEN_UNEM_FILE,UNEM_PAR_BREAK)

    !merge information (path+name) to create default input file names
    UNEM_INPUT = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_INPUT_FILE(1:UNEM_PAR_BREAK_POINT(2)-1)
    UNEM_CONTRIB = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_CONTRIB_FILE(1:UNEM_PAR_BREAK_POINT(3)-1)
    UNEM_SICK = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_SICK_FILE(1:UNEM_PAR_BREAK_POINT(4)-1)
    UNEM_PIT = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_PIT_FILE(1:UNEM_PAR_BREAK_POINT(5)-1)
    UNEM_PENSWELF = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_PENSWELF_FILE(1:UNEM_PAR_BREAK_POINT(6)-1)
    UNEM_FAMAL = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_FAMAL_FILE(1:UNEM_PAR_BREAK_POINT(7)-1)
    UNEM_UNEM = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &UNEM_UNEM_FILE(1:UNEM_PAR_BREAK_POINT(14)-1)
    MIMOSIS_FAMREL = FAMREL_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(8)-1)//&
    &MIMOSIS_FAMREL_FILE(1:UNEM_PAR_BREAK_POINT(9)-1)    
    WAGE_CONSTRUCT = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &WAGE_CONSTRUCT_FILE(1:UNEM_PAR_BREAK_POINT(10)-1)
    MIMOSIS_SAMPWEIG = CONSTANT_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(11)-1)//&
    &MIMOSIS_SAMPWEIG_FILE(1:UNEM_PAR_BREAK_POINT(12)-1)    
    !construct names of output files
    CONTRIB_UNEM = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &CONTRIB_UNEM_FILE(1:UNEM_PAR_BREAK_POINT(13)-1)
    SOCBEN_UNEM = UNEM_INPUT_PATH(1:UNEM_PAR_BREAK_POINT(1)-1)//&
    &SOCBEN_UNEM_FILE(1:UNEM_PAR_BREAK_POINT(15)-1)

    !Trim the result
    !of input files
    CALL TRIM@(UNEM_INPUT)
    CALL TRIM@(UNEM_CONTRIB)
    CALL TRIM@(UNEM_SICK)
    CALL TRIM@(UNEM_PIT)
    CALL TRIM@(UNEM_PENSWELF)
    CALL TRIM@(UNEM_FAMAL)
    CALL TRIM@(UNEM_UNEM)
    CALL TRIM@(MIMOSIS_FAMREL)    
    CALL TRIM@(WAGE_CONSTRUCT)  
    CALL TRIM@(MIMOSIS_SAMPWEIG)
    !of output files
    CALL TRIM@(CONTRIB_UNEM)
    CALL TRIM@(SOCBEN_UNEM)

    !write names to log file
    !of input files
    WRITE(15,'(''Print of reconstructed input and output file names'')')
    CALL DRAWLINE(15,100,'=')
    WRITE(15,*) 'UNEM_INPUT= ',         UNEM_INPUT
    WRITE(15,*) 'UNEM_CONTRIB= ',       UNEM_CONTRIB
    WRITE(15,*) 'UNEM_SICK= ',          UNEM_SICK
    WRITE(15,*) 'UNEM_PIT= ',           UNEM_PIT
    WRITE(15,*) 'UNEM_PENSWELF= ',      UNEM_PENSWELF
    WRITE(15,*) 'UNEM_FAMAL= ',         UNEM_FAMAL
    WRITE(15,*) 'UNEM_UNEM= ',          UNEM_UNEM
    WRITE(15,*) 'MIMOSIS_FAMREL= ',     MIMOSIS_FAMREL
    WRITE(15,*) 'WAGE_CONSTRUCT= ',     WAGE_CONSTRUCT
    WRITE(15,*) 'MIMOSIS_SAMPWEIG= ',   MIMOSIS_SAMPWEIG
    !of output files
    WRITE(15,*) 'CONTRIB_UNEM= ',       CONTRIB_UNEM
    WRITE(15,*) 'SOCBEN_UNEM= ',        SOCBEN_UNEM

    !open parameter file        
    OPEN(UNIT=3, FILE=UNEM_PARLOC, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_PARLOC
        WRITE(15,'("Could not find ", a100)') UNEM_PARLOC
        PAUSE
        STOP
    END IF

    ALLOCATE(UNEM_PAR_NAMES(n_par))
    !read in all parameters by looping over the lines in the parameter file
    WRITE(15,'(/,''Print of read in operation of parameter file'')')
    CALL DRAWLINE(15,100,'=')    
    i=1
    DO
        READ(3,'(a40,2i10,a10)') UNEM_PAR_NAMES(i),UNEM_PAR_L, &
        &UNEM_PAR_C,UNEM_PAR_CURR
        IF (UNEM_PAR_NAMES(i)=='UNEM_FAMREL_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_FAMREL_QT(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_FAMREL_QT(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_FAMREL_QT(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ1_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ1_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ1_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ2_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ2_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ2_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ3_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ3_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ3_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ4_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ4_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ4_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ5_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ5_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ5_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ6_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ6_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ6_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ7_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ7_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ7_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_WAGEDFQ8_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_WAGEDFQ8_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_WAGEDFQ8_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LUSUWB_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_LUSUWB_DAY(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_LUSUWB_DAY(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_LUSUWB_DAY(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LIMLOSW_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_LIMLOSW_DAY(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_LIMLOSW_DAY(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_LIMLOSW_DAY(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LIMLOSW_REF_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_LIMLOSW_REF_DAY(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_LIMLOSW_REF_DAY(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_LIMLOSW_REF_DAY(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_PERLIMUB_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_PERLIMUB_DAY(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_PERLIMUB_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_PERLIMUB_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_PERLIMUB_REF_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_PERLIMUB_REF_DAY(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_PERLIMUB_REF_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_PERLIMUB_REF_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_DUURP') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_DUURP
            WRITE(15,UNEM_PAR_FORMAT) UNEM_DUURP
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LSCUB1_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_LSCUB1_DAY
            WRITE(15,UNEM_PAR_FORMAT) UNEM_LSCUB1_DAY
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LSCUB2_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_LSCUB2_DAY
            WRITE(15,UNEM_PAR_FORMAT) UNEM_LSCUB2_DAY
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LIMWAGE_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_LIMWAGE_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_LIMWAGE_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_EARPAR_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_EARPAR_QT(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_EARPAR_QT(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_EARPAR_QT(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LIMEARR_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_LIMEARR_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_LIMEARR_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENCCB_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENCCB_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENCCB_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENCCB_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPCARE_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPCARE_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPCARE_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPCARE_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENMCARE_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENMCARE_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENMCARE_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENMCARE_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPLEAV_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPLEAV_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPLEAV_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPLEAV_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENSSUP_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENSSUP_DAY(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENSSUP_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENSSUP_DAY(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_EXEMPSF_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_EXEMPSF_DAY(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_EXEMPSF_DAY(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_EXEMPSF_DAY(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LSPEAR_DAY') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_LSPEAR_DAY
            WRITE(15,UNEM_PAR_FORMAT) UNEM_LSPEAR_DAY
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENCCM50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENCCM50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENCCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENCCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENCCP50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENCCP50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENCCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENCCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPCM50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPCM50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPCP50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPCP50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENMCM50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENMCM50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENMCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENMCM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENMCP50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENMCP50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENMCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENMCP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPLM50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPLM50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPLM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPLM50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_BENPLP50_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_BENPLP50_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_BENPLP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_BENPLP50_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_GUARINC_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_GUARINC_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_GUARINC_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_GUARINC_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_PWAMIN_HOUR') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_PWAMIN_HOUR
            WRITE(15,UNEM_PAR_FORMAT) UNEM_PWAMIN_HOUR
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_PWAPLUS_HOUR') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_PWAPLUS_HOUR
            WRITE(15,UNEM_PAR_FORMAT) UNEM_PWAPLUS_HOUR
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_TRANSBEN_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_TRANSBEN_MONTH(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_TRANSBEN_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_TRANSBEN_MONTH(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_REINBEN_MONTH') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_REINBEN_MONTH(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_REINBEN_MONTH(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_REINBEN_MONTH(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_SSCONT_QT') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) UNEM_SSCONT_QT
            WRITE(15,UNEM_PAR_FORMAT) UNEM_SSCONT_QT
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_COSTSCAL_ANN') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_COSTSCAL_ANN(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_COSTSCAL_ANN(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_COSTSCAL_ANN(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_TAXRATES_ANN') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_TAXRATES_ANN(UNEM_PAR_L,UNEM_PAR_C))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) ((UNEM_TAXRATES_ANN(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) ((UNEM_TAXRATES_ANN(j,k),k=1,UNEM_PAR_C),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_REDCHILD_ANN') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_REDCHILD_ANN(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_REDCHILD_ANN(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_REDCHILD_ANN(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='UNEM_LSINGUAR_HOUR') THEN
            WRITE(15,'(a40)') UNEM_PAR_NAMES(i)
            ALLOCATE(UNEM_LSINGUAR_HOUR(UNEM_PAR_L))
            READ(3,'(a20)') UNEM_PAR_FORMAT
            READ(3,UNEM_PAR_FORMAT) (UNEM_LSINGUAR_HOUR(j),j=1,UNEM_PAR_L)
            WRITE(15,UNEM_PAR_FORMAT) (UNEM_LSINGUAR_HOUR(j),j=1,UNEM_PAR_L)        
        ELSE IF (UNEM_PAR_NAMES(i)=='end_parameters') THEN
            EXIT
        END IF
        i=i+1
    END DO
    DEALLOCATE(UNEM_PAR_NAMES)

    !close the parameter file
    CLOSE(UNIT=3)
    WRITE(*,*) 'Subroutine UNEM_PARAMETER_FILEREAD of NORMAL RUN......runned correctly'  

END SUBROUTINE UNEM_PARAMETER_FILEREAD

!============= SUBROUTINE INPUT_FILEREAD ================!
!This subroutines reads in data from INPUT file          !
!========================================================!

SUBROUTINE UNEM_INPUT_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_INPUT, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_INPUT
        WRITE(15,'("Could not find ", a100)') UNEM_INPUT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(A14,I10,A7,I10,I10,I10,F15.2,F15.2,A1,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
            &NAREGNIS_SEX(pers_j),FICHE7_2008(pers_j),DAGEN_2008(pers_j),&
            &DGNDMND_2008(pers_j),BEDRAGEN_2008(pers_j),REDENLO_2008(pers_j),&
            &DUUR_2008(pers_j),URENPWA_2008(pers_j),INRVA(pers_j),&
            &INFODSZ(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_INPUT
            END IF
            !Read in numeric part of the individual indentification character variable
            READ (GECODEERD_INSZ_CHAR(pers_j)(8:), '(I10)') GECODEERD_INSZ_INPUT(pers_j)
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),FICHE7_2008(pers_j),DAGEN_2008(pers_j),&
                &DGNDMND_2008(pers_j),BEDRAGEN_2008(pers_j),REDENLO_2008(pers_j),&
                &DUUR_2008(pers_j),URENPWA_2008(pers_j),INRVA(pers_j),&
                &INFODSZ(pers_j)'')')
                WRITE(15,'(A14,I10,A7,I10,I10,I10,F15.2,F15.2,A1,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),FICHE7_2008(pers_j),DAGEN_2008(pers_j),&
                &DGNDMND_2008(pers_j),BEDRAGEN_2008(pers_j),REDENLO_2008(pers_j),&
                &DUUR_2008(pers_j),URENPWA_2008(pers_j),INRVA(pers_j),&
                &INFODSZ(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(A14,I10,A7,I10,I10,I10,F15.2,F15.2,A1,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_CHAR(pers_j),NAREGNIS_HH_ID(pers_j),NAREGNIS_DATBIRTH(pers_j),&
                &NAREGNIS_SEX(pers_j),FICHE7_2008(pers_j),DAGEN_2008(pers_j),&
                &DGNDMND_2008(pers_j),BEDRAGEN_2008(pers_j),REDENLO_2008(pers_j),&
                &DUUR_2008(pers_j),URENPWA_2008(pers_j),INRVA(pers_j),&
                &INFODSZ(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_INPUT_FILEREAD of NORMAL RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from INPUT_FILE")') pers_j-1

END SUBROUTINE UNEM_INPUT_FILEREAD

!============= SUBROUTINE CONTRIB_FILEREAD ================!
!This subroutines reads in data from the UNEM file         !
!==========================================================!
SUBROUTINE UNEM_CONTRIB_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_CONTRIB, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_CONTRIB
        WRITE(15,'("Could not find ", a100)') UNEM_CONTRIB
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,F15.10,I10,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                    
            &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&                
            &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
            &CONTRIB_WORKHOUR(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_WAGEPRED(pers_j)               
            !write line if error
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_CONTRIB
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                    
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&                
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_WORKHOUR(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_WAGEPRED(pers_j) '')')               
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,F15.10,I10,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                    
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&                
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_WORKHOUR(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_WAGEPRED(pers_j)                 
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10,F15.10,I10,F15.2)') &
                &GECODEERD_INSZ_CONTRIB(pers_j),CONTRIB_REVPRIV_QT(pers_j),CONTRIB_REVPUB_QT(pers_j),&                    
                &CONTRIB_INCSELF_QT(pers_j),CONTRIB_SSPRIV_EMEE_QT(pers_j),CONTRIB_SSPUB_EMEE_QT(pers_j),&                
                &CONTRIB_SSSELF_EMER_QT(pers_j),CONTRIB_LABMSTAT_PRIV_QT(pers_j),CONTRIB_LABMSTAT_PUB_QT(pers_j),&
                &CONTRIB_WORKHOUR(pers_j),MIMOSIS_SIMNUMMT(pers_j),MIMOSIS_WAGEPRED(pers_j)                 
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_CONTRIB_FILEREAD of NORMAL RUN........runned correctly'
    WRITE(15,'(I7, " records uploaded from CONTRIB_FILE")') pers_j-1

END SUBROUTINE UNEM_CONTRIB_FILEREAD

!============= SUBROUTINE SICK_FILEREAD ================!
!This subroutines reads in data from the UNEM file      !
!=======================================================!

SUBROUTINE UNEM_SICK_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_SICK, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_SICK
        WRITE(15,'("Could not find ", a100)') UNEM_SICK
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_SICK(pers_j),SICK_BENINOC_QT(pers_j),SICK_NONEINOC_QT(pers_j)                     
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_SICK
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_SICK(pers_j),SICK_BENINOC_QT(pers_j),SICK_NONEINOC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_BENINOC_QT(pers_j),SICK_NONEINOC_QT(pers_j)                     
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2)') &
                &GECODEERD_INSZ_SICK(pers_j),SICK_BENINOC_QT(pers_j),SICK_NONEINOC_QT(pers_j)                     
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_SICK_FILEREAD of NORMAL RUN...........runned correctly'
    WRITE(15,'(I7, " records uploaded from SICK_FILE")') pers_j-1

END SUBROUTINE UNEM_SICK_FILEREAD

!==================================!
!Reads in data from the PIT file   !
!==================================!

SUBROUTINE UNEM_PIT_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=UNEM_PIT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_PIT
        WRITE(15,'("Could not find ", a100)') UNEM_PIT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)                     
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_PIT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)'')')
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10)') &
                &GECODEERD_INSZ_PIT(pers_j),PIT_DEPCHILD_ANN(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_PIT_FILEREAD of NORMAL RUN............runned correctly'
    WRITE (15,'(I7, " records uploaded from PIT_FILE")') pers_j-1

END SUBROUTINE UNEM_PIT_FILEREAD

!============= SUBROUTINE PENSWELF_FILEREAD ================!
!This subroutines reads in data from the PENSWELF file      !
!===========================================================!
SUBROUTINE UNEM_PENSWELF_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_PENSWELF, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_PENSWELF
        WRITE(15,'("Could not find ", a100)') UNEM_PENSWELF
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)                      
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_PENSWELF
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_PENSWELF(pers_j),PENSWELF_AMOUNTP_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_PENSWELF_FILEREAD of NORMAL RUN.......runned correctly'
    WRITE(15,'(I7, " records uploaded from PENSWELF_FILE")') pers_j-1

END SUBROUTINE UNEM_PENSWELF_FILEREAD

!============= SUBROUTINE FAMAL_FILEREAD ================!
!This subroutines reads in data from the FAMAL file      !
!========================================================!
SUBROUTINE UNEM_FAMAL_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_FAMAL, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_FAMAL
        WRITE(15,'("Could not find ", a100)') UNEM_FAMAL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_FAMAL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_FAMAL(pers_j),FAMAL_AMOUNTF_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_FAMAL_FILEREAD of NORMAL RUN..........runned correctly'
    WRITE(15,'(I7, " records uploaded from FAMAL_FILE")') pers_j-1

END SUBROUTINE UNEM_FAMAL_FILEREAD

!============= SUBROUTINE UNEM_FILEREAD =================!
!This subroutines reads in data from the UNEM file       !
!========================================================!
SUBROUTINE UNEM_UNEM_FILEREAD

    OPEN(UNIT=5, FILE=UNEM_UNEM, STATUS='OLD', ACTION='READ',IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_UNEM
        WRITE(15,'("Could not find ", a100)') UNEM_UNEM
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNPR_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,UNEM_UNEM
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNPR_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNPR_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2)') &
                &GECODEERD_INSZ_UNEM(pers_j),UNEM_BENUNPR_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE(UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_UNEM_FILEREAD of NORMAL RUN...........runned correctly'
    WRITE(15,'(I7, " records uploaded from UNEM_FILE")') pers_j-1

END SUBROUTINE UNEM_UNEM_FILEREAD

!=====================================!
!Reads in data from the FAMREL file   !
!=====================================!

SUBROUTINE UNEM_FAMREL_FILEREAD

    INTEGER:: pers_j
    OPEN (UNIT=5, FILE=MIMOSIS_FAMREL, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_FAMREL
        WRITE(15,'("Could not find ", a100)') MIMOSIS_FAMREL
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ (5,'(7x,I7,I10,I10,I10,I10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
            &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_FAMREL
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)'')')
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,I10,I10,I10,I10)') &
                &GECODEERD_INSZ_FAMREL(pers_j),FAMREL_FAMILY(pers_j),FAMREL_RELATION(pers_j),&
                &FAMREL_FAMTYPE(pers_j),FAMREL_COUPLE(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_FAMREL_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_FAMREL")') pers_j-1

END SUBROUTINE UNEM_FAMREL_FILEREAD

!=======================================!
!Reads in data from the WAGE INPUT file !
!=======================================!

SUBROUTINE UNEM_WAGE_FILEREAD

    OPEN (UNIT=5, FILE=WAGE_CONSTRUCT, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') WAGE_CONSTRUCT
        WRITE(15,'("Could not find ", a100)') WAGE_CONSTRUCT
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.2,F15.2,F15.2)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
            &MIMOSIS_GRINC_QT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,WAGE_CONSTRUCT
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.2,F15.2,F15.2)') &
                &GECODEERD_INSZ_WAGE(pers_j),SALJRS_LAST_CORR(pers_j),MIMOSIS_GRINC_HOUR(pers_j),&
                &MIMOSIS_GRINC_QT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_WAGE_FILEREAD of NORMAL RUN...........runned correctly'
    WRITE (15,'(I7, " records uploaded from WAGE_CONSTRUCT")') pers_j-1 

END SUBROUTINE UNEM_WAGE_FILEREAD

!===========================================!
!Reads in data from the MIMOSIS_WEIGHT file !
!===========================================!

SUBROUTINE UNEM_WEIGHT_FILEREAD

    OPEN (UNIT=5, FILE=MIMOSIS_SAMPWEIG, STATUS='OLD', ACTION='READ', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        WRITE(15,'("Could not find ", a100)') MIMOSIS_SAMPWEIG
        PAUSE
        STOP
    END IF
    IF (ierror==0) THEN
        DO pers_j=1,n_rec
            READ(5,'(7x,I7,F15.10)', IOSTAT=ierror) &
            &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            IF (ierror > 0) THEN
                WRITE(15,'(''An error occured in line '',i10,'' of file '',a100)') pers_j,MIMOSIS_SAMPWEIG
            END IF
            !Write first output line for read check
            IF (pers_j.EQ.1) THEN
                WRITE(15,'(&
                &''GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)'')')
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
            !Write last output line for read check
            IF (pers_j.EQ.n_rec) THEN
                WRITE(15,'(7x,I7,F15.10)') &
                &GECODEERD_INSZ_WEIGHT(pers_j),MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END IF
    CLOSE (UNIT=5)
    WRITE(*,*) 'Subroutine UNEM_WEIGHT_FILEREAD of NORMAL RUN.........runned correctly'
    WRITE (15,'(I7, " records uploaded from MIMOSIS_SAMPWEIG")') pers_j-1 

END SUBROUTINE UNEM_WEIGHT_FILEREAD

!=======================================================================!
!Check whether individual identification numbers are sorted consistently!
!=======================================================================!

SUBROUTINE UNEM_CONSISTENCY_CHECK

    INTEGER:: pers_j
    
    DO pers_j=1, n_rec
        IF (&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_CONTRIB(pers_j))   .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))       .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))  .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))     .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_INPUT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_SICK(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))     .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j)).OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))   .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))  .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))    .OR.&
            &(GECODEERD_INSZ_CONTRIB(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))  .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_PIT(pers_j))        .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))   .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))      .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))       .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_SICK(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_PENSWELF(pers_j))    .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))       .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))        .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))      .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))        .OR.&
            &(GECODEERD_INSZ_PIT(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))      .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMAL(pers_j))  .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))   .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j)) .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))   .OR.&
            &(GECODEERD_INSZ_PENSWELF(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j)) .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_UNEM(pers_j))      .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))    .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))      .OR.&
            &(GECODEERD_INSZ_FAMAL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))    .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_FAMREL(pers_j))     .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))       .OR.&
            &(GECODEERD_INSZ_UNEM(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WAGE(pers_j))     .OR.&
            &(GECODEERD_INSZ_FAMREL(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))   .OR.&
            &(GECODEERD_INSZ_WAGE(pers_j).NE.GECODEERD_INSZ_WEIGHT(pers_j))         &
            &) THEN
                WRITE (*,*) "Error: Ranking of external files does not match!"
                PAUSE
                STOP
        END IF
    END DO   
   
END SUBROUTINE UNEM_CONSISTENCY_CHECK 

!============= SUBROUTINE UNEM_RECODE==========================!
!This subroutines treat missing value                          !
!==============================================================!
SUBROUTINE UNEM_RECODE

    DO pers_j=1,n_rec
        !Exogenous variables
        !UNEM_INPUT_WRITE.SAS
        IF (INT(NAREGNIS_HH_ID(pers_j)).EQ.9999999)         NAREGNIS_HH_ID(pers_j)=0
        CALL TRIM@(NAREGNIS_DATBIRTH(pers_j))
        IF (INT(NAREGNIS_SEX(pers_j)).EQ.9999999)           NAREGNIS_SEX(pers_j)=0
        !/*RVA*/
        IF (INT(FICHE7_2008(pers_j)).EQ.9999999)            FICHE7_2008(pers_j)=0
        IF (INT(DAGEN_2008(pers_j)).EQ.9999999)             DAGEN_2008(pers_j)=0
        IF (INT(DGNDMND_2008(pers_j)).EQ.9999999)           DGNDMND_2008(pers_j)=0
        IF (INT(BEDRAGEN_2008(pers_j)).EQ.9999999)          BEDRAGEN_2008(pers_j)=0
        CALL TRIM@(REDENLO_2008(pers_j))
        IF (INT(DUUR_2008(pers_j)).EQ.9999999)              DUUR_2008(pers_j)=0
        IF (INT(URENPWA_2008(pers_j)).EQ.9999999)           URENPWA_2008(pers_j)=0
        IF (INT(INRVA(pers_j)).EQ.9999999)                  INRVA(pers_j)=0
        !/*FOD SZ*/
        IF (INT(INFODSZ(pers_j)).EQ.9999999)                INFODSZ(pers_j)=0
         !Variables read from file UNEM_CONTRIB.txt
        IF (INT(CONTRIB_REVPRIV_QT(pers_j)).EQ.9999999)     CONTRIB_REVPRIV_QT(pers_j)=0
        IF (INT(CONTRIB_REVPUB_QT(pers_j)).EQ.9999999)      CONTRIB_REVPUB_QT(pers_j)=0
        IF (INT(CONTRIB_INCSELF_QT(pers_j)).EQ.9999999)     CONTRIB_INCSELF_QT(pers_j)=0
        IF (INT(CONTRIB_SSPRIV_EMEE_QT(pers_j)).EQ.9999999) CONTRIB_SSPRIV_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSPUB_EMEE_QT(pers_j)).EQ.9999999)  CONTRIB_SSPUB_EMEE_QT(pers_j)=0
        IF (INT(CONTRIB_SSSELF_EMER_QT(pers_j)).EQ.9999999) CONTRIB_SSSELF_EMER_QT(pers_j)=0
        IF (INT(MIMOSIS_WAGEPRED(pers_j)).EQ.9999999)       MIMOSIS_WAGEPRED(pers_j)=0
        !Variables read from file UNEM_SICK.txt
        IF (INT(SICK_BENINOC_QT(pers_j)).EQ.9999999)        SICK_BENINOC_QT(pers_j)=0
        IF (INT(SICK_NONEINOC_QT(pers_j)).EQ.9999999)       SICK_NONEINOC_QT(pers_j)=0
        !Variables read from file UNEM_PIT.txt
        !Variables read from file UNEM_PENSWELF.txt
        IF (INT(PENSWELF_AMOUNTP_QT(pers_j)).EQ.9999999)    PENSWELF_AMOUNTP_QT(pers_j)=0
        !Variables read from file UNEM_FAMAL.txt
        IF (INT(FAMAL_AMOUNTF_QT(pers_j)).EQ.9999999)       FAMAL_AMOUNTF_QT(pers_j)=0
        !Variables read from file UNEM_UNEM.txt
        IF (INT(UNEM_BENUNPR_QT(pers_j)).EQ.9999999)        UNEM_BENUNPR_QT(pers_j)=0
        !Variables read from file MIMOSIS_FAMREL.txt                                                  
        !WAGE_CONSTRUCT_WRITE.SAS
        IF (INT(SALJRS_LAST_CORR(pers_j)).EQ.9999999)       SALJRS_LAST_CORR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_HOUR(pers_j)).EQ.9999999)     MIMOSIS_GRINC_HOUR(pers_j)=0
        IF (INT(MIMOSIS_GRINC_QT(pers_j)).EQ.9999999)       MIMOSIS_GRINC_QT(pers_j)=0
        !MIMOSIS_SAMPWEIG_WRITE.SAS
        IF (INT(MIMOSIS_WEIGHT(pers_j)).EQ.9999999)         MIMOSIS_WEIGHT(pers_j)=0
   END DO   

END SUBROUTINE UNEM_RECODE

!============= SUBROUTINE UNEM_INFLATE==================================!
!This subroutines inflates nominal values to an appropriate price level !
!=======================================================================!
SUBROUTINE UNEM_INFLATE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        !Exogenous variables
        !Variables read from file UNEM_INPUT.txt                                            
        DGNDMND_2008(pers_j)              = INFLATE_NOMVAL(3)*DGNDMND_2008(pers_j)                    !Amount of the unemployment benefit per day which is granted to the unemployed person 
        BEDRAGEN_2008(pers_j)             = INFLATE_NOMVAL(3)*BEDRAGEN_2008(pers_j)                   !Amount of unemployment benefits received during the trimester 
        !Variables read from file UNEM_CONTRIB.txt
!        CONTRIB_REVPRIV_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_REVPRIV_QT(pers_j)               !Sum of gross labour income and holiday earnings, earned as wage earner on the private labour market in a given quater
!        CONTRIB_REVPUB_QT(pers_j)       = INFLATE_NOMVAL(1)*CONTRIB_REVPUB_QT(pers_j)                !Sum of gross labour income and holiday earnings, earned as wage earner on the public labour market in a given quater
!        CONTRIB_INCSELF_QT(pers_j)      = INFLATE_NOMVAL(1)*CONTRIB_INCSELF_QT(pers_j)               !Gross income earned as self employed in a given quater
!        CONTRIB_SSPRIV_EMEE_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSPRIV_EMEE_QT(pers_j)           !Social security contributions paid on CONTRIB_REVPRIV_QT in a given quater
!        CONTRIB_SSPUB_EMEE_QT(pers_j)   = INFLATE_NOMVAL(1)*CONTRIB_SSPUB_EMEE_QT(pers_j)            !Social security contributions paid on CONTRIB_REVPUB_QT in a given quater
!        CONTRIB_SSSELF_EMER_QT(pers_j)  = INFLATE_NOMVAL(1)*CONTRIB_SSSELF_EMER_QT(pers_j)           !Social security contributions paid on CONTRIB_INCSELF_QT in a given quater
        !Variables read from file UNEM_SICK.txt
!        SICK_BENINOC_QT(pers_j)         = INFLATE_NOMVAL(3)*SICK_BENINOC_QT(pers_j)                  !Gross amount of benefits for industrial accidents and occupational diseases in a given quater
!        SICK_NONEINOC_QT(pers_j)        = INFLATE_NOMVAL(3)*SICK_NONEINOC_QT(pers_j)                 !Gross amount of sickness and disability benefits other than benefits for industrial accidents and occupational diseases in a given quater
        !Variables read from file UNEM_PIT.txt
        !Variables read from file UNEM_PENSWELF.txt
!        PENSWELF_AMOUNTP_QT(pers_j)     = INFLATE_NOMVAL(2)*PENSWELF_AMOUNTP_QT(pers_j)              !Gross amount of pensions received in a given quater
        !Variables read from file UNEM_FAMAL.txt
!        FAMAL_AMOUNTF_QT(pers_j)        = INFLATE_NOMVAL(3)*FAMAL_AMOUNTF_QT(pers_j)                 !Gross amount of family allowances received in a given quater by the recipient
        !Variables read from file UNEM_UNEM.txt
!        UNEM_BENUNPR_QT(pers_j)         = INFLATE_NOMVAL(3)*UNEM_BENUNPR_QT(pers_j)                  !Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage) estimated before unemployment benefits are computed
        !Variables read from file MIMOSIS_FAMREL.txt                                                                        
        !Variables read from file WAGE_CONSTRUCT.txt                                                                           
        SALJRS_LAST_CORR(pers_j)        = INFLATE_NOMVAL(1)*SALJRS_LAST_CORR(pers_j)                 !Estimate of last observed income per day
        MIMOSIS_GRINC_HOUR(pers_j)      = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_HOUR(pers_j)               !Estimate of last observed income per hour (Based on CIMIRe, DWH and minimum-wage)
        MIMOSIS_GRINC_QT(pers_j)        = INFLATE_NOMVAL(1)*MIMOSIS_GRINC_QT(pers_j)                 !Estimate of last observed income per quarter (Based on CIMIRe, DWH and minimum-wage)
        !Variables read from file MIMOSIS_WEIGHT.txt
    END DO   

END SUBROUTINE UNEM_INFLATE

!===============================================!
! HOUSEHOLD_SIZE computes household size        !
! and identifies starting and ending record of  !
! sociological household and stores it in a     !
! vector with household information             !
!Constructs variables                           !
!   UNEM_HH_SIZE                                !
!   UNEM_START_HH                               !
!   UNEM_END_HH                                 !
!   N_HH_UNEM                                   !
!===============================================!

SUBROUTINE UNEM_HOUSEHOLD_SIZE

    INTEGER::  I
    INTEGER::  J
    INTEGER::  K
    INTEGER::  N

    I=1
    J=1
    K=0
    N=0

    DO
        J=J+N
        N=0
        K=K+1
        DO WHILE (NAREGNIS_HH_ID(i)==NAREGNIS_HH_ID(j))
            N=N+1
            I=I+1
            IF (i>n_rec) EXIT    
        END DO
        UNEM_HH_SIZE(k)=n
        UNEM_START_HH(k)=j
        UNEM_END_HH(k)=j+n-1
        IF (i>n_rec.OR.j>n_rec)EXIT
    END DO
    N_HH_UNEM=K

END SUBROUTINE UNEM_HOUSEHOLD_SIZE

!=================================================================
!Computes the age of each individual in terms of years and months!
!=================================================================

!============================================!
! Subroutine main input : NAREGNIS_DATBIRTH  ! 
! Subroutine main output: UNEM_YEAR_AGE      !
!                         UNEM_MONTH_AGE     !
!                         UNEM_YEAR_BIRTH    !
!                         UNEM_MONTH_BIRTH   !   
!============================================!

SUBROUTINE UNEM_AGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec
        UNEM_PAR_BREAK_POINT(1)=INDEX(NAREGNIS_DATBIRTH(pers_j),'-')
        READ (NAREGNIS_DATBIRTH(pers_j)(1:UNEM_PAR_BREAK_POINT(1)-1),  '(I10)') UNEM_MONTH_BIRTH(pers_j)
        READ (NAREGNIS_DATBIRTH(pers_j)(UNEM_PAR_BREAK_POINT(1)+1:),   '(I10)') UNEM_YEAR_BIRTH(pers_j)          
        !determine age as number of years at the end of the year for which data are registered
        UNEM_YEAR_AGE(pers_j)  =MIMOSIS_YEAR-UNEM_YEAR_BIRTH(pers_j)
        !determine number of months as year age plus number of months passed until the end of the year
        UNEM_MONTH_AGE(pers_j) =UNEM_YEAR_AGE(pers_j)*12+(12-UNEM_MONTH_BIRTH(pers_j))
    END DO

END SUBROUTINE UNEM_AGE

!==========================================!
!Draw a line of a give number of characters
!==========================================!

SUBROUTINE DRAWLINE(NUNIT,N,WRITECHAR)

    ! DRAWS A LINE OF N CHARACTERS OF TYPE WRITECHAR ON UNIT NUNIT
    INTEGER                                                 :: N                                    !Number of characters to be drawn
    INTEGER                                                 :: NUNIT                                    !Number of characters to be drawn
    CHARACTER(len=1)                                        :: WRITECHAR                            !Type of character that has to be reproduced N times
    CHARACTER(len=1),DIMENSION(N)                           :: A                                    !Vector of dimension N containing WRITECHAR in the N positions
    CHARACTER(len=15)                                       :: OUTPUTFMT                            !Format used to write result
    CHARACTER(len=7)                                        :: NCHAR                                !Variable containing the character equivalent of the converted value of the integer    
    INTEGER,EXTERNAL                                        :: NONBLK

    A(1:N)=WRITECHAR
    !convert the integer value N into a character value NCHAR with the Fortran internal Write function                                                        
    WRITE (NCHAR, '(I7)') N
    !Construct the output format by aid of the internal Fortran function NONBLK which identifies the first non blank character
    !and read the remainder of the string, i.e. :
    OUTPUTFMT = '(' // NCHAR(NONBLK(NCHAR):) // 'A1)'
    WRITE(NUNIT,OUTPUTFMT)(A(I),I=1,N)

END SUBROUTINE DRAWLINE

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as a result of different rates applied on an input amount.             !
!The rates vary with the level of the input amount and are              !
!constant within brackets                                               !
!The lower and upper ceiling of each bracket are in column 1 and 2 of   !
!the input matrix. The rate is in column 3 of this matrix.              !
!An example of this procedure is the social security contribution of    ! 
!self employed in principal activity before retirement age              !
!In this module to be applied with parameters                           !
!   UNEM_COSTSCAL_ANN
!   UNEM_TAXRATES_ANN
!=======================================================================!

FUNCTION UNEM_BRACRATE_3COLUMNS(INPUT_AMOUNT,NLINES_BRACRATE,MAT_BRACRATE,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_BRACRATE                          !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_BRACRATE,3)  :: MAT_BRACRATE                             !Matrix with brackets and rates
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first bracket to the second last bracket
    DO i=1,NLINES_BRACRATE-1                                                                          
        !if the amount is equal to or larger than the lower limit and smaller than the upper limit of a bracket
        IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,1)).AND.&
        &INT(100.D0*INPUT_AMOUNT).LT.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                             
            !apply the rate to the difference between income and lower limit (and add it up to the deductions computed in the previous brackets)
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
            !exit loop                                                                   
            EXIT
        !otherwise the amount is higher than the upper bracket
        ELSE IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(i,2))) THEN                                                                                     
            !apply rate to the difference between upper and lower limit (and add it up to the amount computed with previous brackets) 
            OUTPUT_AMOUNT=OUTPUT_AMOUNT+(MAT_BRACRATE(i,2)-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)
            FUNCSELE = 1
        END IF
    END DO 
    !in the last bracket: apply rate to all income higher than lower limit
    IF (INT(100.D0*INPUT_AMOUNT).GE.INT(100.D0*MAT_BRACRATE(NLINES_BRACRATE,1))) THEN
        OUTPUT_AMOUNT=OUTPUT_AMOUNT+(INPUT_AMOUNT-MAT_BRACRATE(i,1))*(MAT_BRACRATE(i,3)/100.D0)                                                       
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function UNEM_BRACRATE_3COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION UNEM_BRACRATE_3COLUMNS

!=======================================================================!
!Define a function that computes a nominal amount                       !
!as the sum of different amounts conditional on the value of a status   !
!variable                                                               !
!The amounts are stored in the vector MAT_AMOUNT                        !
!In this module to be applied with parameters                           !
!   UNEM_REDCHILD_ANN                                                   !
!=======================================================================!

FUNCTION UNEM_SUMAMOUNT_1COLUMN(STATUS1,NLINES_AMOUNT,MAT_AMOUNT,INDIV) RESULT (OUTPUT_AMOUNT)

    INTEGER                                             :: STATUS1                                  !Status variable used to compute the sum of nominal amounts 
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_AMOUNT                            !Number of lines in matrix
    INTEGER                                             :: i                                        !Loop variable
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_AMOUNT)      :: MAT_AMOUNT                               !Matrix with status variable and nominal amounts
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !initialize result to zero
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    !loop first from the first value to the second last value
    DO i=1,NLINES_AMOUNT-1                                                                          
        !if the status is larger than the value in column 1 add the amount
        IF (STATUS1.GE.i) THEN
            OUTPUT_AMOUNT = OUTPUT_AMOUNT + MAT_AMOUNT(i)
            FUNCSELE = 1
        END IF
        !if no selection is necessary still keep track of running procedure
        IF (STATUS1.EQ.0) THEN
            FUNCSELE = 1
        END IF
    END DO 
    !in case of the last line a special manipulation is required
    IF (STATUS1.GE.NLINES_AMOUNT) THEN
        OUTPUT_AMOUNT = OUTPUT_AMOUNT+(STATUS1 + 1 - NLINES_AMOUNT)*MAT_AMOUNT(NLINES_AMOUNT)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function UNEM_SUMAMOUNT_1COLUMN has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION UNEM_SUMAMOUNT_1COLUMN

!========================================================================!
!Define a function that computes the prepayments                         !
!We assume the the INPUT_AMOUNT is gross taxable income per year on which!
!prepayments have to be paid                                             !
!========================================================================!

FUNCTION UNEM_PREPAY_COMPUTE(INPUT_AMOUNT,CHILDREN,INDIV) RESULT (OUTPUT_AMOUNT)

    REAL(KIND=KIND_REAL)                                :: INPUT_AMOUNT                             !Amount on which the rates will be applied
    INTEGER                                             :: CHILDREN                                 !Number of children in charge
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    REAL(KIND=KIND_REAL)                                :: UNEM_NTINC_ANN               !Net taxable income per year
    REAL(KIND=KIND_REAL)                                :: UNEM_PREPAY_BASE_ANN         !Basic amount of prepayments paid per year
    REAL(KIND=KIND_REAL)                                :: UNEM_AMOUNT_REDCHILD_ANN     !Annual amount of income, exempted from the bottom up, for dependent children
    REAL(KIND=KIND_REAL)                                :: UNEM_PREPAY_REDCHILD_ANN     !Annual tax reduction because of dependent children
    REAL(KIND=KIND_REAL)                                :: UNEM_PREPAY_ANN              !Annual amount of prepayments
    REAL(KIND=KIND_REAL)                                :: UNEM_PREPAY_QT               !Quarterly amount of prepayments
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected

    !initialize computation variables
    OUTPUT_AMOUNT = 0              
    UNEM_NTINC_ANN = 0             
    UNEM_PREPAY_BASE_ANN = 0       
    UNEM_AMOUNT_REDCHILD_ANN = 0   
    UNEM_PREPAY_REDCHILD_ANN = 0   
    UNEM_PREPAY_ANN = 0             
    UNEM_PREPAY_QT = 0             
    
    !Determine net taxable wages per year
    UNEM_NTINC_ANN = INPUT_AMOUNT - UNEM_BRACRATE_3COLUMNS(INPUT_AMOUNT,SIZE(UNEM_COSTSCAL_ANN,1),UNEM_COSTSCAL_ANN,INDIV)
    !Determine the basic amount of prepayments before deductions are applied    
    UNEM_PREPAY_BASE_ANN = UNEM_BRACRATE_3COLUMNS(UNEM_NTINC_ANN,SIZE(UNEM_TAXRATES_ANN,1),UNEM_TAXRATES_ANN,INDIV)
    !Determine the amount of income that is exempted from the bottom up for dependent children
    UNEM_AMOUNT_REDCHILD_ANN = UNEM_SUMAMOUNT_1COLUMN(CHILDREN,SIZE(UNEM_REDCHILD_ANN),UNEM_REDCHILD_ANN,INDIV)
    !Determine the tax reduction because of dependent children
    UNEM_PREPAY_REDCHILD_ANN = UNEM_BRACRATE_3COLUMNS(UNEM_AMOUNT_REDCHILD_ANN,SIZE(UNEM_TAXRATES_ANN,1),UNEM_TAXRATES_ANN,INDIV)
    !Compute the final amount of prepayments
    UNEM_PREPAY_ANN = UNEM_PREPAY_BASE_ANN - UNEM_PREPAY_REDCHILD_ANN
    IF (UNEM_PREPAY_ANN.LT.0) THEN
        UNEM_PREPAY_ANN = 0
    END IF
    UNEM_PREPAY_QT = UNEM_PREPAY_ANN/4.D0
    
    OUTPUT_AMOUNT = UNEM_PREPAY_QT
    
END FUNCTION UNEM_PREPAY_COMPUTE 

!=======================================================================!
!Define a function that selects a nominal amount                        !
!from a table conditional on the value a row and column                 !
!the row value is the product of two input values                       !
!In this module to be applied with parameters                           !
!   UNEM_BENCCM50_MONTH                                                 !
!   UNEM_BENCCP50_MONTH                                                 !
!   UNEM_BENPCM50_MONTH                                                 !
!   UNEM_BENPCP50_MONTH                                                 !
!   UNEM_BENMCM50_MONTH                                                 !
!   UNEM_BENMCP50_MONTH                                                 !
!   UNEM_BENPLM50_MONTH                                                 !
!   UNEM_BENPLP50_MONTH                                                 !
!=======================================================================!

FUNCTION UNEM_SELAMOUNT_2COLUMNS(STATUS1,STATUS2,STATUS3,NLINES_AMOUNT,MAT_AMOUNT,INDIV) RESULT (OUTPUT_AMOUNT)

    INTEGER                                             :: STATUS1,STATUS2,STATUS3                  !Status variables used to determine the selection cell in the matrix
    REAL(KIND=KIND_REAL)                                :: OUTPUT_AMOUNT                            !Amount that is produced as output 
    INTEGER                                             :: NLINES_AMOUNT                            !Number of lines in matrix
    REAL(KIND=KIND_REAL),DIMENSION (NLINES_AMOUNT,2)    :: MAT_AMOUNT                               !Matrix with status variable and nominal amounts
    INTEGER                                             :: INDIV                                    !Individual identification number for which function is selected
    INTEGER                                             :: FUNCSELE                                 !Indicator to store identification that heart of the function is selected

    !identify variables
    OUTPUT_AMOUNT = 0
    FUNCSELE = 0
    IF (STATUS1*STATUS2.NE.0.AND.STATUS3.NE.0) THEN
        OUTPUT_AMOUNT = MAT_AMOUNT(STATUS1*STATUS2,STATUS3)
        FUNCSELE = 1
    END IF

    IF (FUNCSELE.EQ.0) THEN
        Write(15,'(''Function UNEM_SELAMOUNT_2COLUMNS has been used, but no selection was carried out for individual ''&
        &, i10)') INDIV
    END IF

END FUNCTION UNEM_SELAMOUNT_2COLUMNS

!===================================================================================!
!With subroutine UNEM_CHARAC we reconstruct variables 
!used troughout the different subroutines for constructing the unemployment benefits
!Constructed variables
!   UNEM_STATUS_QT
!   UNEM_NETWEARN_QT
!   UNEM_NETSELF_QT
!Variables for section on unemployed in search of work
!   UNEM_PERMIS_QT
!   UNEM_ELIGUIS_QT
!   UNEM_DISABLED_QT
!   UNEM_NUMMINU_QT
!   UNEM_NUMYWAGE_QT
!   UNEM_DECREARN_QT
!   UNEM_AVGLWAGE_DAY
!   Manipulate DAGEN_QT
!Variables for section on unemployed not in search of work but entitled to RVA benefits
!   UNEM_ELIGUNS1_QT 
!   UNEM_LOSWAGE_QT
!   UNEM_ELIGUNS2_QT 
!   UNEM_FPUNS_QT
!   UNEM_CARBCON_QT
!   UNEM_ELIGUNS3_QT    
!   UNEM_ELIGUNS4_QT    
!   UNEM_EXEMP_QT
!Variables for section on employees entitled to RVA benefits
!   UNEM_ELIGEMP1_QT
!   UNEM_ELIGEMP2_QT
!   UNEM_FPEMP_QT
!   UNEM_TIMEUNS_QT
!   UNEM_ELIGEMP3_QT
!   UNEM_ELIGEMP4_QT
!   UNEM_ELIGEMP5_QT
!   UNEM_ELIGEMP6_QT
!   UNEM_FRACTRAN_QT
!   UNEM_TOWNRATE_QT
!   UNEM_PWAHOURS_QT
!   UNEM_DUURREIN_QT
!   UNEM_FRACREIN_QT
!===================================================================================!

SUBROUTINE UNEM_CHARAC

    INTEGER :: pers_j,i
    REAL(KIND=KIND_REAL)          :: UNEM_GTINC_ANN                       !Gross taxable income per year           
    
    !UNEM_STATUS_QT: Indicator whether an individual is entitled to any sort of benefit paid by the RVA          
    !1  people who receive unemployment benefits after studies
    !2  people who receive unemployment benefits after employment
    !3  people who are on conventional early retirement
    !4  people who are on career break
    !5  the older unemployed who receive a seniority supplement
    !6  people who receive unemployment benefits because of an exemption
    !7  people who are on part-time early retirement
    !8  people who are on part-time career break
    !9  people who receive guaranteed income benefits
    !10 people who are temporarily unemployed
    !11 people who are working for a Plaatselijk Werkgelegenheidsagentschap/Agence locale pour lemploi
    !12 people who are working in some kind of activation program other than the PWA system
    DO pers_j=1,n_rec
        IF          (ANY(FICHE7_2008(pers_j).EQ.(/3, 6, 46/))) THEN
            UNEM_STATUS_QT(pers_j) = 1
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/1, 2, 4, 5, 16, 17, 39, 45, 47, 13, 30, 42, 70, 71, 72/))) THEN
            UNEM_STATUS_QT(pers_j) = 2
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/36, 37, 38, 97, 98, 360, 370, 361, 371/))) THEN
            UNEM_STATUS_QT(pers_j) = 3
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/200, 201, 202, 204, 205, 206, 230, 240/))) THEN
            UNEM_STATUS_QT(pers_j) = 4
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/7/))) THEN
            UNEM_STATUS_QT(pers_j) = 5
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/8, 9, 10, 11, 14, 15, 26, 27, 28, 29, 40, 41, 43, 44/))) THEN
            UNEM_STATUS_QT(pers_j) = 6
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/99/))) THEN
            UNEM_STATUS_QT(pers_j) = 7
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/208, 209, 210, 212, 213, 214, 216, 217, 218, 231,&
        & 232, 241, 242/))) THEN
            UNEM_STATUS_QT(pers_j) = 8
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/18, 19, 48/))) THEN
            UNEM_STATUS_QT(pers_j) = 9
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/20, 21, 22, 23/))) THEN
            UNEM_STATUS_QT(pers_j) = 10
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/31, 32/))) THEN
            UNEM_STATUS_QT(pers_j) = 11
        ELSE IF     (ANY(FICHE7_2008(pers_j).EQ.(/33, 34, 35, 80, 81, 82, 83, 84, 85, 86, 87, 100, &
                    &101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, &
                    &115, 116, 117, 118, 119, 120, 126, 127, 128, 130, 131, 132, 133, 134, &
                    &135, 136, 137, 138, 122, 124, 140, 141, 142, 143, 144, 145, 146, 147, &
                    &148, 149, 150, 151, 152, 153, 159, 160, 161, 162/))) THEN
            UNEM_STATUS_QT(pers_j) = 12
        END IF
!        IF (UNEM_STATUS_QT(pers_j).EQ.0.AND.FICHE7_2008(pers_j).NE.0) THEN
!            WRITE(15,*) 'TestStatusW',GECODEERD_INSZ_INPUT(pers_j),FICHE7_2008(pers_j)
!        END IF
        !add status determination in case of marginal tax simulation
        IF (MIMOSIS_SIMNUMMT(pers_j).NE.0.AND.(ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1,2/)).OR.&
        &ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1,2/)))) THEN
            UNEM_STATUS_QT(pers_j) = 2
        END IF
    END DO

    !UNEM_NETWEARN_QT: Net income earned as wage earner            
    DO pers_j=1,n_rec
        !first determine gross taxable income per year
        UNEM_GTINC_ANN = ((CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))-&
        &(CONTRIB_SSPRIV_EMEE_QT(pers_j) + CONTRIB_SSPUB_EMEE_QT(pers_j)))*4.D0
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute quarterly amount of prepayments 
        UNEM_NETWEARN_QT(pers_j) = (UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
        &GECODEERD_INSZ_INPUT(pers_j))
    END DO

    !UNEM_NETSELF_QT: Net income earned as self employed          
    DO pers_j=1,n_rec
        !first determine gross taxable income per year
        UNEM_GTINC_ANN = (CONTRIB_INCSELF_QT(pers_j) - CONTRIB_SSSELF_EMER_QT(pers_j))*4.D0
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute quarterly amount of prepayments 
        UNEM_NETSELF_QT(pers_j) = (UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
        &GECODEERD_INSZ_INPUT(pers_j))
    END DO

!Unemployment benefits of unemployed in search of work           
    !UNEM_PERMIS_QT: Unemployed is permitted to the system of unemployment benefits on the basis of studies or on the basis of previous employment           
    !1 permission to unemployment benefits because unemployed is a graduated student
    !2 permission to unemployment benefits because unemployed is a student in a study/employment program
    !3 permission to unemployment benefits on the basis of previous employment
    DO pers_j=1,n_rec
        IF (ANY(FICHE7_2008(pers_j).EQ.(/3, 46/))) THEN
            UNEM_PERMIS_QT(pers_j) = 1
        ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/6/))) THEN 
            UNEM_PERMIS_QT(pers_j) = 2
        ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/1, 2, 4, 5, 16, 17, 45, 47, 13, 30, 42, 70, 71, 72/))) THEN 
            UNEM_PERMIS_QT(pers_j) = 3
        END IF            
    END DO

    !UNEM_ELIGUIS_QT: Individual satisfies the eligibility conditions for unemployment benefits and is in search of work           
    DO pers_j=1,n_rec
        IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2/))) THEN
            UNEM_ELIGUIS_QT(pers_j) = 1
        END IF            
    END DO

    !UNEM_DISABLED_QT: Unemployed is disabled yes or no            
    DO pers_j=1,n_rec
        IF (InFODSZ(pers_j).GT.0) THEN
            UNEM_DISABLED_QT(pers_j) = 1
        END IF            
    END DO

    !UNEM_NUMMINU_QT: Number of months the unemployed is already in unemployment          
    DO pers_j=1,n_rec
        UNEM_NUMMINU_QT(pers_j) = DUUR_2008(pers_j)
        !Adapt the number of months in case of marginal tax simulation
        IF (MIMOSIS_SIMNUMMT(pers_j).NE.0.AND.(ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1,2/)).OR.&
        &ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1,2/)))) THEN
            UNEM_NUMMINU_QT(pers_j) = 0
        END IF
    END DO

    !UNEM_NUMYWAGE_QT: The number of years the unemployed has worked as wage earner            
    DO pers_j=1,n_rec
        UNEM_NUMYWAGE_QT(pers_j) = 0
    END DO

    !UNEM_DECREARN_QT: The decrease in earning capacity if the unemployed would be on the labour market            
    DO pers_j=1,n_rec
        UNEM_DECREARN_QT(pers_j) = 0
    END DO

    !UNEM_AVGLWAGE_DAY: The last daily wage earned before entering unemployment         
    DO pers_j=1,n_rec
        UNEM_AVGLWAGE_DAY(pers_j) = MIMOSIS_GRINC_HOUR(pers_j)*7.6D0
    END DO

    !adapt value of DAGEN if unknown or too large
    DO pers_j=1,n_rec
        IF (DAGEN_2008(pers_j).EQ.0.OR.DAGEN_2008(pers_j).EQ.9999999) THEN
            IF (UNEM_NUMMINU_QT(pers_j).EQ.0.OR.UNEM_NUMMINU_QT(pers_j).GT.12) THEN
                DAGEN_2008(pers_j) = 312
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.0.AND.UNEM_NUMMINU_QT(pers_j).LT.12) THEN
                DAGEN_2008(pers_j) = 26*UNEM_NUMMINU_QT(pers_j)
            END IF 
        END IF
        IF (DAGEN_2008(pers_j).GT.312) THEN
            DAGEN_2008(pers_j) = 312
        END IF
    END DO

!Unemployment benefits of unemployed not in search of work who are entitled to RVA-ONEM benefits             
    
    !UNEM_ELIGUNS1_QT: Unemployed is not in search of work and is entitled to early retirement benefits either of the old or the new type          
    !1 unemployed is entitled to early retirement benefits of the old type
    !2 unemployed is entitled to early retirement benefits of the new type
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.3) THEN
            UNEM_ELIGUNS1_QT(pers_j) = 2
        END IF
    END DO

    !UNEM_LOSWAGE_QT: The last quarterly wage earned before entering unemployment         
    DO pers_j=1,n_rec
        UNEM_LOSWAGE_QT(pers_j) = MIMOSIS_GRINC_QT(pers_j)       
        IF (UNEM_LOSWAGE_QT(pers_j).EQ.0.D0) THEN
            IF (CONTRIB_WORKHOUR(pers_j).NE.0.D0) THEN
                UNEM_LOSWAGE_QT(pers_j) = 13.D0*CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j)       
            ELSE
                UNEM_LOSWAGE_QT(pers_j) = 13.D0*38.D0*MIMOSIS_GRINC_HOUR(pers_j)       
            END IF
        END IF
    END DO
    
    !UNEM_ELIGUNS2_QT: Unemployed is not in search of work and is entitled to career break benefits            
    !1 Common career break
    !2 Career break in case of palliative care
    !3 career break benefits in case of medical assistance
    !4 career break benefits in case of parental leave
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.4) THEN
            IF (REDENLO_2008(pers_j)(1:1).EQ.'.'.OR.REDENLO_2008(pers_j)(1:1).EQ.'9') THEN
                UNEM_ELIGUNS2_QT(pers_j) = 1
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'P') THEN
                UNEM_ELIGUNS2_QT(pers_j) = 2
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'G') THEN
                UNEM_ELIGUNS2_QT(pers_j) = 3
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'O') THEN
                UNEM_ELIGUNS2_QT(pers_j) = 4
            END IF                
        END IF
    END DO

    !UNEM_FPUNS_QT: Type of job left by the unemployed not in search of work (i.e. either full or part time)            
    !1 the employee leaves a full time job 
    !2 the employee leaves a part time job
    !Bij code 230 en 240 is niet aangegeven of het om voltijdse of een deeltijdse job gaat die verlaten wordt
    !Er wordt verondersteld dat het om een voltijds job gaat
    DO pers_j=1,n_rec
        IF (ANY(FICHE7_2008(pers_j).EQ.(/200, 201, 202, 230, 240/))) THEN
            UNEM_FPUNS_QT(pers_j) = 1
        ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/204, 205, 206/))) THEN 
            UNEM_FPUNS_QT(pers_j) = 2
        END IF 
    END DO

    !UNEM_CARBCON_QT: Qualification of the career break of the unemployed not in search of work (i.e. number of months and supplement)            
    !1 Standard amount, first 12 months
    !2 Supplement for second child, first 12 months
    !3 Supplement for third child or above, first 12 months
    !4 Standard amount, after 12 months
    !5 Supplement for second child, after 12 months
    !6 Supplement for third child or above, after 12 months
    DO pers_j=1,n_rec
        !Merk op: vroeger werd CARBCON ingevuld op basis de variabelen VERMIN en VERHOG dat kan nu niet meer
        !wegens onbeschikbaarheid 
        !Daarom benaderende procedure op basis van andere variabelen
        IF (UNEM_NUMMINU_QT(pers_j).LE.12) THEN
            IF (PIT_DEPCHILD_ANN(pers_j).LT.2) THEN
                UNEM_CARBCON_QT(pers_j) = 1
            ELSE IF (PIT_DEPCHILD_ANN(pers_j).EQ.2) THEN
                UNEM_CARBCON_QT(pers_j) = 2
            ELSE IF (PIT_DEPCHILD_ANN(pers_j).GT.2) THEN
                UNEM_CARBCON_QT(pers_j) = 3
            END IF
        ELSE
            IF (PIT_DEPCHILD_ANN(pers_j).LT.2) THEN
                UNEM_CARBCON_QT(pers_j) = 4
            ELSE IF (PIT_DEPCHILD_ANN(pers_j).EQ.2) THEN
                UNEM_CARBCON_QT(pers_j) = 5
            ELSE IF (PIT_DEPCHILD_ANN(pers_j).GT.2) THEN
                UNEM_CARBCON_QT(pers_j) = 6
            END IF
        END IF
    END DO

    !UNEM_ELIGUNS3_QT: Unemployed is not in search of work and is entitled to benefits for older unemployed with seniority supplement          
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.5) THEN
            UNEM_ELIGUNS3_QT(pers_j) = 1
        END IF
    END DO

    !UNEM_ELIGUNS4_QT: Unemployed is not in search of work and is entitled to unemployment benefits in case of exemptions          
    !1 the unemployed is exempted because of social and familial reasons 
    !2 the unemployed is exempted because of studies or professional training.
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.6) THEN
            IF (FICHE7_2008(pers_j).EQ.8) THEN
                UNEM_ELIGUNS4_QT(pers_j) = 1
            ELSE
                UNEM_ELIGUNS4_QT(pers_j) = 2
            END IF
        END IF
    END DO

    !UNEM_EXEMP_QT: Number of months the unemployed is already exempted from certain obligations            
    !1 Exempted 24 months or less
    !2 Exempted more than 24 months but less than 72
    !3 Exempted more than 72 months
    DO pers_j=1,n_rec
        IF (FICHE7_2008(pers_j).EQ.8) THEN
            IF (INT(100.D0*DGNDMND_2008(pers_j)).EQ.INT(100.D0*UNEM_EXEMPSF_DAY(1))) THEN
                UNEM_EXEMP_QT(pers_j) = 1
            ELSE IF (INT(100.D0*DGNDMND_2008(pers_j)).EQ.INT(100.D0*UNEM_EXEMPSF_DAY(2))) THEN
                UNEM_EXEMP_QT(pers_j) = 2
            ELSE
                UNEM_EXEMP_QT(pers_j) = 3
            END IF
        END IF
    END DO

!Unemployment benefits of employees entitled to benefits paid by the RVA-ONEM                                 

    !UNEM_ELIGEMP1_QT: Employee is entitled to part time early retirement benefits         
    !1 employee is entitled to early retirement benefits of the old type
    !2 employee is entitled to early retirement benefits of the new type
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.7) THEN
            UNEM_ELIGEMP1_QT(pers_j) = 2
        END IF
    END DO

    !UNEM_ELIGEMP2_QT: Employee is entitled to part time career break benefits         
    !1 Common career break
    !2 Career break in case of palliative care
    !3 career break benefits in case of medical assistance
    !4 career break benefits in case of parental leave
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.8) THEN
            IF (REDENLO_2008(pers_j)(1:1).EQ.'.'.OR.REDENLO_2008(pers_j)(1:1).EQ.'9') THEN
                UNEM_ELIGEMP2_QT(pers_j) = 1
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'P') THEN
                UNEM_ELIGEMP2_QT(pers_j) = 2
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'G') THEN
                UNEM_ELIGEMP2_QT(pers_j) = 3
            ELSE IF (REDENLO_2008(pers_j)(1:1).EQ.'O') THEN
                UNEM_ELIGEMP2_QT(pers_j) = 4
            END IF                
        END IF
    END DO

    !UNEM_FPEMP_QT: Type of job left by the employee entitled to part time career break benefits (i.e. either full or part time)            
    !1 the employee leaves a full time job 
    !2 the employee leaves a part time job
    !Bij geen van deze codes is aangegeven of het om voltijdse of een deeltijdse job gaat die verlaten wordt
    !Er wordt verondersteld dat het om een voltijds job gaat
    DO pers_j=1,n_rec
        IF (ANY(FICHE7_2008(pers_j).EQ.(/208, 209, 210, 212, 213, 214, 216, 217, 218, 231, 232, 241, 242/))) THEN
            UNEM_FPEMP_QT(pers_j) = 1
        END IF 
    END DO
    
    !UNEM_TIMEUNS_QT: Reduction in work time of the employee entitled to part time career break benefits          
    !reconstruction infers backwards from observed values
    DO pers_j=1,n_rec
        !Selection procedure differs with type of career break
        !common part time career break
        IF (UNEM_ELIGEMP2_QT(pers_j).EQ.1) THEN
            UNEM_TIMEUNS_QT(pers_j) = 1
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENCCM50_MONTH((UNEM_CARBCON_QT(pers_j)-1)*4+i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENCCM50_MONTH((UNEM_CARBCON_QT(pers_j)-4)*4+i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENCCP50_MONTH((UNEM_CARBCON_QT(pers_j)-1)*4+i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENCCP50_MONTH((UNEM_CARBCON_QT(pers_j)-4)*4+i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            END IF
        !part time career break in case of palliative care            
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.2) THEN
            UNEM_TIMEUNS_QT(pers_j) = 1
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN            
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPCM50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPCM50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPCP50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPCP50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            END IF
        !part time career break in case of medical assistance       
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.3) THEN
            UNEM_TIMEUNS_QT(pers_j) = 1
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN            
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENMCM50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENMCM50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENMCP50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENMCP50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            END IF
        !part time career break in case of parental leave      
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.4) THEN
            UNEM_TIMEUNS_QT(pers_j) = 1
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN            
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPLM50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPLM50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPLP50_MONTH(i,1)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        !We allow rounding errors after the comma
                        IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_BENPLP50_MONTH(i,2)*3.D0)&
                        &.AND.INT((BEDRAGEN_2008(pers_j)/4.D0)).NE.0) THEN
                            UNEM_TIMEUNS_QT(pers_j) = i
                        END IF
                    END DO
                END IF
            END IF
        END IF
    END DO

    !UNEM_ELIGEMP3_QT: Employee is entitled to guaranteed income benefits paid by the RVA          
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.9) THEN
            UNEM_ELIGEMP3_QT(pers_j) = 1
        END IF
    END DO

    !UNEM_ELIGEMP4_QT: Employee is entitled to temporary unemployment benefits         
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.10) THEN
            UNEM_ELIGEMP4_QT(pers_j) = 1
        END IF
    END DO

    !UNEM_ELIGEMP5_QT: Employee is entitled to PWA benefits            
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.11) THEN
            UNEM_ELIGEMP5_QT(pers_j) = 1
        END IF
    END DO

    !UNEM_ELIGEMP6_QT: Employee is entitled to activation benefits for other than PWA program          
    !1 employee is eligible for benefits of a transition program
    !2 employee is eligible for benefits of a reinsertion program
    !3 employee is eligible for benefits that are not modelled
    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).EQ.12) THEN
            IF (ANY(FICHE7_2008(pers_j).EQ.(/80, 81, 82, 83, 100, 101, 102, 103, 104, 105, 106, &
            &107, 108, 109, 110, 111, 112, 113, 114/))) THEN
                UNEM_ELIGEMP6_QT(pers_j) = 1
            ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/84, 85, 86, 117, 118, 119, 120/))) THEN 
                UNEM_ELIGEMP6_QT(pers_j) = 2
            ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/126, 127, 128, 130, 131, 132, 133, 134, &
            &135, 136, 137, 138, 122, 124, 140, 141, 142, 143, 144, 145, 146, 147, &
            &148, 149, 150, 151, 152, 153, 159, 160, 161, 162/))) THEN 
                UNEM_ELIGEMP6_QT(pers_j) = 3
            END IF 
        END IF
    END DO
    
    !UNEM_FRACTRAN_QT: Time performed by the employee in an activation program, other than PWA, expressed as a fraction of a full time         
    !1 The employee is employed 1/2 or more but less than 4/5
    !2 The employee is employed 4/5 or more
    DO pers_j=1,n_rec
        IF (ANY(FICHE7_2008(pers_j).EQ.(/80, 81, 100, 101, 102, 103, 104, 105, 109, 110, 111/))) THEN
            UNEM_FRACTRAN_QT(pers_j) = 1
        ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/82, 83, 106, 107, 108, 112, 113, 114/))) THEN 
            UNEM_FRACTRAN_QT(pers_j) = 2
        END IF 
    END DO

    !UNEM_TOWNRATE_QT: Degree of unemployment in the town the employee in an activation program is living in           
    !1 the employee is entitled to a benefit of a transition program and the unemployment rate in his town is not high
    !2 the employee is entitled to a benefit of a transition program and the unemployment rate in his town is high
    DO pers_j=1,n_rec
        IF (UNEM_ELIGEMP6_QT(pers_j).EQ.1) THEN
            UNEM_TOWNRATE_QT(pers_j) = 1    
            IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_TRANSBEN_MONTH(1,3)*3.D0).OR.&
            &   INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_TRANSBEN_MONTH(2,3)*3.D0)) THEN
                UNEM_TOWNRATE_QT(pers_j) = 2
            END IF
        END IF
    END DO

    !UNEM_PWAHOURS_QT: Number of hours performed in PWA activity           
    !1 the employee is entitled to a benefit of a transition program and did not perform at least 180 PWA hours in the last 6 months
    !2 the employee is entitled to a benefit of a transition program and did perform at least 180 PWA hours in the last 6 months
    DO pers_j=1,n_rec
        IF (UNEM_ELIGEMP6_QT(pers_j).EQ.1) THEN
            UNEM_PWAHOURS_QT(pers_j) = 1    
            IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_TRANSBEN_MONTH(1,2)*3.D0).OR.&
            &   INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_TRANSBEN_MONTH(2,2)*3.D0)) THEN
                UNEM_PWAHOURS_QT(pers_j) = 2
            END IF
        END IF
    END DO

    !UNEM_DUURREIN_QT: Number of months the employee is in a reinsertion program           
    !1 the employee is entitled to a benefit of a reinsertion program and is active for 36 months or less in the reinstertion program
    !2 the employee is entitled to a benefit of a reinsertion program and is active for more than 36 months in the reinstertion program
    DO pers_j=1,n_rec
        IF (UNEM_ELIGEMP6_QT(pers_j).EQ.2) THEN
            UNEM_DUURREIN_QT(pers_j) = 1    
            IF (INT((BEDRAGEN_2008(pers_j)/4.D0)).EQ.INT(UNEM_REINBEN_MONTH(3)*3.D0)) THEN
                UNEM_DUURREIN_QT(pers_j) = 2
            END IF
        END IF
    END DO

    !UNEM_FRACREIN_QT: Time performed by the employee in a reinsertion program, expressed as a fraction of a full time         
    !1 The employee is employed 1/2 or more but less than 4/5
    !2 The employee is employed 4/5 or more
    DO pers_j=1,n_rec
        IF (ANY(FICHE7_2008(pers_j).EQ.(/84, 85/))) THEN
            UNEM_FRACREIN_QT(pers_j) = 1
        ELSE IF (ANY(FICHE7_2008(pers_j).EQ.(/86, 117, 118, 119, 120/))) THEN 
            UNEM_FRACREIN_QT(pers_j) = 2
        END IF 
    END DO

END SUBROUTINE UNEM_CHARAC

!=============================================================================================================!
!CONSTRUCTION OF UNEM_FAMSIT_QT                                                                               !
!   = The family situation of the unemployed according to the unemployment legislation                        !
!   UNEM_FAMSIT_QT =  1 an unemployed with dependent family                                                   !
!   UNEM_FAMSIT_QT =  2 an unemployed who is single                                                           !
!   UNEM_FAMSIT_QT =  3 an unemployed without dependent family and who has not lost his sole source of income'! 
!and some variables required for the construction of this variable or for later use                           !
!   UNEM_REL_QT                                                                                               !
!   UNEM_CHILINHH_QT                                                                                          !
!   UNEM_DEPINC_QT                                                                                            !
!   UNEM_PRIVICOH_QT                                                                                          !
!=============================================================================================================!

SUBROUTINE UNEM_FAMSIT_CONSTRUC

    INTEGER                         :: hh_i,i
    INTEGER                         :: pers_j,lijn_j,pers_k
    INTEGER                         :: PARTNER                  !indicate whether the unemployed has a partner yes or no 
    INTEGER                         :: NUMOTHER                 !count the number of others in the household
    REAL(KIND=KIND_REAL)            :: INC_PART                 !Variable to store the income of the partner

    !loop over all households
    DO hh_i=1,N_HH_UNEM
        !loop over all members of the sociological household
        DO pers_j=UNEM_START_HH(hh_i),UNEM_END_HH(hh_i)                      
            !Check whether pers_j is unemployed and lives in a sociological household with more than 1 member
            IF (UNEM_STATUS_QT(pers_j).NE.0.AND.UNEM_HH_SIZE(hh_i).GT.1) THEN
                !Determine the line in the parameter UNEM_FAMREL_QT to convert relationship codes
                DO i = 1,SIZE(UNEM_FAMREL_QT,1)
                    IF (FAMREL_RELATION(pers_j).EQ.UNEM_FAMREL_QT(i,1)) THEN
                        lijn_j = i
                    END IF
                END DO
                !Initialise all relationship codes and determine the relation of each member of the sociological
                !household with the unemployed
                PARTNER = 0
                NUMOTHER = 0
                DO pers_k=UNEM_START_HH(hh_i),UNEM_END_HH(hh_i)
                    !initialise variables that will be reused for each individual determination of UNEM_FAMSIT_QT
                    UNEM_REL_QT(pers_k) = 0
                    UNEM_DEPINC_QT(pers_k) = 0
                    IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                        DO i = 1,SIZE(UNEM_FAMREL_QT,2)
                            IF (FAMREL_RELATION(pers_k).EQ.UNEM_FAMREL_QT(1,i)) THEN
                                UNEM_REL_QT(pers_k) = UNEM_FAMREL_QT(lijn_j,i)
                                !If there is a child in the household, store indicator in variable for the unemployed!!!, i.e. pers_j
                                IF (UNEM_FAMREL_QT(lijn_j,i).EQ.2) THEN
                                    UNEM_CHILINHH_QT(pers_j) = 1
                                END IF
                                !If there is a partner, store an indicator of this in the variable PARTNER
                                IF (UNEM_FAMREL_QT(lijn_j,i).EQ.1) THEN
                                    PARTNER = pers_k
                                END IF
                                !count number of other household members
                                IF (UNEM_FAMREL_QT(lijn_j,i).EQ.MAXVAL(UNEM_FAMREL_QT(lijn_j,:),2)) THEN
                                    NUMOTHER = NUMOTHER + 1
                                END IF
                            END IF
                        END DO
                    !Members of the sociological household that are not family member are considered as non relatives
                    ELSE IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).NE.FAMREL_FAMILY(pers_k)) THEN
                        UNEM_REL_QT(pers_k) = MAXVAL(UNEM_FAMREL_QT(lijn_j,:),2)
                        NUMOTHER = NUMOTHER + 1
                    END IF
                END DO
                !Check the income condition per household member, conditional on the relationship code
                DO pers_k=UNEM_START_HH(hh_i),UNEM_END_HH(hh_i)
                    IF (UNEM_REL_QT(pers_k).EQ.1) THEN
                        IF (UNEM_NETWEARN_QT(pers_k).GT.UNEM_WAGEDFQ1_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_NETSELF_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_BENUNPR_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_NONEINOC_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (PENSWELF_AMOUNTP_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_BENINOC_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                    ELSE IF (UNEM_REL_QT(pers_k).EQ.2) THEN
                        IF ((CONTRIB_REVPRIV_QT(pers_k) + CONTRIB_REVPUB_QT(pers_k) + CONTRIB_INCSELF_QT(pers_k)).GT.&
                        &UNEM_WAGEDFQ2_QT)                                  UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_BENUNPR_QT(pers_k).GT.UNEM_WAGEDFQ3_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_NONEINOC_QT(pers_k).GT.UNEM_WAGEDFQ3_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (PENSWELF_AMOUNTP_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_BENINOC_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                    ELSE IF (UNEM_REL_QT(pers_k).EQ.5) THEN
                        IF ((CONTRIB_REVPRIV_QT(pers_k) + CONTRIB_REVPUB_QT(pers_k) + CONTRIB_INCSELF_QT(pers_k)).GT.&
                        &UNEM_WAGEDFQ7_QT)                                  UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_BENUNPR_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_NONEINOC_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (PENSWELF_AMOUNTP_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_BENINOC_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                    ELSE IF (UNEM_REL_QT(pers_k).EQ.3.OR.UNEM_REL_QT(pers_k).EQ.4) THEN
                        IF ((CONTRIB_REVPRIV_QT(pers_k) + CONTRIB_REVPUB_QT(pers_k) + CONTRIB_INCSELF_QT(pers_k)).GT.&
                        &UNEM_WAGEDFQ7_QT)                                  UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_BENUNPR_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_NONEINOC_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_CHILINHH_QT(pers_j).EQ.1.AND.PENSWELF_AMOUNTP_QT(pers_k).GT.&
                        &UNEM_WAGEDFQ4_QT)                                  UNEM_DEPINC_QT(pers_k) = 1 
                        IF (UNEM_CHILINHH_QT(pers_j).EQ.0.AND.PENSWELF_AMOUNTP_QT(pers_k).GT.&
                        &UNEM_WAGEDFQ5_QT)                                  UNEM_DEPINC_QT(pers_k) = 1 
                        IF (SICK_BENINOC_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                    ELSE IF (UNEM_REL_QT(pers_k).EQ.6) THEN
                        IF ((CONTRIB_REVPRIV_QT(pers_k) + CONTRIB_REVPUB_QT(pers_k) + CONTRIB_INCSELF_QT(pers_k)).GT.&
                        &UNEM_WAGEDFQ7_QT)                                  UNEM_DEPINC_QT(pers_k) = 1    
                        IF (UNEM_BENUNPR_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                        IF (SICK_NONEINOC_QT(pers_k).GT.UNEM_WAGEDFQ7_QT)   UNEM_DEPINC_QT(pers_k) = 1    
                        IF (PENSWELF_AMOUNTP_QT(pers_k).GT.UNEM_WAGEDFQ5_QT)UNEM_DEPINC_QT(pers_k) = 1 
                        IF (SICK_BENINOC_QT(pers_k).GT.UNEM_WAGEDFQ6_QT)    UNEM_DEPINC_QT(pers_k) = 1    
                    END IF
                END DO
                !Determine the value of UNEM_FAMSIT_QT(pers_j)
                !Distinguish between the values 1 and 3
                !Check whether the unemployed has a partner yes or no
                IF (PARTNER.NE.0) THEN
                    IF (UNEM_DEPINC_QT(PARTNER).EQ.1) THEN
                        UNEM_FAMSIT_QT(pers_j) = 3
                    ELSE
                        UNEM_FAMSIT_QT(pers_j) = 1
                    END IF
                !if no partner                    
                ELSE                        
                    !A: Check whether an individual violates the income conditions
                    !   Distinguish the case of children (including the child benefit condition and other household members
                    DO pers_k=UNEM_START_HH(hh_i),UNEM_END_HH(hh_i)
                        IF (pers_j.NE.pers_k.AND.FAMREL_FAMILY(pers_j).EQ.FAMREL_FAMILY(pers_k)) THEN
                            DO i = 1,SIZE(UNEM_FAMREL_QT,2)
                                IF (FAMREL_RELATION(pers_k).EQ.UNEM_FAMREL_QT(1,i)) THEN
                                    !If there is a child in the household
                                    IF (UNEM_FAMREL_QT(lijn_j,i).EQ.2) THEN
                                        !check the income condition of this child AND the unemployed is not entitled to child benefits
                                        IF (UNEM_FAMSIT_QT(pers_j).EQ.0.AND.INT(100.D0*FAMAL_AMOUNTF_QT(pers_j)).EQ.0.AND.&
                                        &UNEM_DEPINC_QT(pers_k).EQ.1) THEN
                                            UNEM_FAMSIT_QT(pers_j) = 3 
                                        END IF                                            
                                    !If there is another household member only check the income condition
                                    ELSE IF (UNEM_DEPINC_QT(pers_k).EQ.1) THEN
                                            UNEM_FAMSIT_QT(pers_j) = 3
                                    END IF
                                END IF
                            END DO
                        END IF
                    END DO
                    !B: If the unemployed only cohabits with people classified as other (!!=independent of their income)
                    IF (UNEM_FAMSIT_QT(pers_j).EQ.0.AND.(UNEM_HH_SIZE(hh_i)-NUMOTHER).EQ.1) THEN
                        UNEM_FAMSIT_QT(pers_j) = 3
                    END IF
                    !In all other cases where the unemployed cohabits with other people we consider him to have family charge
                    IF (UNEM_FAMSIT_QT(pers_j).EQ.0) THEN
                        UNEM_FAMSIT_QT(pers_j) = 1
                    END IF
                END IF
                !Check whether the unemployed has a privileged cohabitant
                IF (PARTNER.NE.0) THEN
                    INC_PART =   CONTRIB_REVPRIV_QT(PARTNER)+CONTRIB_REVPUB_QT(PARTNER) + &
                                &CONTRIB_INCSELF_QT(PARTNER)+PENSWELF_AMOUNTP_QT(PARTNER) + &
                                &SICK_BENINOC_QT(PARTNER)+SICK_NONEINOC_QT(PARTNER)
                    IF (ANY(UNEM_STATUS_QT(PARTNER).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/)).AND.&
                    &  (UNEM_BENUNPR_QT(PARTNER).LT.UNEM_WAGEDFQ8_QT).AND.&
                    &  (INT(100.D0*INC_PART).EQ.INT(100.D0*UNEM_WAGEDFQ7_QT))) THEN
                        UNEM_PRIVICOH_QT(pers_j) = 1
                    END IF
                END IF                
            !if the unemployed is single, set UNEM_FAMSIT_QT = 2
            ELSE IF (UNEM_STATUS_QT(pers_j).NE.0.AND.UNEM_HH_SIZE(hh_i).EQ.1) THEN
                UNEM_FAMSIT_QT(pers_j) = 2
            END IF
        END DO
    END DO  

END SUBROUTINE UNEM_FAMSIT_CONSTRUC

!=========================================================!
!With the subroutine UNEM_BENUIS_CONSTRUC we compute the  !
!unemployment benefits of unemployed in search of work    !
!Constructed variables                                    !
!   UNEM_BENSTUD_DAY                                      !
!   UNEM_BENSTUD_QT                                       !
!   UNEM_BENEMPL_DAY                                      !
!   UNEM_BENEMPL_QT                                       !  
!=========================================================!

SUBROUTINE UNEM_BENUIS_CONSTRUC(pers_j)

    INTEGER                         :: pers_j,i
    REAL(KIND=KIND_REAL)            :: ALDW     !Average limited daily wage of the unemployed before entering unemployment
    INTEGER                         :: TYPE_UN  !Identifier to identify the type of unemployment benefit (used to reduce the check on minima and maxima)
    
    !Compute unemployment benefits for unemployed after studies
    !UNEM_BENSTUD_DAY: Unemployment benefits per day for unemployed entitled to benefits after studies         
    !UNEM_BENSTUD_QT: Unemployment benefits per quarter for unemployed entitled to benefits after studies         
    IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/1,2/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
        !Determine the daily amount
        !distinguish the relevant cases conditional on the family charge of the unemployed
        IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
            UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(1)
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
            IF (UNEM_YEAR_AGE(pers_j).LT.18) THEN
                UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(2)
            ELSE IF (UNEM_YEAR_AGE(pers_j).GE.18.AND.UNEM_YEAR_AGE(pers_j).LT.21) THEN
                UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(3)
            ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21.AND.UNEM_YEAR_AGE(pers_j).LT.25) THEN
                UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(4)
            ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21) THEN
                UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(5)
            END IF
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
            !distinguish between unemployed without or with a partner with low unemployment benefits
            IF (UNEM_PRIVICOH_QT(pers_j).EQ.0) THEN
                IF (UNEM_YEAR_AGE(pers_j).LT.18) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(6)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.18.AND.UNEM_YEAR_AGE(pers_j).LT.21) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(7)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21.AND.UNEM_YEAR_AGE(pers_j).LT.25) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(8)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(9)
                END IF
            ELSE IF (UNEM_PRIVICOH_QT(pers_j).EQ.1) THEN
                IF (UNEM_YEAR_AGE(pers_j).LT.18) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(10)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.18.AND.UNEM_YEAR_AGE(pers_j).LT.21) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(11)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21.AND.UNEM_YEAR_AGE(pers_j).LT.25) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(12)
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.21) THEN
                    UNEM_BENSTUD_DAY(pers_j) = UNEM_LUSUWB_DAY(13)
                END IF
            END IF
        END IF
        !Compute the quarterly amount
        UNEM_BENSTUD_QT(pers_j) = UNEM_BENSTUD_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
        !Restriction of the duration of unemployment
        IF (UNEM_NUMMINU_QT(pers_j).GT.UNEM_DUURP) THEN
           UNEM_BENSTUD_QT(pers_j) = 0
        END IF
    END IF

    !Compute the unemployment benefits of unemployed in search of work
    !UNEM_BENEMPL_DAY: Unemployment benefits per day for unemployed entitled to benefits after employment          
    !UNEM_BENEMPL_QT: Unemployment benefits per quarter for unemployed entitled to benefits after employment          
    IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/3/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
        !Determine the daily amount
        !compute the limited lost wage
        ALDW = UNEM_AVGLWAGE_DAY(pers_j)
        IF (ALDW.GT.UNEM_LIMLOSW_DAY(3)) THEN
            IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                ALDW = UNEM_LIMLOSW_DAY(1)
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                ALDW = UNEM_LIMLOSW_DAY(2)
            ELSE
                ALDW = UNEM_LIMLOSW_DAY(3)
            END IF
        END IF
        !distinguish the relevant cases for the computation of the daily amount on the family charge of the unemployed
        !daily amount before correcting minima and maxima
        TYPE_UN = 0
        IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
            !Distinguish disabled from not disabled
            IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(1,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                TYPE_UN = 1
            ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(2,1)/100.D0)*ALDW
                    TYPE_UN = 2
                ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(3,1)/100.D0)*ALDW
                    TYPE_UN = 3
                ELSE
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(4,1)/100.D0)*ALDW
                    TYPE_UN = 4
                END IF
            END IF
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
            !Distinguish disabled from not disabled
            IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(5,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                TYPE_UN = 5
            ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(6,1)/100.D0)*ALDW
                    TYPE_UN = 6
                ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(7,1)/100.D0)*ALDW
                    TYPE_UN = 7
                ELSE
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(8,1)/100.D0)*ALDW
                    TYPE_UN = 8
                END IF
            END IF
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
            !Distinguish disabled from not disabled
            IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(9,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                TYPE_UN = 9
            ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(10,1)/100.D0)*ALDW
                    TYPE_UN = 10
                ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(11,1)/100.D0)*ALDW
                    TYPE_UN = 11
                !take into account the number of years worked as wage earner before unemployment    
                ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.12.AND.UNEM_NUMMINU_QT(pers_j).LE.(15+UNEM_NUMYWAGE_QT(pers_j)*3)) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(12,1)/100.D0)*ALDW
                    TYPE_UN = 12
                ELSE
                    !Remark: arbitrary nesting of conditions
                    !if worked for more than 20 years as wage earner
                    IF (UNEM_NUMYWAGE_QT(pers_j).GT.20) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(13,1)/100.D0)*ALDW
                        TYPE_UN = 13
                    !if disabled for at least 33 percent
                    ELSE IF (INT(100.D0*UNEM_DECREARN_QT(pers_j)).GE.33) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(14,1)/100.D0)*ALDW
                        TYPE_UN = 14
                    !In all other cases
                    ELSE
                        !distinguish between unemployed without or with a partner with low unemployment benefits
                        IF (UNEM_PRIVICOH_QT(pers_j).EQ.0) THEN
                            UNEM_BENEMPL_DAY(pers_j) = UNEM_LSCUB1_DAY
                            TYPE_UN = 15
                        ELSE IF (UNEM_PRIVICOH_QT(pers_j).EQ.1) THEN
                            UNEM_BENEMPL_DAY(pers_j) = UNEM_LSCUB1_DAY + UNEM_LSCUB2_DAY
                            TYPE_UN = 16
                        END IF
                    END IF
                END IF
            END IF
        END IF
        !correct minima and maxima
        IF (TYPE_UN.LE.14) THEN
            !loop over different unemployment types
            DO i=1,14
                IF (TYPE_UN.EQ.i) THEN
                    IF (UNEM_BENEMPL_DAY(pers_j).LT.UNEM_PERLIMUB_DAY(i,2)) THEN
                        UNEM_BENEMPL_DAY(pers_j) = UNEM_PERLIMUB_DAY(i,2)
                    END IF
                    IF (UNEM_BENEMPL_DAY(pers_j).GT.UNEM_PERLIMUB_DAY(i,3).AND.&
                    &   INT(100.D0*UNEM_PERLIMUB_DAY(i,3)).NE.0) THEN
                        UNEM_BENEMPL_DAY(pers_j) = UNEM_PERLIMUB_DAY(i,3)
                    END IF
                END IF
            END DO
        END IF
        !Compute the quarterly amount
        UNEM_BENEMPL_QT(pers_j) = UNEM_BENEMPL_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
        !Restriction of the duration of unemployment
        IF (UNEM_NUMMINU_QT(pers_j).GT.UNEM_DUURP) THEN
           UNEM_BENEMPL_QT(pers_j) = 0
        END IF      
    END IF

END SUBROUTINE UNEM_BENUIS_CONSTRUC

!=======================================================================!
!With the subroutine UNEM_BENREF_CONSTRUC we compute the                !
!reference unemployment benefits of unemployed in not search of work    !
!or employees entitled to RVA benefits.                                 !
!These benefits are computed as if the unemployed would                 !
!have been an unemployed not in search of work entitled                 !
!to benefits after employment                                           !
!Remark: Since additional information is lacking to say who would be    !
!entitled to school leaving benefits and benefits after employement, we !
!apply the rules as if all would be entitled to benefits after          !
!employment                                                             !
!Constructed variables                                                  !
!   UNEM_BENREF_DAY                                                     !
!   UNEM_BENREF_QT                                                      !
!=======================================================================!

SUBROUTINE UNEM_BENREF_CONSTRUC(pers_j)

    INTEGER                         :: pers_j,i
    REAL(KIND=KIND_REAL)            :: ALDW     !Average limited daily wage of the unemployed before entering unemployment
    INTEGER                         :: TYPE_UN  !Identifier to identify the type of unemployment benefit (used to reduce the check on minima and maxima)
    
    !Compute the unemployment benefits as if the the unemployed is in search of work after employment
    !UNEM_BENREF_DAY: Reference unemployment benefits per day 
    !UNEM_BENREF_QT: Reference unemployment benefits per quarter
    
    !Determine the daily amount
    !compute the limited lost wage
    ALDW = UNEM_AVGLWAGE_DAY(pers_j)
    IF (ALDW.GT.UNEM_LIMLOSW_REF_DAY(3)) THEN
        IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
            ALDW = UNEM_LIMLOSW_REF_DAY(1)
        ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
            ALDW = UNEM_LIMLOSW_REF_DAY(2)
        ELSE
            ALDW = UNEM_LIMLOSW_REF_DAY(3)
        END IF
    END IF
    !distinguish the relevant cases for the computation of the daily amount on the family charge of the unemployed
    !daily amount before correcting minima and maxima
    TYPE_UN = 0
    IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
        !Distinguish disabled from not disabled
        IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
            UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(1,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
            TYPE_UN = 1
        ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
            IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(2,1)/100.D0)*ALDW
                TYPE_UN = 2
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(3,1)/100.D0)*ALDW
                TYPE_UN = 3
            ELSE
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(4,1)/100.D0)*ALDW
                TYPE_UN = 4
            END IF
        END IF
    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
        !Distinguish disabled from not disabled
        IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
            UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(5,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
            TYPE_UN = 5
        ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
            IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(6,1)/100.D0)*ALDW
                TYPE_UN = 6
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(7,1)/100.D0)*ALDW
                TYPE_UN = 7
            ELSE
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(8,1)/100.D0)*ALDW
                TYPE_UN = 8
            END IF
        END IF
    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
        !Distinguish disabled from not disabled
        IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
            UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(9,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
            TYPE_UN = 9
        ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
            IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(10,1)/100.D0)*ALDW
                TYPE_UN = 10
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(11,1)/100.D0)*ALDW
                TYPE_UN = 11
            !take into account the number of years worked as wage earner before unemployment    
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.12.AND.UNEM_NUMMINU_QT(pers_j).LE.(15+UNEM_NUMYWAGE_QT(pers_j)*3)) THEN
                UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(12,1)/100.D0)*ALDW
                TYPE_UN = 12
            ELSE
                !Remark: arbitrary nesting of conditions
                !if worked for more than 20 years as wage earner
                IF (UNEM_NUMYWAGE_QT(pers_j).GT.20) THEN
                    UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(13,1)/100.D0)*ALDW
                    TYPE_UN = 13
                !if disabled for at least 33 percent
                ELSE IF (INT(100.D0*UNEM_DECREARN_QT(pers_j)).GE.33) THEN
                    UNEM_BENREF_DAY(pers_j) = (UNEM_PERLIMUB_REF_DAY(14,1)/100.D0)*ALDW
                    TYPE_UN = 14
                !In all other cases
                ELSE
                    !distinguish between unemployed without or with a partner with low unemployment benefits
                    IF (UNEM_PRIVICOH_QT(pers_j).EQ.0) THEN
                        UNEM_BENREF_DAY(pers_j) = UNEM_LSCUB1_DAY
                        TYPE_UN = 15
                    ELSE IF (UNEM_PRIVICOH_QT(pers_j).EQ.1) THEN
                        UNEM_BENREF_DAY(pers_j) = UNEM_LSCUB1_DAY + UNEM_LSCUB2_DAY
                        TYPE_UN = 16
                    END IF
                END IF
            END IF
        END IF
    END IF
    !correct minima and maxima
    IF (TYPE_UN.LE.14) THEN
        !loop over different unemployment types
        DO i=1,14
            IF (TYPE_UN.EQ.i) THEN
                IF (UNEM_BENREF_DAY(pers_j).LT.UNEM_PERLIMUB_REF_DAY(i,2)) THEN
                    UNEM_BENREF_DAY(pers_j) = UNEM_PERLIMUB_REF_DAY(i,2)
                END IF
                IF (UNEM_BENREF_DAY(pers_j).GT.UNEM_PERLIMUB_REF_DAY(i,3).AND.&
                &   INT(100.D0*UNEM_PERLIMUB_REF_DAY(i,3)).NE.0) THEN
                    UNEM_BENREF_DAY(pers_j) = UNEM_PERLIMUB_REF_DAY(i,3)
                END IF
            END IF
        END DO
    END IF
    !Compute the quarterly amount
    UNEM_BENREF_QT(pers_j) = UNEM_BENREF_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)

END SUBROUTINE UNEM_BENREF_CONSTRUC

!=========================================================!
!With the subroutine UNEM_BENUNIS_CONSTRUC we compute the !
!unemployment benefits of unemployed not in search of work!
!entitled to RVA benefits                                 !
!Constructed variables                                    !
!   UNEM_BENUNS1_QT                                       !
!   UNEM_ADDUNS_QT                                        !
!   UNEM_BENUNS2_QT                                       !
!   UNEM_BENUNS3_DAY                                      !
!   UNEM_BENUNS3_QT                                       !
!   UNEM_BENUNS4_QT                                       !
!=========================================================!

SUBROUTINE UNEM_BENUNIS_CONSTRUC(pers_j)

    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: UNEM_GTINC_ANN                       !Gross taxable income per year           
    REAL(KIND=KIND_REAL)            :: UNEM_AMOUNT_MONTH                    !monthly benefit selected from parameter
    INTEGER                         :: TYPE_UN                              !Identifier to identify the type of unemployment benefit (used to reduce the check on minima and maxima)
    REAL(KIND=KIND_REAL)            :: ALDW                                 !Average limited daily wage of the unemployed before entering unemployment

    !Unemployment benefits in case of conventional early retirement
    !The RVA benefits
    !UNEM_BENUNS1_QT: Quarterly early retirement benefit paid by the RVA to an unemployed not in search of work           
    IF (UNEM_ELIGUNS1_QT(pers_j).GT.0) THEN
        !Compute benefit as a percentage of lost but limited quarterly wages
        IF (UNEM_LOSWAGE_QT(pers_j).GT.0) THEN
            UNEM_BENUNS1_QT(pers_j) = (UNEM_EARPAR_QT(1,1)/100.D0)*MIN(UNEM_LOSWAGE_QT(pers_j),UNEM_LIMWAGE_QT)
        ELSE
            UNEM_BENUNS1_QT(pers_j) = (UNEM_EARPAR_QT(1,1)/100.D0)*UNEM_LIMWAGE_QT
        END IF
        !apply a lower and an upper limit    
        IF (UNEM_BENUNS1_QT(pers_j).LT.UNEM_EARPAR_QT(1,2)) THEN
            UNEM_BENUNS1_QT(pers_j) = UNEM_EARPAR_QT(1,2)
        END IF
        IF (UNEM_BENUNS1_QT(pers_j).GT.UNEM_EARPAR_QT(1,3)) THEN
            UNEM_BENUNS1_QT(pers_j) = UNEM_EARPAR_QT(1,3)
        END IF
        !Limiteer bedrag in functie van het aantal te vergoeden dagen
        UNEM_BENUNS1_QT(pers_j) = UNEM_BENUNS1_QT(pers_j)*(DAGEN_2008(pers_j)/312.D0)
    END IF

    !The early retirement benefits paid by the employer
    !UNEM_ADDUNS_QT: Quarterly early retirement benefit paid by the previous employer to an unemployed not in search of work         
    IF (UNEM_ELIGUNS1_QT(pers_j).GT.0) THEN
        !Compute annual income that serves as the basis for the computation of prepayments
        IF (UNEM_LOSWAGE_QT(pers_j).GT.0) THEN
            UNEM_GTINC_ANN = MIN(UNEM_LOSWAGE_QT(pers_j),UNEM_LIMEARR_QT)*4.D0 - &
            &(UNEM_SSCONT_QT/100.D0)*MIN(UNEM_LOSWAGE_QT(pers_j),UNEM_LIMEARR_QT)*4.D0
        ELSE
            UNEM_GTINC_ANN = UNEM_LIMEARR_QT*4.D0 - &
            &(UNEM_SSCONT_QT/100.D0)*UNEM_LIMEARR_QT*4.D0
        END IF
        !put gross income equal to 0 if negative
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute income net of prepayments divided by
        UNEM_ADDUNS_QT(pers_j) = ((UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
        &GECODEERD_INSZ_INPUT(pers_j)))/2.D0
    END IF
        
    !Unemployment benefits in case of career break
    !Benefit for common career break
    !UNEM_BENUNS2_QT: Quarterly career break benefit paid to unemployed not in search of work         
    IF (UNEM_ELIGUNS2_QT(pers_j).EQ.1) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPUNS_QT(pers_j).EQ.2) THEN
            IF (UNEM_CARBCON_QT(pers_j).EQ.2) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(1,1)
            ELSE IF (UNEM_CARBCON_QT(pers_j).EQ.2) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(2,1)
            ELSE IF (UNEM_CARBCON_QT(pers_j).EQ.3) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(3,1)
            ELSE IF (UNEM_CARBCON_QT(pers_j).EQ.4) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(1,2)
            ELSE IF (UNEM_CARBCON_QT(pers_j).EQ.5) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(2,2)
            ELSE IF (UNEM_CARBCON_QT(pers_j).EQ.6) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENCCB_MONTH(3,2)
            END IF
            UNEM_BENUNS2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPUNS_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefits for career break in case of palliative care
    IF (UNEM_ELIGUNS2_QT(pers_j).EQ.2) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPUNS_QT(pers_j).EQ.2) THEN
            IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENPCARE_MONTH(1,1)
            ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENPCARE_MONTH(1,2)
            END IF
            UNEM_BENUNS2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPUNS_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefits for career break in case of medical assistance
    IF (UNEM_ELIGUNS2_QT(pers_j).EQ.3) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPUNS_QT(pers_j).EQ.2) THEN
            IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENMCARE_MONTH(1,1)
            ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENMCARE_MONTH(1,2)
            END IF
            UNEM_BENUNS2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPUNS_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefits for career break in case of parental leave
    IF (UNEM_ELIGUNS2_QT(pers_j).EQ.4) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPUNS_QT(pers_j).EQ.2) THEN
            IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENPLEAV_MONTH(1,1)
            ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                UNEM_AMOUNT_MONTH = UNEM_BENPLEAV_MONTH(1,2)
            END IF
            UNEM_BENUNS2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPUNS_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Unemployment benefits for older unemployed with seniority supplement
    !UNEM_BENUNS3_DAY: Benefit per day for unemployed not in search of work who is entitled to benefits for older unemployed with seniority supplement         
    IF (UNEM_ELIGUNS3_QT(pers_j).EQ.1) THEN
        !Determine the daily amount
        !compute the limited lost wage
        ALDW = UNEM_AVGLWAGE_DAY(pers_j)
        IF (ALDW.GT.UNEM_LIMLOSW_REF_DAY(3)) THEN
            IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                ALDW = UNEM_LIMLOSW_REF_DAY(1)
            ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                ALDW = UNEM_LIMLOSW_REF_DAY(2)
            ELSE
                ALDW = UNEM_LIMLOSW_REF_DAY(3)
            END IF
        END IF
        !distinguish the relevant cases conditional on the family charge of the unemployed
        TYPE_UN = 0
        IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS3_DAY(pers_j) = UNEM_BENSSUP_DAY(1,1)
            TYPE_UN = 1
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
            IF (UNEM_YEAR_AGE(pers_j).GE.55) THEN
                UNEM_BENUNS3_DAY(pers_j) = (UNEM_BENSSUP_DAY(2,1)/100.D0)*ALDW
                TYPE_UN = 2
            ELSE 
                UNEM_BENUNS3_DAY(pers_j) = (UNEM_BENSSUP_DAY(3,1)/100.D0)*ALDW
                TYPE_UN = 3
            END IF
        ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
            !distinguish between unemployed on the time they are in unemployment
            IF (UNEM_NUMMINU_QT(pers_j).LE.15) THEN
                IF (UNEM_YEAR_AGE(pers_j).GE.55.AND.UNEM_YEAR_AGE(pers_j).LT.58) THEN
                    UNEM_BENUNS3_DAY(pers_j) = (UNEM_BENSSUP_DAY(4,1)/100.D0)*ALDW
                    TYPE_UN = 4
                ELSE IF (UNEM_YEAR_AGE(pers_j).GE.58) THEN
                    UNEM_BENUNS3_DAY(pers_j) = (UNEM_BENSSUP_DAY(5,1)/100.D0)*ALDW
                    TYPE_UN = 5
                ELSE
                    UNEM_BENUNS3_DAY(pers_j) = (UNEM_BENSSUP_DAY(6,1)/100.D0)*ALDW
                    TYPE_UN = 6
                END IF
            ELSE 
                UNEM_BENUNS3_DAY(pers_j) = UNEM_BENSSUP_DAY(7,1)
                TYPE_UN = 7
            END IF
        END IF
        !Compute the total daily benefit
        !i.e. the sum of the basic benefit and the daily supplyment
        UNEM_BENUNS3_DAY(pers_j) = UNEM_BENUNS3_DAY(pers_j) + UNEM_BENREF_DAY(pers_j)
        !apply the relevant minima and maxima on the total daily amount,
        !i.e. the sum of the basic benefit and the daily supplyment
        IF (TYPE_UN.LE.7) THEN
            !loop over different unemployment types
            DO i=1,7
                IF (TYPE_UN.EQ.i) THEN
                    IF (UNEM_BENUNS3_DAY(pers_j).LT.UNEM_BENSSUP_DAY(i,2)) THEN
                        UNEM_BENUNS3_DAY(pers_j) = UNEM_BENSSUP_DAY(i,2)
                    END IF
                    IF (UNEM_BENUNS3_DAY(pers_j).GT.UNEM_BENSSUP_DAY(i,3).AND.&
                    &   INT(100.D0*UNEM_BENSSUP_DAY(i,3)).NE.0) THEN
                        UNEM_BENUNS3_DAY(pers_j) = UNEM_BENSSUP_DAY(i,3)
                    END IF
                END IF
            END DO
        END IF
        !Compute the quarterly amount
        UNEM_BENUNS3_QT(pers_j) = UNEM_BENUNS3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
    END IF

    !Unemployment benefits in case of exemption
    !If exemption for social or familial reasons
    !UNEM_BENUNS4_QT: Benefit per quarter for unemployed not in search of work who is entitled to benefits in case of exemption           
    IF (UNEM_ELIGUNS4_QT(pers_j).EQ.1) THEN
        IF (UNEM_EXEMP_QT(pers_j).EQ.1) THEN
            UNEM_BENUNS4_QT(pers_j) = UNEM_EXEMPSF_DAY(1)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_EXEMP_QT(pers_j).EQ.2) THEN 
            UNEM_BENUNS4_QT(pers_j) = UNEM_EXEMPSF_DAY(2)*(DAGEN_2008(pers_j)/4.D0)
        ELSE
            UNEM_BENUNS4_QT(pers_j) = UNEM_EXEMPSF_DAY(3)*(DAGEN_2008(pers_j)/4.D0)
        END IF
    !if exemption because of professional training
    ELSE IF (UNEM_ELIGUNS4_QT(pers_j).EQ.2) THEN
        UNEM_BENUNS4_QT(pers_j) = UNEM_BENREF_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
    END IF

END SUBROUTINE UNEM_BENUNIS_CONSTRUC

!=========================================================!
!With the subroutine UNEM_BENEMP_CONSTRUC we compute the  !
!unemployment benefits of employees entitled to RVA       !
!benefits                                                 !
!Constructed variables                                    !
!   UNEM_BENEMP1_QT                                       !
!   UNEM_ADDEMP_QT                                        !  
!   UNEM_BENEMP2_QT                                       !
!   UNEM_BENEMP3_DAY                                      !
!   UNEM_BENEMP3_QT                                       !
!   UNEM_BENEMP4_QT                                       !
!   UNEM_BENEMP5_QT                                       !
!=========================================================!

SUBROUTINE UNEM_BENEMP_CONSTRUC(pers_j)

    INTEGER                         :: pers_j
    REAL(KIND=KIND_REAL)            :: UNEM_GTINC_ANN                       !Gross taxable income per year           
    REAL(KIND=KIND_REAL)            :: UNEM_AMOUNT_MONTH                    !monthly benefit selected from parameter
    REAL(KIND=KIND_REAL)            :: UNEM_NETW_QT                         !quarterly net income from labour activity (i.e. private and public labour market combined)
    REAL(KIND=KIND_REAL)            :: UNEM_FRACWORK                        !Number of hours worked expressed as fraction of a full time
    REAL(KIND=KIND_REAL)            :: UNEM_FICWAGE_QT                      !Fictive wage earned by the guaranteed income earner
    REAL(KIND=KIND_REAL)            :: UNEM_LIMBEN_QT                       !The guaranteed benefit should be limited to the ficitive wage minus the net earned wage

    !Benefits in case of part time early retirement
    !The RVA benefits
    !UNEM_BENEMP1_QT: Quarterly early retirement benefit paid by the RVA to an employee entitled to part time early retirement benefits           
    IF (UNEM_ELIGEMP1_QT(pers_j).GT.0) THEN
        !Compute benefit as a lump sum amount
        UNEM_BENEMP1_QT(pers_j) = UNEM_LSPEAR_DAY*(DAGEN_2008(pers_j)/4.D0) 
    END IF

    !The early retirement benefits paid by the employer
    !UNEM_ADDEMP_QT: Quarterly early retirement benefit paid by the employer to an employee entitled to part time early retirement benefits          
    IF (UNEM_ELIGEMP1_QT(pers_j).GT.0) THEN
        !Compute annual income that serves as the basis for the computation of prepayments
        IF (UNEM_LOSWAGE_QT(pers_j).GT.0) THEN
            UNEM_GTINC_ANN = MIN(UNEM_LOSWAGE_QT(pers_j),UNEM_LIMEARR_QT)*4.D0 - &
            &(UNEM_SSCONT_QT/100.D0)*MIN(UNEM_LOSWAGE_QT(pers_j),UNEM_LIMEARR_QT)*4.D0
        ELSE
            UNEM_GTINC_ANN = UNEM_LIMEARR_QT*4.D0 - &
            &(UNEM_SSCONT_QT/100.D0)*UNEM_LIMEARR_QT*4.D0
        END IF
        !Put gross income equal to 0 if negative
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute income net of prepayments divided by
        UNEM_ADDEMP_QT(pers_j) = ((UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),&
        &GECODEERD_INSZ_INPUT(pers_j)))/4.D0 - (5.D0/4.D0)*UNEM_BENEMP1_QT(pers_j)
        IF (UNEM_ADDEMP_QT(pers_j).LT.0) THEN
            UNEM_ADDEMP_QT(pers_j) = 0
        END IF
    END IF

    !Benefits in case of part time career break
    !Benefit for common part time career break
    !UNEM_BENEMP2_QT: Quarterly career break benefits paid to employee entitled to part time career break benefits            
    IF (UNEM_ELIGEMP2_QT(pers_j).EQ.1) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        !Bij part time job raam je uitkering. Anders neem je het geobserveerde bedrag
        IF (UNEM_FPEMP_QT(pers_j).EQ.2) THEN
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENCCM50_MONTH((UNEM_CARBCON_QT(pers_j)-1)*4+i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENCCM50_MONTH((UNEM_CARBCON_QT(pers_j)-4)*4+i,2)
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENCCP50_MONTH((UNEM_CARBCON_QT(pers_j)-1)*4+i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENCCP50_MONTH((UNEM_CARBCON_QT(pers_j)-4)*4+i,2)
                        END IF
                    END DO
                END IF
            END IF
            UNEM_BENEMP2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPEMP_QT(pers_j).EQ.1) THEN
            UNEM_BENEMP2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefit for part time career break in case of palliative care
    IF (UNEM_ELIGEMP2_QT(pers_j).EQ.2) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPEMP_QT(pers_j).EQ.2) THEN
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPCM50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPCM50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPCP50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPCP50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            END IF
            UNEM_BENEMP2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPEMP_QT(pers_j).EQ.1) THEN
            UNEM_BENEMP2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefit for part time career break in case of medical assistance
    IF (UNEM_ELIGEMP2_QT(pers_j).EQ.3) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPEMP_QT(pers_j).EQ.2) THEN
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENMCM50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENMCM50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENMCP50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENMCP50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            END IF
            UNEM_BENEMP2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPEMP_QT(pers_j).EQ.1) THEN
            UNEM_BENEMP2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefit for part time career break in case of parental leave
    IF (UNEM_ELIGEMP2_QT(pers_j).EQ.4) THEN
        UNEM_AMOUNT_MONTH = 0
        !distinguish people who leave a full time job from those who leave a part time job
        IF (UNEM_FPEMP_QT(pers_j).EQ.2) THEN
            !Table to be checked differs with age
            IF (UNEM_YEAR_AGE(pers_j).LT.50) THEN
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPLM50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPLM50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            ELSE
                !selection depends on time in career break and eligility for supplement
                IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/1, 2, 3/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPLP50_MONTH(i,1)
                        END IF
                    END DO
                ELSE IF (ANY(UNEM_CARBCON_QT(pers_j).EQ.(/4, 5, 6/))) THEN
                    !loop over possible values
                    DO i=1,4
                        IF (UNEM_TIMEUNS_QT(pers_j).EQ.i) THEN
                            UNEM_AMOUNT_MONTH = UNEM_BENPLP50_MONTH(i,2)
                        END IF
                    END DO
                END IF
            END IF
            UNEM_BENEMP2_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        ELSE IF (UNEM_FPEMP_QT(pers_j).EQ.1) THEN
            UNEM_BENEMP2_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
        END IF
    END IF

    !Benefits in case of guaranteed income benefits of the RVA
    !UNEM_BENEMP3_DAY: Guaranteed income benefit for a day paid by the RVA to an employee entitled to these benefits           
    !UNEM_BENEMP3_QT: Guaranteed income benefit per quarter paid by the RVA to an employee entitled to these benefits         
    IF (UNEM_ELIGEMP3_QT(pers_j).EQ.1) THEN
        !STEP 1: Compute the reference benefit
        !Benefits in case of guaranteed income benefits of the RVA
        !UNEM_BENEMP3_DAY: Guaranteed income benefit for a day paid by the RVA to an employee entitled to these benefits           
        !UNEM_BENEMP3_QT: Guaranteed income benefit per quarter paid by the RVA to an employee entitled to these benefits         
        !Compute the normal daily and quarterly benefit
        UNEM_BENEMP3_DAY(pers_j)    = UNEM_BENREF_DAY(pers_j) 
        !Veronderstel hier 26 vergoedbare dagen voor individuen met recht op benefit en niet aantal dagen gedeeld door 4
!        UNEM_BENEMP3_QT(pers_j)     = UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
        UNEM_BENEMP3_QT(pers_j)     = UNEM_BENEMP3_DAY(pers_j)*(26.D0*3.D0)
        UNEM_AMOUNT_MONTH   = 0
        !Pas werktijd aan: in principe geobserveerde waarde maar deze is gebaseerd op een jaarschatting met arbeidsmarkt-gegevens
        !als de geobserveerde waarde buiten de normale range valt, dan klopt er iets niet want op basis van de gegevens weten we
        !dat het individu IGU gerechtigd was voor een bepaalde periode. We leggen daarom het aantal uur voor de IGU berekening 
        !hier kunstmatig vast
        IF (CONTRIB_WORKHOUR(pers_j).LT.12.66D0) THEN
            CONTRIB_WORKHOUR(pers_j) = 19.D0
        ELSE IF (CONTRIB_WORKHOUR(pers_j).GT.30.4D0) THEN
            CONTRIB_WORKHOUR(pers_j) = 19.D0
        END IF
        !STEP 2: Add the additional benefit
        !Select the monthly amount of the additonal benefit
        !Distinguish the rules before and after 2005
        UNEM_FRACWORK       = CONTRIB_WORKHOUR(pers_j)/38.D0
        !If worked hours is lower than 33% of full time, the switch benefit to 0
        IF (UNEM_FRACWORK.GT.0.D0.AND.UNEM_FRACWORK.LT.0.3333D0) THEN 
            UNEM_BENEMP3_QT(pers_j)     = 0.D0
        ELSE IF (UNEM_FRACWORK.GE.0.3333D0.AND.UNEM_FRACWORK.LT.0.8000D0) THEN 
            IF (SIMYEAR.LT.2006) THEN
            !Select the monthly amount, selection depends on the family charge of the unemployed
                IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                    UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(1,1)
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                    UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(2,1)
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                    UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(3,1)
                END IF
            ELSE
                IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                    UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(1)
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                    UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(2)
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                    UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(3)
                END IF
            END IF
        END IF
        !add quarterly supplement (i.e. month times 3) to quarterly benefits
        UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j) + 3.D0*UNEM_AMOUNT_MONTH
        !STEP 3: Subtract the net wage from the benefit
        !Compute quarterly benefit minus net wages
        !first determine gross taxable income per year
!Hier bepaald op fictieve basis ipv geobserveerd inkomen te gebruiken
!Voordeel geobserveerd inkomen is dat het juist bijdrageverminderingen heeft
!Nadeel is dat het om een bedrag op jaarbasis gaat, terwijl je hier het bedrag nodig hebt dat
!verdiend is in de maand waarin het recht op de IGU geopend wordt
!        UNEM_GTINC_ANN = ((CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))-&
!        &(CONTRIB_SSPRIV_EMEE_QT(pers_j) + CONTRIB_SSPUB_EMEE_QT(pers_j)))*4.D0
        UNEM_GTINC_ANN = 52.D0*CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j)-&
        &0.1307*(52.D0*CONTRIB_WORKHOUR(pers_j)*MIMOSIS_GRINC_HOUR(pers_j))
        !Put gross income equal to 0 if negative
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute the quarterly amount of prepayments 
        !so that net income can be subtracted from unemployment benefit
        UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j) - ((UNEM_GTINC_ANN/4.D0) - &
        &UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j)))
        !Set benefits equal to 0 if they become negative
        IF (UNEM_BENEMP3_QT(pers_j).LT.0) THEN
            UNEM_BENEMP3_QT(pers_j) = 0
        END IF
        !STEP 4: The total benefit can not be larger than a given amount
        !Distinguish the rules before and after 2005
        !Correct maximal amount
        IF (SIMYEAR.LT.2006) THEN
            !Check whether benefit does not exceed a percentage treshold of the normal benefit
            !% to be checked also depends on family type
            IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(1,2)/100.D0)*&
                    &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                        UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(1,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
                END IF
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(2,2)/100.D0)*&
                    &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                        UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(2,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
                END IF
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(3,2)/100.D0)*&
                    &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                        UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(3,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
                END IF
            END IF
        ELSE
            !Compute the quarterly ficitive wage at full time
            !We do not take into account the wage bonus here (is system from August 2008 on)
            UNEM_FICWAGE_QT = 13.0D0*38.0D0*MIMOSIS_GRINC_HOUR(pers_j) - 0.1307D0*(13.D0*38.D0*MIMOSIS_GRINC_HOUR(pers_j))
            !Compute the amount that serves as a limit for the guaranteed income benefit, i.e. the fictive wage at full
            !time minus the net wage
            UNEM_LIMBEN_QT = (UNEM_FICWAGE_QT - &
            &UNEM_PREPAY_COMPUTE(UNEM_FICWAGE_QT*4,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j))) - &
            &((UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j)))
            !Set limit equal to 0 if they become negative
            IF (UNEM_LIMBEN_QT.LT.0) THEN
                UNEM_LIMBEN_QT = 0
            END IF
            !Test whether the benefit exceeds the limit. If so limit it.
            IF (UNEM_BENEMP3_QT(pers_j).GT.UNEM_LIMBEN_QT) THEN
                UNEM_BENEMP3_QT(pers_j) = UNEM_LIMBEN_QT
            END IF
        END IF
        !STEP 5: The amount computed so far is a net amount and must be converted to 
        !a gross amount (including the tax pre-payment of 10.09%)
        UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j)*(100.D0/89.91D0) 
        IF (UNEM_BENEMP3_QT(pers_j).EQ.0.D0.AND.BEDRAGEN_2008(pers_j).GT.0.D0) THEN
            UNEM_BENEMP3_QT(pers_j) = BEDRAGEN_2008(pers_j)/4.D0
        END IF
    END IF

    !Benefits in case of temporary unemployment
    !UNEM_BENEMP4_QT: Quarterly benefits paid to employee entitled to temporary unemployment benefits         
    IF (UNEM_ELIGEMP4_QT(pers_j).EQ.1) THEN
        UNEM_BENEMP4_QT(pers_j) = (BEDRAGEN_2008(pers_j)/4.D0)
    END IF

    !Benefits paid to people entitled to PWA benefits
    !UNEM_BENEMP5_DAY: PWA benefits paid per day           
    !UNEM_BENEMP5_QT: Quarterly amount of PWA benefits            
    IF (UNEM_ELIGEMP5_QT(pers_j).EQ.1) THEN
        !Compute the normal daily and quarterly benefit
        UNEM_BENEMP5_DAY(pers_j) = UNEM_BENREF_DAY(pers_j) 
        UNEM_BENEMP5_QT(pers_j) = UNEM_BENEMP5_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0)
        !Subtract and add the lump sum benefits for each hour worked with a PWA contract
        UNEM_BENEMP5_QT(pers_j) = UNEM_BENEMP5_QT(pers_j) + (UNEM_PWAPLUS_HOUR - UNEM_PWAMIN_HOUR)*(URENPWA_2008(pers_j)/4.D0)
        !Set benefits equal to 0 if they become negative
        IF (UNEM_BENEMP5_QT(pers_j).LT.0) THEN
            UNEM_BENEMP5_QT(pers_j) = 0
        END IF
    END IF

    !UNEM_BENEMP6_QT: Quarterly benefits paid to employee entitled to benefits for activation program other than PWA program          
    !Benefits from a transition program
    IF (UNEM_ELIGEMP6_QT(pers_j).EQ.1) THEN
        !Select the monthly benefit
        UNEM_AMOUNT_MONTH = 0
        !the selection depends on 3 different conditions
        !A: the fraction of work time compared to a full time employee
        !the employee performs between 1/2 and 4/5
        IF (UNEM_FRACTRAN_QT(pers_j).EQ.1) THEN
            !B: A high unemployment rate in town yes or no
            IF (UNEM_TOWNRATE_QT(pers_j).EQ.2) THEN
                UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(1,3)
            ELSE
                !C: Number of PWA hours performed in the last 6 months            
                IF (UNEM_PWAHOURS_QT(pers_j).EQ.2) THEN
                    UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(1,2)
                ELSE
                    UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(1,1)
                END IF
            END IF
        !the employee performs 4/5 or more
        ELSE IF (UNEM_FRACTRAN_QT(pers_j).EQ.2) THEN
            !B: A high unemployment rate in town yes or no
            IF (UNEM_TOWNRATE_QT(pers_j).EQ.2) THEN
                UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(2,3)
            ELSE
                !C: Number of PWA hours performed in the last 6 months            
                IF (UNEM_PWAHOURS_QT(pers_j).EQ.2) THEN
                    UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(2,2)
                ELSE
                    UNEM_AMOUNT_MONTH = UNEM_TRANSBEN_MONTH(2,1)
                END IF
            END IF
        END IF
        UNEM_BENEMP6_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        !Compute quarterly net wages
        !first determine gross taxable income per year
        UNEM_GTINC_ANN = ((CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))-&
        &(CONTRIB_SSPRIV_EMEE_QT(pers_j) + CONTRIB_SSPUB_EMEE_QT(pers_j)))*4.D0
        !Put gross income equal to 0 if negative
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute the quarterly amount of prepayments 
        UNEM_NETW_QT = 0
        UNEM_NETW_QT = (UNEM_GTINC_ANN/4.D0) - &
        &UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j))
        !limit unemployment benefit to net wage if necessary
        IF (INT(100.D0*UNEM_NETW_QT).GT.0.AND.UNEM_NETW_QT.LT.UNEM_BENEMP6_QT(pers_j)) THEN
            UNEM_BENEMP6_QT(pers_j) = UNEM_NETW_QT
        END IF
    END IF

    !Benefits from a reinsertion program
    IF (UNEM_ELIGEMP6_QT(pers_j).EQ.2) THEN
        !Select the monthly benefit
        UNEM_AMOUNT_MONTH = 0
        !the selection depends on 2 different conditions
        !A: the time already active in the reinsertion program
        !the employee is active for 36 months or less in the reinsertion program
        IF (UNEM_DUURREIN_QT(pers_j).EQ.1) THEN
            !B: the fraction of work time compared to a full time employee
            !the employee performs between 1/2 and 4/5
            IF (UNEM_FRACREIN_QT(pers_j).EQ.1) THEN
                UNEM_AMOUNT_MONTH = UNEM_REINBEN_MONTH(1)
            !the employee performs 4/5 or more
            ELSE IF (UNEM_FRACREIN_QT(pers_j).EQ.2) THEN
                UNEM_AMOUNT_MONTH = UNEM_REINBEN_MONTH(2)
            END IF
        !the employee is active for more than 36 months in the reinsertion program    
        ELSE IF (UNEM_DUURREIN_QT(pers_j).EQ.2) THEN
            UNEM_AMOUNT_MONTH = UNEM_REINBEN_MONTH(3)
        END IF
        UNEM_BENEMP6_QT(pers_j) = (UNEM_AMOUNT_MONTH/26.D0)*(DAGEN_2008(pers_j)/4.D0)
        !Compute quarterly net wages
        !first determine gross taxable income per year
        UNEM_GTINC_ANN = ((CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))-&
        &(CONTRIB_SSPRIV_EMEE_QT(pers_j) + CONTRIB_SSPUB_EMEE_QT(pers_j)))*4.D0
        !Put gross income equal to 0 if negative
        IF (UNEM_GTINC_ANN.LT.0) THEN
            UNEM_GTINC_ANN = 0
        END IF
        !Use the function UNEM_PREPAY_COMPUTE to compute the quarterly amount of prepayments 
        UNEM_NETW_QT = 0
        UNEM_NETW_QT = (UNEM_GTINC_ANN/4.D0) - &
        &UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j))
        !limit unemployment benefit to net wage if necessary
        IF (INT(100.D0*UNEM_NETW_QT).GT.0.AND.UNEM_NETW_QT.LT.UNEM_BENEMP6_QT(pers_j)) THEN
            UNEM_BENEMP6_QT(pers_j) = UNEM_NETW_QT
        END IF
    END IF
    
    !Benefits not modelled
    IF (UNEM_STATUS_QT(pers_j).EQ.12.AND.UNEM_BENEMP6_QT(pers_j).EQ.0.D0) THEN
        UNEM_BENEMP6_QT(pers_j) = BEDRAGEN_2008(pers_j)/4.D0
    END IF
    
END SUBROUTINE UNEM_BENEMP_CONSTRUC

!===========================================================!
!With the subroutine UNEM_ADASIMMT we adapt the             !
!unemployment benefits of those for which the marginal tax  !
!simulation has been carried out                            !
!===========================================================!

SUBROUTINE UNEM_ADASIMMT(pers_j)

    INTEGER                         :: pers_j,i
    REAL(KIND=KIND_REAL)            :: ALDW                                 !Average limited daily wage of the unemployed before entering unemployment
    INTEGER                         :: TYPE_UN                              !Identifier to identify the type of unemployment benefit (used to reduce the check on minima and maxima)
    REAL(KIND=KIND_REAL)            :: UNEM_FRACWORK                        !Number of hours worked expressed as fraction of a full time
    REAL(KIND=KIND_REAL)            :: UNEM_GTINC_ANN                       !Gross taxable income per year           
    REAL(KIND=KIND_REAL)            :: UNEM_AMOUNT_MONTH                    !monthly benefit selected from parameter
    REAL(KIND=KIND_REAL)            :: UNEM_FICWAGE_QT                      !Fictive wage earned by the guaranteed income earner
    REAL(KIND=KIND_REAL)            :: UNEM_LIMBEN_QT                       !The guaranteed benefit should be limited to the ficitive wage minus the net earned wage
    
    !Distinguish cases where simulated number of hours worked is 0 or not
    IF (INT(100.D0*CONTRIB_WORKHOUR(pers_j)).EQ.0) THEN
        !Compute a benefit for those not receiving one yet
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1,2/)).OR.ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1,2/))) THEN
            !Fill in the unemployment benefit variables that capture the benefit after work
            !UNEM_BENEMPL_DAY: Unemployment benefits per day for unemployed entitled to benefits after employment          
            !UNEM_BENEMPL_QT: Unemployment benefits per quarter for unemployed entitled to benefits after employment          
            !Determine the daily amount
            !compute the limited lost wage
            ALDW = MIMOSIS_GRINC_HOUR(pers_j)*7.6D0
            IF (ALDW.GT.UNEM_LIMLOSW_DAY(3)) THEN
                IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                    ALDW = UNEM_LIMLOSW_DAY(1)
                ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                    ALDW = UNEM_LIMLOSW_DAY(2)
                ELSE
                    ALDW = UNEM_LIMLOSW_DAY(3)
                END IF
            END IF
            !distinguish the relevant cases for the computation of the daily amount on the family charge of the unemployed
            !daily amount before correcting minima and maxima
            TYPE_UN = 0
            IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                !Distinguish disabled from not disabled
                IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(1,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                    TYPE_UN = 1
                ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                    IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(2,1)/100.D0)*ALDW
                        TYPE_UN = 2
                    ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(3,1)/100.D0)*ALDW
                        TYPE_UN = 3
                    ELSE
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(4,1)/100.D0)*ALDW
                        TYPE_UN = 4
                    END IF
                END IF
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                !Distinguish disabled from not disabled
                IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(5,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                    TYPE_UN = 5
                ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                    IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(6,1)/100.D0)*ALDW
                        TYPE_UN = 6
                    ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(7,1)/100.D0)*ALDW
                        TYPE_UN = 7
                    ELSE
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(8,1)/100.D0)*ALDW
                        TYPE_UN = 8
                    END IF
                END IF
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                !Distinguish disabled from not disabled
                IF (UNEM_DISABLED_QT(pers_j).EQ.1) THEN
                    UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(9,1)/100.D0)*UNEM_AVGLWAGE_DAY(pers_j)
                    TYPE_UN = 9
                ELSE IF (UNEM_DISABLED_QT(pers_j).EQ.0) THEN
                    IF (UNEM_NUMMINU_QT(pers_j).LE.6) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(10,1)/100.D0)*ALDW
                        TYPE_UN = 10
                    ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.6.AND.UNEM_NUMMINU_QT(pers_j).LE.12) THEN   
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(11,1)/100.D0)*ALDW
                        TYPE_UN = 11
                    !take into account the number of years worked as wage earner before unemployment    
                    ELSE IF (UNEM_NUMMINU_QT(pers_j).GT.12.AND.UNEM_NUMMINU_QT(pers_j).LE.(15+UNEM_NUMYWAGE_QT(pers_j)*3)) THEN
                        UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(12,1)/100.D0)*ALDW
                        TYPE_UN = 12
                    ELSE
                        !Remark: arbitrary nesting of conditions
                        !if worked for more than 20 years as wage earner
                        IF (UNEM_NUMYWAGE_QT(pers_j).GT.20) THEN
                            UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(13,1)/100.D0)*ALDW
                            TYPE_UN = 13
                        !if disabled for at least 33 percent
                        ELSE IF (INT(100.D0*UNEM_DECREARN_QT(pers_j)).GE.33) THEN
                            UNEM_BENEMPL_DAY(pers_j) = (UNEM_PERLIMUB_DAY(14,1)/100.D0)*ALDW
                            TYPE_UN = 14
                        !In all other cases
                        ELSE
                            !distinguish between unemployed without or with a partner with low unemployment benefits
                            IF (UNEM_PRIVICOH_QT(pers_j).EQ.0) THEN
                                UNEM_BENEMPL_DAY(pers_j) = UNEM_LSCUB1_DAY
                                TYPE_UN = 15
                            ELSE IF (UNEM_PRIVICOH_QT(pers_j).EQ.1) THEN
                                UNEM_BENEMPL_DAY(pers_j) = UNEM_LSCUB1_DAY + UNEM_LSCUB2_DAY
                                TYPE_UN = 16
                            END IF
                        END IF
                    END IF
                END IF
            END IF
            !correct minima and maxima
            IF (TYPE_UN.LE.14) THEN
                !loop over different unemployment types
                DO i=1,14
                    IF (TYPE_UN.EQ.i) THEN
                        IF (UNEM_BENEMPL_DAY(pers_j).LT.UNEM_PERLIMUB_DAY(i,2)) THEN
                            UNEM_BENEMPL_DAY(pers_j) = UNEM_PERLIMUB_DAY(i,2)
                        END IF
                        IF (UNEM_BENEMPL_DAY(pers_j).GT.UNEM_PERLIMUB_DAY(i,3).AND.&
                        &   INT(100.D0*UNEM_PERLIMUB_DAY(i,3)).NE.0) THEN
                            UNEM_BENEMPL_DAY(pers_j) = UNEM_PERLIMUB_DAY(i,3)
                        END IF
                    END IF
                END DO
            END IF
            !Compute the quarterly amount
            UNEM_BENEMPL_QT(pers_j) = UNEM_BENEMPL_DAY(pers_j)*78.D0
            !Restriction of the duration of unemployment
            IF (UNEM_NUMMINU_QT(pers_j).GT.UNEM_DUURP) THEN
               UNEM_BENEMPL_QT(pers_j) = 0
            END IF  
        END IF
    ELSE
        !Apply the rules to compute the benefit including additional income guarantee
        !Compute a benefit for a limited number of people
        IF (ANY(CONTRIB_LABMSTAT_PRIV_QT(pers_j).EQ.(/1,2/)).OR.&
            &ANY(CONTRIB_LABMSTAT_PUB_QT(pers_j).EQ.(/1,2/)).OR.&
            &ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2/))) THEN
            !STEP 1: Compute the reference benefit
            !Benefits in case of guaranteed income benefits of the RVA
            !UNEM_BENEMP3_DAY: Guaranteed income benefit for a day paid by the RVA to an employee entitled to these benefits           
            !UNEM_BENEMP3_QT: Guaranteed income benefit per quarter paid by the RVA to an employee entitled to these benefits         
            !Compute the normal daily and quarterly benefit
            UNEM_BENEMP3_DAY(pers_j)    = UNEM_BENREF_DAY(pers_j) 
            UNEM_BENEMP3_QT(pers_j)     = UNEM_BENEMP3_DAY(pers_j)*78.D0
            !STEP 2: Add the additional benefit
            !Select the monthly amount of the additonal benefit
            !Distinguish the rules before and after 2005
            UNEM_AMOUNT_MONTH   = 0
            UNEM_FRACWORK       = CONTRIB_WORKHOUR(pers_j)/38.D0
            !If worked hours is lower than 33% of full time, the switch benefit to 0
            IF (UNEM_FRACWORK.LT.0.3333D0) THEN 
                UNEM_BENEMP3_QT(pers_j)     = 0.D0
            ELSE IF (UNEM_FRACWORK.GE.0.3333D0.AND.UNEM_FRACWORK.LT.0.8000D0) THEN 
                IF (SIMYEAR.LT.2006) THEN
                !Select the monthly amount, selection depends on the family charge of the unemployed
                    IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                        UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(1,1)
                    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                        UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(2,1)
                    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                        UNEM_AMOUNT_MONTH = UNEM_GUARINC_MONTH(3,1)
                    END IF
                ELSE
                    IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                        UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(1)
                    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                        UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(2)
                    ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                        UNEM_AMOUNT_MONTH = (4.3333D0*CONTRIB_WORKHOUR(pers_j) - 55.0D0)*UNEM_LSINGUAR_HOUR(3)
                    END IF
                END IF
            END IF
            !add quarterly supplement (i.e. month times 3) to quarterly benefits
            UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j) + 3.D0*UNEM_AMOUNT_MONTH
            !STEP 3: Subtract the net wage from the benefit
            !Compute quarterly benefit minus net wages
            !first determine gross taxable income per year
            UNEM_GTINC_ANN = ((CONTRIB_REVPRIV_QT(pers_j) + CONTRIB_REVPUB_QT(pers_j))-&
            &(CONTRIB_SSPRIV_EMEE_QT(pers_j) + CONTRIB_SSPUB_EMEE_QT(pers_j)))*4.D0
            !Put gross income equal to 0 if negative
            IF (UNEM_GTINC_ANN.LT.0) THEN
                UNEM_GTINC_ANN = 0
            END IF
            !Use the function UNEM_PREPAY_COMPUTE to compute the quarterly amount of prepayments 
            !so that net income can be subtracted from unemployment benefit
            UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j) - ((UNEM_GTINC_ANN/4.D0) - &
            &UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j)))
            !Set benefits equal to 0 if they become negative
            IF (UNEM_BENEMP3_QT(pers_j).LT.0) THEN
                UNEM_BENEMP3_QT(pers_j) = 0
            END IF
            !STEP 4: The total benefit can not be larger than a given amount
            !Distinguish the rules before and after 2005
            !Correct maximal amount
            IF (SIMYEAR.LT.2006) THEN
                !Check whether benefit does not exceed a percentage treshold of the normal benefit
                !% to be checked also depends on family type
                IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                    IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(1,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                            UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(1,2)/100.D0)*&
                            &UNEM_BENEMP3_DAY(pers_j)*78.D0
                    END IF
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                    IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(2,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                            UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(2,2)/100.D0)*&
                            &UNEM_BENEMP3_DAY(pers_j)*78.D0
                    END IF
                ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                    IF (UNEM_BENEMP3_QT(pers_j).GT.((UNEM_GUARINC_MONTH(3,2)/100.D0)*&
                        &UNEM_BENEMP3_DAY(pers_j)*(DAGEN_2008(pers_j)/4.D0))) THEN
                            UNEM_BENEMP3_QT(pers_j) = (UNEM_GUARINC_MONTH(3,2)/100.D0)*&
                            &UNEM_BENEMP3_DAY(pers_j)*78.D0
                    END IF
                END IF
            ELSE
                !Compute the quarterly ficitive wage at full time
                UNEM_FICWAGE_QT = 13.0D0*38.0D0*MIMOSIS_GRINC_HOUR(pers_j) - 0.1307D0*(13.D0*38.D0*MIMOSIS_GRINC_HOUR(pers_j))
                !Compute the amount that serves as a limit for the guaranteed income benefit, i.e. the fictive wage at full
                !time minus the net wage
                UNEM_LIMBEN_QT = (UNEM_FICWAGE_QT - &
                &UNEM_PREPAY_COMPUTE(UNEM_FICWAGE_QT*4,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j))) - &
                &((UNEM_GTINC_ANN/4.D0) - UNEM_PREPAY_COMPUTE(UNEM_GTINC_ANN,PIT_DEPCHILD_ANN(pers_j),GECODEERD_INSZ_INPUT(pers_j)))
                !Set limit equal to 0 if they become negative
                IF (UNEM_LIMBEN_QT.LT.0) THEN
                    UNEM_LIMBEN_QT = 0
                END IF
                !Test whether the benefit exceeds the limit. If so limit it.
                IF (UNEM_BENEMP3_QT(pers_j).GT.UNEM_LIMBEN_QT) THEN
                    UNEM_BENEMP3_QT(pers_j) = UNEM_LIMBEN_QT
                END IF
            END IF
            !STEP 5: The amount computed so far is a net amount and must be converted to 
            !a gross amount (including the tax pre-payment of 10.09%)
            UNEM_BENEMP3_QT(pers_j) = UNEM_BENEMP3_QT(pers_j)*(100.D0/89.91D0) 
        END IF
    END IF

END SUBROUTINE UNEM_ADASIMMT

!===================================================================================!
!With subroutine UNEM_EXCHANGE we construct the variables                           !
!based on the unemployment computations, that have to be exchanged to               !
!other modules                                                                      !
!   UNEM_OLDUNSS_QT                                                                 !
!   UNEM_OTHERUN_QT                                                                 !
!   UNEM_BENUN_QT                                                                   !
!   UNEM_TOTBEN_QT                                                                  !
!   UNEM_FULL6MON_QT
!===================================================================================!

SUBROUTINE UNEM_EXCHANGE

    INTEGER:: pers_j

    DO pers_j=1,n_rec

        !Variables constructed for the CONTRIB module
        !UNEM_OLDUNSS_QT: Unemployment benefits of older unemployed with seniority supplement, paid in the given quarter
        UNEM_OLDUNSS_QT(pers_j) = UNEM_BENUNS3_QT(pers_j)
        !UNEM_OTHERUN_QT: Sum of all gross unemployment benefits, other than early retirement benefits            
        UNEM_OTHERUN_QT(pers_j) = &
        &UNEM_BENSTUD_QT(pers_j) + UNEM_BENEMPL_QT(pers_j) + &
        &UNEM_BENUNS2_QT(pers_j) + UNEM_BENUNS4_QT(pers_j) + &
        &UNEM_BENEMP2_QT(pers_j) + UNEM_BENEMP3_QT(pers_j) + UNEM_BENEMP4_QT(pers_j) + &
        &UNEM_BENEMP5_QT(pers_j) + UNEM_BENEMP6_QT(pers_j)
        
        !Variables constructed for the SICK and UNEM module
        !UNEM_BENUN_QT: Gross amount of real unemployment benefits in a given quarter (no benefits paid by RVA as supplement to wage)          
        UNEM_BENUN_QT(pers_j) = &
        &UNEM_BENSTUD_QT(pers_j) + UNEM_BENEMPL_QT(pers_j) + &
        &UNEM_BENUNS1_QT(pers_j) + UNEM_BENUNS2_QT(pers_j) + &
        &UNEM_BENUNS3_QT(pers_j) + UNEM_BENUNS4_QT(pers_j) + &
        &UNEM_BENEMP1_QT(pers_j) + UNEM_BENEMP2_QT(pers_j) + &
        &UNEM_BENEMP3_QT(pers_j) + UNEM_BENEMP4_QT(pers_j) + &
        &UNEM_ADDUNS_QT(pers_j)  + UNEM_ADDEMP_QT(pers_j)  
        
        !Variables constructed for the FAMAL module
        !UNEM_TOTBEN_QT: Gross amount of unemployment benefits           
        UNEM_TOTBEN_QT(pers_j) = &
        &UNEM_BENUNS1_QT(pers_j) + UNEM_ADDUNS_QT(pers_j) + &
        &UNEM_BENEMP1_QT(pers_j) + UNEM_ADDEMP_QT(pers_j) + &
        &UNEM_OLDUNSS_QT(pers_j) + UNEM_OTHERUN_QT(pers_j)

        !UNEM_FULL6MON_QT: The unemployed is unemployed for 6 months or more and 
        !assumed to be eligible for social supplement in child benefits
        !People with guaranteed benefits are inculded here because they are assumed to be in the system of reactivation
        !after unemployment
        !Hence only time credits are excluded
        IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2, 3, 5, 6, 9, 11, 12/))) THEN
            UNEM_FULL6MON_QT(pers_j) = UNEM_NUMMINU_QT(pers_j)
            IF (UNEM_NUMMINU_QT(pers_j).EQ.0) THEN
                UNEM_FULL6MON_QT(pers_j) = INT(DBLE(DAGEN_2008(pers_j))/26.D0)
            END IF                
        END IF
!Aangepast        
!        IF (UNEM_STATUS_QT(pers_j).GT.0.AND.UNEM_STATUS_QT(pers_j).NE.4.AND.UNEM_STATUS_QT(pers_j).NE.8&
!        &.AND.UNEM_FULL6MON_QT(pers_j).EQ.0) THEN
!            WRITE(15,*) pers_j,UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j),DAGEN_2008(pers_j)
!        END IF

    END DO
    
END SUBROUTINE UNEM_EXCHANGE

!==========================================================!
!Produce some aggregate statistics to check the constuction
!==========================================================!

SUBROUTINE UNEM_TEST

    INTEGER                                                 :: pers_j,i
    INTEGER                                                 :: NumAnal                              !Number of summary statistics computed 
    REAL(KIND=DPRECI)                                       :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)            :: UNEM_RESULT_TEST                     !matrix with test results
    CHARACTER(len=75),ALLOCATABLE,DIMENSION(:)              :: UNEM_LAB_TEST                        !Labels of test variables
    REAL(KIND=DPRECI),DIMENSION(22)                         :: SumGross                             !Vector with quarterly amounts on benefits

    WRITE(15,'(/,''Print of test statistics'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))
    
    !Compute aggregate statistics on unemployment status

    NumAnal = 13
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "Not unemployed                                                            "
    UNEM_LAB_TEST(2) = "people who receive unemployment benefits after studies                    "
    UNEM_LAB_TEST(3) = "people who receive unemployment benefits after employment                 "
    UNEM_LAB_TEST(4) = "people who are on conventional early retirement                           "
    UNEM_LAB_TEST(5) = "people who are on career break                                            "
    UNEM_LAB_TEST(6) = "the older unemployed who receive a seniority supplement                   "
    UNEM_LAB_TEST(7) = "people who receive unemployment benefits because of an exemption          "
    UNEM_LAB_TEST(8) = "people who are on part-time early retirement                              "
    UNEM_LAB_TEST(9) = "people who are on part-time career break                                  "
    UNEM_LAB_TEST(10) = "people who receive guaranteed income benefits                            "
    UNEM_LAB_TEST(11) = "people who are temporarily unemployed                                    "
    UNEM_LAB_TEST(12) = "people who are working for a Plaatselijk Werkgelegenheidsagentschap/Agence"
    UNEM_LAB_TEST(13) = "people who are working in some kind of activation program other than PWA sy"
    
    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (UNEM_STATUS_QT(pers_j).EQ.i-1) THEN
                UNEM_RESULT_TEST(i,1) = UNEM_RESULT_TEST(i,1) + 1
                UNEM_RESULT_TEST(i,2) = UNEM_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable UNEM_STATUS_QT: unemployment status of the unemployed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    !Compute some tests of reconstructed variables: people NOT eligible for benefit but benefit NOT equal to 0

    NumAnal = 11
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "Not Unemployed after studies but benefit for this <>"
    UNEM_LAB_TEST(2) = "Not in Early retirement of Un not in search of work but benefit for this <>"
    UNEM_LAB_TEST(3) = "No Career break of Un not in search of work but benefit for this <>"
    UNEM_LAB_TEST(4) = "Not eligible for Un not in search of work with seniority supplement but benefit <>0"
    UNEM_LAB_TEST(5) = "Not eligible for Un not in search of work with exemption but benefit <>0"
    UNEM_LAB_TEST(6) = "Employee not eligible for early retirement but benefit <>0"
    UNEM_LAB_TEST(7) = "Employee not eligible for career break but benefit <>0"
    UNEM_LAB_TEST(8) = "Employee not eligible for guaranteed benefits but benefit <>0"
    UNEM_LAB_TEST(9) = "Employee not eligible for temporary unemployment benefits but benefit <>0"
    UNEM_LAB_TEST(10) = "Employee not eligible for PWA benefits but benefit <>0"
    UNEM_LAB_TEST(11) = "Employee not eligible for benefits of activation program othet than PWA but benefit <>0"

    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).NE.1.AND.INT(100.D0*UNEM_BENSTUD_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(1,1) = UNEM_RESULT_TEST(1,1) + 1
            UNEM_RESULT_TEST(1,2) = UNEM_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS1_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENUNS1_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(2,1) = UNEM_RESULT_TEST(2,1) + 1
            UNEM_RESULT_TEST(2,2) = UNEM_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS2_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENUNS2_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(3,1) = UNEM_RESULT_TEST(3,1) + 1
            UNEM_RESULT_TEST(3,2) = UNEM_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS3_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENUNS3_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(4,1) = UNEM_RESULT_TEST(4,1) + 1
            UNEM_RESULT_TEST(4,2) = UNEM_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS4_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENUNS4_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(5,1) = UNEM_RESULT_TEST(5,1) + 1
            UNEM_RESULT_TEST(5,2) = UNEM_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP1_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP1_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(6,1) = UNEM_RESULT_TEST(6,1) + 1
            UNEM_RESULT_TEST(6,2) = UNEM_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP2_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP2_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(7,1) = UNEM_RESULT_TEST(7,1) + 1
            UNEM_RESULT_TEST(7,2) = UNEM_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP3_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP3_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(8,1) = UNEM_RESULT_TEST(8,1) + 1
            UNEM_RESULT_TEST(8,2) = UNEM_RESULT_TEST(8,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP4_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP4_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(9,1) = UNEM_RESULT_TEST(9,1) + 1
            UNEM_RESULT_TEST(9,2) = UNEM_RESULT_TEST(9,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP5_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP5_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(10,1) = UNEM_RESULT_TEST(10,1) + 1
            UNEM_RESULT_TEST(10,2) = UNEM_RESULT_TEST(10,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP6_QT(pers_j).EQ.0.AND.INT(100.D0*UNEM_BENEMP6_QT(pers_j)).NE.0) THEN
            UNEM_RESULT_TEST(11,1) = UNEM_RESULT_TEST(11,1) + 1
            UNEM_RESULT_TEST(11,2) = UNEM_RESULT_TEST(11,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of some reconstructed variables: people NOT eligible for benefit but benefit <> 0'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    !Compute some tests of reconstructed variables: people eligible for benefit but benefit equal to 0

    NumAnal = 13
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "Unemployed but family charge unknown"
    UNEM_LAB_TEST(2) = "Unemployed after studies without recomputed benefit for this"
    UNEM_LAB_TEST(3) = "Unemployed after employement without recomputed benefit for this"
    UNEM_LAB_TEST(4) = "Early retirement benefits of Un not in search of work"
    UNEM_LAB_TEST(5) = "Career break benefits of Un not in search of work"
    UNEM_LAB_TEST(6) = "Un not in search of work with seniority supplement but benefit 0"
    UNEM_LAB_TEST(7) = "Un not in search of work with exemption but benefit 0"
    UNEM_LAB_TEST(8) = "Employee eligible for early retirement but benefit 0"
    UNEM_LAB_TEST(9) = "Employee eligible for career break but benefit 0"
    UNEM_LAB_TEST(10) = "Employee eligible for guaranteed benefits but benefit 0"
    UNEM_LAB_TEST(11) = "Employee eligible for temporary unemployment benefits but benefit 0"
    UNEM_LAB_TEST(12) = "Employee eligible for PWA benefits but benefit 0"
    UNEM_LAB_TEST(13) = "Employee eligible for benefits of activation program othet than PWA but benefit 0"

    DO pers_j=1,n_rec
        IF (UNEM_STATUS_QT(pers_j).NE.0.AND.UNEM_FAMSIT_QT(pers_j).EQ.0) THEN
            UNEM_RESULT_TEST(1,1) = UNEM_RESULT_TEST(1,1) + 1
            UNEM_RESULT_TEST(1,2) = UNEM_RESULT_TEST(1,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_STATUS_QT(pers_j).EQ.1.AND.INT(100.D0*UNEM_BENSTUD_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(2,1) = UNEM_RESULT_TEST(2,1) + 1
            UNEM_RESULT_TEST(2,2) = UNEM_RESULT_TEST(2,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_STATUS_QT(pers_j).EQ.2.AND.INT(100.D0*UNEM_BENEMPL_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(3,1) = UNEM_RESULT_TEST(3,1) + 1
            UNEM_RESULT_TEST(3,2) = UNEM_RESULT_TEST(3,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS1_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENUNS1_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(4,1) = UNEM_RESULT_TEST(4,1) + 1
            UNEM_RESULT_TEST(4,2) = UNEM_RESULT_TEST(4,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS2_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENUNS2_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(5,1) = UNEM_RESULT_TEST(5,1) + 1
            UNEM_RESULT_TEST(5,2) = UNEM_RESULT_TEST(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS3_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENUNS3_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(6,1) = UNEM_RESULT_TEST(6,1) + 1
            UNEM_RESULT_TEST(6,2) = UNEM_RESULT_TEST(6,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGUNS4_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENUNS4_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(7,1) = UNEM_RESULT_TEST(7,1) + 1
            UNEM_RESULT_TEST(7,2) = UNEM_RESULT_TEST(7,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP1_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP1_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(8,1) = UNEM_RESULT_TEST(8,1) + 1
            UNEM_RESULT_TEST(8,2) = UNEM_RESULT_TEST(8,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP2_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP2_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(9,1) = UNEM_RESULT_TEST(9,1) + 1
            UNEM_RESULT_TEST(9,2) = UNEM_RESULT_TEST(9,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP3_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP3_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(10,1) = UNEM_RESULT_TEST(10,1) + 1
            UNEM_RESULT_TEST(10,2) = UNEM_RESULT_TEST(10,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP4_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP4_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(11,1) = UNEM_RESULT_TEST(11,1) + 1
            UNEM_RESULT_TEST(11,2) = UNEM_RESULT_TEST(11,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP5_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP5_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(12,1) = UNEM_RESULT_TEST(12,1) + 1
            UNEM_RESULT_TEST(12,2) = UNEM_RESULT_TEST(12,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
        IF (UNEM_ELIGEMP6_QT(pers_j).NE.0.AND.INT(100.D0*UNEM_BENEMP6_QT(pers_j)).EQ.0) THEN
            UNEM_RESULT_TEST(13,1) = UNEM_RESULT_TEST(13,1) + 1
            UNEM_RESULT_TEST(13,2) = UNEM_RESULT_TEST(13,2) + MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of some reconstructed variables: people eligible for benefit but benefit equal to 0'')')
    Write(15,'(''0 benefit sometimes possible, f.e.:'')')
    Write(15,'(''   With unemployed not in search of work but exempted: the lump sum amount becomes 0 after some time'')')
    Write(15,'(''   With employees on career break: if break for palliative care or parental leave benefits might be 0 &
    &for some time reductions'')')
    Write(15,'(''   When either unemployed or employees leave a part time job the observed value bedragen_qt is used'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    !Compute aggregate statistics on the family charge of the unemployed

    NumAnal = 4
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "Not unemployed"
    UNEM_LAB_TEST(2) = "An unemployed with dependent family "
    UNEM_LAB_TEST(3) = "An unemployed who is single"
    UNEM_LAB_TEST(4) = "An unemployed without dependent family and who has not lost his sole sourc"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (UNEM_FAMSIT_QT(pers_j).EQ.i-1) THEN
                UNEM_RESULT_TEST(i,1) = UNEM_RESULT_TEST(i,1) + 1
                UNEM_RESULT_TEST(i,2) = UNEM_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable UNEM_FAMSIT_QT: family charge of the unemployed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    !Compute aggregate statistics on unemployed with a partner with the privileged income condtion

    NumAnal = 2
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "Not unemployed"
    UNEM_LAB_TEST(2) = "Unemployed cohabits with a partner with a privileged income condition"

    DO pers_j=1,n_rec
        DO i=1,NumAnal
            IF (UNEM_PRIVICOH_QT(pers_j).EQ.i-1) THEN
                UNEM_RESULT_TEST(i,1) = UNEM_RESULT_TEST(i,1) + 1
                UNEM_RESULT_TEST(i,2) = UNEM_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable UNEM_PRIVICOH_QT: privileged income condtion of the partner of the unemployed'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO
    
    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    Write(15,'(/,''Some statistics on unemployment benefits for unemployed in search of work'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics
    NumAnal = 6

    !Allocate and initialise test result
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,4))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    UNEM_LAB_TEST(1) = "Daily benefits for unemployed in search of work: benefits after studies"
    UNEM_LAB_TEST(2) = "Quart benefits for unemployed in search of work: benefits after studies"
    UNEM_LAB_TEST(3) = "Daily benefits for unemployed in search of work: benefits after employment"
    UNEM_LAB_TEST(4) = "Quart Benefits for unemployed in search of work: benefits after employment"
    UNEM_LAB_TEST(5) = "Daily benefits for other than unemployed in search of work: benefits after employment rules"
    UNEM_LAB_TEST(6) = "Quart Benefits for other than unemployed in search of work: benefits after employment rules"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = UNEM_BENSTUD_DAY(pers_j)
        SumGross(2) = UNEM_BENSTUD_QT(pers_j)
        SumGross(3) = UNEM_BENEMPL_DAY(pers_j)
        SumGross(4) = UNEM_BENEMPL_QT(pers_j)
        SumGross(5) = UNEM_BENREF_DAY(pers_j)
        SumGross(6) = UNEM_BENREF_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                UNEM_RESULT_TEST(j,1)=UNEM_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                UNEM_RESULT_TEST(j,2)=UNEM_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(UNEM_RESULT_TEST(i,2)).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=UNEM_RESULT_TEST(i,1)/UNEM_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,4)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST) 

    Write(15,'(/,''Some statistics on unemployment benefits for unemployed not in search of work with RVA benefit '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics
    NumAnal = 9

    !Allocate and initialise test result
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,4))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    UNEM_LAB_TEST(1) = "Quarterly RVA benefit of people with conventional early retirement benefit"
    UNEM_LAB_TEST(2) = "Quarterly benefit paid by employer to people on conventional early retirement"
    UNEM_LAB_TEST(3) = "Quarterly career break benefit of unemployed not in search of work: common car break"
    UNEM_LAB_TEST(4) = "Quarterly career break benefit of unemployed not in search of work: palliative care"
    UNEM_LAB_TEST(5) = "Quarterly career break benefit of unemployed not in search of work: medical assist"
    UNEM_LAB_TEST(6) = "Quarterly career break benefit of unemployed not in search of work: parental leave"
    UNEM_LAB_TEST(7) = "Daily unemployment benefits paid to unemployed with seniority supplement: sup + ben"
    UNEM_LAB_TEST(8) = "Quarterly unemployment benefits paid to unemployed with seniority supplement: sup + ben"
    UNEM_LAB_TEST(9) = "Quarterly unemployment benefits paid to unemployed with exemption"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = UNEM_BENUNS1_QT(pers_j)
        SumGross(2) = UNEM_ADDUNS_QT(pers_j)
        IF (UNEM_ELIGUNS2_QT(pers_j).EQ.1) THEN
            SumGross(3) = UNEM_BENUNS2_QT(pers_j)
        ELSE IF (UNEM_ELIGUNS2_QT(pers_j).EQ.2) THEN
            SumGross(4) = UNEM_BENUNS2_QT(pers_j)
        ELSE IF (UNEM_ELIGUNS2_QT(pers_j).EQ.3) THEN
            SumGross(5) = UNEM_BENUNS2_QT(pers_j)
        ELSE IF (UNEM_ELIGUNS2_QT(pers_j).EQ.4) THEN
            SumGross(6) = UNEM_BENUNS2_QT(pers_j)
        END IF
        SumGross(7) = UNEM_BENUNS3_DAY(pers_j)
        SumGross(8) = UNEM_BENUNS3_QT(pers_j)
        SumGross(9) = UNEM_BENUNS4_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                UNEM_RESULT_TEST(j,1)=UNEM_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                UNEM_RESULT_TEST(j,2)=UNEM_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(UNEM_RESULT_TEST(i,2)).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=UNEM_RESULT_TEST(i,1)/UNEM_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,4)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST) 

    Write(15,'(/,''Some statistics on employees with RVA benefit '')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics
    NumAnal = 12

    !Allocate and initialise test result
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,4))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    UNEM_LAB_TEST(1) = "Quarterly RVA benefit of people with early retirement benefit"
    UNEM_LAB_TEST(2) = "Quarterly benefit paid by employer to employees with RVA benefit"
    UNEM_LAB_TEST(3) = "Quarterly benefit paid by employer to employees with RVA benefit: common car break"
    UNEM_LAB_TEST(4) = "Quarterly benefit paid by employer to employees with RVA benefit: palliative care"
    UNEM_LAB_TEST(5) = "Quarterly benefit paid by employer to employees with RVA benefit: medical assist"
    UNEM_LAB_TEST(6) = "Quarterly benefit paid by employer to employees with RVA benefit: parental leave"
    UNEM_LAB_TEST(7) = "Quarterly benefit paid in case guaranteed income benefits of the RVA"
    UNEM_LAB_TEST(8) = "Quarterly benefit paid in case of temporary unemployment benefits"
    UNEM_LAB_TEST(9) = "Basic daily benefit of PWA before correction"
    UNEM_LAB_TEST(10) = "Quarterly benefit of people working with a PWA contract"
    UNEM_LAB_TEST(11) = "Quarterly benefit of people working an activation program other than PWA: transtion"
    UNEM_LAB_TEST(12) = "Quarterly benefit of people working an activation program other than PWA: reinsertion"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) = UNEM_BENEMP1_QT(pers_j)
        SumGross(2) = UNEM_ADDEMP_QT(pers_j)
        IF (UNEM_ELIGEMP2_QT(pers_j).EQ.1) THEN
            SumGross(3) = UNEM_BENEMP2_QT(pers_j)
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.2) THEN
            SumGross(4) = UNEM_BENEMP2_QT(pers_j)
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.3) THEN
            SumGross(5) = UNEM_BENEMP2_QT(pers_j)
        ELSE IF (UNEM_ELIGEMP2_QT(pers_j).EQ.4) THEN
            SumGross(6) = UNEM_BENEMP2_QT(pers_j)
        END IF
        SumGross(7) = UNEM_BENEMP3_QT(pers_j)
        SumGross(8) = UNEM_BENEMP4_QT(pers_j)
        SumGross(9) = UNEM_BENEMP5_DAY(pers_j)
        SumGross(10) = UNEM_BENEMP5_QT(pers_j)
        IF (UNEM_ELIGEMP6_QT(pers_j).EQ.1) THEN
            SumGross(11) = UNEM_BENEMP6_QT(pers_j)
        ELSE IF (UNEM_ELIGEMP6_QT(pers_j).EQ.2) THEN
            SumGross(12) = UNEM_BENEMP6_QT(pers_j)
        END IF            
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                UNEM_RESULT_TEST(j,1)=UNEM_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                UNEM_RESULT_TEST(j,2)=UNEM_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,NumAnal
        IF (INT(UNEM_RESULT_TEST(i,2)).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=UNEM_RESULT_TEST(i,1)/UNEM_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,4)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST) 

    !Compute statistics on time reduction of employees on career break

    NumAnal = 5
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,3))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:3)=0.D0    

    !Define labels
    UNEM_LAB_TEST(1) = "No time reduction                                                           "
    UNEM_LAB_TEST(2) = "work time reduction is 1/5                                                  "
    UNEM_LAB_TEST(3) = "work time reduction is 1/4                                                  "
    UNEM_LAB_TEST(4) = "work time reduction is 1/3                                                  "
    UNEM_LAB_TEST(5) = "work time reduction is 1/2                                                  "
    
    DO pers_j=1,n_rec
        IF (UNEM_ELIGEMP2_QT(pers_j).NE.0) THEN
            DO i=1,NumAnal
                IF (UNEM_TIMEUNS_QT(pers_j).EQ.i-1) THEN
                    UNEM_RESULT_TEST(i,1) = UNEM_RESULT_TEST(i,1) + 1
                    UNEM_RESULT_TEST(i,2) = UNEM_RESULT_TEST(i,2) + MIMOSIS_WEIGHT(pers_j)
                END IF
            END DO
        END IF
    END DO                

    DO i=1,NumAnal
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                             TotSamp  TotSampW       %Pop'')')
    Write(15,'(''Counts of variable UNEM_TIMEUNS_QT: time reduction of employees on career break'')')
    DO i=1,NumAnal
        Write(15,'(a75,2f10.0,f10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,3)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST)

    Write(15,'(/,''Compute global budget concepts: unemployment benefits paid by the RVA per year'')')
    CALL DRAWLINE(15,100,'-')    

    !Compute aggregate statistics
    NumAnal = 3

    !Allocate and initialise test result
    ALLOCATE(UNEM_RESULT_TEST(NumAnal,4))
    ALLOCATE(UNEM_LAB_TEST(NumAnal))
    UNEM_RESULT_TEST(1:NumAnal,1:4)=0.D0

    !Define labels
    UNEM_LAB_TEST(1) = "Unemployment benefits"
    UNEM_LAB_TEST(2) = "Early retirement benefits"
    UNEM_LAB_TEST(3) = "Career break benefits"

    !Compute aggregate statistics
    DO pers_j=1,n_rec                                                                                 
        SumGross(1:NumAnal)=0.D0
        !Compute quarterly amounts
        SumGross(1) =    UNEM_BENSTUD_QT(pers_j) + UNEM_BENEMPL_QT(pers_j) + &
                        &UNEM_BENUNS3_QT(pers_j) + UNEM_BENUNS4_QT(pers_j) + &
                        &UNEM_BENEMP3_QT(pers_j) + UNEM_BENEMP4_QT(pers_j) + &
                        &UNEM_BENEMP5_QT(pers_j) + UNEM_BENEMP6_QT(pers_j)
        SumGross(2) =    UNEM_BENUNS1_QT(pers_j) + UNEM_BENEMP1_QT(pers_j)
        SumGross(3) =    UNEM_BENUNS2_QT(pers_j) + UNEM_BENEMP2_QT(pers_j)
        DO j=1,NumAnal
            IF (INT(100.D0*SumGross(j)).NE.0) THEN
                UNEM_RESULT_TEST(j,1)=UNEM_RESULT_TEST(j,1)+MIMOSIS_WEIGHT(pers_j)*SumGross(j)
                UNEM_RESULT_TEST(j,2)=UNEM_RESULT_TEST(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !convert values to annual values
    UNEM_RESULT_TEST(1:NumAnal,1)=UNEM_RESULT_TEST(1:NumAnal,1)*4.D0

    DO i=1,NumAnal
        IF (INT(UNEM_RESULT_TEST(i,2)).NE.0) THEN
            UNEM_RESULT_TEST(i,3)=UNEM_RESULT_TEST(i,1)/UNEM_RESULT_TEST(i,2)
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_TEST(i,4)=100.D0*UNEM_RESULT_TEST(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'(/,''                                                                                     AmountEuro&
    &       NumIndividual   Average      %Pop'')')
    DO i=1,NumAnal
        Write(15,'(a75,F20.2,F20.0,F10.2,F10.2)')UNEM_LAB_TEST(i),(UNEM_RESULT_TEST(i,j),j=1,4)
    END DO

    DEALLOCATE(UNEM_LAB_TEST)
    DEALLOCATE(UNEM_RESULT_TEST) 

END SUBROUTINE UNEM_TEST

!==========================================================!
!Produce aggregate statistics that can be compared with
!statistics in other sources
!==========================================================!

SUBROUTINE UNEM_COMPARE

    INTEGER                                                             :: pers_j,i,j
    INTEGER                                                             :: NumAnal                              !Number of summary statistics computed 
    !variables for computation of aggregates
    REAL(KIND=DPRECI)                                                   :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: UNEM_AGGREGA                         !Vector with aggregates of unem data comparable with other sources computed for 1 individual
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: UNEM_RESULT_COMPARE                  !Vector with aggregates of unem data comparable with other sources computed over all individuals
    CHARACTER(len=100),ALLOCATABLE,DIMENSION(:)                         :: UNEM_LAB_COMPARE                     !Vector with labels of the different reconstructed aggregates                       
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: UNEM_EXTERN                          !Vector with aggregates taken from external sources comparable with the reconstructed concepts
    REAL(KIND=DPRECI)                                                   :: AantMaand                            !Aantal maanden geobserveerd op jaarbasis
    REAL(KIND=DPRECI)                                                   :: Sum_Amount                           !Som niet gealloceerde bedragen
 
    WRITE(15,'(/,''Print statistics comparable with other sources'')')
    CALL DRAWLINE(15,136,'=')    

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !Number of unemployed entitled to unemployment benefits after employment
    NumAnal = 13
    ALLOCATE(UNEM_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(UNEM_LAB_COMPARE(NumAnal))     
    ALLOCATE(UNEM_EXTERN(NumAnal))        

    !Initialise results
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1:6) = 0.D0
    UNEM_EXTERN(1:SIZE(UNEM_EXTERN,1)) = 0.D0           
    
    !Define labels
    UNEM_LAB_COMPARE(1) = "Unemployed in search of work who receive benefits after studies"
    UNEM_LAB_COMPARE(2) = "Unemployed in search of work who receive benefits after employment"
    UNEM_LAB_COMPARE(3) = "People on conventional early retirement"
    UNEM_LAB_COMPARE(4) = "People with a full time career break who receive an unemployment benefit"
    UNEM_LAB_COMPARE(5) = "Older unemployed with a seniority supplement"
    UNEM_LAB_COMPARE(6) = "People who receive unemployment benefits because of an exemption"
    UNEM_LAB_COMPARE(7) = "People on part time early retirement"
    UNEM_LAB_COMPARE(8) = "People on part time career break"
    UNEM_LAB_COMPARE(9) = "People who receive guaranteed income benefits"
    UNEM_LAB_COMPARE(10) = "People who are temporarily unemployed"
    UNEM_LAB_COMPARE(11) = "People who are working for a PWA"
    UNEM_LAB_COMPARE(12) = "People who are working in some kind of activation program other than PWA"
    UNEM_LAB_COMPARE(13) = "Total"
    
    !Define values taken from external sources
    UNEM_EXTERN(1)    = 0
    UNEM_EXTERN(2)    = 0
    UNEM_EXTERN(3)    = 0
    UNEM_EXTERN(4)    = 0
    UNEM_EXTERN(5)    = 0
    UNEM_EXTERN(6)    = 0
    UNEM_EXTERN(7)    = 0
    UNEM_EXTERN(8)    = 0
    UNEM_EXTERN(9)    = 0
    UNEM_EXTERN(10)   = 0
    UNEM_EXTERN(11)   = 0
    UNEM_EXTERN(12)   = 0
    UNEM_EXTERN(13)   = 0
    
    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Bereken aantal maanden in het bestand
        IF (DUUR_2008(pers_j).GT.0) THEN
            AantMaand = DBLE(DUUR_2008(pers_j))    
        ELSE
            AantMaand = (DBLE(DAGEN_2008(pers_j))/26.D0)  
        END IF
        IF (AantMaand.GT.12.D0) THEN
            AantMaand = 12.D0
        END IF
        !If person is eligible to unemployment benefits after studies
        IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/1,2/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
            UNEM_RESULT_COMPARE(1,1) = UNEM_RESULT_COMPARE(1,1) + 1
            UNEM_RESULT_COMPARE(1,2) = UNEM_RESULT_COMPARE(1,2) + MIMOSIS_WEIGHT(pers_j)
            UNEM_RESULT_COMPARE(1,6) = UNEM_RESULT_COMPARE(1,6) + MIMOSIS_WEIGHT(pers_j)*(AantMaand/12.D0)
            !total
            UNEM_RESULT_COMPARE(13,1) = UNEM_RESULT_COMPARE(13,1) + 1
            UNEM_RESULT_COMPARE(13,2) = UNEM_RESULT_COMPARE(13,2) + MIMOSIS_WEIGHT(pers_j)
        END IF    
        !If person is eligible to unemployment benefits after employment
        IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/3/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
            UNEM_RESULT_COMPARE(2,1) = UNEM_RESULT_COMPARE(2,1) + 1
            UNEM_RESULT_COMPARE(2,2) = UNEM_RESULT_COMPARE(2,2) + MIMOSIS_WEIGHT(pers_j)
            UNEM_RESULT_COMPARE(2,6) = UNEM_RESULT_COMPARE(2,6) + MIMOSIS_WEIGHT(pers_j)*(AantMaand/12.D0)
            !total
            UNEM_RESULT_COMPARE(13,1) = UNEM_RESULT_COMPARE(13,1) + 1
            UNEM_RESULT_COMPARE(13,2) = UNEM_RESULT_COMPARE(13,2) + MIMOSIS_WEIGHT(pers_j)
        END IF    
        DO i=3,12
            IF (UNEM_STATUS_QT(pers_j).EQ.i) THEN
                UNEM_RESULT_COMPARE(i,1) = UNEM_RESULT_COMPARE(i,1) + 1
                UNEM_RESULT_COMPARE(i,2) = UNEM_RESULT_COMPARE(i,2) + MIMOSIS_WEIGHT(pers_j)
                UNEM_RESULT_COMPARE(i,6) = UNEM_RESULT_COMPARE(i,6) + MIMOSIS_WEIGHT(pers_j)*(AantMaand/12.D0)
!                IF (i.EQ.12) THEN
!                    WRITE(15,*) 'TestStat',GECODEERD_INSZ_INPUT(pers_j),DAGEN_2008(pers_j),UNEM_RESULT_COMPARE(i,6)
!                END IF
                !total
                UNEM_RESULT_COMPARE(13,1) = UNEM_RESULT_COMPARE(13,1) + 1
                UNEM_RESULT_COMPARE(13,2) = UNEM_RESULT_COMPARE(13,2) + MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        !difference between real value and estimate
        UNEM_RESULT_COMPARE(i,3)= UNEM_RESULT_COMPARE(i,2) - UNEM_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(UNEM_EXTERN(i)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,4)= 100.D0*(UNEM_RESULT_COMPARE(i,3)/UNEM_EXTERN(i))    
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_COMPARE(i,5)=100.D0*UNEM_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Number of unemployed by class'')')
    Write(15,'(''Comparison with external source of 2008 data'')')
    Write(15,'(''                                                                                                        &
    &Extern   NotWeig      Weig   Abs dif      %Dif      %Pop       Month Me'')')
    Write(15,'(''                                                                                                         &
    &   C1                  C2     C2-C1 100*C2/C1'')')
    CALL DRAWLINE(15,175,'-')    
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        Write(15,'(a100,4f10.0,f10.2,f10.2,F15.2)')UNEM_LAB_COMPARE(i),UNEM_EXTERN(i),(UNEM_RESULT_COMPARE(i,j),j=1,6)
    END DO

    DEALLOCATE(UNEM_RESULT_COMPARE)
    DEALLOCATE(UNEM_LAB_COMPARE)     
    DEALLOCATE(UNEM_EXTERN)        

    !Number of unemployed entitled to benefits after studies
    NumAnal = 5
    ALLOCATE(UNEM_RESULT_COMPARE(NumAnal,5))
    ALLOCATE(UNEM_LAB_COMPARE(NumAnal))     
    ALLOCATE(UNEM_EXTERN(NumAnal))        

    !Initialise results
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1:5) = 0.D0
    UNEM_EXTERN(1:SIZE(UNEM_EXTERN,1)) = 0.D0           
    
    !Define labels
    UNEM_LAB_COMPARE(1) = "Unemployed with dependent family"
    UNEM_LAB_COMPARE(2) = "Single unemployed"
    UNEM_LAB_COMPARE(3) = "Cohabitating unemployed"
    UNEM_LAB_COMPARE(4) = "Others"
    UNEM_LAB_COMPARE(5) = "Total"
    
    !Define values taken from external sources
    UNEM_EXTERN(1)   = 0
    UNEM_EXTERN(2)   = 0
    UNEM_EXTERN(3)   = 0
    UNEM_EXTERN(4)   = 0
    UNEM_EXTERN(5)   = 0
    
    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !If person is eligible to benefits after studies
        IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/1,2/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
            IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                UNEM_RESULT_COMPARE(1,1) = UNEM_RESULT_COMPARE(1,1) + 1
                UNEM_RESULT_COMPARE(1,2) = UNEM_RESULT_COMPARE(1,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                UNEM_RESULT_COMPARE(2,1) = UNEM_RESULT_COMPARE(2,1) + 1
                UNEM_RESULT_COMPARE(2,2) = UNEM_RESULT_COMPARE(2,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                UNEM_RESULT_COMPARE(3,1) = UNEM_RESULT_COMPARE(3,1) + 1
                UNEM_RESULT_COMPARE(3,2) = UNEM_RESULT_COMPARE(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF            
            !total
            UNEM_RESULT_COMPARE(5,1) = UNEM_RESULT_COMPARE(5,1) + 1
            UNEM_RESULT_COMPARE(5,2) = UNEM_RESULT_COMPARE(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
    END DO

    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        !difference between real value and estimate
        UNEM_RESULT_COMPARE(i,3)= UNEM_RESULT_COMPARE(i,2) - UNEM_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(UNEM_EXTERN(i)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,4)= 100.D0*(UNEM_RESULT_COMPARE(i,3)/UNEM_EXTERN(i))    
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_COMPARE(i,5)=100.D0*UNEM_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Number of unemployed in search of work entitled to unemployment benefits after studies'')')
    Write(15,'(''Comparison with external source of 2008 data'')')
    Write(15,'(''                                                                                                        &
    &Extern   NotWeig      Weig   Abs dif      %Dif      %Pop'')')
    Write(15,'(''                                                                                                         &
    &   C1                  C2     C2-C1 100*C2/C1'')')
    CALL DRAWLINE(15,160,'-')    
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        Write(15,'(a100,4f10.0,f10.2,f10.2)')UNEM_LAB_COMPARE(i),UNEM_EXTERN(i),(UNEM_RESULT_COMPARE(i,j),j=1,5)
    END DO

    DEALLOCATE(UNEM_RESULT_COMPARE)
    DEALLOCATE(UNEM_LAB_COMPARE)     
    DEALLOCATE(UNEM_EXTERN)        

    !Number of unemployed entitled to unemployment benefits after employment
    NumAnal = 5
    ALLOCATE(UNEM_RESULT_COMPARE(NumAnal,5))
    ALLOCATE(UNEM_LAB_COMPARE(NumAnal))     
    ALLOCATE(UNEM_EXTERN(NumAnal))        

    !Initialise results
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1:5) = 0.D0
    UNEM_EXTERN(1:SIZE(UNEM_EXTERN,1)) = 0.D0           
    
    !Define labels
    UNEM_LAB_COMPARE(1) = "Unemployed with dependent family"
    UNEM_LAB_COMPARE(2) = "Single unemployed"
    UNEM_LAB_COMPARE(3) = "Cohabitating unemployed"
    UNEM_LAB_COMPARE(4) = "Others"
    UNEM_LAB_COMPARE(5) = "Total"
    
    !Define values taken from external sources
    UNEM_EXTERN(1)   = 0
    UNEM_EXTERN(2)   = 0
    UNEM_EXTERN(3)   = 0
    UNEM_EXTERN(4)   = 0
    UNEM_EXTERN(5)   = 0
    
    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !If person is eligible to benefits after employment
        IF (ANY(UNEM_PERMIS_QT(pers_j).EQ.(/3/)).AND.UNEM_ELIGUIS_QT(pers_j).EQ.1) THEN
            IF (UNEM_FAMSIT_QT(pers_j).EQ.1) THEN
                UNEM_RESULT_COMPARE(1,1) = UNEM_RESULT_COMPARE(1,1) + 1
                UNEM_RESULT_COMPARE(1,2) = UNEM_RESULT_COMPARE(1,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.2) THEN
                UNEM_RESULT_COMPARE(2,1) = UNEM_RESULT_COMPARE(2,1) + 1
                UNEM_RESULT_COMPARE(2,2) = UNEM_RESULT_COMPARE(2,2) + MIMOSIS_WEIGHT(pers_j)
            ELSE IF (UNEM_FAMSIT_QT(pers_j).EQ.3) THEN
                UNEM_RESULT_COMPARE(3,1) = UNEM_RESULT_COMPARE(3,1) + 1
                UNEM_RESULT_COMPARE(3,2) = UNEM_RESULT_COMPARE(3,2) + MIMOSIS_WEIGHT(pers_j)
            END IF            
            !total
            UNEM_RESULT_COMPARE(5,1) = UNEM_RESULT_COMPARE(5,1) + 1
            UNEM_RESULT_COMPARE(5,2) = UNEM_RESULT_COMPARE(5,2) + MIMOSIS_WEIGHT(pers_j)
        END IF            
    END DO

    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        !difference between real value and estimate
        UNEM_RESULT_COMPARE(i,3)= UNEM_RESULT_COMPARE(i,2) - UNEM_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(UNEM_EXTERN(i)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,4)= 100.D0*(UNEM_RESULT_COMPARE(i,3)/UNEM_EXTERN(i))    
        END IF
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_COMPARE(i,5)=100.D0*UNEM_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Number of unemployed in search of work entitled to unemployment benefits after employment'')')
    Write(15,'(''Comparison with external source of 2008 data'')')
    Write(15,'(''                                                                                                        &
    &Extern   NotWeig      Weig   Abs dif      %Dif      %Pop'')')
    Write(15,'(''                                                                                                         &
    &   C1                  C2     C2-C1 100*C2/C1'')')
    CALL DRAWLINE(15,160,'-')    
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        Write(15,'(a100,4f10.0,f10.2,f10.2)')UNEM_LAB_COMPARE(i),UNEM_EXTERN(i),(UNEM_RESULT_COMPARE(i,j),j=1,5)
    END DO

    DEALLOCATE(UNEM_RESULT_COMPARE)
    DEALLOCATE(UNEM_LAB_COMPARE)     
    DEALLOCATE(UNEM_EXTERN)        

    !Unemployment benefits
    NumAnal = 6
    ALLOCATE(UNEM_AGGREGA(NumAnal))       
    ALLOCATE(UNEM_RESULT_COMPARE(NumAnal,6))
    ALLOCATE(UNEM_LAB_COMPARE(NumAnal))     
    ALLOCATE(UNEM_EXTERN(NumAnal))        

    !Initialise results
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1:6) = 0.D0
    UNEM_EXTERN(1:SIZE(UNEM_EXTERN,1)) = 0.D0           
    
    !Define labels
    UNEM_LAB_COMPARE(1) = "Full time unemployment benefits"
    UNEM_LAB_COMPARE(2) = "Temporary unemployment"
    UNEM_LAB_COMPARE(3) = "Activation measures"
    UNEM_LAB_COMPARE(4) = "Early retirement benefits"
    UNEM_LAB_COMPARE(5) = "Career break benefits and time credits"
    UNEM_LAB_COMPARE(6) = "Total"
    
    !Define values taken from external sources
    UNEM_EXTERN(1)   = 0
    UNEM_EXTERN(2)   = 0
    UNEM_EXTERN(3)   = 0
    UNEM_EXTERN(4)   = 0
    UNEM_EXTERN(5)   = 0
    UNEM_EXTERN(6)   = 0
    
    !Compute aggregate statistics
    Sum_Amount = 0.D0
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        UNEM_AGGREGA(1:SIZE(UNEM_AGGREGA,1)) = 0.D0
        !Compute quarterly amounts
        UNEM_AGGREGA(1)  = UNEM_AGGREGA(1)      + UNEM_BENSTUD_QT(pers_j) + UNEM_BENEMPL_QT(pers_j) &
                                            &   + UNEM_BENUNS3_QT(pers_j) + UNEM_BENUNS4_QT(pers_j) &
                                            &   + UNEM_BENEMP3_QT(pers_j) 
        UNEM_AGGREGA(2)  = UNEM_AGGREGA(2)      + UNEM_BENEMP4_QT(pers_j) 
        UNEM_AGGREGA(3)  = UNEM_AGGREGA(3)      + UNEM_BENEMP5_QT(pers_j) + UNEM_BENEMP6_QT(pers_j) 
        UNEM_AGGREGA(4)  = UNEM_AGGREGA(4)      + UNEM_BENUNS1_QT(pers_j) + UNEM_BENEMP1_QT(pers_j)
        UNEM_AGGREGA(5)  = UNEM_AGGREGA(5)      + UNEM_BENUNS2_QT(pers_j) + UNEM_BENEMP2_QT(pers_j)
        UNEM_AGGREGA(6)  = SUM(UNEM_AGGREGA(1:SIZE(UNEM_AGGREGA,1)-1))
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(UNEM_AGGREGA,1)
            IF (INT(100.D0*UNEM_AGGREGA(j)).NE.0) THEN
                UNEM_RESULT_COMPARE(j,1)=UNEM_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*UNEM_AGGREGA(j)
                UNEM_RESULT_COMPARE(j,2)=UNEM_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
!        IF (UNEM_STATUS_QT(pers_j).GT.0.AND.UNEM_AGGREGA(6).EQ.0.D0) THEN
!            WRITE(15,*) 'TestAm',GECODEERD_INSZ_INPUT(pers_j),UNEM_STATUS_QT(pers_j),BEDRAGEN_2008(pers_j),FICHE7_2008(pers_j),&
!            &REDENLO_2008(pers_j),UNEM_ELIGUNS2_QT(pers_j),UNEM_ELIGEMP2_QT(pers_j),UNEM_TIMEUNS_QT(pers_j),UNEM_FPEMP_QT(pers_j),&
!            &UNEM_CARBCON_QT(pers_j)
!            Sum_Amount = Sum_Amount + MIMOSIS_WEIGHT(pers_j)*BEDRAGEN_2008(pers_j)
!        END IF
    END DO
!    WRITE(15,*) 'TotBedrag',Sum_Amount

    !Inflate contributions to annual values
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1)=(UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1)*4.D0)/1000.D0

    !Compute columns 3 to 6 of the vector with comparable aggregates
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        !difference between real value and estimate
        UNEM_RESULT_COMPARE(i,3)= UNEM_RESULT_COMPARE(i,1) - UNEM_EXTERN(i)
        !difference expressed as percentage of external aggregate
        IF (INT(UNEM_EXTERN(i)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,4)= 100.D0*(UNEM_RESULT_COMPARE(i,3)/UNEM_EXTERN(i))    
        END IF
        !Average amount
        IF (INT(UNEM_RESULT_COMPARE(i,2)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,5)=UNEM_RESULT_COMPARE(i,1)/UNEM_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_COMPARE(i,6)=100.D0*UNEM_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Write results
    Write(15,'('' '')')
    Write(15,'(''Unemployment benefits by type'')')
    Write(15,'(''Nominal amounts in 1000 Euro in prices of 2008'')')
    Write(15,'(''                                                                                                         &
    &         Extern              Sample   Number of individ            Abs dif       %Dif      Mean      %Pop'')')
    Write(15,'(''                                                                                                         &
    &             C1                  C2                                  C2-C1  100*C2/C1 per indiv'')')
    CALL DRAWLINE(15,210,'-')    
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        Write(15,'(a100,F20.2,F20.2,F20.0,F20.0,F10.2,F10.2,F10.2)')UNEM_LAB_COMPARE(i),UNEM_EXTERN(i),&
        &(UNEM_RESULT_COMPARE(i,j),j=1,6)
    END DO

    DEALLOCATE(UNEM_AGGREGA)      
    DEALLOCATE(UNEM_RESULT_COMPARE)
    DEALLOCATE(UNEM_LAB_COMPARE)     
    DEALLOCATE(UNEM_EXTERN)        

END SUBROUTINE UNEM_COMPARE

!============= SUBROUTINE UNEM_GLOBBUD ======================================!
!This subroutine computes the global budget results and writes the results to
!the global budget file
!============================================================================!

SUBROUTINE UNEM_GLOBBUD

    INTEGER                                                             :: pers_j,i,j
    INTEGER                                                             :: NumAnal                              !Number of summary statistics computed 
    !variables for computation of aggregates
    REAL(KIND=DPRECI)                                                   :: SumW                                 !Variable containing sum of weights    
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:)                          :: UNEM_AGGREGA                         !Vector with aggregates of unem data comparable with other sources computed for 1 individual
    REAL(KIND=DPRECI),ALLOCATABLE,DIMENSION(:,:)                        :: UNEM_RESULT_COMPARE                  !Vector with aggregates of unem data comparable with other sources computed over all individuals
    INTEGER,ALLOCATABLE,DIMENSION(:)                                    :: ClassifyGlob                         !Identification number saying in which line of the global budget table the amount must come

    !Compute sum of weights
    SumW=SUM(MIMOSIS_WEIGHT(1:n_rec))

    !compute unemployment benefits
    NumAnal = 3
    ALLOCATE(UNEM_AGGREGA(NumAnal))       
    ALLOCATE(UNEM_RESULT_COMPARE(NumAnal,4))
    ALLOCATE(ClassifyGlob(NumAnal))       

    !Initialise results
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1:4) = 0.D0

    !Compute aggregate statistics
    DO pers_j=1,n_rec
        !Make vector with local aggregates empty
        UNEM_AGGREGA(1:SIZE(UNEM_AGGREGA,1)) = 0.D0
        !Compute quarterly amounts
        UNEM_AGGREGA(1)  = UNEM_AGGREGA(1)      + UNEM_BENSTUD_QT(pers_j) + UNEM_BENEMPL_QT(pers_j) &
                                            &   + UNEM_BENUNS3_QT(pers_j) + UNEM_BENUNS4_QT(pers_j) &
                                            &   + UNEM_BENEMP3_QT(pers_j)                           &    
                                            &   + UNEM_BENEMP4_QT(pers_j)                           &
                                            &   + UNEM_BENEMP5_QT(pers_j) + UNEM_BENEMP6_QT(pers_j) 
        UNEM_AGGREGA(2)  = UNEM_AGGREGA(2)      + UNEM_BENUNS1_QT(pers_j) + UNEM_BENEMP1_QT(pers_j)
        UNEM_AGGREGA(3)  = UNEM_AGGREGA(3)      + UNEM_BENUNS2_QT(pers_j) + UNEM_BENEMP2_QT(pers_j)
        !Add all the individual results to a vector that aggregates results over all individuals
        DO j=1,SIZE(UNEM_AGGREGA,1)
            IF (INT(100.D0*UNEM_AGGREGA(j)).NE.0) THEN
                UNEM_RESULT_COMPARE(j,1)=UNEM_RESULT_COMPARE(j,1)+MIMOSIS_WEIGHT(pers_j)*UNEM_AGGREGA(j)
                UNEM_RESULT_COMPARE(j,2)=UNEM_RESULT_COMPARE(j,2)+MIMOSIS_WEIGHT(pers_j)
            END IF
        END DO
    END DO

    !Inflate contributions to annual values expressed in 1000 Euro's (=format of global budget)
    UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1)=(UNEM_RESULT_COMPARE(1:SIZE(UNEM_RESULT_COMPARE,1),1)*4.D0)/1000.D0

    !Compute columns 3 to 4 of the vector with comparable aggregates
    DO i=1,SIZE(UNEM_RESULT_COMPARE,1)
        !Average amount
        IF (INT(UNEM_RESULT_COMPARE(i,2)).NE.0) THEN
            UNEM_RESULT_COMPARE(i,3)=UNEM_RESULT_COMPARE(i,1)/UNEM_RESULT_COMPARE(i,2)
        END IF
        !Fraction of population with non zero value
        IF (INT(SumW).NE.0) THEN
            UNEM_RESULT_COMPARE(i,4)=100.D0*UNEM_RESULT_COMPARE(i,2)/SumW
        END IF        
    END DO

    !Open output files
    OPEN(unit=16,FILE=GLOBBUD_EMPL,POSITION='APPEND',ACTION='WRITE')
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') GLOBBUD_EMPL
        WRITE(15,'("Could not find ", a100)') GLOBBUD_EMPL
        PAUSE
        STOP
    END IF

    !Determine in which line of the global budget the computed result must be stored
    ClassifyGlob(1) = 10
    ClassifyGlob(2) = 11
    ClassifyGlob(3) = 12

    write(16,'(''Benefits in case of unemployment                                                                    '',&
    &2F20.2,F10.2,F10.1,I10)')(UNEM_RESULT_COMPARE(1,j),j=1,4),ClassifyGlob(1)
    write(16,'(''Benefits in case of early retirement                                                                '',&
    &2F20.2,F10.2,F10.1,I10)')(UNEM_RESULT_COMPARE(2,j),j=1,4),ClassifyGlob(2)
    write(16,'(''Benefits in case of career break                                                                    '',&
    &2F20.2,F10.2,F10.1,I10)')(UNEM_RESULT_COMPARE(3,j),j=1,4),ClassifyGlob(3)

    CLOSE(UNIT=16)

    DEALLOCATE(UNEM_AGGREGA)      
    DEALLOCATE(UNEM_RESULT_COMPARE)
    DEALLOCATE(ClassifyGlob)    

END SUBROUTINE UNEM_GLOBBUD

!============= SUBROUTINE OUTPUT_FILEWRIT ==============!
!This subroutine open output file and write output      !
!=======================================================!

SUBROUTINE UNEM_OUTPUT_FILEWRIT

    INTEGER :: pers_j
    
    OPEN(UNIT=12, FILE=CONTRIB_UNEM,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') CONTRIB_UNEM
        WRITE(15,'("Could not find ", a100)') CONTRIB_UNEM
        PAUSE
        STOP
    END IF
    OPEN(UNIT=13, FILE=SOCBEN_UNEM,ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') SOCBEN_UNEM
        WRITE(15,'("Could not find ", a100)') SOCBEN_UNEM
        PAUSE
        STOP
    END IF

    DO pers_j=1,n_rec
        WRITE(12,'(A14,F15.2,F15.2,F15.2,F15.2,F15.2,F15.2,I10,I10)')   GECODEERD_INSZ_CHAR(pers_j),UNEM_BENUNS1_QT(pers_j),&
        &UNEM_ADDUNS_QT(pers_j),UNEM_BENEMP1_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_OTHERUN_QT(pers_j),UNEM_OLDUNSS_QT(pers_j),&
        &UNEM_STATUS_QT(pers_j),UNEM_NUMMINU_QT(pers_j)
        WRITE(13,'(A14,I10)')   GECODEERD_INSZ_CHAR(pers_j),UNEM_NUMMINU_QT(pers_j)
    END DO

    CLOSE(UNIT=12)
    CLOSE(UNIT=13)

    WRITE(*,*) 'Subroutine UNEM_OUTPUT_FILEWRIT of NORMAL RUN.........runned correctly'

END SUBROUTINE UNEM_OUTPUT_FILEWRIT

!=================================================================================!
!Routine to write UNEM variables for pre run
!=================================================================================!

SUBROUTINE WRITEPRE_UNEM

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\UNEM_PRERUN.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,I10)') GECODEERD_INSZ_CHAR(pers_j),UNEM_FULL6MON_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_UNEM..............................runned correctly'

END SUBROUTINE WRITEPRE_UNEM

!=================================================================================!
!Routine to write UNEM variables for the labsupply data set
!=================================================================================!

SUBROUTINE WRITELAB_UNEM

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME

    DATA FILENAME /'C:\temp\UNEM_LABSUPPLY.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec                                                                                
        WRITE(12,'(A14,I10,3F15.2,I10)') GECODEERD_INSZ_CHAR(pers_j),UNEM_STATUS_QT(pers_j),UNEM_TOTBEN_QT(pers_j),&
        &UNEM_ADDUNS_QT(pers_j),UNEM_ADDEMP_QT(pers_j),UNEM_FAMSIT_QT(pers_j)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_UNEM..............................runned correctly'

END SUBROUTINE WRITELAB_UNEM

!=================================================================================!
!Routine to write UNEM variables for the computation of individual unemployment benefits
!=================================================================================!

SUBROUTINE WRITEIND_UNEM

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME
    REAL(KIND=DPRECI)                                                   :: HPERDAY      !Variable containing number of hours worked per day

    DATA FILENAME /'C:\temp\UNEM_INDIVINF.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        HPERDAY = 7.6D0
        IF (UNEM_STATUS_QT(pers_j).NE.0) THEN
            WRITE(12,'(A14,I10,F15.2,I10,I10,I10,F15.2,F15.2,F15.10)') GECODEERD_INSZ_CHAR(pers_j),UNEM_STATUS_QT(pers_j),&
            &UNEM_TOTBEN_QT(pers_j),UNEM_FAMSIT_QT(pers_j),UNEM_DISABLED_QT(pers_j),UNEM_NUMMINU_QT(pers_j),HPERDAY,&
            &MIMOSIS_GRINC_HOUR(pers_j),MIMOSIS_WEIGHT(pers_j)
        END IF
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEPRE_UNEM..............................runned correctly'

END SUBROUTINE WRITEIND_UNEM

!=================================================================================!
!Routine to write UNEM variables for the computation of individual unemployment benefits
!=================================================================================!

SUBROUTINE WRITEFAM_UNEM

    INTEGER :: pers_j
    CHARACTER(len=200) :: FILENAME
    REAL(KIND=DPRECI)                                                   :: HPERDAY      !Variable containing number of hours worked per day

    DATA FILENAME /'C:\temp\UNEM_INDIVINF.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        HPERDAY = 7.6D0
        IF (UNEM_STATUS_QT(pers_j).NE.0) THEN
            WRITE(12,'(A14,I10)') GECODEERD_INSZ_CHAR(pers_j),UNEM_FAMSIT_QT(pers_j)
        END IF
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITEFAM_UNEM..............................runned correctly'

END SUBROUTINE WRITEFAM_UNEM

!=================================================================================!
!Routine to write UNEM variables for a descriptive note on the whole model
!=================================================================================!

SUBROUTINE WRITE_UNEMINFO

    INTEGER                                        :: pers_j
    CHARACTER(len=200)                             :: FILENAME
    REAL(KIND=KIND_REAL),DIMENSION(1:4)            :: VEC_UNEM     ! vector met bedragen in functie van "leeftijd" werkloosheidsuitkering

    DATA FILENAME /'C:\temp\UNEM_UNEMINFO.txt'/    

    OPEN (UNIT=12, FILE=FILENAME,ACTION='WRITE', IOSTAT=ierror)

    DO pers_j=1,n_rec
        VEC_UNEM(1:4) = 0
        !Dagbedragen voltijds werkzoekende werklozen
        IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/2/)).AND.UNEM_NUMMINU_QT(pers_j).LT.12) THEN
            VEC_UNEM(1) = UNEM_BENEMPL_DAY(pers_j)
        ELSE IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/2/)).AND.UNEM_NUMMINU_QT(pers_j).GE.12&
        &.AND.UNEM_NUMMINU_QT(pers_j).LT.24) THEN
            VEC_UNEM(2) = UNEM_BENEMPL_DAY(pers_j)
        ELSE IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/2/)).AND.UNEM_NUMMINU_QT(pers_j).GE.24&
        &.AND.UNEM_NUMMINU_QT(pers_j).LT.60) THEN
            VEC_UNEM(3) = UNEM_BENEMPL_DAY(pers_j)
        ELSE IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/2/)).AND.UNEM_NUMMINU_QT(pers_j).GE.60) THEN
            VEC_UNEM(4) = UNEM_BENEMPL_DAY(pers_j)
        END IF
        WRITE(12,'(A14,I10,4F15.2)') GECODEERD_INSZ_CHAR(pers_j),InRVA(pers_j),(VEC_UNEM(j),j=1,4)
    END DO

    CLOSE (UNIT=12)

    WRITE(*,*) 'Subroutine WRITE_UNEMINFO...........................runned correctly'

END SUBROUTINE WRITE_UNEMINFO

!==========================================================!
!The subroutine UNEM_CALC is used to call the other        ! 
!subroutines and to carry out the necessary computations   ! 
!==========================================================!

SUBROUTINE UNEM_CALC

!Program heading

!=============================================================================
    WRITE (*,*) ' '
    WRITE (*,*) 'MIMOSIS UNEM IS RUNNING '
    WRITE (*,*) ' '
!=============================================================================

    !open log file
    OPEN(UNIT=15, FILE=UNEM_LOG, STATUS='REPLACE', ACTION='WRITE', IOSTAT=ierror)
    IF (ierror.NE.0) THEN
        WRITE(*,'("Could not find ", a100)') UNEM_LOG
        WRITE(15,'("Could not find ", a100)') UNEM_LOG
        PAUSE
        STOP
    END IF

    !!!REMARK: allocation of exogenous and endogenous variables is split here
    !!!if they are allocated with a single subroutine the program fails with a message
    !!!that it tries to access non available memory. No explanantion found, but splitting
    !!!the allocation in two steps solved the problem
    !!!At the time the problem occured, the UNEM module was certainly not the module with the
    !!!largest memory allocation requirements
    !Allocate all exogenous vectors with dimension n_rec 
    CALL UNEM_ALLOC_EXOG(n_rec)
    !Reads in PARAMETER file
    CALL UNEM_PARAMETER_FILEREAD
    !Read in files with input data
    CALL UNEM_INPUT_FILEREAD
    CALL UNEM_CONTRIB_FILEREAD
    CALL UNEM_SICK_FILEREAD
    CALL UNEM_PIT_FILEREAD
    CALL UNEM_PENSWELF_FILEREAD
    CALL UNEM_FAMAL_FILEREAD
    CALL UNEM_UNEM_FILEREAD
    CALL UNEM_FAMREL_FILEREAD
    CALL UNEM_WAGE_FILEREAD
    CALL UNEM_WEIGHT_FILEREAD

    WRITE(15,'(/,''Print of function calls for which no selection was carried out'')')
    CALL DRAWLINE(15,100,'=')                

    !Allocate all endogenous vectors with dimension n_rec 
    CALL UNEM_ALLOC_ENDO(n_rec)
    !Initialise endogenous variables
    CALL UNEM_INITENDO(n_rec)
    
    !Rank and Sort all variables read in
    CALL UNEM_RANKANDSORT

    !Recode missing values
    CALL UNEM_RECODE
    !Inflate values
    CALL UNEM_INFLATE
    !Determine household size  
    CALL UNEM_HOUSEHOLD_SIZE
    !Determine age of all individuals
    CALL UNEM_AGE

    !Construct variables necessary to run the different Unem computation subroutines 
    CALL UNEM_CHARAC            !construction of different characteristics of the unemployed (mainly eligibility variables)
    CALL UNEM_FAMSIT_CONSTRUC   !determine the household charge of each unemployed

    !Loop over all households  
    DO hh_i=1,N_HH_UNEM
        !Loop over all persons within the household
        DO pers_j=UNEM_START_HH(hh_i),UNEM_END_HH(hh_i)
            !Compute benefits of unemployed in search of work 
            IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2/))) THEN            
                CALL UNEM_BENUIS_CONSTRUC(pers_j)
            END IF                
            !Compute reference unemployment benefits of unemployed not in search of work or employees entitled to RVA benefits 
            !Benefits currently computed as if the unemployed was entitled to benefits of unemployed not in search of work after employment
            IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12/))) THEN            
                CALL UNEM_BENREF_CONSTRUC(pers_j)
            END IF                
            !Compute benefits of unemployed not in search of work 
            IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/3, 4, 5, 6/))) THEN            
                CALL UNEM_BENUNIS_CONSTRUC(pers_j)
           END IF                
            !Compute benefits of employees entitled to RVA benefit
            IF (ANY(UNEM_STATUS_QT(pers_j).EQ.(/7, 8, 9, 10, 11, 12/))) THEN            
                CALL UNEM_BENEMP_CONSTRUC(pers_j)
            END IF       
            !Call adaptation routine in case of marginal tax simulation 
            IF (MIMOSIS_SIMNUMMT(pers_j).NE.0) THEN
                CALL UNEM_ADASIMMT(pers_j)
            END IF
        END DO            
    END DO  

    !Construct exchange concepts
    CALL UNEM_EXCHANGE

    !test construction of unemployment constructions
    CALL UNEM_TEST
    CALL UNEM_COMPARE

    !Call routine to create global budget concepts
    CALL UNEM_GLOBBUD

    !Write output to output file
    CALL UNEM_OUTPUT_FILEWRIT
    !Write file with UNEM variables for pre run
!    CALL WRITEPRE_UNEM
!    CALL WRITELAB_UNEM
!    CALL WRITEIND_UNEM
!    CALL WRITEFAM_UNEM
!    CALL WRITE_UNEMINFO

    !Deallocate allocated variables
    CALL UNEM_DEALLOC
    CLOSE(15)

END SUBROUTINE UNEM_CALC

END MODULE UNEM_SUBROUTINES
